package edu.gwu.blaise.version2.backend.admin;
/**
 * This is the backend module that handles all the administrator function.
 * These include:Paper Interaction (all papers, but limited to paper header
 * information), Course Interaction, Writer Interaction, Grader Interaction, 
 * University Interaction, and Campus Interaction.
 *
 * @author Michael Gaiman
 * @author Jared Kiraly
 * @version 2 Jan 19, 2004
 */
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.packagers.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.blaise.version2.documents.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

public class AdminServlet extends BaseServlet{

   
    /** 
     * A string of characters that is passed with every request.
     * If the passcode is missing or incorrect, a request will not be processed.
     */
    private static final String PASS_CODE = "4";
    private boolean didGet=false;
    private File tempDir=null;
    public AdminServlet(){
    }
    public void doGet (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException{
        didGet=true;
        doPost(req,res);
    }

    /**
     * Handles the request initially.  It will verify that the request is valid, then handle the response
     * in a version specific way.
     */
    public void doPost (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException{
        if(didGet||req.getContentType().equals("application/x-www-form-urlencoded")){        
            try{
                res.setContentType ("text/plain");
                PrintWriter out = res.getWriter ();
                
                //first, validate the client
                String reqPC=req.getParameter("passCode");
                String reqVersion=req.getParameter("version");
                //bounce back an access denied message if invalid
                if(!validateClient(reqPC,reqVersion)){
                    out.print(new ResultWriter(false,"","Access Denied").toString());
                    out.flush();
                    out.close();
                    return;
                }
                String reqCommand=req.getParameter("command");
                
                String result=null;
                try{
                    //pass the request on to the Packagers.
                    //for all packages (except validate) check for a valid u/p/uID combination before proceeding.
                    if(reqCommand.startsWith("validate") && reqCommand.endsWith("Login")){
                        PackageValidateLogin p=new PackageValidateLogin(reqCommand,req,PackagingConstants.LIMIT_admin);
                        result=p.execute();
                    }
                    else{
                        result=validateUser(req.getParameter("myUserName"),req.getParameter("myEPass"));
                        if(result==null){
                            if(reqCommand.startsWith("get") && reqCommand.endsWith("ID")){
                            //including getPaperID
                                PackageGetID p=new PackageGetID(reqCommand,req,PackagingConstants.LIMIT_admin);
                                result=p.execute();
                            }
                            else if(reqCommand.indexOf("Paper")>=0){//excluding getPaperID
                                //PackagePaper shouldn't be called here.  Something must be wrong.
                                result = new ResultWriter(false,"","PackagePaper called from the wrong place.").toString();
                            }
                            else if(reqCommand.startsWith("get") && !reqCommand.endsWith("ID")){
                                PackageGet p=new PackageGet(reqCommand,req,PackagingConstants.LIMIT_admin);
                                result=p.execute();
                            }
                            else if(reqCommand.startsWith("edit")){
                                PackageEdit p=new PackageEdit(reqCommand,req,PackagingConstants.LIMIT_admin);
                                result=p.execute();
                            }
                            else if(reqCommand.startsWith("new")){
                                PackageNew p=new PackageNew(reqCommand,req,PackagingConstants.LIMIT_admin);
                                result=p.execute();
                            }
                            else if(reqCommand.startsWith("delete")){
                                PackageDelete p=new PackageDelete(reqCommand,req,PackagingConstants.LIMIT_admin);
                                result=p.execute();
                            }
                            else{
                                result=new ResultWriter(false,"","Invalid Command").toString();
                            }
                        }
                    }
                }catch(NoConnectionException e){
                    result=new ResultWriter(false,"",e.toString()).toString();
                }
                out.print(result);
                out.flush();
                out.close();
            }catch(Throwable t){
                res.setContentType ("text/plain");
                PrintWriter out = res.getWriter ();
                out.print(new ResultWriter(false,"","An unknown error has occurred.  The administrator has been notified.").toString());
                out.flush();
                out.close();
                StringBuffer stringbuf=new StringBuffer();
                stringbuf.append(t);
                StackTraceElement[] st= t.getStackTrace();
                for(int i=0;i<st.length;i++){
                  stringbuf.append("\n");
                  stringbuf.append(st[i].toString()); 
                }
                if(BaseServlet.debugOn()){
                    BaseServlet.writeDebug(stringbuf.toString());
                }
                else{
                    BaseServlet.writeLog(stringbuf.toString());
                }
            }finally{
                didGet=false;
            }
        }
        else{
            try{
                res.setContentType("application/octet-stream");
                DataOutputStream out=new DataOutputStream(res.getOutputStream());
                InputStream in=req.getInputStream();
                byte[] result=null;            
                 
                
                //now read the input stream into an appropriately sized array
                byte[] both=DocumentUtilities.getInputStreamAsByteArray(in);
                //get the properties byte array
                byte[] first=DualByteArray.extractFirst(both);
                //make the properties object
                Properties p=DualByteArray.toProperties(first);
                
                String reqPC=p.getProperty("passCode");
                String reqVersion=p.getProperty("version");
                String reqCommand=p.getProperty("command");
                
                //first, validate the client
                //bounce back an access denied message if invalid
                if(!validateClient(reqPC,reqVersion)){
                    out.write(PackagePaper.sendMessage(new ResultWriter(false,"","Access Denied")));
                    out.flush();
                    out.close();
                    return;
                }
                
                //might as well extract the paper here
                Blaise_Paper bp=null;
                byte[] second=DualByteArray.extractSecond(both);
                if(second.length>0){//if no paper was sent, this would have length zero
                    bp=BlaiseFile.byteArrayToPaper(second);
                }
    
                try{            
                    if(reqCommand.indexOf("Paper")>=0){//excluding getPaperID
                        tempDir=(File) getServletContext().getAttribute("javax.servlet.context.tempdir");
                        PackagePaper pp=new PackagePaper(reqCommand,req,PackagingConstants.LIMIT_writer,tempDir,p,bp);
                        result=pp.execute(true);
                    }            
                    else{//this part of the if/else clause should only call PackagePaper, send an error message
                        result=new ResultWriter(false,"","Binary submission sent in without PackagePaper request").toString().getBytes();
                    }
                }catch(NoConnectionException e){
                    result=PackagePaper.sendMessage(new ResultWriter(false,"",e.toString()));
                }
                out.write(result);
                out.flush();
                out.close();
            }catch(Throwable t){
                res.setContentType("application/octet-stream");
                DataOutputStream out=new DataOutputStream(res.getOutputStream());
                out.write(PackagePaper.sendMessage(new ResultWriter(false,"","An unknown error has occurred.  The administrator has been notified.")));
                out.flush();
                out.close();
                StringBuffer stringbuf=new StringBuffer();
                stringbuf.append(t);
                StackTraceElement[] st= t.getStackTrace();
                for(int i=0;i<st.length;i++){
                  stringbuf.append("\n");
                  stringbuf.append(st[i].toString()); 
                }
                if(BaseServlet.debugOn()){
                    BaseServlet.writeDebug(stringbuf.toString());
                }
                else{
                    BaseServlet.writeLog(stringbuf.toString());
                }
            }finally{
                didGet=false;
            }
        }
    }
    /**
     * checks to see if this request is a valid one by comparing the supplied
     * pass code to the valid one for the version of adminServlet they are trying to 
     * use.
     * @param reqPC the supplied pass code to check for validity.
     * @param reqVersion the version of the client they are using
     * @return <code>boolean</code> <code>true</code> if valid.  </code>false</code> otherwise.
     */
    boolean validateClient(String reqPC, String reqVersion){
        //only one version thus far.  Will need to change this method
        //once more than one client is out.
        if(reqPC==null || reqVersion==null){ 
            return false;
        }
        return reqPC.equals(PASS_CODE);
    }
    
    
    /**
     * Checks to see that the user is valid by performing a database query on the data.
     * @param u the username
     * @param p the password
     * @return <code>null</code> if the user is valid.  If the user isn't valid, a String
     * containing an error message is returned to the user.
     */
    String validateUser(String u,String p){
        if(u==null || p==null){
            return new 
            ResultWriter(false,"","Access Denied.  Authentication Required.").toString();
        }
        
        boolean b=PackageValidateLogin.isValid(u,p,null,PackagingConstants.LIMIT_admin,null);
        if(b){
            return null;
        }
        else{
            return new ResultWriter(false,"","Invalid username/password combination.").toString();
        }
    }
    
    /////////////////////////////////////////////////////////////
    //
    // edu.gwu.servlets required methods
    //
    /////////////////////////////////////////////////////////////
  
    public void shutdown ()
    {
        // Nothing to be done.
        return;
    }
    
    public void reset ()
    {
        // Nothing to be done.
        return;
    }
    
    public void save ()
    {
        // Nothing to be done.
        return;
    }
    
    public void test (PrintWriter out)
    {
        out.println ("<p> AdminServlet.test: successful </p>");
    }
}
