package edu.gwu.blaise.version2.backend.packagers;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.queries.*;
import edu.gwu.servlets.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*; 
import java.util.Vector;
import java.util.StringTokenizer;
/**
 * Performs a delete query and handles the packaging of query data into 
 * a form that is appropriate for transmission.
 * 
 * @author Jared Kiraly
 * @version 2 May 19, 2004
 */
/*
 * Please note:  All Actual SQL-type stuff is marked by //------SQL Here------ tags.
 */
public class PackageDelete extends Packager{

    public PackageDelete(String commandType, HttpServletRequest request, String limitBy){
        super(commandType,request,limitBy);
    }

    public byte[] execute(boolean b){/*this method is useless in this context, but it has to be in here*/         
        //this method should never be called, but it has to be here.  try to send an error message with the proper
        //encoding, otherwise send one with whatever encoding we can get
        try{
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes("UTF-8");
        }catch(java.io.UnsupportedEncodingException e){
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes();
        }
    }

    /**
     * Performs the actual query and packaging.
     * @return The result from the query in a form that is appropriate for transmission.
     */
    public String execute(){
        //first, make sure that this is a valid query.
        if(request==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No request object in PackageDelete.)").toString();
        }
        if(commandType==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (No commandType object in PackageDelete.)").toString();
        }
        //check to see that it is limitBy for administrator: writers and graders are not allowed to delete
        if(limitBy==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (No limitBy object in PackageDelete.)").toString();
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer) || limitBy.equals(PackagingConstants.LIMIT_grader)){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (Attempt at unauthorized access in PackageDelete.)").toString();
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        }
        else{
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (invalid limitBy object in PackageGet.)").toString();
        }
        BaseServlet.writeLog("PackageDelete: "+limitBy+" is doing: "+commandType);
        Vector IDs=null;
        String table=null;
        //now handle the request (also check for invalid commandType)
        if(commandType.equals(TransmissionConstants.deleteWriter)){
            try{
                IDs=getLongVector(request.getParameter("writerID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid writerID format.").toString();
            }
            table=PackagingConstants.TABLE_writer;
        }
        else if(commandType.equals(TransmissionConstants.deleteGrader)){
            try{
                IDs=getLongVector(request.getParameter("graderID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid graderID format.").toString();
            }
            table=PackagingConstants.TABLE_grader;
        }
        else if(commandType.equals(TransmissionConstants.deleteCourse)){
            try{
                IDs=getLongVector(request.getParameter("courseID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid courseID format.").toString();
            }
            table=PackagingConstants.TABLE_course;
        }
        else if(commandType.equals(TransmissionConstants.deleteUniversity)){
            try{
                IDs=getIntegerVector(request.getParameter("universityID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid universityID format.").toString();
            }
            table=PackagingConstants.TABLE_university;
        }
        else if(commandType.equals(TransmissionConstants.deleteCourseToGrader)){
            try{
                IDs=getLongVector(request.getParameter("courseToGraderID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid courseToGraderID format.").toString();
            }
            table=PackagingConstants.TABLE_ctg;
        }
        else if(commandType.equals(TransmissionConstants.deleteCourseToWriter)){
            try{
                IDs=getLongVector(request.getParameter("courseToWriterID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid courseToWriterID format.").toString();
            }
            table=PackagingConstants.TABLE_ctw;
        }
        else if(commandType.equals(TransmissionConstants.deleteAssignment)){
            try{
                IDs=getLongVector(request.getParameter("assignID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid assignID format.").toString();
            }
            table=PackagingConstants.TABLE_assignment;
        }
        else if(commandType.equals(TransmissionConstants.deleteDraftStructure)){
            try{
                IDs=getLongVector(request.getParameter("draftStructureID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid draftStructureID format.").toString();
            }
            table=PackagingConstants.TABLE_ds;
        }
        else if(commandType.equals(TransmissionConstants.deleteWriterAsGrader)){
            try{
                IDs=getLongVector(request.getParameter("writerAsGraderID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid writerAsGraderID format.").toString();
            }
            table=PackagingConstants.TABLE_wag;
        }
        else if(commandType.equals(TransmissionConstants.deleteStaging)){
            try{
                IDs=getLongVector(request.getParameter("stagingID"));
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid stagingID format.").toString();
            }
            table=PackagingConstants.TABLE_staging;
        }
        else{
            return new ResultWriter(false,"","Invalid command: "+commandType).toString();
        }
        
        //----------------SQL HERE----------------------
        //Now perform the query.  
        DeleteQuery dq=new DeleteQuery(IDs,table,limitBy);
        String results=null;
        try{
            int rs=dq.execute();
            results=new String("Success: "+rs+" entries deleted.");//deleteQuery returns the number of rows changed
        }
        catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageDelete:"+ex.toString()+
                " With parameters:" +" IDs:"+IDs +" table:"+table +
                " limitBy:"+limitBy);
            return dbError.toString();//return the error message...
        }
        catch(NullPointerException n){//DeleteQuery will return null if any of the parameter were wrong
            return new ResultWriter(false,"","An internal server error occurred."+
                                    " Please report to an Administrator.").toString();
        }
        finally{
            dq.free();//let the connection return to the pool.
        }
        
        return results;
    }
    
    ///////////////////////////////////////////////////
    //
    // Utility Methods
    //
    ///////////////////////////////////////////////////    
    
    /**
     * pulls any and all ids from the string, and returns them in Long form packaged in a Vector.
     * @param id The String containing all the IDs. (may be null, or empty).
     * @return the IDs packed in a Vector in Long form. (<code>null</code> if param id is null or empty).
     */
    private Vector getLongVector(String id) throws NumberFormatException{
        Vector v=null;
        if(id!=null && !id.equals("")){
            StringTokenizer st=new StringTokenizer(id,TransmissionConstants.ID_SEPARATOR);
            v=new Vector();
            while(st.hasMoreTokens()){
                v.add(new Long(st.nextToken()));
            }
        }
        return v;
    }
    
    /**
     * pulls any and all ids from the string, and returns them in Integer form packaged in a Vector.
     * @param id The String containing all the IDs. (may be null, or empty).
     * @return the IDs packed in a Vector in Integer form. (<code>null</code> if param id is null or empty).
     */
    private Vector getIntegerVector(String id) throws NumberFormatException{
        Vector v=null;
        if(id!=null && !id.equals("")){
            StringTokenizer st=new StringTokenizer(id,TransmissionConstants.ID_SEPARATOR);
            v=new Vector();
            while(st.hasMoreTokens()){
                v.add(new Integer(st.nextToken()));
            }
        }
        return v;
    }
    
    /**
     * Converts The result set into a {@link edu.gwu.Blaise.version2.transmission.ResultWriter}
     * and then returns the String of it.
     * @param rs The ResultSet to get the data from.
     * @return String appropriate for transmission back to the user.
     */
    private String WriteResults(ResultSet rs) throws SQLException{
    //----------------SQL HERE-------------------------------
        ResultWriter out=new ResultWriter();
        ResultSetMetaData rsmd=rs.getMetaData();//gets column names and numCols
        
        int numCols=rsmd.getColumnCount();
        boolean[] passwordColumn=new boolean[numCols];//do not release passwords columns
        for(int i=0;i<numCols;i++){//initialize array
            passwordColumn[i]=false;
        }
        
        //set the format
        StringBuffer format=new StringBuffer();
        for(int i=1;i<=numCols;i++){
            if(rsmd.getColumnLabel(i).equalsIgnoreCase("password")){
            //do not release password columns
                passwordColumn[i-1]=true;
                continue;
            }
            format.append(rsmd.getColumnLabel(i));
            if(i<numCols){
            //don't append column separator after last column.
                format.append(TransmissionConstants.COL_SEPARATOR);
            }
        }
        format.append(TransmissionConstants.ROW_SEPARATOR);
        out.setFormat(format.toString());
        
        //get the data
        while(rs.next()){
            for(int i=1;i<=numCols;i++){
                if(passwordColumn[i-1]){
                //do not release password columns
                    continue;
                }
                
                /*Object o=rs.getObject(i);
                if(o!=null){
                    out.append(o.toString());
                }
                else{
                    out.append("null");
                }*/
                out.append(rs.getString(i));
                if(i<numCols){
                //don't append column separator after last column.
                    out.append(TransmissionConstants.COL_SEPARATOR);
                }
            }
            if(!rs.isLast()){
                out.append(TransmissionConstants.ROW_SEPARATOR);
            }
        }
        //all the results have been gotten...so it was successful
        out.setStatus(true);
        return out.toString();
    }
}
