package edu.gwu.blaise.version2.backend.packagers;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.queries.*;
import edu.gwu.servlets.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*; 
import java.util.Vector;
import java.util.StringTokenizer;
/**
 * Performs a edit query and handles the packaging of query data into 
 * a form that is appropriate for transmission.
 * 
 * @author Jared Kiraly
 * @version 2 May 19, 2004
 */
/*
 * Please note:  All Actual SQL-type stuff is marked by //------SQL Here------ tags.
 */
public class PackageEdit extends Packager{
    
    public PackageEdit(String commandType, HttpServletRequest request, String limitBy){
        super(commandType,request,limitBy);
    }

    public byte[] execute(boolean b){/*this method is useless in this context, but it has to be in here*/
        //this method should never be called, but it has to be here.  try to send an error message with the proper
        //encoding, otherwise send one with whatever encoding we can get
        try{
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes("UTF-8");
        }catch(java.io.UnsupportedEncodingException e){
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes();
        }
    }

    /**
     * Checks to make sure all of the arguments are valid.
     * @return <code>null</code> if the arguments are valid. Otherwise a message to be returned to 
     * the requester stating error.
     */
    private String validate(){
        //first check the request that the request is not null.
        if(request==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No request object in PackageEdit.)").toString();
        }
        
        //next check the commandType.
        if(commandType==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No commandType object in PackageEdit.)").toString();
        }
        else if(commandType.equals(TransmissionConstants.editWriter)){}
        else if(commandType.equals(TransmissionConstants.editGrader)){}
        else if(commandType.equals(TransmissionConstants.editCourse)){}
        else if(commandType.equals(TransmissionConstants.editUniversity)){}
        else if(commandType.equals(TransmissionConstants.editCourseToGrader)){}
        else if(commandType.equals(TransmissionConstants.editCourseToWriter)){}
        else{
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (Invalid commandType object in PackageEdit.)").toString();
        }
        
        //finally check that the limitBy is supplied, that they are not trying to do something that
        //is not allowed, and that they have the required parameters for their request.
        if(limitBy==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No limitBy object in PackageEdit.)").toString();
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                return new ResultWriter(false,"","An access error occurred. Writers and may not edit").toString();
        } 
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                return new ResultWriter(false,"","An access error occurred. Graders and may not edit").toString();
        } 
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        //in the future, an admin userName/password might be needed too, here is where the check would be.
        }
        else{
        //it wasn't one of the valid limitBy strings. 
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (Invalid limitBy object in PackageEdit.)").toString();
        }
        BaseServlet.writeLog("PackageEdit: "+limitBy+" is doing: "+commandType);
        //everything is okay.  This is one valid request.  GO GO GO!
        return null;
    }
    
    
    /**
     * Performs the actual query and packaging.
     * @return The result from the query in a form that is appropriate for transmission.
     */
    public String execute(){
        //first, make sure that this is a valid query.
        String v=validate();
        if(v!=null){
            return v;
        }
        
        if(commandType.equals(TransmissionConstants.editWriter)){
            return doEditWriter();
        }
        else if(commandType.equals(TransmissionConstants.editGrader)){
            return doEditGrader();
        }
        else if(commandType.equals(TransmissionConstants.editCourse)){
            return doEditCourse();
        }
        else if(commandType.equals(TransmissionConstants.editUniversity)){
            return doEditUniversity();
        }
        else if(commandType.equals(TransmissionConstants.editCourseToGrader)){
            return doEditCourseToGrader();
        }
        else if(commandType.equals(TransmissionConstants.editCourseToWriter)){
            return doEditCourseToWriter();
        }
        else{
            return new ResultWriter(false,"","Illegal command").toString();
        }
    }
    
    ////////////////////////////////////////////////////////
    //
    // Query callers
    //
    ////////////////////////////////////////////////////////
    
    /**
     * execute and package a <code>editWriter</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doEditWriter(){
        String writerID=		request.getParameter("writerID");
	String universityID=		request.getParameter("universityID");
        String studentIdentifier=	request.getParameter("studentIdentifier");
	String firstName=		request.getParameter("firstName");
	String lastName=		request.getParameter("lastName");
        String email=			request.getParameter("email");
	String userName=		request.getParameter("userName");
        String password=		request.getParameter("password");
        String userNameType=		request.getParameter("userNameType");
        
        //fix the values
        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        Long wID=null;
        try{
            wID=new Long(writerID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        EditWriterQuery ew=new EditWriterQuery(wID,uID,studentIdentifier,firstName,
                                                    lastName,email,userName,password,userNameType);
        int result;
        try{
            result=ew.execute();
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageEdit.doEditWriter: "+ex.toString()+
                " With parameters: writerID: "+writerID+" firstName:"+firstName + " lastName:"+lastName +
                " email:"+email +" userName:"+userName +"password: "+password+" universityID:"+universityID +
                " studentIdentifier"+studentIdentifier );
            return dbError.toString();//return the error message...
        }
        //return the answer.
        return new String("Success: "+result+" entries changed.");
    }
    
    
    /**
     * execute and package a <code>editGrader</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doEditGrader(){
        String graderID=		request.getParameter("graderID");
        String universityID=		request.getParameter("universityID");
        String firstName=		request.getParameter("firstName");
        String lastName=		request.getParameter("lastName");
        String userName=		request.getParameter("userName");
        String email=			request.getParameter("email");
        String status=			request.getParameter("status");
        String similarityCheckedDate=	request.getParameter("similarityCheckedDate");
        String password=		request.getParameter("password");

        //fix the values
        Timestamp sCD=null;
        if(similarityCheckedDate!=null && !similarityCheckedDate.equals("")){
            Long i=null;
            try{
                i=new Long(similarityCheckedDate);
                sCD=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid similarityCheckedDate format.").toString();
            }
        }
        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        Long gID=null;
        try{
            gID=new Long(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        EditGraderQuery eg=new EditGraderQuery(gID,uID,firstName,lastName,userName,password,email,status,sCD);
        int result;
        try{
            result=eg.execute();
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageEdit.doEditGrader: "+ex.toString()+
                " With parameters: graderID: "+graderID +" firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" email:"+email +" status:"+status +
                " similarityCheckedDate"+similarityCheckedDate);
            return dbError.toString();//return the error message...
        }
        //return the answer.
        return new String("Success: "+result+" entries changed.");
    }
    
    /**
     * execute and package a <code>editCourse</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doEditCourse(){
        String courseID=	request.getParameter("courseID");
        String contactInfo=	request.getParameter("contactInfo");
	String universityID=	request.getParameter("universityID");
	String dept=		request.getParameter("dept");
	String number=		request.getParameter("number");
	String section=		request.getParameter("section");
	String semester=	request.getParameter("semester");
	String year=		request.getParameter("year");
        String isCurrent=	request.getParameter("isCurrent");

        //fix the values
        Integer y=null;
        try{
            y=new Integer(year);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid year format.").toString();
        }
        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        Long cID=null;
        try{
            cID=new Long(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        EditCourseQuery ec=new EditCourseQuery(cID,contactInfo,uID,dept,number,section,semester,y,isCurrent);
        int result;
        try{
            result=ec.execute();
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageEdit.doEditCourse: "+ex.toString()+
                " With parameters: courseID: "+courseID+" contactInfo:"+contactInfo +" universityID:"+universityID +
                " dept:"+dept +" number:"+number +" section:"+section +" semester:"+semester +
                " year"+year+" isCurrent:"+isCurrent);
            return dbError.toString();//return the error message...
        }
        //return the answer.
        return new String("Success: "+result+" entries changed.");
    }
    
    
    /**
     * execute and package a <code>editUniversity</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doEditUniversity(){
        String universityID=	request.getParameter("universityID");
        String name=		request.getParameter("name");
	String contactInfo=	request.getParameter("contactInfo");
	String currentSemester=	request.getParameter("currentSemester");
	String currentYear=	request.getParameter("currentYear");
        
        //fix the values
        Integer cY=null;
        try{
            cY=new Integer(currentYear);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid currentYear format.").toString();
        }
        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        EditUniversityQuery eu=new EditUniversityQuery(uID,name,contactInfo,currentSemester,cY);
        int result;
        try{
            result=eu.execute();
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageEdit.doEditUniversity: "+ex.toString()+
                " With parameters: universityID: "+universityID+" name:"+name + " contactInfo:"+contactInfo +
                " currentSemester:"+currentSemester +" currentYear:"+currentYear);
            return dbError.toString();//return the error message...
        }
        //return the answer.
        return new String("Success: "+result+" entries changed.");
    }
    
    
    /**
     * execute and package a <code>editCourseToGrader</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doEditCourseToGrader(){
        String courseToGraderID=request.getParameter("courseToGraderID");
        String courseID=	request.getParameter("courseID");
	String graderID=	request.getParameter("graderID");
        
        //fix values
        Long ctgID=null;
        try{
            ctgID=new Long(courseToGraderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseToGraderID format.").toString();
        }
        Long cID=null;
        try{
            cID=new Long(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        Long gID=null;
        try{
            gID=new Long(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        EditCourseToGraderQuery ectg=new EditCourseToGraderQuery(ctgID,cID,gID);
        int result;
        try{
            result=ectg.execute();
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageEdit.doEditCourseToGrader: "
                +ex.toString()+" With parameters: courseToGraderID: "+courseToGraderID+" courseID:"+courseID+ " graderID"+graderID);
            return dbError.toString();//return the error message...
        }
        //return the answer.
        return new String("Success: "+result+" entries changed.");
    }
    
    /**
     * execute and package a <code>editCourseToWriterID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doEditCourseToWriter(){
        String courseToWriterID=request.getParameter("courseToWriterID");
        String courseID=	request.getParameter("courseID");
	String writerID=	request.getParameter("writerID");
        
        //fix values
        Long ctwID=null;
        try{
            ctwID=new Long(courseToWriterID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseToWriterID format.").toString();
        }
        Long cID=null;
        try{
            cID=new Long(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        Long wID=null;
        try{
            wID=new Long(writerID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        EditCourseToWriterQuery ectw=new EditCourseToWriterQuery(ctwID,cID,wID);
        int result;
        try{
            result=ectw.execute();
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageEdit.doEditCourseToWriter: "
                +ex.toString()+" With parameters: courseToWriterID: "+courseToWriterID+" courseID:"+courseID+ " writerID"+writerID);
            return dbError.toString();//return the error message...
        }
        //return the answer.
        return new String("Success: "+result+" entries changed.");
    }    
    
    ///////////////////////////////////////////////////
    //
    // Utility Methods
    //
    ///////////////////////////////////////////////////    
    /**
     * pulls a long value from the string and then converts it into a {@link java.sql.Date}.
     * @param s The String containing the data in millis format.
     * @return the Date in Date format.
     */
    private Timestamp getDate(String s) throws NumberFormatException{
        Timestamp d=null;
        if(s!=null && !s.equals("")){
            Long l=null;
            l=new Long(s);
            d=new Timestamp(l.longValue());//make a date from the time in millis
        }
        return d;
    }    
}
