package edu.gwu.blaise.version2.backend.packagers;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.queries.*;
import edu.gwu.servlets.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*; 
import java.util.Vector;
import java.util.StringTokenizer;
/**
 * Performs a getID query and handles the packaging of query data into 
 * a form that is appropriate for transmission.
 * 
 * @author Michael Gaiman
 * @version 2 Jan 19, 2004
 */
/*
 * Please note:  All Actual SQL-type stuff is marked by //------SQL Here------ tags.
 */
public class PackageGetID extends Packager{
    
    public PackageGetID(String commandType, HttpServletRequest request, String limitBy){
        super(commandType,request,limitBy);
    }
    /**
     * Checks to make sure all of the arguments are valid.
     * @return <code>null</code> if the arguments are valid. Otherwise a message to be returned to 
     * the requester stating error.
     */

    public byte[] execute(boolean b){/*this method is useless in this context, but it has to be in here*/
        //this method should never be called, but it has to be here.  try to send an error message with the proper
        //encoding, otherwise send one with whatever encoding we can get
        try{
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes("UTF-8");
        }catch(java.io.UnsupportedEncodingException e){
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes();
        }
    }

    private String validate(){
        //first check the request that the request is not null.
        if(request==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No request object in PackageGetID.)").toString();
        }
        
        //next check the commandType.
        if(commandType==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No commandType object in PackageGetID.)").toString();
        }
        else if(commandType.equals(TransmissionConstants.getWriterID)){}
        else if(commandType.equals(TransmissionConstants.getGraderID)){}
        else if(commandType.equals(TransmissionConstants.getCourseID)){}
        else if(commandType.equals(TransmissionConstants.getPaperID)){}
        else if(commandType.equals(TransmissionConstants.getUniversityID)){}
        else if(commandType.equals(TransmissionConstants.getAdminID)){}
        else if(commandType.equals(TransmissionConstants.getCourseToGraderID)){}
        else if(commandType.equals(TransmissionConstants.getCourseToWriterID)){}
        else if(commandType.equals(TransmissionConstants.getCheckoutID)){}
        else if(commandType.equals(TransmissionConstants.getPaperUngradedID)){}
        else if(commandType.equals(TransmissionConstants.getEmailQueueID)){}
        else if(commandType.equals(TransmissionConstants.getAssignID)){}
        else if(commandType.equals(TransmissionConstants.getWriterAsGraderID)){}
        else if(commandType.equals(TransmissionConstants.getDraftStructureID)){}
        else if(commandType.equals(TransmissionConstants.getStagingID)){}
        else{
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (Invalid commandType object in PackageGetID.)").toString();
        }
        
        //finally check that the limitBy is supplied, that they are not trying to do something that
        //is not allowed, and that they have the required parameters for their request.
        if(limitBy==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No limitBy object in PackageGetID.)").toString();
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
        //the request was from the writer servlet.  Writers are only allowed to look up
        //their own papers, their own courses, and assignments/draft structures in those courses.
            if(commandType.equals(TransmissionConstants.getPaperID)){
            }
            else if(commandType.equals(TransmissionConstants.getCourseToWriterID)){
            }
            else if(commandType.equals(TransmissionConstants.getCourseID)){
            }
            else if(commandType.equals(TransmissionConstants.getAssignID)){
            }
            else if(commandType.equals(TransmissionConstants.getWriterAsGraderID)){
            }
            else if(commandType.equals(TransmissionConstants.getDraftStructureID)){
            }
            else if(commandType.equals(TransmissionConstants.getPaperUngradedID)){
            }            
            else if(commandType.equals(TransmissionConstants.getDraftStructureID)){
            }
            else if(commandType.equals(TransmissionConstants.getStagingID)){
            }
            else{
                return new ResultWriter(false,"","An access error occurred. Writers may not perform command:"+
                                                 commandType).toString();
            }
            //it was a valid request, So check to see if they supplied the required parameters.
            if(request.getParameter("writerID")==null){
                return new ResultWriter(false,"","An access error occurred. A writerID is required for command:"+
                                                 commandType).toString();
            }
            //in the future, a userName/password might be needed too.  Here is where the check should be.
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
        //the request was from the grader servlet.  Check to make sure it is an allowed ID lookup.
            if(commandType.equals(TransmissionConstants.getPaperID)){
            //limited to papers in the grader's courses.
            }
            else if(commandType.equals(TransmissionConstants.getWriterID)){
            //limited to writers who have submitted papers for grader's courses.
            }
            else if(commandType.equals(TransmissionConstants.getCourseID)){
            //limited to the grader's courses.
            }
            else if(commandType.equals(TransmissionConstants.getPaperUngradedID)){
            //limited to papers from grader's courses that have ungraded drafts.
            }
            else if(commandType.equals(TransmissionConstants.getWriterAsGraderID)){
            //limited to writers from grader's courses
            }
            else if(commandType.equals(TransmissionConstants.getStagingID)){
            //limited to papers from grader's courses
            }
            else if(commandType.equals(TransmissionConstants.getAssignID)){
            //limited to assignments from grader's courses
            }
            else if(commandType.equals(TransmissionConstants.getDraftStructureID)){
            //limited to drafts from grader's courses
            }
            else{//grader can't lookup any other IDs...
                return new ResultWriter(false,"","An access error occurred. Graders may not perform command:"+
                                                 commandType).toString();
            }
            
            //the graderID is needed for all of the above commands (when LIMIT_grader).
            if(request.getParameter("graderID")==null){
                return new ResultWriter(false,"","An access error occurred. A graderID is required for"+
                    " command:"+commandType).toString();
            }
            //in the future, a userName/password might be needed too.  Here is where the check should be.
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        //the request was made from the admin servlet.  Admins can do any ID lookup.
        //in the future, a userName/password might be needed too, here is where the check would be.
        }
        else{
        //it wasn't one of the valid limitBy strings. 
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (Invalid limitBy object in PackageGetID.)").toString();
        }
        
        //everything is okay.  This is one valid request.  GO GO GO!
        BaseServlet.writeLog("PackageGetID: "+limitBy+" is doing: "+commandType);
        return null;
    }
    
    
    /**
     * Performs the actual query and packaging.
     * @return The result from the query in a form that is appropriate for transmission.
     */
    public String execute(){
        //first, make sure that this is a valid query.
        String v=validate();
        if(v!=null){
            return v;
        }
        
        if(commandType.equals(TransmissionConstants.getWriterID)){
            return doGetWriterID();
        }
        else if(commandType.equals(TransmissionConstants.getGraderID)){
            return doGetGraderID();
        }
        else if(commandType.equals(TransmissionConstants.getCourseID)){
            return doGetCourseID();
        }
        else if(commandType.equals(TransmissionConstants.getPaperID)){
            return doGetPaperID();
        }
        else if(commandType.equals(TransmissionConstants.getUniversityID)){
            return doGetUniversityID();
        }
        else if(commandType.equals(TransmissionConstants.getAdminID)){
            return doGetAdminID();
        }
        else if(commandType.equals(TransmissionConstants.getCourseToGraderID)){
            return doGetCourseToGraderID();
        }
        else if(commandType.equals(TransmissionConstants.getCourseToWriterID)){
            return doGetCourseToWriterID();
        }
        else if(commandType.equals(TransmissionConstants.getCheckoutID)){
            return doGetCheckoutID();
        }
        else if(commandType.equals(TransmissionConstants.getPaperUngradedID) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return doGetPaperID();
        }
        else if(commandType.equals(TransmissionConstants.getPaperUngradedID)){
            return doGetPaperUngradedID();
        }        
        else if(commandType.equals(TransmissionConstants.getEmailQueueID)){
            return doGetEmailQueueID();
        }
        else if(commandType.equals(TransmissionConstants.getAssignID)){
            return doGetAssignID();
        }
        else if(commandType.equals(TransmissionConstants.getWriterAsGraderID)){
            return doGetWriterAsGraderID();
        }
        else if(commandType.equals(TransmissionConstants.getDraftStructureID)){
            return doGetDraftStructureID();
        }
        else if(commandType.equals(TransmissionConstants.getStagingID)){
            return doGetStagingID();
        }
        else{
            return new ResultWriter(false,"","Illegal command").toString();
        }
    }
    
    ////////////////////////////////////////////////////////
    //
    // Query callers
    //
    ////////////////////////////////////////////////////////
    
    /**
     * execute and package a <code>getWriterID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetWriterID(){
	String universityID=		request.getParameter("universityID");
        String studentIdentifier=	request.getParameter("studentIdentifier");
	String firstName=		request.getParameter("firstName");
	String lastName=		request.getParameter("lastName");
	String userName=		request.getParameter("userName");
        String userNameType=		request.getParameter("userNameType");
        String email=			request.getParameter("email");
	String creationDate=		request.getParameter("creationDate");
        String courseID=		request.getParameter("courseID");
        String graderID=		request.getParameter("graderID");//for LIMIT_grader
        
        //fix the non-String values
        Timestamp cD=null;
        try{
            cD=getDate(creationDate);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid creationDate format.").toString();
        }
        
        //package any courseIDs into a Vector.
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        //package any universityIDs into a Vector.
        Vector uID=null;
        try{
            uID=getIntegerVector(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        //package any graderIDs into a vector.
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetWriterIDQuery gwID=new GetWriterIDQuery(uID,studentIdentifier,firstName,
                                                    lastName,userName,userNameType,email,cD,cID,gID);
        ResultWriter out=null;
        try{
            out=doGetID(gwID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("writerID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetWriterID: "+ex.toString()+
                " With parameters:" +" creationDate:"+creationDate +" firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" email:"+email +" universityID:"+universityID +
                " studentIdentifier"+studentIdentifier +" courseID:"+courseID+ " graderID"+graderID);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gwID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    
    /**
     * execute and package a <code>getGraderID</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doGetGraderID(){
        String creationDate=		request.getParameter("creationDate");
        String universityID=		request.getParameter("universityID");
        String firstName=		request.getParameter("firstName");
        String lastName=		request.getParameter("lastName");
        String userName=		request.getParameter("userName");
        String email=			request.getParameter("email");
        String status=			request.getParameter("status");
        String similarityCheckedDate=	request.getParameter("similarityCheckedDate");
        String courseID=		request.getParameter("courseID");
        
        //fix the values
        Timestamp cD=null;
        if(creationDate!=null && !creationDate.equals("")){
            Long i=null;
            try{
                i=new Long(creationDate);
                cD=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid creationDate format.").toString();
            }
        }
        Timestamp sCD=null;
        if(similarityCheckedDate!=null && !similarityCheckedDate.equals("")){
            Long i=null;
            try{
                i=new Long(similarityCheckedDate);
                sCD=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid similarityCheckedDate format.").toString();
            }
        }
        //package any universityIDs into a Vector.
        Vector uID=null;
        try{
            uID=getIntegerVector(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        //package any courseIDs into a Vector.
        Vector cID=null;
        if(courseID!=null && !courseID.equals("")){
            StringTokenizer st=new StringTokenizer(courseID,TransmissionConstants.ID_SEPARATOR);
            cID=new Vector();
            while(st.hasMoreTokens()){
                try{
                    cID.add(new Long(st.nextToken()));
                }catch(NumberFormatException e){
                    return new ResultWriter(false,"","Invalid courseID format.").toString();
                }
            }
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetGraderIDQuery ggID=new GetGraderIDQuery(cD,uID,firstName,lastName,userName,email,status,sCD,cID);
        ResultWriter out=null;
        try{
            out=doGetID(ggID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("graderID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetGraderID: "+ex.toString()+
                " With parameters:" +" creationDate:"+creationDate +" firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" email:"+email +" status:"+status +
                " similarityCheckedDate"+similarityCheckedDate +" courseID:"+courseID);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            ggID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getCourseID</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doGetCourseID(){
        String contactInfo=	request.getParameter("contactInfo");
	String universityID=	request.getParameter("universityID");
	String dept=		request.getParameter("dept");
	String number=		request.getParameter("number");
	String section=		request.getParameter("section");
	String semester=	request.getParameter("semester");
	String year=		request.getParameter("year");
        String graderID=	request.getParameter("graderID");//to support LIMIT_grader
        String writerID=	request.getParameter("writerID");//to support LIMIT_writer
        String isCurrent=	request.getParameter("isCurrent");
        
        //fix the values
        Integer y=null;
        if(year!=null && !year.equals("")){
            try{
                y=new Integer(year);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid year format.").toString();
            }
        }
        
        //get all of the universityID values (if any).
        Vector uID=null;
        try{
            uID=getIntegerVector(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        //get all of the graderID values (if any).
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        //get all of the writer values (if any).
        Vector wID=null;
        try{
            wID=getLongVector(writerID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetCourseIDQuery gcID=new GetCourseIDQuery(contactInfo,uID,dept,number,section,semester,y,gID,wID,isCurrent);
        ResultWriter out=null;
        try{
            out=doGetID(gcID);
            out.setStatus(true);
            out.setFormat("courseID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetCourseID: "+ex.toString()+
                " With parameters:" +" contactInfo:"+contactInfo +" universityID:"+universityID +
                " dept:"+dept +" number:"+number +" section:"+section +" semester:"+semester +
                " year"+year +" graderID:"+graderID+" isCurrent:"+isCurrent);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gcID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getPaperID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetPaperID(){
	String version=			request.getParameter("paperVersion");
	String writerID=		request.getParameter("writerID");
	String universityID=		request.getParameter("universityID");
	String courseID=		request.getParameter("courseID");
	String graderID=		request.getParameter("graderID");
	String archivalPractice=	request.getParameter("archivalPractice");
        String lastName=		request.getParameter("lastName");
        String studentIdent=		request.getParameter("studentIdent");
        String email=			request.getParameter("email");
        String wag=			request.getParameter("writerAsGraderID");
        String assignID=		request.getParameter("assignID");
        String hasBeenDownloaded=	request.getParameter("hasBeenDownloaded");
        String completed=		request.getParameter("completed");
        
        //fix the non-String values
        Integer v=null;
        if(version!=null && !version.equals("")){
            try{
                v=new Integer(version);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid version format.").toString();
            }
        }
        //package any writerIDs into a Vector.
        Vector wID=null;
        try{
            wID=getLongVector(writerID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        //package any courseIDs into a Vector.
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        //package any universityIDs into a Vector.
        Vector uID=null;
        try{
            uID=getIntegerVector(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        //package any graderIDs into a vector.
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        //package any writerAsGraderIDs into a Vector.
        Vector wagID=null;
        try{
            wagID=getLongVector(wag);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerAsGraderID format.").toString();
        }
        //package any assignIDs into a Vector
        Vector aID=null;
        try{
            aID=getLongVector(assignID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid assignID format.").toString();
        }            
        
        //do the query (remember, null values are okay to pass to Queries)
        GetPaperIDQuery gpID=new GetPaperIDQuery(v,wID,uID,cID,gID,archivalPractice,lastName,studentIdent,email,wagID,limitBy,aID,hasBeenDownloaded,completed);
        ResultWriter out=null;
        try{
            out=doGetID(gpID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("paperID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetPaperID: "+ex.toString()+
                " With parameters:" +" version:"+version+
                " archivalPractice:"+archivalPractice+" universityID:"+universityID +
                " writerID"+writerID +" courseID:"+courseID+ " graderID"+graderID+" writerAsGraderID"+wag+
                " limitBy"+limitBy+"assignID:"+assignID+" hasBeenDownloaded:"+hasBeenDownloaded+" completed:"+completed);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gpID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getPaperUngradedID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetPaperUngradedID(){
        String graderID=		request.getParameter("graderID");
        String lastName=		request.getParameter("lastName");
        String studentIdent=		request.getParameter("studentIdent");
        String email=			request.getParameter("email");
        String courseID=		request.getParameter("courseID");
        String assignID=		request.getParameter("assignID");
        
        //package any graderIDs into a vector.
        if(graderID==null){
            return new ResultWriter(false,"","graderID parameter required.").toString();
        }
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        //package any courseIDs into a Vector.
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        //package any assignIDs into a Vector
        Vector aID=null;
        try{
            aID=getLongVector(assignID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid assignID format.").toString();
        }

        
        //do the query (remember, null values are okay to pass to Queries)
        GetPaperUngradedIDQuery gpID=new GetPaperUngradedIDQuery(gID,lastName,studentIdent,email,cID,aID);
        ResultWriter out=null;
        try{
            out=doGetID(gpID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("paperID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetUngradedPaperID: "+ex.toString()+
                " With parameters:" +" graderID:"+graderID+" courseID:"+courseID+
                " lastName:"+lastName+" studentIdent:"+studentIdent +" email:"+email+" assignID:"+assignID);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gpID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getUniversityID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetUniversityID(){
        String name=		request.getParameter("name");
	String contactInfo=	request.getParameter("contactInfo");
	String currentSemester=	request.getParameter("currentSemester");
	String currentYear=	request.getParameter("currentYear");
        
        //fix the values
        Integer cY=null;
        if(currentYear!=null && !currentYear.equals("")){
            try{
                cY=new Integer(currentYear);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid currentYear format.").toString();
            }
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetUniversityIDQuery guID=new GetUniversityIDQuery(name,contactInfo,currentSemester,cY);
        ResultWriter out=null;
        try{
            out=doGetID(guID);
            out.setStatus(true);
            out.setFormat("universityID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetUniversityID: "+ex.toString()+
                " With parameters:" +" name:"+name + " contactInfo:"+contactInfo +
                " currentSemester:"+currentSemester +" currentYear:"+currentYear);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            guID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getAdminID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetAdminID(){
        String creationDate=		request.getParameter("creationDate");
        String firstName=		request.getParameter("firstName");
        String lastName=		request.getParameter("lastName");
        String userName=		request.getParameter("userName");
        String email=			request.getParameter("email");
        
        //fix the values
        Timestamp cD=null;
        if(creationDate!=null && !creationDate.equals("")){
            Long i=null;
            try{
                i=new Long(creationDate);
                cD=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid creationDate format.").toString();
            }
            
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetAdminIDQuery gaID=new GetAdminIDQuery(cD,firstName,lastName,userName,email);
        ResultWriter out=null;
        try{
            out=doGetID(gaID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("adminID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetAdminID: "+ex.toString()+
                " With parameters:" +" creationDate:"+creationDate +" firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" email:"+email);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gaID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getCourseToGraderID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetCourseToGraderID(){
        String courseID=	request.getParameter("courseID");
	String graderID=	request.getParameter("graderID");
        
        //package any courseIDs into a Vector.
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        //package any graderIDs into a vector.
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetCourseToGraderIDQuery gctgID=new GetCourseToGraderIDQuery(cID,gID);
        ResultWriter out=null;
        try{
            out=doGetID(gctgID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("courseToGraderID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetCourseToGraderID: "
                +ex.toString()+" With parameters: courseID:"+courseID+ " graderID"+graderID);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gctgID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getCourseToWriterID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetCourseToWriterID(){
        String courseID=	request.getParameter("courseID");
	String writerID=	request.getParameter("writerID");
        
        //package any courseIDs into a Vector.
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        //package any writerIDs into a vector.
        Vector wID=null;
        try{
            wID=getLongVector(writerID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetCourseToWriterIDQuery gctwID=new GetCourseToWriterIDQuery(cID,wID);
        ResultWriter out=null;
        try{
            out=doGetID(gctwID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("courseToWriterID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetCourseToWriterID: "
                +ex.toString()+" With parameters: courseID:"+courseID+ " writerID"+writerID);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gctwID.free();
        }
        //return the answer.
        return out.toString();
    }

    /**
     * execute and package a <code>getCheckoutID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetCheckoutID(){
        String graderID=	request.getParameter("graderID");
	String paperID=		request.getParameter("paperID");
	String time=		request.getParameter("time");
        
        //package any graderIDs into a Vector.
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        //package any paperIDs into a vector.
        Vector pID=null;
        try{
            pID=getLongVector(paperID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid paperID format.").toString();
        }
        //package the time string into a Date
        Timestamp t=null;
        if(time!=null && !time.equals("")){
            Long i=null;
            try{
                i=new Long(time);
                t=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid time format.").toString();
            }
            
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetCheckoutIDQuery gcID=new GetCheckoutIDQuery(gID,pID,t,limitBy);
        ResultWriter out=null;
        try{
            out=doGetID(gcID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("checkoutID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetCheckoutID: "
                +ex.toString()+" With parameters: graderID:"+graderID+ " paperID:"+paperID+" time:"+time+" type:"+limitBy);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gcID.free();
        }
        //return the answer.
        return out.toString();
    }

    /**
     * execute and package a <code>getEmailQueueID</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doGetEmailQueueID(){
	String paperID=		request.getParameter("paperID");
	String time=		request.getParameter("time");
        String type=		request.getParameter("type");
        
        //package any paperIDs into a vector.
        Vector pID=null;
        try{
            pID=getLongVector(paperID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid paperID format.").toString();
        }
        //package the time string into a Date
        Timestamp t=null;
        if(time!=null && !time.equals("")){
            Long i=null;
            try{
                i=new Long(time);
                t=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid time format.").toString();
            }
            
        }
        //get type as an Integer
        Integer ty=null;
        if(type!=null && !type.equals("")){
            try{
                ty=new Integer(type);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid type format.").toString();
            }
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetEmailQueueIDQuery geqID=new GetEmailQueueIDQuery(pID,t,ty);
        ResultWriter out=null;
        try{
            out=doGetID(geqID);//The query is done by this method.
            out.setStatus(true);
            out.setFormat("queueID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetEmailQueueID: "
                +ex.toString()+" With parameters: paperID"+paperID+" time:"+time+" type:"+type);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            geqID.free();
        }
        //return the answer.
        return out.toString();
    }

    /**
     * execute and package a <code>getAssignID</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doGetAssignID(){
        String courseID=	request.getParameter("courseID");
	String name=		request.getParameter("name");
	String numDrafts=	request.getParameter("numDrafts");
	String assignNum=	request.getParameter("assignNum");
        
        //fix the values
        Integer nD=null;
        if(numDrafts!=null && !numDrafts.equals("")){
            try{
                nD=new Integer(numDrafts);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid numDrafts format.").toString();
            }
        }
        Integer aN=null;
        if(assignNum!=null && !assignNum.equals("")){
            try{
                aN=new Integer(assignNum);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid assignNum format.").toString();
            }
        }
        
        //get all of the courseID values (if any).
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetAssignIDQuery gaID=new GetAssignIDQuery(cID,name,nD,aN);
        ResultWriter out=null;
        try{
            out=doGetID(gaID);
            out.setStatus(true);
            out.setFormat("assignID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetAssignID: "+ex.toString()+
                " With parameters:" +" courseID:"+courseID +" name:"+name +
                " numDrafts:"+numDrafts +" assignNum:"+assignNum);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gaID.free();
        }
        //return the answer.
        return out.toString();
    }

    /**
     * execute and package a <code>getCourseID</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doGetWriterAsGraderID(){
        //Since the writer_as_grader table contains the IDs of the writer who wrote the paper, and the writer
        //who is grading the paper, we must distinguish between the two.
        //We will assume that the writer who is grading would be searching for his or her papers to grade
        //so "writerID" is the ID of the writer who is grading.
        String draftStructureID=	request.getParameter("draftStructureID");
	String writerAsGraderID=	request.getParameter("writerID");
	String courseID=		request.getParameter("courseID");
        String writerAsWriterID=	request.getParameter("writerAsWriterID");
        String completed=		request.getParameter("completed");
                
        //get all of the draftID values (if any).
        Vector dID=null;
        try{
            dID=getLongVector(draftStructureID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid draftStructureID format.").toString();
        }
        //get all of the grader values (if any).
        Vector gID=null;
        try{
            gID=getLongVector(writerAsGraderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        //get all of the writer values (if any).
        Vector wID=null;
        try{
            wID=getLongVector(writerAsWriterID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerAsWriterID format.").toString();
        }
        //get all of the draftID values (if any).
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetWriterAsGraderIDQuery gwagID=new GetWriterAsGraderIDQuery(dID,gID,wID,cID,completed);
        ResultWriter out=null;
        try{
            out=doGetID(gwagID);
            out.setStatus(true);
            out.setFormat("writerAsGraderID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetWriterAsGraderID: "+ex.toString()+
                " With parameters:" +" draftStructureID:"+draftStructureID +" writerAsWriterID:"+writerAsWriterID +" courseID:"+courseID+
                " writerAsGraderID:"+writerAsGraderID+" completed:"+completed);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gwagID.free();
        }
        //return the answer.
        return out.toString();
    }

    /**
     * execute and package a <code>getDraftStructureID</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doGetDraftStructureID(){
        String draftNum=	request.getParameter("draftNum");
	String assignID=	request.getParameter("assignID");
	String gradingType=	request.getParameter("gradingType");
        String noChangeDate=	request.getParameter("noChangeDate");
        
        //fix the values
        Integer dN=null;
        if(draftNum!=null && !draftNum.equals("")){
            try{
                dN=new Integer(draftNum);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid year format.").toString();
            }
        }
        Timestamp nCD=null;
        if(noChangeDate!=null && !noChangeDate.equals("")){
            try{
                nCD=getDate(noChangeDate);
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid noChangeDate format.").toString();
            }
        }

        
        //get all of the assignID values (if any).
        Vector aID=null;
        try{
            aID=getLongVector(assignID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid assignID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetDraftStructureIDQuery gdsID=new GetDraftStructureIDQuery(dN,aID,gradingType,nCD);
        ResultWriter out=null;
        try{
            out=doGetID(gdsID);
            out.setStatus(true);
            out.setFormat("draftStructureID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.getDraftStructureID: "+ex.toString()+
                " With parameters:" +" draftNum:"+draftNum +" assignID:"+assignID + " gradingType:"+gradingType+
                " noChangeDate:"+noChangeDate);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gdsID.free();
        }
        //return the answer.
        return out.toString();
    }
    
    /**
     * execute and package a <code>getStagingID</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doGetStagingID(){
        String courseID=	request.getParameter("courseID");
	String graderID=	request.getParameter("graderID");
	String paperID=		request.getParameter("paperID");
	String status=		request.getParameter("status");
        
        //get all of the courseID values (if any).
        Vector cID=null;
        try{
            cID=getLongVector(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }
        //get all of the graderID values (if any).
        Vector gID=null;
        try{
            gID=getLongVector(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        //get all of the paperID values (if any).
        Vector pID=null;
        try{
            pID=getLongVector(paperID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        GetStagingIDQuery gsID=new GetStagingIDQuery(cID,gID,pID,status);
        ResultWriter out=null;
        try{
            out=doGetID(gsID);
            out.setStatus(true);
            out.setFormat("stagingID"+TransmissionConstants.ID_SEPARATOR);
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.getStagingID: "+ex.toString()+
                " With parameters:" +" courseID:"+courseID +" graderID:"+graderID +
                " paperID:"+paperID +" status:"+status);
            return dbError.toString();//return the error message...
        }finally{
            //free it if an error occurred, free it if one didn't.
            gsID.free();
        }
        //return the answer.
        return out.toString();
    }

    
    ///////////////////////////////////////////////////
    //
    // Utility Methods
    //
    ///////////////////////////////////////////////////
    
    /**
     * Gets the result set from the {@link GetIDQuery#execute} method, puts the IDs
     * returned into a {@link ResultWriter} that <b>does not have its status or format set</b>.
     * @param query The GetIDQuery to be performed.  The Object is expected to already be instantiated.
     * Note: doGetID does not call the {@link GetIDQuery#free} method, you are expected to do that in a
     * <code>finally block</code>.
     * @return The ResultWriter containing the results of the query in String form (delimitated by 
     * {@link TransmissionConstants#ID_SEPARATOR}.
     */
    private ResultWriter doGetID(GetIDQuery query) throws SQLException{
    //--------------------------SQL HERE-------------------------------
        ResultWriter out=new ResultWriter();
        ResultSet rs=query.execute();
        while(rs.next()){
            out.append(Long.toString(rs.getLong(1)));//String representation of a long
            if(!rs.isLast()){
                out.append(TransmissionConstants.ID_SEPARATOR);
            }
        }
        return out;
    }
    /**
     * pulls any and all ids from the string, and returns them in Long form packaged in a Vector.
     * @param id The String containing all the IDs. (may be null, or empty).
     * @return the IDs packed in a Vector in Long form. (<code>null</code> if param id is null or empty).
     */
    private Vector getLongVector(String id) throws NumberFormatException{
        Vector v=null;
        if(id!=null && !id.equals("")){
            StringTokenizer st=new StringTokenizer(id,TransmissionConstants.ID_SEPARATOR);
            v=new Vector();
            while(st.hasMoreTokens()){
                v.add(new Long(st.nextToken()));
            }
        }
        return v;
    }
    
    /**
     * pulls any and all ids from the string, and returns them in Integer form packaged in a Vector.
     * @param id The String containing all the IDs. (may be null, or empty).
     * @return the IDs packed in a Vector in Integer form. (<code>null</code> if param id is null or empty).
     */
    private Vector getIntegerVector(String id) throws NumberFormatException{
        Vector v=null;
        if(id!=null && !id.equals("")){
            StringTokenizer st=new StringTokenizer(id,TransmissionConstants.ID_SEPARATOR);
            v=new Vector();
            while(st.hasMoreTokens()){
                v.add(new Integer(st.nextToken()));
            }
        }
        return v;
    }
    
    /**
     * pulls a long value from the string and then converts it into a {@link java.sql.Date}.
     * @param s The String containing the data in millis format.
     * @return the Date in Date format.
     */
    private Timestamp getDate(String s) throws NumberFormatException{
        Timestamp d=null;
        if(s!=null && !s.equals("")){
            Long l=null;
            l=new Long(s);
            d=new Timestamp(l.longValue());//make a date from the time in millis
        }
        return d;
    }
    
}
