package edu.gwu.blaise.version2.backend.packagers;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.queries.*;
import edu.gwu.servlets.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*; 
import java.util.Vector;
import java.util.StringTokenizer;
/**
 * Performs a new query and handles the packaging of query data into 
 * a form that is appropriate for transmission.
 * 
 * @author Jared Kiraly
 * @version 2 May 19, 2004
 */
/*
 * Please note:  All Actual SQL-type stuff is marked by //------SQL Here------ tags.
 */
public class PackageNew extends Packager{
    
    public PackageNew(String commandType, HttpServletRequest request, String limitBy){
        super(commandType,request,limitBy);
    }

    public byte[] execute(boolean b){/*this method is useless in this context, but it has to be in here*/
        //this method should never be called, but it has to be here.  try to send an error message with the proper
        //encoding, otherwise send one with whatever encoding we can get
        try{
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes("UTF-8");
        }catch(java.io.UnsupportedEncodingException e){
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes();
        }
    }

    /**
     * Checks to make sure all of the arguments are valid.
     * @return <code>null</code> if the arguments are valid. Otherwise a message to be returned to 
     * the requester stating error.
     */
    private String validate(){
        //first check the request that the request is not null.
        if(request==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No request object in PackageNew.)").toString();
        }
        
        //next check the commandType.
        if(commandType==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No commandType object in PackageNew.)").toString();
        }
        else if(commandType.equals(TransmissionConstants.newWriter)){}
        else if(commandType.equals(TransmissionConstants.newGrader)){}
        else if(commandType.equals(TransmissionConstants.newCourse)){}
        else if(commandType.equals(TransmissionConstants.newUniversity)){}
        else if(commandType.equals(TransmissionConstants.newCourseToGrader)){}
        else if(commandType.equals(TransmissionConstants.newCourseToWriter)){}
        else{
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (Invalid commandType object in PackageNew.)").toString();
        }
        
        //finally check that the limitBy is supplied, that they are not trying to do something that
        //is not allowed, and that they have the required parameters for their request.
        if(limitBy==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No limitBy object in PackageNew.)").toString();
        }
        //writers and graders are not allowed to create courses, etc.
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
            return new ResultWriter(false,"","An access error occurred. Writers and may not perform command:"+
                                                 commandType).toString();
        } 
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
            return new ResultWriter(false,"","An access error occurred. Graders and may not perform command:"+
                                                 commandType).toString();
        } 
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        //the request was made from the admin servlet.  Admins are the only ones who can create new things.
        //in the future, a userName/password might be needed too, here is where the check would be.
        }
        else{
        //it wasn't one of the valid limitBy strings. 
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (Invalid limitBy object in PackageGetID.)").toString();
        }
        
        //everything is okay.  This is one valid request.  GO GO GO!
        BaseServlet.writeLog("PackageNew: "+limitBy+" is doing: "+commandType);
        return null;
    }
    
    
    /**
     * Performs the actual query and packaging.
     * @return The result from the query in a form that is appropriate for transmission.
     */
    public String execute(){
        //first, make sure that this is a valid query.
        String v=validate();
        if(v!=null){
            return v;
        }
        
        if(commandType.equals(TransmissionConstants.newWriter)){
            return doNewWriter();
        }
        else if(commandType.equals(TransmissionConstants.newGrader)){
            return doNewGrader();
        }
        else if(commandType.equals(TransmissionConstants.newCourse)){
            return doNewCourse();
        }
        else if(commandType.equals(TransmissionConstants.newUniversity)){
            return doNewUniversity();
        }
        else if(commandType.equals(TransmissionConstants.newCourseToGrader)){
            return doNewCourseToGrader();
        }
        else if(commandType.equals(TransmissionConstants.newCourseToWriter)){
            return doNewCourseToWriter();
        }
        else{
            return new ResultWriter(false,"","Illegal command").toString();
        }
    }
    
    ////////////////////////////////////////////////////////
    //
    // Query callers
    //
    ////////////////////////////////////////////////////////
    
    /**
     * execute and package a <code>newWriter</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doNewWriter(){
	String universityID=		request.getParameter("universityID");
        String studentIdentifier=	request.getParameter("studentIdentifier");
	String firstName=		request.getParameter("firstName");
	String lastName=		request.getParameter("lastName");
	String userName=		request.getParameter("userName");
        String password=		request.getParameter("password");
        String userNameType=		request.getParameter("userNameType");
        String email=			request.getParameter("email");
        
        //fix the values
        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        NewWriterQuery nw=new NewWriterQuery(uID,studentIdentifier,firstName,
                                                    lastName,email,userName,password,userNameType);
        int result;
        try{
            result=nw.execute();//The query is done here
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageNew.doNewWriter: "+ex.toString()+
                " With parameters:"  +" firstName:"+firstName +
                " lastName:"+lastName+" email:"+email +" userName:"+userName +"password: "+password +" universityID:"+universityID +
                " studentIdentifier"+studentIdentifier);
            return dbError.toString();//return the error message...
        }/*finally{
            //free it if an error occurred, free it if one didn't.
            nw.free();
        }*/
        //return the answer in a form that makes sense to the user.
        return new String("Success: "+result+" entries added.");
    }
    
    
    /**
     * execute and package a <code>newGrader</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doNewGrader(){
        String universityID=		request.getParameter("universityID");
        String firstName=		request.getParameter("firstName");
        String lastName=		request.getParameter("lastName");
        String userName=		request.getParameter("userName");
        String password=		request.getParameter("password");
        String email=			request.getParameter("email");
        String status=			request.getParameter("status");
        String similarityCheckedDate=	request.getParameter("similarityCheckedDate");
        
        //fix the values
        Timestamp sCD=null;
        if(similarityCheckedDate!=null && !similarityCheckedDate.equals("")){
            Long i=null;
            try{
                i=new Long(similarityCheckedDate);
                sCD=new Timestamp(i.longValue());//make a date from the time in millis
            }catch(NumberFormatException e){
                return new ResultWriter(false,"","Invalid similarityCheckedDate format.").toString();
            }
        }

        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        NewGraderQuery ng=new NewGraderQuery(uID,firstName,lastName,userName,password,email,status,sCD);
        int result;
        try{
            result=ng.execute();//The query is done here
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageNew.doNewGrader: "+ex.toString()+
                " With parameters:" +" firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +"password: "+password+" email:"+email +" status:"+status +
                " similarityCheckedDate"+similarityCheckedDate);
            return dbError.toString();//return the error message...
        }/*finally{
            //free it if an error occurred, free it if one didn't.
            ng.free();
        }*/
        //return the answer.
        return new String("Success: "+result+" entries added.");
    }
    
    /**
     * execute and package a <code>NewCourse</code> query.
     * @return the query results in a form appriopriate for transmission to user.
     */
    private String doNewCourse(){
        String contactInfo=	request.getParameter("contactInfo");
	String universityID=	request.getParameter("universityID");
	String dept=		request.getParameter("dept");
	String number=		request.getParameter("number");
	String section=		request.getParameter("section");
	String semester=	request.getParameter("semester");
	String year=		request.getParameter("year");
        String isCurrent=	request.getParameter("isCurrent");
        
        //fix the values
        Integer y=null;
        try{
            y=new Integer(year);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid year format.").toString();
        }
        
        //make universityID a Long (if it exists).
        Integer uID=null;
        try{
            uID=new Integer(universityID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid universityID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        NewCourseQuery nc=new NewCourseQuery(contactInfo,uID,dept,number,section,semester,y,isCurrent);
        int result;
        try{
            result=nc.execute();//The query is done here
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageNew.doNewCourse: "+ex.toString()+
                " With parameters:" +" contactInfo:"+contactInfo +" universityID:"+universityID +
                " dept:"+dept +" number:"+number +" section:"+section +" semester:"+semester +
                " year"+year+" isCurrent:"+isCurrent);
            return dbError.toString();//return the error message...
        }/*finally{
            //free it if an error occurred, free it if one didn't.
            nc.free();
        }*/
        //return the answer.
        return new String("Success: "+result+" entries added.");
    }
    
    
    /**
     * execute and package a <code>newUniversity</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doNewUniversity(){
        String name=		request.getParameter("name");
	String contactInfo=	request.getParameter("contactInfo");
	String currentSemester=	request.getParameter("currentSemester");
	String currentYear=	request.getParameter("currentYear");
        
        //fix the values
        Integer cY=null;
        try{
            cY=new Integer(currentYear);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid currentYear format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        NewUniversityQuery nu=new NewUniversityQuery(name,contactInfo,currentSemester,cY);
        int result;
        try{
            result=nu.execute();//The query is done here
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageNew.doNewUniversity: "+ex.toString()+
                " With parameters:" +" name:"+name + " contactInfo:"+contactInfo +
                " currentSemester:"+currentSemester +" currentYear:"+currentYear);
            return dbError.toString();//return the error message...
        }/*finally{
            //free it if an error occurred, free it if one didn't.
            nu.free();
        }*/
        //return the answer.
        return new String("Success: "+result+" entries added.");
    }
    
    
    /**
     * execute and package a <code>newCourseToGrader</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doNewCourseToGrader(){
        String courseID=	request.getParameter("courseID");
	String graderID=	request.getParameter("graderID");
        
        //fix the values
        Long cID=null;
        try{
            cID=new Long(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }

        Long gID=null;
        try{
            gID=new Long(graderID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid graderID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        NewCourseToGraderQuery nctg=new NewCourseToGraderQuery(cID,gID);
        int result;
        try{
            result=nctg.execute();//The query is done here
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageGetID.doGetCourseToGraderID: "
                +ex.toString()+" With parameters: courseID:"+courseID+ " graderID"+graderID);
            return dbError.toString();//return the error message...
        }/*finally{
            //free it if an error occurred, free it if one didn't.
            nctg.free();
        }*/
        //return the answer.
        return new String("Success: "+result+" entries added.");
    }
    
    /**
     * execute and package a <code>newCourseToWriter</code> query.
     * @return the query results in a form that is okay for transmission to user.
     */
    private String doNewCourseToWriter(){
        String courseID=	request.getParameter("courseID");
	String writerID=	request.getParameter("writerID");
        
        //package any courseIDs into a Vector.
        Long cID=null;
        try{
            cID=new Long(courseID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid courseID format.").toString();
        }

        Long wID=null;
        try{
            wID=new Long(writerID);
        }catch(NumberFormatException e){
            return new ResultWriter(false,"","Invalid writerID format.").toString();
        }
        
        //do the query (remember, null values are okay to pass to Queries)
        NewCourseToWriterQuery nctw=new NewCourseToWriterQuery(cID,wID);
        int result;
        try{
            result=nctw.execute();//The query is done here
        }catch(SQLException e){//an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageNew.doNewCourseToWriter: "
                +ex.toString()+" With parameters: courseID:"+courseID+ " writerID"+writerID);
            return dbError.toString();//return the error message...
        }/*finally{
            //free it if an error occurred, free it if one didn't.
            nctw.free();
        }*/
        //return the answer.
        return new String("Success: "+result+" entries added.");
    }

    
    ///////////////////////////////////////////////////
    //
    // Utility Method
    //
    ///////////////////////////////////////////////////
        
    /**
     * pulls a long value from the string and then converts it into a {@link java.sql.Date}.
     * @param s The String containing the data in millis format.
     * @return the Date in Date format.
     */
    private Timestamp getDate(String s) throws NumberFormatException{
        Timestamp d=null;
        if(s!=null && !s.equals("")){
            Long l=null;
            l=new Long(s);
            d=new Timestamp(l.longValue());//make a date from the time in millis
        }
        return d;
    }    
}
