package edu.gwu.blaise.version2.backend.packagers;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.queries.*;
import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.servlets.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*; 
import java.util.Vector;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.io.*;
import javax.mail.MessagingException;
import java.util.*;
/**
 * Performs most paper-related queries and handles the packaging of query data into 
 * a form that is appropriate for transmission - uses DualByteArray.
 * 
 * @author Michael Gaiman 
 * @author Jared Kiraly
 * @version 2 Jun 3, 2004
 */
/*
 * Please note:  All Actual SQL-type stuff is marked by //------SQL Here------ tags.
 */
public class PackagePaper extends Packager{
    private File tempDir=null;//used for parsing papers.
    Properties p=null;//used instead of req.getParameter()
    Blaise_Paper bp=null;//was already gotten by the servlet, if it exists
    /**
      * @param tempDir the directory temporary files should be written to.
      * @See Packager
      */
    public PackagePaper(String commandType, HttpServletRequest request, String limitBy,File tempDir,Properties p,Blaise_Paper bp){
        super(commandType,request,limitBy);
        this.tempDir=tempDir;
        this.p=p;
        this.bp=bp;
    }
    
    public String execute(){/*this method is useless in this context, but it has to be in here*/
        return new ResultWriter(false,"","Error: PackagePaper called from a non binary context.").toString();
    }
    
    /**
     * Performs the actual query and packaging.
     * @return The result from the query in a form that is appropriate for transmission.
     * The boolean variable parameter isn't used, it is just here to distinguish this method from execute()
     */
    public byte[] execute(boolean b){
        BaseServlet.writeLog("PackagePaper: "+limitBy+" is doing: "+commandType);        
        //first, make sure that this is a valid query.
        if(request==null){
            return sendMessage(new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No request object in PackagePaper.)"));
        }
        if(commandType==null){
            return sendMessage(new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (No commandType object in PackagePaper.)"));
        }
        //check to see that it is one of the three valid limitBys
        if(limitBy==null){
            return sendMessage(new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (No limitBy object in PackagePaper.)"));
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        }
        else{
            return sendMessage(new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (invalid limitBy object in PackagePaper.)"));
        }
        
        //passed to the query
        Long limitID=null;
        Vector pIDs=null;
    
        //now handle the request (also check for invalid commandType, and invalid commandType/limitBy combination)
        if(commandType.equals(TransmissionConstants.getPaperHeader)){
            //check for valid credentials.  Then get the PaperIDs they supplied (if any).
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return  sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("graderID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid graderID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one graderID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                //no limitID needed.
            }
            try{
                pIDs=getLongVector(p.getProperty("paperID"));//null okay
            }catch(NumberFormatException e){
                return sendMessage(new ResultWriter(false,"","Invalid paperID format."));
            }
            return doAssemblePaper(pIDs,limitBy,limitID,true);//just the headers
        }
        else if(commandType.equals(TransmissionConstants.checkOutPaper)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("graderID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid graderID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one graderID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Administrators may not perform desired action."));
            }
            
            //get the paperIDs first
            try{
                pIDs=getLongVector(p.getProperty("paperID"));
            }catch(NumberFormatException e){
                return sendMessage(new ResultWriter(false,"","Invalid paperID format."));
            }
            if(pIDs==null){
                try{
                    pIDs=getPaperIDVector(limitBy,limitID);
                }catch(SQLException e){
                    ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                    dbError.append(e.getMessage());
                    StringBuffer ex=new StringBuffer(e.getMessage());
                    while((e=e.getNextException())!=null){
                        ex.append(e.getMessage());
                    }
                    //from edu.gwu.servlets
                    BaseServlet.writeDebug(
                        "Database error occurred in PackagePaper.getPaperIDVector: "+ex.toString()+
                        " With parameters:" +" limitBy:"+limitBy+" limitID:"+limitID);
                    return sendMessage(dbError);//return the error message...
                }
            }
            //then do the checkout.
            return doCheckOutPaper(pIDs,limitID);//moved into a function to make things a little cleaner.
        }
        else if(commandType.equals(TransmissionConstants.checkInPaper)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("graderID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid graderID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one graderID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Administrators may not perform desired action."));
            }

            if(bp==null){
                return sendMessage(new ResultWriter(false,"","A transmission error occurred. Required parameter \"paper\" not found.\nPaper not checked in"));
            }
            return doCheckInPaper(bp,limitID);
        }
        else if(commandType.equals(TransmissionConstants.getPaperFull)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("graderID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid graderID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one graderID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Administrators may not perform desired action."));
            }
            try{
                pIDs=getLongVector(p.getProperty("paperID"));//if null, get all papers by limitID.
            }catch(NumberFormatException e){
                return sendMessage(new ResultWriter(false,"","Invalid paperID format."));
            }
            return doAssemblePaper(pIDs,limitBy,limitID,false);
        }
        else if(commandType.equals(TransmissionConstants.submitPaper)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Graders may not perform desired action."));
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Administrators may not perform desired action."));
            }

            return doSubmitPaper(bp);
        }
        else if(commandType.equals(TransmissionConstants.stagePaper)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("graderID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid graderID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one graderID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Administrators may not perform desired action."));
            }

            if(bp==null){
                return sendMessage(new ResultWriter(false,"","A transmission error occurred. Required parameter \"paper\" not found.\nPaper not checked in"));
            }
            return doStagePaper(bp,limitID);
        }
        else if(commandType.equals(TransmissionConstants.sendStagedPaper)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("writerID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid writerID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one writerID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=null;
                try{
                    v=getLongVector(p.getProperty("graderID"));
                }catch(NumberFormatException e){
                    return sendMessage(new ResultWriter(false,"","Invalid graderID format."));
                }
                if(v==null || v.size()!=1){
                    return sendMessage(new ResultWriter(false,"","Exacly one graderID required."));
                }
                else{
                    limitID=(Long)v.firstElement();
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                return sendMessage(new ResultWriter(false,"","Access Denied.  Administrators may not perform desired action."));
            }

            Vector sIDs=null;
            try{
                sIDs=getLongVector(p.getProperty("stagingID"));
            }catch(NumberFormatException e){
                return sendMessage(new ResultWriter(false,"","Invalid paperID format."));
            }
            return doSendStagedPaper(sIDs,limitID);
        }
        else{ //invalid commandType for this class.
            return sendMessage(new ResultWriter(false,"","Invalid command: "+commandType));
        }
    }
    
    /**
     * Handles the actual pulling the paper-pieces from the database and assembling them.
     * @param pIDs The paperIDs of the papers to get. (<code>null</code> if all papers, limited by limitID, 
     * should be returned.)
     * @param limitBy Who is performing the query (writer,grader,admin).
     * @param limitID the ID of the person performing the query.
     * @param headerOnly Include only the header of the paper, or the full paper with content.
     * @return A ResultWriter containing either the papers, or an appriopriate error.
     */
    private byte[] doAssemblePaper(Vector pIDs,String limitBy,Long limitID,boolean headerOnly){
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("doAssemblePaper called with parameters, pIDs:"+pIDs+" limitBy:"+limitBy+
                " limitID:"+limitID+" headerOnly:"+headerOnly);
        }
        try{
            if(pIDs==null){///get all of the papers the user is allowed to get.
                pIDs=getPaperIDVector(limitBy,limitID);
            }
            Blaise_PaperVectorGetter getter = new Blaise_PaperVectorGetter(pIDs,limitBy,limitID);
            Vector dIDs=getDraftIDVector(pIDs);
            Vector cIDs=getCommentIDVector(pIDs);
            Vector papers=getter.getBlaise_PaperVector();
            Vector drafts=getBlaise_DraftVector(dIDs,limitBy,limitID);
            Vector comments=getBlaise_CommentVector(cIDs,limitBy,limitID);
            
            long currentPID;
            Blaise_Paper currentP;
            long currentDID;
            Blaise_Draft currentD;
            Blaise_Comment currentC;
            Blaise_Draft workingD;
            boolean isInStaging=false;
            
            int papersSize=papers.size();
            
            //for each paperID
            for(int paperIndex=0;paperIndex<papersSize;paperIndex++){
                currentP=(Blaise_Paper)papers.get(paperIndex);
                currentPID=Long.parseLong(currentP.getPaperID());
                if(limitBy.equals(PackagingConstants.LIMIT_writer) && commandType.equals(TransmissionConstants.getPaperFull)){
                    SetHasBeenDownloadedQuery shbdq=null;
                    shbdq=new SetHasBeenDownloadedQuery(new Long(currentPID),"Y");
                    int hbd=shbdq.execute();
                }
                workingD=currentP.getWorkingDraft();
                //for each draftID
                for(int draftIndex=0;draftIndex<drafts.size();draftIndex++){
                    currentD=(Blaise_Draft)drafts.get(draftIndex);
                    if(Long.parseLong(currentD.getPaperID())==currentPID){
                        currentDID=Long.parseLong(currentD.getDraftID());
                        //for each commentID
                        for(int commentIndex=0;commentIndex<comments.size();commentIndex++){
                            currentC=(Blaise_Comment)comments.get(commentIndex);
                            if(Long.parseLong(currentC.getDraftID())==currentDID){
                                //remove the body information if it is headerOnly
                                if(headerOnly){
                                    currentC.setBody("");//clear out body
                                    currentC.setScribbles(new Vector());
                                    currentC.setCorrections(new Vector());
                                }
                                //add this comment to the current draft
                                currentD.addComment(currentC);
                                //remove this comment from the comments Vector
                                comments.remove(commentIndex);
                                commentIndex--;//subtract 1 from index so that when it is incremented
                                               //1 in the for, it will be back at current position, which
                                               //will be the next element as this one was removed and all were
                                               //shifted left one.
                            }
                        }
                        //all comments for this draft have been assembled.  Now finish the draft and add it to the
                        //current paper.
                        if(headerOnly){
                            currentD.setBody("");
                        }
                        currentP.addDraft(currentD);
                        drafts.remove(draftIndex);
                        draftIndex--;//subtract 1 from index so that when it is incremented
                                     //1 in the for, it will be back at current position, which
                                     //will be the next element as this one was removed and all were
                                     //shifted left one.
                    }
                }
            }
            
            //now we have all of the papers in a vector of Blaise_Paper.
            //convert it to sizeNDualByteArray.
            Properties p=new Properties();
            p.setProperty("status", TransmissionConstants.SUCCESS);
            DualByteArray dba=new DualByteArray(p);
            Blaise_Paper pap=null;
            byte[] pape=null;
            for(int i=0;i<papers.size();i++){
                pap=(Blaise_Paper)papers.elementAt(i);
                try{
                    pape=BlaiseFile.paperToByteArray(pap);
                    dba.add(pape);
                }catch(IOException e){
                    StringBuffer s=new StringBuffer();
                    BaseServlet.writeLog("BlaiseFile error occurred in PackagePaper.doAssemblePaper: "+
                    e+" With parameters: pIDs:"+pIDs+" limitBy:"+limitBy+" limitID:"+limitID+" headerOnly:"+headerOnly);
                    return sendMessage(new ResultWriter(false,"","A file error has occurred."));
                }  
            }                
            return dba.getSizeNDualByteArray();
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog(
                "Database error occurred in PackagePaper.doAssemblePaper: "+ex.toString()+
                " With parameters:" +" pIDs:"+pIDs+" limitBy:"+limitBy+" limitID:"+limitID+
                " headerOnly:"+headerOnly);
            return sendMessage(dbError);//return the error message...
        }catch(NumberFormatException n){
            //from edu.gwu.servlets
            BaseServlet.writeLog("NumberFormatException:"+n+" in PackagePaper.doAssemblePaper."+
                " With parameters:" +" pIDs:"+pIDs+" limitBy:"+limitBy+" limitID:"+limitID+
                " headerOnly:"+headerOnly);
            return sendMessage(new ResultWriter(false,"","A database number formatting error occurred. Please notify an administrator."));
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
    }
    
    /**
     * Performs all the steps required to check-out papers.
     * @param pIDs The paperIDs of the papers to get.
     * @param limitID the ID of the grader checking out the papers
     * @return A ResultWriter containing either the papers, or an appriopriate error.
     */ 
    private byte[] doCheckOutPaper(Vector pIDs, Long limitID){
        //first check that none of the paper's that are requested are already checked out by the grader.
        try{
            if(isCheckedOut(pIDs,limitID)){
                //already checked-out.
                return sendMessage(new ResultWriter(false,"","You have already checked-out one or more of the requested papers."));
            }
        }catch(SQLException e){
            //an error occurred in isCheckedOut.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.isCheckedOut: "+ex.toString());
            return sendMessage(dbError);//return the error message...
        }
        
        //now, get them.
        byte[] papers=doAssemblePaper(pIDs,limitBy,limitID,false);
        byte[] first=DualByteArray.extractFirst(papers);
        Properties props=DualByteArray.toProperties(first);
        //if it was successful, add them to checkout table
        if(props.getProperty("status").equals(TransmissionConstants.FAILURE)){
            //there was some sort of error... so return it to the user without completing
            return papers;
        }
        
        //add papers to the checkout table
        byte[] error=doCheckout(pIDs,limitID);
        
        //if there was a checkout error, return it, else return the papers.
        if(error==null){
            return papers;
        }else{
            return error;
        }
    }
    
    /**
     * Adds entries to the checkout table in the database showing that the specified grader has
     * checked out the specified papers to grade.
     * @param pIDs A Vector containing the paperIDs of the papers should be added.
     * @param graderID the grader who has checked the papers out to grade.
     * @return A String containing any errors encountered.  <code>null</code> if no errors were encountered.
     */ 
    private byte[] doCheckout(Vector pIDs, Long graderID){
        int size=pIDs.size();
        try{
            for(int i=0;i<size;i++){
                NewCheckOutQuery ncoq=null;
                ncoq=new NewCheckOutQuery(graderID,(Long)pIDs.get(i),limitBy);
                ncoq.execute();
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doCheckout: "+ex.toString()+
                " With parameters:" +" pIDs:"+pIDs+" graderID:"+graderID+" type:"+limitBy);
            return sendMessage(dbError);//return the error message...
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        return null;//everything was okay
    }
    
    /**
     * Performs the check in of the papers by adding the comments and removing the checkout entries.
     * @param bp a Blaise_Paper to checkin.
     * @param graderID The graderID of the grader who wrote these comments.
     * @return A String containing either errors or an okay, in a form okay to transfer to the user.
     */
    private byte[] doCheckInPaper(Blaise_Paper bp,Long graderID){
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("doCheckInPaper: parameters, graderID:"+graderID);
        }
        
        Long paperID;
        try{
            paperID=new Long(bp.getPaperID());
        }catch(NumberFormatException e){
            BaseServlet.writeLog(
            "NumberFormatException "+e+" in PackagePaper.doCheckInPaper (invalid paperID) with"+
            " parameters: paperID:"+bp.getPaperID());
            return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper.  Please contact an administrator."));
        }
        
        //check that it has been checked out--don't allow an unchecked out paper to be checked in,
        //as most likely it is a double click on the client or something like that.
        //if it is checked out, then delete the checkout.
        GetCheckoutIDQuery gcq=null;
        DeleteCheckoutQuery dcq=null;
        Vector g=new Vector();
        Vector p=new Vector();
        g.add(graderID);
        p.add(paperID);
        try{
            //------------------SQL HERE---------------------------
            gcq=new GetCheckoutIDQuery(g,p,null,limitBy);
            ResultSet rs=gcq.execute();
            Long checkoutID=null;
            if(rs.next()){
                checkoutID=new Long(rs.getLong(1));
            }
            
            
            if(checkoutID!=null){
                Vector v=new Vector();
                v.add(checkoutID);
                dcq=new DeleteCheckoutQuery(v);
                dcq.execute();
            }
            else{
                //this is bad, a paper was checked in that wasn't checked out.
                BaseServlet.writeLog(
                    "PackagePaper.doCheckInPaper: A paper was checked in that wasn't checked out."+
                    " With paperID:"+p+" and graderID:"+g);
                return sendMessage(new ResultWriter(false,"","Invalid: You are checking a paper that wasn't checked out."));
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog(
            "Database error occurred in PackagePaper.doCheckInPaper (checkout table related)"+
            ": "+ex.toString()+" With parameters:" +" graderID:"+g+" paperID:"+p+
            " PLEASE CHECK DATABASE FOR INTEGRITY.");
            return sendMessage(dbError);//return the error message...
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }finally{
            if(gcq!=null){
                gcq.free();
            }
        }

        Vector drafts=bp.getDrafts();
        Vector comments=null;
        Blaise_Comment bc=null;
        Blaise_Draft bd=null;
        int dSize=drafts.size();
        int cSize;
        int cIndex;
        String graderIDString=graderID.toString();
        
        Long draftID;
        Long commentID;
        Boolean isPublic;
        String ip;
        String cID;
        NewCommentQuery ncq;
        EditCommentQuery ecq;
        for(int dIndex=0;dIndex<dSize;dIndex++){
            //for each draft, get the comments.
            bd=(Blaise_Draft)drafts.get(dIndex);
            
            if(!bd.getIsModified().equals("YES")){
                //the grader's comment for this draft has not changed, so ignore it.
                continue;
            }
            comments=bd.getComments();
            cSize=comments.size();
            for(cIndex=0;cIndex<cSize;cIndex++){
                //for each comment in the draft.
                bc=(Blaise_Comment)comments.get(cIndex);
            
                if(!bc.getGraderID().equals(graderIDString)){
                    //it is by a different grader, so ignore it.
                    continue;
                }
                
                //now, if it is a new comment, insert it.  If it is an existing comment, updated it.
                
                //get the data from the Blaise_Comment
                try{
                    //paperID=new Long(bc.getPaperID());
                    draftID=new Long(bc.getDraftID());
                    graderID=new Long(bc.getGraderID());
                    cID=bc.getCommentID();
                    if(cID.equals("")){//new comment
                        commentID=null;
                    }
                    else{//existing comment
                        commentID=new Long(cID);
                    }
                    ip=bc.getIsPublic();
                    if(ip.equals("YES") || ip.equals("1")){
                        isPublic=new Boolean(true);
                    }else if(ip.equals("NO") || ip.equals("0")){
                        isPublic=new Boolean(false);
                    }else{
                        BaseServlet.writeLog("Invalid isPublic submitted in PackagePaper.doCheckInPaper."+
                        " isPublic:"+ip);
                        return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper (isPublic). Please contact an administrator."));
                    }
                }catch(NumberFormatException e){
                    BaseServlet.writeLog("NumberFormatException "+e+" in PackagePaper.doCheckInPaper with"+
                    " parameters:"+" paperID:"+bc.getPaperID()+" draftID:"+bc.getDraftID()+" commentID:"+
                    bc.getCommentID()+" graderID:"+bc.getGraderID());
                    return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper.  Please contact an administrator."));
                }

                if(commentID==null){
                    //So if we're here, then the comment is seemingly valid, and isn't already in the database.
                    //so add it.
                    if(BaseServlet.debugOn()){
                        BaseServlet.writeDebug("Adding new comment to database for draftID:"+draftID);
                    }
                    try{
                        ncq=new NewCommentQuery(bc,paperID,draftID,graderID,BlaiseFile.commentToByteArray(bc),isPublic);
                        ncq.execute();
                    }catch(SQLException e){
                        ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                        dbError.append(e.getMessage());
                        StringBuffer ex=new StringBuffer(e.getMessage());
                        while((e=e.getNextException())!=null){
                            ex.append(e.getMessage());
                        }
                        //from edu.gwu.servlets
                        BaseServlet.writeLog(
                        "Database error occurred in PackagePaper.doCheckInPaper (NewCommentQuery): "+
                        ex.toString()+" With parameters: draftID:"+draftID+" graderID"+graderID+
                        " PLEASE CHECK COMMENT TABLE FOR INTEGRITY.");
                        return sendMessage(dbError);//return the error message...
                    }catch(IOException e){
                        BaseServlet.writeLog(
                        "BlaiseFile error occurred in PackagePaper.getBlaise_CommentVector: "+
                        e+" With parameter: draftID:"+draftID+" graderID:"+graderID);
                    }catch(NoConnectionException e){
                        return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                    }  

                    //if this is a peer graded paper, then change the completed status in the writer as grader table
                    //only need to change it to completed when this writer as grader adds a new comment, in case one student
                    //grades multiple drafts of the same paper
                    if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                        //first get the draft structure IDs for this draft
                        Vector dsids=new Vector();
                        try{
                            Vector v=new Vector();
                            v.add(new Long(bp.getAssignmentID()));
                            GetDraftStructureIDQuery gdsidq=new GetDraftStructureIDQuery(new Integer(bd.getDraftNumber()),v,"Peer",null);
                            ResultSet rs=gdsidq.execute();
                            while(rs.next()){
                                dsids.add(new Long(rs.getLong(1)));
                            }
                        }catch(SQLException e){
                            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                            dbError.append(e.getMessage());
                            StringBuffer ex=new StringBuffer(e.getMessage());
                            while((e=e.getNextException())!=null){
                                ex.append(e.getMessage());
                            }
                            //from edu.gwu.servlets
                            BaseServlet.writeLog(
                            "Database error occurred in PackagePaper.doCheckInPaper (GetDraftStructureIDQuery): "+
                            ex.toString()+" With parameters: graderID"+graderID);
                            return sendMessage(dbError);//return the error message...
                        }catch(NumberFormatException e){
                            BaseServlet.writeLog("NumberFormatException thrown in PackagePaper with parameters: graderID"+graderID);
                            return sendMessage(new ResultWriter(false,"","Number format error when attempting to set status flag.  Please notify an administrator."));
                        }
                        
                        //now that we have the draft structure IDs, get the rowIDs
                        Vector rowids=new Vector();
                        try{
                            Vector v=new Vector();
                            v.add(new Long(bp.getAssignmentID()));
                            Vector wagID=new Vector();
                            Vector wawID=new Vector();
                            Vector coID=new Vector();
                            wagID.add(graderID);
                            wawID.add(new Long(bp.getWriterID()));
                            coID.add(new Long(bp.getCourseID()));
                            GetWriterAsGraderIDQuery gwagidq=new GetWriterAsGraderIDQuery(dsids,wagID,wawID,coID,"N");
                            ResultSet rs=gwagidq.execute();
                            while(rs.next()){
                                rowids.add(new Long(rs.getLong(1)));
                            }
                        }catch(SQLException e){
                            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                            dbError.append(e.getMessage());
                            StringBuffer ex=new StringBuffer(e.getMessage());
                            while((e=e.getNextException())!=null){
                                ex.append(e.getMessage());
                            }
                            //from edu.gwu.servlets
                            BaseServlet.writeLog(        
                            "Database error occurred in PackagePaper.doCheckInPaper (GetWriterAsGraderIDQuery): "+
                            ex.toString()+" With parameters: graderID"+graderID);
                            return sendMessage(dbError);//return the error message...
                        }catch(NumberFormatException e){
                            BaseServlet.writeLog("NumberFormatException thrown in PackagePaper with parameters: graderID"+graderID);
                            return sendMessage(new ResultWriter(false,"","Number format error when attempting to set status flag.  Please notify an administrator."));
                        }
                    
                        //finally, make the changes to the status field in the writer as grader table
                        EditWriterAsGraderQuery ewagidq=null;
                        Long rowid=null;
                        for(int i=0;i<rowids.size();i++){
                            rowid=(Long)rowids.elementAt(i);
                            try{
                                ewagidq=new EditWriterAsGraderQuery(rowid,null,null,null,null,"Y");
                                int j=ewagidq.execute();
                            }catch(NoConnectionException e){
                                return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                            }catch(SQLException e){
                                ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                                dbError.append(e.getMessage());
                                StringBuffer ex=new StringBuffer(e.getMessage());
                                while((e=e.getNextException())!=null){
                                    ex.append(e.getMessage());
                                }
                                //from edu.gwu.servlets
                                BaseServlet.writeLog(
                                "Database error occurred in PackagePaper.doCheckInPaper (EditWriterAsGraderQuery): "+
                                ex.toString()+" With parameters: graderID"+graderID);
                                return sendMessage(dbError);//return the error message...
                            }
                        }
                    }//end editing writer as grader table
                }
                else{//is not a peer graded paper
                    //update the existing comment.
                    if(BaseServlet.debugOn()){
                        BaseServlet.writeDebug("updating comment with commentID:"+commentID);
                    }
                    try{
                        ecq=new EditCommentQuery(commentID,null,null,null,BlaiseFile.commentToByteArray(bc),isPublic);
                        ecq.execute();
                    }catch(SQLException e){
                        ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                        dbError.append(e.getMessage());
                        StringBuffer ex=new StringBuffer(e.getMessage());
                        while((e=e.getNextException())!=null){
                            ex.append(e.getMessage());
                        }
                        //from edu.gwu.servlets
                        BaseServlet.writeLog(
                        "Database error occurred in PackagePaper.doCheckInPaper (EditCommentQuery): "+
                        ex.toString()+" With parameters: commentID:"+commentID+" graderID"+graderID);
                        return sendMessage(dbError);//return the error message...
                    }catch(IOException e){
                        BaseServlet.writeLog(
                        "BlaiseFile error occurred in PackagePaper.doCheckInPaper: "+
                        e+" With parameters: commentID:"+commentID+" graderID:"+graderID);
                    }catch(NoConnectionException e){
                        return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                    }  
                }
            }//end comment for loop
        }//end draft for loop

        //change the has been downloaded status in the db
        SetHasBeenDownloadedQuery shbdq=null;
        try{
            shbdq=new SetHasBeenDownloadedQuery(paperID,"N");
            int exe=shbdq.execute();
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
            " Your grade has been submitted, but an error occurred when notifying student of download status. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doCheckInPaper"+
            " (during set has been downloaded step): "+ex.toString()+
            " With parameters:" +" paperID:"+paperID+" graderID"+graderID);
            return sendMessage(dbError);//return the error message...
        }        
        
            
        //Now, go ahead and notify the writer that their paper has been graded.
        //do emailQueue first, so don't need as much error notification
        //don't put it in email queue if has been peer graded, only notify when a grader grades

        try{
            NewEmailQueueQuery neqq=null;
            if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                neqq=new NewEmailQueueQuery(paperID,new Integer(2),graderID); 
                neqq.execute();
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
            " Your grade has been submitted, but an error occurred when sending graded paper to student. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doCheckInPaper"+
            " (during send paper step): "+ex.toString()+
            " With parameters:" +" paperID:"+paperID+" graderID"+graderID);
            return sendMessage(dbError);//return the error message...
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        return sendMessage(new ResultWriter(true,"","Graded paper has been added to database."));
    }
    
    /**
     * Takes a Blaise_Paper in byte array form and adds new bits to the database.
     * @return The IDs of the new bits in a form that is okay to transmit to user application.
     */
    private byte[] doSubmitPaper(Blaise_Paper bp){
        StringBuffer format=new StringBuffer();
        ResultWriter result=new ResultWriter();
        
        //now we've got the paper, do the adding to db.
        Integer version=null;
        Long writerID=null;
        Integer universityID=null;
        Long courseID=null;
        try{
            String pIDS=bp.getPaperID();
            if(pIDS.equals("")){
                //this is a new paper so a new row in the paper table needs to be added.
                String s=bp.getVersion();
                version=new Integer(s);
                s=bp.getWriterID();
                writerID=new Long(s);
                s=bp.getUniversityID();
                universityID=new Integer(s);
                s=bp.getCourseID();
                courseID=new Long(s);
                Long aID=new Long(bp.getAssignmentID());
                NewPaperQuery npq=null;
                try{
                    npq=new NewPaperQuery(version,writerID,universityID,courseID,"standard",aID,"Y");
                }catch(NoConnectionException e){
                    return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                }
                npq.execute();
                //now put the paperID in the ResultWriter, and the drafts.
                Long pID=npq.getLastInsertID();
                result.append(pID.toString()+TransmissionConstants.ID_SEPARATOR);
                format.append("paperID"+TransmissionConstants.ID_SEPARATOR);
                Vector drafts=bp.getDrafts();
                for(int i=0;i<drafts.size();i++){
                    Blaise_Draft bd=(Blaise_Draft)drafts.get(i);
                    bd.setPaperID(pID.toString());
                }
            }else{
                //do some checking to make sure this paper lines up with the existing one
                GetPaperIDQuery gpIDq=null;
                try{
                    Long paperID=new Long(pIDS);
                    Vector v=new Vector();
                    writerID=new Long(bp.getWriterID());
                    v.add(writerID);
                    gpIDq=new GetPaperIDQuery(null,v,null,null,null,null,null,null,null,null,null,null,null,null);
                    ResultSet rs=gpIDq.execute();
                    
                    boolean isOne=false;
                    long newID;
                    long thePID=paperID.longValue();
                    while(rs.next()){
                        newID=rs.getLong(1);
                        if(newID==thePID){
                            isOne=true;
                            break;
                        }
                    }
                    
                    if(!isOne){
                        //this paper was authored by a different writer to start (or a database corruption issue)
                        BaseServlet.writeLog("A paper was submitted that had already been submitted by a"+
                        " different writter (or a database corruption).  PaperID:"+pIDS+
                        ", this writerID:"+writerID);                        
                        return sendMessage(new ResultWriter(false,"","The paper was not submitted because it was submitted earlier by a different writer."));
                    }                        
                }catch(NoConnectionException e){
                    return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                }finally{
                    if(gpIDq!=null){
                        gpIDq.free();
                    }
                }
                 
                //If the assignment number of the submitted paper doesn't match the one in the db, we should change the one in the db
                Long paperID=new Long(pIDS);
                Vector v=new Vector();
                writerID=new Long(bp.getWriterID());
                v.add(paperID);        
                ResultSet rrs=null;
                String assignID;
                GetQuery gq=null;
                EditPaperQuery epq=null;
                gq=new GetQuery(v,PackagingConstants.TABLE_paper,PackagingConstants.LIMIT_writer,writerID);
                rrs=gq.execute();
                if(rrs.next()){
                    assignID=Long.toString(rrs.getLong("assignID"));
                    if(!assignID.equals(bp.getAssignmentID())){
                        epq=new EditPaperQuery(paperID,new Long(assignID));
                        int tableNum=epq.execute();
                        if(tableNum!=1){
                            return sendMessage(new ResultWriter(false,"","The paper was not submitted due to a database error: the assignment number of the paper has been incorrectly reassigned."));
                        }
                    }
                }
                else{
                    BaseServlet.writeLog("Database error: paper "+bp.getPaperID()+" could not be gotten.");
                    return sendMessage(new ResultWriter(false,"","The paper was not submitted due to a database error: paper could not be retrieved."));
                }
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doSubmitPaper (NewPaperQuery/GetQuery)"+
            ": "+ex.toString()+" With parameter: version:"+version+" writerID:"+writerID+
            " universityID:"+universityID+" courseID:"+courseID+
            " PLEASE CHECK DATABASE FOR INTEGRITY.");
            return sendMessage(dbError);//return the error message...
        }catch(NumberFormatException e){
            BaseServlet.writeLog(
            "NumberFormatException "+e+" in PackagePaper.doSubmitPaper (NewPaperQuery/GetQuery) with"+
            " parameters:"+" paperID:"+bp.getPaperID()+" version:"+bp.getVersion()+
            " writerID:"+bp.getWriterID()+
            " universityID:"+bp.getUniversityID()+" courseID:"+bp.getCourseID());
            return sendMessage(new ResultWriter(false,"","Invalid fields found in submitted paper.  Please contact an administrator."));
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }                        
        
        //Now add any new drafts.
        format.append("draftID"+TransmissionConstants.ID_SEPARATOR);
        Long paperID=null;
        Integer draftNumber=null;//we don't care about the reported draftNumber, as it can be wrong
                                    //if multiple copies of the paper exist.
        int dNum=0;
        String title=null;
        byte[] body=null;
        Timestamp sqlDate=null;
        Blaise_Draft bd=null;
        try{
            Vector drafts=bp.getDrafts();
            int dSize=drafts.size();
            String dIDS;
            NewDraftQuery ndq;
            String s;
            String type;
            java.util.Date utilDate;
            SimpleDateFormat df=new SimpleDateFormat(TransmissionConstants.dateFormat);
            long dMillis;
            Long draftID;
            for(int dIndex=0;dIndex<dSize;dIndex++){
                bd=(Blaise_Draft)drafts.get(dIndex);
                dIDS=bd.getDraftID();
                if(dIDS.equals("")){//this is a new draft.
                    //pull all the info from the Blaise_Draft and format it into proper objects (from String)
                    s=bd.getPaperID();
                    paperID=new Long(s);
                    //s=bd.getDraftNumber();
                    //draftNumber=new Integer(s);
                    dNum=(getNumberOfDrafts(paperID))+1;
                    draftNumber=new Integer(dNum);
                    title=bd.getTitle();
                    type=bd.getExternalBodyFilename();
                    if(type==null){
                        type="Blaise";
                        body=bd.getBody().getBytes();
                    }
                    else{
                        int dot=type.lastIndexOf(".");
                        type=type.substring(dot+1);
	                    try{
	                        body=BlaiseFile.draftToByteArray(bd);
	                    }catch(IOException e){
	                        BaseServlet.writeLog(
	                        "BlaiseFile error occurred in PackagePaper.doSubmitPaper (NewDraftQuery): "+
	                        e+" With parameter: paperID:"+paperID+" draftNumber:"+draftNumber+" title:"+title+" date:"+sqlDate);
	                        return sendMessage(new ResultWriter(false,"","Draft body could not be extracted."));//return the error message...
	                    }   
                    }
                    Vector aidvec=new Vector();
                    Long dsID=null;
                    aidvec.add(new Long(bp.getAssignmentID()));
                    GetDraftStructureIDQuery gdsidq=null;

                    gdsidq=new GetDraftStructureIDQuery(draftNumber,aidvec,null,null);

                    ResultSet dsrs=gdsidq.execute();
                    if(dsrs.next()){
                        dsID=new Long(dsrs.getLong(1));
                    }
                    
                    ndq=new NewDraftQuery(paperID,draftNumber,title,body,dsID,type);
                    ndq.execute();
                    //new draft added to db.  Now fill in its draftID, and add it to results.
                    draftID=ndq.getLastInsertID();
                    bd.setDraftID(draftID.toString());
                    result.append(draftID.toString()+TransmissionConstants.ID_SEPARATOR);
                }
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog(
            "Database error occurred in PackagePaper.doSubmitPaper (NewDraftQuery or GetNumberOfDraftsQuery): "+
            ex.toString()+" With parameter: paperID:"+paperID+
            " draftNumber:"+draftNumber+" title:"+title+" date:"+sqlDate+
            " PLEASE CHECK DATABASE FOR INTEGRITY.");
            return sendMessage(dbError);//return the error message...
        }catch(NumberFormatException e){
            BaseServlet.writeLog(
            "NumberFormatException "+e+" in PackagePaper.doSubmitPaper (NewDraftQuery) with"+
            " parameters: paperID:"+bd.getPaperID()+" draftNumber:"+
            bd.getDraftNumber()+" title:"+bd.getTitle()+" date:"+bd.getTurninDate()+" PLEASE CHECK DATABASE FOR INTEGRITY.");
            return sendMessage(new ResultWriter(false,"","Invalid fields found in submitted paper.  Please contact an administrator."));
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        
        //finally, send a receipt to the student.
        //emailQueue is used automatically instead of trying to send it manually first, so error stuff is unnecessary
        NewEmailQueueQuery neqq=null;
        try{
        	if(bp.getPaperID()!=null && !bp.getPaperID().equals("")){
	            neqq=new NewEmailQueueQuery(new Long(bp.getPaperID()),new Integer(1),writerID);
	        }else if(paperID!=null){
	            neqq=new NewEmailQueueQuery(paperID, new Integer(1), writerID);
	        }else{
                return sendMessage(new ResultWriter(false,"","Could not obtain paper ID."));	        
	        }
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        try{
            neqq.execute();
        }catch(SQLException se){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. Your paper has been submitted but an error occurred when sending you an email receipt.\n");
            dbError.append(se.getMessage());
            StringBuffer ex=new StringBuffer(se.getMessage());
            while((se=se.getNextException())!=null){
                ex.append(se.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doSubmitPaper (NewEmailQueueQuery): " 
            +ex.toString()+" With parameters: paperID:"+paperID+" type: 1");
            return sendMessage(dbError);//return the error message...
        }
      
        //send back results.
        result.setStatus(true);
        result.setFormat(format.toString());
        return sendMessage(result);
    }

    /**
     * Performs the staging of the papers by adding the comments and removing the checkout entries.
     * @param bp a Blaise_Paper to stage.
     * @param graderID The graderID of the grader who wrote these comments.
     * @return A byte array containing either errors or an okay, in a form okay to transfer to the user.
     */
    private byte[] doStagePaper(Blaise_Paper bp,Long graderID){
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("doStagePaper: parameters, graderID:"+graderID);
        }
        
        Long paperID;
        try{
            paperID=new Long(bp.getPaperID());
        }catch(NumberFormatException e){
            BaseServlet.writeLog(
            "NumberFormatException "+e+" in PackagePaper.doStagePaper (invalid paperID) with"+
            " parameters: paperID:"+bp.getPaperID());
            return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper.  Please contact an administrator."));
        }
       
        Long courseID;
        try{
            courseID=new Long(bp.getCourseID());
        }catch(NumberFormatException e){
            BaseServlet.writeLog(
            "NumberFormatException "+e+" in PackagePaper.doStagePaper (invalid courseID) with"+
            " parameters: courseID:"+bp.getCourseID());
            return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper.  Please contact an administrator."));
        }
        
        //check that it has been checked out--don't allow an unchecked out paper to be checked in,
        //as most likely it is a double click on the client or something like that.
        //if it is checked out, then delete the checkout.
        GetCheckoutIDQuery gcq=null;
        DeleteCheckoutQuery dcq=null;
        Vector g=new Vector();
        Vector p=new Vector();
        g.add(graderID);
        p.add(paperID);
        try{
            //------------------SQL HERE---------------------------
            gcq=new GetCheckoutIDQuery(g,p,null,limitBy);
            ResultSet rs=gcq.execute();
            Long checkoutID=null;
            if(rs.next()){
                checkoutID=new Long(rs.getLong(1));
            }
            
            
            if(checkoutID!=null){
                Vector v=new Vector();
                v.add(checkoutID);
                dcq=new DeleteCheckoutQuery(v);
                dcq.execute();
            }
            else{
                //this is bad, a paper was checked in that wasn't checked out.
                BaseServlet.writeLog(
                    "PackagePaper.doStagePaper: A paper was checked in that wasn't checked out."+
                    " With paperID:"+p+" and graderID:"+g);
                return sendMessage(new ResultWriter(false,"","Invalid: You are checking a paper that wasn't checked out."));
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog(
            "Database error occurred in PackagePaper.doStagePaper (checkout table related)"+
            ": "+ex.toString()+" With parameters:" +" graderID:"+g+" paperID:"+p+
            " PLEASE CHECK DATABASE FOR INTEGRITY.");
            return sendMessage(dbError);//return the error message...
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }finally{
            if(gcq!=null){
                gcq.free();
            }
        }

        
        Vector drafts=bp.getDrafts();
        Vector comments=null;
        Blaise_Comment bc=null;
        Blaise_Draft bd=null;
        int dSize=drafts.size();
        int cSize;
        int cIndex;
        String graderIDString=graderID.toString();
        
        Long draftID;
        Long commentID;
        Boolean isPublic;
        String ip;
        String cID;
        NewCommentQuery ncq;
        EditCommentQuery ecq;
        for(int dIndex=0;dIndex<dSize;dIndex++){
            //for each draft, get the comments.
            bd=(Blaise_Draft)drafts.get(dIndex);
            
            if(!bd.getIsModified().equals("YES")){
                //the grader's comment for this draft has not changed, so ignore it.
                continue;
            }
            
            comments=bd.getComments();
            cSize=comments.size();
            for(cIndex=0;cIndex<cSize;cIndex++){
                //for each comment in the draft.
                bc=(Blaise_Comment)comments.get(cIndex);
            
                if(!bc.getGraderID().equals(graderIDString)){
                    //it is by a different grader, so ignore it.
                    continue;
                }
                
                //now, if it is a new comment, insert it.  If it is an existing comment, updated it.
                
                //get the data from the Blaise_Comment
                try{
                    //paperID=new Long(bc.getPaperID());
                    draftID=new Long(bc.getDraftID());
                    graderID=new Long(bc.getGraderID());
                    cID=bc.getCommentID();
                    if(cID.equals("")){//new comment
                        commentID=null;
                    }
                    else{//existing comment
                        commentID=new Long(cID);
                    }
                    ip=bc.getIsPublic();
                    if(ip.equals("YES") || ip.equals("1")){
                        isPublic=new Boolean(true);
                    }else if(ip.equals("NO") || ip.equals("0")){
                        isPublic=new Boolean(false);
                    }else{
                        BaseServlet.writeLog("Invalid isPublic submitted in PackagePaper.doStagePaper."+
                        " isPublic:"+ip);
                        return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper (isPublic). Please contact an administrator."));
                    }
                }catch(NumberFormatException e){
                    BaseServlet.writeLog("NumberFormatException "+e+" in PackagePaper.doStagePaper with"+
                    " parameters:"+" paperID:"+bc.getPaperID()+" draftID:"+bc.getDraftID()+" commentID:"+
                    bc.getCommentID()+" graderID:"+bc.getGraderID());
                    return sendMessage(new ResultWriter(false,"","Invalid fields found in graded paper.  Please contact an administrator."));
                }

                if(commentID==null){
                    //So if we're here, then the comment is seemingly valid, and isn't already in the database.
                    //so add it.
                    if(BaseServlet.debugOn()){
                        BaseServlet.writeDebug("Adding new comment to database for draftID:"+draftID);
                    }
                    try{
                        ncq=new NewCommentQuery(bc,paperID,draftID,graderID,BlaiseFile.commentToByteArray(bc),isPublic);
                        ncq.execute();
                    }catch(SQLException e){
                        ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                        dbError.append(e.getMessage());
                        StringBuffer ex=new StringBuffer(e.getMessage());
                        while((e=e.getNextException())!=null){
                            ex.append(e.getMessage());
                        }
                        //from edu.gwu.servlets
                        BaseServlet.writeLog(
                        "Database error occurred in PackagePaper.doStagePaper (NewCommentQuery): "+
                        ex.toString()+" With parameters: draftID:"+draftID+" graderID"+graderID+
                        " PLEASE CHECK COMMENT TABLE FOR INTEGRITY.");
                        return sendMessage(dbError);//return the error message...
                    }catch(IOException e){
                        BaseServlet.writeLog(
                        "BlaiseFile error occurred in PackagePaper.getBlaise_CommentVector: "+
                        e+" With parameter: draftID:"+draftID+" graderID:"+graderID);
                    }catch(NoConnectionException e){
                        return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                    }  
                }
                else{
                    //update the existing comment.
                    if(BaseServlet.debugOn()){
                        BaseServlet.writeDebug("updating comment with commentID:"+commentID);
                    }
                    try{
                        ecq=new EditCommentQuery(commentID,null,null,null,BlaiseFile.commentToByteArray(bc),isPublic);
                        ecq.execute();
                    }catch(SQLException e){
                        ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
                        dbError.append(e.getMessage());
                        StringBuffer ex=new StringBuffer(e.getMessage());
                        while((e=e.getNextException())!=null){
                            ex.append(e.getMessage());
                        }
                        //from edu.gwu.servlets
                        BaseServlet.writeLog(
                        "Database error occurred in PackagePaper.doStagePaper (EditCommentQuery): "+
                        ex.toString()+" With parameters: commentID:"+commentID+" graderID"+graderID);
                        return sendMessage(dbError);//return the error message...
                    }catch(IOException e){
                        BaseServlet.writeLog(
                        "BlaiseFile error occurred in PackagePaper.doStagePaper: "+
                        e+" With parameters: commentID:"+commentID+" graderID:"+graderID);
                    }catch(NoConnectionException e){
                        return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
                    }  
                }
            }//end comment for loop
        }//end draft for loop
            
        //Now, go ahead and put the paper in the staging table, to be sent later
        //IF the minimum number of graders has been reached for this draft.

        try{
            NewStagingQuery nsq=null;
            nsq=new NewStagingQuery(courseID, graderID, paperID, TransmissionConstants.INCOMPLETE);        
            nsq.execute();
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
            " Your grade has been submitted, but an error occurred when sending graded paper to staging. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doStagePaper"+
            " (during send paper step): "+ex.toString()+
            " With parameters:" +" paperID:"+paperID+" graderID"+graderID);
            return sendMessage(dbError);//return the error message...
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        return sendMessage(new ResultWriter(true,"","Graded paper has been added to database."));
    }
    
    /**
     * Performs the sending of the staged papers by adding them to the emailQueue table.
     * @param courseIDs a Vector of course IDs for the papers to be sent
     * @param stagingIDs a Vector of staging IDs for the papers to be sent
     * @param graderID The graderID of the grader who is sending the papers.
     * @return A byte[] containing either errors or an okay, in a form okay to transfer to the user.
     */
    private byte[] doSendStagedPaper(Vector stagingIDs,Long graderID){
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("doSendStagedPaper: parameters, stagingIDs:"+stagingIDs+" graderID:"+graderID);
        }        
        
        //put graderID into a vector
        Vector gvec=new Vector();
        gvec.add(graderID);
        //get the IDs of the papers to send
        Vector paperIDs=new Vector();
        GetPaperIDForStagingQuery gpIDfsq=null;
        try{
            gpIDfsq=new GetPaperIDForStagingQuery(stagingIDs,gvec);
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        try{
            ResultSet rs=gpIDfsq.execute();
            int i=1;
            while(rs.next()){
                paperIDs.add(new Long(rs.getLong(i)));
            }
            if(paperIDs==null){
                return sendMessage(new ResultWriter(false,"","No paperIDs turned up in search for doSendStagedPaper"));
            }
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
            " An error occurred when sending staged paper to student. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doSendStagedPaper"+
            " (during send paper step): "+ex.toString()+
            " With parameters: stagingIDs:"+stagingIDs+" graderID"+graderID);
            return sendMessage(dbError);//return the error message...
        }            
        //Now, go ahead and notify the writer that their paper has been graded.
        //IF the minimum number of graders has been reached for this draft.
        //do emailQueue first, so don't need as much error notification
 
        int size=paperIDs.size();
        for(int i=0;i<size;i++){
            SetHasBeenDownloadedQuery shbdq=null;
            try{
                shbdq=new SetHasBeenDownloadedQuery((Long)paperIDs.elementAt(i),"N");
                int exe=shbdq.execute();
            }catch(NoConnectionException e){
                return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
            }catch(SQLException e){
                ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
                " An error occurred when setting download status of paper number "+i+" out of "+size+" for student. ");
                dbError.append(e.getMessage());
                StringBuffer ex=new StringBuffer(e.getMessage());
                while((e=e.getNextException())!=null){
                    ex.append(e.getMessage());
                }
                //from edu.gwu.servlets
                BaseServlet.writeLog("Database error occurred in PackagePaper.doSendStagedPaper"+
                " (during set has been downloaded step): "+ex.toString()+
                " With parameters:" +" paperIDs:"+paperIDs+" stagingIDs:"+stagingIDs+" graderID"+graderID+
                "paper number "+i+" out of "+size);
                return sendMessage(dbError);//return the error message...
            }
            
            NewEmailQueueQuery neqq=null;
            try{
                neqq=new NewEmailQueueQuery((Long)paperIDs.elementAt(i),new Integer(2),graderID); 
            }catch(NoConnectionException e){
                return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
            }
            try{
                neqq.execute();
            }catch(SQLException e){
                ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
                " An error occurred when sending staged paper number "+i+" out of "+size+" to student. ");
                dbError.append(e.getMessage());
                StringBuffer ex=new StringBuffer(e.getMessage());
                while((e=e.getNextException())!=null){
                    ex.append(e.getMessage());
                }
                //from edu.gwu.servlets
                BaseServlet.writeLog("Database error occurred in PackagePaper.doSendStagedPaper"+
                " (during send paper step): "+ex.toString()+
                " With parameters:" +" paperIDs:"+paperIDs+" stagingIDs:"+stagingIDs+" graderID"+graderID+
                "paper number "+i+" out of "+size);
                return sendMessage(dbError);//return the error message...
            }
        }
        DeleteQuery dq=null;
        try{
            dq=new DeleteQuery(stagingIDs,PackagingConstants.TABLE_staging,PackagingConstants.LIMIT_grader);
        }catch(NoConnectionException e){
            return sendMessage(new ResultWriter(false,"","Could not connect to the database."));
        }
        try{
            dq.execute();
        }catch(SQLException e){
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred."+
            " An error occurred when deleting papers from staging table. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackagePaper.doSendStagedPaper"+
            " (during send paper step): "+ex.toString()+
            " With parameters:" +" paperIDs:"+paperIDs+" stagingIDs:"+stagingIDs+" graderID"+graderID);
            return sendMessage(dbError);//return the error message...
        }finally{
            if(dq!=null){
                dq.free();
            }
        }
        return sendMessage(new ResultWriter(true,"","Graded paper has been added to database."));
    }


    /**
     * checks to see if any of the supplied papers are checked-out by the grader.
     * @param pIDs Vector containing the paperID of each paper to check for checkout.
     * @param graderID Long of the graderID who would have checked out the papers.
     * @return boolean <code>true</code> if any of the papers were checked-out.  <code>false</code> otherwise.
     */
    private boolean isCheckedOut(Vector pIDs, Long graderID) throws SQLException{
        Vector g=new Vector();
        g.add(graderID);
        GetCheckoutIDQuery q=null;
        try{
            q=new GetCheckoutIDQuery(g,pIDs,null,limitBy);
        }catch(NoConnectionException e){
            return false;
        }
        boolean co=false;
        try{
            //------------------SQL HERE---------------------------
            ResultSet rs=q.execute();
            if(rs.next()){//if there is one or more rows, than there were checkouts.
                co=true;
            }
            else{
                co=false;
            }
        }finally{
            q.free();
        }
        return co;
    }
    
    
    /**
     * pulls any and all ids from the string, and returns them in Long form packaged in a Vector.
     * @param id The String containing all the IDs. (may be null, or empty).
     * @return the IDs packed in a Vector in Long form. (<code>null</code> if param id is null or empty).
     */
    private Vector getLongVector(String id) throws NumberFormatException{
        Vector v=null;
        if(id!=null && !id.equals("")){
            StringTokenizer st=new StringTokenizer(id,TransmissionConstants.ID_SEPARATOR);
            v=new Vector();
            while(st.hasMoreTokens()){
                v.add(new Long(st.nextToken()));
            }
        }
        return v;
    }
    
    /**
     * Gets all of the paperIDs that are available from the given limit combination
     * and puts them in a Vector of Long objects.
     */ 
    private Vector getPaperIDVector(String limitBy,Long limitID)throws SQLException{
        Vector pIDs=new Vector();        

        GetPaperIDQuery IDq=null;
        try{
            //------------------SQL HERE---------------------------
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                Vector v=new Vector();
                v.add(limitID);
                try{
                    IDq=new GetPaperIDQuery(null,v,null,null,null,null,null,null,null,null,PackagingConstants.LIMIT_writer,null,null,null);
                }catch(NoConnectionException e){
                    return null;
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                Vector v=new Vector();
                v.add(limitID);
                try{
                    IDq=new GetPaperIDQuery(null,null,null,null,v,null,null,null,null,null,PackagingConstants.LIMIT_writer,null,null,null);
                }catch(NoConnectionException e){
                    return null;
                }
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                try{
                    IDq=new GetPaperIDQuery(null,null,null,null,null,null,null,null,null,null,PackagingConstants.LIMIT_admin,null,null,null);
                }catch(NoConnectionException e){
                    return null;
                }
            }
            //do the look up
            ResultSet rs=IDq.execute();
            while(rs.next()){
                pIDs.add(new Long(rs.getLong(1)));
            }
        }finally{
            if(IDq!=null){
                IDq.free();//always return the connection.
            }
        }
        return pIDs;
    }
    
    /**
     * Gets the draftIDs that correspond to the paperIDs supplied.
     * @return Vector containing the draftIDs in Long form.
     */
    private Vector getDraftIDVector(Vector pIDs)throws SQLException{
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("getDraftIDVector: parameters, pIDs:"+pIDs);
        }
        Vector dIDs=new Vector();
        //------------------SQL HERE---------------------------
        GetDraftIDQuery IDq=null;
        try{
            IDq=new GetDraftIDQuery(pIDs,null,null,null,null,null,null);
        }catch(NoConnectionException e){
            return null;
        }
        try{
            ResultSet rs=IDq.execute();
            while(rs.next()){
                dIDs.add(new Long(rs.getLong(1)));
            }
        }finally{
            if(IDq!=null){
                IDq.free();//always return the connection.
            }
        }
        return dIDs;
    }
    /**
     * Gets the commentIDs that correspond to the paperIDs supplied.
     * @return Vector containing the commentIDs in Long form.
     */
    private Vector getCommentIDVector(Vector pIDs)throws SQLException{
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("getCommentIDVector: parameters, pIDs:"+pIDs);
        }
        Vector cIDs=new Vector();
        //------------------SQL HERE---------------------------
        GetCommentIDQuery IDq=null;
        try{
            IDq=new GetCommentIDQuery(pIDs,null,null,null,null,null);
        }catch(NoConnectionException e){
            return null;
        }
        try{
            ResultSet rs=IDq.execute();
            while(rs.next()){
                cIDs.add(new Long(rs.getLong(1)));
            }
        }finally{
            if(IDq!=null){
                IDq.free();//always return the connection.
            }
        }
        return cIDs;
    }    
    
    /**
     * Returns a Vector of Blase_Drafts with all relevant parts (no Comments) from a Vector of draftIDs.
     */
    private Vector getBlaise_DraftVector(Vector dIDs, String limitBy, Long limitID) throws SQLException{
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("getBlaise_DraftVector: parameters, dIDs:"+dIDs+" limitBy:"+limitBy+
                " limitID:"+limitID);
        }
        //execute query to get stuff from draft table
        GetQuery gq=null;
        Vector drafts=new Vector();
        String draftID;
        String paperID;
        String draftNumber;
        String title;
        String body;
        String date;
        Blaise_Draft bd=null;
        Long dsID;
        GetQuery dsq=null;
        GetQuery aq=null;
        Vector dsv=null;
        Long aID;
        Vector av=null;
        String assignID;
        String assignName;
        byte[] barray;
        try{
            //------------------SQL HERE---------------------------
            try{
                gq=new GetQuery(dIDs,PackagingConstants.TABLE_draft,limitBy,limitID);
            }catch(NoConnectionException e){
                return null;
            }
            ResultSet rs=gq.execute();
            while(rs.next()){
                barray=rs.getBytes("body");
                //if the draft is type Blaise, get the info. otherwise use the byte array as the body
                if(rs.getString("draftType").equals("Blaise")){
	                bd=new Blaise_Draft();
                    body=new String(barray);
                    bd.setBody(body);
	                bd.setDraftID(rs.getString("draftID"));
	                bd.setPaperID(rs.getString("paperID"));
	                bd.setDraftNumber(rs.getString("draftNumber"));
	                bd.setTitle(rs.getString("title"));
	                bd.setTurninDate(rs.getString("date"));
                }else{
                	try{
	                    bd=BlaiseFile.byteArrayToDraft(barray);
		                bd.setDraftID(rs.getString("draftID"));
		                bd.setPaperID(rs.getString("paperID"));
		                bd.setDraftNumber(rs.getString("draftNumber"));
		                bd.setTitle(rs.getString("title"));
		                bd.setTurninDate(rs.getString("date"));
	                }catch(IOException e){
	                    BaseServlet.writeLog("Database error occurred in PackagePaper.getBlaise_DraftVector"+
                        e.toString()+" With parameters:" +" limitBy:"+limitBy+" limitID:"+limitID+" dIDs"+dIDs);
	                }
                }
                drafts.add(bd);
            }
        }finally{
            gq.free();//no matter what, the connection should get freed.
        }
        return drafts;
    }
    
    /**
     * returns the number of drafts
     */ 
    private int getNumberOfDrafts(Long paperID) throws SQLException{
        GetNumberOfDraftsQuery gq=null;
        try{
            gq=new GetNumberOfDraftsQuery(paperID);
        }catch(NoConnectionException e){
            return -1;
        }
        return gq.execute();
    }
    
    /**
     * Returns a Vector of Blase_Comment with all relevant parts from a Vector of commentIDs.
     */
    private Vector getBlaise_CommentVector(Vector cIDs, String limitBy, Long limitID) throws SQLException{
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("getBlaise_CommentVector: parameters, cIDs:"+cIDs+" limitBy:"+limitBy+
                " limitID:"+limitID);
        }
        //execute query to get stuff from comments table
        Vector comments=new Vector();
        if(cIDs==null || cIDs.isEmpty()){
            return comments;
        }
        GetQuery gq=null;
        byte[] comment;
        Blaise_Comment bc;
        try{
            //------------------SQL HERE---------------------------
            try{
                gq=new GetQuery(cIDs,PackagingConstants.TABLE_comment,limitBy,limitID);
            }catch(NoConnectionException e){
                return null;
            }
            ResultSet rs=gq.execute();
            while(rs.next()){
                comment=rs.getBytes("remarks");
                try{
                    bc=BlaiseFile.byteArrayToComment(comment);
                    bc.setCommentID(rs.getString("commentID"));
                    bc.setPaperID(rs.getString("paperID"));
                    bc.setDraftID(rs.getString("draftID"));
                    bc.setIsPublic(rs.getString("isPublic"));
                    bc.setGraderID(rs.getString("graderID"));
                    comments.add(bc);
                }catch(IOException e){
                    BaseServlet.writeLog(
                    "BlaiseFile error occurred in PackagePaper.getBlaise_CommentVector: "+
                    e+" With parameter: cIDs:"+cIDs+" limitBy:"+limitBy+" limitID:"+limitID);
                }   
            }
        }finally{
            gq.free();
        }
        return comments;
    }
    
    /**
     * Sends the paper specified by paperID to the writer.
     */
    private void doSendPaper(Long paperID,Long graderID)throws MessagingException,IOException{
        Vector v=new Vector();
        v.add(paperID);
        byte[] pRW=doAssemblePaper(v,PackagingConstants.LIMIT_grader,graderID,false);
        byte[] pRW2=DualByteArray.extractSecond(pRW);
        Blaise_Paper bp=BlaiseFile.byteArrayToPaper(pRW2);
        MailUtility mu=new MailUtility(tempDir);
        mu.sendGraded(bp);
    }      
    
    /**
     * Formats the response for transmission of error messages.
     */
    public static byte[] sendMessage(ResultWriter rw){
        Properties prop = new Properties();
        if(rw.getStatus()){
            prop.setProperty("status", TransmissionConstants.SUCCESS);
        }
        else{
            prop.setProperty("status", TransmissionConstants.FAILURE);
        }
        prop.setProperty("message", rw.toString());
        byte[] firstPart=DualByteArray.toByteArray(prop);
        byte[] secondPart=new byte[0];
        byte[] bothParts=DualByteArray.makeDualArray(firstPart, secondPart);
        return bothParts;
    }     

    /**
     * Formats the response for transmission of error messages.
     */
    public static byte[] sendMessage(boolean success, String message){
        Properties prop = new Properties();
        if(success){
            prop.setProperty("status", TransmissionConstants.SUCCESS);
        }
        else{
            prop.setProperty("status", TransmissionConstants.FAILURE);
        }
        prop.setProperty("message", message);
        byte[] firstPart=DualByteArray.toByteArray(prop);
        byte[] secondPart=new byte[0];
        byte[] bothParts=DualByteArray.makeDualArray(firstPart, secondPart);
        return bothParts;
    }     
}
