package edu.gwu.blaise.version2.backend.packagers;
import javax.servlet.*;
import javax.servlet.http.*;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.NoConnectionException;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.queries.ValidateLoginQuery;
import java.sql.*;

/**
 * Performs and then packages a ValidateXXXLogin Query.
 * @author Michael Gaiman
 * @version 2 Jan 19, 2004
 */
/*
 * Please note:  All Actual SQL-type stuff is marked by //------SQL Here------ tags.
 */
public class PackageValidateLogin extends Packager{
    public PackageValidateLogin(String commandType, HttpServletRequest request, String limitBy){
        super(commandType,request,limitBy);
    }

    public byte[] execute(boolean b){/*this method is useless in this context, but it has to be in here*/
        //this method should never be called, but it has to be here.  try to send an error message with the proper
        //encoding, otherwise send one with whatever encoding we can get
        try{
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes("UTF-8");
        }catch(java.io.UnsupportedEncodingException e){
            return new ResultWriter(false,"","This method isn't supposed to be called here.").toString().getBytes();
        }
    }

    /**
     * Performs the actual query and packaging.
     * @return The result from the query in a form that is appropriate for transmission.
     */
     public String execute(){
        if(request==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to administrator."+
                                             " (No request object in PackageValidateLogin.)").toString();
        }
        if(commandType==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (No commandType object in PackageValidateLogin.)").toString();
        }
        //check to see that it is one of the three valid limitBys
        if(limitBy==null){
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (No limitBy object in PackageValidateLogin.)").toString();
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        }
        else{
            return new ResultWriter(false,"","An Internal Server Error Occurred, please report to Administrator."+
                                            " (invalid limitBy object in PackageValidateLogin.)").toString();
        }
        
        BaseServlet.writeLog("PackageValidateLogin: "+limitBy+" is doing: "+commandType);
       
        if(commandType.equals(TransmissionConstants.validateWriterLogin)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                return new 
                ResultWriter(false,"","Access Denied.  Graders may not perform desired action.").toString();
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
            }
        }
        else if(commandType.equals(TransmissionConstants.validateGraderLogin)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                return new 
                ResultWriter(false,"","Access Denied.  Writers may not perform desired action.").toString();
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
            }
        }
        else if(commandType.equals(TransmissionConstants.validateAdminLogin)){
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                return new 
                ResultWriter(false,"","Access Denied.  Writers may not perform desired action.").toString();
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                return new 
                ResultWriter(false,"","Access Denied.  Graders may not perform desired action.").toString();
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
            }
        }
        else{
            return new ResultWriter(false,"","Invalid command: "+commandType).toString();
        }
        //username/password/universityID
        String u=null;
        String p=null;
        Integer uID=null;
        
        u=request.getParameter("userName");
        if(u==null){
            return new ResultWriter(false,"","Missing required userName parameter.").toString();
        }
        p=request.getParameter("password");
        if(p==null){
            return new ResultWriter(false,"","Missing required password parameter.").toString();
        }
        try{
            p=Encryption.decryptPassword(p);
        }catch(java.util.NoSuchElementException e){
            return new ResultWriter(false,"","Password submitted in invalid format").toString();
        }
        if(p==null){
            return new ResultWriter(false,"","Password submitted in invalid format").toString();
        }
        String uni=request.getParameter("universityID");
        if(uni==null){
            return new ResultWriter(false,"","Missing required universityID parameter.").toString();
        }
        //parse the universityID
        if(!limitBy.equals(PackagingConstants.LIMIT_admin)){//admins are system-wide
            try{
                uID=new Integer(uni);
            }catch(NumberFormatException e){
                return new 
                ResultWriter(false,"","Invalid universityID format.  Please notify an administrator").toString();
            }
        }
        
        ValidateLoginQuery vq=new ValidateLoginQuery(u,p,uID,limitBy);
        try{
            //-------------SQL HERE---------------------
            ResultSet rs=vq.execute();
            if(!rs.next()){
                return new ResultWriter(false,"","Invalid username/password combination").toString();
            }
            ResultWriter rw=new ResultWriter();
            
            if(limitBy.equals(PackagingConstants.LIMIT_writer)){
                rw.setFormat("writerID");
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
                rw.setFormat("graderID");
            }
            else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
                rw.setFormat("adminID");
            }
            
            rw.append(rs.getString(1));
            rw.setStatus(true);
            return rw.toString();
        }catch(SQLException e){
            //an error occurred.  Tell user some info, and log it.
            ResultWriter dbError= new ResultWriter(false,"","A database error has occurred. ");
            dbError.append(e.getMessage());
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageValidateLogin:"+ex.toString()+
                " With parameters:" +" userName:"+u +" password:"+p +" universityID:"+uID+
                " limitBy:"+limitBy);
            return dbError.toString();//return the error message...
        }finally{
            if(vq!=null){
                vq.free();
            }
        }
     }
     
     /**
      * Determines if a username/password/universityID combination is valid.
      * @param limitBy the type of user: writer, grader, or admin.
      * @param u the userName
      * @param eP The encrypted password.
      * @param uID The universityID (<code>null</code> if admin).
      * @param limitID (String) optional.  If a ID was supplied, then check to make sure they equal.
      * @return <code>true</code> if the combination is valid.  <code>false</code> otherwise.
      */
     public static boolean isValid(String u,String eP,Integer uID,String limitBy,String limitID){
        if(eP==null || eP.equals("") || 
           u==null || u.equals("") || 
           limitBy==null || limitBy.equals("")){
            return false;
        }
        
        if(!limitBy.equals(PackagingConstants.LIMIT_admin) && uID==null){//admins are system wide
            return false;
        }
        String p;
        try{
            p=Encryption.decryptPassword(eP);
        }catch(java.util.NoSuchElementException e){
            return false;
        }
        if(p==null){
            return false;
        }
        ValidateLoginQuery vq=new ValidateLoginQuery(u,p,uID,limitBy);
        try{
            //-------------SQL HERE---------------------
            ResultSet rs=vq.execute();
            if(rs.next()){
                if(limitID==null){
                    return true;
                }
                else{//check if they equal
                    if(limitID.equals(rs.getString(1))){
                        return true;
                    }
                    else{
                        BaseServlet.writeLog("PackageValidateLogin.isValid:  A user logged in successfully, but supplied a different userID number.  Parameters: userName:"+u+" universityID:"+uID+" userID number:"+limitID);
                        return false;
                    }
                }
            }else{
                return false;
            }
        }catch(SQLException e){
            StringBuffer ex=new StringBuffer(e.getMessage());
            while((e=e.getNextException())!=null){
                ex.append(e.getMessage());
            }
            //from edu.gwu.servlets
            BaseServlet.writeLog("Database error occurred in PackageValidateLogin:"+ex.toString()+
                " With parameters:" +" userName:"+u +" password:"+p +" universityID:"+uID+
                " limitBy:"+limitBy);
            
            return false;
        }finally{
            if(vq!=null){
                vq.free();
            }
        }
     }
}
