package edu.gwu.blaise.version2.backend.packagers;
import javax.servlet.*;
import javax.servlet.http.*;
import edu.gwu.blaise.version2.backend.util.NoConnectionException;

/**
 * The interface that all PackageXXX classes implement.
 * @author Michael Gaiman
 * @version 2 Jan 19, 2004
 */
public abstract class Packager {
    /**
     * Used to allow easy access to the request parameters
     */
    HttpServletRequest request=null;
    /**
     * Limits a results to a similar field, such as PackagingConstants.GRADER_graderID limits
     * the results from a getCourseID call to only supplying one graderID, and thus only
     * courses that grader is in.
     */
    String limitBy=null;
    /**
     * The actual specific request that is wanted.  For example, PackagingConstants.COMMAND_getWriterID
     */
    String commandType=null;
    
    //constructors
    /**
     * Creates an instance of a getID type command.  
     * @param commandType is which type of ID you would like to get (specified in 
     * {@link edu.gwu.Blaise.version2.transmission.TransmissionConstants} as fields that have getXXXID).
     * @param request The actual request that was received.  This is required to provide an easy way 
     * of getting access to the various parameters required to do this command.
     * @param limitBy Should the command be limited by anything?  Often it is specified by 
     * {@link PackagingConstants} such as PackagingConstants.GRADER_graderID limits
     * the results from a getCourseID call to only supplying one graderID, and thus only
     * courses that grader is in.
     */
    public Packager(String commandType, HttpServletRequest request, String limitBy){
        this.commandType=commandType;
        this.request=request;
        this.limitBy=limitBy;
    }
    
    public abstract String execute();
    //the boolean is useless, I just needed a different return type
    public abstract byte[] execute(boolean b);
}
