package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a deleteCheckout query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2, 1/20/2004
 */
public class DeleteCheckoutQuery implements SetQuery{
    
    Vector checkoutID=null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public DeleteCheckoutQuery(Vector checkoutID){
        //in the non-String, non-Vector objects, null values are not checked for.
        if(checkoutID!=null && !checkoutID.isEmpty()){
            this.checkoutID=checkoutID;
        }else{
            throw new IllegalArgumentException("DeleteCheckoutQuery: invalid checkoutID Vector");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*DELETE FROM checkout WHERE (checkoutID=? OR checkoutID=?);
        */
            StringBuffer sqlCommand=new StringBuffer("DELETE FROM checkout WHERE");
            
            if(checkoutID!=null){
                //IDs can have more than one.  So return results that match any checkoutID.
                int size=checkoutID.size();
                sqlCommand.append(" (checkoutID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" checkoutID=?");
                }
                sqlCommand.append(")");
            }
            sqlCommand.append(";");
            
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "DeleteCheckoutQuery: for parameters:"+"("+
                    " checkoutID:"+checkoutID+
                    ") the following statement was assembled: "+sqlCommand
                );
            }
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(checkoutID!=null){
                //get each graderID as a long
                Enumeration e=checkoutID.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
