package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.packagers.PackagingConstants;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs delete queries.  
 * Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/19/2004
 */
public class DeleteQuery{
    private Vector IDs=		null;
    private String table=	null;
    private String limitBy=	null;
    
    /** Our pooled connection. */
    private Connection c=StaticContext.getSQLConnection();
    
    public DeleteQuery(Vector IDs, String table, String limitBy){
    
        if(IDs!=null && !IDs.isEmpty()){
            this.IDs=IDs;
        }
        if(table!=null && !table.equals("")){
            this.table=table;
        }
        if(limitBy!=null && !limitBy.equals("")){
            this.limitBy=limitBy;
        }
                
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        if(table==null){//table is required.
            return 0;
        }
        //check limitBy type
        if(limitBy==null){//limitBy is required.
            return 0;
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader) && !table.equals(PackagingConstants.TABLE_assignment) && !table.equals(PackagingConstants.TABLE_ds) && !table.equals(PackagingConstants.TABLE_wag) && !table.equals(PackagingConstants.TABLE_staging)){
            //grader is allowed to delete assignments, draft structures, and writer as grader entries
            return 0;
        }
        else if(!limitBy.equals(PackagingConstants.LIMIT_admin) && !limitBy.equals(PackagingConstants.LIMIT_grader)){
            return 0;
        }
        
        StringBuffer sqlCommand=new StringBuffer("DELETE FROM");
        
        if(table.equals(PackagingConstants.TABLE_course)){
            sqlCommand.append(" course ");
        }
        else if(table.equals(PackagingConstants.TABLE_writer)){
            sqlCommand.append(" writer ");
        }
        else if(table.equals(PackagingConstants.TABLE_grader)){
            sqlCommand.append(" grader ");
        }
        else if(table.equals(PackagingConstants.TABLE_admin)){
            sqlCommand.append(" admin ");
        }
        else if(table.equals(PackagingConstants.TABLE_university)){
            sqlCommand.append(" university ");
        }
        else if(table.equals(PackagingConstants.TABLE_ctg)){
            sqlCommand.append(" course_to_grader ");
        }
        else if(table.equals(PackagingConstants.TABLE_ctw)){
            sqlCommand.append(" course_to_writer ");
        }
        else if(table.equals(PackagingConstants.TABLE_checkout)){
            sqlCommand.append(" checkout ");
        }
        else if(table.equals(PackagingConstants.TABLE_paper)){
            sqlCommand.append(" paper ");
        }
        else if(table.equals(PackagingConstants.TABLE_draft)){
            sqlCommand.append(" draft ");
        }
        else if(table.equals(PackagingConstants.TABLE_comment)){
            sqlCommand.append(" comment ");
        }
        else if(table.equals(PackagingConstants.TABLE_eq)){
            sqlCommand.append(" emailQueue ");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment)){
            sqlCommand.append(" assignment ");
        }
        else if(table.equals(PackagingConstants.TABLE_ds)){
            sqlCommand.append(" draft_structure ");
        }
        else if(table.equals(PackagingConstants.TABLE_wag)){
            sqlCommand.append(" writer_as_grader ");
        }
        else if(table.equals(PackagingConstants.TABLE_staging)){
            sqlCommand.append(" staging ");
        }

        
        if(IDs!=null){
            sqlCommand.append(" WHERE");
        }
        
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        if(table.equals(PackagingConstants.TABLE_course)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any courseID.
                int size=IDs.size();
                sqlCommand.append(" (courseID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" courseID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_writer)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any writerID.
                int size=IDs.size();
                sqlCommand.append(" (writerID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" writerID=?");
                }
                sqlCommand.append(")");
            }
        }


        else if(table.equals(PackagingConstants.TABLE_grader)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any graderID.
                int size=IDs.size();
                sqlCommand.append(" (graderID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" graderID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any adminID.
                int size=IDs.size();
                sqlCommand.append(" (adminID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" adminID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_university)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any universityID.
                int size=IDs.size();
                sqlCommand.append(" (universityID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" universityID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctg)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any courseToGraderID.
                int size=IDs.size();
                sqlCommand.append(" (courseToGraderID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" courseToGraderID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctw)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any courseToWriterID.
                int size=IDs.size();
                sqlCommand.append(" (courseToWriterID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" courseToWriterID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_checkout)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any checkoutID.
                int size=IDs.size();
                sqlCommand.append(" (checkoutID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" checkoutID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_paper)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any paperID.
                int size=IDs.size();
                sqlCommand.append(" (paperID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" paperID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_draft)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any draftID.
                int size=IDs.size();
                sqlCommand.append(" (draftID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" draftID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_comment)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any commentID.
                int size=IDs.size();
                sqlCommand.append(" (commentID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" commentID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_eq)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (queueID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" queueID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_assignment)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any assignID.
                int size=IDs.size();
                sqlCommand.append(" (assignID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" assignID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ds)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any draftStructureID.
                int size=IDs.size();
                sqlCommand.append(" (draftStructureID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" draftStructureID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_wag)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any writerAsGraderID.
                int size=IDs.size();
                sqlCommand.append(" (rowID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" rowID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_staging)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any stagingID.
                int size=IDs.size();
                sqlCommand.append(" (stagingID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" stagingID=?");
                }
                sqlCommand.append(")");
            }
        }        
        sqlCommand.append(";");
        
        //COMMAND ASSEMBLED
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "DeleteQuery: for parameters: ( IDs:"+IDs+" table:"+table+" limitBy:"+limitBy+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(table.equals(PackagingConstants.TABLE_course)){
            if(IDs!=null){
                //get each courseID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_writer)){
            if(IDs!=null){
                //get each writerID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_grader)){
            if(IDs!=null){
                //get each graderID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_admin)){
            if(IDs!=null){
                //get each adminID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_university)){
            if(IDs!=null){
                //get each universityID as a integer
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setInt(currentIndex,((Integer)e.nextElement()).intValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctg)){
            if(IDs!=null){
                //get each courseToGraderID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctw)){
            if(IDs!=null){
                //get each courseToWriterID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_checkout)){
            if(IDs!=null){
                //get each checkoutID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_paper)){
            if(IDs!=null){
                //get each paperID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_draft)){
            if(IDs!=null){
                //get each draftID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_comment)){
            if(IDs!=null){
                //get each commentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_eq)){
            if(IDs!=null){
                //get each queueID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_assignment)){
            if(IDs!=null){
                //get each assignID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ds)){
            if(IDs!=null){
                //get each draftStructureID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_wag)){
            if(IDs!=null){
                //get each rowID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_staging)){
            if(IDs!=null){
                //get each stagingID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeUpdate();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
