package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editComment query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class EditCommentQuery implements SetQuery{
    
    Long commentID=null;
    Long paperID=null;
    Long draftID=null;
    Long graderID=null;
    byte[] remarks=null;
    Boolean isPublic=null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public EditCommentQuery(Long commentID,
                            Long paperID,
                            Long draftID,
                            Long graderID,
                            byte[] remarks,
                            Boolean isPublic){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.commentID=commentID;
        this.paperID=paperID;
        this.draftID=draftID;
        this.remarks=remarks;
        this.isPublic=isPublic;
        
        if(commentID==null){
            throw new IllegalArgumentException("EditCommentQuery: commentID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE comment SET paperID=?, draftID=?, graderID=?, remarks=?, isPublic=?, date=now() WHERE commentID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE comment SET date=now()");
            if(paperID!=null){
                sqlCommand.append(", paperID=?");
            }
            if(draftID!=null){
                sqlCommand.append(", draftID=?");
            }
            if(graderID!=null){
                sqlCommand.append(", graderID=?");
            }
            if(remarks!=null){
                sqlCommand.append(", remarks=?");
            }
            if(isPublic!=null){
                sqlCommand.append(", isPublic=?");
            }
            sqlCommand.append(" WHERE commentID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(paperID!=null){
                ps.setLong(currentIndex,paperID.longValue());
                currentIndex++;
            }
            if(draftID!=null){
                ps.setLong(currentIndex,draftID.longValue());
                currentIndex++;
            }
            if(graderID!=null){    
                ps.setLong(currentIndex,graderID.longValue());
                currentIndex++;
            }
            if(remarks!=null){
                ps.setBytes(currentIndex,remarks);
                currentIndex++;
            }
            if(isPublic!=null){
                ps.setInt(currentIndex,(isPublic.booleanValue())?1:0);//1 if true, 0 if false
                currentIndex++;
            }
            
            ps.setLong(currentIndex,commentID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
