package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editGrader query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/19/2004
 */
public class EditCourseQuery implements SetQuery{
    
    Long courseID=		null;
    String contactInfo=		null;
    Integer universityID=	null;
    String dept=		null;
    String number=		null;
    String section=		null;
    String semester=		null;
    Integer year=		null;
    String isCurrent=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditCourseQuery(Long courseID,
                         String contactInfo,
                         Integer universityID,
                         String dept,
                         String number,
                         String section,
                         String semester,
                         Integer year,
                         String isCurrent){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.courseID=courseID;
        this.universityID=universityID;
        this.contactInfo=contactInfo;
        this.dept=dept;
        this.number=number;
        this.section=section;
        this.semester=semester;
        this.year=year;
        this.isCurrent=isCurrent;
        
        if(courseID==null){
            throw new IllegalArgumentException("EditCourseQuery: courseID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE course SET contactInfo=?, universityID=?, dept=?, number=?, section=?, year=?, numberGraders=?, isCurrent=? WHERE courseID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE course SET");
            if(contactInfo!=null){
                sqlCommand.append(" contactInfo=?,");
            }
            if(universityID!=null){
                sqlCommand.append(" universityID=?,");
            }
            if(dept!=null){
                sqlCommand.append(" dept=?,");
            }
            if(number!=null){
                sqlCommand.append(" number=?,");
            }
            if(section!=null){
                sqlCommand.append(" section=?,");
            }
            if(semester!=null){
                sqlCommand.append(" semester=?,");
            }
            if(year!=null){
                sqlCommand.append(" year=?,");
            }
            if(isCurrent!=null){
                sqlCommand.append(" isCurrent=?,");
            }
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE courseID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(contactInfo!=null){
                ps.setString(currentIndex,contactInfo);
                currentIndex++;
            }
            if(universityID!=null){
                ps.setInt(currentIndex,universityID.intValue());
                currentIndex++;
            }
            if(dept!=null){
                ps.setString(currentIndex,dept);
                currentIndex++;
            }
            if(number!=null){
                ps.setString(currentIndex,number);
                currentIndex++;
            }
            if(section!=null){
                ps.setString(currentIndex,section);
                currentIndex++;
            }
            if(semester!=null){
                ps.setString(currentIndex,semester);
                currentIndex++;
            }
            if(year!=null){
                ps.setInt(currentIndex,year.intValue());
                currentIndex++;
            }
            if(isCurrent!=null){
                ps.setString(currentIndex,isCurrent);
                currentIndex++;
            }
            
            ps.setLong(currentIndex, courseID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
