package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editGrader query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/19/2004
 */
public class EditDraftQuery implements SetQuery{
    
    Long draftID=		null;
    Long paperID=		null;
    Integer draftNumber=	null;
    String title=		null;
    Timestamp date=		null;
    Long draftStructureID=	null;
    String type=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditDraftQuery(Long draftID,
                        Long paperID,
                        Integer draftNumber,
                        String title,
                        Timestamp date,
                        Long draftStructureID,
                        String type){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.draftID=draftID;
        this.paperID=paperID;
        this.draftNumber=draftNumber;
        this.date=date;
        this.title=title;
        this.type=type;
        this.draftStructureID=draftStructureID;
                
        if(draftID==null){
            throw new IllegalArgumentException("EditDraftQuery: draftID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE draft SET paperID=?, draftNumber=?, date=?, title=?, type=?, draftStructureID=? WHERE draftID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE draft SET");
            if(paperID!=null){
                sqlCommand.append(" paperID=?,,");
            }
            if(draftNumber!=null){
                sqlCommand.append(" draftNumber=?,");
            }
            if(date!=null){
                sqlCommand.append(" date=?,");
            }
            if(title!=null){
                sqlCommand.append(" title=?,");
            }
            if(type!=null){
                sqlCommand.append(" type=?,");
            }
            if(draftStructureID!=null){
                sqlCommand.append(" draftStructureID=?,");
            }
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE draftID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(paperID!=null){
                ps.setLong(currentIndex,paperID.longValue());
                currentIndex++;
            }
            if(draftNumber!=null){
                ps.setInt(currentIndex,draftNumber.intValue());
                currentIndex++;
            }
            if(date!=null){
                ps.setTimestamp(currentIndex,date);
                currentIndex++;
            }
            if(title!=null){
                ps.setString(currentIndex,title);
                currentIndex++;
            }
            if(type!=null){
                ps.setString(currentIndex,type);
                currentIndex++;
            }
            if(draftStructureID!=null){
                ps.setLong(currentIndex,draftStructureID.longValue());
                currentIndex++;
            }
            
            ps.setLong(currentIndex, draftID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
