package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editDraftStructure query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/24/2004
 */
public class EditDraftStructureQuery implements SetQuery{
    
    Long draftStructureID=	null;
    Integer draftNum=		null;
    Long assignID=		null;
    String gradingType=		null;
    Timestamp noChangeDate=	null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditDraftStructureQuery(Long draftStructureID,
                                 Integer draftNum,
                                 Long assignID,
                                 String gradingType,
                                 Timestamp noChangeDate){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.draftStructureID=draftStructureID;
        this.draftNum=draftNum;
        this.assignID=assignID;
        this.gradingType=gradingType;
        this.noChangeDate=noChangeDate;   
        
        if(draftStructureID==null){
            throw new IllegalArgumentException("EditDraftStructureQuery: draftStructureID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE draft_structure SET draftNum=?, assignID=?, gradingType=?, noChangeDate=? WHERE draftStructureID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE draft_structure SET");
            if(draftNum!=null){
                sqlCommand.append(" draftNum=?,");
            }
            if(assignID!=null){
                sqlCommand.append(" assignID=?,");
            }
            if(gradingType!=null){
                sqlCommand.append(" gradingType=?,");
            }
            if(noChangeDate!=null){
                sqlCommand.append(" noChangeDate=?,");
            }

            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE draftStructureID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(draftNum!=null){
                ps.setInt(currentIndex,draftNum.intValue());
                currentIndex++;
            }
            if(assignID!=null){
                ps.setLong(currentIndex,assignID.longValue());
                currentIndex++;
            }
            if(gradingType!=null){
                ps.setString(currentIndex,gradingType);
                currentIndex++;
            }
            if(noChangeDate!=null){
                ps.setTimestamp(currentIndex,noChangeDate);
                currentIndex++;
            }
            
            ps.setLong(currentIndex, draftStructureID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
