package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editGrader query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/19/2004
 */
public class EditGraderQuery implements SetQuery{
    
    Long graderID=			null;
    Integer universityID=		null;
    String firstName=			null;
    String lastName=			null;
    String userName=			null;
    String password=			null;
    String email=			null;
    String status=			null;
    Timestamp similarityCheckedDate=	null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditGraderQuery(Long graderID,
                        Integer universityID,
                        String firstName,
                        String lastName,
                        String userName,
                        String password,
                        String email,
                        String status,
                        Timestamp similarityCheckedDate){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.graderID=graderID;
        this.universityID=universityID;
        this.firstName=firstName;
        this.lastName=lastName;
        this.userName=userName;
        this.password=password;
        this.email=email;
        this.status=status;       
        this.similarityCheckedDate=similarityCheckedDate;
        
        if(graderID==null){
            throw new IllegalArgumentException("EditGraderQuery: graderID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE grader SET universityID=?, firstName=?, lastName=?, userName=?, password=?, email=?, status=?, similarityCheckedDate=?  WHERE graderID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE grader SET");
            if(universityID!=null){
                sqlCommand.append(" universityID=?,");
            }
            if(firstName!=null){
                sqlCommand.append(" firstName=?,");
            }
            if(lastName!=null){
                sqlCommand.append(" lastName=?,");
            }
            if(userName!=null){
                sqlCommand.append(" userName=?,");
            }
            if(password!=null){
                sqlCommand.append(" password=?,");
            }
            if(email!=null){
                sqlCommand.append(" email=?,");
            }
            if(status!=null){
                sqlCommand.append(" status=?,");
            }
            if(similarityCheckedDate!=null){
                sqlCommand.append(" similarityCheckedDate=?,");
            }
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE graderID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(universityID!=null){
                ps.setInt(currentIndex,universityID.intValue());
                currentIndex++;
            }
            if(firstName!=null){
                ps.setString(currentIndex,firstName);
                currentIndex++;
            }
            if(lastName!=null){
                ps.setString(currentIndex,lastName);
                currentIndex++;
            }
            if(userName!=null){
                ps.setString(currentIndex,userName);
                currentIndex++;
            }
            if(password!=null){
                ps.setString(currentIndex,password);
                currentIndex++;
            }
            if(email!=null){
                ps.setString(currentIndex,email);
                currentIndex++;
            }
            if(status!=null){
                ps.setString(currentIndex,status);
                currentIndex++;
            }
            if(similarityCheckedDate!=null){
                ps.setTimestamp(currentIndex,similarityCheckedDate);
                currentIndex++;
            }
            
            ps.setLong(currentIndex, graderID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
