package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editStaging query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 6/10/2004
 */
public class EditStagingQuery implements SetQuery{
    
    Long stagingID=		null;
    Long courseID=		null;
    Long graderID=		null;
    Long paperID=		null;
    String status=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditStagingQuery(Long stagingID,
                         Long courseID,
                         Long graderID,
                         Long paperID,
                         String status){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.stagingID=stagingID;
        this.courseID=courseID;
        this.graderID=graderID;        
        this.paperID=paperID;        
        this.status=status;
        if(stagingID==null){
            throw new IllegalArgumentException("EditStagingQuery: stagingID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE staging SET courseID=?, graderID=?, paperID=?, status=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE staging SET");
            if(courseID!=null){
                sqlCommand.append(" courseID=?,");
            }
            if(graderID!=null){
                sqlCommand.append(" graderID=?,");
            }
            if(paperID!=null){
                sqlCommand.append(" paperID=?,");
            }
            if(status!=null){
                sqlCommand.append(" status=?,");
            }
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE stagingID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(courseID!=null){
                ps.setLong(currentIndex,courseID.longValue());
                currentIndex++;
            }
            if(graderID!=null){
                ps.setLong(currentIndex,graderID.longValue());
                currentIndex++;
            }
            if(paperID!=null){
                ps.setLong(currentIndex,paperID.longValue());
                currentIndex++;
            }
            if(status!=null){
                ps.setString(currentIndex,status);
                currentIndex++;
            }
            
            ps.setLong(currentIndex, courseID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
