package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editUniversity query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/19/2004
 */
public class EditUniversityQuery implements SetQuery{
    
    Integer universityID=	null;
    String name=		null;
    String contactInfo=		null;
    String currentSemester=	null;
    Integer currentYear=	null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditUniversityQuery(Integer universityID,
                             String name,
                             String contactInfo,
                             String currentSemester,
                             Integer currentYear){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.universityID=universityID;
        this.name=name;
        this.contactInfo=contactInfo;
        this.currentSemester=currentSemester;
        this.currentYear=currentYear;      
        
        if(universityID==null){
            throw new IllegalArgumentException("EditUniversityQuery: universityID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE university SET name=?, contactInfo=?, currentSemester=?, currentYear=? WHERE universityID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE university SET");
            if(name!=null){
                sqlCommand.append(" name=?,");
            }
            if(contactInfo!=null){
                sqlCommand.append(" contactInfo=?,");
            }
            if(currentSemester!=null){
                sqlCommand.append(" currentSemester=?,");
            }
            if(currentYear!=null){
                sqlCommand.append(" currentYear=?,");
            }
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE universityID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(name!=null){
                ps.setString(currentIndex,name);
                currentIndex++;
            }
            if(contactInfo!=null){
                ps.setString(currentIndex,contactInfo);
                currentIndex++;
            }
            if(currentSemester!=null){
                ps.setString(currentIndex,currentSemester);
                currentIndex++;
            }
            if(currentYear!=null){
                ps.setInt(currentIndex,currentYear.intValue());
                currentIndex++;
            }
            
            ps.setInt(currentIndex, universityID.intValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
