package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editWriterAsGrader query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/24/2004
 */
public class EditWriterAsGraderQuery implements SetQuery{
    Long writerAsGraderID=	null;
    Long draftStructureID=	null;
    Long writerAsWriterID=	null;
    Long courseID=		null;
    Long rowID=			null;
    String completed=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditWriterAsGraderQuery(Long rowID,
                                 Long draftStructureID,
                                 Long writerAsGraderID,
                                 Long writerAsWriterID,
                                 Long courseID,
                                 String completed){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.rowID=rowID;
        this.draftStructureID=draftStructureID;
        this.writerAsWriterID=writerAsWriterID;
        this.courseID=courseID;    
        this.writerAsGraderID=writerAsGraderID;
        if(completed!=null && !completed.equals("")){
            this.completed=completed;
        }
        
        if(rowID==null){
            throw new IllegalArgumentException("EditWriterAsGraderQuery: rowID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE course_to_writer SET courseID=?, writerID=?, draftStructureID=?, writerAsWriterID=?, writerAsGraderID=?,completed=? WHERE rowID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE writer_as_grader SET");
            if(draftStructureID!=null){
                sqlCommand.append(" draftStructureID=?,");
            }
            if(writerAsWriterID!=null){
                sqlCommand.append(" writerAsWriterID=?,");
            }
            if(writerAsGraderID!=null){
                sqlCommand.append(" writerAsGraderID=?,");
            }
            if(courseID!=null){
                sqlCommand.append(" courseID=?,");
            }
            if(completed!=null){
                sqlCommand.append(" completed=?,");
            }
    
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE rowID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
    
            if(draftStructureID!=null){
                ps.setLong(currentIndex,draftStructureID.longValue());
                currentIndex++;
            }
            if(writerAsWriterID!=null){
                ps.setLong(currentIndex,writerAsWriterID.longValue());
                currentIndex++;
            }
            if(writerAsGraderID!=null){
                ps.setLong(currentIndex,writerAsGraderID.longValue());
                currentIndex++;
            }
            if(courseID!=null){
                ps.setLong(currentIndex,courseID.longValue());
                currentIndex++;
            }
            if(completed!=null){
                ps.setString(currentIndex,completed);
                currentIndex++;
            }
            
            ps.setLong(currentIndex, rowID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
        }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
