package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a editWriter query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/19/2004
 */
public class EditWriterQuery implements SetQuery{
    
    Long writerID=		null;
    Integer universityID=	null;
    String studentIdentifier=	null;
    String firstName=		null;
    String lastName=		null;
    String email=		null;
    String userName=		null;
    String password=		null;
    String userNameType=	null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
  public EditWriterQuery(Long writerID,
                        Integer universityID, 
                        String studentIdentifier, 
                        String firstName, 
                        String lastName, 
                        String email, 
                        String userName, 
                        String password, 
                        String userNameType){
        //in the non-String, non-Vector objects, null values are not checked for.
        this.writerID=writerID;
        this.universityID=universityID;
        this.studentIdentifier=studentIdentifier;
        this.firstName=firstName;
        this.lastName=lastName;
        this.email=email;
        this.userName=userName;
        this.password=password;
        this.userNameType=userNameType;   
        
        if(writerID==null){
            throw new IllegalArgumentException("EditWriterQuery: writerID required.");
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int update=0;
        try{
        /*UPDATE comment SET universityID=?, studentIdentifier=?, firstName=?, lastName=?, email=?, userName=?, password=?, userNameType=?
          WHERE writerID=?;
        */
            StringBuffer sqlCommand=new StringBuffer("UPDATE writer SET");
            if(universityID!=null){
                sqlCommand.append(" universityID=?,");
            }
            if(studentIdentifier!=null){
                sqlCommand.append(" studentIdentifier=?,");
            }
            if(firstName!=null){
                sqlCommand.append(" firstName=?,");
            }
            if(lastName!=null){
                sqlCommand.append(" lastName=?,");
            }
            if(email!=null){
                sqlCommand.append(" email=?,");
            }
            if(userName!=null){
                sqlCommand.append(" userName=?,");
            }
            if(password!=null){
                sqlCommand.append(" password=?,");
            }
            if(userNameType!=null){
                sqlCommand.append(" userNameType=?,");
            }
            sqlCommand.deleteCharAt(sqlCommand.length()-1);
            sqlCommand.append(" WHERE writerID=?;");
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
            ps.clearParameters();
            
            int currentIndex=1;
            //fill in the ?s
            if(universityID!=null){
                ps.setInt(currentIndex,universityID.intValue());
                currentIndex++;
            }
            if(studentIdentifier!=null){
                ps.setString(currentIndex,studentIdentifier);
                currentIndex++;
            }
            if(firstName!=null){
                ps.setString(currentIndex,firstName);
                currentIndex++;
            }
            if(lastName!=null){
                ps.setString(currentIndex,lastName);
                currentIndex++;
            }
            if(email!=null){
                ps.setString(currentIndex,email);
                currentIndex++;
            }
            if(userName!=null){
                ps.setString(currentIndex,userName);
                currentIndex++;
            }
            if(password!=null){
                ps.setString(currentIndex,password);
                currentIndex++;
            }
            if(userNameType!=null){
                ps.setString(currentIndex,userNameType);
                currentIndex++;
            }
            
            ps.setLong(currentIndex, writerID.longValue());
            
            //everything is filled in.  Do this shiznit...
            update=ps.executeUpdate();
         }finally{
            StaticContext.freeSQLConnection(c);
        }            
        return update;
    }
}
