package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getAdminID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetAdminIDQuery implements GetIDQuery {
    Timestamp creationDate=null;
    String firstName=null;
    String lastName=null;
    String userName=null;
    String email=null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetAdminIDQuery(Timestamp creationDate,
                            String firstName,
                            String lastName,
                            String userName,
                            String email){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        this.creationDate=creationDate;
        if(firstName!=null && !firstName.equals("")){
            this.firstName=firstName;
        }
        if(lastName!=null && !lastName.equals("")){
            this.lastName=lastName;
        }
        if(userName!=null && !userName.equals("")){
            this.userName=userName;
        }
        if(email!=null && !email.equals("")){
            this.email=email;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT a.adminID FROM admin a WHERE LEFT(a.creationDate,10)=? AND a.firstName=? AND
      a.lastName=? AND a.userName=? AND a.email=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT a.adminID FROM admin a");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(creationDate!=null || firstName!=null || lastName!=null || userName!=null || email!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(creationDate!=null){
            sqlCommand.append(" LEFT(a.creationDate,10)=?");//LEFT is used to have day scope.
            showAnd=true;
        }
        if(firstName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.firstName=?");
        }
        if(lastName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.lastName=?");
        }
        if(userName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.userName=?");
        }
        if(email!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.email=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetAdminIDQuery: for parameters:"+"("+
                " creationDate:"+creationDate+ " firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" email:"+email +
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(creationDate!=null){
            ps.setTimestamp(currentIndex,creationDate);
            currentIndex++;
        }
        if(firstName!=null){
            ps.setString(currentIndex,firstName);
            currentIndex++;
        }
        if(lastName!=null){
            ps.setString(currentIndex,lastName);
            currentIndex++;
        }
        if(userName!=null){
            ps.setString(currentIndex,userName);
            currentIndex++;
        }
        if(email!=null){
            ps.setString(currentIndex,email);
            currentIndex++;
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
