package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getAssignID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetAssignIDQuery implements GetIDQuery{
    Vector courseID=		null;
    String name=		null;
    Integer numDrafts=		null;
    Integer assignNum=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetAssignIDQuery(Vector courseID,
                            String name,
                            Integer numDrafts,
                            Integer assignNum){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(name!=null && !name.equals("")){
            this.name=name;
        }
        this.numDrafts=numDrafts;
        this.assignNum=assignNum;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT a.assignID FROM assignment a WHERE (a.courseID=? OR a.courseID=?)
      AND a.name=? AND a.numDrafts=? AND a.assignNum=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT a.assignID FROM assignment a");
        boolean showAnd=false;//tells whether to put the AND in or not.
                
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(courseID!=null || name!=null || numDrafts!=null || assignNum!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any assignID.
            int size=courseID.size();
            sqlCommand.append(" (a.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" a.courseID=?");
            }
            sqlCommand.append(")");
        }
        if(name!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.name=?");
        }
        if(numDrafts!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.numDrafts=?");
        }
        if(assignNum!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.assignNum=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetAssignIDQuery: for parameters: ( courseID:"+courseID+" name:"+name+
                " numDrafts:"+numDrafts+" assignNum:"+assignNum +
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(courseID!=null){
            //get each courseID, as an integer
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(name!=null){
            ps.setString(currentIndex,name);
            currentIndex++;
        }
        if(numDrafts!=null){
            ps.setInt(currentIndex,numDrafts.intValue());
            currentIndex++;
        }
        if(assignNum!=null){
            ps.setInt(currentIndex,assignNum.intValue());
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}

