package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getCheckoutID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetCheckoutIDQuery implements GetIDQuery{
    Vector graderID=		null;
    Vector paperID=		null;
    Timestamp time=		null;
    String type=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetCheckoutIDQuery(Vector graderID, Vector paperID, Timestamp time, String type){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
    
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(paperID!=null && !paperID.isEmpty()){
            this.paperID=paperID;
        }
        this.time=time;
        if(type!=null && !type.equals("")){
            this.type=type;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT c.checkoutID FROM checkout c WHERE (c.graderID=? OR c.graderID=?) AND
      (c.paperID=? OR c.paperID=?) AND LEFT(c.time,10)=? AND type=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT c.checkoutID FROM checkout c");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(graderID!=null || paperID!=null || time!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (c.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" c.graderID=?");
            }
            sqlCommand.append(")");
        }
        if(paperID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any paperID.
            int size=paperID.size();
            sqlCommand.append(" (c.paperID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" c.paperID=?");
            }
            sqlCommand.append(")");
        }
        if(time!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" LEFT(c.time,10)=?");//LEFT is used to have day scope.
            showAnd=true;
        }
        if(type!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" type=?");
            showAnd=true;
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetCheckoutIDQuery: for parameters:"+"("+
                " graderID:"+graderID+" paperID:"+paperID+" time:"+time+" type:"+type+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(paperID!=null){
            //get each paperID as a long
            Enumeration e=paperID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(time!=null){
            ps.setTimestamp(currentIndex,time);
            currentIndex++;
        }
        if(type!=null){
            ps.setString(currentIndex,type);
            currentIndex++;
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}

