package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getCommentID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetCommentIDQuery implements GetIDQuery{
    Vector paperID=		null;
    Vector draftID=		null;
    Vector graderID=		null;
    byte[] remarks=		null;
    Boolean isPublic=		null;
    Timestamp date=		null;
        
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetCommentIDQuery(Vector paperID,
                            Vector draftID,
                            Vector graderID,
                            byte[] remarks,
                            Boolean isPublic,
                            Timestamp date){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(paperID!=null && !paperID.isEmpty()){
            this.paperID=paperID;
        }
        if(draftID!=null && !draftID.isEmpty()){
            this.draftID=draftID;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(remarks!=null && remarks.length!=0){
            this.remarks=remarks;
        }
        this.isPublic=isPublic;
        this.date=date;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT co.commentID FROM comment co WHERE (co.paperID=? OR co.paperID=?) AND 
      (co.draftID=? OR co.draftID=?) AND(co.graderID=? OR co.graderID=?) AND co.remarks=? AND 
      co.isPublic=? AND LEFT(co.date,10)=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT co.commentID FROM comment co");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(paperID!=null || draftID!=null || graderID!=null || remarks!=null || isPublic!=null || date!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(paperID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any paperID.
            int size=paperID.size();
            sqlCommand.append(" (co.paperID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" co.paperID=?");
            }
            sqlCommand.append(")");
        }
        if(draftID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any draftID.
            int size=draftID.size();
            sqlCommand.append(" (co.draftID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" co.draftID=?");
            }
            sqlCommand.append(")");
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (co.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" co.graderID=?");
            }
            sqlCommand.append(")");
        }
        if(remarks!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" co.remarks=?");
        }
        if(isPublic!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" co.isPublic=?");
        }
        if(date!=null){
            sqlCommand.append(" LEFT(co.date,10)=?");//LEFT is used to have day scope.
            showAnd=true;
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetCommentIDQuery: for parameters:"+"("+
                " paperID:"+paperID+" draftID:"+draftID +" graderID:"+graderID +" remarks"+remarks +
                " isPublic:"+isPublic+" date:"+date+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(paperID!=null){
            //get each paperID as a long
            Enumeration e=paperID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(draftID!=null){
            //get each draftID as a long
            Enumeration e=draftID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(remarks!=null){
            ps.setBytes(currentIndex,remarks);
            currentIndex++;
        }
        if(isPublic!=null){
            int i=(isPublic.booleanValue())?1:0;//1 if true, 0 if false.
            ps.setInt(currentIndex,i);
            currentIndex++;
        }
        if(date!=null){
            ps.setTimestamp(currentIndex,date);
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
