package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getCourseID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetCourseIDQuery implements GetIDQuery{
    String contactInfo=		null;
    Vector universityID=	null;
    String dept=		null;
    String number=		null;
    String section=		null;
    String semester=		null;
    Integer year=		null;
    Vector graderID=		null;
    Vector writerID=		null;
    String isCurrent=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetCourseIDQuery(String contactInfo,
                            Vector universityID,
                            String dept,
                            String number,
                            String section,
                            String semester,
                            Integer year,
                            Vector graderID,
                            Vector writerID,
                            String isCurrent){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        this.year=year;
        if(universityID!=null && !universityID.isEmpty()){
            this.universityID=universityID;
        }
        if(contactInfo!=null && !contactInfo.equals("")){
            this.contactInfo=contactInfo;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(writerID!=null && !writerID.isEmpty()){
            this.writerID=writerID;
        }
        if(dept!=null && !dept.equals("")){
            this.dept=dept;
        }
        if(number!=null && !number.equals("")){
            this.number=number;
        }
        if(section!=null && !section.equals("")){
            this.section=section;
        }
        if(semester!=null && !semester.equals("")){
            this.semester=semester;
        }
        if(isCurrent!=null && !isCurrent.equals("")){
            this.isCurrent=isCurrent;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT c.courseID FROM course c, course_to_grader cg WHERE c.contactInfo=? AND (c.universityID=? OR 
      c.universityID=?) AND c.dept=? AND c.number=? AND c.section=? AND c.semester=? AND c.year=? AND 
      (cg.graderID=? OR cg.graderID=?) AND c.courseID=cg.courseID AND c.isCurrent=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT c.courseID FROM course c");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //finish the sqlCommand
        if(graderID!=null){
            sqlCommand.append(", course_to_grader cg");
        }
        if(writerID!=null){
            sqlCommand.append(", course_to_writer cw");
        }
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(contactInfo!=null || universityID!=null || dept!=null || number!=null || section!=null ||
           semester!=null || year!=null || graderID!=null || writerID!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(contactInfo!=null){
            sqlCommand.append(" c.contactInfo=?");
            showAnd=true;
        }
        if(universityID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any universityID.
            int size=universityID.size();
            sqlCommand.append(" (c.universityID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" c.universityID=?");
            }
            sqlCommand.append(")");
        }
        if(dept!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" c.dept=?");
        }
        if(number!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" c.number=?");
        }
        if(section!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" c.section=?");
        }
        if(semester!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" c.semester=?");
        }
        if(year!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" c.year=?");
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (cg.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.graderID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND c.courseID=cg.courseID");
        }
        if(writerID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any writerID.
            int size=writerID.size();
            sqlCommand.append(" (cw.writerID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cw.writerID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND c.courseID=cw.courseID");
        }
        if(isCurrent!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" c.isCurrent=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetCourseIDQuery: for parameters: ( graderID:"+graderID+" writerID:"+writerID+
                " contactInfo:"+contactInfo+" universityID:"+universityID +" dept:"+dept + 
                " number:"+number +" section:"+section +" semester:"+semester +" isCurrent:"+isCurrent+
                " year"+year+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(contactInfo!=null){
            ps.setString(currentIndex,contactInfo);
            currentIndex++;
        }
        if(universityID!=null){
            //get each universityID, as an integer
            Enumeration e=universityID.elements();
            while(e.hasMoreElements()){
                ps.setInt(currentIndex,((Integer)e.nextElement()).intValue());
                currentIndex++;
            }
        }
        if(dept!=null){
            ps.setString(currentIndex,dept);
            currentIndex++;
        }
        if(number!=null){
            ps.setString(currentIndex,number);
            currentIndex++;
        }
        if(section!=null){
            ps.setString(currentIndex,section);
            currentIndex++;
        }
        if(semester!=null){
            ps.setString(currentIndex,semester);
            currentIndex++;
        }
        if(year!=null){
            ps.setInt(currentIndex,year.intValue());
            currentIndex++;
        }
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(writerID!=null){
            //get each writerID as a long
            Enumeration e=writerID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(isCurrent!=null){
            ps.setString(currentIndex,isCurrent);
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}

