package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getCourseToGraderID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetCourseToGraderIDQuery implements GetIDQuery{
    Vector courseID=		null;
    Vector graderID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetCourseToGraderIDQuery(Vector courseID, Vector graderID){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
    
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT cg.courseToGraderID FROM course_to_grader cg WHERE (cg.courseID=? OR cg.courseID=?) AND
      (cg.graderID=? OR cg.graderID=?);
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT cg.courseToGraderID FROM course_to_grader cg");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(courseID!=null || graderID!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any courseID.
            int size=courseID.size();
            sqlCommand.append(" (cg.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.courseID=?");
            }
            sqlCommand.append(")");
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (cg.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.graderID=?");
            }
            sqlCommand.append(")");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetCourseToGraderIDQuery: for parameters:"+"("+
                " courseID:"+courseID+" graderID:"+graderID+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(courseID!=null){
            //get each courseID as a long
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}

