package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getDraftID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetDraftIDQuery implements GetIDQuery{
    Vector paperID=		null;
    Integer draftNumber=	null;
    String title=		null;
    String body=		null;
    Timestamp date=		null;
    Vector draftStructureID=	null;
    String draftType=		null;
        
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetDraftIDQuery( Vector paperID,
                            Integer draftNumber,
                            String title,
                            String body,
                            Timestamp date,
                            Vector draftStructureID,
                            String draftType){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(paperID!=null && !paperID.isEmpty()){
            this.paperID=paperID;
        }
        this.draftNumber=draftNumber;
        if(title!=null && !title.equals("")){
            this.title=title;
        }
        if(body!=null && !body.equals("")){
            this.body=body;
        }
        if(draftStructureID!=null && !draftStructureID.equals("")){
            this.draftStructureID=draftStructureID;
        }
        if(draftType!=null && !draftType.equals("")){
            this.draftType=draftType;
        }
        this.date=date;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT d.draftID FROM draft d WHERE (d.paperID=? OR d.paperID=?) AND d.draftNumber=? 
      AND d.title=? AND d.body=? AND LEFT(d.date,10)=? AND (d.draftStructureID=? OR d.draftStructureID=?
      AND draftType=?);
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT d.draftID FROM draft d");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(paperID!=null || draftNumber!=null || title!=null || body!=null || date!=null || draftStructureID!=null || draftType!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(paperID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any paperID.
            int size=paperID.size();
            sqlCommand.append(" (d.paperID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" d.paperID=?");
            }
            sqlCommand.append(")");
        }
        if(draftNumber!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" d.draftNumber=?");
        }
        if(title!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" d.title=?");
        }
        if(body!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" d.body=?");
        }
        if(date!=null){
            sqlCommand.append(" LEFT(d.date,10)=?");//LEFT is used to have day scope.
            showAnd=true;
        }
        if(draftStructureID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any paperID.
            int size=draftStructureID.size();
            sqlCommand.append(" (d.draftStructureID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" d.draftStructureID=?");
            }
            sqlCommand.append(")");
        }
        if(draftType!=null){
            sqlCommand.append(" d.draftType=?");
            showAnd=true;
        }

        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetDraftIDQuery: for parameters:"+"("+
                " paperID:"+paperID+" draftNumber:"+draftNumber +" title:"+title +
                " body"+body +" date:"+date+" draftStructureID:"+draftStructureID+
                "draftType:"+draftType+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(paperID!=null){
            //get each paperID as a long
            Enumeration e=paperID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(draftNumber!=null){
            ps.setInt(currentIndex,draftNumber.intValue());
            currentIndex++;
        }
        if(title!=null){
            ps.setString(currentIndex,title);
            currentIndex++;
        }
        if(body!=null){
            ps.setString(currentIndex,body);
            currentIndex++;
        }
        if(date!=null){
            ps.setTimestamp(currentIndex,date);
            currentIndex++;
        }
        if(draftStructureID!=null){
            //get each paperID as a long
            Enumeration e=draftStructureID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(draftType!=null){
            ps.setString(currentIndex, draftType);
            currentIndex++;
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
