package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getDraftStructureID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetDraftStructureIDQuery implements GetIDQuery{
    Integer draftNum=		null;
    Vector assignID=		null;
    String gradingType=		null;
    Timestamp noChangeDate=	null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetDraftStructureIDQuery(Integer draftNum,
                            Vector assignID,
                            String gradingType,
                            Timestamp noChangeDate){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        this.draftNum=draftNum;
        if(assignID!=null && !assignID.isEmpty()){
            this.assignID=assignID;
        }
        if(gradingType!=null && !gradingType.equals("")){
            this.gradingType=gradingType;
        }
        this.noChangeDate=noChangeDate;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT ds.draftStructureID FROM draft_structure ds WHERE ds.draftNum=? AND (ds.assignID=? OR 
      ds.assignID=?) AND ds.gradingType=? AND a.noChangeDate=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT ds.draftStructureID FROM draft_structure ds");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(draftNum!=null || assignID!=null || gradingType!=null || noChangeDate!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(draftNum!=null){
            sqlCommand.append(" ds.draftNum=?");
            showAnd=true;
        }
        if(assignID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any assignID.
            int size=assignID.size();
            sqlCommand.append(" (ds.assignID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" ds.assignID=?");
            }
            sqlCommand.append(")");
        }
        if(gradingType!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ds.gradingType=?");
        }
        if(noChangeDate!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" a.noChangeDate=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetDraftStructureIDQuery: for parameters: ( draftNum:"+draftNum+" assignID:"+assignID+
                " gradingType:"+gradingType+" noChangeDate:"+noChangeDate +") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(draftNum!=null){
            ps.setInt(currentIndex,draftNum.intValue());
            currentIndex++;
        }
        if(assignID!=null){
            //get each assignID, as an integer
            Enumeration e=assignID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(gradingType!=null){
            ps.setString(currentIndex,gradingType);
            currentIndex++;
        }
        if(noChangeDate!=null){
            ps.setTimestamp(currentIndex,noChangeDate);
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}

