package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getGraderID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetGraderIDQuery implements GetIDQuery {
    Timestamp creationDate=null;
    Vector universityID=null;
    String firstName=null;
    String lastName=null;
    String userName=null;
    String email=null;
    String status=null;
    Timestamp similarityCheckedDate=null;
    Vector courseID=null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetGraderIDQuery(Timestamp creationDate,
                            Vector universityID,
                            String firstName,
                            String lastName,
                            String userName,
                            String email,
                            String status,
                            Timestamp similarityCheckedDate,
                            Vector courseID){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        this.creationDate=creationDate;
        if(firstName!=null && !firstName.equals("")){
            this.firstName=firstName;
        }
        if(lastName!=null && !lastName.equals("")){
            this.lastName=lastName;
        }
        if(userName!=null && !userName.equals("")){
            this.userName=userName;
        }
        if(email!=null && !email.equals("")){
            this.email=email;
        }
        if(status!=null && !status.equals("")){
            this.status=status;
        }
        this.similarityCheckedDate=similarityCheckedDate;
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(universityID!=null && !universityID.isEmpty()){
            this.universityID=universityID;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT g.graderID FROM grader g, course_to_grader cg WHERE LEFT(g.creationDate,10)=? AND 
      (g.universityID=? OR g.universityID=?) AND g.firstName=? AND g.lastName=? AND g.userName=? AND 
      g.email=? AND g.status=? AND LEFT(g.similarityCheckedDate,10)=? AND
      cg.courseID=? AND g.graderID=cg.graderID;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT g.graderID FROM grader g");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //finish the sqlCommand
        if(courseID!=null){
            sqlCommand.append(", course_to_grader cg");
        }
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(creationDate!=null || universityID!=null || firstName!=null || lastName!=null || 
           userName!=null || email!=null || status!=null || similarityCheckedDate!=null || courseID!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(creationDate!=null){
            sqlCommand.append(" LEFT(g.creationDate,10)=?");//LEFT is used to have day scope.
            showAnd=true;
        }
        if(universityID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any universityID.
            int size=universityID.size();
            sqlCommand.append(" (g.universityID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" g.universityID=?");
            }
            sqlCommand.append(")");
        }
        if(firstName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" g.firstName=?");
        }
        if(lastName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" g.lastName=?");
        }
        if(userName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" g.userName=?");
        }
        if(email!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" g.email=?");
        }
        if(status!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" g.status=?");
        }
        if(similarityCheckedDate!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" LEFT(g.similarityCheckedDate,10)=?");//LEFT is used to have day scope.
        }
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any courseID.
            int size=courseID.size();
            sqlCommand.append(" (cg.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.courseID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND g.graderID=cg.graderID");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetGraderIDQuery: for parameters:"+"("+
                " creationDate:"+creationDate+ " universityID:"+universityID+ " firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" email:"+email +" status:"+status +
                " similarityCheckedDate"+similarityCheckedDate +" courseID:"+courseID+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(creationDate!=null){
            ps.setTimestamp(currentIndex,creationDate);
            currentIndex++;
        }
        if(universityID!=null){
            //get each universityID, as an integer
            Enumeration e=universityID.elements();
            while(e.hasMoreElements()){
                ps.setInt(currentIndex,((Integer)e.nextElement()).intValue());
                currentIndex++;
            }
        }
        if(firstName!=null){
            ps.setString(currentIndex,firstName);
            currentIndex++;
        }
        if(lastName!=null){
            ps.setString(currentIndex,lastName);
            currentIndex++;
        }
        if(userName!=null){
            ps.setString(currentIndex,userName);
            currentIndex++;
        }
        if(email!=null){
            ps.setString(currentIndex,email);
            currentIndex++;
        }
        if(status!=null){
            ps.setString(currentIndex,status);
            currentIndex++;
        }
        if(similarityCheckedDate!=null){
            ps.setTimestamp(currentIndex,similarityCheckedDate);
            currentIndex++;
        }
        if(courseID!=null){
            //get each courseID as a long
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
