package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * This is a speed query, it returns the number of drafts a paper has.
 * Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetNumberOfDraftsQuery{
    Long paperID=null;
    
    
    static final String statement="SELECT MAX(draftNumber) FROM draft WHERE paperID=? LIMIT 1;";
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetNumberOfDraftsQuery(Long paperID){
        this.paperID=paperID;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    /**
     * Returns the current number of drafts for a paper (by the max of the draftNumber column of that paper)
     */
    public int execute() throws SQLException{
        try{
            if(paperID==null){
                return 0;
            }
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setLong(1,paperID.longValue());
                    
            //everything is filled in.  Do this shiznit...
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                return rs.getInt(1);
            }
            else{
                return 0;//no drafts for that paperID
            }
        }finally{
            //return the connection to the pool
            StaticContext.freeSQLConnection(c);
        }
    }
}
