package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getPaperForStagingID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 6/11/2004
 */
public class GetPaperIDForStagingQuery implements GetIDQuery{
    Vector stagingID=		null;
    Vector graderID=		null;   //there will probably be only one graderID, but this is left as a Vector in case
                                        //this command becomes an allowed action for admins
            
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetPaperIDForStagingQuery( Vector stagingID,
                            Vector graderID){

        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(stagingID!=null && !stagingID.isEmpty()){
            this.stagingID=stagingID;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT s.paperID FROM paper p, staging s, course_to_grader cg WHERE (s.stagingID=? OR s.stagingID=?) 
      AND (p.graderID=? OR p.graderID=?) AND p.graderID=s.graderID
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT s.paperID FROM paper p, staging s, course_to_grader cg");
                
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(stagingID!=null || graderID!=null ){
            sqlCommand.append(" WHERE");
        }
        
        if(stagingID!=null){
            showAnd=true;
            //IDs can have more than one.  So return results that match any universityID.
            int size=stagingID.size();
            sqlCommand.append(" (s.stagingID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" s.stagingID=?");
            }
            sqlCommand.append(")");
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (cg.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.graderID=?");
            }
            sqlCommand.append(") AND cg.graderID=s.graderID AND cg.courseID=p.courseID");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperIDForStagingQuery: for parameters: ( stagingID:"+stagingID +
                " graderID:"+graderID+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(stagingID!=null){
            //get each stagingID, as a long
            Enumeration e=stagingID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
