package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.blaise.version2.transmission.TransmissionConstants;
import edu.gwu.blaise.version2.backend.packagers.PackagingConstants;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getPaperID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetPaperIDQuery implements GetIDQuery{
    Integer version=		null;
    Vector writerID=		null;
    Vector universityID=	null;
    Vector courseID=		null;
    Vector graderID=		null;
    String archivalPractice=	null;
    String lastName=		null;
    String studentIdentifier=	null;
    String email=		null;
    Vector wagID=		null;
    String type=		null;
    Vector aID=			null;
    String hasBeenDownloaded=	null;
    String completed=		null;
            
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetPaperIDQuery( Integer version,
                            Vector writerID,
                            Vector universityID,
                            Vector courseID,
                            Vector graderID,
                            String archivalPractice,
                            String lastName,
                            String studentIdentifier,
                            String email,
                            Vector wagID,
                            String type,
                            Vector aID,
                            String hasBeenDownloaded,
                            String completed){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(version!=null){
	        this.version=version;
	    }
        if(type!=null && !type.equals("")){
            this.type=type;
        }
        if(universityID!=null && !universityID.isEmpty()){
            this.universityID=universityID;
        }
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(writerID!=null && !writerID.isEmpty()){
            this.writerID=writerID;
        }
        if(archivalPractice!=null && !archivalPractice.equals("")){
            this.archivalPractice=archivalPractice;
        }
        if(lastName!=null && !lastName.equals("")){
            this.lastName=lastName;
        }
        if(studentIdentifier!=null && !studentIdentifier.equals("")){
            this.studentIdentifier=studentIdentifier;
        }
        if(email!=null && !email.equals("")){
            this.email=email;
        }
        if(wagID!=null && !wagID.isEmpty()){
            this.wagID=wagID;
        }
        if(aID!=null && !aID.isEmpty()){
            this.aID=aID;
        }
        if(hasBeenDownloaded!=null && !hasBeenDownloaded.equals("")){
            this.hasBeenDownloaded=hasBeenDownloaded;
        }
        if(completed!=null && !completed.equals("")){
            this.completed=completed;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT p.paperID FROM paper p, course_to_grader cg, course_to_writer cw, writer w, writer_as_grader wag, assignment a, draft d, draft_structure ds
      WHERE p.version=? AND p.archivalPractice=? AND (p.courseID=? OR p.courseID=?) AND 
      (p.writerID=? OR p.writerID=?) AND (p.universityID=? OR p.universityID=?) AND (cg.graderID=? 
      OR cg.graderID=?) AND (wag.rowID=? OR wag.rowID=?) AND wag.writerAsGraderID=w.writerID AND cg.courseID=cw.courseID AND 
      cw.writerID=p.writerID AND p.writerID=w.writerID AND w.email=? AND w.studentIdentifier=? AND w.lastName=? AND
      (a.assignID=? OR a.assignID=?) AND p.hasBeenDownloaded=? AND a.assignID=p.assignID AND d.draftStructureID=wag.draftStructureID 
      AND d.paperID=p.paperID AND wag.completed=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT p.paperID FROM paper p");
        if(graderID!=null){
            sqlCommand.append(", course_to_grader cg, course_to_writer cw");
        }
        if(lastName!=null || studentIdentifier!=null || email!=null){
            sqlCommand.append(", writer w");
        }
        if(wagID!=null){
            sqlCommand.append(", writer_as_grader wag");
        }
        if(aID!=null){
            sqlCommand.append(", assignment a, draft_structure ds");
        }
        if(aID!=null || wagID!=null){
            sqlCommand.append(", draft d");
        }
        
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(version!=null || archivalPractice!=null || universityID!=null || writerID!=null 
           || courseID!=null || graderID!=null || lastName!=null || studentIdentifier!=null || email!=null 
           || wagID!=null || aID!=null || type!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(version!=null){
            sqlCommand.append(" p.version=?");
            showAnd=true;
        }
        if(archivalPractice!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" p.archivalPractice=?");
        }
        if(universityID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any universityID.
            int size=universityID.size();
            sqlCommand.append(" (p.universityID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" p.universityID=?");
            }
            sqlCommand.append(")");
        }
        if(writerID!=null && (wagID==null || !type.equals(PackagingConstants.LIMIT_writer))){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any writerID.
            int size=writerID.size();
            sqlCommand.append(" (p.writerID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" p.writerID=?");
            }
            sqlCommand.append(")");
        }
        if(writerID!=null && (wagID!=null && type.equals(PackagingConstants.LIMIT_writer))){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any writerID.
            int size=writerID.size();
            sqlCommand.append(" (wag.writerAsGraderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" wag.writerAsGraderID=?");
            }
            sqlCommand.append(")");
        }
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any courseID.
            int size=courseID.size();
            sqlCommand.append(" (p.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" p.courseID=?");
            }
            sqlCommand.append(")");
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (cg.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.graderID=?");
            }
            sqlCommand.append(") AND p.courseID=cg.courseID");
        }
        if(wagID!=null && type.equals(PackagingConstants.LIMIT_writer)){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any wagID.
            int size=wagID.size();
            sqlCommand.append(" (wag.rowID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" wag.rowID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND d.draftStructureID=wag.draftStructureID AND d.paperID=p.paperID AND wag.writerAsWriterID=p.writerID");
            if(lastName!=null || email!=null || studentIdentifier!=null){
                sqlCommand.append(" AND wag.writerAsGraderID=w.writerID");
            }
        }

        if(lastName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.lastName=?");
        }
        if(email!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.email=?");
        }
        if(studentIdentifier!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.studentIdentifier=?");
        }
        if(aID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any assignID.
            int size=aID.size();
            sqlCommand.append(" (a.assignID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" a.assignID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND a.assignID=p.assignID");
        }
        if(hasBeenDownloaded!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" p.hasBeenDownloaded=?");
        }
        
        if(lastName!=null || studentIdentifier!=null || email!=null){
            sqlCommand.append(" AND p.writerID=w.writerID");
        }
        if(wagID!=null && completed!=null){
            sqlCommand.append(" AND wag.completed=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperIDQuery: for parameters: ( version:"+version+" archivalPractice:"+
                archivalPractice +" universityID:"+universityID +
                " writerID"+writerID +" courseID:"+courseID+" graderID:"+graderID+" lastName:"+lastName +
                " studentIdentifier:"+studentIdentifier +" email:"+email+" completed:"+completed+" assignments:"+aID+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(version!=null){
            ps.setInt(currentIndex,version.intValue());
            currentIndex++;
        }
        if(archivalPractice!=null){
            ps.setString(currentIndex,archivalPractice);
            currentIndex++;
        }
        if(universityID!=null){
            //get each universityID, as an integer
            Enumeration e=universityID.elements();
            while(e.hasMoreElements()){
                ps.setInt(currentIndex,((Integer)e.nextElement()).intValue());
                currentIndex++;
            }
        }
        if(writerID!=null && !(wagID!=null && !type.equals(PackagingConstants.LIMIT_writer))){
            //get each writerID as a long
            Enumeration e=writerID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(courseID!=null){
            //get each courseID as a long
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(wagID!=null && type.equals(PackagingConstants.LIMIT_writer)){
            //get each wagID as a long
            Enumeration e=wagID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(lastName!=null){
            ps.setString(currentIndex,lastName);
            currentIndex++;
        }
        if(email!=null){
            ps.setString(currentIndex,email);
            currentIndex++;
        }
        if(studentIdentifier!=null){
            ps.setString(currentIndex,studentIdentifier);
            currentIndex++;
        }
        if(aID!=null){
            //get each assignID as a long
            Enumeration e=aID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(wagID!=null && completed!=null){
            ps.setString(currentIndex,completed);
            currentIndex++;
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
