package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getPaperUngradedID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetPaperUngradedIDQuery implements GetIDQuery{
    Vector graderID=		null;
    String lastName=		null;
    String studentIdent=	null;
    String email=		null;
    Vector courseID=		null;
    Vector aID=			null;
            
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetPaperUngradedIDQuery(Vector graderID,String lastName,String studentIdent,String email,Vector courseID,Vector aID){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(lastName!=null && !lastName.equals("")){
            this.lastName=lastName;
        }
        if(studentIdent!=null && !studentIdent.equals("")){
            this.studentIdent=studentIdent;
        }
        if(email!=null && !email.equals("")){
            this.email=email;
        }
        if(aID!=null && !aID.isEmpty()){
            this.aID=aID;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
        /*CREATE TEMPORARY TABLE myDraft
        SELECT DISTINCT d.* FROM draft d, course_to_writer cw, paper p, writer w, assignment a, draft_structure ds WHERE p.courseID=cg.courseID AND (cg.graderID=? OR cg.graderID=?) AND d.paperID=p.paperID AND w.lastName=? AND w.studentIdentifier=? AND w.email=? AND (cw.courseID=? OR cw.courseID=?) AND (a.assignID=? OR a.assignID=?) AND a.assignID=p.assignID AND ds.assignID=a.assignID AND ds.draftStructureID=d.draftStructureID; 
        CREATE TEMPORARY TABLE myComment
        SELECT c.* FROM comment c WHERE (c.graderID=? OR c.graderID=?);
        CREATE TEMPORARY TABLE myUngradedPaper
        SELECT DISTINCT d.paperID, d.draftID FROM myDraft d LEFT JOIN myComment c ON d.draftID=c.draftID WHERE c.draftID IS NULL;
        DROP TABLE myDraft;
        DROP TABLE myComment;
        CREATE TEMPORARY TABLE myCheckout
        SELECT DISTINCT ch.paperID FROM checkout ch WHERE (ch.graderID=? OR ch.graderID=?);
        SELECT DISTINCT p.paperID FROM myUngradedPaper p LEFT JOIN myCheckout ch ON p.paperID=ch.paperID WHERE ch.paperID IS NULL;
        DROP TABLE myUngradedPaper;
        DROP TABLE myCheckout;*/
        
        
        if(graderID==null){//graderIDs are required.
            return null;
        }
        
        //first get all drafts that match the parameters.
        StringBuffer sqlCommand=new 
        StringBuffer("CREATE TEMPORARY TABLE myDraft SELECT DISTINCT d.* FROM draft d, course_to_grader cg, paper p");
        
        if(lastName!=null || studentIdent!=null || email!=null){
            sqlCommand.append(", writer w");
        }
        if(aID!=null){
            sqlCommand.append(", assignment a, draft_structure ds");
        }
        
        sqlCommand.append(" WHERE p.courseID=cg.courseID AND d.paperID=p.paperID AND");
        
        if(lastName!=null || studentIdent!=null || email!=null){
            sqlCommand.append(" p.writerID=w.writerID AND");
        }
        
        //IDs can have more than one.  So return results that match any graderID.
        int size=graderID.size();
        sqlCommand.append(" (cg.graderID=?");	//the first one doesn't have an OR infront
        for(int i=1;i<size;i++){
            sqlCommand.append(" OR");		//the next ones do.
            sqlCommand.append(" cg.graderID=?");
        }
        sqlCommand.append(")");
        if(lastName!=null){
            sqlCommand.append(" AND w.lastName=?");
        }
        if(studentIdent!=null){
            sqlCommand.append(" AND w.studentIdentifier=?");
        }
        if(email!=null){
            sqlCommand.append(" AND w.email=?");
        }
        if(courseID!=null){
            size=courseID.size();
            sqlCommand.append(" AND (p.courseID=?");
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");
                sqlCommand.append(" p.courseID=?");
            }
            sqlCommand.append(")");
        }
        if(aID!=null){
            size=courseID.size();
            sqlCommand.append(" AND (a.assignID=?");
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");
                sqlCommand.append(" a.assignID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND p.assignID=a.assignID");
        }

        sqlCommand.append(";");
        
        //first query is assembled.
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (1): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +" aID:"+aID+
                " email:"+email+" courseID:"+courseID+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //get each graderID as a long
        Enumeration e=graderID.elements();
        while(e.hasMoreElements()){
            ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
            currentIndex++;
        }
        
        if(lastName!=null){
            ps.setString(currentIndex,lastName);
            currentIndex++;
        }
        if(studentIdent!=null){
            ps.setString(currentIndex,studentIdent);
            currentIndex++;
        }
        if(email!=null){
            ps.setString(currentIndex,email);
            currentIndex++;
        }
        if(courseID!=null){
            e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(aID!=null){
            e=aID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        //okay, the first query is done, so execute it.
        ps.execute();
        
        //next, get all of my comments.
        sqlCommand=new StringBuffer("CREATE TEMPORARY TABLE myComment SELECT c.* FROM comment c WHERE");
        //IDs can have more than one.  So return results that match any graderID.
        sqlCommand.append(" (c.graderID=?");	//the first one doesn't have an OR infront
        for(int i=1;i<size;i++){
            sqlCommand.append(" OR");		//the next ones do.
            sqlCommand.append(" c.graderID=?");
        }
        sqlCommand.append(");");
                
        //prepare the statement.
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (2): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +
                " email:"+email+") the following statement was assembled: "+sqlCommand
            );
        }
        ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        currentIndex=1;
        //get each graderID as a long
        e=graderID.elements();
        while(e.hasMoreElements()){
            ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
            currentIndex++;
        }
        
        ps.execute();
        
        //now get my ungraded papers.
        //prepare the statement. (no parameters to fill in)
        sqlCommand=new StringBuffer("CREATE TEMPORARY TABLE myUngradedPaper SELECT DISTINCT d.paperID, d.draftID FROM myDraft d LEFT JOIN myComment c ON d.draftID=c.draftID WHERE c.draftID IS NULL;");
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (3): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +
                " email:"+email+") the following statement was assembled: "+sqlCommand
            );
        }
        ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        ps.execute();
        
        //drop the first two tables.
        //prepare the statement. (no parameters to fill in)
        sqlCommand=new StringBuffer("DROP TABLE myDraft;");
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (4): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +
                " email:"+email+") the following statement was assembled: "+sqlCommand
            );
        }
        ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        ps.execute();
        sqlCommand=new StringBuffer("DROP TABLE myComment;");
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (5): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +
                " email:"+email+") the following statement was assembled: "+sqlCommand
            );
        }
        ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        ps.execute();
        
        //get all of my current checkouts.
        sqlCommand=new StringBuffer("CREATE TEMPORARY TABLE myCheckout SELECT DISTINCT ch.paperID FROM checkout ch WHERE");
        sqlCommand.append(" (ch.graderID=?");	//the first one doesn't have an OR infront
        for(int i=1;i<size;i++){
            sqlCommand.append(" OR");		//the next ones do.
            sqlCommand.append(" ch.graderID=?");
        }
        sqlCommand.append(");");
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (6): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +
                " email:"+email+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        currentIndex=1;
        //get each graderID as a long
        e=graderID.elements();
        while(e.hasMoreElements()){
            ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
            currentIndex++;
        }
        ps.execute();

        //finally, finally, finally-get the ResultSet.
        sqlCommand=new StringBuffer("SELECT DISTINCT p.paperID FROM myUngradedPaper p LEFT JOIN myCheckout ch ON p.paperID=ch.paperID WHERE ch.paperID IS NULL;");
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetPaperUngradedIDQuery (7): for parameters:"+"("+
                " graderID:"+graderID+" lastName:"+lastName +" studentIdent:"+studentIdent +
                " email:"+email+") the following statement was assembled: "+sqlCommand
            );
        }
        ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            //drop those two tables...
            try{
                PreparedStatement ps=c.prepareStatement("DROP TABLE myUngradedPaper;");
                ps.clearParameters();
                ps.execute();
                ps=c.prepareStatement("DROP TABLE myCheckout;");
                ps.clearParameters();
                ps.execute();
            }catch(SQLException e){//log it and ignore it.
                StringBuffer ex=new StringBuffer(e.getMessage());
                while((e=e.getNextException())!=null){
                    ex.append(e.getMessage());
                }
                //from edu.gwu.servlets
                BaseServlet.writeLog("Database error occurred in GetPaperUngradedIDQuery (free): "+ex.toString()+
                    " With parameters:" +" graderID:"+graderID+
                    " lastName:"+lastName+" studentIdent:"+studentIdent +
                    " email:"+email);
            }

            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
