package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.packagers.PackagingConstants;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs get queries (except getPaperFull/getPaperHeader queries).  
 * Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetQuery implements GetIDQuery{
    private Vector IDs=		null;
    private String table=	null;
    private String limitBy=	null;
    private Long limitID=	null;
    
    /** Our pooled connection. */
    private Connection c=StaticContext.getSQLConnection();
    
    public GetQuery(Vector IDs, String table, String limitBy, Long limitID){
    
        if(IDs!=null && !IDs.isEmpty()){
            this.IDs=IDs;
        }
        if(table!=null && !table.equals("")){
            this.table=table;
        }
        if(limitBy!=null && !limitBy.equals("")){
            this.limitBy=limitBy;
        }
        this.limitID=limitID;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
        if(table==null){//table is required.
            return null;
        }
        //check limitBy type
        if(limitBy==null){//limitBy is required.
            return null;
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(limitID==null){
                return null;//needs to have their graderID included.
            }
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(limitID==null){
                return null;//needs to have their writerID included.
            }
        }
        else{
            return null;//invalid limitBy type
        }
        
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT");
        
        if(table.equals(PackagingConstants.TABLE_course)){
            sqlCommand.append(" c.*");
        }
        else if(table.equals(PackagingConstants.TABLE_writer)){
            sqlCommand.append(" w.*");
        }
        else if(table.equals(PackagingConstants.TABLE_grader)){
            sqlCommand.append(" g.*");
        }
        else if(table.equals(PackagingConstants.TABLE_admin)){
            sqlCommand.append(" a.*");
        }
        else if(table.equals(PackagingConstants.TABLE_university)){
            sqlCommand.append(" u.*");
        }
        else if(table.equals(PackagingConstants.TABLE_ctg)){
            sqlCommand.append(" cg.*");
        }
        else if(table.equals(PackagingConstants.TABLE_ctw)){
            sqlCommand.append(" cw.*");
        }
        else if(table.equals(PackagingConstants.TABLE_checkout)){
            sqlCommand.append(" ch.*");
        }
        else if(table.equals(PackagingConstants.TABLE_paper)){
            sqlCommand.append(" p.*");
        }
        else if(table.equals(PackagingConstants.TABLE_draft)){
            sqlCommand.append(" d.*");
        }
        else if(table.equals(PackagingConstants.TABLE_comment)){
            sqlCommand.append(" co.*");
        }
        else if(table.equals(PackagingConstants.TABLE_eq)){
            sqlCommand.append(" e.*");
        }
        else if(table.equals(PackagingConstants.TABLE_ds)){
            sqlCommand.append(" ds.*");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment)){
            sqlCommand.append(" ag.*");
        }
        else if(table.equals(PackagingConstants.TABLE_wag)){
            sqlCommand.append(" wag.*");
        }
        else if(table.equals(PackagingConstants.TABLE_staging)){
            sqlCommand.append(" s.*");
        }
        else{
            return null;//either invalid table name, or not handled by this class
        }
        
        sqlCommand.append(" FROM");
        
        if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" course c");
        }
        else if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit by courses grader is in.
            sqlCommand.append(" course c, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit to courses writer is in.
            sqlCommand.append(" course c, course_to_writer cw");
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" writer w");
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit to writers who are in the grader's course.
            sqlCommand.append(" writer w, course_to_writer cw, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" writer w, writer_as_grader wag");
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" grader g");
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" grader g");
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_admin) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" admin a");
        }
        else if(table.equals(PackagingConstants.TABLE_admin) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            return null;//invalid command for graders
        }
        else if(table.equals(PackagingConstants.TABLE_admin) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_university) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" university u");
        }
        else if(table.equals(PackagingConstants.TABLE_university) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            return null;//invalid command for graders
        }
        else if(table.equals(PackagingConstants.TABLE_university) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_ctg) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_ctg) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            return null;//invalid command for graders
        }
        else if(table.equals(PackagingConstants.TABLE_ctg) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_ctw) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" course_to_writer cw");
        }
        else if(table.equals(PackagingConstants.TABLE_ctw) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            return null;//invalid command for graders
        }
        else if(table.equals(PackagingConstants.TABLE_ctw) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_checkout) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" checkout ch");
        }
        else if(table.equals(PackagingConstants.TABLE_checkout) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            return null;//invalid command for graders
        }
        else if(table.equals(PackagingConstants.TABLE_checkout) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" paper p");
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" paper p, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" paper p, writer_as_grader wag");
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" draft d");
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" draft d, paper p, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" draft d, paper p, writer_as_grader wag, draft_structure ds, assignment a");
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" comment co");
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" comment co, paper p, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" comment co, paper p, writer_as_grader wag, draft d");
        }
        else if(table.equals(PackagingConstants.TABLE_eq) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" emailQueue e");
        }
        else if(table.equals(PackagingConstants.TABLE_eq) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            return null;//invalid command for graders
        }
        else if(table.equals(PackagingConstants.TABLE_eq) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" draft_structure ds");
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" draft_structure ds, course_to_grader cg, assignment ag");
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" draft_structure ds, course_to_writer cw, assignment ag, writer_as_grader wag");
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" writer_as_grader wag");
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" writer_as_grader wag, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" writer_as_grader wag");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" assignment ag");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" assignment ag, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" assignment ag, course_to_writer cw, writer_as_grader wag");
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" staging s");
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" staging s, course_to_grader cg");
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            return null;//invalid command for writers
        }
        
        if(IDs!=null || limitID!=null){
            sqlCommand.append(" WHERE");
        }
        
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any courseID.
                int size=IDs.size();
                sqlCommand.append(" (c.courseID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" c.courseID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit by courses grader is in.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any courseID.
                int size=IDs.size();
                sqlCommand.append(" (c.courseID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" c.courseID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cg.graderID=? AND cg.courseID=c.courseID");
        }
        else if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit by courses writer is in.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any courseID.
                int size=IDs.size();
                sqlCommand.append(" (c.courseID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" c.courseID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cw.writerID=? AND cw.courseID=c.courseID");
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any writerID.
                int size=IDs.size();
                sqlCommand.append(" (w.writerID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" w.writerID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit to writers who have submitted papers in grader's course.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                //IDs can have more than one.  So return results that match any writerID.
                int size=IDs.size();
                sqlCommand.append(" (w.writerID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" w.writerID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cw.writerID=w.writerID AND cw.courseID=cg.courseID AND cg.graderID=?");
            
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //writers can lookup information on all writers, due to peer grading
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                //IDs can have more than one.  So return results that match any writerID.
                int size=IDs.size();
                sqlCommand.append(" (w.writerID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" w.writerID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" (w.writerID=? OR (wag.writerAsGraderID=? AND wag.writerAsWriterID=w.writerID)");
            sqlCommand.append(" OR (wag.writerAsWriterID=? AND wag.writerAsGraderID=w.writerID))");
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any graderID.
                int size=IDs.size();
                sqlCommand.append(" (g.graderID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" g.graderID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //graders can only lookup information about one writer--themselves.
            sqlCommand.append(" g.graderID=?");
        }
        else if(table.equals(PackagingConstants.TABLE_admin) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any adminID.
                int size=IDs.size();
                sqlCommand.append(" (a.adminID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" a.adminID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_university) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any universityID.
                int size=IDs.size();
                sqlCommand.append(" (u.universityID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" u.universityID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctg) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any courseToGraderID.
                int size=IDs.size();
                sqlCommand.append(" (cg.courseToGraderID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" cg.courseToGraderID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctw) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any courseToWriterID.
                int size=IDs.size();
                sqlCommand.append(" (cw.courseToWriterID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" cw.courseToWriterID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_checkout) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any checkoutID.
                int size=IDs.size();
                sqlCommand.append(" (ch.checkoutID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ch.checkoutID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any paperID.
                int size=IDs.size();
                sqlCommand.append(" (p.paperID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" p.paperID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit by courses grader is in.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any paperID.
                int size=IDs.size();
                sqlCommand.append(" (p.paperID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" p.paperID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cg.graderID=? AND cg.courseID=p.courseID");
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit by papers writer has written or is grading (must check for this when getting paper IDs).
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any paperID.
                int size=IDs.size();
                sqlCommand.append(" (p.paperID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" p.paperID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" (p.writerID=? OR wag.writerAsGraderID=?)");
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any draftID.
                int size=IDs.size();
                sqlCommand.append(" (d.draftID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" d.draftID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit by courses grader is in.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any draftID.
                int size=IDs.size();
                sqlCommand.append(" (d.draftID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" d.draftID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cg.graderID=? AND cg.courseID=p.courseID AND d.paperID=p.paperID");
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit by papers writer has written.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any draftID.
                int size=IDs.size();
                sqlCommand.append(" (d.draftID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" d.draftID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ((d.paperID=p.paperID AND p.writerID=?)");
//            sqlCommand.append(" OR (wag.writerAsGraderID=? AND wag.draftStructureID=d.draftStructureID))");
            sqlCommand.append(" OR (wag.writerAsGraderID=? AND wag.draftStructureID=ds.draftStructureID AND ds.assignID=p.assignID AND p.paperID=d.paperID))");			
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any commentID.
                int size=IDs.size();
                sqlCommand.append(" (co.commentID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" co.commentID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit by courses grader is in.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any commentID.
                int size=IDs.size();
                sqlCommand.append(" (co.commentID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" co.commentID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cg.graderID=? AND cg.courseID=p.courseID AND co.paperID=p.paperID");
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit by papers writer has written or is grading.
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }                
                //IDs can have more than one.  So return results that match any commentID.
                int size=IDs.size();
                sqlCommand.append(" (co.commentID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" co.commentID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ((co.paperID=p.paperID AND p.writerID=?) OR");
            sqlCommand.append(" (co.draftID=d.draftID AND d.draftStructureID=wag.draftStructureID AND wag.writerAsGraderID=?))");
        }
        else if(table.equals(PackagingConstants.TABLE_eq) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (e.queueID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" e.queueID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (ds.draftStructureID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ds.draftStructureID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                int size=IDs.size();
                sqlCommand.append(" (ds.draftStructureID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ds.draftStructureID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ds.assignID=ag.assignID AND ag.courseID=cw.courseID AND ");
            sqlCommand.append("(cw.writerID=? OR (wag.writerAsGraderID=? AND cw.writerID=wag.writerAsWriterID))");
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (ds.draftStructureID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ds.draftStructureID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ds.assignID=ag.assignID AND ag.courseID=cg.courseID AND cg.graderID=?");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (ag.assignID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ag.assignID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (ag.assignID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ag.assignID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ag.courseID=cw.courseID AND ");
            sqlCommand.append("(cw.writerID=? OR (wag.writerAsGraderID=? AND cw.writerID=wag.writerAsWriterID))");
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (ag.assignID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" ag.assignID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" ag.courseID=cg.courseID AND cg.graderID=?");
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any rowID.
                int size=IDs.size();
                sqlCommand.append(" (wag.rowID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" wag.rowID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                //IDs can have more than one.  So return results that match any rowID.
                int size=IDs.size();
                sqlCommand.append(" (wag.rowID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" wag.rowID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" wag.courseID=cg.courseID AND cg.graderID=?");
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any rowID.
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                int size=IDs.size();
                sqlCommand.append(" (wag.rowID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" wag.rowID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" (wag.writerAsGraderID=? OR wag.writerAsWriterID=?)");
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                int size=IDs.size();
                sqlCommand.append(" (s.stagingID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" s.stagingID=?");
                }
                sqlCommand.append(")");
            }
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                //IDs can have more than one.  So return results that match any queueID.
                if(showAnd){
                    sqlCommand.append(" AND");
                }else{
                    showAnd=true;
                }
                int size=IDs.size();
                sqlCommand.append(" (s.stagingID=?");	//the first one doesn't have an OR infront
                for(int i=1;i<size;i++){
                    sqlCommand.append(" OR");		//the next ones do.
                    sqlCommand.append(" s.stagingID=?");
                }
                sqlCommand.append(")");
            }
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" cg.courseID=s.courseID AND cg.graderID=?");
        }
        
        sqlCommand.append(";");
        
        //COMMAND ASSEMBLED
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetQuery: for parameters: ( IDs:"+IDs+" table:"+table+" limitBy:"+limitBy+" limitID:"+limitID+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each courseID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit by courses grader is in.
            if(IDs!=null){
                //get each courseID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_course) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit by courses writer is in.
            if(IDs!=null){
                //get each courseID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each writerID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit to writers who have submitted papers in grader's course.
            if(IDs!=null){
                //get each writerID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_writer) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //writers can only lookup information about one writer--themselves.
            if(IDs!=null){
                //get each writerID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each graderID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_grader) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //graders can only lookup information about one writer--themselves.
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_admin) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each adminID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_university) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each universityID as a integer
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setInt(currentIndex,((Integer)e.nextElement()).intValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctg) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each courseToGraderID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ctw) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each courseToWriterID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_checkout) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each checkoutID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each paperID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit to papers from writers who have submitted papers in grader's course.
            if(IDs!=null){
                //get each paperID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_paper) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit to writer's own papers, or writer's peer grading papers
            if(IDs!=null){
                //get each paperID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            if(IDs!=null){
                //get each draftID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit to drafts from writers who have submitted papers in grader's course.
            if(IDs!=null){
                //get each draftID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_draft) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit to drafts from writer's own papers.
            if(IDs!=null){
                //get each draftID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each commentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_grader)){
        //limit to comments from papers from writers who have submitted papers in grader's course.
            if(IDs!=null){
                //get each commentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_comment) && limitBy.equals(PackagingConstants.LIMIT_writer)){
        //limit to comments for papers from writer's own papers.
            if(IDs!=null){
                //get each commentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_eq) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each queueID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each draftStructureID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(IDs!=null){
                //get each draftStructureID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_ds) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                //get each draftStructureID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each assignmentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(IDs!=null){
                //get each assignmentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_assignment) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                //get each assignmentID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each rowID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                //get each rowID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_wag) && limitBy.equals(PackagingConstants.LIMIT_writer)){
            if(IDs!=null){
                //get each rowID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_admin)){
            if(IDs!=null){
                //get each stagingID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
        }
        else if(table.equals(PackagingConstants.TABLE_staging) && limitBy.equals(PackagingConstants.LIMIT_grader)){
            if(IDs!=null){
                //get each stagingID as a long
                Enumeration e=IDs.elements();
                while(e.hasMoreElements()){
                    ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                    currentIndex++;
                }
            }
            //finally add the limitID in there
            ps.setLong(currentIndex,limitID.longValue());
            currentIndex++;
        }

        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
