package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getCourseID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetStagingIDQuery implements GetIDQuery{
    Vector courseID=		null;
    Vector graderID=		null;
    Vector paperID=		null;
    String status=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetStagingIDQuery(Vector courseID,
                            Vector graderID,
                            Vector paperID,
                            String status){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(paperID!=null && !paperID.isEmpty()){
            this.paperID=paperID;
        }
        if(status!=null && !status.equals("")){
            this.status=status;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT s.stagingID FROM staging s WHERE (s.courseID=? OR 
      s.courseID=?) AND (s.graderID=? OR s.graderID=?) AND (s.paperID=? OR s.paperID=?) AND s.status=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT s.stagingID FROM staging s");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(courseID!=null || graderID!=null || paperID!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any courseID.
            int size=courseID.size();
            sqlCommand.append(" (s.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" s.courseID=?");
            }
            sqlCommand.append(")");
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (s.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" s.graderID=?");
            }
            sqlCommand.append(")");
        }
        if(paperID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any paperID.
            int size=paperID.size();
            sqlCommand.append(" (s.paperID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" s.paperID=?");
            }
            sqlCommand.append(")");
        }
        if(status!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }
            showAnd=true;
            sqlCommand.append(" s.status=?");
        }    
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetStagingIDQuery: for parameters: ( courseID:"+courseID+" graderID:"+graderID+
                " paperID:"+paperID+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(courseID!=null){
            //get each universityID, as an integer
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(graderID!=null){
            //get each graderID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(paperID!=null){
            //get each writerID as a long
            Enumeration e=paperID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(status!=null){
            ps.setString(currentIndex, status);
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
