package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getUniversityID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetUniversityIDQuery implements GetIDQuery{
    String name=		null;
    String contactInfo=		null;
    String currentSemester=	null;
    Integer currentYear=	null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetUniversityIDQuery(String name,
                                String contactInfo,
                                String currentSemester,
                                Integer currentYear){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        this.currentYear=currentYear;
        if(name!=null && !name.equals("")){
            this.name=name;
        }
        if(contactInfo!=null && !contactInfo.equals("")){
            this.contactInfo=contactInfo;
        }
        if(currentSemester!=null && !currentSemester.equals("")){
            this.currentSemester=currentSemester;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT u.universityID FROM university u WHERE u.name=? AND u.contactInfo=? AND u.currentSemester=? 
      AND u.currentYear=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT u.universityID FROM university u");
        boolean showAnd=false;//tells whether to put the AND in or not.
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(name!=null || contactInfo!=null || currentSemester!=null || currentYear!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(name!=null){
            sqlCommand.append(" u.name=?");
            showAnd=true;
        }
        if(contactInfo!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" u.contactInfo=?");
        }
        if(currentSemester!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" u.currentSemester=?");
        }
        if(currentYear!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" u.currentYear=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetUniversityIDQuery: for parameters: ( name:"+name+
                " contactInfo:"+contactInfo+" currentSemester:"+currentSemester + 
                " currentYear:"+currentYear+") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(name!=null){
            ps.setString(currentIndex,name);
            currentIndex++;
        }
        if(contactInfo!=null){
            ps.setString(currentIndex,contactInfo);
            currentIndex++;
        }
        if(currentSemester!=null){
            ps.setString(currentIndex,currentSemester);
            currentIndex++;
        }
        if(currentYear!=null){
            ps.setInt(currentIndex,currentYear.intValue());
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
