package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getWriterAsGraderID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetWriterAsGraderIDQuery implements GetIDQuery{
    Vector draftStructureID=	null;
    Vector writerAsWriterID=	null;
    Vector courseID=		null;
    Vector writerAsGraderID=	null;
    String completed=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetWriterAsGraderIDQuery(Vector draftStructureID,
                                    Vector writerAsGraderID,
                                    Vector writerAsWriterID,
                                    Vector courseID,
                                    String completed){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        if(draftStructureID!=null && !draftStructureID.isEmpty()){
            this.draftStructureID=draftStructureID;
        }
        if(writerAsGraderID!=null && !writerAsGraderID.equals("")){
            this.writerAsGraderID=writerAsGraderID;
        }
        if(writerAsWriterID!=null && !writerAsWriterID.equals("")){
            this.writerAsWriterID=writerAsWriterID;
        }
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(completed!=null && !completed.equals("")){
            this.completed=completed;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT wag.writerAsGraderID FROM writer_as_grader wag WHERE (wag.draftStructureID=? OR 
      wag.draftStructureID=?) AND (wag.graderID=? OR wag.graderID=?) AND (wag.writerID=? OR wag.writerID=?) AND (wag.courseID=? OR wag.courseID=?)
      AND wag.completed=?;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT wag.rowID FROM writer_as_grader wag");
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(draftStructureID!=null || writerAsWriterID!=null || courseID!=null || writerAsGraderID!=null){
            sqlCommand.append(" WHERE");
        }
        
        boolean showAnd=false;
        
        if(draftStructureID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any universityID.
            int size=draftStructureID.size();
            sqlCommand.append(" (wag.draftStructureID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" wag.draftStructureID=?");
            }
            sqlCommand.append(")");
        }
        if(writerAsGraderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=writerAsGraderID.size();
            sqlCommand.append(" (wag.writerAsGraderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" wag.writerAsGraderID=?");
            }
            sqlCommand.append(")");
        }
        if(writerAsWriterID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any writerID.
            int size=writerAsWriterID.size();
            sqlCommand.append(" (wag.writerAsWriterID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" wag.writerAsWriterID=?");
            }
            sqlCommand.append(")");
        }
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=courseID.size();
            sqlCommand.append(" (wag.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" wag.courseID=?");
            }
            sqlCommand.append(")");
        }
        if(completed!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" wag.completed=?");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetWriterAsGraderIDQuery: for parameters: ( draftStructureID:"+draftStructureID+" writerAsWriterID:"+writerAsWriterID+
                " courseID:"+courseID+" writerAsGraderID:"+writerAsGraderID+" completed:"+completed+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(draftStructureID!=null){
            //get each draftStructureID, as an integer
            Enumeration e=draftStructureID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(writerAsGraderID!=null){
            //get each writerAsGraderID as a long
            Enumeration e=writerAsGraderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(writerAsWriterID!=null){
            //get each writerAsWriterID as a long
            Enumeration e=writerAsWriterID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(courseID!=null){
            //get each courseID as a long
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(completed!=null){
            ps.setString(currentIndex,completed);
            currentIndex++;
        }
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
