package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a getWriterID query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class GetWriterIDQuery implements GetIDQuery{
    Vector universityID=	null;
    String studentIdentifier=	null;
    String firstName=		null;
    String lastName=		null;
    String userName=		null;
    String userNameType=	null;
    String email=		null;
    Timestamp creationDate=	null;
    Vector courseID=		null;
    Vector graderID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public GetWriterIDQuery(Vector universityID,
                            String studentIdentifier,
                            String firstName,
                            String lastName,
                            String userName,
                            String userNameType,
                            String email,
                            Timestamp creationDate,
                            Vector courseID,
                            Vector graderID){
        //in the non-String, non-Vector objects, null values are not checked for.
        //null values can just be passed on.
        //in the String objects, you must check for nulls so as to 
        //not have empty strings if they are null (thus causing a NullPointerException).
        //in the Vector objects, you must check for nulls so as to 
        //not have Vectors with no elements if they are null (thus causing a NullPointerException).
        
        this.creationDate=creationDate;
        if(universityID!=null && !universityID.isEmpty()){
            this.universityID=universityID;
        }
        if(courseID!=null && !courseID.isEmpty()){
            this.courseID=courseID;
        }
        if(graderID!=null && !graderID.isEmpty()){
            this.graderID=graderID;
        }
        if(studentIdentifier!=null && !studentIdentifier.equals("")){
            this.studentIdentifier=studentIdentifier;
        }
        if(firstName!=null && !firstName.equals("")){
            this.firstName=firstName;
        }
        if(lastName!=null && !lastName.equals("")){
            this.lastName=lastName;
        }
        if(userName!=null && !userName.equals("")){
            this.userName=userName;
        }
        if(userNameType!=null && !userNameType.equals("")){
            this.userNameType=userNameType;
        }
        if(email!=null && !email.equals("")){
            this.email=email;
        }
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public ResultSet execute() throws SQLException{
    /*SELECT DISTINCT w.writerID FROM writer w, course_to_writer cw, course_to_grader cg WHERE 
      LEFT(w.creationDate,10)=? AND w.firstName=? AND w.lastName=? AND w.userName=? AND 
      (w.universityID=? OR w.universityID=?) AND w.studentIdentifier=? AND w.email=? AND 
      (cw.courseID=? OR cw.courseID=?) AND (cg.graderID=? OR cg.graderID=?) AND w.writerID=cw.writerID;
    */
        StringBuffer sqlCommand=new StringBuffer("SELECT DISTINCT w.writerID FROM writer w");
        boolean showAnd=false;//tells whether to put the AND in or not.
        boolean addWCWwriterID=true;//tells whether or not to add AND w.writerID=cw.writerID (which is needed by
                                   //both graderID and courseID
        
        //finish the sqlCommand
        if(courseID!=null || graderID!=null){
            sqlCommand.append(", course_to_writer cw");
        }
        if(graderID!=null){
            sqlCommand.append(", course_to_grader cg");
        }
        
        //must check to see if no parameters were given...if that is the case, then there should be no "WHERE"
        if(creationDate!=null || firstName!=null || lastName!=null || userName!=null || userNameType!=null 
           || email!=null || universityID!=null || studentIdentifier!=null || courseID!=null || graderID!=null){
            sqlCommand.append(" WHERE");
        }
        
        if(creationDate!=null){
            sqlCommand.append(" LEFT(w.creationDate,10)=?");//LEFT is used to have day scope.
            showAnd=true;
        }
        if(firstName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.firstName=?");
        }
        if(lastName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.lastName=?");
        }
        if(userName!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.userName=?");
        }
        if(userNameType!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.userNameType=?");
        }
        if(email!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.email=?");
        }
        if(universityID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any universityID.
            int size=universityID.size();
            sqlCommand.append(" (w.universityID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" w.universityID=?");
            }
            sqlCommand.append(")");
        }
        if(studentIdentifier!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            sqlCommand.append(" w.studentIdentifier=?");
        }
        if(courseID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any courseID.
            int size=courseID.size();
            sqlCommand.append(" (cw.courseID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cw.courseID=?");
            }
            sqlCommand.append(")");
            sqlCommand.append(" AND w.writerID=cw.writerID");
            addWCWwriterID=false;//now it has been added.
        }
        if(graderID!=null){
            if(showAnd){
                sqlCommand.append(" AND");
            }else{
                showAnd=true;
            }
            //IDs can have more than one.  So return results that match any graderID.
            int size=graderID.size();
            sqlCommand.append(" (cg.graderID=?");	//the first one doesn't have an OR infront
            for(int i=1;i<size;i++){
                sqlCommand.append(" OR");		//the next ones do.
                sqlCommand.append(" cg.graderID=?");
            }
            sqlCommand.append(")");
            if(addWCWwriterID){
                sqlCommand.append(" AND w.writerID=cw.writerID");
                addWCWwriterID=false;//now it has been added.
            }
            sqlCommand.append(" AND cg.courseID=cw.courseID");
        }
        sqlCommand.append(";");
        
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug(
                "GetWriterIDQuery: for parameters:"+"("+
                " creationDate:"+creationDate+ " firstName:"+firstName +
                " lastName:"+lastName +" userName:"+userName +" userNameType:"+userNameType+
                " email:"+email +" universityID:"+universityID +
                " studentIdentifier"+studentIdentifier +" courseID:"+courseID+" graderID:"+graderID+
                ") the following statement was assembled: "+sqlCommand
            );
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        int currentIndex=1;
        //fill in the ?s
        if(creationDate!=null){
            ps.setTimestamp(currentIndex,creationDate);
            currentIndex++;
        }
        if(firstName!=null){
            ps.setString(currentIndex,firstName);
            currentIndex++;
        }
        if(lastName!=null){
            ps.setString(currentIndex,lastName);
            currentIndex++;
        }
        if(userName!=null){
            ps.setString(currentIndex,userName);
            currentIndex++;
        }
        if(userNameType!=null){
            ps.setString(currentIndex,userNameType);
            currentIndex++;
        }
        if(email!=null){
            ps.setString(currentIndex,email);
            currentIndex++;
        }
        if(universityID!=null){
            //get each universityID, as an integer
            Enumeration e=universityID.elements();
            while(e.hasMoreElements()){
                ps.setInt(currentIndex,((Integer)e.nextElement()).intValue());
                currentIndex++;
            }
        }
        if(studentIdentifier!=null){
            ps.setString(currentIndex,studentIdentifier);
            currentIndex++;
        }
        if(courseID!=null){
            //get each courseID as a long
            Enumeration e=courseID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        if(graderID!=null){
            //get each courseID as a long
            Enumeration e=graderID.elements();
            while(e.hasMoreElements()){
                ps.setLong(currentIndex,((Long)e.nextElement()).longValue());
                currentIndex++;
            }
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
