package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.blaise.version2.documents.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
import edu.gwu.blaise.version2.util.*;
/**
 * Performs a newComment query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class NewCommentQuery implements NewQuery{
    Long paperID=		null;
    Long draftID=		null;
    Long graderID=		null;
    byte[] remarks=		null;
    Boolean isPublic=		null;
    Blaise_Comment bc=		null;
    final String statement="INSERT INTO comment VALUES(null,?,?,?,?,?,now());";
    
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    final String modify="UPDATE comment SET remarks=? WHERE commentID=?;";
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewCommentQuery(Blaise_Comment bc,
                            Long paperID,
                            Long draftID,
                            Long graderID,
                            byte[] remarks,
                            Boolean isPublic){
        this.bc=bc;
        this.paperID=paperID;
        this.draftID=draftID;
        this.graderID=graderID;
        this.remarks=remarks;
        this.isPublic=isPublic;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
           //prepare the statement.
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setLong(1,paperID.longValue());
            ps.setLong(2,draftID.longValue());
            ps.setLong(3,graderID.longValue());
            ps.setBytes(4,remarks);
            ps.setInt(5,(isPublic.booleanValue())?1:0);//1 if true, 0 if false
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ps.clearParameters();
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
            
            //update the remarks to actually contain the commentID
            remarks=fixComment();
            ps=c.prepareStatement(modify);
            ps.clearParameters();
            ps.setBytes(1,remarks);
            ps.setLong(2,lastID.longValue());
            ps.executeUpdate();

        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }

    public Long getLastInsertID(){
        return lastID;
    }
    
    /**
     * This method will take a Blaise_Comment and update it to contain the commentID, 
     * and then convert into string form.
     */
    byte[] fixComment(){
        bc.setCommentID(lastID.toString());
        try{
            return BlaiseFile.commentToByteArray(bc);
        }catch(java.io.IOException e){
            if(BaseServlet.debugOn()){
                BaseServlet.writeDebug("IOException encountered in NewCommentQuery.fixComment: "+e);
            }
            return null;
        }
    }
}
