package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newCourse query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/18/2004
 */

public class NewCourseQuery implements NewQuery{
    String contactInfo=		null;
    Integer universityID=	null;
    String dept=		null;
    String number=		null;
    String section=		null;
    String semester=		null;
    Integer year=		null;
    String isCurrent=		null;
    final String statement="INSERT INTO course VALUES(null,?,?,?,?,?,?,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewCourseQuery(String contactInfo, Integer universityID, String dept, String number, String section, String semester, Integer year,String isCurrent){
        this.universityID=universityID;
        this.contactInfo=contactInfo;
        this.dept=dept;
        this.number=number;
        this.section=section;
        this.semester=semester;
        this.year=year;
        this.isCurrent=isCurrent;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewCourseQuery: for parameters: ( contactInfo: "+contactInfo+" isCurrent:"+isCurrent+
                    " universityID:"+universityID+" dept: "+dept+" number: "+number+" section: "+section+
                    "semester: "+semester+" year: "+year+") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setString(1,contactInfo);
            ps.setInt(2,universityID.intValue());
            ps.setString(3,dept);
            ps.setString(4,number);
            ps.setString(5,section);
            ps.setString(6,semester);
            ps.setInt(7,year.intValue());
            ps.setString(8,isCurrent);
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
