package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newCourseToGrader query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/18/2004
 */
public class NewCourseToGraderQuery implements NewQuery{
    Long courseID=		null;
    Long graderID=		null;
    final String statement="INSERT INTO course_to_grader VALUES(null,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewCourseToGraderQuery(Long courseID, Long graderID){
        this.graderID=graderID;
        this.courseID=courseID;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewCourseToGraderQuery: for parameters: ( courseID:"+courseID+
                    " graderID:"+graderID+") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setLong(1,courseID.longValue());
            ps.setLong(2,graderID.longValue());
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
