package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newDraft query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class NewDraftQuery implements NewQuery{
    Long paperID=		null;
    Integer draftNumber=	null;
    String title=		null;
    byte[] body=		null;
    Long draftStructureID=	null;
    String draftType=		null;
    
    final String statement="INSERT INTO draft VALUES(null,?,?,?,?,now(),?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewDraftQuery(Long paperID,
                        Integer draftNumber,
                        String title,
                        byte[] body,
                        Long draftStructureID,
                        String draftType){
        this.paperID=paperID;
        this.draftNumber=draftNumber;
        this.title=title;
        this.body=body;
        this.draftStructureID=draftStructureID;
        this.draftType=draftType;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setLong(1,paperID.intValue());
            ps.setInt(2,draftNumber.intValue());
            ps.setString(3,title);
            ps.setBytes(4,body);
            if(draftStructureID!=null){
                ps.setLong(5,draftStructureID.longValue());
            }else{
                ps.setNull(5,Types.BIGINT);
            }
            ps.setString(6,draftType);
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.    
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
