package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newDraftStructure query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/24/2004
 */
public class NewDraftStructureQuery implements NewQuery{
    Integer draftNum=		null;
    Long assignID=		null;
    String gradingType=		null;
    Timestamp noChangeDate=		null;    
    final String statement="INSERT INTO draft_structure VALUES(null,?,?,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
   
    public NewDraftStructureQuery(Integer draftNum,
                                  Long assignID,
                                  String gradingType,
                                  Timestamp noChangeDate){
        this.draftNum=draftNum;
        this.assignID=assignID;
        this.gradingType=gradingType;
        this.noChangeDate=noChangeDate;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewDraftStructureQuery: for parameters: ( draftNum: "+draftNum+
                    " assignID: "+assignID+" gradingType: "+gradingType+" noChangeDate: "+noChangeDate+") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setInt(1,draftNum.intValue());
            ps.setLong(2,assignID.longValue());
            ps.setString(3,gradingType);
            ps.setTimestamp(4,noChangeDate);
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
