package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newGrader query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/18/2004
 */
public class NewGraderQuery implements NewQuery{
    Integer universityID=		null;
    String firstName=			null;
    String lastName=			null;
    String userName=			null;
    String password=			null;
    String email=			null;
    String status=			null;
    Timestamp similarityCheckedDate=	null;
    final String statement="INSERT INTO grader VALUES(null,now(),?,?,?,?,?,?,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=			null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewGraderQuery(Integer universityID, String firstName, String lastName, String userName, String password, String email, String status, Timestamp similarityCheckedDate){
        this.universityID=universityID;
        this.firstName=firstName;
        this.lastName=lastName;
        this.userName=userName;
        this.password=password;
        this.email=email;
        this.status=status;        
        this.similarityCheckedDate=similarityCheckedDate;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewGraderQuery: for parameters: ( universityID:"+universityID+" firstName: "+
                    firstName+" lastName: "+lastName+" userName: "+
                    userName+" password: "+password+" email: "+email+" status: "+
                    status+") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setInt(1,universityID.intValue());
            ps.setString(2,firstName);
            ps.setString(3,lastName);
            ps.setString(4,userName);
            ps.setString(5,password);
            ps.setString(6,email);
            ps.setString(7,status);
            ps.setTimestamp(8,similarityCheckedDate);
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
