package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newPaper query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class NewPaperQuery implements NewQuery{
    Integer version=		null;
    Long writerID=		null;
    Integer universityID=	null;
    Long courseID=		null;
    String archivalPractice=	null;
    Long assignID=		null;
    String hasBeenDownloaded=	null;
    
    final String statement="INSERT INTO paper VALUES(null,?,?,?,?,?,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewPaperQuery(Integer version,
                        Long writerID,
                        Integer universityID,
                        Long courseID,
                        String archivalPractice,
                        Long assignID,
                        String hasBeenDownloaded){
        this.version=version;
        this.writerID=writerID;
        this.universityID=universityID;
        this.courseID=courseID;
        this.archivalPractice=archivalPractice;
        this.assignID=assignID;
        this.hasBeenDownloaded=hasBeenDownloaded;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewPaperQuery: for parameters: ( version: "+version+" writerID: "+writerID+
                    " universityID: "+universityID+" courseID: "+courseID+" archivalPractice: "+
                    archivalPractice+" hasBeenDownloaded: "+hasBeenDownloaded+" assignID:"+assignID+
                    ") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setInt(1,version.intValue());
            ps.setLong(2,writerID.longValue());
            ps.setInt(3,universityID.intValue());
            ps.setLong(4,courseID.longValue());
            ps.setString(5,archivalPractice);
            ps.setLong(6,assignID.longValue());
            ps.setString(7,hasBeenDownloaded);
                    
            //everything is filled in.  Do this shiznit...
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
