package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newStaging query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/24/2004
 */
public class NewStagingQuery implements NewQuery{
    Long courseID=		null;
    Long graderID=		null;
    Long paperID=		null;
    String status=		null;
    final String statement="INSERT INTO staging VALUES(null,?,?,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewStagingQuery(Long courseID, Long graderID, Long paperID, String status){
        this.graderID=graderID;
        this.courseID=courseID;
        this.paperID=paperID;
        this.status=status;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewStagingQuery: for parameters: ( courseID:"+courseID+" graderID:"+graderID+
                    " paperID: "+paperID+" status: "+status+") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
            ps.setLong(1,courseID.longValue());
            ps.setLong(2,graderID.longValue());
            ps.setLong(3,paperID.longValue());
            ps.setString(4,status);
                    
            //everything is filled in.  Do this shiznit...    
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
