package edu.gwu.blaise.version2.backend.queries;

import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import java.sql.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * Performs a newUniversity query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Jared Kiraly
 * @version 2 5/18/2004
 */

public class NewUniversityQuery implements NewQuery{
    String name=		null;
    String contactInfo=		null;
    String currentSemester=	null;
    Integer currentYear=	null;
    final String statement="INSERT INTO university VALUES(null,?,?,?,?);";
    final String lastIDStatement="SELECT LAST_INSERT_ID();";
    Long lastID=		null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection();
    
    public NewUniversityQuery(String name, String contactInfo, String currentSemester, Integer currentYear){
        this.name=name;
        this.contactInfo=contactInfo;
        this.currentSemester=currentSemester;
        this.currentYear=currentYear;

        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
    public int execute() throws SQLException{
        int i=0;
        try{
            //prepare the statement.
            if(BaseServlet.debugOn()){//from edu.gwu.servlets
                BaseServlet.writeDebug(
                    "NewUniversityQuery: for parameters: ( name:"+name+" contactInfo: "+contactInfo+" currentSemester: "+
                    currentSemester+" currentYear: "+currentYear+") the following statement was assembled: "+statement
                );
            }
            PreparedStatement ps=c.prepareStatement(statement);
            ps.clearParameters();
            //fill in the ?s
    
            ps.setString(1,name);   
            ps.setString(2,contactInfo);   
            ps.setString(3,currentSemester);   
            ps.setInt(4,currentYear.intValue());   
                    
            //everything is filled in.  Do this shiznit...    
            i=ps.executeUpdate();
            
            //get the last ID.
            ps=c.prepareStatement(lastIDStatement);
            ResultSet rs=ps.executeQuery();
            if(rs.next()){
                lastID=new Long(rs.getLong(1));
            }
        //return the connection to the pool
        }finally{
            StaticContext.freeSQLConnection(c);
        }
        return i;
    }
    
    public Long getLastInsertID(){
        return lastID;
    }
}
