package edu.gwu.blaise.version2.backend.queries;
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.blaise.version2.backend.packagers.PackagingConstants;
import java.sql.*;
/**
 * Performs a validateXXXLogin query.  Please note, all error checking needs to be in the
 * caller as this class does no error checking on its parameters.
 * @author Michael Gaiman
 * @version 2 1/20/2004
 */
public class ValidateLoginQuery implements GetIDQuery {
    private String userName=null;
    private String password=null;
    private Integer universityID=null;
    private String limitBy=null;
    
    /** Our pooled connection. */
    Connection c=StaticContext.getSQLConnection(); 

    public ValidateLoginQuery(String userName,String password,Integer universityID,String limitBy){
        this.userName=userName;
        this.password=password;
        this.universityID=universityID;
        this.limitBy=limitBy;
        
        //the connection wasn't available. (too busy, or something).
        if(c==null){
            throw new NoConnectionException();
        }
    }
    
     public ResultSet execute() throws SQLException{
     /* SELECT x.xxxID FROM xxx x WHERE x.userName=? AND x.password=? AND x.universityID=?*/
     
        //check for null/invalid parameters
        if(userName==null){
            return null;
        }
        if(password==null){
            return null;
        }
        if(universityID==null && !limitBy.equals(PackagingConstants.LIMIT_admin)){//admin is system-wide
            return null;
        }
        
        if(limitBy==null){
            return null;
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_writer)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
        }
        else{
            return null;
        }
        
        //assemble the command
        StringBuffer sqlCommand=new StringBuffer("SELECT");
        if(limitBy.equals(PackagingConstants.LIMIT_writer)){
            sqlCommand.append(" w.writerID FROM writer w WHERE");
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" g.graderID FROM grader g WHERE");
        }
        else if(limitBy.equals(PackagingConstants.LIMIT_admin)){
            sqlCommand.append(" a.adminID FROM admin a WHERE");
        }
        
        sqlCommand.append(" userName=? AND password=?");
        
        //admins are system wide.
        if(limitBy.equals(PackagingConstants.LIMIT_writer) || 
           limitBy.equals(PackagingConstants.LIMIT_grader)){
            sqlCommand.append(" AND universityID=?");
        }
        
        sqlCommand.append(";");
        
        //COMMAND ASSEMBLED      
        if(BaseServlet.debugOn()){//from edu.gwu.servlets
            BaseServlet.writeDebug("ValidateLoginQuery: for parameters: ( username:"+userName+" password:"+password+" limitBy:"+
                                   limitBy+" universityID:"+universityID+") the following statement was assembled: "+sqlCommand);
        }
        //prepare the statement.
        PreparedStatement ps=c.prepareStatement(sqlCommand.toString());
        ps.clearParameters();
        
        ps.setString(1,userName);
        ps.setString(2,password);
        if(limitBy.equals(PackagingConstants.LIMIT_writer) || 
           limitBy.equals(PackagingConstants.LIMIT_grader)){
            ps.setInt(3,universityID.intValue());
        }
        
        //everything is filled in.  Do this shiznit...
        return ps.executeQuery();
    }
    
    public void free(){
        if(c!=null){
            StaticContext.freeSQLConnection(c);
            c=null;
        }
    }
}
