package edu.gwu.blaise.version2.backend.util;
/**
 * This is the backend module that takes error messages from TransmissionHandler
 * and stores them all in a file for debugging 
 *
 * @author Jared Kiraly
 * @version 2 June 22, 2004
 */
import edu.gwu.servlets.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.backend.packagers.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.blaise.version2.documents.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

public class ErrorFileServlet extends BaseServlet{
    public ErrorFileServlet(){}
    
    public void doGet (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException{
        doPost(req,res);
    }
    
    public void doPost (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException{
        try{
            res.setContentType ("text/plain");
            PrintWriter out = res.getWriter ();
        
            String header=req.getParameter("header");
            String body=req.getParameter("body");        

            boolean b=printToFile(header,body);

            if(!b){
                out.print(TransmissionConstants.FAILURE);
                out.flush();
                out.close();
                return;
            }
                                                
            out.print(TransmissionConstants.SUCCESS);
            out.flush();
            out.close();
        }catch(Throwable t){
            res.setContentType ("text/plain");
            PrintWriter out = res.getWriter ();
            out.print(TransmissionConstants.FAILURE);
            out.flush();
            out.close();
            StringBuffer stringbuf=new StringBuffer();
            stringbuf.append(t);
            StackTraceElement[] st= t.getStackTrace();
            for(int i=0;i<st.length;i++){
              stringbuf.append("\n");
              stringbuf.append(st[i].toString()); 
            }
            if(BaseServlet.debugOn()){
                BaseServlet.writeDebug(stringbuf.toString());
            }
            else{
                BaseServlet.writeLog(stringbuf.toString());
            }
        }
    }    
    
    private static synchronized boolean printToFile(String header, String body){
        File f=new File(TransmissionConstants.debugFile);
        FileOutputStream fout=null;
        try{
            fout=new FileOutputStream(f, true);
        }catch(FileNotFoundException e){
            return false;
        }
        PrintWriter pw=new PrintWriter(fout);
        pw.println("    "+header);
        pw.println("");
        pw.println("    "+body);
        pw.println(TransmissionConstants.DebugEntrySeparator);
        pw.flush();
        pw.close();     
        return true;   
    }
    
    /////////////////////////////////////////////////////////////
    //
    // edu.gwu.servlets required methods
    //
    /////////////////////////////////////////////////////////////
  
    public void shutdown ()
    {
        // Nothing to be done.
        return;
    }
    
    public void reset ()
    {
        // Nothing to be done.
        return;
    }
    
    public void save ()
    {
        // Nothing to be done.
        return;
    }
    
    public void test (PrintWriter out)
    {
        out.println ("<p> ErrorFileServlet.test: successful </p>");
    }
}
