package edu.gwu.blaise.version2.backend.util;
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

import java.util.*;
import java.io.*;

import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.servlets.*;

/**
 * The Mail Utility is used to email students their receipt upon turning in a document and their graded paper after it has been graded
 *
 * @author Sean T. Hanlon
 * @author Michael Gaiman
 */
public class MailUtility{
    private String host = "smtp.gwu.edu";
    private String from = "blaise@gwu.edu";
    File tempDir=null;

    /** Constructor */
    public MailUtility(File tempDir){
        this.tempDir=tempDir;
    }

    /** Sends receipt for Blaise_Paper parameter */
    public void sendReceipt(Blaise_Paper paper) throws MessagingException{
    
        // Get system properties
        Properties props = System.getProperties();
    
        // Setup mail server
        props.put("mail.smtp.host", host);
    
        // Get session
        Session session = Session.getDefaultInstance(props, null);
    
        // Define message
        MimeMessage message = new MimeMessage(session);
        // Set the from address
        message.setFrom(new InternetAddress(from));
    
        // Set the to address
        message.addRecipient(Message.RecipientType.TO, new InternetAddress(paper.getEmail()));
    
        // Set the subject
        message.setSubject("Paper Submission Confirmation");
    
        // Set the content
        message.setContent(formatMessage(paper), "text/html");
    
        // Send message
        Transport.send(message);
    }

    //Formats message to be sent to writer
    private String formatMessage(Blaise_Paper paper){

            StringBuffer buffer = new StringBuffer();
            buffer.append("<html><body>");
            buffer.append("This is your receipt to prove that your paper submission was received.");
            buffer.append("<table border=\"0\">");
            buffer.append("<tr><td>Writer:</td><td>"+paper.getFirst()+" "+paper.getLast()+"</td></tr>");
            buffer.append("<tr><td>Department:</td><td>"+paper.getDept()+"</td></tr>");
            buffer.append("<tr><td>Course Number:</td><td>"+paper.getNumber()+"</td></tr>");
            buffer.append("<tr><td>Section:</td><td>"+paper.getSection()+"</td></tr>");
            buffer.append("</table>");
            buffer.append("</body></html>");

            return buffer.toString();

    }

    /** Sends receipt for Blaise_Paper parameter */
    public void sendGraded(Blaise_Paper paper) throws MessagingException{
        if(BaseServlet.debugOn()){
            BaseServlet.writeDebug("MailUtility.sendGraded called.");
        }
    
        // Get system properties
        Properties props = System.getProperties();
    
        // Setup mail server
        props.put("mail.smtp.host", host);
    
        // Get session
        Session session = Session.getDefaultInstance(props, null);
    
        // Define message
        MimeMessage message = new MimeMessage(session);
        // Set the from address
        message.setFrom(new InternetAddress(from));
    
        // Set the to address
        message.addRecipient(Message.RecipientType.TO, new InternetAddress(paper.getEmail()));
    
        // Set the subject
        message.setSubject("Paper Grading Notification");
    
        // Set the content
        message.setContent(formatGraded(paper), "text/html");
    
        // Send message
        Transport.send(message);
    }

    //Formats message to be sent to writer
    private String formatGraded(Blaise_Paper paper){

            StringBuffer buffer = new StringBuffer();
            buffer.append("<html><body>");
            buffer.append("This is your notification that your paper has been graded.");
            buffer.append("<table border=\"0\">");
            buffer.append("<tr><td>Writer:</td><td>"+paper.getFirst()+" "+paper.getLast()+"</td></tr>");
            buffer.append("<tr><td>Department:</td><td>"+paper.getDept()+"</td></tr>");
            buffer.append("<tr><td>Course Number:</td><td>"+paper.getNumber()+"</td></tr>");
            buffer.append("<tr><td>Section:</td><td>"+paper.getSection()+"</td></tr>");
            buffer.append("</table>");
            buffer.append("</body></html>");

            return buffer.toString();

    }

    //returns html version of body, minus html, header, and body tags
    private String fixBody(String body){

        //strip out <html><head><body></body></head></html>
        StringBuffer buffer = new StringBuffer();
        int start = 0;
        int end = 0;

        for(int i=0;i<body.length();i++){
                char at = body.charAt(i);
                if((at == '<') && (i < body.length() - 7)){
                        if(body.charAt(i+1) == 'b'){
                                if(body.charAt(i+2) == 'o'){
                                        if(body.charAt(i+3) == 'd'){
                                                if(body.charAt(i+4) == 'y'){
                                                        if(body.charAt(i+5) == '>'){
                                                                start = i + 6;
                                                        }
                                                }
                                        }
                                }
                        }
                        else if(body.charAt(i+1) == '/'){
                                if(body.charAt(i+2) == 'b'){
                                        if(body.charAt(i+3) == 'o'){
                                                if(body.charAt(i+4) == 'd'){
                                                        if(body.charAt(i+5) == 'y'){
                                                                if(body.charAt(i+6) == '>'){
                                                                        end = i - 1;
                                                                }
                                                        }
                                                }
                                        }
                                }
                        }
                }
        }

        body = body.substring(start,end);
        return body;
    }
}
