package edu.gwu.blaise.version2.client;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.plaf.*;
import javax.swing.undo.*;
import javax.swing.UIManager;
import java.util.regex.*;

import java.beans.*;
import java.util.Vector;

import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.util.find.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.documents.*;

/**
 * BlaiseEditorPane is the lower portion of the Split Pane in Blaise Editor.
 * Additionally, it serves as the main "switchboard" for editing functionality.
 *
 * @author 	Sean T. Hanlon
 * @author      Michael Gaiman
 * @author 	Rizwan Satar
 *
 */
public class BlaiseEditorPane extends JPanel{

    /*private static final byte OPEN = 0;
    private static final byte SAVE = 1;
    private static final byte TRANSMIT = 2;*/
    /**@see saveIfNeeded()*/
    private static final int NO_SAVE_NEEDED=999;
    
    BlaiseEditorPane thisPane;
    
    private ProgressView progView;
    
    /**Controls the enabling and disabling of menu and toolbar items based on context*/
    private BlaiseEditorPane.UIStateDelegate myUIStateDelegate;
    
    //Whether or not to use encryption on documents
    boolean encrypt = false;
    
    //Paper encryption instance
    PaperCrypto crypt;
    
    //The text component of the paper.
    BlaiseJTextPane mainView;
    
    //Email address of writer
    String email = "";
    
    //our instance of the ClientFileHandler.
    private ClientFileHandler cfHandler=new ClientFileHandler();
        
    private Thread saveThread;
    private Thread openThread;
    private Thread finalizeDraftThread;
    private boolean fileChooserClosed=false;//set to true by save action when the file chooser is closed.
    
    //JFrame representing the container holding this JPanel
    BlaiseSplitPane parent;
    
    JScrollPane scroll;
    File saveFile = null;
    
    /** Used for default numbering with new documents.*/
    private int documentNumber=1;
    
    //the instance of findFrame in use
    private FindFrame findFrame=null;
    
    //used do handle multiple undos/redos.
    private UndoManager undoManager;
    
    /**
     *Because draftFinalization must get user input (the title) it can always be cancelled.
     *As such, any processes that require the working draft to be finalized before proceeding
     *need to know if the finalize has been cancelled.
     */
    private boolean finalizeCancelled;
    
    /**
     * some actions can perform their action quietly (with little or no user interaction).  If quiet
     * action is needed set the command name to this (a reference).
     */
    private static final String QUIETLY="do-action-quietly";
        
    //references to the default actions that BlaiseJTextPane is using.
    private BoldAction boldAction;
    private ItalicAction italicAction;
    private UnderlineAction underlineAction;
    private Action copyAction;//default is good
    private Action cutAction;//default is good
    private Action pasteAction;//default is good
    private LeftAlignAction leftAlignAction;
    private CenterAlignAction centerAlignAction;
    private RightAlignAction rightAlignAction;
    private Action selectAllAction;//default is good.
    //local instances of the actions
    private NewPaperAction newPaperAction;
    private OpenAction openAction;
    private CloseAction closeAction;
    private SaveAction saveAction;
    private SaveAsAction saveAsAction;
    private MergeAction mergeAction;
    private SubmitAction submitAction;
    private ExitAction exitAction;
    private UndoAction undoAction;
    private RedoAction redoAction;
    private FindAction findAction;
    private FindAgainAction findAgainAction;
    private ToggleScribblesAction toggleScribblesAction;
    private ToggleNotesAction toggleNotesAction;
    private ExpandAllNotesAction expandAllNotesAction;
    private CollapseAllNotesAction collapseAllNotesAction;
    private NewDraftAction newDraftAction;
    private DuplicateDraftAction duplicateDraftAction;
    private AddExternalDraftAction addExternalDraftAction;
    private RemoveExternalDraftAction removeExternalDraftAction;
    private FinalizeDraftAction finalizeDraftAction;
    private ToggleRealTimeSpellCheckAction toggleRealTimeSpellCheckAction;
    private ThesaurusAction thesaurusAction;
    private WordCountAction wordCountAction;
    private HelpAction helpAction;
    private AboutAction aboutAction;
    
    //this menubar and toolbar
    JMenuBar menubar;
    JToolBar toolbar;
    
    //the menu items that might need to be disabled/enabled
    private JMenuItem undoMI;
    private JMenuItem redoMI;
    private JMenuItem cutMI;
    private JMenuItem copyMI;
    private JMenuItem pasteMI;
    private JMenuItem findAgainMI;
    private JMenu showDraftM;
    private JMenu showGradedM;
    private JCheckBoxMenuItem toggleScribblesCBMI;//show/hide scribbles
    private JCheckBoxMenuItem toggleNotesCBMI;//show/hide notes
    private JCheckBoxMenuItem boldCBMI;
    private JCheckBoxMenuItem italicCBMI;
    private JCheckBoxMenuItem underlineCBMI;
    private JCheckBoxMenuItem leftAlignCBMI;
    private JCheckBoxMenuItem centerAlignCBMI;
    private JCheckBoxMenuItem rightAlignCBMI;
    private JMenuItem expandNotesMI;
    private JMenuItem collapseNotesMI;
    private JMenuItem duplicateDraftMI;
    private JMenuItem addExternalDraftMI;
    private JMenuItem removeExternalDraftMI;
    private JMenuItem submitPaperMI;
    private JMenuItem finalizeDraftMI;
    private JCheckBoxMenuItem toggleRealTimeSpellCheckCBMI;
    
    //the toolbar buttons for actions that need to be set selected and unselected
    private JButton cutB;
    private JButton copyB;
    private JButton pasteB;
    private JToggleButton boldTB;
    private JToggleButton italicTB;
    private JToggleButton underlineTB;
    private JToggleButton leftAlignTB;
    private JToggleButton rightAlignTB;
    private JToggleButton centerAlignTB;
    
    /**
     * Creates an instance of BlaiseEditorPane.
     */
    public BlaiseEditorPane(BlaiseSplitPane parent){
        
        this.parent = parent;
        thisPane = this;
        this.setLayout(new BorderLayout());
        
        //Prompt for email to ensure proper saving and loading
        while(email.equals("") && encrypt){
            email = (String)JOptionPane.showInputDialog(this,"Please enter your email address", "Email Address", JOptionPane.QUESTION_MESSAGE, null, null, "");
            
            //If email is not valid, exit application
            if(email == null){
                System.exit(0);
            }
            
            //Confirm validity - Email must have an '@' and a '.' not first or last
            int ema = email.indexOf('@');
            int dot = email.indexOf('.');
            if((ema == -1) || (ema == 0) || (ema == email.length()-1)){
                JOptionPane.showMessageDialog(this, "An email address must contain an '@' symbol in a valid location. ", "FAILURE", JOptionPane.ERROR_MESSAGE);
                email = "";
            }
            else if((dot == -1) || (dot == 0) || (dot == email.length()-1)){
                JOptionPane.showMessageDialog(this, "An email address must contain a '.' symbol in a valid location. ", "FAILURE", JOptionPane.ERROR_MESSAGE);
                email = "";
            }
        }

        //Initialize crypto functionality
        crypt = new PaperCrypto();

        //Initialize content pane
        mainView = BlaiseJTextPane.createContent("");
        
        undoManager=new UndoManager();
        
        //find the Actions
        Hashtable actionLookup=new Hashtable();
        Action[] actionList=mainView.getActions();
        for(int i=0;i<actionList.length;++i){
            actionLookup.put(actionList[i].getValue(Action.NAME),actionList[i]);
            //System.out.println(actionList[i].getValue(Action.NAME));
        }
        selectAllAction=(Action)actionLookup.get(DefaultEditorKit.selectAllAction);
        cutAction=(Action)actionLookup.get(DefaultEditorKit.cutAction);
        copyAction=(Action)actionLookup.get(DefaultEditorKit.copyAction);
        pasteAction=(Action)actionLookup.get(DefaultEditorKit.pasteAction);
        
        //instantiate the various actions.
        boldAction=new BoldAction(this);
        italicAction=new ItalicAction(this);
        underlineAction=new UnderlineAction(this);
        leftAlignAction=new LeftAlignAction(this);
        centerAlignAction=new CenterAlignAction(this);
        rightAlignAction=new RightAlignAction(this);
        //local instances of the actions
        newPaperAction=new NewPaperAction(this);
        openAction=new OpenAction(this);
        closeAction=new CloseAction(this);
        saveAction=new SaveAction(this);
        saveAsAction=new SaveAsAction(this);
        mergeAction=new MergeAction(this);
        submitAction=new SubmitAction(this);
        exitAction=new ExitAction(this);
        undoAction=new UndoAction(this);
        redoAction=new RedoAction(this);
        findAction=new FindAction(this);
        findAgainAction=new FindAgainAction(this);
        toggleScribblesAction=new ToggleScribblesAction(this);
        toggleNotesAction=new ToggleNotesAction(this);
        expandAllNotesAction=new ExpandAllNotesAction(this);
        collapseAllNotesAction=new CollapseAllNotesAction(this);
        newDraftAction=new NewDraftAction(this);
        duplicateDraftAction=new DuplicateDraftAction(this);
        addExternalDraftAction=new AddExternalDraftAction(this);
        removeExternalDraftAction=new RemoveExternalDraftAction(this);
        finalizeDraftAction=new FinalizeDraftAction(this);
        toggleRealTimeSpellCheckAction=new ToggleRealTimeSpellCheckAction(this,null);
        thesaurusAction=new ThesaurusAction(this);
        wordCountAction=new WordCountAction(this);
        helpAction=new HelpAction(this);
        aboutAction=new AboutAction(this);
        
        //install custom Actions for bold, italic, underline...
        Keymap keymap=mainView.getKeymap();
        int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();//platform independent
        keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_B,aKey),boldAction);
        keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_I,aKey),italicAction);
        keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_U,aKey),underlineAction);
        
        createToolBar();
        createMenuBar();
        
        myUIStateDelegate=new UIStateDelegate(this);
        parent.draft_pane.addDraftViewListener(myUIStateDelegate);
        
        if(parent.draft_pane!=null){
            myUIStateDelegate.populateShowDraftMenu();
            myUIStateDelegate.populateShowGradedMenu();
        }
        
        //sync spelling and the cbmi
        toggleRealTimeSpellCheckAction.toggleChecking(true);
        toggleRealTimeSpellCheckCBMI.setSelected(true);
        
        //Add text area to a scroll pane to accomodate long documents and add scroll to frame
        if(Constants.isOSX()){//osx always displays the vertical bar.
            scroll = new JScrollPane(mainView, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        }
        else{
            scroll = new JScrollPane(mainView, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        }
        scroll.setViewportView(mainView);
        this.add(scroll, BorderLayout.CENTER);
        
        //Create menu and toolbar
        JPanel top = new JPanel(new BorderLayout());
        //top.add(createMenuBar(), BorderLayout.NORTH);
        top.add(toolbar, BorderLayout.SOUTH);
        this.add(top, BorderLayout.NORTH);
        
        myUIStateDelegate.setTitle(null);
        //Set preferred size of text area to be the size of this frame
        this.repaint();
        
        parent.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        parent.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
                quitAction();
            }
        });
    }
    
	//Paints the screen. Handles resizing of text area in terms of frame size.
    public void paint(Graphics g){
        super.paint(g);
        mainView.setSize(this.getSize());
        mainView.setPreferredSize(this.getSize());
        mainView.setMinimumSize(this.getSize());
        mainView.setMaximumSize(this.getSize());
        mainView.requestFocus();
    }

    /**
     * Creates and returns tool bar
     *
     * @return A JToolBar with all functionality listeners initialized
     */
    private void createToolBar() {
        
        //Get path to icons
        String iconPath = Constants.getIconPath();
        
        //Initialize horizontal toolbar and set to non-floatable
        toolbar = new JToolBar(JToolBar.HORIZONTAL);
        toolbar.setFloatable(false);
                
        //"New" document button: create, name, and register
        JButton button=new JButton(newPaperAction);
        button.setText("");
        button.setIcon(new ImageIcon(iconPath+"new.png"));
        button.setFocusable(false);
        toolbar.add(button);
        
        //"Open" document button: create, name, and register
        button=new JButton(openAction);
        button.setText("");
        button.setIcon(new ImageIcon(iconPath+"open.png"));
        button.setFocusable(false);
        toolbar.add(button);
        
        //"Save" document button: create, name, and register
        button=new JButton(saveAction);
        button.setText("");
        button.setIcon(new ImageIcon(iconPath+"save.png"));
        button.setFocusable(false);
        toolbar.add(button);
        
        //Separate file operations and clipboard text formatting operations
        toolbar.addSeparator();
        
        //"Cut" document button: create, name, and register
        cutB=new JButton(cutAction);
        cutB.setText("");
        cutB.setIcon(new ImageIcon(iconPath+"cut.png"));
        cutB.setFocusable(false);
        toolbar.add(cutB);
        
        //"Copy" document button: create, name, and register
        copyB=new JButton(copyAction);
        copyB.setText("");
        copyB.setIcon(new ImageIcon(iconPath+"copy.png"));
        copyB.setFocusable(false);
        toolbar.add(copyB);
        
        //"Paste" document button: create, name, and register
        pasteB=new JButton(pasteAction);
        pasteB.setText("");
        pasteB.setIcon(new ImageIcon(iconPath+"paste.png"));
        pasteB.setFocusable(false);
        toolbar.add(pasteB);
        
        //Separate clipboard text formatting from text formatting
        toolbar.addSeparator();
        
        //"Bold" document button: create, name, and register
        boldTB=new JToggleButton(boldAction);
        boldTB.setText("");
        boldTB.setIcon(new ImageIcon(iconPath+"bold.png"));
        boldTB.setFocusable(false);
        toolbar.add(boldTB);
        
        //"Italics" document button: create, name, and register
        italicTB=new JToggleButton(italicAction);
        italicTB.setText("");
        italicTB.setIcon(new ImageIcon(iconPath+"italic.png"));
        italicTB.setFocusable(false);
        toolbar.add(italicTB);
        
        //"Underline" document button: create, name, and register
        underlineTB=new JToggleButton(underlineAction);
        underlineTB.setText("");
        underlineTB.setIcon(new ImageIcon(iconPath+"underline.png"));
        underlineTB.setFocusable(false);
        toolbar.add(underlineTB);
                
        //Separate text formatting from paragraph formatting
        toolbar.addSeparator();
        
        //"Left Align" document button: create, name, and register
        leftAlignTB=new JToggleButton(leftAlignAction);
        leftAlignTB.setText("");
        leftAlignTB.setIcon(new ImageIcon(iconPath+"leftAlign.png"));
        leftAlignTB.setFocusable(false);
        
        //"Center Align" document button: create, name, and register
        centerAlignTB=new JToggleButton(centerAlignAction);
        centerAlignTB.setText("");
        centerAlignTB.setIcon(new ImageIcon(iconPath+"centerAlign.png"));
        centerAlignTB.setFocusable(false);
        
        //"Right Align" document button: create, name, and register
        rightAlignTB=new JToggleButton(rightAlignAction);
        rightAlignTB.setText("");
        rightAlignTB.setIcon(new ImageIcon(iconPath+"rightAlign.png"));
        rightAlignTB.setFocusable(false);
        
        //add a button group for the alignment buttons--making them exclusive.
        ButtonGroup alignGroup=new ButtonGroup();
        alignGroup.add(leftAlignTB);
        alignGroup.add(centerAlignTB);
        alignGroup.add(rightAlignTB);
        toolbar.add(leftAlignTB);
        toolbar.add(centerAlignTB);
        toolbar.add(rightAlignTB);
        
        //by default...
        leftAlignTB.setSelected(true);
    }

    /**
     * Creates the Blaise Editor menu bar structure.
     */
    private void createMenuBar(){
        menubar=new JMenuBar();
        
        JMenu fileMenu  =new JMenu("File");
        JMenu editMenu  =new JMenu("Edit");
        JMenu viewMenu  =new JMenu("View");
        JMenu formatMenu=new JMenu("Format");
        JMenu draftsMenu=new JMenu("Drafts");
        JMenu toolsMenu =new JMenu("Tools");
        JMenu helpMenu  =new JMenu("Help");
        
        menubar.add(fileMenu);
        menubar.add(editMenu);
        menubar.add(viewMenu);
        menubar.add(formatMenu);
        menubar.add(draftsMenu);
        menubar.add(toolsMenu);
        menubar.add(helpMenu);
        
        int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();//platform independent
        //file menu items
        JMenuItem item=new JMenuItem(newPaperAction);
        item.setText("New Paper");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,aKey));
        fileMenu.add(item);
        
        item=new JMenuItem(openAction);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,aKey));
        item.setText("Open...");
        fileMenu.add(item);
        
        item=new JMenuItem(closeAction);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,aKey));
        item.setText("Close");
        fileMenu.add(item);
        
        fileMenu.addSeparator();
        
        item=new JMenuItem(saveAction);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,aKey));
        item.setText("Save");
        fileMenu.add(item);
        
        item=new JMenuItem(saveAsAction);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,aKey|InputEvent.SHIFT_MASK));
        item.setText("Save As...");
        fileMenu.add(item);
        
        fileMenu.addSeparator();
        
        item=new JMenuItem(mergeAction);
        item.setText("Combine Papers...");
        fileMenu.add(item);
        
        submitPaperMI=new JMenuItem(submitAction);
        submitPaperMI.setText("Submit Paper...");
        fileMenu.add(submitPaperMI);
        
        if(!Constants.isOSX()){//on windows add exit to file menu
            fileMenu.addSeparator();
            
            item=new JMenuItem(exitAction);
            item.setText("Exit");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,InputEvent.ALT_MASK));//alt-F4
            fileMenu.add(item);
        }
        
        //edit menu stuff
        undoMI=new JMenuItem(undoAction);
        undoMI.setText("Undo");
        undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,aKey));
        editMenu.add(undoMI);
        
        redoMI=new JMenuItem(redoAction);
        redoMI.setText("Redo");
        redoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,aKey|InputEvent.SHIFT_MASK));
        editMenu.add(redoMI);
        
        editMenu.addSeparator();
        
        cutMI=new JMenuItem(cutAction);
        cutMI.setText("Cut");
        cutMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,aKey));
        editMenu.add(cutMI);
        
        copyMI=new JMenuItem(copyAction);
        copyMI.setText("Copy");
        copyMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,aKey));
        editMenu.add(copyMI);
        
        pasteMI=new JMenuItem(pasteAction);
        pasteMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,aKey));
        pasteMI.setText("Paste");
        editMenu.add(pasteMI);
        
        item=new JMenuItem(selectAllAction);
        item.setText("Select All");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,aKey));
        editMenu.add(item);
        
        editMenu.addSeparator();
        
        item=new JMenuItem(findAction);
        item.setText("Find...");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,aKey));
        editMenu.add(item);
        
        findAgainMI=new JMenuItem(findAgainAction);
        findAgainMI.setText("Find Again");
        findAgainMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G,aKey));//REVISIT: G or shift-F?
        editMenu.add(findAgainMI);
        
        //view menu
        //A submenu with all available drafts
        showDraftM=new JMenu("Show Draft");
        viewMenu.add(showDraftM);
        
        //A submenu with all graded copies of the currently selected draft.
        showGradedM=new JMenu("Show Graded Draft");
        viewMenu.add(showGradedM);
        
        //viewMenu.addSeparator();
        
        toggleScribblesCBMI=new JCheckBoxMenuItem(toggleScribblesAction);
        toggleScribblesCBMI.setText("Display Scribbles");
        //toggleScribblesCBMI.setEnabled(false);//most be enabled when a graded draft is showing. (initial disable handled by UIStateDelegate)
        //viewMenu.add(toggleScribblesCBMI);
        
        toggleNotesCBMI=new JCheckBoxMenuItem(toggleNotesAction);
        toggleNotesCBMI.setText("Display Notes (NI)");
        //toggleNotesCBMI.setEnabled(false);//most be enabled when a graded draft is showing. (initial disable handled by UIStateDelegate)
        //viewMenu.add(toggleNotesCBMI);
        
        //viewMenu.addSeparator();
    
        expandNotesMI=new JMenuItem(expandAllNotesAction);
        expandNotesMI.setText("Expand All Notes (NI)");
        expandNotesMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_CLOSE_BRACKET,aKey));
        //expandNotesMI.setEnabled(false);//most be enabled when a graded draft is showing. (initial disable handled by UIStateDelegate)
        //viewMenu.add(expandNotesMI);
        
        collapseNotesMI=new JMenuItem(collapseAllNotesAction);
        collapseNotesMI.setText("Collapse All Notes (NI)");
        collapseNotesMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_OPEN_BRACKET,aKey));
        //collapseNotesMI.setEnabled(false);//most be enabled when a graded draft is showing. (initial disable handled by UIStateDelegate)
        //viewMenu.add(collapseNotesMI);
        
        //Format menu
        //styles submenu
        JMenu menu=new JMenu("Styles");
        
        boldCBMI=new JCheckBoxMenuItem(boldAction);
        boldCBMI.setText("Bold");
        boldCBMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,aKey));
        menu.add(boldCBMI);
        
        italicCBMI=new JCheckBoxMenuItem(italicAction);
        italicCBMI.setText("Italic");
        italicCBMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,aKey));
        menu.add(italicCBMI);
        
        underlineCBMI=new JCheckBoxMenuItem(underlineAction);
        underlineCBMI.setText("Underline");
        underlineCBMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U,aKey));
        menu.add(underlineCBMI);
        
        formatMenu.add(menu);
        
        //Paragraph submenu
        menu=new JMenu("Paragraph");
        
        leftAlignCBMI=new JCheckBoxMenuItem(leftAlignAction);
        leftAlignCBMI.setText("Left Align");
        menu.add(leftAlignCBMI);
        
        centerAlignCBMI=new JCheckBoxMenuItem(centerAlignAction);
        centerAlignCBMI.setText("Center Align");
        menu.add(centerAlignCBMI);
        
        rightAlignCBMI=new JCheckBoxMenuItem(rightAlignAction);
        rightAlignCBMI.setText("Right Align");
        menu.add(rightAlignCBMI);
        
        //only one can be checked at a time.
        ButtonGroup alignGroup=new ButtonGroup();
        alignGroup.add(leftAlignCBMI);
        alignGroup.add(centerAlignCBMI);
        alignGroup.add(rightAlignCBMI);
        
        formatMenu.add(menu);
        
        //Drafts menu
        item=new JMenuItem(newDraftAction);
        item.setText("New Blank Draft");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,aKey|InputEvent.SHIFT_MASK));
        draftsMenu.add(item);
        
        duplicateDraftMI=new JMenuItem(duplicateDraftAction);
        duplicateDraftMI.setText("Duplicate Existing Draft...");
        draftsMenu.add(duplicateDraftMI);
        
        draftsMenu.addSeparator();
        
        addExternalDraftMI=new JMenuItem(addExternalDraftAction);
        addExternalDraftMI.setText("Attach External Document as Draft...");
        draftsMenu.add(addExternalDraftMI);
        
        removeExternalDraftMI=new JMenuItem(removeExternalDraftAction);
        removeExternalDraftMI.setText("Detach External Document");
        draftsMenu.add(removeExternalDraftMI);
        
        draftsMenu.addSeparator();
        
        finalizeDraftMI=new JMenuItem(finalizeDraftAction);
        finalizeDraftMI.setText("Finalize Draft...");
        draftsMenu.add(finalizeDraftMI);
        
        //Tools menu
        toggleRealTimeSpellCheckCBMI=new JCheckBoxMenuItem(toggleRealTimeSpellCheckAction);
        toggleRealTimeSpellCheckCBMI.setText("Check Spelling As You Type");
        toolsMenu.add(toggleRealTimeSpellCheckCBMI);
        
        item=new JMenuItem(thesaurusAction);
        item.setText("Thesaurus... (NI)");
        toolsMenu.add(item);
        
        toolsMenu.addSeparator();
        
        item=new JMenuItem(wordCountAction);
        item.setText("Word Count");
        toolsMenu.add(item);
        
        //help menu
        item=new JMenuItem(helpAction);
        item.setText("Blaise Editor Help");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_SLASH,aKey|InputEvent.SHIFT_MASK));//makes the "?"
        helpMenu.add(item);
        
        if(!Constants.isOSX()){
            ///show the about menu on windows. (OSX places it in the Application menu)
            item=new JMenuItem(aboutAction);
            item.setText("About");
            helpMenu.add(item);
        }
    }
    
    /**
     * Returns this BlaiseEditorPane's menu structure.
     */
    public JMenuBar getMenuBar(){
        return menubar;
    }
    
    /**
     * Starts a progress view with the supplied properties. (replacing an existing one if needed)
     */
    synchronized void startProgView(String title, int timeSeconds, boolean isIndeterminate){
        //starting progress view is thread safe.
        if(progView==null){
            progView=new ProgressView(parent,title,timeSeconds,isIndeterminate);
        }
        else{
            progView.done();
            progView=new ProgressView(parent,title,timeSeconds,isIndeterminate);
        }
    }
    
    /**
     * Starts an indeterminate progress view, with the supplied title. (replacing an existing one if needed)
     */
    void startProgView(String title){
        startProgView(title,0,true);
    }
    
    /**
     * Stops the progress view (if it exists)
     */
    synchronized void stopProgView(){
        if(progView!=null){
            //done is thread safe.
            progView.done();
            progView=null;
        }
    }
    
    /**
     * If the document has changed it will present a dialog asking them to save the document.
     * @return an int representing their choice.  See JOptionPane for the values.  Or, if no save was needed,
     * it will return BlaiseEditorPane.NO_SAVE_NEEDED.
     * @param confirm if a change is needed and confirm is true, it will prompt the user to make sure they want it.  If confirm is false, it will just save.
     */
    private int saveIfNeeded(boolean confirm){
        if(mainView.isModified()){
            //Prompt to save any changes
            if(confirm){
                int answer = JOptionPane.showConfirmDialog(parent,"If you don't save, your changes will be lost." , "Save Changes to Document?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
                
                if(answer == JOptionPane.YES_OPTION){
                    saveAction.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,saveAction.NAME));
                    if(fileChooserClosed){
                        answer=JOptionPane.CANCEL_OPTION;
                    }
                }
                return answer;
            }
            else{
                saveAction.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,saveAction.NAME));
                if(fileChooserClosed){
                    return JOptionPane.CANCEL_OPTION;
                }
                else{
                    return JOptionPane.YES_OPTION;
                }
            }
        }
        else{
            return BlaiseEditorPane.NO_SAVE_NEEDED;
        }
    }
    
    /**
     * Not super thread safe.  The saving will be done in another thread, but this thread
     * will return immediately.
     */
    private int saveIfNeeded(){
        return saveIfNeeded(true);
    }
    
    public void quitAction(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                exitAction.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,exitAction.NAME));
            }
        });
    }
    
    /**
     * Opens the document from the supplied filename.
     * Please note: no real checking is made to make sure this filename is a valid
     * blaise paper.
     */
    public void openAction(final String filename){
        if(filename==null){
            throw new IllegalArgumentException("Invalid filename");
        }
        
        openAction.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,filename));
    }
    
    /**
     * Displays the about box.  (Method used by mac version)
     */
    public void aboutAction(){
        aboutAction.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,aboutAction.NAME));
    }
    
    /**
     * Returns the first Blaise_Paper from the file or null if the file doesn't have a Blaise_Paper.
     * @param open true if this file should "open" it, false if you just want the Blaise_Paper object.
     */
    public Blaise_Paper getBPFromFile(File file,boolean open) throws IOException{
        /* //old version 1 style.
        //Parse document XML into Blaise_Paper object
        Blaise_XML xml = new Blaise_XML();
        Vector paperVec = xml.parse(file.getAbsolutePath());
        //If no paper found, reject writer from opening document
        if(paperVec.size() == 0){
            JOptionPane.showMessageDialog(this, "The file does not seem to be a valid Blaise Paper.", "Error Opening Paper", JOptionPane.ERROR_MESSAGE);
            return null;
        }
        else{
            //Extract paper
            return (Blaise_Paper)paperVec.elementAt(0);
        }*/
        Blaise_Paper bp=new Blaise_Paper();
        BlaiseFile bf;
        if(open){
            bf=new BlaiseFile(bp,file,new ClientFileHandler());
        }
        else{
            bf=new BlaiseFile(bp,file,new BackendFileHandler());
        }
        bf.openPaper();
        return bp;
    }
    
    public Blaise_Paper getBPFromFile(File file) throws IOException{
        return getBPFromFile(file,true);
    }
    
    /**
     * Method for obtaining a File for input/output using a JFileChooser dialog.
     * Enforces blaise only file types.
     *
     * @param mainView The JTextPane representing the contents of the document
     * @param startDir Directory where file chooser should begin
     * @param dialogType Type of file being chosen. [Open or Save]
     * @return File chosen by the writer
     */
    public static File getFileFromChooser(Component owner, String startDir, int dialogType) {
        return getFileFromChooser(owner,startDir,dialogType,true);
    }
    
	static File chooserFile;
	
    /**
     * Method for obtaining a File for input/output using a JFileChooser dialog
     *
     * @param mainView The JTextPane representing the contents of the document
     * @param startDir Directory where file chooser should begin
     * @param dialogType Type of file being chosen. [Open or Save]
     * @param blaiseLimit boolean on whether or not to limit the allowed files to be blaise files.
     * @return File chosen by the writer
     */
    public static File getFileFromChooser(final Component owner, final String startDir, final int dialogType, final boolean blaiseLimit) {
		//always do this from swing thread
		if(!SwingUtilities.isEventDispatchThread()){
			chooserFile=null;
			try{
				SwingUtilities.invokeAndWait(new Runnable(){
					public void run(){
						chooserFile=getFileFromChooser(owner,startDir,dialogType,blaiseLimit);
					}
				});
			}catch(Exception e){
			}
			return chooserFile;
		}
        //Initialize new chooser
        JFileChooser jfileDialog = new JFileChooser(startDir);
        jfileDialog.setDialogType(dialogType);
        int optionSelected = JFileChooser.CANCEL_OPTION;

        //Based on type, determine allowed file filters
        if(blaiseLimit){
            if(dialogType == JFileChooser.OPEN_DIALOG) {
                jfileDialog.setMultiSelectionEnabled(false);
                jfileDialog.addChoosableFileFilter(new ClientFileFilter("kjm","A Saved Blaise Paper (*.kjm)"));
                jfileDialog.addChoosableFileFilter(new ClientFileFilter("bls","A Saved Blaise Paper (*.bls)"));
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
            else if(dialogType == JFileChooser.SAVE_DIALOG) {
                jfileDialog.setMultiSelectionEnabled(false);
                jfileDialog.addChoosableFileFilter(new ClientFileFilter("bls","A Saved Blaise Paper (*.bls)"));
                optionSelected = jfileDialog.showSaveDialog(owner);
            }
            else {
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
        }
        else{
            if(dialogType == JFileChooser.OPEN_DIALOG) {
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
            else if(dialogType == JFileChooser.SAVE_DIALOG) {
                optionSelected = jfileDialog.showSaveDialog(owner);
            }
            else {
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
        }

        //If approved operation is selected, return file
        if(optionSelected == JFileChooser.APPROVE_OPTION){
            File file=jfileDialog.getSelectedFile();
            if(dialogType == JFileChooser.SAVE_DIALOG && blaiseLimit){
                String name=file.getName();
                if(!name.toLowerCase().endsWith(".bls")){
                    return new File(file.getAbsolutePath()+".bls");
                }
            }
            return file;
        }
        return null;
    }
    
    /**
     * Deletes the paper directory that the current paper is using.
     */
    protected void deleteExistingDir(){
        try{
            cfHandler.closePaper();
        }catch(IOException e){
            System.out.println("While trying to delete paper: "+e);
        }
    }
    
    public class NewPaperAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="new-paper";
        public NewPaperAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Start a new Blaise Paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        /**
         * Peforms a New Paper action. (First checking to see if the current document needs
         * to be saved.
         */
        public void actionPerformed(ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    int save=bep.saveIfNeeded();
                    if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                        return;
                    }
                    else if(save==JOptionPane.YES_OPTION){
                        try{
                            bep.saveThread.join();
                        }catch(InterruptedException e){
                        }
                    }
                    bep.deleteExistingDir();//take care to close the paper.
                    bep.parent.document=new Blaise_Paper();
                    BlaiseFile bf=new BlaiseFile(bep.parent.document,new File(Constants.getTemplate()),bep.cfHandler);
                    try{
                        bf.openPaper();
                    }catch(IOException e){
                        System.out.println(e);
                    }
                    bep.documentNumber+=1;
                    bep.myUIStateDelegate.setTitle(null);//default for platform with an incremented document number
                    bep.mainView.setText("");
                    bep.saveFile=null;
                    bep.parent.updateDraftsWindow();
                    bep.myUIStateDelegate.resetButtonsAndMenus();
                    bep.myUIStateDelegate.setModified(false);
                }
            };
            thread.start();
        }
    }
    
    public class OpenAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="open-paper";
        public OpenAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Open an existing Blaise Paper for editing");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
		private int save=0;
		private boolean saveRan=false;
        /**
         * Handles the opening of a file.  If the filename was received through other means
         * the actionCommand will be different from OpenAction.NAME
         */
        public void actionPerformed(final ActionEvent ae){
			SwingUtilities.invokeLater(new Runnable(){
				public void run(){
					save=bep.saveIfNeeded();//that should be ran from swing thread
					saveRan=true;
				}
			});
            bep.openThread=new Thread(){
                public void run(){
                    while(!saveRan){
						try{
							Thread.sleep(10);
						}catch(InterruptedException e){
						}
					}
					saveRan=false;
                    if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                        return;
                    }
                    else if(save==JOptionPane.YES_OPTION){
                        try{
                            bep.saveThread.join();
                        }catch(InterruptedException e){
                        }
                    }
                    bep.deleteExistingDir();//take care to close the paper.
                    
                    String command=ae.getActionCommand();
                    File openFile;
                    if(command.equals(NAME)){
                        openFile = bep.getFileFromChooser(bep.parent, Constants.getUserDirectory(), JFileChooser.OPEN_DIALOG);
                    }
                    else{
                        //Open file from the string.
                        openFile=new File(command);
                    }
                    if(openFile==null || (!openFile.exists())){//for whatever reason we don't have a filename... (they probably cancelled).
                        return;
                    }
                    
                    try{
                        bep.startProgView("Opening Blaise Paper...");
                        Blaise_Paper p=bep.getBPFromFile(openFile);
                        if(p==null){
                            return;
                        }
                        bep.parent.document=p;
                        //Set the text of the JTextPane with the contents of the document just opened
                        //bep.mainView.setText(bep.parent.document.getWorkingBody());
                        
                        final Blaise_Draft workingDraft=bep.parent.document.getWorkingDraft();
						bep.saveFile = openFile;
						SwingUtilities.invokeLater(new Runnable(){
							public void run(){
								if(workingDraft.hasExternalBody()){
									bep.myUIStateDelegate.setExternalBody(true);
								}
								else{
									bep.mainView.setText(workingDraft.getBody());
									//check to see if this working body is really just white space.
									Document doc=bep.mainView.getDocument();
									try{
										if(Pattern.matches("(\\p{Space})+",doc.getText(0,doc.getLength()))){
											bep.mainView.setText("");
										}
									}catch(BadLocationException e){}
								}
								
								bep.parent.updateDraftsWindow();
							}
						});
                        //uistate stuff is thread safe...
                        bep.myUIStateDelegate.setTitle(bep.saveFile.getAbsolutePath());
                        bep.myUIStateDelegate.resetButtonsAndMenus();
						
                    }
                    catch(IOException e){
                        bep.stopProgView();
                        JOptionPane.showMessageDialog(bep.parent,"An Error Occurred that prevented the Blaise Document from opening.","Cannot Open Paper",JOptionPane.ERROR_MESSAGE);
                    }
                    finally{
                        bep.stopProgView();
                    }
                }//end run
            };
            bep.openThread.start();
        }
        
    }
    
    public class CloseAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="close-paper";
        public CloseAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Close the current Blaise Paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //As currently implemented, close is the same as new.  If we switch to a multidocument model,
            //close will actually close the window...
            bep.newPaperAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED,NewPaperAction.NAME));
        }
    }
    
    public class SaveAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="save-paper";
        public SaveAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Saves changes made to the current Blaise Paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //Initialize saveFile, a temp vector and a temp Document
            Vector tempVec = new Vector();
            
            //if nameOfFile is null, prompt for new name
            if(bep.saveFile == null || bep.saveFile.getName().equals("")){
                bep.saveFile = bep.getFileFromChooser(bep.parent, Constants.getUserDirectory(), JFileChooser.SAVE_DIALOG);
            }
            
            if(saveFile == null){//cancel or something.
                bep.fileChooserClosed=true;
                return;
            }
            else{
                bep.fileChooserClosed=false;
                doSave();
            }
        }
        
        private void doSave(){
        
             bep.saveThread=new Thread(){ 
                public void run(){
                    try{
                        bep.startProgView("Saving Blaise Paper...");
                        
                        //Confirm name ends in .bls
                        if(!(saveFile.getName()).endsWith(".bls")){
                            saveFile = new File(saveFile.getAbsolutePath()+".bls");
                        }
                        
                        //Fill working body
                        bep.parent.document.setWorkingBody(bep.mainView.getText());
                        
                        if(bep.parent.document.getVersion().equals("")){
                            bep.parent.document.setVersion(Constants.getVersion());
                        }
                        
                        /*Blaise_XML xml = new Blaise_XML();
                        xml.printPaperToFile(bep.saveFile.getAbsolutePath(), bep.parent.document);*/
                        BlaiseFile bf=new BlaiseFile(bep.parent.document,bep.saveFile,new ClientFileHandler());
                        bf.savePaper();
                        
                        bf.openPaper();//so the directory structure is updated.
                        
                        bep.myUIStateDelegate.setModified(false);
    
                        //bep.mainView.requestFocus();

                        bep.myUIStateDelegate.setTitle(saveFile.getAbsolutePath());
                        //bep.parent.updateDraftsWindow();
                    }
                    catch(IOException e){
                        bep.stopProgView();
                        JOptionPane.showMessageDialog(bep.parent,"An Error Occurred that prevented the Blaise Document from saving.","Cannot Save Paper",JOptionPane.ERROR_MESSAGE);
                        e.printStackTrace();
                    }
                    finally{
                        bep.stopProgView();
                    }
                }
            };
            bep.saveThread.start();
        }
    }
    
    public class SaveAsAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="save-as-paper";
        public SaveAsAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Saves changes made to the current Blaise Paper to a new file");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            bep.saveFile=null;
            bep.saveAction.actionPerformed(new ActionEvent(ae.getSource(),ActionEvent.ACTION_PERFORMED,saveAction.NAME));
        }
    }
    
    public class MergeAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="merge-paper";
        public MergeAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Combine drafts from different Blaise Papers into one Paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            int res=JOptionPane.showConfirmDialog(bep.parent,"<html><p>Combining papers allows drafts from two versions of the same paper to be combined into one updated paper.<br>Two file choosers will pop up, please select the two versions of the paper.</p></html>","Combine Papers",JOptionPane.OK_CANCEL_OPTION,JOptionPane.INFORMATION_MESSAGE);
            if(res!=JOptionPane.OK_OPTION){
                return;
            }
            Thread mergeThread=new Thread(){
                public void run(){
                    int save=bep.saveIfNeeded();
                    if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                        return;
                    }
                    else if(save==JOptionPane.YES_OPTION){
                        try{
                            bep.saveThread.join();
                        }catch(InterruptedException e){
                        }
                    }
                    //no option can just continue...
                    
                    //Prompt for files to merge
                    File file1 = bep.getFileFromChooser(bep.parent, Constants.getUserDirectory(), JFileChooser.OPEN_DIALOG);
                    //If the file exists and is the proper format
                    if(file1 == null){
                        return;
                    }
                    File file2 = bep.getFileFromChooser(bep.parent, Constants.getUserDirectory(), JFileChooser.OPEN_DIALOG);
                    //If the file exists and is the proper format
                    if(file2 == null){
                        return;
                    }
                    Blaise_Paper p1=null,p2=null;
                    try{
                        p1=bep.getBPFromFile(file1,false);
                        p2=bep.getBPFromFile(file2,false);
                    }catch(IOException e){
                        bep.stopProgView();
                        JOptionPane.showMessageDialog(bep.parent,"An Error Occurred that prevented the Blaise Document from opening.","Cannot Open Paper",JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                    if(p1==null || p2==null){
                        return;
                    }
                    BlaisePaperMergeDialog merge = new BlaisePaperMergeDialog(bep,p1,p2,file1,file2);
                    File newFile=merge.getFile();
                    if(newFile!=null){
                        bep.openAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED,newFile.getAbsolutePath()));
                    }
                }
            };
            mergeThread.start();
        }
    }
    
    public class SubmitAction extends AbstractAction implements SubmissionHandler{
        private BlaiseEditorPane bep;
        public static final String NAME="submit-paper";
        public SubmitAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Electronically submit this Blaise Paper for grading");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //must be online.
            if(!TransmissionHandler.isOnline()){
                JOptionPane.showMessageDialog(bep.parent,"Blaise could not find an internet connection.  Please check your settings and try again.","Cannot Submit Paper",JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            //check to see if there is a filename in save file.  If not prompt and get it.
            //the makes saving easier later, as there will be no user interaction.
            if(bep.saveFile==null || bep.saveFile.getName().equals("")){
                bep.saveFile = bep.getFileFromChooser(bep.parent, Constants.getUserDirectory(), JFileChooser.SAVE_DIALOG);
                //if it still ==null then they cancelled, and we should stop.
                if(bep.saveFile==null){
                    JOptionPane.showMessageDialog(bep.parent,"You must save your Blaise Paper to submit it.\nPlease save your paper and submit again.","Paper Not Submitted",JOptionPane.ERROR_MESSAGE);
                    return;
                }
            }
            new SubmissionWizard(bep.parent,this);
            if(true){
                return;
            }
            
            Thread thread=new Thread(){
                public void run(){
                    int answer=JOptionPane.showConfirmDialog(bep.parent,"Submission cannot be undone as the submitted paper will be saved.\nAre you sure you want to submit?","Confirm Submission Request",JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
                    
                    if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.NO_OPTION || answer==JOptionPane.CLOSED_OPTION){
                        return;
                    }
                    
                    //check to see if there is a filename in save file.  If not prompt and get it.
                    //the makes saving easier later, as there will be no user interaction.
                    if(bep.saveFile==null || bep.saveFile.getName().equals("")){
                        bep.saveFile = bep.getFileFromChooser(bep.parent, Constants.getUserDirectory(), JFileChooser.SAVE_DIALOG);
                        //if it still ==null then they cancelled, and we should stop.
                        if(bep.saveFile==null){
                            JOptionPane.showMessageDialog(bep.parent,"You must save your Blaise Paper to submit it.\nPlease save your paper and submit again.","Paper Not Submitted",JOptionPane.ERROR_MESSAGE);
                            return;
                        }
                    }
                    
                    if(!bep.mainView.isEmpty()){
                        if(bep.parent.document.hasDrafts()){
                            answer = JOptionPane.showConfirmDialog(bep.parent, "Would you like to finalize the current working draft before submission?", "Finalize Working Draft?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
                        }
                        else{
                            answer = JOptionPane.showConfirmDialog(bep.parent, "You must finalize working draft in order to submit this paper.\nFinalize now?", "Finalize Working Draft?",JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
                        }
                        
                        if(answer==JOptionPane.YES_OPTION || answer==JOptionPane.OK_OPTION){
                            bep.finalizeDraftAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED,bep.QUIETLY));
                            try{
                                bep.finalizeDraftThread.join();
                            }catch(InterruptedException e){}
                            if(bep.finalizeCancelled){
                                bep.finalizeCancelled=false;
                                return;
                            }
                        }
                        else if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.CLOSED_OPTION){
                            return;
                        }
                    }
                    
                    //now we're ready to actually get started.
                    //fill in paper stuff
                    if(!fillInPaper()){//true if it worked
                        //revert to saved.
                        bep.openAction.actionPerformed(new ActionEvent(bep.parent,ActionEvent.ACTION_PERFORMED,bep.saveFile.getAbsolutePath()));
                        try{
                            bep.openThread.join();
                        }catch(InterruptedException e){}
                        return;
                    }
                    
                    if(!doSubmit()){//true if it worked
                        //revert to saved.
                        bep.openAction.actionPerformed(new ActionEvent(bep.parent,ActionEvent.ACTION_PERFORMED,bep.saveFile.getAbsolutePath()));
                        try{
                            bep.openThread.join();
                        }catch(InterruptedException e){}
                        return;
                    }
                    
                    //success.  Save changes.
                    bep.saveAction.actionPerformed(new ActionEvent(bep.parent,ActionEvent.ACTION_PERFORMED,SaveAction.NAME));
                    try{
                        bep.saveThread.join();
                    }catch(InterruptedException e){}
                    
                }//end run
            };
            thread.start();
        }
        private boolean fillInPaper(){
            LoginUtility login = new LoginUtility(bep.parent, LoginUtility.CLIENT);
            login.confirmLogin();
            Thread lThread=login.getLoginThread();
            if(login.getOutput()==login.CANCEL_OPTION){
                return false;
            }
            try{
                while(lThread==null){
                    //swing hasn't had a chance to paint the dialog yet.
                    //which means it hasn't taken model control, so sleep for a little
                    //bit and try again.
                    Thread.sleep(100);
                    lThread=login.getLoginThread();
                }
                //now wait for the thread the submit button spawned to finish.
                lThread.join();
            }catch(InterruptedException e){}
            Object result=login.getOutput();
            login.dispose();
            if(result == login.CANCEL_OPTION){
                return false;
            }
            else if(result instanceof String){
                JOptionPane.showMessageDialog(bep.parent,result, "Submission Failure", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            WriterLoginQuery myInfo = (WriterLoginQuery)result;
            
            //Fill in writer information for paper
            edu.gwu.blaise.version2.transmission.Writer writer = myInfo.getWriter();
            bep.parent.document.setWriterID(Long.toString(writer.getWriterID()));
            bep.parent.document.setUniversityID(Integer.toString(writer.getUniversityID()));
            bep.parent.document.setStudentIdentifier(writer.getStudentIdent());
            bep.parent.document.setFirst(writer.getFirstName());
            bep.parent.document.setLast(writer.getLastName());
            bep.parent.document.setEmail(writer.getEmail());
            
            //Fill in course information for the paper
            if(bep.parent.document.getCourseID().equals("")){
                PaperSubmissionDisplay cour = new PaperSubmissionDisplay(parent, myInfo);
                cour.determineCourse();
                Object course = cour.getCourse();
                cour.dispose();
                if(course == null){
                    JOptionPane.showMessageDialog(bep.parent, "There was an error with course selection.  Please see a Blaise Administrator.", "Blaise Submission Error", JOptionPane.ERROR_MESSAGE);
                    return false;
                }
                Course myCourse = (Course)course;
                
                if(parent.document.getVersion().equals("")){
                    parent.document.setVersion(Constants.getVersion());
                }
                parent.document.setCourseID(Long.toString(myCourse.getCourseID()));
                parent.document.setDept(myCourse.getDept());
                parent.document.setNumber(myCourse.getNumber());
                parent.document.setSection(myCourse.getSection());
            }
            return true;
        }
        private boolean doSubmit(){
            try{
                bep.startProgView("Submitting Blaise Paper...");
                SubmitPaperQuery submit = TransmissionHandler.getSubmitPaper(bep.parent.document);
                if(!submit.transmit()){
                    JOptionPane.showMessageDialog(bep.parent, submit.getMessage(), "Submission Failure", JOptionPane.ERROR_MESSAGE);
                    bep.stopProgView();
                    return false;
                }
                Blaise_Paper newPaper = submit.getUpdatedPaper();
                
                bep.parent.document = newPaper;
                bep.parent.updateDraftsWindow();
            }
            finally{
                bep.stopProgView();
            }
            return true;
        }
        
        //--------------------------------
        //SubmissionHandler methods
        //--------------------------------
        public boolean savePaper(Blaise_Paper paper){
            bep.parent.document=paper;
            bep.saveAction.actionPerformed(new ActionEvent(bep.parent,ActionEvent.ACTION_PERFORMED,SaveAction.NAME));
            try{
                bep.saveThread.join();
            }catch(InterruptedException e){
                System.out.println(e);
                return false;
            }
            return true;
        }
        
        public void revertToSaved(){
            bep.openAction.actionPerformed(new ActionEvent(bep.parent,ActionEvent.ACTION_PERFORMED,bep.saveFile.getAbsolutePath()));
            try{
                bep.openThread.join();
            }catch(InterruptedException e){}
        }
        
        public boolean hasWorkingDraft(){
            return !bep.mainView.isEmpty();
        }
        
        public boolean mustFinalizeDraft(){
            Vector drafts=bep.parent.document.getDrafts();
            if(drafts==null || drafts.size()==0){
                return true;
            }
            return false;
        }
        
        public boolean finalizeDraft(){
            bep.finalizeDraftAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED,bep.QUIETLY));
            try{
                bep.finalizeDraftThread.join();
            }catch(InterruptedException e){}
            if(bep.finalizeCancelled){
                bep.finalizeCancelled=false;
                return false;
            }
            return true;
        }
        
        public Blaise_Paper getPaper(){
            return bep.parent.document;
        }
        
        public void startProgView(String title){
            bep.startProgView(title);
        }
        
        public void stopProgView(){
            bep.stopProgView();
        }
        
        public void done(){
            bep.parent.updateDraftsWindow();
        }
    }
    
    public class ExitAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="exit-application";
        public ExitAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Exit Blaise Editor");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    int save=bep.saveIfNeeded();
                    if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                        return;
                    }
                    else if(save==JOptionPane.YES_OPTION){
                        try{
                            bep.saveThread.join();
                        }catch(InterruptedException e){
                        }
                    }
                    bep.deleteExistingDir();//take care to close the paper.
                    
                    //quit.
                    System.exit(0);
                }
            };
            thread.start();
        }
    }
    /**
     * Basic support for undo/redo from O'Reilly's Java Swing (2nd ed) pg 828-830
     * and advanced (multiple undos from 658-660.
     */
    public class UndoAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="undo-action";
        public UndoAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Undo the last performed action");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            try{
                bep.undoManager.undo();
            }catch(CannotUndoException e){
                e.printStackTrace();
            }
            finally{
                bep.myUIStateDelegate.updateUndoUI();
            }
        }
    }
    
    public class RedoAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="redo-action";
        public RedoAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Redo the last undone action");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            try{
                bep.undoManager.redo();
            }catch(CannotRedoException e){
                e.printStackTrace();
            }
            finally{
                bep.myUIStateDelegate.updateUndoUI();
            }
        }
    }
    
    public class FindAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="find-action";
        public FindAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Find text within the document");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            bep.findFrame=FindFrame.showFind(bep.mainView);
            
            //assume now that find again is okay (may be a slightly invalid assumption,
            //but it wont break anything).
            bep.myUIStateDelegate.setEnabled(bep.findAgainMI,true);
        }
    }
    
    public class FindAgainAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="find-again-action";
        public FindAgainAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Find text within the document");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            if(bep.findFrame!=null){
                findFrame.doFindNext();
            }
        }
    }
    
    public class ShowDraftCommentAction extends AbstractAction{
        private BlaiseEditorPane bep;
        private boolean draft;
        private int index;
        public static final String NAME="show-draft-or-comment-action";
        public ShowDraftCommentAction(BlaiseEditorPane bep, boolean draft, int index){
            super(NAME);
            this.bep=bep;
            this.draft=draft;
            this.index=index;
            if(draft){
                putValue(Action.SHORT_DESCRIPTION,"Show the selected draft");
            }
            else{
                putValue(Action.SHORT_DESCRIPTION,"Show the selected graded draft");
            }
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            if(draft){
                bep.parent.draft_pane.selectDraft(index);
            }
            else{
                bep.parent.draft_pane.selectComment(index);
            }
        }
    }
    
    public class ToggleScribblesAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="toggle-scribbles-action";
        public ToggleScribblesAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Show or Hide scribbles in the current graded draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            boolean vis=bep.parent.draft_pane.areScribblesVisible();
            bep.parent.draft_pane.setScribblesVisible(!vis);
        }
    }
    
    public class ToggleNotesAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="toggle-notes-action";
        public ToggleNotesAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Show or Hide notes in the current graded draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
        }
    }
    
    public class ExpandAllNotesAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="expand-all-notes-action";
        public ExpandAllNotesAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Expand all the notes in the current graded draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
        }
    }
    
    public class CollapseAllNotesAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="collapse-all-notes-action";
        public CollapseAllNotesAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Collapse all the notes in the current graded draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
        }
    }
    
    public class NewDraftAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="new-draft-action";
        public NewDraftAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Create a new blank draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //basically this is just finalize draft.
            if(bep.mainView.isEmpty()){
                JOptionPane.showMessageDialog(bep.parent,"In order to create a new blank draft, the current\nworking draft must be finalized and the working draft is empty.","Cannot Finalize Current Draft",JOptionPane.WARNING_MESSAGE);
            }
            else{
                bep.finalizeDraftAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED, FinalizeDraftAction.NAME));
            }
        }
    }
    
    public class DuplicateDraftAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="duplicate-draft-action";
        public DuplicateDraftAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Create a new draft from an existing draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    int answer=JOptionPane.showConfirmDialog(bep.parent,"Are you sure you want to duplicate a new draft?  Doing so will finalize any current working drafts.","Confirm Duplicate Draft",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
                    if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.CLOSED_OPTION){
                        return;
                    }
                    if(!bep.mainView.isEmpty()){
                        bep.finalizeDraftAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED, bep.QUIETLY));
                        try{
                            bep.finalizeDraftThread.join();
                            if(bep.finalizeCancelled){
                                bep.finalizeCancelled=false;
                                return;
                            }
                        }catch(InterruptedException e){
                        }
                    }
                    JComboBox desiredDraft = new JComboBox();
                    Vector possibles = parent.document.getDrafts();
                    if(possibles.size()==0){
                        JOptionPane.showMessageDialog(bep.parent,"There are no drafts to duplicate.","No Drafts",JOptionPane.INFORMATION_MESSAGE);
                        return;
                    }
                    Object[] possibleValues = new Object[possibles.size()];
                    for(int i=0;i<possibles.size();i++){
                        possibleValues[i] = ((((Blaise_Draft)possibles.elementAt(i)).getDraftNumber()) +" - "+(((Blaise_Draft)possibles.elementAt(	i)).getTitle()));
                    }
                    
                    Object selectedValue = JOptionPane.showInputDialog(mainView, "Please select the draft to duplicate.", "Duplicate Existing Draft",	JOptionPane.QUESTION_MESSAGE, null, possibleValues, possibleValues[0]);
                    
                    if(selectedValue==null){
                        return;
                    }
                    Blaise_Draft current;
                    for(int i=0;i<possibles.size();i++){
                        current = (Blaise_Draft)possibles.elementAt(i);
                        if(selectedValue.equals((current.getDraftNumber()+" - "+current.getTitle()))){
                            bep.mainView.setText(current.getBody());
                            bep.myUIStateDelegate.resetButtonsAndMenus();
                            break;
                        }
                    }
                    bep.myUIStateDelegate.setModified(true);
                }
            };
            thread.start();
        }
    }
    
    public class AddExternalDraftAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="add-external-draft-action";
        public AddExternalDraftAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Set the specified external document as the current working draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(final ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    //ask what file they'd like to attach.
                    File attachFile=getFileFromChooser(bep.parent,null,JFileChooser.OPEN_DIALOG,false);//just need a number that isn't OPEN_DIALOG or SAVE_DIALOG
                    if(attachFile==null){
                        return;
                    }
                    
                    int answer = JOptionPane.showConfirmDialog(bep.parent,"A copy of the selected file will be made and attached as this working draft.\nAny new changes made to the original document will not be reflected in this copy." , "Confirm Attachment Request", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
                    if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.CLOSED_OPTION){
                        return;
                    }
                    String backupText=bep.mainView.getText();//so we can undo if an IOException happens.
                    
                    File internalFile=null;
                    try{
                        bep.startProgView("Attaching external draft as working body...");
                        String papDir=Constants.getClientPaperDirectory();
                        String attachFilename=attachFile.getName();
                        internalFile=new File(papDir+"WorkingDraft/Files/"+attachFilename);
                        //copy the old file into the new file...
                        DocumentUtilities.duplicateFile(attachFile,internalFile);
                        Blaise_Draft draft=bep.parent.document.getWorkingDraft();
                        draft.setExternalBodyLink(internalFile);
                        //internalFile.setReadOnly();
                        
                        //now update the mainview stuff.
                        bep.myUIStateDelegate.setExternalBody(true);
                    }
                    catch(IOException e){
                        //an error occurred.  Tell the user, and then remove any external body...
                        JOptionPane.showMessageDialog(bep.parent, e.toString(), "Could not attach document", JOptionPane.ERROR_MESSAGE);
                        Blaise_Draft draft=bep.parent.document.getWorkingDraft();
                        draft.removeExternalBody();
                        bep.myUIStateDelegate.setExternalBody(false);//undo any shiznit that changed in the UI.
                        bep.myUIStateDelegate.setModified(false);//undo that.
                        if(internalFile!=null && internalFile.exists()){
                            internalFile.delete();
                        }
                        bep.mainView.setText(backupText);
                    }
                    finally{
                        bep.stopProgView();
                    }
                }
            };
            thread.start();
        }
    }
    
    public class RemoveExternalDraftAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="remove-external-draft-action";
        public RemoveExternalDraftAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Detach the external document from being the current working draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(final ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    Blaise_Draft draft=bep.parent.document.getWorkingDraft();
                    File internalFile=draft.getExternalBodyLink();
                    draft.removeExternalBody();
                    bep.myUIStateDelegate.setExternalBody(false);
                    if(internalFile!=null && internalFile.exists()){
                        internalFile.delete();
                    }
                }
            };
            thread.start();
        }
    }
    
    public class FinalizeDraftAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="finalize-draft-action";
        public FinalizeDraftAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Designate the current working draft as finished and ready for submission");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(final ActionEvent ae){
            bep.finalizeDraftThread=new Thread(){
                public void run(){
                    final boolean quietly=(ae.getActionCommand()==bep.QUIETLY);//don't interact with the user.
                    if(mainView.isEmpty()){//not needed
                        if(!quietly)JOptionPane.showMessageDialog(bep.parent,"Working body is empty.","Cannot Finalize Current Draft",JOptionPane.WARNING_MESSAGE);
                        return;
                    }
                    else{
                        if(!quietly){
                            int answer=JOptionPane.showConfirmDialog(bep.parent,"Draft finalization cannot be undone.","Confirm Draft Finalization",JOptionPane.OK_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE);//NOTE: not actually true, we just need to save a copy of the current blaie_paper... a pain but doable.
                            if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.CLOSED_OPTION){
                                return;
                            }
                        }
                    }
                    //first, save.
                    int save=bep.saveIfNeeded(false);
                    if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                        return;
                    }
                    else if(save==JOptionPane.YES_OPTION){
                        try{
                            bep.saveThread.join();
                        }catch(InterruptedException e){
                        }
                    }
                    
                    //Determine last title
                    String title;
                    Blaise_Draft last=bep.parent.document.getLastDraft();
                    if(last==null){
                        title="";
                    }
                    else{
                        title=last.getTitle();
                    }
                    //String title = bep.parent.document.getLastDraft().getTitle();
                    //Prompt for title
                    String myTitle = JOptionPane.showInputDialog(bep.parent, "Please enter the title of the new draft",title);
                    if(myTitle == null){//cancelled.
                        bep.finalizeCancelled=true;
                        return;
                    }
                    //move the working draft to be a full, real draft.
                    Blaise_Draft draft=bep.parent.document.getWorkingDraft();
                    draft.setPaperID(bep.parent.document.getPaperID());
                    draft.setDraftID("");
                    draft.setDraftNumber(bep.parent.document.draftCount());
                    draft.setTitle(myTitle);
                    final boolean wasExt=draft.hasExternalBody();
                    if(!wasExt){
                        draft.setBody(bep.mainView.getText());
                    }
                    bep.parent.document.addDraft(draft);
                    
                    //we need to move any files the draft may have.
                    //and we need to update any file references.
                    String newDir=Constants.getClientPaperDirectory()+"Draft"+draft.getDraftNumber()+"/";
                    try{
                        DocumentUtilities.moveExternalFiles(draft,new File(newDir));
                    }catch(IOException e){
                        e.printStackTrace();
                    }
                    
                    
                    bep.parent.document.setWorkingDraft(new Blaise_Draft());
                    bep.mainView.setText("");
                    
                    //save (after all they may not undo this).
                    bep.saveAction.actionPerformed(new ActionEvent(bep,ActionEvent.ACTION_PERFORMED,bep.saveAction.NAME));
                    //a save will be performed with out user intervention as the file name is already there.
                    try{
                        bep.saveThread.join();
                    }catch(InterruptedException e){
                    }
                    
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            bep.parent.updateDraftsWindow();
                            if(wasExt){
                                bep.myUIStateDelegate.setExternalBody(false);
                            }
                        }
                    });
                    bep.myUIStateDelegate.resetButtonsAndMenus();
                    
                }//end run
            };
            bep.finalizeDraftThread.start();
        }
    }
    
    public class ToggleRealTimeSpellCheckAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="toggle-real-time-spell-check";
        private RealTimeSpelling rts;
        /**
         * Instantiates a new realtime spellchecker action.
         * @param ignoredWords a list containing words to load into the ignored list.  It can be null.
         */
        public ToggleRealTimeSpellCheckAction(BlaiseEditorPane bep,java.util.List ignoredWords){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Turns spell check as you type on or off");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
            rts=new RealTimeSpelling(bep.mainView,ignoredWords);
        }
        
        public void actionPerformed(ActionEvent ae){
            toggleChecking(bep.toggleRealTimeSpellCheckCBMI.isSelected());
        }
        
        /**
         * Turns spell checking on or off
         */
        public void toggleChecking(boolean start){
            if(start){
                rts.startRealTimeSpelling();
            }
            else{
                rts.stopRealTimeSpelling();
            }
        }
        
        public java.util.List getIgnoredWords(){
            return rts.getIgnoredWords();
        }
        
        public void addIgnoredWords(java.util.List ignoredWords){
            rts.addIgnoredWords(ignoredWords);
        }
    }
    
    public class ThesaurusAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="thesaurus-action";
        public ThesaurusAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Lookup a word in the thesaurus");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
        }
    }
    
    public class WordCountAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="word-count-action";
        public WordCountAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Provide a word count for the current working draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            if(bep.mainView.isEmpty()){
                JOptionPane.showMessageDialog(bep.parent, ("There are no words in this document."), "Word Count Results", JOptionPane.INFORMATION_MESSAGE);
                return;
            }
            Document doc;
            doc=bep.mainView.getDocument();
            int total=0;
            try{
                String s=doc.getText(0,doc.getLength());
                total=s.split("\\p{Space}").length;
            }catch(BadLocationException e){}
            if(total == 0){
                JOptionPane.showMessageDialog(bep.parent, ("There are no words in this document."), "Word Count Results", JOptionPane.INFORMATION_MESSAGE);
            }
            else if(total == 1){
                JOptionPane.showMessageDialog(bep.parent, ("There is "+total+" word in this document."), "Word Count Results", JOptionPane.INFORMATION_MESSAGE);
            }
            else if(total > 1){
                JOptionPane.showMessageDialog(bep.parent, ("There are "+total+" words in this document."), "Word Count Results", JOptionPane.INFORMATION_MESSAGE);
            }
        }
    }
    
    public class HelpAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="help-action";
        public HelpAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Get help using the Blaise Editor");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            DocumentOpener docO=new DocumentOpener(new File("docs/editor/index.html"));
            if(!docO.open()){
                JOptionPane.showMessageDialog(bep.parent, docO.getErrorMsg(), "Could not open help", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    public class AboutAction extends AbstractAction{
        private BlaiseEditorPane bep;
        public static final String NAME="about-action";
        public AboutAction(BlaiseEditorPane bep){
            super(NAME);
            this.bep=bep;
            putValue(Action.SHORT_DESCRIPTION,"Find out about Blaise, and the Blaise Editor");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            new AboutDialog(bep.parent,AboutDialog.EDITOR_TYPE);
        }
    }
    //extend to make it toggle the toolbar button.
    public class BoldAction extends StyledEditorKit.BoldAction{
        private BlaiseEditorPane bep;
        public BoldAction(BlaiseEditorPane bep){
            super();
            this.bep=bep;
        }
        
        public void actionPerformed(ActionEvent ae){
            System.out.println("Bold action");
            super.actionPerformed(ae);
            StyledEditorKit kit=getStyledEditorKit(getEditor(ae));
            MutableAttributeSet attr=kit.getInputAttributes();
            boolean bold=StyleConstants.isBold(attr);
            bep.boldTB.setSelected(bold);
        }
    }
    
    public class ItalicAction extends StyledEditorKit.ItalicAction{
        private BlaiseEditorPane bep;
        public ItalicAction(BlaiseEditorPane bep){
            super();
            this.bep=bep;
        }
        
        public void actionPerformed(ActionEvent ae){
            super.actionPerformed(ae);
            StyledEditorKit kit=getStyledEditorKit(getEditor(ae));
            MutableAttributeSet attr=kit.getInputAttributes();
            boolean italic=StyleConstants.isItalic(attr);
            bep.italicTB.setSelected(italic);
        }
    }
    
    public class UnderlineAction extends StyledEditorKit.UnderlineAction{
        private BlaiseEditorPane bep;
        public UnderlineAction(BlaiseEditorPane bep){
            super();
            this.bep=bep;
        }
        
        public void actionPerformed(ActionEvent ae){
            super.actionPerformed(ae);
            StyledEditorKit kit=getStyledEditorKit(getEditor(ae));
            MutableAttributeSet attr=kit.getInputAttributes();
            boolean underline=StyleConstants.isUnderline(attr);
            bep.underlineTB.setSelected(underline);
        }
    }
    
    public class LeftAlignAction extends StyledEditorKit.AlignmentAction{
        private BlaiseEditorPane bep;
        public LeftAlignAction(BlaiseEditorPane bep){
            super("left-justify", StyleConstants.ALIGN_LEFT);
            this.bep=bep;
        }
        
        public void actionPerformed(ActionEvent ae){
            super.actionPerformed(ae);
            bep.leftAlignTB.setSelected(true);
        }
    }
    
    public class RightAlignAction extends StyledEditorKit.AlignmentAction{
        private BlaiseEditorPane bep;
        public RightAlignAction(BlaiseEditorPane bep){
            super("right-justify", StyleConstants.ALIGN_RIGHT);
            this.bep=bep;
        }
        
        public void actionPerformed(ActionEvent ae){
            super.actionPerformed(ae);
            bep.rightAlignTB.setSelected(true);
        }
    }
    
    public class CenterAlignAction extends StyledEditorKit.AlignmentAction{
        private BlaiseEditorPane bep;
        public CenterAlignAction(BlaiseEditorPane bep){
            super("center-justify", StyleConstants.ALIGN_CENTER);
            this.bep=bep;
        }
        
        public void actionPerformed(ActionEvent ae){
            super.actionPerformed(ae);
            bep.centerAlignTB.setSelected(true);
        }
    }
    
    /**
     * Anything that changes the UIState should be done through this class.
     * Much of the time it just passes on the request, but sometimes it will check to see if
     * the request is valid for the current state, or perform other checking actions.
     *
     * @author Michael Gaiman
     * @version 1 3/18/2004
     */
    private class UIStateDelegate implements CaretListener, DraftViewListener, UndoableEditListener{
        BlaiseEditorPane bep;
        public UIStateDelegate(BlaiseEditorPane bep){
            this.bep=bep;
            
            bep.mainView.addCaretListener(this);
            bep.mainView.getDocument().addUndoableEditListener(this);
            
            resetButtonsAndMenus();
        }
        /**
         * Unlike most UIStateDelegate methods, caretUpdate isn't threadsafe.
         */
        public void caretUpdate(final CaretEvent e){
            int caret = e.getDot();
            int end = e.getMark();
            
            boolean selection = (caret!=end);
            bep.cutMI.setEnabled(selection);
            bep.cutB.setEnabled(selection);
            bep.copyMI.setEnabled(selection);
            bep.copyB.setEnabled(selection);
            
            //want formatting to be from left side of selection
            if(end < caret){
                caret = end;
            }
            AttributeSet a;
            if(caret==0){
                a = ((JTextPane)(e.getSource())).getStyledDocument().getCharacterElement(caret).getAttributes();
            }
            else{
                a = ((JTextPane)(e.getSource())).getStyledDocument().getCharacterElement(caret-1).getAttributes();
            }
            if(StyleConstants.isBold(a)){
                bep.boldTB.setSelected(true);
                bep.boldCBMI.setSelected(true);
            }
            else{
                bep.boldTB.setSelected(false);
                bep.boldCBMI.setSelected(false);
            }
            if(StyleConstants.isItalic(a)){
                bep.italicTB.setSelected(true);
                bep.italicCBMI.setSelected(true);
            }
            else{
                bep.italicTB.setSelected(false);
                bep.italicCBMI.setSelected(false);
            }
            
            if(StyleConstants.isUnderline(a)){
                bep.underlineTB.setSelected(true);
                bep.underlineCBMI.setSelected(true);
            }
            else{
                bep.underlineTB.setSelected(false);
                bep.underlineCBMI.setSelected(false);
            }
            
            int align = StyleConstants.getAlignment(a);
            if(align == StyleConstants.ALIGN_LEFT){
                bep.leftAlignTB.setSelected(true);
                bep.leftAlignCBMI.setSelected(true);
            }
            else if(align == StyleConstants.ALIGN_CENTER){
                bep.centerAlignTB.setSelected(true);
                bep.centerAlignCBMI.setSelected(true);
            }
            else if(align == StyleConstants.ALIGN_RIGHT){
                bep.rightAlignTB.setSelected(true);
                bep.rightAlignCBMI.setSelected(true);
            }
            
            //update the finalizeDraftMI and submitMI
            if(bep.mainView.isEmpty()){
                bep.finalizeDraftMI.setEnabled(false);
                bep.submitPaperMI.setEnabled(bep.parent.document.hasDrafts());
            }
            else{
                bep.finalizeDraftMI.setEnabled(true);
                bep.submitPaperMI.setEnabled(true);
            }
        }
        
        /**
         * Handles the updating of the draft and comment submenus.
         */
        public void draftViewChanged(DraftViewEvent e){
            System.out.println(e);
            int change=e.getChangeType();
            if(change==e.VIEW_UPDATE){
                populateShowDraftMenu();
                populateShowGradedMenu();
            }
            else if(change==e.DRAFT_SELECTED){
                populateShowDraftMenu(e.getSelectedIndex());
                populateShowGradedMenu(((BlaiseDraftView)e.getSource()).getSelectedCommentIndex());
            }
            else if(change==e.COMMENT_SELECTED){
                populateShowGradedMenu(e.getSelectedIndex());
            }
        }
        
        /**
         * Takes care of the undo manager and updating UI.
         */
        public void undoableEditHappened(UndoableEditEvent e){
            //System.out.println(e);
            bep.undoManager.addEdit(e.getEdit());
            updateUndoUI();
        }
        
        public void updateUndoUI(){
            bep.undoMI.setText(bep.undoManager.getUndoPresentationName());
            bep.redoMI.setText(bep.undoManager.getRedoPresentationName());
            bep.undoMI.setEnabled(bep.undoManager.canUndo());
            bep.redoMI.setEnabled(bep.undoManager.canRedo());
        }
        
        /**
         * Sets the title to "Blaise Editor: <title string>" or if title is null, the current document number.
         * Thread safe.
         */
        public void setTitle(final String title){
            if(title==null){
                if(Constants.isOSX()){//windows and osx have different preferences for their default name.
                    setTitle("Untitled "+bep.documentNumber);
                }
                else{
                    setTitle("Document "+bep.documentNumber);
                }
            }
            else{
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        bep.parent.setTitle("Blaise Editor: "+title);
                    }
                });
            }
        }
        
        private boolean hasExternal=false;
        private ExternalFileView efView=null;
        private Blaise_Draft efDraft=null;
        
        /**
         * Updates the UI to reflect the fact that the working body is either
         * internal or external.
         */
        public void setExternalBody(boolean hasExternal){
            if(hasExternal && this.hasExternal){//check to see if this new call is referring to a different draft
                    if(efDraft==bep.parent.document.getWorkingDraft()){
                        return;//not to worry...
                    }
                    else{
                        //uninstall the old one...
                        efView.uninstall();
                    }
            }
            else if(this.hasExternal==hasExternal){
                //ignore it.
                return;
            }
            this.hasExternal=hasExternal;
            efDraft=bep.parent.document.getWorkingDraft();
            if(efView==null){
               efView=new ExternalFileView(bep.parent);
            }
            
            setModified(true);
            bep.mainView.setEditable(!hasExternal);
            //turn off spelling.
            bep.toggleRealTimeSpellCheckAction.toggleChecking(!hasExternal);
            bep.toggleRealTimeSpellCheckCBMI.setSelected(!hasExternal);
            bep.toggleRealTimeSpellCheckCBMI.setEnabled(!hasExternal);
            bep.addExternalDraftMI.setEnabled(!hasExternal);
            bep.removeExternalDraftMI.setEnabled(hasExternal);
            if(hasExternal){
                efView.install(bep.mainView,efDraft);
                bep.finalizeDraftMI.setEnabled(true);
            }
            else{
                efView.uninstall();
                bep.mainView.setEditable(true);
                bep.mainView.setText("");
                bep.mainView.setCaretPosition(0);
                bep.finalizeDraftMI.setEnabled(false);
            }
        }
        /**
         * Resets the toggle buttons, and dynamic menus back to their initial state.
         */
        private void resetButtonsAndMenus(){
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    bep.mainView.setEditable(true);//incase last was external.
					bep.mainView.setCaretPosition(0);//always reset caret to beginning
                
                    bep.leftAlignTB.setSelected(true);
                    bep.leftAlignCBMI.setSelected(true);
                    
                    bep.boldTB.setSelected(false);
                    bep.boldCBMI.setSelected(false);
                    bep.italicTB.setSelected(false);
                    bep.italicCBMI.setSelected(false);
                    bep.underlineTB.setSelected(false);
                    bep.underlineCBMI.setSelected(false);
                    
                    bep.undoManager.discardAllEdits();//empty the undo manager.
                    bep.undoMI.setEnabled(false);
                    bep.undoMI.setText("Undo");//reset any text rename.
                    bep.redoMI.setEnabled(false);
                    bep.redoMI.setText("Redo");
                    
                    bep.cutMI.setEnabled(false);
                    bep.cutB.setEnabled(false);
                    bep.copyMI.setEnabled(false);
                    bep.copyB.setEnabled(false);
                    
                    bep.findAgainMI.setEnabled(false);
                    
                    populateShowDraftMenu();
                    populateShowGradedMenu();
                    
                    bep.toggleScribblesCBMI.setEnabled(false);
                    bep.toggleNotesCBMI.setEnabled(false);
                    
                    bep.expandNotesMI.setEnabled(false);
                    bep.collapseNotesMI.setEnabled(false);
                    
                    boolean mvHasUseful=!bep.mainView.isEmpty();
                    boolean haveDrafts=bep.parent.document.hasDrafts();
                    boolean external=bep.parent.document.getWorkingDraft().hasExternalBody();
                    bep.finalizeDraftMI.setEnabled(mvHasUseful || external);
                    bep.submitPaperMI.setEnabled(mvHasUseful || haveDrafts || external);
                    bep.duplicateDraftMI.setEnabled(haveDrafts);
                    bep.addExternalDraftMI.setEnabled(!external);
                    bep.removeExternalDraftMI.setEnabled(external);
                    
                    setModified(false);
                }
            });
        }
        
        /**
         * All changes external programmatic changes to the enabledness of the menu or toolbar items
         * should be performed via this method.  In most cases it will just pass the request on, but
         * sometimes it will satisfy dependancies or handle related happenings.
         * Thread Safe
         * @param button The toolbar button or menu bar item to be enabled or disabled.
         */
        public void setEnabled(final AbstractButton button, final boolean enabled){
            //for now just pass it on.
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    button.setEnabled(enabled);
                }
            });
        }
        
        /**
         * Calls back to the BlaiseDraftView for a list of drafts and puts them
         * in the menu (with proper actions)
         */
        public void populateShowDraftMenu(){
            populateShowDraftMenu(bep.parent.draft_pane.getSelectedDraftIndex());
        }
        
        private void populateShowDraftMenu(int index){
            //for the moment let's be lame about this and not optimize.
            //with each call we'll remake the menu.  We could store some stuff
            //and check to see if the menu has changed... but this is quicker and dirtier.
            //fun stuff.
            
            String[] list=bep.parent.draft_pane.getDraftList();
            JMenuItem item;
            
            bep.showDraftM.removeAll();
            for(int i=0;i<list.length;i++){
                item=new JCheckBoxMenuItem(new ShowDraftCommentAction(bep,true,i));
                item.setText(list[i]);
                bep.showDraftM.add(item);
                if(i==index){
                    item.setSelected(true);
                }
            }
            
            if(list.length==0){
                item=new JMenuItem();
                item.setEnabled(false);
                item.setText("No Drafts");
                bep.showDraftM.add(item);
            }
        }
        /**
         * Calls back to the BlaiseDraftView for a list of comments and puts them
         * in the menu (with proper actions)
         */
        public void populateShowGradedMenu(){
            int c=-1;
            if(bep.parent.draft_pane.getSelectedDraftIndex()>=0){
                c=bep.parent.draft_pane.getSelectedCommentIndex();
            }
            populateShowGradedMenu(c);
        }
        
        private void populateShowGradedMenu(int index){
            String[] list=bep.parent.draft_pane.getCommentList();
            JMenuItem item;
            
            bep.showGradedM.removeAll();
            for(int i=0;i<list.length;i++){
                item=new JCheckBoxMenuItem(new ShowDraftCommentAction(bep,false,i));
                item.setText(list[i]);
                bep.showGradedM.add(item);
                if(i==index){
                    item.setSelected(true);
                }
            }
            
            if(list.length==0){
                item=new JMenuItem();
                item.setEnabled(false);
                item.setText("No Graded Drafts");
                bep.showGradedM.add(item);
                
                //update toggle scribbles
                bep.toggleScribblesCBMI.setSelected(false);
                bep.toggleScribblesCBMI.setEnabled(false);
            }
            else{
                bep.toggleScribblesCBMI.setSelected(bep.parent.draft_pane.areScribblesVisible());
                bep.toggleScribblesCBMI.setEnabled(true);
            }
            
        }
        
        /**
         * Changes the internal mainView modified property.
         * Not thread safe (but no need).
         */
        public void setModified(boolean modified){
            bep.mainView.setModified(modified);
        }
    }//end UIStateDelegate
}