package edu.gwu.blaise.version2.client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

//import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;

/**
 * Combines two blaie papers into one paper taking the newest parts from each.
 * @author Sean Hanlon
 * @author Michael Gaiman (small overhaul)
 * @version 7/20/2004
 */
public class BlaisePaperMergeDialog extends JDialog{
    
    JDialog thisDialog;
    BlaiseEditorPane parent;
    File file1;
    File file2;
    File new_file;
    Blaise_Paper new_paper;
    Blaise_Paper doc1;
    Blaise_Paper doc2;
    byte[] selected;	//0 for doc1, 1 for doc2
    byte longer;//0 for doc1, 1 for doc2
    
    JLabel fileLabel;
    
    public BlaisePaperMergeDialog(BlaiseEditorPane parent, Blaise_Paper paper1, Blaise_Paper paper2, File file1, File file2){
        super(parent.parent,"Combine Blaise Papers", true);
        
        thisDialog = this;
        this.parent = parent;
        
        this.file1 = file1;
        this.file2 = file2;
        
        doc1 = paper1;
        doc2 = paper2;
        
        String pID1=paper1.getPaperID(),pID2=paper2.getPaperID();
        if(!pID1.equals("")){
            if(!pID2.equals("")){
                if(!pID1.equals(pID2)){
                    //bad..trying to combine different papers.
                    JOptionPane.showMessageDialog(parent.parent,"Cannot different papers.  Papers must be different versions of the same paper.","Cannot Combine Papers",JOptionPane.ERROR_MESSAGE);
                    dispose();
                    return;
                }
            }
        }
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private static final int WIDTH=470;
    private static final int HEIGHT=500;
    
    private void createAndShowGUI(){
        setSize(WIDTH,HEIGHT);
        
        int most_drafts = Integer.parseInt(doc1.draftCount());
        longer=0;
        if(Integer.parseInt(doc2.draftCount()) > most_drafts){
            most_drafts = Integer.parseInt(doc2.draftCount());
            longer=1;
        }
        
        //draft count has one too many drafts, but that works out because we need
        //a spot for working draft, too (which will be the last one).
        
        JPanel total_pane = new JPanel(new GridLayout(most_drafts+2, 1));
        selected = new byte[most_drafts];
        
        total_pane.add(createHeaderLine());
        total_pane.add(createDraftLine(-1));//do the working draft row.
        for(int i=0;i<most_drafts-1;i++){
            total_pane.add(createDraftLine(i+1));
        }
        
        total_pane.add(createFileAndButtons());
        
        this.getContentPane().add(new JScrollPane(total_pane,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.parent.getX();
            pY=parent.parent.getY();
            pW=parent.parent.getWidth();
            pH=parent.parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
        setLocation(myX,myY);
        //setResizable(false);
        this.setVisible(true);
    }
    
    private JPanel createHeaderLine(){
        JPanel output = new JPanel(new BorderLayout());
        JPanel inner = new JPanel(new GridLayout(1,2));
        inner.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        
        JLabel lab1 = new JLabel("<html><p>"+file1.getName()+"</p></html>", SwingConstants.CENTER);
        JLabel lab2 = new JLabel("<html><p>"+file2.getName()+"</p></html>", SwingConstants.CENTER);
        
        lab1.setBorder(BorderFactory.createMatteBorder(0,0,0,1,Color.BLACK));
        lab2.setBorder(BorderFactory.createMatteBorder(0,1,0,0,Color.BLACK));
        
        inner.add(lab1);
        inner.add(lab2);
        
        output.add(inner, BorderLayout.CENTER);
        
        return output;
    }
    
    /** -1 is the working draft*/
    private JPanel createDraftLine(final int index){
        Blaise_Draft dr1,dr2;
        if(index==-1){
            dr1 = doc1.getWorkingDraft();
            dr2 = doc2.getWorkingDraft();
        }
        else{
            dr1 = doc1.getDraftNumber(index);
            dr2 = doc2.getDraftNumber(index);
        }
        
        int dr1_count = 0;
        int dr2_count = 0;
        
        JPanel output = new JPanel(new BorderLayout());
        JPanel inner = new JPanel(new GridLayout(1,2));
        
        ButtonGroup group = new ButtonGroup();
        JRadioButton but1;
        JRadioButton but2;
        if(dr1 != null){
            if(index==-1){
                but1 = new JRadioButton("Working Draft");
            }
            else{
                but1 = new JRadioButton(dr1.getTitle());
            }
            dr1_count = dr1.getCommentSize();
            group.add(but1);
            inner.add(but1);
            but1.setSelected(true);
            if(index==-1){
                selected[selected.length-1] = 0;
            }
            else{
                selected[index-1] = 0;
            }
            
            but1.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e){
					//System.out.println("selected["+index+"] = 0");
                    if(index==-1){
                        selected[selected.length-1] = 0;
                    }
                    else{
                        selected[index-1] = 0;
                    }
                }
            });
        }
        else{
            inner.add(new JPanel());
        }
        
        if(dr2 != null){
            if(index==-1){
                but2 = new JRadioButton("Working Draft");
            }
            else{
                but2 = new JRadioButton(dr2.getTitle());
            }
            group.add(but2);
            inner.add(but2);
            
            dr2_count = dr2.getCommentSize();
            
            //neithor of these things will be true with working drafts
            if(dr1 == null){
                but2.setSelected(true);
                selected[index-1] = 1;
            }
            else{
                if(dr2_count > dr1_count){
                    but2.setSelected(true);
                    selected[index-1] = 1;
                }
            }
            
            if(index==-1){
                int size1=dr1.getBody().length();
                int size2=dr2.getBody().length();
                if(size2>size1){
                    but2.setSelected(true);
                    selected[selected.length-1]=1;
                }
            }
            
            but2.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e){
					//System.out.println("selected["+index+"] = 1");
                    if(index==-1){
                        selected[selected.length-1] = 1;
                    }
                    else{
                        selected[index-1] = 1;
                    }
                }
            });
        }
        else{
            inner.add(new JPanel());
        }
        
        inner.setBorder(BorderFactory.createMatteBorder(0,1,1,1,Color.BLACK));
        String labText;
        if(index==-1){
            labText="<pre> </pre>Working Draft";
        }
        else{
            labText="<pre> </pre>Draft "+index;
        }
        JLabel label = new JLabel("<html><p>"+labText+"</p></html>", SwingConstants.CENTER);
        label.setBorder(BorderFactory.createMatteBorder(0,0,1,0,Color.BLACK));
        
        output.add(label, BorderLayout.WEST);
        output.add(inner, BorderLayout.CENTER);
        
        return output;
    }
    
    private JPanel createFileAndButtons(){
        JPanel output = new JPanel(new BorderLayout());
        
        output.add(createFileLine(), BorderLayout.CENTER);
        output.add(createButtonLine(), BorderLayout.SOUTH);
        
        return output;
    }
    
    private JPanel createFileLine(){
        JPanel inner = new JPanel(new FlowLayout());
        fileLabel = new JLabel("<html><p>Output File:"+file1.getAbsolutePath()+"</p></html>",JLabel.LEFT);
        new_file=new File(file1.getAbsolutePath());
        
        JButton selectFile = new JButton("Select New File");
        selectFile.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                File file = parent.getFileFromChooser(parent.parent, file1.getAbsolutePath(), JFileChooser.SAVE_DIALOG);
                if(file != null){
                    fileLabel.setText("<html><p>Output File:"+file.getAbsolutePath()+"</p></html>");
                    new_file=new File(file.getAbsolutePath());
                }
            }
        });
        
        inner.add(fileLabel);
        inner.add(selectFile);
        return inner;
    }
    
    private JPanel createButtonLine(){
        JPanel output = new JPanel();
        
        JButton ok = new JButton("Combine Papers");
        ok.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                byte sel=0;
                //figure out the paper to get paper-wide properties from.
                String pID1=doc1.getPaperID(),pID2=doc2.getPaperID();
                if(pID1.equals("")){
                    if(pID2.equals("")){
                        sel=longer;
                    }
                    else{
                        sel=1;
                    }
                }
                else{
                    if(pID2.equals("")){
                        sel=0;
                    }
                    else{
                        sel=longer;
                    }
                }
            
                new_paper = new Blaise_Paper();
                Blaise_Paper old;
                
                if(sel==0){
                    old=doc1;
                }
                else{
                    old=doc2;
                }
                new_paper.setPaperID(old.getPaperID());
                new_paper.setVersion(old.getVersion());
                new_paper.setWriterID(old.getWriterID());
                new_paper.setUniversityID(old.getUniversityID());
                new_paper.setCourseID(old.getCourseID());
                new_paper.setFirst(old.getFirst());
                new_paper.setLast(old.getLast());
                new_paper.setStudentIdentifier(old.getStudentIdentifier());
                new_paper.setEmail(old.getEmail());
                new_paper.setDept(old.getDept());
                new_paper.setNumber(old.getNumber());
                new_paper.setSection(old.getSection());
                new_paper.setSemester(old.getSemester());
                new_paper.setYear(old.getYear());
                new_paper.setAssignmentID(old.getAssignmentID());
                new_paper.setAssignmentNumber(old.getAssignmentNumber());
                new_paper.setAssignmentName(old.getAssignmentName());
                
                //Create paper object (using headers from initial paper)
                new_paper.removeAllDrafts();
                
                //Add drafts according to the selected drafts
                for(int i=0;i<selected.length-1;i++){
                    if(selected[i] == 0){
                        new_paper.addDraft(doc1.getDraftNumber(i+1));
                    }
                    else{
                        new_paper.addDraft(doc2.getDraftNumber(i+1));
                    }
                }
                if(selected[selected.length-1]==0){
                    new_paper.setWorkingDraft(doc1.getWorkingDraft());
                }
                else{
                    new_paper.setWorkingDraft(doc2.getWorkingDraft());
                }
                
                //write the paper out.
                Thread thread=new Thread(){
                    public void run(){
                        try{
                            parent.startProgView("Saving Combined Paper...");
                            BlaiseFile bf=new BlaiseFile(new_paper,new_file,new BackendFileHandler());
                            bf.savePaper();
                        }catch(IOException e){
                            JOptionPane.showMessageDialog(parent.parent,"An Error Occurred that prevented the Blaise Document from saving.","Cannot Save Paper",JOptionPane.ERROR_MESSAGE);
                            return;
                        }finally{
                            parent.stopProgView();
                            SwingUtilities.invokeLater(new Runnable(){
                                public void run(){
                                    BlaisePaperMergeDialog.this.dispose();
                                }
                            });
                        }
                        JOptionPane.showMessageDialog(parent.parent,"Your papers have been successfully combined.","Success",JOptionPane.INFORMATION_MESSAGE);
                    }
                };
                thread.start();
            }
        });
        
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                new_file = null;
                dispose();
            }
        });
        
        output.add(ok);
        output.add(cancel);
        
        return output;
    }
    
    public File getFile(){
        return new_file;
    }
}