package edu.gwu.blaise.version2.client;
import java.io.*;
import javax.swing.*;
/**
 * The ClientFileFilter allows files to be filtered based on file extension when used in conjunction with a JFileChooser.
 * @author Sean T. Hanlon
 * @version 2 (moved from BlaiseEditorPane.java)
 */
class ClientFileFilter extends javax.swing.filechooser.FileFilter{

	//What to allow
	private String extension;

	//What to display to describe each file
	private String description;

	/**
	  * Constructor
	  *
	  * @param ext File extension to allow to pass through this filter
	  */
	public ClientFileFilter(String ext){
		extension = ext;
	}

	/**
	  * Constructor
	  *
	  * @param ext File extension to allow to pass through this filter
	  * @param desc A description of the file type
	  */
	public ClientFileFilter(String ext, String desc){
		extension = ext;
		description = desc;
	}

	/**
	 * Determines whether or not a file is acceptable based on the extension filters
	 *
	 * @param f File to test for acceptability
	 * @return <code>true</code> if file meets requirements; <code>false</out> otherwise.
	 */
	public boolean accept(File f){

		//Allow directories to be recursed through
		if(f.isDirectory()){
			return true;
		}

		//If extension is found at the end of a file, allow the file
		String ext = f.getName().toLowerCase();
		if(ext.endsWith(extension.toLowerCase())){
			return true;
		}
		else{
			return false;
		}
	}

	/**
	 * Returns description of file filter
	 *
	 * @return String of descriptor
	 */
	public String getDescription(){
		return description;
	}

}