//LOC:119
package edu.gwu.blaise.version2.client;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.Hashtable;
import java.util.StringTokenizer;
import javax.swing.*;
import javax.swing.BorderFactory.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.plaf.*;
import javax.swing.UIManager;

/** Class for implementing HTML format actions
   * (NOTE : Does not toggle. User must use the "Clear Styles" option to remove formatting correctly.)
 */
 public class FormatAction extends StyledEditorKit.StyledTextAction {
        HTML.Tag htmlTag;
        Hashtable htmlAttribs;
        JTextPane parentTextPane;

		/**
		 * Constructor
		 *
		 * @param actionName Name of action to be performed
		 * @param inTag HTML Tag regarding action
		 * @param mainView The contents of the document being operated on
	 	*/
        public FormatAction(String actionName, HTML.Tag inTag, JTextPane mainView) {
            super(actionName);
            htmlTag = inTag;
            htmlAttribs = new Hashtable(); // used for font attributes or anchor attributes
            parentTextPane = mainView;
        }

		/**
		 * Performs actions
		 *
		 * @param ae An ActionEvent
		 */
        public void actionPerformed(ActionEvent ae) {
            String selText = parentTextPane.getSelectedText();
            if(selText == null) {
                return;
            }
            // get the attributes that are set for where the cursor is
            SimpleAttributeSet sasText = new SimpleAttributeSet(parentTextPane.getCharacterAttributes());
            SimpleAttributeSet sasAttr = new SimpleAttributeSet();
            if(htmlTag.toString().equals(new HTML.UnknownTag("").toString())) {
                sasText = new SimpleAttributeSet(); // empty all the tags which we will apply later

            } else if(htmlTag.toString().equals(HTML.Tag.FONT.toString())) {
                sasText = new SimpleAttributeSet(parentTextPane.getCharacterAttributes());
                // open font dialog
                Font newFont = FontChooser.showDialog(null, "Choose Font", null);
                if(newFont != null) {
                    htmlAttribs.put("face", newFont.getFontName());
                    htmlAttribs.put("size", newFont.getSize()+"");
                    htmlAttribs.put("color", Color.black.toString()); // change later
                } else {
                    return;
                }
            } else if(htmlTag.toString().equals(HTML.Tag.A.toString())) {

                // creating a link, open link dialog
                if(!htmlAttribs.containsKey("href")) {
                    String newAnchor = JOptionPane.showInputDialog(parentTextPane,
                                                                   "Please enter the link address",
                                                                   "Create Link",
                                                                   JOptionPane.INFORMATION_MESSAGE);
                    if(newAnchor != null && newAnchor.length() > 0) {
                        htmlAttribs.put("href", newAnchor); // add the href attribute to the list
                    } else {
                        return;
                    }
                }
            }
            if(htmlAttribs.size() > 0) {
                // We have some attributes for this tag, so add them accordingly
                java.util.Enumeration attribEntries = htmlAttribs.keys();
                while(attribEntries.hasMoreElements()) {
                    Object entryKey   = attribEntries.nextElement();
                    Object entryValue = htmlAttribs.get(entryKey);
                    sasAttr.addAttribute(entryKey, entryValue);
                    htmlAttribs.remove(entryKey);
                }
            }
            int textLength = selText.length();

            // add this attribute to the attributes that are already there
            sasText.addAttribute(htmlTag, sasAttr);
            if(textLength > 0) {

                int caretOffset = parentTextPane.getSelectionStart();
                parentTextPane.select(caretOffset, caretOffset + textLength);
                // set the new attributes to the text
                if(htmlTag.toString().equals(new HTML.UnknownTag("").toString())) {
                    // replace the current tag set (from Custom Action)
                    StyledEditorKit sekBase = getStyledEditorKit(parentTextPane);
                    MutableAttributeSet masAttribs = sekBase.getInputAttributes();
                    masAttribs.addAttributes(sasText);

                    ActionEvent actionEvent = new ActionEvent(parentTextPane, 0, selText);
                    new DefaultEditorKit.InsertContentAction().actionPerformed(actionEvent);
                } else {
                    parentTextPane.setCharacterAttributes(sasText, false); // add to the current tag set
                }
                parentTextPane.select(caretOffset, caretOffset + textLength);
            }
            // reload the page
            parentTextPane.setText(parentTextPane.getText());
        }
    } // end class FormatAction
