//LOC:156
package edu.gwu.blaise.version2.client;

import java.io.*;
import java.util.*;

import javax.crypto.*;
import javax.crypto.spec.*;
import java.math.*;
import java.security.*;
import java.security.spec.*;

/**
 * The PaperCrypto class provides encryption/decryption functionality for the Client Editor.
 * The functionality encryption is done using Password-Based Encryption. The password is generated using an MD5 hash of the passed String
 * DES is the cryptological algorithm used.
 *
 * @author 	Sean T. Hanlon
 *
 */

public class PaperCrypto{

	boolean debug = true;	//For debugging output messages

	//Parameters to generate and utilize key
	PBEKeySpec pbeKeySpec;
	PBEParameterSpec pbeParamSpec;

	//Initialization of buffer and salt key used in generating a key-spec
	byte[] buffer = new byte[20480];
	byte[] salt = {(byte)0xc7, (byte)0x73, (byte)0x21, (byte)0x8c, (byte)0x7e, (byte)0xc8, (byte)0xee, (byte)0x99};
	int count;	//how many iterations to perform

	/**
	 * Constructor
	 */
	public PaperCrypto(){

	    // Iteration count
	    count = 1000;

	    // Create PBE parameter set
	    pbeParamSpec = new PBEParameterSpec(salt, count);
	}

    /**
     * Receives a key and file, and encrypts file back into original location
     *
     * @param key String representation of the desired key
     * @param File File top be encrypted
     */
	public void encrypt(String key, File file) {

		if(debug)System.out.print("Encrypting file...");

		try{
			//Get an input stream from file
			FileInputStream fr = new FileInputStream(file);

			//Create temp file to write results into
		    String outputString = file.getAbsolutePath();
			File output = new File(outputString.substring(0, outputString.length() - 3) + "tmp");
			FileOutputStream outFile = new FileOutputStream(output);

			//Generate key spec based on key, salt, and iterations and generate key itself
		    pbeKeySpec = new PBEKeySpec(key.toCharArray(),salt,count);
		    SecretKey pbeKey = SecretKeyFactory.getInstance("PBEWithMD5AndDES").generateSecret(pbeKeySpec);

		    // Create PBE Cipher
		    Cipher pbeCipher = Cipher.getInstance("PBEWithMD5AndDES");

		    // Initialize PBE Cipher with key and parameters
		    pbeCipher.init(Cipher.ENCRYPT_MODE, pbeKey, pbeParamSpec);

			//Create cipher output stream out stream to the temp file
			OutputStream out = new CipherOutputStream(outFile, pbeCipher);

			//Read from buffer and write into output stream
			int numRead = 0;
			while((numRead=fr.read(buffer)) >= 0){
				out.write(buffer,0,numRead);
				//System.out.println("Writing: "+buffer);
			}

			//Cleanup
			fr.close();
			out.close();
			outFile.close();

			if(debug)System.out.println("Encrypt - Deleting File");

			//delete original file and rename temp
			file.delete();
			output.renameTo(file);
		}
		catch(Exception e){
			if(debug)System.out.println(e);
		}

		if(debug)System.out.println("Done");
	}

    /**
     * Receives a key and file, and decrypts file back into original location
     *
     * @param key String representation of the desired key
     * @param File File top be decrypted
     */
     public void decrypt(String key, File file){

		if(debug)System.out.print("Decrypting file...");

		try{
			//Get an input stream from file
			FileInputStream fr = new FileInputStream(file);

			//Create temp file to write results into
		    String outputString = file.getAbsolutePath();
			File output = new File(outputString.substring(0, outputString.length() - 3) + "tmp");
			FileOutputStream out = new FileOutputStream(output);

			//Generate key spec based on key, salt, and iterations and generate key itself
		    pbeKeySpec = new PBEKeySpec(key.toCharArray(),salt,count);
		    SecretKey pbeKey = SecretKeyFactory.getInstance("PBEWithMD5AndDES").generateSecret(pbeKeySpec);

		    // Create PBE Cipher
		    Cipher pbeCipher = Cipher.getInstance("PBEWithMD5AndDES");

		    // Initialize PBE Cipher with key and parameters
		    pbeCipher.init(Cipher.DECRYPT_MODE, pbeKey, pbeParamSpec);

			//Create cipher input stream to perform decryption
			InputStream in = new CipherInputStream(fr, pbeCipher);

			//Read from buffer and write into output stream
			int numRead = 0;
			while((numRead=in.read(buffer)) >= 0){
				out.write(buffer,0,numRead);
			System.out.println("Writing: "+buffer);
			}

			//Cleanup
			fr.close();
			out.close();

			if(debug)System.out.println("Decrypt - Deleting File");

			//delete original file and rename temp
			file.delete();
			output.renameTo(file);
		}
		catch(Exception e){
			if(debug)System.out.println(e);
		}

		if(debug)System.out.println("Done");
	}
}