package edu.gwu.blaise.version2.client;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.documents.*;
/**
 * The interface that needs to be implemented to support the
 * SubmissionWizard.  The Wizard is only the view--the implementor of
 * this class does the real work.
 * Note: all methods except done() will be called from a thread other than
 * the event thread.  done() will be called from the event thread.
 * @author Michael Gaiman
 * @version 1 7/11/2004
 */
public interface SubmissionHandler {
    
    /**
     * Causes the supplied paper to be saved.  The supplied paper should
     * replace any existing papers...
     * @returns true if the save was successful, false if the user cancelled
     * out of any presented dialogs (and thus the paper wasn't saved).
     */
    public boolean savePaper(Blaise_Paper paper);
    
    /**
     * The Submission was cancelled, so revert to last saved copy to loose
     * settings changes.
     */
    public void revertToSaved();
    
    /**
     * true if the user has text in the working draft.
     * false otherwise.
     */
    public boolean hasWorkingDraft();
    
    /**
     * true if the user must finalize the current working draft
     */
    public boolean mustFinalizeDraft();
    
    /**
     * Causes the current working draft to be finalized.
     * @return true if the finalization was successful, false if the user cancelled
     * out of any presented dialogs (and thus the draft wasn't finalized)
     */
    public boolean finalizeDraft();
    
    /**
     * Returns a reference to the actual paper.
     */
    public Blaise_Paper getPaper();
    
    /**
     * Starts the progress view with the supplied title.
     * (This method should just plug into whatever progress view structure is already
     * set up)
     */
    public void startProgView(String title);
    
    /**
     * Stops any progress view that is running.
     */
    public void stopProgView();
    
    /**
     * Called when the wizard is finished.
     */
    public void done();
}
