package edu.gwu.blaise.version2.client;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
/**
 * Guides the writer through the submission of their paper.
 * (Fills in the assignment and course fields).
 * @author Michael Gaiman
 * @version 1 7/11/2004
 */
public class SubmissionWizard extends JDialog{
    private JFrame parent;
    private SubmissionHandler handler;
    private CardLayout cardLayout;
    private Container cardCont;
    private Blaise_Paper paper;
    private Course[] courses;
    private Vector assignments;
    private Writer writer;
    private JButton cancelB;
    private JButton nextB;//changes text depending upon action--but it always the proceed button.
    private JLabel loginStatus;
    private JTextField userField;
    private JPasswordField passField;
    private JComboBox courseCB,assignmentCB;
    private JCheckBox finalizeCB;
    private JPanel reviewP,finishedP;
    private String oldCID,oldDept,oldNumber,oldSection,oldAID,oldANum,oldAName;
    private boolean wasFinalized=false;
    private boolean success=false;//so that the finalize action knows whether to revert or to save...

    public SubmissionWizard(JFrame parent,SubmissionHandler handler){
        super(parent,"Paper Submission Wizard");
        this.parent=parent;
        this.handler=handler;
        paper=handler.getPaper();
        oldCID=paper.getCourseID();
        oldDept=paper.getDept();
        oldNumber=paper.getNumber();
        oldSection=paper.getSection();
        oldAID=paper.getAssignmentID();
        oldANum=paper.getAssignmentNumber();
        oldAName=paper.getAssignmentName();
        
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private static final int WIDTH=400;
    private static final int HEIGHT=280;
    
    private void createAndShowGUI(){
        JPanel cardP=new JPanel();
        cardLayout=new CardLayout();
        cardP.setLayout(cardLayout);
        cardCont=(Container)cardP;
        
        //setup core gui
        Container cont=getContentPane();
        cont.setLayout(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        c.fill=c.BOTH;
        c.weightx=1;
        c.weighty=.9;
        c.gridwidth=4;
        cont.add(cardP,c);
        
        c.gridy=1;
        c.gridwidth=c.RELATIVE;
        c.weighty=.1;
        c.weightx=.3;
        c.fill=c.NONE;
        cancelB=new JButton(new CancelAction());
        cancelB.setText("Cancel");
        cont.add(cancelB,c);
        
        c.gridwidth=c.REMAINDER;
        nextB=new JButton(new NextAction());
        cont.add(nextB,c);
        
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.getX();
            pY=parent.getY();
            pW=parent.getWidth();
            pH=parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
    
        setSize(WIDTH,HEIGHT);
        setLocation(myX,myY);
        setResizable(false);
        
        
        //make initial card
        cardP.add(getLoginCard(),"login");
        cardP.add(getSelectionCard(),"selection");
        cardP.add(reviewP=getReviewCard(),"review");
        cardP.add(finishedP=new JPanel(),"finished");
        
        getRootPane().setDefaultButton(nextB);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        setVisible(true);
    }

    private JPanel getLoginCard(){
        JPanel container=new JPanel(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        
        c.fill=c.HORIZONTAL;
        c.anchor=c.PAGE_START;
        c.weightx=.95;
        c.weighty=.5;
        c.gridwidth=c.REMAINDER;
        container.add(new JLabel("<html><center><p>Blaise Editor Paper Submission Wizard</p><p>This wizard will guide you through the submission of your paper.  Login to begin.</p></center></html>"),c);
        
        //create the username row
        c.fill=c.NONE;
        c.anchor=c.CENTER;
        c.gridwidth=c.RELATIVE;
        c.weightx=.4;
        c.weighty=0;
        container.add(new JLabel("Username:",JLabel.RIGHT),c);
        userField=new JTextField(20);
        Dimension prefSize=userField.getPreferredSize();
        userField.setMinimumSize(prefSize);
        //userField.setMaximumSize(prefSize);
        userField.requestFocus();
        c.gridwidth=c.REMAINDER;
        c.weightx=.6;
        container.add(userField,c);
        
        c.fill=c.NONE;
        c.gridwidth=c.RELATIVE;
        c.weightx=.4;
        //create password row
        container.add(new JLabel("Password:",JLabel.RIGHT),c);
        //passField is not set by columns but rather by prefSize.
        //this fixes a strangeness from windows.
        passField=new JPasswordField();
        passField.setMinimumSize(prefSize);
        passField.setPreferredSize(prefSize);
        //passField.setMaximumSize(prefSize);
        c.gridwidth=c.REMAINDER;
        c.weightx=.6;
        container.add(passField,c);
        
        loginStatus=new JLabel(" ");
        c.fill=c.HORIZONTAL;
        c.weightx=1;
        c.weighty=.3;
        container.add(loginStatus,c);
        
        class UPListener implements DocumentListener{
            private JTextField userField;
            private JPasswordField passField;
            private JButton loginButton;
            private boolean uUseful=false;
            private boolean pUseful=false;
            
            public UPListener(JTextField u, JPasswordField p, JButton b){
                userField=u;
                passField=p;
                loginButton=b;
                loginButton.setText("Login");
                loginButton.setEnabled(false);
            }
            
            public void insertUpdate(DocumentEvent e){
                if(userField.getText().length()>0){
                    uUseful=true;
                }

                if(passField.getPassword().length>0){
                    pUseful=true;
                }
                
                if(uUseful && pUseful){
                    loginButton.setEnabled(true);
                }
                else{
                    loginButton.setEnabled(false);
                }
            }
            
            public void removeUpdate(DocumentEvent e){
                //check to see if it still has something.
                if(userField.getText().length()==0){
                    uUseful=false;
                }

                if(passField.getPassword().length==0){
                    pUseful=false;
                }
                
                if(uUseful && pUseful){
                    loginButton.setEnabled(true);
                }
                else{
                    loginButton.setEnabled(false);
                }
            }
            
            public void changedUpdate(DocumentEvent e){}
        };
        UPListener upListener=new UPListener(userField,passField,nextB);
        passField.getDocument().addDocumentListener(upListener);
        userField.getDocument().addDocumentListener(upListener);

        return container;
    }
    
    private JPanel getSelectionCard(){
        JPanel panel=new JPanel(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        
        c.fill=c.HORIZONTAL;
        c.anchor=c.PAGE_START;
        c.weightx=1;
        c.weighty=.3;
        c.gridwidth=c.REMAINDER;
        panel.add(new JLabel("<html><center><p>Selections</p><p>Select the course and assignment for this paper.  If you choose to finalize the current working draft, the new paper state will be saved.</p></center></html>"),c);
        
        c.fill=c.NONE;
        courseCB=new JComboBox();
        courseCB.setEditable(false);
        courseCB.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e){
                if(e.getStateChange()==ItemEvent.SELECTED){
                    fillInAssignments();
                }
            }
        });
        
        c.weightx=.4;
        c.weighty=0;
        c.gridwidth=c.RELATIVE;
        panel.add(new JLabel("Course",JLabel.RIGHT),c);
        
        c.weightx=.6;
        c.gridwidth=c.REMAINDER;
        c.fill=c.HORIZONTAL;
        panel.add(courseCB,c);
        
        assignmentCB=new JComboBox();
        assignmentCB.setEditable(false);
        
        c.weightx=.4;
        c.gridwidth=c.RELATIVE;
        c.fill=c.NONE;
        panel.add(new JLabel("Assignment",JLabel.RIGHT),c);
        
        c.weightx=.6;
        c.gridwidth=c.REMAINDER;
        c.fill=c.HORIZONTAL;
        panel.add(assignmentCB,c);
        
        //finalize checkbox
        finalizeCB=new JCheckBox("Finalize current working draft");
        finalizeCB.setEnabled(handler.hasWorkingDraft());
        
        if(handler.mustFinalizeDraft()){
            finalizeCB.setSelected(true);
            finalizeCB.setEnabled(false);
        }
        
        c.weightx=1;
        c.gridwidth=c.REMAINDER;
        c.fill=c.HORIZONTAL;
        panel.add(finalizeCB,c);
        
        return panel;
    }
    
    private JPanel getReviewCard(){
        JPanel panel=new JPanel(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        
        c.fill=c.HORIZONTAL;
        c.anchor=c.PAGE_START;
        c.weightx=1;
        c.weighty=.2;
        c.gridwidth=c.REMAINDER;
        panel.add(new JLabel("<html><center><p>Review Paper</p><p>The following is a summary of the paper you are about to submit.</p></center></html>"),c);
        
        return panel;
    }
    
    /**
     * @param message null if succesful.
     */
    private void doFinishedPanel(String message){
        if(message==null){
            success=true;
            finishedP.setLayout(new GridBagLayout());
            GridBagConstraints c=new GridBagConstraints();
            
            c.fill=c.HORIZONTAL;
            c.anchor=c.PAGE_START;
            c.weightx=1;
            c.weighty=.2;
            c.gridwidth=c.REMAINDER;
            finishedP.add(new JLabel("<html><center><p>Submission Successful</p><p>Your paper was successfully submitted.  You should receive an email receipt shortly.</p></center></html>"),c);
        }
        else{
            success=false;
            finishedP.setLayout(new GridBagLayout());
            GridBagConstraints c=new GridBagConstraints();
            
            c.fill=c.HORIZONTAL;
            c.anchor=c.PAGE_START;
            c.weightx=1;
            c.weighty=.2;
            c.gridwidth=c.REMAINDER;
            finishedP.add(new JLabel("<html><center><p>Submission Failed</p><p>Your paper could not be submitted.  Failure Reason: "+message+"</p></center></html>"),c);
            
        }
    }
    
    /**
     * updates the course and assignment combo boxes.
     */
    private void updateCoursesAndAssignments(){
        String cID=paper.getCourseID();
        int i;
        courseCB.removeAllItems();
        
        if(cID==null || cID.equals("")){
            for(i=0;i<courses.length;i++){
                courseCB.addItem(courses[i].getDept()+' '+courses[i].getNumber()+'-'+courses[i].getSection());
            }
            if(courses.length>0){
                assignments=courses[0].getAssignments();
            }
        }
        else{
            courseCB.addItem(paper.getDept()+' '+paper.getNumber()+'-'+paper.getSection());
            courseCB.setEnabled(false);
            //find the assigns vector for this course...
            for(i=0;i<courses.length;i++){
                if(cID.equals(Long.toString(courses[i].getCourseID()))){
                    assignments=courses[i].getAssignments();
                    break;
                }
            }
        }
        fillInAssignments();
    }
    
    private void fillInAssignments(){
        if(courseCB.isEnabled()){//paper didn't already have a course
            assignments=courses[courseCB.getSelectedIndex()].getAssignments();
        }
        
        assignmentCB.removeAllItems();
        //put the assignments in the combo box.
        String aID=paper.getAssignmentID();
        int size=assignments.size();
        boolean nFound=true;
        Assignment a;
        for(int i=0;i<size;i++){
            a=(Assignment)assignments.elementAt(i);
            assignmentCB.addItem(new StringBuffer('(').append(a.assignNum()).append(") ").append(a.name()));
            if(aID!=null && nFound){
                if(aID.equals(Long.toString(a.getAssignID()))){
                    assignmentCB.setSelectedIndex(i);
                    nFound=false;//found now...
                }
            }
        }
    }
        
    private class CancelAction extends AbstractAction{
        public static final String NAME="cancel-action";
        public CancelAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Cancel the submission of this paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            revert();
            handler.done();
            dispose();
        }
    }
    
    private void revert(){
        if(wasFinalized){
            Thread thread=new Thread(){
                public void run(){
                    handler.revertToSaved();
                }
            };
            thread.run();
        }
        //revert to the older values
        paper.setCourseID(oldCID);
        paper.setDept(oldDept);
        paper.setNumber(oldNumber);
        paper.setSection(oldSection);
        paper.setAssignmentID(oldAID);
        paper.setAssignmentNumber(oldANum);
        paper.setAssignmentName(oldAName);
    }
    
    private class NextAction extends AbstractAction{
        public static final String NAME="next-action";
        /**
         * lets NextAction know which card is being displayed (and thus which action
         * the NextAction should perform).  This int is kept insync internally.
         */
        private int card=0;
        public NextAction(){
            super(NAME);
            //putValue(Action.SHORT_DESCRIPTION,"changes...");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            switch(card){
                case 0:
                    doLogin();
                    return;
                case 1:
                    doSelection();
                    return;
                case 2:
                    doSubmit();
                    return;
                case 3:
                    doFinish();
                    return;
                default:
                    return;
            }
        }
        
        private void doLogin(){
            Thread thread=new Thread(){
                public void run(){
                    try{
                        userField.setEnabled(false);
                        passField.setEnabled(false);
                        nextB.setEnabled(false);
                        handler.startProgView("Logging in...");
                        loginStatus.setText("<html><p>Logging in...</p></html>");
                        WriterLoginQuery wlq=TransmissionHandler.getWriterLogin(userField.getText(),new String(passField.getPassword()),Constants.getUniversityID());
                        if(wlq.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                            writer=wlq.getWriter();
                            //set the course[]
                            java.util.Vector cs=wlq.getCourses();
                            courses=new Course[cs.size()];
                            courses=(Course[])cs.toArray(courses);
                            card++;
                            handler.stopProgView();
                            SwingUtilities.invokeLater(new Runnable(){
                                public void run(){
                                    updateCoursesAndAssignments();
                                    cardLayout.next(cardCont);
                                    nextB.setText("Next");
                                    nextB.setEnabled(true);
                                }
                            });
                            return;
                        }
                        else{
                            userField.setEnabled(true);
                            passField.setEnabled(true);
                            passField.setText("");
                            passField.requestFocus();
                            loginStatus.setText("<html><p>"+wlq.getMessage()+"</p></html>");
                        }
                    }finally{
                        handler.stopProgView();
                    }
                }
            };
            thread.start();
        }
        
        private void doSelection(){
            Thread thread=new Thread(){
                public void run(){
                    if(finalizeCB.isSelected()){
                        if(handler.finalizeDraft()){
                            wasFinalized=true;//so cancel can know...
                        }
                        else{
                            //cancelled--nothing important has happened yet...just finish
                            setVisible(false);
                            dispose();
                            return;
                        }
                    }
                    //set the generic info
                    paper.setWriterID(Long.toString(writer.getWriterID()));
                    paper.setUniversityID(Integer.toString(writer.getUniversityID()));
                    paper.setStudentIdentifier(writer.getStudentIdent());
                    paper.setFirst(writer.getFirstName());
                    paper.setLast(writer.getLastName());
                    paper.setEmail(writer.getEmail());
                    
                    //set the course and assignment.
                    int index;
                    if(courseCB.isEnabled()){
                        index=courseCB.getSelectedIndex();
                        paper.setCourseID(Long.toString(courses[index].getCourseID()));
                        paper.setDept(courses[index].getDept());
                        paper.setNumber(courses[index].getNumber());
                        paper.setSection(courses[index].getSection());
                    }
                    index=assignmentCB.getSelectedIndex();
                    Assignment a=(Assignment)assignments.elementAt(index);
                    paper.setAssignmentID(Long.toString(a.getAssignID()));
                    paper.setAssignmentNumber(Integer.toString(a.assignNum()));
                    paper.setAssignmentName(a.name());
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            card++;
                            GridBagConstraints c=new GridBagConstraints();
                            c.fill=c.BOTH;
                            c.weightx=1;
                            c.weighty=.8;
                            reviewP.add(new JScrollPane(BlaiseUtilities.getPaperTreeView(paper),JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED),c);
                            nextB.setText("Submit");
                            cardLayout.next(cardCont);
                        }
                    });
                }
            };
            thread.start();
        }
            String message=null;//for use with doSubmit
        private void doSubmit(){
            Thread thread=new Thread(){
                public void run(){
                    try{
                        handler.startProgView("Submitting paper...");
                        SubmitPaperQuery submit = TransmissionHandler.getSubmitPaper(paper);
                        if(!submit.transmit()){
                            message=submit.getMessage();
                        }
                        else{
                            paper = submit.getUpdatedPaper();
                        }
                    }finally{
                        handler.stopProgView();
                    }
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            doFinishedPanel(message);
                            card++;
                            nextB.setText("Finish");
                            cardLayout.next(cardCont);
                        }
                    });
                }
            };
            thread.start();
        }
        
        private void doFinish(){
            Thread thread=new Thread(){
                public void run(){
                    if(success){
                        handler.savePaper(paper);
                    }
                    else{
                        revert();
                    }
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            handler.done();
                            dispose();
                        }
                    });
                }
            };
            thread.start();
        }
    }
}
