//LOC:66
package edu.gwu.blaise.version2.documents;

import java.util.*;
import java.io.*;
import edu.gwu.blaise.version2.util.*;

/**
* A Blaise_Comment is the representation of grader-created comment encapsulating scribbles, correction sets, and the grade assigned.
 *
 * @author Sean T. Hanlon
 * @version 4/18/2004 (move to documents)
 */
public class Blaise_Comment implements ExternalFileContainer{
    
    private String commentID;
    private String paperID;
    private String graderID;
    private String draftID;
    private String isPublic;
    
    private String grader_name;
    private String assigned_grade;
    
    private String aspect_x;
    private String aspect_y;
    
    private String body;
    
    private Vector scribbles; 	//Vector of Scribbles
    private Vector corrections;	//Vector of CorrectionSet
    private Vector notes;
    
    private boolean hasExternalBody=false;
    private ExternalFile externalBody=null;
    
    private Vector externalFiles=null;//contains ExternalFile objects
    
    /**
     * @deprecated Does not allow for future versions.  Use the empty constructor instead.
     */
    public Blaise_Comment(String commentID, String paperID, String graderID, String draftID, String isPublic, String grader_name, String assigned_grade, String aspect_x, String aspect_y, String body, Vector scribbles, Vector corrections){
        
        this.commentID = commentID;
        this.paperID = paperID;
        this.graderID = graderID;
        this.draftID = draftID;
        this.isPublic = isPublic;
        this.grader_name = grader_name;
        this.assigned_grade = assigned_grade;
        this.aspect_x = aspect_x;
        this.aspect_y = aspect_y;
        this.body = body;
        
        
        this.scribbles = scribbles;
        this.corrections = corrections;
        notes=new Vector();
    }
    
    public Blaise_Comment(){
        this.commentID = "";
        this.paperID = "";
        this.graderID = "";
        this.draftID = "";
        this.isPublic = "";
        this.grader_name = "";
        //this.assigned_grade = "";
        this.aspect_x = "";
        this.aspect_y = "";
        this.body = "";
        scribbles=new Vector();
        corrections=new Vector();
        notes=new Vector();
    }
    
    /** Gets Comment ID */
    public String getCommentID(){ return commentID;}
    /** Sets Comment ID */
    public void setCommentID(String id){commentID = id;}
    
    /** Gets Paper ID */
    public String getPaperID(){ return paperID;}
    /** Sets Paper ID */
    public void setPaperID(String id){paperID = id;}
    
    /** Gets graderID */
    public String getGraderID(){return graderID;}
    /** Sets graderID */
    public void setGraderID(String id){graderID = id;}
    
    /** Gets draftID */
    public String getDraftID(){return draftID;}
    /** Sets draftID */
    public void setDraftID(String id){draftID = id;}
    
    /** Gets isPublic */
    public String getIsPublic(){return isPublic;}
    /** Sets isPublic */
    public void setIsPublic(String isPub){isPublic = isPub;}
    
    /** Gets grader name */
    public String getGraderName(){return grader_name;}
    /** Sets grader name */
    public void setGraderName(String name){grader_name = name;}
    
    /** Gets grade */
    public String getAssignedGrade(){return assigned_grade;}
    /** Sets grade */
    public void setAssignedGrade(String grade){assigned_grade = grade;}
    
    /** Gets Aspect X */
    public String getAspectX(){return aspect_x;}
    /** Sets Aspect X */
    public void setAspectX(String x){aspect_x = x;}
    
    /** Gets Aspect Y */
    public String getAspectY(){return aspect_y;}
    /** Sets Aspect Y */
    public void setAspectY(String y){aspect_y = y;}
    
    /** Gets Body */
    public String getBody(){return body;}
    /** Sets Body */
    public void setBody(String body){this.body = body;}
    
    /** Gets Scribbles */
    public Vector getScribbles(){
        return scribbles;
    }
    /** Sets Scribbles */
    public void setScribbles(Vector scribs){
        scribbles = scribs;
    }
    
    /** Gets Correction Sets */
    public Vector getCorrections(){
        return corrections;
    }
    /** Sets Correction Sets */
    public void setCorrections(Vector corrects){
        corrections = corrects;
    }
    
    /** Gets a Vector of Note objects*/
    public Vector getNotes(){
        return notes;
    }
    /** Sets the Notes Vector*/
    public void setNotes(Vector notes){
        this.notes=notes;
    }
    
    
    /*--------
     * External Body
     *--------*/
    public boolean hasExternalBody(){
        return hasExternalBody;
    }
    
    public void removeExternalBody(){
        hasExternalBody=false;
        externalBody=null;
    }
    
    /**
     * If no external body has been specified,
     * null is returned.  Otherwise the filename is returned.
     */
    public String getExternalBodyFilename(){
        if(externalBody==null){
            return null;
        }
        return externalBody.getFilename();
    }
    
    /**
     * If no external body has been specified,
     * a new ExternalFile is instantiated.
     *
     * This method is only really needed if you're storing the external body
     * internally as a byte[].  The File link method fills this in automatically.
     * @see ExternalFile
     */
    public void setExternalBodyFilename(String fn){
        if(externalBody==null){
            externalBody=new ExternalFile();
        }
        externalBody.setFilename(fn);
    }
    
    /** returns a byte array containing the external file
     * returns null if there is no external body, or if it is a link only.
     */
    public byte[] getExternalBody(){
        if(externalBody==null){
            return null;
        }
        return externalBody.getByteArray();
    }
    
    /** sets the external file.  This will make the body be 
     * embedded into this Blaise_Draft.
     */
    public void setExternalBody(byte[] body){
        if(externalBody==null){
            externalBody=new ExternalFile();//hopefully they'll fill in the filename soon.
        }
        externalBody.setByteArray(body);
        hasExternalBody=true;
    }
    
    /** returns a File linking to the external file in the filesystem.
     * returns null if there is no external body, or if it is embedded only.
     */
    public File getExternalBodyLink(){
        if(externalBody==null){
            return null;
        }
        return externalBody.getFileReference();
    }
    
    /** Sets the link to an external file.*/
    public void setExternalBodyLink(File link){
        if(externalBody==null){
            externalBody=new ExternalFile();
        }
        externalBody.setFilename(link.getName());
        externalBody.setFileReference(link);
        hasExternalBody=true;
    }
    
    /**
     * Adds the ExternalFile to the end of the external files vector.
     */
    public void addExternalFile(ExternalFile ef){
        if(externalFiles==null){
            externalFiles=new Vector();
        }
        externalFiles.add(ef);
    }
    /**
     * Replaces the current Vector holding external files
     * with Vector v.  Returns the old Vector (maybe null).
     */
    public Vector setExternalFiles(Vector v){
        Vector temp=externalFiles;
        externalFiles=v;
        return temp;
    }
    /**
     * Returns the ExernalFile at index position.
     * @see java.util.Vector.elementAt
     */
    public ExternalFile getExternalFile(int index){
        if(externalFiles==null){
            return null;
        }
        return (ExternalFile)externalFiles.elementAt(index);
    }
    
    /**
     * Returns a reference to the external Files Vector.
     * may be null.
     */
    public Vector getExternalFiles(){
        return externalFiles;
    }

    
    /** Prints the Blaise_Comment to the screen (debugging) */
    public void print(){
        System.out.println("==========Blaise_Comment===========");
        System.out.println("CommentID: "+commentID);
        System.out.println("PaperID: "+paperID);
        System.out.println("GraderID: "+graderID);
        System.out.println("DraftID: "+draftID);
        
        System.out.println("Grader Name: "+grader_name);
        System.out.println("Assigned Grade: "+assigned_grade);
        System.out.println("isPublic: "+isPublic);
        System.out.println("AspectX: "+aspect_x);
        System.out.println("AspectY: "+aspect_y);
        System.out.println("Body: "+body);
        
        if(hasExternalBody()){
            System.out.println("External Body: "+externalBody);
        }
        
        int i,size;
        
        System.out.println("External Files:");
        if(externalFiles==null || externalFiles.size()==0){
            System.out.println("none");
        }
        else{
            size=externalFiles.size();
            for(i=0;i<size;i++){
                System.out.println(externalFiles.elementAt(i));
            }
        }
        
        if(scribbles!=null){
            size=scribbles.size();
            for(i=0;i<size;i++){
                ((Scribble)scribbles.elementAt(i)).print();
            }
        }
        
        if(corrections!=null){
            size=corrections.size();
            for(i=0;i<size;i++){
                ((CorrectionSet)corrections.elementAt(i)).print();
            }
        }
        
        System.out.println("=========End Blaise_Comment=========");
    }
}