package edu.gwu.blaise.version2.documents;

import java.util.*;
import java.io.*;
/**
 * A Blaise_Draft is the representation of draft.
 * A draft may be either a proper, native blaise draft, or
 * it can be an external blaise draft.  If it is an external draft
 * it can be either an embedded, a link, or both.  An embedded is
 * where the bytes are stored internally in a byte array.  A link
 * is where the external draft is stored via a File object link to
 * a file on the filesystem.
 *
 * @author Sean T. Hanlon
 * @author Michael Gaiman
 * @version 4/18/2004 (move to documents)
 */
public class Blaise_Draft implements ExternalFileContainer{
    
    private String draftID;		//Server-specific
    private String paperID;
    private String draft_number;
    private String title;
    private String body;
    private String turninDate;
    private String isModified;
    
    private Vector comments;	//Blaise_Comment
    
    private boolean hasExternalBody;
    private ExternalFile externalBody;
    
    private Vector externalFiles=null;//contains ExternalFile objects
    
    public Blaise_Draft(){
        draftID = "";
        paperID = "";
        draft_number = "";
        title = "";
        body = "";
        turninDate = "";
        comments = new Vector();
        isModified = "";
        
        hasExternalBody=false;
        externalBody = null;
    }
    
    /**
     * @depricated supports only a v1 Blaise_Draft.
     */
    public Blaise_Draft(String draftID, String paperID, String draft_number, String title, String body, String turninDate, Vector comments){
        
        this.draftID = draftID;
        this.paperID = paperID;
        this.draft_number = draft_number;
        this.title = title;
        this.body = body;
        this.turninDate = turninDate;
        this.comments = comments;
        isModified = "";
    }
    
    /** Gets Draft ID */
    public String getDraftID(){ return draftID;}
    /** Sets Draft ID */
    public void setDraftID(String id){draftID = id;}
    
    /** Gets Paper ID */
    public String getPaperID(){ return paperID;}
    /** Sets Paper ID */
    public void setPaperID(String id){paperID = id;}
    
    /** Gets Draft Number */
    public String getDraftNumber(){ return draft_number;}
    /** Sets Draft Number */
    public void setDraftNumber(String draft_numb){draft_number = draft_numb;}
    
    /** Gets Title */
    public String getTitle(){ return title;}
    /** Sets Title */
    public void setTitle(String title){this.title = title;}
    
    /** Gets Body */
    public String getBody(){ return body;}
    /** Sets Body */
    public void setBody(String body){this.body = body;}
    
    /** Gets turnin Date */
    public String getTurninDate(){ return turninDate;}
    /** Sets turnin Date */
    public void setTurninDate(String date){turninDate = date;}
    
    /** Gets isModified */
    public String getIsModified(){ return isModified;}
    /** Sets Creation Date */
    public void setIsModified(String mod){isModified = mod;}
    
    
	//----------------
	// Comments
	//----------------
    /** Gets the Comments */
    public Vector getComments(){return comments;}
    
    /** Sets the Comments */
    public void setComments(Vector comments){this.comments = comments;}
    
    /** Removes the comment of a given graderID */
    public Blaise_Comment removeComment(String graderName){
        
        for(int i=0;i<comments.size();i++){
            Blaise_Comment current = (Blaise_Comment)comments.elementAt(i);
            if((current.getGraderName()).equals(graderName)){
                comments.removeElementAt(i);
                return current;
            }
        }
        return null;
    }
    
    public int getCommentSize(){
        return comments.size();
    }
    
    /** Adds a Blaise_Comment to the set of comments for this draft*/
    public void addComment(Blaise_Comment new_comment){
        if(comments==null){
            comments=new Vector();
        }
        comments.addElement(new_comment);
    }
    
    /** Return true if the draft has comments */
    public boolean hasComments(){
        if(comments.size() > 0){
            return true;
        }
        else{
            return false;
        }
    }
    
    public boolean equals(Blaise_Draft in){
        if(draft_number == in.getDraftNumber()){
            return true;
        }
        else{
            return false;
        }
    }
    
    /*--------
     * External Body
     *--------*/
    
    public boolean hasExternalBody(){
        return hasExternalBody;
    }
    
    public void removeExternalBody(){
        hasExternalBody=false;
        externalBody=null;
    }
    
    /**
     * If no external body has been specified,
     * null is returned.  Otherwise the filename is returned.
     */
    public String getExternalBodyFilename(){
        if(externalBody==null){
            return null;
        }
        return externalBody.getFilename();
    }
    
    /**
     * If no external body has been specified,
     * a new ExternalFile is instantiated.
     *
     * This method is only really needed if you're storing the external body
     * internally as a byte[].  The File link method fills this in automatically.
     * @see ExternalFile
     */
    public void setExternalBodyFilename(String fn){
        if(externalBody==null){
            externalBody=new ExternalFile();
        }
        externalBody.setFilename(fn);
    }
    
    /** returns a byte array containing the external file
     * returns null if there is no external body, or if it is a link only.
     */
    public byte[] getExternalBody(){
        if(externalBody==null){
            return null;
        }
        return externalBody.getByteArray();
    }
    
    /** sets the external file.  This will make the body be 
     * embedded into this Blaise_Draft.
     */
    public void setExternalBody(byte[] body){
        if(externalBody==null){
            externalBody=new ExternalFile();//hopefully they'll fill in the filename soon.
        }
        externalBody.setByteArray(body);
        hasExternalBody=true;
    }
    
    /** returns a File linking to the external file in the filesystem.
     * returns null if there is no external body, or if it is embedded only.
     */
    public File getExternalBodyLink(){
        if(externalBody==null){
            return null;
        }
        return externalBody.getFileReference();
    }
    
    /** Sets the link to an external file.*/
    public void setExternalBodyLink(File link){
        if(externalBody==null){
            externalBody=new ExternalFile();
        }
        externalBody.setFilename(link.getName());
        externalBody.setFileReference(link);
        hasExternalBody=true;
    }
    
    /**
     * Adds the ExternalFile to the end of the external files vector.
     */
    public void addExternalFile(ExternalFile ef){
        if(externalFiles==null){
            externalFiles=new Vector();
        }
        externalFiles.add(ef);
    }
    /**
     * Replaces the current Vector holding external files
     * with Vector v.  Returns the old Vector (maybe null).
     */
    public Vector setExternalFiles(Vector v){
        Vector temp=externalFiles;
        externalFiles=v;
        return temp;
    }
    /**
     * Returns the ExernalFile at index position.
     * @see java.util.Vector.elementAt
     */
    public ExternalFile getExternalFile(int index){
        if(externalFiles==null){
            return null;
        }
        return (ExternalFile)externalFiles.elementAt(index);
    }
    
    /**
     * Returns a reference to the external Files Vector.
     * may be null.
     */
    public Vector getExternalFiles(){
        return externalFiles;
    }
    
    /** Prints the draft to the screen (debugging) */
    public void print(){
        System.out.println("************************** Blaise Draft ********************************");
        System.out.println("DraftID: "+draftID);
        System.out.println("PaperID: "+paperID);        
        System.out.println("Draft Number: "+draft_number);
        System.out.println("Title: "+title);
        System.out.println("Turnin Date: "+turninDate);
        System.out.println("isModified: "+isModified);
        System.out.println("Body: "+body);
        
        if(hasExternalBody()){
            System.out.println("External Body: "+externalBody);
        }
        
        int i,size;
        
        System.out.println("External Files:");
        if(externalFiles==null || externalFiles.size()==0){
            System.out.println("none");
        }
        else{
            size=externalFiles.size();
            for(i=0;i<size;i++){
                System.out.println(externalFiles.elementAt(i));
            }
        }
        
        size=comments.size();
        System.out.println("Comments:");
        for(i=0;i<size;i++){
            ((Blaise_Comment)comments.elementAt(i)).print();
        }
        
        System.out.println("************************* End Blaise Draft ******************************");
    }
}
