package edu.gwu.blaise.version2.documents;

import java.util.*;
import java.awt.*;

/**
* The Object which represents a Blaise Paper.  It contains a number of 
 * properties aswell as a Vector of Blaise_Draft Objects.
 *
 * @author Sean T. Hanlon
 * @author Michael Gaiman
 * @version 4/25/2004 (added v2 props)
 */
public class Blaise_Paper{
    
    private String blaise_version;
    private String paperID;
    private String writerID;
    private String universityID;
    private String courseID;
    
    private String first;
    private String last;
    private String student_identifier;
    private String email;
    private String dept;
    private String number;
    private String section;
    
    private String semester;
    private String year;
    
    private String assignID;
    private String assignNum;
    private String assignName;
    
    private Vector drafts;		//Blaise_Draft
    //private String workingBody;
    private Blaise_Draft workingDraft;
    
    /**
        * Constructor
     */
    public Blaise_Paper(){
        this.blaise_version = "";
        this.paperID = "";
        this.writerID = "";
        this.universityID = "";
        this.courseID = "";
        
        this.first = "";
        this.last = "";
        this.student_identifier = "";
        this.email = "";
        this.dept = "";
        this.number = "";
        this.section = "";
        
        this.semester= "";
        this.year="";
        
        assignID="";
        assignNum="";
        assignName="";
        
        this.drafts = new Vector();
        //this.working_body = "";
        workingDraft=new Blaise_Draft();
        
    }
    
    /**
     * Constructor
     * @depricated Supports only version1 properties.
     */
    public Blaise_Paper(String blaise_version, String paperID, String writerID, String universityID, String courseID, String first, String last, String student_identifier, String email, String dept, String number, String section, Vector drafts, String working_body){
        
        this.blaise_version = blaise_version;
        this.paperID = paperID;
        this.writerID = writerID;
        this.universityID = universityID;
        this.courseID = courseID;
        
        this.first = first;
        this.last = last;
        this.student_identifier = student_identifier;
        this.email = email;
        this.dept = dept;
        this.number = number;
        this.section = section;
        
        this.drafts = drafts;
        workingDraft=new Blaise_Draft();
        workingDraft.setBody(working_body);
        
        assignID="";
        assignNum="";
        assignName="";
    }
    
	//-----------------
	// Paper Info
	//-----------------
    /** Gets Version */
    public String getVersion(){ return blaise_version;}
    /** Sets Version */
    public void setVersion(String version){blaise_version = version;}
    
    /** Gets Paper ID */
    public String getPaperID(){ return paperID;}
    /** Sets Paper ID */
    public void setPaperID(String id){paperID = id;}
    
    /** Gets Writer ID */
    public String getWriterID(){ return writerID;}
    /** Sets Writer ID */
    public void setWriterID(String id){writerID = id;}
    
    /** Gets University ID */
    public String getUniversityID(){ return universityID;}
    /** Sets University ID */
    public void setUniversityID(String id){universityID = id;}
    
    /** Gets Course ID */
    public String getCourseID(){ return courseID;}
    /** Sets Course ID */
    public void setCourseID(String id){courseID = id;}
    
    /** 
     * A convenience, legacy method that calls getBody() on
     * the working draft object.
     */
    public String getWorkingBody(){ return workingDraft.getBody();}
    /** 
     * A convenience, legacy method that calls setBody() on
     * the working draft object.
     */
    public void setWorkingBody(String working){workingDraft.setBody(working);}
    
    /**
     * Sets the specified Blaise_Draft to be the workingDraft.
     * Note: no checking on the draft is done to see if it really makes a valid
     * working draft.  Be good.
     */
    public void setWorkingDraft(Blaise_Draft d){
        workingDraft=d;
    }
    
    /**
     * Returns the Blaise_Draft that is storing the information for the
     * current working draft.
     */
    public Blaise_Draft getWorkingDraft(){
        return workingDraft;
    }
    
	//---------------
	// Student Info
	//---------------
    /** Gets First Name */
    public String getFirst(){return first;}
    /** Sets First Name */
    public void setFirst(String name){first = name;}
    
    /** Gets Last Name */
    public String getLast(){return last;}
    /** Sets Last Name */
    public void setLast(String name){last = name;}
    
    /** Gets Student Number */
    public String getStudentIdentifier(){return student_identifier;}
    /** Sets Student Number */
    public void setStudentIdentifier(String number){student_identifier = number;}
    
    /** Gets Email of the writer */
    public String getEmail(){return email;}
    /** Sets Email of the writer */
    public void setEmail(String value){email = value;}
    
	//---------------
	// Course Info
	//---------------
    /** Gets Department */
    public String getDept(){return dept;}
    /** Sets Department */
    public void setDept(String dep){dept = dep;}
    
    /** Gets Course Number */
    public String getNumber(){return number;}
    /** Sets Course Number */
    public void setNumber(String value){number = value;}
    
    /** Gets Course Section */
    public String getSection(){return section;}
    /** Sets Course Section */
    public void setSection(String value){section = value;}
    
    /** Gets the semester of the course*/
    public String getSemester(){return semester;}
    /** Sets the semesteer of the course*/
    public void setSemester(String value){semester = value;}
    
    /** Gets the year of the course*/
    public String getYear(){return year;}
    /** Sets the year of the course*/
    public void setYear(String value){year = value;}
    
    public String getAssignmentNumber(){return assignNum;}
    public void setAssignmentNumber(String value){assignNum=value;}
    
    public String getAssignmentID(){return assignID;}
    public void setAssignmentID(String value){assignID=value;}
    
    public String getAssignmentName(){return assignName;}
    public void setAssignmentName(String value){assignName=value;}
	//----------------
	// Drafts
	//----------------
    /** Gets the Drafts */
    public Vector getDrafts(){return drafts;}
    
    public Blaise_Draft getDraft(int index){
        if(index >= drafts.size()){
            return null;
        }
        else{
            return (Blaise_Draft)drafts.elementAt(index);
        }
    }
    
    public Blaise_Draft getDraftNumber(int number){
        for(int i=0;i<drafts.size();i++){
            Blaise_Draft draft = (Blaise_Draft)drafts.elementAt(i);
            if(Integer.parseInt(draft.getDraftNumber()) == number){
                return draft;
            }
        }
        
        return null;
    }
    
    /** Sets the Drafts */
    public void setDrafts(Vector drafts){this.drafts = drafts;}
    
    /** Adds a draft */
    public void addDraft(Blaise_Draft draft){drafts.add(draft);}
    
    /** Returns index of next desired draft */
    public String draftCount(){return (Integer.toString(drafts.size()+1));}
    
    /** Returns true if drafts exist */
    public boolean hasDrafts(){
        if(drafts.size() != 0){
            return true;
        }
        else{
            return false;
        }
    }
    
    /** Removes all drafts */
    public void removeAllDrafts(){
        drafts = new Vector();
    }
    
    /** Returns the topic of the most recent draft */
    //USE getLastDraft().getTitle() instead.
    /*public String getMostRecentTitle(){
        if(drafts.size() == 0) return "";
        Blaise_Draft dr = (Blaise_Draft)drafts.elementAt(drafts.size()-1);
        return dr.getTitle();
    }*/
    
    public Blaise_Draft getLastDraft(){
        int size=drafts.size();
        if(size>0){
            return (Blaise_Draft)drafts.elementAt(size-1);
        }
        else{
            return null;
        }
    }
    
    /** Compares this paper to another passed paper */
    public boolean equals(Object o){
        Blaise_Paper p2=(Blaise_Paper)o;
        if(!(p2.getFirst()).equals(first)){return false;}
        else if(!(p2.getPaperID()).equals(paperID)){return false;}
        else if(!(p2.getLast()).equals(last)){return false;}
        else if(!(p2.getStudentIdentifier()).equals(student_identifier)){return false;}
        else if(!(p2.getEmail()).equals(email)){return false;}
        else if(!(p2.getDept()).equals(dept)){return false;}
        else if(!(p2.getNumber()).equals(number)){return false;}
        else if(!(p2.getSection()).equals(section)){return false;}
        
        return true;
    }
    
    /** Prints the paper to the screen (debugging) */
    public void print(){
        System.out.println("************************** Blaise Paper ********************************");
        System.out.println("Blaise Version: "+blaise_version);
        System.out.println("PaperID: "+paperID);
        System.out.println("WriterID: "+writerID);
        System.out.println("UniversityID: "+universityID);
        System.out.println("CourseID: "+courseID);
        
        System.out.println("First: "+first);
        System.out.println("Last: "+last);
        System.out.println("Email: "+email);
        System.out.println("Student Number: "+student_identifier);
        System.out.println("Dept: "+dept);
        System.out.println("Number: "+number);
        System.out.println("Section: "+section);
        System.out.println("Semester: "+semester);
        System.out.println("Year: "+year);
        
        System.out.println("AssignmentID: "+assignID);
        System.out.println("Assignment Number: "+assignNum);
        System.out.println("Assignment Name: "+assignName);
        
        System.out.println("Working Draft:");
        workingDraft.print();
        
        for(int i=0;i<drafts.size();i++){
            ((Blaise_Draft)drafts.elementAt(i)).print();
        }
        
        System.out.println("************************* End Blaise Paper ******************************");
    }
    
    /**Sets any draft comments with "" as graderID to the passed values*/
    public void setGraderInfo(String graderID, String username){
        for(int i=0;i<drafts.size();i++){
            Blaise_Draft draft = (Blaise_Draft)drafts.elementAt(i);
            Vector comments = draft.getComments();
            for(int j=0;j<comments.size();j++){
                Blaise_Comment comm = (Blaise_Comment)comments.elementAt(j);
                if(comm.getGraderID().equals("")){
                    comm.setGraderID(graderID);
                    comm.setGraderName(username);
                }
            }
        }
    }
    
    /**Clears the graderID and gradername fields to store the current graders ID*/
    public void clearGraderInfo(String graderID, String username){
        for(int i=0;i<drafts.size();i++){
            Blaise_Draft draft = (Blaise_Draft)drafts.elementAt(i);
            Vector comments = draft.getComments();
            for(int j=0;j<comments.size();j++){
                Blaise_Comment comm = (Blaise_Comment)comments.elementAt(j);
                if(comm.getGraderID().equals(graderID)){
                    comm.setGraderID("");
                    comm.setGraderName("My Marks");
                }
            }
        }
    }
    
    public boolean isEmpty(){
        if(paperID.equals("") && writerID.equals("") && first.equals("") && last.equals("")){
            return true;
        }
        else{
            return false;
        }
    }
}
