package edu.gwu.blaise.version2.documents;

import java.util.*;
import java.awt.*;
import java.io.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import edu.gwu.blaise.version2.util.*;

/**
 * The Blaise_XML generates and parses XML content for Blaise_Papers, Blaise_Drafts, and Blaise_Comments
 *
 * @deprecated Starting with blaise v2, the blaise file format has changed, use BlaiseFile.
 * @author Sean T. Hanlon
 * @version 4/18/2004 (move to documents)
 */
public class Blaise_XML {

	private boolean debug = false;

	private Document doc;
	private Vector documents;
	private Blaise_Draft current_draft;
	private Blaise_Comment current_comment;
	private StringBuffer xmlOutput;

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// XML Generation
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Prints a given Blaise_Paper to the given file
	 *
	 * @param filename Location to write XML representation to
	 * @param Blaise_Paper that are converted to XML
	 */
	public void printPaperToFile(String filename, Blaise_Paper paper) {

		File file = new File(filename);
		Vector vector = new Vector();
		vector.add(paper);
		printPaperToFile(filename, vector);
    }

	/**
	 * Prints a given Vector of Blaise_Papers to the given file
	 *
	 * @param filename Location to write XML representation to
	 * @param vector Blaise_Papers that are converted to XML
	 */
	public void printPaperToFile(String filename, Vector vector) {

		File file = new File(filename);

		try{
			//if file exists, delete it
			if(file.exists()){
				if(debug)System.out.println("Blaise_XML -- File Existed, Deleting Previous Version.");
				file.delete();
			}

			if(!file.createNewFile()){
				if(debug)System.out.println("Blaise_XML -- Could not create save file");
			}
			else{
				if(debug)System.out.println("Blaise_XML -- Writing XML to file");
				FileWriter out = new FileWriter(file,true);
				String value = paperToXml(vector);
				out.write(value,0,value.length());
				out.close();
			}
			if(debug)System.out.println("Finished Blaise_XML");
		}
		catch(IOException e){
			if(debug)System.out.println("Document_Generator -- IOException Found in writeToFile:" + e);
		}
    }

	/**
	 * Prints a given Blaise_Comment to the given file
	 *
	 * @param filename Location to write XML representation to
	 * @param gr Blaise_Comment that is converted to XML
	 */
	public void printCommentToFile(String filename, Blaise_Comment gr) {

		File file = new File(filename);

		try{
			//if file exists, delete it
			if(file.exists()){
				if(debug)System.out.println("Document_Generator -- File Existed, Deleting Previous Version.");
				file.delete();
			}

			if(!file.createNewFile()){
				if(debug)System.out.println("Document_Generator -- Could not create save file");
			}
			else{
				FileWriter out = new FileWriter(file,false);
				String value = commentToXml(gr);
				out.write(value,0,value.length());
				out.close();
			}
		}
		catch(IOException e){
			if(debug)System.out.println("Document_Generator -- IOException Found in writeToFile:" + e);
		}

    }

	/**
	 * Prints a given Blaise_Draft to the given file
	 *
	 * @param filename Location to write XML representation to
	 * @param gr Blaise_Draft that is converted to XML
	 */
	public void printDraftToFile(String filename, Blaise_Draft gr) {

		File file = new File(filename);

		try{
			//if file exists, delete it
			if(file.exists()){
				if(debug)System.out.println("Document_Generator -- File Existed, Deleting Previous Version.");
				file.delete();
			}

			if(!file.createNewFile()){
				if(debug)System.out.println("Document_Generator -- Could not create save file");
			}
			else{
				FileWriter out = new FileWriter(file,false);
				String value = draftToXml(gr);
				out.write(value,0,value.length());
				out.close();
			}
		}
		catch(IOException e){
			if(debug)System.out.println("Document_Generator -- IOException Found in writeToFile:" + e);
		}

    }

	/**
	 * Converts a Blaise_Paper to XML
	 *
	 * @param Blaise_Paper that is to be converted to XML
	 * @return <code>String</code> representation of Vector in XML
	 */
	public String paperToXml(Blaise_Paper document) {
		xmlOutput = new StringBuffer();
		Vector temp = new Vector();
		temp.add(document);

		return paperToXml(temp);
	}

	/**
	 * Converts a Vector of Blaise_Papers to XML
	 *
	 * @param vector Blaise_Papers that are converted to XML
	 * @return <code>String</code> representation of Vector in XML
	 */
	public String paperToXml(Vector documents) {
		xmlOutput = new StringBuffer();
		if(debug)System.out.print("Blaise_XML -- Generating Paper Tree...");
		generatePaperTree(documents);
        if(debug)System.out.println("Done.");
        if(debug)System.out.print("Blaise_XML -- Walking along tree...");
        long startTime = System.currentTimeMillis();
        walk(doc);
        double timeTaken = System.currentTimeMillis() - startTime;
        if(debug)System.out.println("Done. Took "+timeTaken+"ms");

		//return xmlOutput;
		return xmlOutput.toString();
	}

	/**
	 * Converts a Blaise_Comment to XML
	 *
	 * @param g Blaise_Comment that is converted to XML
	 * @return <code>String</code> representation of Blaise_Comment in XML
	 */
	public String commentToXml(Blaise_Comment g) {

		xmlOutput = new StringBuffer();
		generateCommentTree(g);
        walk(doc);

		return xmlOutput.toString();
	}

	/**
	 * Converts a Blaise_Draft to XML
	 *
	 * @param g Blaise_Draft that is converted to XML
	 * @return <code>String</code> representation of Blaise_Draft in XML
	 */
	public String draftToXml(Blaise_Draft g) {

		xmlOutput = new StringBuffer();
		generateDraftTree(g);
        walk(doc);

		return xmlOutput.toString();
	}

	//Takes a vector and generates an XML-tree
	private void generatePaperTree(Vector documents) {

		try {
            // create Document
			DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			doc = db.newDocument();

		} catch(ParserConfigurationException pce) {
			if(debug)System.out.println("A New Error " + pce);
        }

        Element root;			//Root of xml
        Element paper;			//Paper Tag

        // the top element
        root = doc.createElement("Blaise_Paper_Set");
		for(int i=0;i<documents.size();i++) {

			Blaise_Paper current = (Blaise_Paper)documents.elementAt(i);

			// init the xml element for the paper
			root.appendChild(doc.createTextNode("\n\t"));
			paper = doc.createElement("Blaise_Paper");

			//Fill Paper
			fillHeaderInfo(paper, current);
			fillDrafts(paper, current.getDrafts());
			root.appendChild(paper);
		}

		root.appendChild(doc.createTextNode("\n"));
        doc.appendChild(root);
    }

	//Takes a Blaise_Comment and generates an XML-tree
	private void generateCommentTree(Blaise_Comment comment) {

		try {
            // create Document
			DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			doc = db.newDocument();

		} catch(ParserConfigurationException pce) {
			if(debug)System.out.println("A New Error " + pce);
        }

		Vector temp = new Vector();
		temp.add(comment);
		fillComments(null,temp);

    }

	//Takes a Blaise_Comment and generates an XML-tree
	private void generateDraftTree(Blaise_Draft draft) {

		try {
            // create Document
			DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			doc = db.newDocument();

		} catch(ParserConfigurationException pce) {
			if(debug)System.out.println("A New Error " + pce);
        }

		Vector temp = new Vector();
		temp.add(draft);
		fillDrafts(null,temp);
    }

	private void fillHeaderInfo(Element node, Blaise_Paper paper){

		Element item;
		for(int i=0;i<13;i++){
			node.appendChild(doc.createTextNode("\n\t\t"));
			switch(i) {
				case 0:item = doc.createElement("blaise_version"); item.appendChild(doc.createTextNode(escape(paper.getVersion())));node.appendChild(item);break;
				case 1:item = doc.createElement("paperID"); item.appendChild(doc.createTextNode(escape(paper.getPaperID())));node.appendChild(item);break;
				case 2:item = doc.createElement("writerID"); item.appendChild(doc.createTextNode(escape(paper.getWriterID())));node.appendChild(item);break;
				case 3:item = doc.createElement("universityID"); item.appendChild(doc.createTextNode(escape(paper.getUniversityID())));node.appendChild(item);break;
				case 4:item = doc.createElement("courseID"); item.appendChild(doc.createTextNode(escape(paper.getCourseID())));node.appendChild(item);break;
				case 5:item = doc.createElement("first"); item.appendChild(doc.createTextNode(escape(paper.getFirst())));node.appendChild(item);break;
				case 6:item = doc.createElement("last"); item.appendChild(doc.createTextNode(escape(paper.getLast())));node.appendChild(item);break;
				case 7:item = doc.createElement("student_identifier"); item.appendChild(doc.createTextNode(escape(paper.getStudentIdentifier())));node.appendChild(item);break;
				case 8:item = doc.createElement("email"); item.appendChild(doc.createTextNode(escape(paper.getEmail())));node.appendChild(item);break;
				case 9:item = doc.createElement("dept"); item.appendChild(doc.createTextNode(escape(paper.getDept())));node.appendChild(item);break;
				case 10:item = doc.createElement("number"); item.appendChild(doc.createTextNode(escape(paper.getNumber())));node.appendChild(item);break;
				case 11:item = doc.createElement("section"); item.appendChild(doc.createTextNode(escape(paper.getSection())));node.appendChild(item);break;
				case 12:item = doc.createElement("working_body"); item.appendChild(doc.createTextNode(escape(paper.getWorkingBody())));node.appendChild(item);break;
			}
		}

		if(!paper.hasDrafts()){
			node.appendChild(doc.createTextNode("\n\t"));
		}
	}

	private void fillDrafts(Element node, Vector drafts){

		Element draft = doc.createElement("temp");
		Element item;

		for(int i=0;i<drafts.size();i++){
			if(node != null){node.appendChild(doc.createTextNode("\n\t\t"));}
			draft = doc.createElement("Blaise_Draft");
			Blaise_Draft current = (Blaise_Draft)drafts.elementAt(i);

			//Fill header fields of draft
			for(int j=0;j<7;j++){
				draft.appendChild(doc.createTextNode("\n\t\t\t"));
				switch(j) {
					case 0:item = doc.createElement("draftID"); item.appendChild(doc.createTextNode(escape(current.getDraftID())));draft.appendChild(item);break;
					case 1:item = doc.createElement("paperID"); item.appendChild(doc.createTextNode(escape(current.getPaperID())));draft.appendChild(item);break;
					case 2:item = doc.createElement("draft_number"); item.appendChild(doc.createTextNode(escape(current.getDraftNumber())));draft.appendChild(item);break;
					case 3:item = doc.createElement("title"); item.appendChild(doc.createTextNode(escape(current.getTitle())));draft.appendChild(item);break;
					case 4:item = doc.createElement("body"); item.appendChild(doc.createTextNode(escape(current.getBody())));draft.appendChild(item);break;
					//case 5:item = doc.createElement("creation_date"); item.appendChild(doc.createTextNode(escape(current.getCreationDate())));draft.appendChild(item);break;
					case 6:item = doc.createElement("isModified"); item.appendChild(doc.createTextNode(escape(current.getIsModified())));draft.appendChild(item);break;
				}
			}

			fillComments(draft, current.getComments());

			if(node != null){
				node.appendChild(draft);
			}
			else{
				doc.appendChild(draft);
			}
		}
	}

	private void fillComments(Element node, Vector comments){

		Element comment,item,correction,correction_item;

		for(int i=0;i<comments.size();i++){
			if(node!= null){node.appendChild(doc.createTextNode("\n\t\t\t"));}
			comment = doc.createElement("Blaise_Comment");
			Blaise_Comment current = (Blaise_Comment)comments.elementAt(i);

			//Fill header fields of comment
			for(int j=0;j<=9;j++){
				comment.appendChild(doc.createTextNode("\n\t\t\t\t"));
				switch(j) {
					case 0:item = doc.createElement("commentID"); item.appendChild(doc.createTextNode(escape(current.getCommentID())));comment.appendChild(item);break;
					case 1:item = doc.createElement("paperID"); item.appendChild(doc.createTextNode(escape(current.getPaperID())));comment.appendChild(item);break;
					case 2:item = doc.createElement("graderID"); item.appendChild(doc.createTextNode(escape(current.getGraderID())));comment.appendChild(item);break;
					case 3:item = doc.createElement("draftID"); item.appendChild(doc.createTextNode(escape(current.getDraftID())));comment.appendChild(item);break;
					case 4:item = doc.createElement("isPublic"); item.appendChild(doc.createTextNode(escape(current.getIsPublic())));comment.appendChild(item);break;
					case 5:item = doc.createElement("grader_name"); item.appendChild(doc.createTextNode(escape(current.getGraderName())));comment.appendChild(item);break;
					case 6:item = doc.createElement("assigned_grade"); item.appendChild(doc.createTextNode(escape(current.getAssignedGrade())));comment.appendChild(item);break;
					case 7:item = doc.createElement("aspect_x"); item.appendChild(doc.createTextNode(escape(current.getAspectX())));comment.appendChild(item);break;
					case 8:item = doc.createElement("aspect_y"); item.appendChild(doc.createTextNode(escape(current.getAspectY())));comment.appendChild(item);break;
					case 9:item = doc.createElement("body"); item.appendChild(doc.createTextNode(escape(current.getBody())));comment.appendChild(item);break;
				}
			}

			//Fill Scribbles
			Vector scribbles = current.getScribbles();
			for(int j=0;j<scribbles.size();j++){
				//node.appendChild(doc.createTextNode("\n\t\t\t\t"));
				comment.appendChild(doc.createTextNode("\n\t\t\t\t"));
				correction = doc.createElement("Scribble");
				Scribble scrib = (Scribble)scribbles.elementAt(j);

				//Fill scribble fields
				for(int k=0;k<=5;k++){
					correction.appendChild(doc.createTextNode("\n\t\t\t\t\t"));
					switch(k) {
						case 0:item = doc.createElement("width"); item.appendChild(doc.createTextNode(Integer.toString(scrib.getWidth())));correction.appendChild(item);break;
						case 1:item = doc.createElement("color"); item.appendChild(doc.createTextNode(escape(scrib.getColorString())));correction.appendChild(item);break;
						case 2:item = doc.createElement("offset"); item.appendChild(doc.createTextNode(Integer.toString(scrib.getOffset())));correction.appendChild(item);break;
						case 3:item = doc.createElement("xDist"); item.appendChild(doc.createTextNode(Double.toString(scrib.getXDistanceFromOffset())));correction.appendChild(item);break;
						case 4:item = doc.createElement("height"); item.appendChild(doc.createTextNode(Double.toString(scrib.getHeight())));correction.appendChild(item);break;
						case 5:item = doc.createElement("lines"); item.appendChild(doc.createTextNode(escape(scrib.getLineString())));correction.appendChild(item);break;
					}
				}
				comment.appendChild(correction);
			}

			//Fill Correction Sets
			Vector correct_sets = current.getCorrections();
			for(int j=0;j<correct_sets.size();j++){
				//node.appendChild(doc.createTextNode("\n\t\t\t\t"));
				comment.appendChild(doc.createTextNode("\n\t\t\t\t"));
				correction = doc.createElement("Correction_Set");
				CorrectionSet corr = (CorrectionSet)correct_sets.elementAt(j);

				//Fill correction set fields
				for(int k=0;k<=2;k++){
					correction.appendChild(doc.createTextNode("\n\t\t\t\t\t"));
					switch(k) {
						case 0:item = doc.createElement("type"); item.appendChild(doc.createTextNode(escape(corr.getType())));correction.appendChild(item);break;
						case 1:item = doc.createElement("corr_color"); item.appendChild(doc.createTextNode(escape(corr.getColorString())));correction.appendChild(item);break;
						case 2:item = doc.createElement("location"); item.appendChild(doc.createTextNode(escape(corr.getLocationString())));correction.appendChild(item);break;
					}
				}
				comment.appendChild(correction);
			}
			if(node != null){
				node.appendChild(comment);
			}
			else{
				doc.appendChild(comment);
			}
		}
	}


	//Walks along the XML tree
    private void walk(Node node) {

        int type = node.getNodeType();

		switch(type) {
            case Node.DOCUMENT_NODE: {
	            xmlOutput.append("<?xml version=\"1.0\" encoding=\""+"UTF-8" + "\"?>" + "\n");
	            break;
            }//end of document

            case Node.ELEMENT_NODE: {
                xmlOutput.append('<' + node.getNodeName());
                NamedNodeMap nnm = node.getAttributes();
                if(nnm != null ) {
                    int len = nnm.getLength() ;
                    Attr attr;
                    for ( int i = 0; i < len; i++ ) {
                        attr = (Attr)nnm.item(i);
                    	xmlOutput.append(' ');
                    	xmlOutput.append(attr.getNodeName());
                    	xmlOutput.append("=\"");
                    	xmlOutput.append(attr.getNodeValue());
                    	xmlOutput.append('"');
                    }
                }
                xmlOutput.append('>');
                break;
            }//end of element

            case Node.ENTITY_REFERENCE_NODE: {
               xmlOutput.append('&');
               xmlOutput.append(node.getNodeName());
               xmlOutput.append(';');
               break;
            }//end of entity

            case Node.CDATA_SECTION_NODE: {
				xmlOutput.append("<![CDATA[");
				xmlOutput.append(node.getNodeValue());
				xmlOutput.append("]]>");
				break;
            }

            case Node.TEXT_NODE: {
                xmlOutput.append(node.getNodeValue());
                break;
            }

			case Node.PROCESSING_INSTRUCTION_NODE: {
				xmlOutput.append("<?" + node.getNodeName());
				String data = node.getNodeValue();
				if ( data != null && data.length() > 0 ) {
					xmlOutput.append(' ');
					xmlOutput.append(data);
				}
				xmlOutput.append("?>");
				break;
			}

        }//end of switch

        //recurse
        for(Node child = node.getFirstChild(); child != null; child = child.getNextSibling()) {
            walk(child);
        }

        //without this the ending tags will miss
        if ( type == Node.ELEMENT_NODE ) {
        	xmlOutput.append("</" + node.getNodeName() + ">");
        }
    }

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// XML PARSING
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Start parsing XML in given filename from the root node
	 */
	public Vector parse(String filename) {
		try{
			// create Document
			DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();

			// start the parsing
			documents = new Vector();
			doc = db.parse(filename);
			traversePaperTree(doc);

			// return
			return documents;
		}
		catch(Exception e){
			System.out.println(e);
		}

		return new Vector();
	}

	/**
	 * Parse XML representation of a Blaise_Draft in given filename
	 */
	public Blaise_Draft parseDraft(String filename) {

		try{
			// create Document
			DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();

			// start the parsing
			documents = new Vector();
			doc = db.parse(filename);
			NodeList list = doc.getChildNodes();
			traverseDraftTree(list.item(0));

			// return
			return current_draft;
		}
		catch(Exception e){
			System.out.println(e);
		}

		return null;
	}

	/**
	 * Parse XML representation of a Blaise_Comment in given filename
	 */
	public Blaise_Comment parseComment(String filename) {

		try{
			// create Document
			DocumentBuilderFactory dbf =DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();

			// start the parsing
			documents = new Vector();
			doc = db.parse(filename);
			NodeList list = doc.getChildNodes();
			traverseCommentTree(list.item(0));

			// return
			return current_comment;
		}
		catch(Exception e){
			System.out.println(e);
		}

		return null;
	}

	private void traversePaperTree(Node node){

		int type = node.getNodeType();

        switch (type) {
        case Node.ATTRIBUTE_NODE:
            break;
        case Node.ELEMENT_NODE:

			//Start of an event
			if (node.getNodeName().equals("Blaise_Paper")) {

				if(debug)System.out.println("Node Found - Blaise_Paper");

				NodeList childList = node.getChildNodes();

				String blaise_version = "";
				String paperID = "";
				String writerID = "";
				String universityID = "";
				String courseID = "";

				String first = "";
				String last = "";
				String student_identifier = "";
				String email = "";
				String topic = "";
				String dept = "";
				String number = "";
				String section = "";
				Vector drafts = new Vector();
				String working_body = "";

				//Loop thru all the properties of the object
				for(int i=0;i<childList.getLength();i++) {

					Node childNode = childList.item(i);

					if (childNode.getNodeType() == Node.ELEMENT_NODE) {

						//Get the node info
						String name = childNode.getNodeName();

						if(name.equals("blaise_version")){
							if(debug)System.out.print("\tBlaise_Version=");
							if (childNode.getFirstChild() != null) {
								blaise_version = childNode.getFirstChild().getNodeValue();
							}
							else {
								blaise_version = "";
							}
							if(debug)System.out.println(blaise_version);
						}
						else if(name.equals("paperID")){
							if(debug)System.out.print("\tpaperID=");
							if (childNode.getFirstChild() != null) {
								paperID = childNode.getFirstChild().getNodeValue();
							}
							else {
								paperID = "";
							}
							if(debug)System.out.println(paperID);
						}
						else if(name.equals("writerID")){
							if(debug)System.out.print("\twriterID=");
							if (childNode.getFirstChild() != null) {
								writerID = childNode.getFirstChild().getNodeValue();
							}
							else {
								writerID = "";
							}
							if(debug)System.out.println(writerID);
						}
						else if(name.equals("universityID")){
							if(debug)System.out.print("\tuniversityID=");
							if (childNode.getFirstChild() != null) {
								universityID = childNode.getFirstChild().getNodeValue();
							}
							else {
								universityID = "";
							}
							if(debug)System.out.println(universityID);
						}
						else if(name.equals("courseID")){
							if(debug)System.out.print("\tcourseID=");
							if (childNode.getFirstChild() != null) {
								courseID = childNode.getFirstChild().getNodeValue();
							}
							else {
								courseID = "";
							}
							if(debug)System.out.println(courseID);
						}
						else if(name.equals("first")){
							if(debug)System.out.print("\tFirst=");
							if (childNode.getFirstChild() != null) {
								first = childNode.getFirstChild().getNodeValue();
							}
							else {
								first = "";
							}
							if(debug)System.out.println(first);
						}
						else if(name.equals("last")){
							if(debug)System.out.print("\tLast=");
							if (childNode.getFirstChild() != null) {
								last = childNode.getFirstChild().getNodeValue();
							}
							else {
								last = "";
							}
							if(debug)System.out.println(last);
						}
						else if(name.equals("student_identifier")){
							if(debug)System.out.print("\tstudent_identifier=");
							if (childNode.getFirstChild() != null) {
								student_identifier = childNode.getFirstChild().getNodeValue();
							}
							else {
								student_identifier = "";
							}
							if(debug)System.out.println(student_identifier);
						}
						else if(name.equals("email")){
							if(debug)System.out.print("\temail=");
							if (childNode.getFirstChild() != null) {
								email = childNode.getFirstChild().getNodeValue();
							}
							else {
								email = "";
							}
							if(debug)System.out.println(email);
						}
						else if(name.equals("dept")){
							if(debug)System.out.print("\tdept=");
							if (childNode.getFirstChild() != null) {
								dept = childNode.getFirstChild().getNodeValue();
							}
							else {
								dept = "";
							}
							if(debug)System.out.println(dept);
						}
						else if(name.equals("number")){
							if(debug)System.out.print("\tnumber=");
							if (childNode.getFirstChild() != null) {
								number = childNode.getFirstChild().getNodeValue();
							}
							else {
								number = "";
							}
							if(debug)System.out.println(number);
						}
						else if(name.equals("section")){
							if(debug)System.out.print("\tsection=");
							if (childNode.getFirstChild() != null) {
								section = childNode.getFirstChild().getNodeValue();
							}
							else {
								section = "";
							}
							if(debug)System.out.println(section);
						}
						else if(name.equals("working_body")){
							if(debug)System.out.print("\tworking_body=");
							if (childNode.getFirstChild() != null) {
								working_body = childNode.getFirstChild().getNodeValue();
							}
							else {
								working_body = "";
							}
							if(debug)System.out.println(working_body);
						}
						else if(name.equals("Blaise_Draft")){
							if(debug)System.out.println("\tFound Blaise_Draft Tag");

							current_draft = null;
							traverseDraftTree(childNode);
							if(current_draft != null){
								drafts.add(current_draft);
							}
						}
					}
				}

				Blaise_Paper current_paper = new Blaise_Paper(blaise_version, paperID, writerID, universityID, courseID, first, last, student_identifier, email, dept, number, section, drafts, working_body);
				documents.add(current_paper);
				if(debug)System.out.println("\tBlaise_Paper Added");
			} //Blaise_Paper Tag
			break;
        default:
            break;
        }

		//Recursively call children
        for (Node child = node.getFirstChild(); child != null; child = child.getNextSibling()) {
            traversePaperTree(child);
        }
	}

	private void traverseDraftTree(Node node){

		String draftID = "";
		String paperID = "";
		String draft_number = "";
		String title = "";
		String body = "";
		String creation_date = "";
		String isModified = "";
		Vector comments = new Vector();

		NodeList draftList = node.getChildNodes();

		//Loop through all properties of the draft object
		for(int j=0;j<draftList.getLength();j++) {
			Node draftNode = draftList.item(j);
			String value = draftNode.getNodeName();
			if(value.equals("draftID")){
				if(debug)System.out.print("\t\tdraftID=");
				if(draftNode.getFirstChild() != null) {
					draftID = draftNode.getFirstChild().getNodeValue();
				}
				else {
					draftID = "";
				}
				if(debug)System.out.println(draftID);
			}
			else if(value.equals("paperID")){
				if(debug)System.out.print("\t\tpaperID=");
				if(draftNode.getFirstChild() != null) {
					paperID = draftNode.getFirstChild().getNodeValue();
				}
				else {
					paperID = "";
				}
				if(debug)System.out.println(paperID);
			}
			else if(value.equals("draft_number")){
				if(debug)System.out.print("\t\tdraft_number=");
				if(draftNode.getFirstChild() != null) {
					draft_number = draftNode.getFirstChild().getNodeValue();
				}
				else {
					draft_number = "";
				}
				if(debug)System.out.println(draft_number);
			}
			else if(value.equals("title")){
				if(debug)System.out.print("\t\ttitle=");
				if(draftNode.getFirstChild() != null) {
					title = draftNode.getFirstChild().getNodeValue();
				}
				else {
					title = "";
				}
				if(debug)System.out.println(title);
			}
			else if(value.equals("body")){
				if(debug)System.out.print("\t\tbody=");
				if(draftNode.getFirstChild() != null) {
					body = draftNode.getFirstChild().getNodeValue();
				}
				else {
					body = "";
				}
				if(debug)System.out.println(body);
			}
			else if(value.equals("creation_date")){
				if(debug)System.out.print("\t\tcreation_date=");
				if(draftNode.getFirstChild() != null) {
					creation_date = draftNode.getFirstChild().getNodeValue();
				}
				else {
					creation_date = "";
				}
				if(debug)System.out.println(creation_date);
			}
			else if(value.equals("isModified")){
				if(debug)System.out.print("\t\tisModified=");
				if(draftNode.getFirstChild() != null) {
					isModified = draftNode.getFirstChild().getNodeValue();
				}
				else {
					isModified = "";
				}
				if(debug)System.out.println(isModified);
			}
			else if(value.equals("Blaise_Comment")){
				if(debug)System.out.println("\tFound Blaise_Comment Tag");

				current_comment = null;
				traverseCommentTree(draftNode);
				if(current_comment != null){
					comments.add(current_comment);
				}
			}
		}

	    current_draft = new Blaise_Draft(draftID, paperID, draft_number, title, body, creation_date, comments);
		current_draft.setIsModified(isModified);
	}

	private void traverseCommentTree(Node node){
		String commentID = "";
		String paperID = "";
		String graderID = "";
		String draftID = "";
		String isPublic = "";
		String grader_name = "";
		String assigned_grade = "";
		String aspectX = "";
		String aspectY = "";
		String body = "";

		Vector scribbles = new Vector();
		Vector corrections = new Vector();

		NodeList commentList = node.getChildNodes();

		//Loop through all properties of the draft object
		for(int j=0;j<commentList.getLength();j++) {
			Node commentNode = commentList.item(j);
			String value = commentNode.getNodeName();

			if(value.equals("commentID")){
				if(debug)System.out.print("\t\t\tcommentID=");
				if(commentNode.getFirstChild() != null) {
					commentID = commentNode.getFirstChild().getNodeValue();
				}
				else {
					commentID = "";
				}
				if(debug)System.out.println(commentID);
			}
			else if(value.equals("paperID")){
				if(debug)System.out.print("\t\t\tpaperID=");
				if(commentNode.getFirstChild() != null) {
					paperID = commentNode.getFirstChild().getNodeValue();
				}
				else {
					paperID = "";
				}
				if(debug)System.out.println(paperID);
			}
			else if(value.equals("graderID")){
				if(debug)System.out.print("\t\t\tgraderID=");
				if(commentNode.getFirstChild() != null) {
					graderID = commentNode.getFirstChild().getNodeValue();
				}
				else {
					graderID = "";
				}
				if(debug)System.out.println(graderID);
			}
			else if(value.equals("draftID")){
				if(debug)System.out.print("\t\t\tdraftID=");
				if(commentNode.getFirstChild() != null) {
					draftID = commentNode.getFirstChild().getNodeValue();
				}
				else {
					draftID = "";
				}
				if(debug)System.out.println(draftID);
			}
			else if(value.equals("isPublic")){
				if(debug)System.out.print("\t\t\tisPublic=");
				if(commentNode.getFirstChild() != null) {
					isPublic = commentNode.getFirstChild().getNodeValue();
				}
				else {
					isPublic = "";
				}
				if(debug)System.out.println(isPublic);
			}
			else if(value.equals("grader_name")){
				if(debug)System.out.print("\t\t\tgrader_name=");
				if(commentNode.getFirstChild() != null) {
					grader_name = commentNode.getFirstChild().getNodeValue();
				}
				else {
					grader_name = "";
				}
				if(debug)System.out.println(grader_name);
			}
			else if(value.equals("assigned_grade")){
				if(debug)System.out.print("\t\t\tassigned_grade=");
				if(commentNode.getFirstChild() != null) {
					assigned_grade = commentNode.getFirstChild().getNodeValue();
				}
				else {
					assigned_grade = "";
				}
				if(debug)System.out.println(assigned_grade);
			}
			else if(value.equals("aspect_x")){
				if(debug)System.out.print("\t\t\taspectX=");
				if(commentNode.getFirstChild() != null) {
					aspectX = commentNode.getFirstChild().getNodeValue();
				}
				else {
					aspectX = "";
				}
				if(debug)System.out.println(aspectX);
			}
			else if(value.equals("aspect_y")){
				if(debug)System.out.print("\t\t\taspectY=");
				if(commentNode.getFirstChild() != null) {
					aspectY = commentNode.getFirstChild().getNodeValue();
				}
				else {
					aspectY = "";
				}
				if(debug)System.out.println(aspectY);
			}
			else if(value.equals("body")){
				if(debug)System.out.print("\t\t\tbody=");
				if(commentNode.getFirstChild() != null) {
					body = commentNode.getFirstChild().getNodeValue();
				}
				else {
					body = "";
				}
				if(debug)System.out.println(body);
			}
			else if(value.equals("Scribble")){
				if(debug)System.out.println("\t\t\tFound Scribble Tag");

				Scribble current_scrib = traverseScribbleTree(commentNode);
				scribbles.add(current_scrib);
			}
			else if(value.equals("Correction_Set")){
				if(debug)System.out.println("\t\t\tFound Correction_Set Tag");

				CorrectionSet current_correct = traverseCorrectionSetTree(commentNode);
				corrections.add(current_correct);
			}
		}

		current_comment = new Blaise_Comment(commentID, paperID, graderID, draftID, isPublic, grader_name, assigned_grade, aspectX, aspectY, body, scribbles, corrections);
	}

	private Scribble traverseScribbleTree(Node node){

		int width = 0;
		Color color = new Color(0,0,0);
		int offset = 0;
		double height = 0.0;
		double xDist = 0.0;
		Vector lines = new Vector();

		NodeList scribList = node.getChildNodes();
		for(int k=0;k<scribList.getLength();k++){
			Node scribNode = scribList.item(k);
			String scrib_val = scribNode.getNodeName();
			//Set color
			if(scrib_val.equals("color")){
				if(debug)System.out.print("\t\t\t\tscrib_color=");
				if (scribNode.getFirstChild() != null) {

					//Parse color to acquire RGB values
					String col = scribNode.getFirstChild().getNodeValue();
					StringTokenizer tok = new StringTokenizer(col, "|");
					String red = (String)tok.nextToken();
					String green = (String)tok.nextToken();
					String blue = (String)tok.nextToken();

					int r = Integer.parseInt(red);
					int g = Integer.parseInt(green);
					int b = Integer.parseInt(blue);

					color = new Color(r,g,b);
				}
				else {
					color = new Color(0,0,0);
				}
				if(debug)System.out.println(color.getRed()+","+color.getGreen()+","+color.getBlue());
			}
			//Parse and acquire width of scribble
			else if(scrib_val.equals("width")){
				if(debug)System.out.print("\t\t\t\twidth=");
				if (scribNode.getFirstChild() != null) {
					width = Integer.parseInt(scribNode.getFirstChild().getNodeValue());
				}
				else {
					width = 1;
				}
				if(debug)System.out.println(width);
			}
			//Acquire offset
			else if(scrib_val.equals("offset")){
				if(debug)System.out.print("\t\t\t\toffset=");
				if (scribNode.getFirstChild() != null) {
					offset = Integer.parseInt(scribNode.getFirstChild().getNodeValue());
				}
				else {
					offset = 0;
				}
				if(debug)System.out.println(offset);
			}
			//Acquire height
			else if(scrib_val.equals("height")){
				if(debug)System.out.print("\t\t\t\theight=");
				if (scribNode.getFirstChild() != null) {
					height = Double.parseDouble(scribNode.getFirstChild().getNodeValue());
				}
				else {
					height = 0.0;
				}
				if(debug)System.out.println(height);
			}
			//Acquire X-Distance
			else if(scrib_val.equals("xDist")){
				if(debug)System.out.print("\t\t\t\txDist=");
				if (scribNode.getFirstChild() != null) {
					xDist = Double.parseDouble(scribNode.getFirstChild().getNodeValue());
				}
				else {
					xDist = 0.0;
				}
				if(debug)System.out.println(xDist);
			}
			//Acquire and parse all line of scribble
			else if(scrib_val.equals("lines")){
				if(debug)System.out.print("\t\t\t\tParsing Lines...");
				if (scribNode.getFirstChild() != null) {

					String temp = scribNode.getFirstChild().getNodeValue();
					StringTokenizer tok1 = new StringTokenizer(temp,"#");
					double prev_x = -1;
					double prev_y = -1;
					while(tok1.hasMoreTokens()){
						StringTokenizer tok2 = new StringTokenizer((String)tok1.nextElement(), "*");
						if(prev_x == -1){
							prev_x = Double.parseDouble((String)tok2.nextElement());
							prev_y = Double.parseDouble((String)tok2.nextElement());
						}
						else{
							double new_x = Double.parseDouble((String)tok2.nextElement());
							double new_y = Double.parseDouble((String)tok2.nextElement());
							lines.add(new ScribbleLine(prev_x, prev_y, new_x, new_y));
							prev_x = new_x;
							prev_y = new_y;
						}
					}
				}
				if(debug)System.out.println("Found "+lines.size()+" Lines.");
			}
		}

		return new Scribble(width, color, offset, height, xDist, lines);
	}

	private CorrectionSet traverseCorrectionSetTree(Node node){

		String type = "";
		Color color = new Color(0,0,0);
		Vector starts = new Vector();
		Vector ends = new Vector();

		NodeList corrList = node.getChildNodes();
		for(int k=0;k<corrList.getLength();k++){
			Node corrNode = corrList.item(k);
			String corr_val = corrNode.getNodeName();
			//Type
			if(corr_val.equals("type")){
				if(debug)System.out.print("\t\t\t\ttype=");
				if (corrNode.getFirstChild() != null) {
					type = corrNode.getFirstChild().getNodeValue();
				}
				else {
					type = "";
				}
				if(debug)System.out.println(type);
			}
			//Color
			else if(corr_val.equals("corr_color")){
				if(debug)System.out.print("\t\t\t\tcorr_color=");
				if (corrNode.getFirstChild() != null) {
					String col = corrNode.getFirstChild().getNodeValue();
					StringTokenizer tok = new StringTokenizer(col, "|");
					String red = (String)tok.nextToken();
					String green = (String)tok.nextToken();
					String blue = (String)tok.nextToken();

					int r = Integer.parseInt(red);
					int g = Integer.parseInt(green);
					int b = Integer.parseInt(blue);
					color = new Color(r,g,b);
				}
				else {
					color = new Color(0,0,0);
				}
				if(debug)System.out.println(color.getRed()+","+color.getGreen()+","+color.getBlue());
			}
			//Locations
			else if(corr_val.equals("location")){
				if(debug)System.out.print("\t\t\t\tParsing Locations...");
				if (corrNode.getFirstChild() != null) {

					//Parse all start-end pairs
					String temp = corrNode.getFirstChild().getNodeValue();
					StringTokenizer tok1 = new StringTokenizer(temp,"|");
					while(tok1.hasMoreTokens()){
						StringTokenizer tok2 = new StringTokenizer((String)tok1.nextElement(), ",");
						starts.add(new Integer((String)tok2.nextElement()));
						ends.add(new Integer((String)tok2.nextElement()));
					}
				}
				if(debug)System.out.println("Found "+starts.size()+" Sets.");
			}
		}

		return new CorrectionSet(type,color,starts,ends);
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	// UTILITIES
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Replaces escape characters
	 *
	 * @param given The String to have escape characters replaced
	 */
	public String escape(String given){

		StringBuffer buffer = new StringBuffer();

		for(int i=0;i<given.length();i++){
			char at = given.charAt(i);
			if(at == '<'){
				buffer.append("&lt;");
			}
			else if(at == '>'){
				buffer.append("&gt;");
			}
			else if(at == '&'){
				buffer.append("&amp;");
			}
			else if(at == '\''){
				buffer.append("&apos;");
			}
			else if(at == '\"'){
				buffer.append("&quot;");
			}
			else{
				buffer.append(at);
			}
		}

		return buffer.toString();
	}

	public static void main(String[] argv){

		//Papers
		Blaise_Paper paper1 = new Blaise_Paper("version 1", "1", "2", "3", "4", "Sean", "Hanlon", "117682973", "seanh@gwu.edu", "CSCI", "141", "10", new Vector(), "This is the working area.");
		Blaise_Paper paper2 = new Blaise_Paper("version 1", "1", "2", "3", "4", "Sean", "Hanlon", "117682973", "seanh@gwu.edu", "CSCI", "141", "10", new Vector(), "This is the working area.");

		//Drafts
		Blaise_Draft draft1 = new Blaise_Draft("1","2","3","This is the title of my draft","This is the body of my draft.","05-25-03",new Vector());
		Blaise_Draft draft2 = new Blaise_Draft("1","2","3","This is the title of my draft","This is the body of my draft.","05-25-03",new Vector());
		draft1.setIsModified("YES");

		//Comments
		Scribble scrib1 = new Scribble(1, new java.awt.Color(2,3,4),5,6,7);
		scrib1.addLine(0,0,1,1);
		scrib1.addLine(1,1,2,2);
		scrib1.addLine(2,2,3,3);

		CorrectionSet correct1 = new CorrectionSet(new java.awt.Color(1,2,3), "Test Type");
		correct1.add(1,2);
		correct1.add(3,4);


		Vector scribbles = new Vector();
		scribbles.add(scrib1);
		scribbles.add(scrib1);

		Vector corrects = new Vector();
		corrects.add(correct1);
		corrects.add(correct1);

		Blaise_Comment comment1 = new Blaise_Comment("1","2","3","4","YES","Lindeman","A","5","6","This is the body",scribbles, corrects);

		draft1.addComment(comment1);
		draft1.addComment(comment1);

		paper1.addDraft(draft1);
		paper1.addDraft(draft2);

		Vector papers = new Vector();
		papers.add(paper1);
		papers.add(paper2);
		//papers.add(paper1);

		Blaise_XML xml = new Blaise_XML();
		//String out = xml.paperToXml(papers);
		//String out = xml.draftToXml(draft1);
		//String out = xml.commentToXml(comment1);
		//System.out.println(out);

		xml.printPaperToFile("D:\\Blaise\\data\\temp.file", papers);
		//xml.printDraftToFile("D:\\Blaise\\data\\temp.file", draft1);
		//xml.printCommentToFile("E:\\Blaise\\data\\temp.file", comment1);

		Vector result_papers = xml.parse("D:\\Blaise\\data\\temp.file");
		((Blaise_Paper)result_papers.elementAt(0)).print();

		//Blaise_Draft dr = xml.parseDraft("E:\\Blaise\\data\\temp.file");
		//dr.print();

		//Blaise_Comment com = xml.parseComment("E:\\Blaise\\data\\temp.file");
		//com.print();
	}
}
