package edu.gwu.blaise.version2.documents;
import java.io.*;
import edu.gwu.blaise.version2.util.*;
/**
 * Causes external documents to be opened in their native programs.
 * This is a platform independent interface, the actual work
 * is done in package private classes.
 * @author Michael Gaiman
 * @version 5/24/2004
 */
public class DocumentOpener {
    private File doc;
    private OpenerInterface opener;
    private String msg;
    
    public DocumentOpener(File document){
        doc=document;
        if(Constants.isOSX()){
            opener=new OSXOpener();
        }
        else if(Constants.isWindows()){
            opener=new WindowsOpener();
        }
        else{
            throw new IllegalStateException("Cannot open documents on unsupported platforms.");
        }
    }
    
    /**
     * Attempts to open the file in it's default application.
     * @returns true if successful, false otherwise.  If it was false
     * the getErrorMsg() will return the cause (if any).
     */
    public boolean open(){
        try{
            if((msg=opener.open(doc))==null){
                return true;
            }
            else{
                return false;
            }
        }catch(IOException e){
            msg=e.toString();
            return false;
        }
    }
    
    /**
     * Returns any failure message incurred when the file failed to be opened.
     * Null if none exist.
     */
    public String getErrorMsg(){
        return msg;
    }
}
