package edu.gwu.blaise.version2.documents;
import java.io.File;
/**
 * ExternalFile represents an external file of some sort.
 * It does this in a combination of three ways.
 * An external file can either be stored internally as a byte array or as
 * an Object,
 * or a File reference is kept to the file on the filesystem. (or a combination of the three).
 *
 * This class is really just a simple container.
 *
 * Note: As per BlaiseFile.addEFContainer() the order of serialization for ExternalFile is as follows:
 * First it will try the file reference.  If that is null, it will try the byte array.  If that is also null,
 * it will try to serialize the Object.
 * 
 * @author Michael Gaiman
 * @version 5/13/2004
 */
public class ExternalFile {
    private String filename;
    private File fileReference;
    private byte[] byteArray;
    private Object object;
    
    
    /**
     * An empty ExternalFile object
     */
    public ExternalFile(){
    }
    
    public ExternalFile(String filename, byte[] byteArray, Object object, File fileReference){
        this.filename=filename;
        this.byteArray=byteArray;
        this.object=object;
        this.fileReference=fileReference;
    }
    
    public void setFilename(String filename){
        this.filename=filename;
    }
    
    public String getFilename(){
        return filename;
    }
    
    public void setByteArray(byte[] ba){
        byteArray=ba;
    }
    
    public byte[] getByteArray(){
        return byteArray;
    }
    
    public void setObject(Object o){
        object=o;
    }
    
    public Object getObject(){
        return object;
    }
    
    public void setFileReference(File ref){
        fileReference=ref;
    }
    
    public File getFileReference(){
        return fileReference;
    }
    
    public String toString(){
        StringBuffer sb=new StringBuffer("[externalFile ");
        sb.append(filename);
        sb.append(": {");
        if(byteArray==null){
            sb.append("null} ");
        }
        else{
            for(int i=0;i<byteArray.length;i++){
                sb.append(byteArray[i]);
                sb.append(" ");
            }
            sb.append("} ");
        }
        sb.append(object);
        sb.append(" ");
        sb.append(fileReference);
        sb.append("]");
        return sb.toString();
    }
}
