package edu.gwu.blaise.version2.documents;
import java.io.*;
import java.util.*;
/**
 * An interface that Blaise_Draft and Blaise_Comment objects implement, that 
 * gives access to their possible external bodies and all of the embedded files.
 * @author Michael Gaiman
 * @version 5/20/2004
 */
public interface ExternalFileContainer {
    /**
     * @returns true when this Container has an external body.  false otherwise.
     */
    public boolean hasExternalBody();
    
    /**
     * removes any external body this container contains.
     */
    public void removeExternalBody();
    
    /**
     * If no external body has been specified,
     * null is returned.  Otherwise the filename is returned.
     */
    public String getExternalBodyFilename();
    
    /** returns a byte array containing the external file
     * returns null if there is no external body, or if it is a link only.
     */
    public byte[] getExternalBody();
    
    /** sets the external file.  This will make the body be 
     * embedded into this Blaise_Draft.
     */
    public void setExternalBody(byte[] body);
    
    /** returns a File linking to the external file in the filesystem.
     * returns null if there is no external body, or if it is embedded only.
     */
    public File getExternalBodyLink();
    
    /** Sets the link to an external file.*/
    public void setExternalBodyLink(File link);
    
    /**
     * Adds the ExternalFile to the end of the external files vector.
     */
    public void addExternalFile(ExternalFile ef);
    /**
     * Replaces the current Vector holding external files
     * with Vector v.  Returns the old Vector (maybe null).
     */
    public Vector setExternalFiles(Vector v);
    /**
     * Returns the ExernalFile at index position.
     * @see java.util.Vector.elementAt
     */
    public ExternalFile getExternalFile(int index);
    
    /**
     * Returns a reference to the external Files Vector.
     * may be null.
     */
    public Vector getExternalFiles();
}
