package edu.gwu.blaise.version2.documents;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.*;
import java.beans.PropertyVetoException;
import edu.gwu.blaise.version2.util.*;
/**
 * The Note class serves as both the model and the view for Blaise's
 * Note grading construct.
 *
 * @author Michael Gaiman
 * @version 1 9/28/2004
 */
public class Note{
    /**Pointer to the NoteFrame for this Note*/
    private NoteFrame myFrame=null;
    private NoteIcon myIcon=null;
    private NoteControllerInterface controller;
    
    /*Note: the Note class is used for two purposes.  It is used to
     *Store the settings that make up a note (Model) and it is used to
     *display those settings as a Note, via the NoteFrame.
     *As such, when in model mode only, the calls to get methods
     *return stored values, whereas when in view mode they return the
     *live values (found via additional method calls).
     *The switch between model and view modes is made when myFrame is set
     *to non-null.
     */
    private String nID;
    private Point iconPoint;
    private Color nColor;
    private String text;
    private boolean open;
    private String title;
    private Rectangle frameRectangle;
    
    /**
     * Creates a new, editable, Note.
     */
    public Note(NoteControllerInterface controller, String nID, Point iconPoint, Color nColor, boolean open){
        this(nID,iconPoint,nColor,"Note "+nID,new Rectangle(iconPoint.x,iconPoint.y,200,200),"",true);
        System.out.println("New note at "+iconPoint);
        display(controller,true);
    }
    
    /**
     * Constructs a pre-existing Note ready for display, and displays that Note.
     * The note is displayed in a non-editable fashion.
     */
    public Note(NoteControllerInterface controller, String nID, Point iconPoint, Color nColor, 
        String title, Rectangle frameRectangle, String text, boolean open){

        this(nID,iconPoint,nColor,title,frameRectangle,text,open);
        display(controller,false);
    }
    
    /**
     * Constructs a pre-existing Note, for storage.  To display, call the display method.
     */
    public Note(String nID,Point iconPoint, Color nColor, 
        String title, Rectangle frameRectangle, String text, boolean open){
        
        this.nID=nID;
        this.iconPoint=iconPoint;
        this.nColor=nColor;
        this.title=title;
        this.frameRectangle=frameRectangle;
        this.text=text;
        this.open=open;
    }
    
    /**
     * This is the view for the actual frame.
     */
    public class NoteFrame extends JInternalFrame implements InternalFrameListener{
        private JTextArea text=null;
        private Note parentNote;
        
        
        NoteFrame (Note parentNote,String title,String initialText,Color color,boolean editable){
            super (title, true, true, false, false);
            this.parentNote=parentNote;
            text=new JTextArea(initialText);
            text.setEditable(editable);
            text.setBackground(color);
            this.getContentPane().add(new JScrollPane(text));
            addInternalFrameListener(this);
        }
        
        private JTextComponent getTextComponent(){
            return text;
        }
        
        public void setOpen(boolean o){
            if(o){
                show();
            }
            else{
                hide();
            }
        }
        
        public Note getParentNote(){
            return parentNote;
        }
        
        public void internalFrameActivated (InternalFrameEvent e){
            controller.selectedAndToFront(this);
        }
        public void internalFrameClosed (InternalFrameEvent e){
            controller.noteClosed(this);
        }
        public void internalFrameClosing (InternalFrameEvent e){
        }
        public void internalFrameDeactivated (InternalFrameEvent e){
            try{
                setSelected(false);
            }catch(PropertyVetoException pve){
                System.out.println(pve);
            }
        }
        public void internalFrameDeiconified (InternalFrameEvent e){}
        public void internalFrameIconified (InternalFrameEvent e){}
        public void internalFrameOpened (InternalFrameEvent e){
            controller.noteOpened(this);
        }
    }
    
    /**
     * NoteIcon is the icon or collapsed view of a Note.
     * It contains a small icon with the NoteID.
     */
    public class NoteIcon extends JPanel{
        private Note parentNote;
        private ImageIcon icon;
        private String nID;
        private boolean noBorder=true;
        
        private NoteIcon(Note parentNote,String nID){
            //for the moment just use 1 standard icon...
            super(null);//do the layout ourselves...
            icon=new ImageIcon(Constants.getIconPath()+"note1.png");
            System.out.println("ImageIcon:"+icon);
            System.out.println("size:"+icon.getIconWidth()+" "+icon.getIconHeight());
            JLabel label=new JLabel(icon);
            int width=icon.getIconWidth(),height=icon.getIconHeight();
            label.setBounds(0,0,width,height);
            add(label);
            System.out.println("NoteIcon:"+this);
            setBounds(iconPoint.x,iconPoint.y,width,height);
            this.nID=nID;
            this.parentNote=parentNote;
            //arrange the size based upon the nID.
            //register with controller and paint on parent.
        }
        
        /**
         * Calls super.contains after converting the Point to be relative to the panel.
         */
        public boolean containsMousePoint(Point p){
            return super.contains(p.x-getX(),p.y-getY());
        }
        
        /**
         * Draws a border around this note icon
         */
        public void select(){
            if(noBorder){
                noBorder=false;
                this.setBorder(BorderFactory.createLineBorder(Color.black));
            }
        }
        
        /**
         * removes border around icon
         */
        public void deselect(){
            noBorder=true;
            this.setBorder(null);
        }
        
        public Note getParentNote(){
            return parentNote;
        }
        
        /*public void paintComponent (Graphics g)
        {
            super.paintComponent (g);
            //System.out.println ("NoteIcon.In paintComponent()");
            //icon.paintIcon (this, g, 0, 0);
            //point the nID on top.
        }*/

    }
    
    /**
     * Calling display transitions Note from a storage class to a display class.
     * This method should be called from the swing thread.
     * Ignores multiple call without first remove()ing it...
     */
    public void display(NoteControllerInterface controller,boolean editable){
        /*if(this.controller!=null){
            return;
        }*/
        this.controller=controller;
        
        myIcon=new NoteIcon(this,nID);
        myIcon.setLocation(iconPoint);
        myFrame=new NoteFrame(this,title,text,nColor,editable);
        myFrame.setBounds(frameRectangle);
        myFrame.setMaximumSize(new Dimension(400,400));
        myFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        controller.addNoteToPane(myFrame,myIcon);
        if(open){
            myFrame.show();
        }
    }
    
    /**
     * Reverses actions taken in display().
     */
    public void remove(){
        controller.removeNoteFromPane(myFrame,myIcon);
        myIcon.removeAll();
        myIcon=null;
        myFrame=null;
        this.controller=null;
    }
    
    /**
     * Called to transition Note from a display class to a storage class.
     */
    public void store(){
        //get the latest values...
        nColor=getNoteColor();
        iconPoint=getIconPoint();
        title=getTitle();
        frameRectangle=getFrameRectangle();
        text=getText();
        open=isOpen();
        
        myFrame.dispose();
        myFrame=null;
        myIcon=null;
    }
    
    public Color getNoteColor(){
        if(myFrame==null){
            return nColor;
        }
        else{
            return myFrame.getTextComponent().getBackground();
        }
    }
    
    public void setNoteColor(Color c){  
        nColor=c;
        if(myFrame!=null){
            myFrame.getTextComponent().setBackground(c);
        }
    }
    
    public String getNoteID(){
        return nID;
    }
    
    public Point getIconPoint(){
        if(myFrame==null){
            return iconPoint;
        }
        else{
            return myIcon.getLocation(null);
        }
    }
    
    public void setIconPoint(Point ip){
        if(myFrame==null){
            iconPoint=ip;
        }
        else{
            myIcon.setLocation(ip);
        }
    }
    
    public String getTitle(){
        if(myFrame==null){
            return title;
        }
        else{
            return myFrame.getTitle();
        }
    }
    
    public Rectangle getFrameRectangle(){
        if(myFrame==null){
            return frameRectangle;
        }
        else{
            return new Rectangle(myFrame.getX(),myFrame.getY(),myFrame.getWidth(),myFrame.getHeight());
        }
    }
    
    public String getText(){
        if(myFrame==null){
            return text;
        }
        else{
            return myFrame.getTextComponent().getText();
        }
    }
    
    public void setText(String t){
        if(myFrame==null){
            text=t;
        }
        else{
            myFrame.getTextComponent().setText(t);
        }
    }
    
    public boolean isOpen(){
        if(myFrame==null){
            return open;
        }
        else{
            return !myFrame.isClosed();
        }
    }
    
    public void setOpen(boolean o){
        if(myFrame==null){
            open=o;
        }
        else{
            myFrame.setOpen(o);
        }
    }
	
	public String toString(){
		return "[Note "+getNoteID()+": "+getText()+"]";
	}
    
}
