//LOC:326
package edu.gwu.blaise.version2.documents;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/**
* A Scribble is a customizable component of a PGS_Grade object that is drawn by the grader.
 * <p>
 * Scribbles are "smart" in that they are drawn relative to a given offset within the document. What is more, they are stored a certain height and x-distance from the offset.
 *
 * @author Sean T. Hanlon
 */
public class Scribble {
    
    private Vector gesture; //of ScribbleLine's
    private int width;		//width of scribble
    private Color color;	//color of scribble
    
	//Bounds of Scribble
    Rectangle2D.Double rect;
    public double minX;
    public double minY;
    public double maxX;
    public double maxY;
    
    private int offset;		//document index
    private double xDist;	//distance from offset
    private double height;	//heigth from index
    private Point2D.Double start;
    
    /** Constructor */
    public Scribble(){
        this(1, Color.RED,0,0);
    }
    
    /** Constructor */
    public Scribble(int wid, Color col, int off, double h){
        gesture = new Vector();
        width = wid;
        color = col;
        offset = off;
        height = h;
        
        minX = 2147483646;
        minY = 2147483646;
        maxX = -1;
        maxY = -1;
    }
    
    /** Constructor */
    public Scribble(int wid, Color col, int off, double h, double xD){
        gesture = new Vector();
        width = wid;
        color = col;
        offset = off;
        height = h;
        xDist = xD;
        
        minX = 2147483646;
        minY = 2147483646;
        maxX = -1;
        maxY = -1;
    }
    
    /** Constructor */
    public Scribble(int wid, Color col, int off, double h, double xD, Vector lines){
        gesture = new Vector();
        width = wid;
        color = col;
        offset = off;
        height = h;
        xDist = xD;
        
        minX = 2147483646;
        minY = 2147483646;
        maxX = -1;
        maxY = -1;
        
        int size=lines.size();
        for(int i=0;i<size;i++){
            addLine((ScribbleLine)lines.elementAt(i));
        }
    }
    
    /** Constructor */
    public Scribble(int wid, Color col, double x1, double y1, double x2, double y2, int off, double h, double xD){
        this(wid, col, off, h);
        start = new Point2D.Double(x1,y1);
        addLine(x1,y1,x2,y2);
        
        xDist = xD;
    }
    
    /** Adds a line to this scribble */
    public void addLine(double x1, double y1, double x2, double y2){
        ScribbleLine line = new ScribbleLine(x1,y1,x2,y2);
        updateBounds(x1,y1,x2,y2);	//must update bounds
        gesture.addElement(line);
    }
    
    /** Adds a ScribbleLine (a specialized line) to the scribble */
    public void addLine(ScribbleLine line){
        updateBounds(line.getX1(), line.getY1(), line.getX2(), line.getY2());
        gesture.addElement(line);
    }
    
    /**Keeps track of min (x and y) and max (x and y)*/
    private void updateBounds(double x1, double y1, double x2, double y2){
        
        if(x1 < minX){
            minX = x1;
        }
        else if(x1 > maxX){
            maxX = x1;
        }
        
        if(x2 < minX){
            minX = x2;
        }
        else if(x2 > maxX){
            maxX = x2;
        }
        
        if(y1 < minY){
            minY = y1;
        }
        else if(y1 > maxY){
            maxY = y1;
        }
        
        if(y2 < minY){
            minY = y2;
        }
        else if(y2 > maxY){
            maxY = y2;
        }
        
        rect = new Rectangle2D.Double(minX, minY, (maxX-minX+1), (maxY-minY+1));
        
    }
    
    /** Returns the number of line segments in the scribble */
    public int size(){
        return gesture.size();
    }
    
    /** Returns true if the given point is within the Rectangle created by the bounds of the scribble */
    public boolean contains(Point2D pt){
        return rect.contains(pt);
    }
    
    /** Gets the bounding rectangle */
    public Rectangle2D.Double getRect(){
        return rect;
    }
    
    /** Returns the area of the bounding rectangle */
    public double area(){
        return (rect.getHeight() * rect.getWidth());
    }
    
    /** Gets the rectangle in String form */
    public String getInfo(){
        return rect.toString();
    }
    
    /** Adds an offset to account for document repositioning */
    public void addOffset(double x, double y){
        minX = minX + x;
        minY = minY + y;
        maxX = maxX + x;
        maxY = maxY + y;
        rect = new Rectangle2D.Double(minX, minY, (maxX-minX+1), (maxY-minY+1));
        
        int size=gesture.size();
        for(int i=0;i<size;i++){
            ((ScribbleLine)gesture.elementAt(i)).addOffset(x,y);
        }
    }
    
    /** Gets the indexed ScribbleLine */
    public ScribbleLine getLine(int i){
        return (ScribbleLine)gesture.elementAt(i);
    }
    
    /** Gets all the lines of the scribble */
    public Vector getLines(){
        return gesture;
    }
    
    /** Gets the color of the Scribble */
    public Color getColor(){
        return color;
    }
    
    /** Gets the width of the Scribble */
    public int getWidth(){
        return width;
    }
    
    /** Sets the color of the scribble */
    public void setColor(Color c){
        color = c;
    }
    
    /** Sets the width */
    public void setWidth(int w){
        width = w;
    }
    
    /** Gets the offset */
    public int getOffset(){
        return offset;
    }
    
    /** Sets the offset */
    public void setOffset(int off){
        offset = off;
    }
    
    /** Fix the position */
    public void fixPosition(double x, double y){
        
        y = y + height;
        
        if(start == null){
            double x1 = x;
            double y1 = y;
            start = new Point2D.Double(x1,y1);
        }
        
        double xOff =  x - start.getX();
        double yOff =  y - start.getY();
        
        addOffset(xOff, yOff);
        
        start.setLocation(x,y);
        
    }
    
    /** Get x-component of first point on first line in scribble */
    public double getX(){
        return ((ScribbleLine)gesture.elementAt(0)).getX1();
    }
    
    /** Get y-component of first point on first line in scribble */
    public double getY(){
        return ((ScribbleLine)gesture.elementAt(0)).getY1();
    }
    
    /** Set start point of scribble */
    public void setStart(double x, double y){
        start.setLocation(x,y);
    }
    
    /** Sets height from offset */
    public void setHeight(double h){
        height = h;
    }
    
    /** Gets height from offset */
    public double getHeight(){
        return height;
    }
    
    /** Sets X-Distance from offset */
    public void setXDistanceFromOffset(double x){
        xDist = x;
    }
    
    /** Gets X-Distance from the offset */
    public double getXDistanceFromOffset(){
        return xDist;
    }
    
    /** Gets the color of the scribble as a String */
    public String getColorString(){
        return new StringBuffer().append(color.getRed()).append('|').append(color.getGreen()).append('|').append(color.getBlue()).toString();
    }
    
    /** Generates a string of the lines of this scribble */
    public String getLineString(){
        StringBuffer output = new StringBuffer();
        int size=gesture.size();
        for(int i=0;i<size;i++){
            output.append(((ScribbleLine)gesture.elementAt(i)).generateStoredContent());
            output.append('#');
        }
        
        ScribbleLine last = (ScribbleLine)gesture.elementAt(gesture.size()-1);
        output.append(last.getX2());
        output.append('*');
        output.append(last.getY2());
        
        return output.toString();
    }
    
    /** Generates String representation of Scribble --> width@R@G@B@offset@height@xDist@ScribbleLine#ScribbleLine */
    public String generateStoredContent(){
        
        StringBuffer output = new StringBuffer();
        output.append(width);
        output.append('@');
        output.append(color.getRed());
        output.append('@');
        output.append(color.getGreen());
        output.append('@');
        output.append(color.getBlue());
        output.append('@');
        output.append(offset);
        output.append('@');
        output.append(height);
        output.append('@');
        output.append(xDist);
        output.append('@');
        
        for(int i=0;i<gesture.size();i++){
            output.append(((ScribbleLine)gesture.elementAt(i)).generateStoredContent());
            output.append('#');
        }
        
        ScribbleLine last = (ScribbleLine)gesture.elementAt(gesture.size()-1);
        output.append(last.getX2());
        output.append('*');
        output.append(last.getY2());
        
        return output.toString();
    }
    
    /** Prints info to screen (debugging) */
    public void print(){
        System.out.println("-----Scribble------");
        System.out.println("Width: "+width);
        System.out.println("Color: "+color);
        System.out.println("Offset: "+offset);
        System.out.println("Height: "+height);
        System.out.println("xDist: "+xDist);
        
        for(int i=0;i<gesture.size();i++){
            ((ScribbleLine)gesture.elementAt(i)).print();
        }
        System.out.println("-----End Scribble------");
    }
}
