package edu.gwu.blaise.version2.grader;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.io.*;
import java.util.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.plaf.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.HTMLDocument;
import javax.swing.UIManager;
import javax.swing.undo.*;
import java.util.regex.*;

import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.util.find.*;
import edu.gwu.blaise.version2.recognizer.*;
import edu.gwu.blaise.version2.documents.*;

import java.net.URL;


/**
* The GradingSheet displays a document and allows a grader to use scribbles and gestures to grade the document
 */
public class BlaiseGradingPane extends JPanel implements ActionListener, BlaiseCommentViewInterface, ComponentListener{
    private static final boolean DEBUG = true;
    
    /* Restrurcturing for Blaise v2 */
    
    /**@see saveIfNeeded() */
    private static final int NO_SAVE_NEEDED=999;
    
    BlaiseGradingPane thisPane;
    JPanel  gradingView;
    
    private ProgressView progView;
    
    /**Controls the enabling and disabling of menu and toolbar items based on context */
    private BlaiseGradingPane.UIStateDelegate myUIStateDelegate;
    
        // The bottom pane.  Contains the mainView and Note related functionality.
    private BlaiseLayeredCommentPane layeredView;
	// The text component (bottom pane) of the paper
    BlaiseCommentView mainView;
    
	// The instance of the GradingFileHandler
	//private GradingFileHandler gfHandler = new GradingFileHandler();
    
	// Required thread variables
    private Thread saveThread;
    private Thread openThread;
    private boolean fileChooserClosed = false; /* set to true by save action when the file chooser is closed */
    
	// JFrame representing the container holding this JPanel
    BlaiseSplitPane parent;
    
	// JLabel holding messages when a draft isn't in the bottom pane
    JLabel draftMessageLabel;
    
    JScrollPane scroll;
    JScrollBar bar;
	///File saveFile = null;
    
	// The instance of findFrame in use
    private FindFrame findFrame = null;
    
	// Used to handle multiple undo/redos
    private UndoManager undoManager=new UndoManager();
    
	// State boolean to check whether the grader is in standalone mode or not
    private boolean STATEStandalone = false;
	// State boolean to check whether the grader has a paper currently being graded
    private boolean STATEMarkingModeActive = false;
    
    /**
        * Some actions can perform their action quietly (with little or no user interaction). If quiet action is
     * needed, set the command name to this (a reference).
     */
    private static final String QUIETLY = "do-action-quietly";
    
    /* MENUS 
        *****************************************/
    JMenuBar menubar;
    
    private JMenu fileMenu;
	// The menu items that might need to be disabled/enabled
    private JMenuItem undoMI;
    private JMenuItem redoMI;
    private JMenuItem cutMI;
    private JMenuItem copyMI;
    private JMenuItem pasteMI;
	//private JMenuItem gradeDraftMI;
    private JMenuItem openMI;
    private JMenuItem saveMI;
    private JMenuItem findNotesMI;
    private JMenuItem findTextMI;
    private JMenuItem paperInfoMI;
    private JMenuItem scribbleModeMI;
    private JMenuItem editModeMI;
    private JMenuItem notesModeMI;
    private JMenuItem spacesModeMI;
    private JMenuItem finishGradingPaperMI;
    private JMenuItem finishGradingDraftMI;
    //private JMenuItem showHideCommonNotesMI;
    private JMenuItem deleteSelectedMI;
    
    /* SUBMENUS */
    private JMenu selectDraftM;
    private JMenu gradeDraftM;
    private JMenu scribbleWidthM;
    //private JMenu scribbleColorM;
    private JMenu noteColorM;
    
    /* MENU ACTIONS (References to default actions that the grader will use) *****************************************/        
    private Action copyAction; /* default is good */
    private Action cutAction; /* default is good */
    private Action pasteAction; /* default is good */
    private Action selectAllAction; /* default is good */
    private HelpAction		helpAction;
    private AboutAction		aboutAction;
    private ExitAction		exitAction;
    private OpenAction		openAction;
    /*
     private CloseAction	closeAction;
     */
    private SaveAction		saveAction;
    private UndoAction		undoAction;
    private RedoAction		redoAction;
    private GradeDraftAction	gradeDraftAction;
    private FinishGradingDraftAction	finishGradingDraftAction;
    private FinishGradingPaperAction	finishGradingPaperAction;
    private PaperInformationAction		paperInformationAction;
    private FindInTextAction			findInTextAction;
    private FindInNotesAction			findInNotesAction;
    private DeleteSelectedAction                deleteSelectedAction;
    /*
     private FindInTextAgainAction		findInTextAgainAction;
     private FindInNotesAgainAction		findInNotesAgainAction;
     private ToggleScribblesAction		toggleScribblesAction;
     private ToggleNotesAction			toggleNotesAction;
     private ExpandAllNotesAction		expandAllNotesAction;
     private CollapseAllNotesAction		collapseAllNotesAction;
     */
    private ScribbleAction		switchScribbleAction;
    private EditAction			switchEditAction;
    private InsertSpaceAction	switchInsertSpacesAction;
    private NotesAction		switchNotesAction;
    
    /* End restructure */
    
    public static byte SAVE = 0;
    public static byte OPEN = 1;
    
    private byte NONE = 0;
    private byte SPELLING = 1;
    private byte DELETE = 2;
    private byte CAPITALIZE = 3;
    private byte lastGesture = NONE;
    
    JMenuItem formatScrib;
    boolean started = false;	//used to signal end of work for a moment
    String assigned_grade = "";
    
    int aspect_x;
    int aspect_y;
    
    /*****************************************
        * STATES
        *****************************************/
    int CREATE_SCRIBBLE = 0;
    int EDIT_SCRIBBLE = 1;
    int INSERT_SPACE = 2;
    int CREATE_NOTES = 3;
    int state;
    
    /******************************************
        * SCRIBBLE VARIABLES
        ******************************************/
    private Vector scribbles;
    private Vector removedScribbles = new Vector();
    private Vector spaces = new Vector();
    private Vector removedSpaces = new Vector();
    int current_x;
    int current_y;
    private int width;
    private Color color;
    private Scribble selected;
    private Scribble tempScribble;
    private boolean aSelection;
    private Scribble current;
    
    colorSwatch c1;
    colorSwatch c2;
    colorSwatch c3;
    colorSwatch c4;
    colorSwatch c5;
    colorSwatch c6;
    colorSwatch c7;
    colorSwatch c8;
    colorSwatch c9;
    colorSwatch c10;
    
    /*********************************************
        * NOTES VARIABLES
        *********************************************/
    private Vector notes;
    private int lastNoteID;
    
    /*********************************************
        * GESTURE RECOGNITION VARIABLES
        *********************************************/
    int gest_x;
    int gest_y;
    Vector gest_lines;
    Recognizer recognizer;
    
    /*********************************************
        * GESTURE RECOGNITION VARIABLES
        *********************************************/
    int preX, preY, postX, postY, X, Y, prevCaretPos;
    int Count = 0;
    
    /**********************************************
        * CORRECTION SETS
        *********************************************/
    Vector corrections;
    
    Point initial;
    JLabel currentModeLabel;
    JPanel currentModePanel;
    JToggleButton scribbleButton;
    JToggleButton editButton;
    //JToggleButton gestureButton;
    JToggleButton spacesButton;
    JToggleButton notesButton;
    JToolBar toolBar;
    JComboBox sizeBox;
    
    JPanel tempPanel;
    JPanel widthAchiever;
    JPanel heightAchiever;
    
    JFrame gestureFrame;		//for gesture info
    
    String graderID		= "-42";		// Grader who checked this document out of system
    String graderName   = "Sensei";		// Name of grader who checked this document out
    String commentPrivate = "NO";
    
    /**
        * Constructor
     *
     * This will set up the grader as a portal-spawned instance
     * @param parent Parent BlaiseSplitPane instance
     */
    public BlaiseGradingPane(BlaiseSplitPane parent) {
		// Set up as portal-spawned grader (Arguments: parent BSP, standalone)
		//this(parent, false);
        this(parent, true);
    }
    
    /**
        * Constructor
     *
     * @param parent Parent BlaiseSplitPane instance
     * @param standalone Whether this instance is a standalone instance or not
     */
    public BlaiseGradingPane(BlaiseSplitPane parent, boolean standalone){        
		// Initialize critical internal data
        this.parent = parent;
        this.STATEStandalone = standalone;
        this.thisPane = this;
        this.setLayout(new BorderLayout());
        
		// Initialize grader information
        graderID = "";
        graderName = "My Marks";
        
		//Instantiate actions
        openAction					= new OpenAction(this);
        aboutAction					= new AboutAction(this);
        exitAction					= new ExitAction(this);
        saveAction					= new SaveAction(this);
        undoAction					= new UndoAction(this);
        redoAction					= new RedoAction(this);
        gradeDraftAction			= new GradeDraftAction(this);
        finishGradingDraftAction	= new FinishGradingDraftAction(this);
        finishGradingPaperAction	= new FinishGradingPaperAction(this);
        paperInformationAction		= new PaperInformationAction(this);
        findInTextAction			= new FindInTextAction(this);
        findInNotesAction			= new FindInNotesAction(this);
        
        switchScribbleAction		= new ScribbleAction(this);
        switchEditAction			= new EditAction(this);
        switchInsertSpacesAction	= new InsertSpaceAction(this);
        switchNotesAction			= new NotesAction(this);
        deleteSelectedAction		= new DeleteSelectedAction(this);
		helpAction					= new HelpAction(this);
        
		//Initialize the menu bar
        if(DEBUG)   System.out.println("Creating menu bar");
        createMenuBar();
        
        if(DEBUG)   System.out.println("Setting up myUIStateDelegate");
        myUIStateDelegate=new UIStateDelegate(this);
        
		// Set up the UIStateDelegate to monitor the draft pane (top pane of split pane)
        parent.draft_pane.addDraftViewListener(myUIStateDelegate);
        
        if(parent.draft_pane!=null){
			 // If there's a draft, populate the draft and graded menus
            myUIStateDelegate.populateGradeDraftMenu();
            myUIStateDelegate.populateSelectDraftMenu();
        }
        
        if(DEBUG)   System.out.println("done.");
        
        this.setBackground(Color.WHITE);
        
        // If this is a standalone grader, display the initial "open the paper" message
        if(STATEStandalone){
            draftMessageLabel = new JLabel("<html><h3>Please open the paper you would like to mark.</h3></html>", SwingConstants.CENTER);
        }
		// Otherwise, we should have received a Blaise Paper from the portal. Display the "select draft" message
        else {
            /* Open paper and update the drafts pane */
            draftMessageLabel = new JLabel("<html><h3>Please select the draft you would like to mark.</h3></html>", SwingConstants.CENTER);
        }
        
		// Add the message label to the BGP pane
        this.add(draftMessageLabel);
        
        /*
         if(DEBUG)   System.out.println("Preparing to init...");
         this.init();
         if(DEBUG)   System.out.println("Inited...");
         */
        
		// Capture the main window's "close" events
        parent.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        parent.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
                quitAction();
            }
        });		
    } //BlaiseGradingPane(BSP parent);
    
    private void prepareBlaiseCommentView() {
        if(DEBUG)   System.out.println("*** Preparing to enter MarkingMode...");
        
        aspect_x = -1;
        aspect_y = -1;
        
        state = CREATE_SCRIBBLE;	//set current state
        corrections = new Vector();
        
        notes = new Vector();
        
		/// INITIALIZE SCRIBBLE-VARIABLES
        initial = new Point();
        
        scribbles = new Vector();
        aSelection = false;
        width = 1;
        color = Color.RED;
        
        /// INITIALIZE DOCUMENT
        
        mainView = new BlaiseCommentView(this);
        mainView.isOriginal = false;
        
        /// INITIALIZE CORRECTIONS
        CorrectionSet spellingSet = new CorrectionSet(Color.GREEN, "Spelling Error");
        CorrectionSet capitalizeSet = new CorrectionSet(new Color(179,0,221), "Capitalize");
        CorrectionSet deleteSet = new CorrectionSet(Color.RED, "Delete");
        
		/// PARSE FOR ANY EXISTING GRADES BY THIS GRADER
        if(parent.current_draft == null){
            return;
        }
        
		// Load the draft text
        mainView.setText(parent.current_draft.getBody());
        
        Vector incoming_grades = parent.current_draft.getComments();
        for(int i = 0; i < incoming_grades.size(); i++){
            Blaise_Comment gradeobject = (Blaise_Comment)incoming_grades.elementAt(i);
            
			//If this grader already has grade marks, load them
            if((gradeobject.getGraderName()).equals(graderName)){
				//System.out.println("----------------------------------------------------------------------------------");
                System.out.println("Draft has already been marked by this grader...");
                
                aspect_x = (int)Double.parseDouble(gradeobject.getAspectX());
                aspect_y = (int)Double.parseDouble(gradeobject.getAspectY());
                mainView.setText(gradeobject.getBody());
                mainView.specialWidth = aspect_x;
                
				//System.out.println("X: "+aspect_x+", Y:"+aspect_y);
				//System.out.println("----------------------------------------------------------------------------------");
                
                Vector vec = gradeobject.getScribbles();
                for(int j=0;j<vec.size();j++){
                    scribbles.addElement((Scribble)vec.elementAt(j));
                }
                
                Vector cor = gradeobject.getCorrections();
                for(int j=0;j<cor.size();j++){
                    CorrectionSet cur = (CorrectionSet)cor.elementAt(j);
                    Vector starts = cur.getStarts();
                    Vector ends = cur.getEnds();
                    if(cur.getType().equals("Spelling Error")){
                        for(int c=0;c<starts.size();c++){
                            spellingSet.add(((Integer)starts.elementAt(c)).intValue(),((Integer)ends.elementAt(c)).intValue());
                        }
                    }
                    else if(cur.getType().equals("Capitalize")){
                        for(int c=0;c<starts.size();c++){
                            capitalizeSet.add(((Integer)starts.elementAt(c)).intValue(),((Integer)ends.elementAt(c)).intValue());
                        }
                    }
                    else if(cur.getType().equals("Delete")){
                        for(int c=0;c<starts.size();c++){
                            deleteSet.add(((Integer)starts.elementAt(c)).intValue(),((Integer)ends.elementAt(c)).intValue());
                        }
                    }
                }
                //load up the Notes, if any.
                Vector not = gradeobject.getNotes();
				System.out.println("gradeobject.getNotes:"+not);
                lastNoteID=0;
                int id;
                for(int j=0;j<not.size();j++){
                    Note n=(Note)not.elementAt(j);
                    notes.addElement(n);
                    //find the last note ID for use with creating new note IDs
                    id=Integer.parseInt(n.getNoteID());
                    if(id>lastNoteID){
                        lastNoteID=id;
                    }
                }
                break; //found this grader, do not check the other comments
            }
        }
        
        if(DEBUG)   System.out.println("Loading Corrections...");
        corrections.add(spellingSet);
        corrections.add(capitalizeSet);
        corrections.add(deleteSet);
        
        /*
		 /// INITIALIZE GESTURE RECOGNITION
         recognizer = new Recognizer(new File(Constants.getGestureFile()));
         */
        
		/// ADD MOUSE LISTENERS TO BlaiseSplitPane TO CAPTURE SCRIBBLES
        mainView.addMouseListener(new MouseAdapter() {
			//When the mouse is pressed
            public void mousePressed(MouseEvent mouseevent){
                //If in Scribble Mode
                if(state == CREATE_SCRIBBLE)
                {
                    System.out.println("mouseevent:"+mouseevent);
                    //Initialize new scribble
                    int x = mouseevent.getX();
                    int y = mouseevent.getY();
                    current_x = x;
                    current_y = y;
                    try
                    {
                        Rectangle rectangle = mainView.modelToView(mainView.viewToModel(new Point(x, y)));
                        double d = ((double)x - rectangle.getX()) + 1.0D;
                        current = new Scribble(width, color, x, y, x, y, mainView.viewToModel(new Point(x, y)), (double)y - rectangle.getY(), d);
                    }
                    catch(BadLocationException badlocationexception) { }
                }
                //If in Edit Gesture Mode
                else if(state == EDIT_SCRIBBLE) {
                    //Determine where mouse click was and perfform necessary intitialization
                    initial = new Point(mouseevent.getX(), mouseevent.getY());
                    if(aSelection) {
                        scribbles.addElement(selected);
                        aSelection = false;
                        thisPane.repaint();
                    }
                    
                    int l = isSelectedScribbles(new Point(mouseevent.getX(), mouseevent.getY()));
                    if(l != -1) {
                        removeSelectedScribbles(l);
                        aSelection = true;
                        if(formatScrib != null)formatScrib.setEnabled(true);
                    }
                    layeredView.deselectAllIcons();//a click somewhere not on an icon should deselect them all.
                }
				//If in Insert Spaces Mode
                else if(state == INSERT_SPACE) {
                    X = mouseevent.getX();
                    Y = mouseevent.getY();
                    prevCaretPos = mainView.viewToModel(new Point(X, Y)); //mainView.getCaretPosition(); //
                    System.out.println("[mousePressed]  X: "+X+" Y: "+Y+" caret: "+prevCaretPos);
                    preX = X;
                    preY = Y;
                    Count = 0;
                }
                /*
				//If in GEsture Recognition Mode, prepare to capture a new gesture
                 else if(state == GESTURE_RECOGNITION) {
                     int x = mouseevent.getX();
                     int y = mouseevent.getY();
                     gest_x = x;
                     gest_y = y;
                     
                     gest_lines = new Vector();
                 }
                 */
            }
            
            public void mouseClicked(MouseEvent mouseevent) {
                if(state == EDIT_SCRIBBLE) {
                    int k = isSelectedScribbles(new Point(mouseevent.getX(), mouseevent.getY()));
                    if(k != -1)
                    {
                        removeSelectedScribbles(k);
                        aSelection = true;
                    }
                    layeredView.deselectAllIcons();//a click somewhere not on an icon should deselect them all.
                }
                else if (state == INSERT_SPACE) {
                    Count = mouseevent.getClickCount();
                    System.out.println("[mouseClicked]  Count: "+Count);
                    if (Count >= 2) {
                        HTMLEditorKit kit = (HTMLEditorKit)mainView.getEditorKit();
                        HTMLDocument doc = (HTMLDocument)mainView.getDocument();
						//pane.replaceSelection("text");
						//			Element[] elm;
                        try {
                            kit.insertHTML(doc, mainView.getCaretPosition(), "<BR>",
                                           0, 0, HTML.Tag.BR);
                            System.out.println("Inserting BR.");
							//				elm = kit.HTMLTextAction.getElementsAt(doc, pane.getCaretPosition());
                        } catch (Exception ex) {
                            System.out.println("HTML Error at VSMouse.mouseClicked()");
                            ex.printStackTrace();
                        }
						/*Document d=mainView.getDocument();
						try{
							d.insertString(mainView.getCaretPosition(),"\n",null);
						}catch(BadLocationException ex){
							ex.printStackTrace();
						}*/
						//System.out.println("mainView height after:"+mainView.getHeight());
						
						//change the size of the text pane, too
						AttributeSet as=mainView.getCharacterAttributes();
						StyleContext sc=StyleContext.getDefaultStyleContext();
						Font f=sc.getFont(as);
						//System.out.println("font:"+f);
						FontMetrics fm=sc.getFontMetrics(f);
						//System.out.println("font metrics:"+fm);
						int size=fm.getHeight();
						//System.out.println("font metrics height:"+(size));
						int height=mainView.getHeight();
						size=2*size+height+5;
						//System.out.println("setting mainView height:"+size);
						mainView.setPreferredSize(new Dimension(mainView.getWidth(),size));
						//System.out.println("mainView size:"+mainView.getSize());
                    }
                    Count = 0;
                }
                else if (state == CREATE_NOTES){
                    Note newNote=new Note(layeredView,Integer.toString(++lastNoteID),mouseevent.getPoint(),Color.YELLOW,true);
                    notes.addElement(newNote);
                }
            }
            
            public void mouseReleased(MouseEvent mouseevent) {
                //Add scribble
                if(state == CREATE_SCRIBBLE){
                    scribbles.addElement(current);
                    /* Flag that the current draft has been modified */
                    myUIStateDelegate.setModified(true);
                }
                //Determine location of new scribble and update
                else if(state == EDIT_SCRIBBLE) {
                    if(aSelection) {
                        int k = mainView.viewToModel(new Point((int)selected.getX(), (int)selected.getY()));
                        selected.setOffset(k);
                        try {
                            Rectangle rectangle = mainView.modelToView(mainView.viewToModel(new Point((int)selected.getX(), (int)selected.getY())));
                            selected.setXDistanceFromOffset(selected.getX() - rectangle.getX());
                            selected.setHeight(selected.getY() - rectangle.getY());
                        }
                        catch(BadLocationException badlocationexception) { }
                        selected.setStart(selected.getX(), selected.getY());
                        scribbles.addElement(selected);
                        aSelection = false;
                        mainView.repaint();
                        /* Flag that the current draft has been modified */
                        myUIStateDelegate.setModified(true);
                    }
                }
                else if (state == INSERT_SPACE) {
                    X = mouseevent.getX();
                    Y = mouseevent.getY();
                    
                    Font thisFont = mainView.getFont();
                    FontMetrics currentMetrics = mainView.getFontMetrics(thisFont);
                    int fontHeight = currentMetrics.getHeight();
                    int BRcount = (1+(Math.abs(Y-preY))/fontHeight);
					System.out.println("BRcount:"+BRcount);
                    
                    if(DEBUG) System.out.println("[mouseReleased] X: "+X+"/"+preX+" Y: "+Y+"/"+preY);
                    if(DEBUG) System.out.println("     delta Y: "+(Math.abs(Y-preY)) +","+BRcount+ " // " +prevCaretPos+" | "+fontHeight);
                    Count = 0;
                    
                    HTMLEditorKit kit = (HTMLEditorKit)mainView.getEditorKit();
                    HTMLDocument doc = (HTMLDocument)mainView.getDocument();
					//pane.replaceSelection("text");
					//Element[] elm;
                    for (int i=0 ; i<BRcount ; i++) {
                        try {
                            kit.insertHTML(doc, prevCaretPos, "<BR>", 0, 0, HTML.Tag.BR);
							//elm = HTMLEditorKit.HTMLTextAction.getElementsAt(doc, prevCaretPos);
							//doc.insertAfterEnd(elm[0], "<BR>");
                            spaces.add(new Integer(prevCaretPos));
                            if(DEBUG) System.out.println("Inserting BR.");
                             //Flag that the current draft has been modified
                            myUIStateDelegate.setModified(true);
                        } catch (Exception ex) {
                            System.out.println("HTML Error at VSMouse.mouseClicked()");
                            ex.printStackTrace();
                        }
						/*try{
							doc.insertString(prevCaretPos,"\n",null);
						}catch(BadLocationException ex){
							ex.printStackTrace();
						}*/
						//System.out.println("mainView height before:"+mainView.getHeight());
                    }
					
					
					//change the size of the text pane, too
					AttributeSet as=mainView.getCharacterAttributes();
					StyleContext sc=StyleContext.getDefaultStyleContext();
					Font f=sc.getFont(as);
					//System.out.println("font:"+f);
					//size+=StyleConstants.getSpaceAbove(as);
					//System.out.println("get space above:"+StyleConstants.getSpaceAbove(as));
					//size+=StyleConstants.getSpaceBelow(as);
					//System.out.println("get space below:"+StyleConstants.getSpaceBelow(as));
					FontMetrics fm=sc.getFontMetrics(f);
					//System.out.println("font metrics:"+fm);
					int size=fm.getHeight();
					//System.out.println("font metrics height:"+(size));
					int height=mainView.getHeight();
					size=(5+size)*BRcount+height;
					//System.out.println("setting mainView height:"+size);
					mainView.setPreferredSize(new Dimension(mainView.getWidth(),size));
					//System.out.println("mainView size:"+mainView.getSize());
					
					//try { doc.remove(prevCaretPos, 2); } catch (Exception ex) {}
                }
                /*
				//Try to recognize gesture
                 else if(state == GESTURE_RECOGNITION) {
                     if(gest_lines.size() <= 5)return;
                     
                     String matched = recognizer.match(gest_lines);
                     
                     int barVal = bar.getValue();
                     if(matched.equalsIgnoreCase("Spelling")) {
                    	//Spelling errors - take mid line in gesture
                         Line2D.Double line = (Line2D.Double)gest_lines.elementAt(gest_lines.size()/2);
                         int index = mainView.viewToModel(new Point((int)line.getX1(), (int)line.getY1()+barVal));
                         spellingError(index);
                     }
                     else if(matched.equalsIgnoreCase("Capitalize")) {
                    	//Capitalize errors - take mid line in gesture
                         Line2D.Double line = (Line2D.Double)gest_lines.elementAt(gest_lines.size()/2);
                         int index = mainView.viewToModel(new Point((int)line.getX1(), (int)line.getY1()+barVal));
                         capitalizeError(index);
                     }
                     else if(matched.equalsIgnoreCase("Delete")) {
                    	//Delete errors - take first line in gesture
                         Line2D.Double line = (Line2D.Double)gest_lines.elementAt(0);
                         int index = mainView.viewToModel(new Point((int)line.getX1(), (int)line.getY1()+barVal));
                         deleteError(index);
                     }
                     else if(matched.equalsIgnoreCase("Vertical_Space")) {
                    	//Vertical Space - take first line
                         Line2D.Double line = (Line2D.Double)gest_lines.elementAt(0);
                         int index = mainView.viewToModel(new Point((int)line.getX1(), (int)line.getY1()+barVal));
                         verticalInsert(index);
                     }                    
                     mainView.repaint();
                 }
                 */
                
				//Enable undos
                myUIStateDelegate.updateUndoUI();
            }});
        
        mainView.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent mouseevent) {
                //On mouse drag, add lines to current scribble
                if(state == CREATE_SCRIBBLE) {
                    int x = mouseevent.getX();
                    int y = mouseevent.getY();
					//Graphics2D graphics2d = (Graphics2D)scroll.getGraphics();
                    Graphics2D graphics2d = (Graphics2D)mainView.getGraphics();
                    
                    //Set width and color of line and draw accounting for offset of scrollbar
                    graphics2d.setPaint(color);
                    graphics2d.setStroke(new BasicStroke(width));
                    //int barVal = bar.getValue();
                    //java.awt.geom.Line2D.Double double1 = new java.awt.geom.Line2D.Double(current_x, current_y - barVal, x, y - barVal);
                    java.awt.geom.Line2D.Double double1 = new java.awt.geom.Line2D.Double(current_x, current_y, x, y);
                    graphics2d.draw(double1);
                    current.addLine(current_x, current_y, x, y);
                    current_x = x;
                    current_y = y;
                }
                //If editting scribble, move selected scribble
                else if (state == EDIT_SCRIBBLE) {
                    if(aSelection) {
                        selected.addOffset((double)mouseevent.getX() - initial.getX(), (double)mouseevent.getY() - initial.getY());
                        thisPane.repaint();
                        initial = new Point(mouseevent.getX(), mouseevent.getY());
                    }
                }
                else if (state == INSERT_SPACE) {
                }
                /*
				 //If recognizing gesture, account for scrollbar and continue capturing lines
                 else if(state == GESTURE_RECOGNITION){
                     int x = mouseevent.getX();
                     int y = mouseevent.getY();
					 //Graphics2D graphics2d = (Graphics2D)scroll.getGraphics();
                     Graphics2D graphics2d = (Graphics2D)mainView.getGraphics();
                     graphics2d.setPaint(Color.BLACK);
                     graphics2d.setStroke(new BasicStroke(1));
                     int barVal = bar.getValue();
					 //Line2D.Double double1 = new java.awt.geom.Line2D.Double(gest_x, gest_y - barVal, x, y - barVal);
                     Line2D.Double double1 = new java.awt.geom.Line2D.Double(gest_x, gest_y, x, y);
                     graphics2d.draw(double1);
                     gest_lines.add(new Line2D.Double(gest_x, gest_y-barVal, x, y-barVal));
                     gest_x = x;
                     gest_y = y;
                 }
                 */
            }
            
        });
        
		//Initialize the scrollbar
        /*bar = new JScrollBar();
        bar.setOrientation(1);
        bar.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent adjustmentevent){
                mainView.repaint();	//repaint on every adjustment (needed to draw scribbles)
                
            }
        });*/
        
        gradingView = new JPanel(new BorderLayout());
        gradingView.setBackground(Color.GRAY);
        
        tempPanel = new JPanel(new BorderLayout());
        widthAchiever = new JPanel();
        heightAchiever = new JPanel();
        widthAchiever.setBackground(Color.GRAY);
        heightAchiever.setBackground(Color.GRAY);
        
        layeredView=new BlaiseLayeredCommentPane(mainView,new EditModeManager(){
            public boolean getEditMode(){
                if(state==EDIT_SCRIBBLE){
                    return true;
                }
                return false;
            }
            public void deselectAll(){
                if(aSelection) {
                    scribbles.addElement(selected);
                    aSelection = false;
                    thisPane.repaint();
                }
            }
        });
        tempPanel.add(layeredView, BorderLayout.CENTER);
        tempPanel.add(heightAchiever, BorderLayout.EAST);
        tempPanel.add(widthAchiever, BorderLayout.SOUTH);
        
        /*make sure the mainView gets a size or everything will go all crazy*/
        Dimension mvD=mainView.getPreferredSize();
        Dimension pD=this.getSize();
        aspect_x=pD.width-20;//-20 so that it is small enough that the scrollbar doesn't get in the way.
        mainView.setDimensions(aspect_x,mvD.height);
                
        //Add text area to a scroll pane to accomodate long documents and add scroll to frame
        if(Constants.isOSX()){//osx always displays the vertical bar.
            scroll = new JScrollPane(tempPanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        else{
            scroll = new JScrollPane(tempPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        //scroll.setVerticalScrollBar(bar);

		//initializeMenu();
    	gradingView.add(initializeToolbar(), BorderLayout.NORTH);
    	gradingView.add(scroll, BorderLayout.CENTER);
        
        int i,size=notes.size();
        for(i=0;i<size;i++){
            ((Note)notes.elementAt(i)).display(layeredView,true);//causes the note to get added to the layeredView
        }
        
    	this.resize();
        
		//Enable paper-related menu items
        myUIStateDelegate.updatePaperUI();
        if(findFrame!=null && findFrame.isVisible()){
            findFrame.setJTextComponent(mainView);
        }
    }
    
    public void resize(){
        if(aspect_x != -1){
            
            if(aspect_y == -1){
                aspect_y = mainView.getHeight();
            }
            mainView.specialWidth = -1;
            mainView.setDimensions(aspect_x, aspect_y);
			layeredView.setPreferredSize(new Dimension(aspect_x,aspect_y));
            layeredView.setSize(new Dimension(aspect_x,aspect_y));
			layeredView.revalidate();
            Dimension dimen = mainView.getSize();
            System.out.println("mainView.getSize():"+dimen);
            Dimension panelSize = gradingView.getSize();
            System.out.println("gradingView.getSize():"+panelSize);
			//If text was marked at a larger size, stretch
            if(dimen.getWidth() > panelSize.getWidth()){
                scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                Dimension wide_dimen = new Dimension((int)dimen.getWidth(),1);
                widthAchiever.setPreferredSize(wide_dimen);
                widthAchiever.setMaximumSize(wide_dimen);
                widthAchiever.setMinimumSize(wide_dimen);
                widthAchiever.setSize(wide_dimen);
                
                heightAchiever.setSize(new Dimension(1,1));
                heightAchiever.setPreferredSize(new Dimension(1,1));
            }
			//Else If text was marked at a smaller sizw, use height achiever to shrink size
            else if(dimen.getWidth() < panelSize.getWidth()){
                scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
                Dimension high_dimen = new Dimension(((int)panelSize.getWidth() - (int)dimen.getWidth()),1);
                heightAchiever.setPreferredSize(high_dimen);
                heightAchiever.setMaximumSize(high_dimen);
                heightAchiever.setMinimumSize(high_dimen);
                heightAchiever.setSize(high_dimen);
                
                widthAchiever.setSize(new Dimension(1,1));
                widthAchiever.setPreferredSize(new Dimension(1,1));
            }
            else{
                scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            }
            
            parent.setStatus("  Now marking draft at: "+aspect_x+" x "+aspect_y);
            
        }
        else{
            mainView.specialWidth = gradingView.getWidth();
            System.out.println("gradingView.getWidth():"+mainView.specialWidth);
            aspect_x = gradingView.getWidth();
            parent.setStatus("  Now marking draft");
        }
        
        widthAchiever.validate();
        heightAchiever.validate();
        layeredView.validate();
        tempPanel.validate();
        scroll.validate();
        gradingView.validate();
        this.validate();
    }
    
	//////////////////////////////////////////////////////////////////////
	//
	// Blaise 2 - New stuff
	//
	/////////////////////////////////////////////////////////////////////
    
    /**
        * Returns the first Blaise_Paper from the file or null if the file doesn't have a Blaise_Paper.
     * @param open true if this file should "open" it, false if you just want the Blaise_Paper object.
     */
    public Blaise_Paper getBPFromFile(File file,boolean open) throws IOException{
        /* //old version 1 style.
		   //Parse document XML into Blaise_Paper object
        Blaise_XML xml = new Blaise_XML();
        Vector paperVec = xml.parse(file.getAbsolutePath());
        //If no paper found, reject writer from opening document
        if(paperVec.size() == 0){
            JOptionPane.showMessageDialog(this, "The file does not seem to be a valid Blaise Paper.", "Error Opening Paper", JOptionPane.ERROR_MESSAGE);
            return null;
        }
        else{
            //Extract paper
            return (Blaise_Paper)paperVec.elementAt(0);
        }*/
        Blaise_Paper bp=new Blaise_Paper();
        BlaiseFile bf;
        if(open){
            bf=new BlaiseFile(bp,file,new GraderFileHandler());
        }
        else{
            bf=new BlaiseFile(bp,file,new BackendFileHandler());
        }
        bf.openPaper();
        return bp;
    }
    
    public Blaise_Paper getBPFromFile(File file) throws IOException{
        return getBPFromFile(file,true);
    }
    
    /**
        * Method for obtaining a File for input/output using a JFileChooser dialog.
     * Enforces blaise only file types.
     *
     * @param mainView The JTextPane representing the contents of the document
     * @param startDir Directory where file chooser should begin
     * @param dialogType Type of file being chosen. [Open or Save]
     * @return File chosen by the writer
     */
    public static File getFileFromChooser(Component owner, String startDir, int dialogType) {
        return getFileFromChooser(owner,startDir,dialogType,true);
    }
    
    /**
        * Method for obtaining a File for input/output using a JFileChooser dialog
     *
     * @param mainView The JTextPane representing the contents of the document
     * @param startDir Directory where file chooser should begin
     * @param dialogType Type of file being chosen. [Open or Save]
     * @param blaiseLimit boolean on whether or not to limit the allowed files to be blaise files.
     * @return File chosen by the writer
     */
    public static File getFileFromChooser(Component owner, String startDir, int dialogType, boolean blaiseLimit) {
        //Initialize new chooser
        JFileChooser jfileDialog = new JFileChooser(startDir);
        jfileDialog.setDialogType(dialogType);
        int optionSelected = JFileChooser.CANCEL_OPTION;
        
        //Based on type, determine allowed file filters
        if(blaiseLimit){
            if(dialogType == JFileChooser.OPEN_DIALOG) {
                jfileDialog.setMultiSelectionEnabled(false);
                jfileDialog.addChoosableFileFilter(new BlaiseFileFilter("bls","A Saved Blaise Paper (*.bls)"));
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
            else if(dialogType == JFileChooser.SAVE_DIALOG) {
                jfileDialog.setMultiSelectionEnabled(false);
                jfileDialog.addChoosableFileFilter(new BlaiseFileFilter("bls","A Saved Blaise Paper (*.bls)"));
                optionSelected = jfileDialog.showSaveDialog(owner);
            }
            else {
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
        }
        else{
            if(dialogType == JFileChooser.OPEN_DIALOG) {
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
            else if(dialogType == JFileChooser.SAVE_DIALOG) {
                optionSelected = jfileDialog.showSaveDialog(owner);
            }
            else {
                optionSelected = jfileDialog.showOpenDialog(owner);
            }
        }
        
        //If approved operation is selected, return file
        if(optionSelected == JFileChooser.APPROVE_OPTION){
            File file=jfileDialog.getSelectedFile();
            if(dialogType == JFileChooser.SAVE_DIALOG && blaiseLimit){
                String name=file.getName();
                if(!name.toLowerCase().endsWith(".bls")){
                    return new File(file.getAbsolutePath()+".bls");
                }
            }
            return file;
        }
        return null;
    }
    
    /**
        * Starts a progress view with the supplied properties. (replacing an existing one if needed)
     */
    synchronized void startProgView(String title, int timeSeconds, boolean isIndeterminate){
        //starting progress view is thread safe.
        if(progView==null){
            System.out.println("Creating new progress view");
            progView=new ProgressView(parent,title,timeSeconds,isIndeterminate);
        }
        else{
            progView.done();
            System.out.println("Creating newer progress view");
            progView=new ProgressView(parent,title,timeSeconds,isIndeterminate);
        }
    }
    
    /**
        * Starts an indeterminate progress view, with the supplied title. (replacing an existing one if needed)
     */
    void startProgView(String title){
        startProgView(title,0,true);
    }
    
    /**
        * Stops the progress view (if it exists)
     */
    synchronized void stopProgView(){
        if(progView!=null){
            //done is thread safe.
            progView.done();
            progView=null;
        }
    }
    
    public void quitAction(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                exitAction.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,exitAction.NAME));
            }
        });
    }
    
    /**
        * Opens the document from the supplied filename.
     * Please note: no real checking is made to make sure this filename is a valid
     * blaise paper.
     */
    public void openAction(final String filename){
        if(filename==null){
            throw new IllegalArgumentException("Invalid filename");
        }
        
        System.out.println("public void openAction()");
        openAction.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,filename));
    }
    
    /**
        * Displays the about box.  (Method used by mac version)
     */
    public void aboutAction(){
        aboutAction.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,aboutAction.NAME));
    }
    
	//////////////////////////////////////////////////////////////////////
	//
	// Scribble Editting Methods
	//
	/////////////////////////////////////////////////////////////////////
	//Clears all scribbles
    public void removeAllScribbles(){
        removedScribbles = scribbles;
        scribbles = new Vector();
        repaint();
    }
    
	//Removes last draw scribble
    public void removeLastScribbles(){
        if(scribbles.size() > 0){
			//System.out.println("removeLastScribbles: ["+scribbles.size()+"/"+removedScribbles.size()+"]");
            tempScribble = (Scribble)scribbles.remove(scribbles.size() - 1);
            removedScribbles.add(tempScribble);
            repaint();
        } else {
			//System.out.println("Nothing to undo.");
        }
    }
    
	//Undo scribble removal
    public void undoRemoveLastScribbles(){
        if(removedScribbles.size() > 0){
			//System.out.println("undoRemoveLastScribbles: ["+removedScribbles.size()+"/"+scribbles.size()+"]");
            tempScribble = (Scribble)removedScribbles.remove(removedScribbles.size() - 1);
            scribbles.add(tempScribble);
            repaint();
        } else {
			//System.out.println("Nothing to redo.");
        }
    }
    
	//Removes the selected scribble
    private void removeSelectedScribbles(int i){
        selected = (Scribble)scribbles.elementAt(i);
        scribbles.removeElementAt(i);
    }
    
	//Determines if any scribble is selected based on point
    private int isSelectedScribbles(Point point){
        double d = -1D;
        int i = -1;
        for(int j = 0; j < scribbles.size(); j++){
            Scribble scrib = (Scribble)scribbles.elementAt(j);
            if(scrib.contains(point) && (scrib.area() < d || d == -1D)){
                d = scrib.area();
                i = j;
            }
        }
        
        return i;
    }
    
	///////////////////////////////////////////////////////////////////////
	//
	// Correction Methods
	//
	///////////////////////////////////////////////////////////////////////
	//Called when a spelling error is found - will determine bounds of word and add a correctionSet object
    private void spellingError(int i){
        CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(0);
        try{
            String s = mainView.getText(i, 1);
            //Blank space - no need to highlight
            if(s.equals(" ")){}
            else{
                String s1 = s;
                int j = i;
                do{
                    j++;
                    s1 = mainView.getText(j, 1);
                }
                while(!isEnded(s1));
                
                s1 = s;
                int k = i;
                do{
                    k--;
                    s1 = mainView.getText(k, 1);
                }
                while(!isEnded(s1));
                k++;
                correctionset.add(k, j);	//add start and end positions to correctionset
                lastGesture = SPELLING;
            }
        }
        catch(BadLocationException badlocationexception){
            //System.out.println("GradingSheet BadLocationException: " + badlocationexception);
        }
    }
    
	//Called when a capitalize error is found - will determine bounds of word and add a correctionSet object
    private void capitalizeError(int i){
        CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(1);
        try{
            String s = mainView.getText(i, 1);
            if(s.equals(" ")){}
            else{
                String s1 = s;
                int j = i;
                do{
                    j++;
                    s1 = mainView.getText(j, 1);
                }
                while(!isEnded(s1));
                
                s1 = s;
                int k = i;
                do{
                    k--;
                    s1 = mainView.getText(k, 1);
                }
                while(!isEnded(s1));
                k++;
                correctionset.add(k, j);
                lastGesture = CAPITALIZE;
            }
        }
        catch(BadLocationException badlocationexception){
            //System.out.println("GradingSheet BadLocationException: " + badlocationexception);
        }
    }
    
	//Called when a delete error is found - will determine bounds of word and add a correctionSet object
    private void deleteError(int i){
        CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(2);
        try{
            String s = mainView.getText(i, 1);
            //No need to worry about a space
            if(s.equals(" ")){}
            else{
                String s1 = s;
                int j = i;
                do{
                    j++;
                    s1 = mainView.getText(j, 1);
                }
                while(!isEnded(s1));
                
                s1 = s;
                int k = i;
                do{
                    k--;
                    s1 = mainView.getText(k, 1);
                }
                while(!isEnded(s1));
                k++;
                correctionset.add(k, j);
            	lastGesture = DELETE;
            }
        }
        catch(BadLocationException badlocationexception){
            //System.out.println("GradingSheet BadLocationException: " + badlocationexception);
        }
    }
	//Called when a vertical space gesture is recognized
    private void verticalInsert(int i){
        System.out.println("Aspect_Y: "+aspect_y);
        if(aspect_y <= 0)resize();
        
        try{
            HTMLDocument htmldocument = (HTMLDocument)mainView.getDocument();
            Element element = htmldocument.getCharacterElement(i);
            htmldocument.insertBeforeStart(element, "<br>");
            incrementOffset(element.getStartOffset());
            
            lastGesture = NONE;
            //Increment aspect_y to ensure proper sizing
            aspect_y += 20;
            resize();
        }
        catch(BadLocationException badlocationexception){
            //System.out.println("Bad exception location " + badlocationexception);
        }
        catch(IOException ioexception){
            //System.out.println("IOException " + ioexception);
        }
    }
    
	//Increments the offset of scribbles and correction sets if vertical space is entered before them
    private void incrementOffset(int i){
        for(int j = 0; j < scribbles.size(); j++){
            Scribble scrib = (Scribble)scribbles.elementAt(j);
            if(scrib.getOffset() >= i)
                scrib.setOffset(scrib.getOffset() + 1);
        }
        
        for(int k = 0; k < corrections.size(); k++){
            CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(k);
            correctionset.increment(i);
        }
    }
	//////////////////////////////////////////////////////////////////////
	//
	// Menu and Toolbar
	//
	/////////////////////////////////////////////////////////////////////
    /*
     Creates the menu structure for this pane's menu bar
     */
    private void createMenuBar() {
        menubar = new JMenuBar();
        
        fileMenu = new JMenu("File");
        JMenu editMenu = new JMenu("Edit");
        JMenu viewMenu = new JMenu("View");
        JMenu markupMenu = new JMenu("Markup");
        JMenu scribblesMenu = new JMenu("Scribbles");
        JMenu notesMenu = new JMenu("Notes");
        JMenu helpMenu = new JMenu("Help");
        JMenuItem item;
        
		//Add the menus to the menubar
        menubar.add(fileMenu);
        menubar.add(editMenu);
        menubar.add(viewMenu);
        menubar.add(markupMenu);
        menubar.add(scribblesMenu);
        menubar.add(notesMenu);
        menubar.add(helpMenu);
        
		//platform independent
        int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
        
		//File Menu
        openMI = new JMenuItem(openAction);
        openMI.setText("Open...");
        openMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,aKey));
        fileMenu.add(openMI);
        
        saveMI = new JMenuItem(saveAction);
        saveMI.setText("Save Markings");
        saveMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,aKey));
        fileMenu.add(saveMI);
        
        fileMenu.addSeparator();
        
        finishGradingDraftMI=new JMenuItem(finishGradingDraftAction);
        finishGradingDraftMI.setText("Finish Grading Draft");
        finishGradingDraftMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,aKey|InputEvent.SHIFT_MASK));
        fileMenu.add(finishGradingDraftMI);
        
        finishGradingPaperMI=new JMenuItem(finishGradingPaperAction);
        finishGradingPaperMI.setText("Finish Grading Paper");
        finishGradingPaperMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,aKey|InputEvent.ALT_MASK));
        fileMenu.add(finishGradingPaperMI);
        
        fileMenu.addSeparator();
        
        paperInfoMI=new JMenuItem(paperInformationAction);
        paperInfoMI.setText("Paper Information");
        paperInfoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,aKey));
        fileMenu.add(paperInfoMI);
        
        if(!Constants.isOSX()){//on windows add exit to file menu
            fileMenu.addSeparator();
            
			item=new JMenuItem(exitAction);
            //item=new JMenuItem();
            item.setText("Exit");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,InputEvent.ALT_MASK));//alt-F4
                fileMenu.add(item);
        }
        
        //MG: modify seems to be a debugging thing, commenting it out 4/8/05
        //item = new JMenuItem(new SetModifiedAction(this));
        //item.setText("Modify");
        //fileMenu.add(item);
        
		//Edit Menu
		//undoMI=new JMenuItem(undoAction);
        undoMI=new JMenuItem(undoAction);
        undoMI.setText("Undo");
        undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,aKey));
        editMenu.add(undoMI);
        
		//redoMI=new JMenuItem(redoAction);
        redoMI=new JMenuItem(redoAction);
        redoMI.setText("Redo");
        redoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,aKey|InputEvent.SHIFT_MASK));
        editMenu.add(redoMI);
        
        editMenu.addSeparator();
        
        cutMI=new JMenuItem(cutAction);
        cutMI.setText("Cut");
        cutMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,aKey));
        editMenu.add(cutMI);
        
        copyMI=new JMenuItem(copyAction);
        copyMI.setText("Copy");
        copyMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,aKey));
        editMenu.add(copyMI);
        
        pasteMI=new JMenuItem(pasteAction);
        pasteMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,aKey));
        pasteMI.setText("Paste");
        editMenu.add(pasteMI);
        
        //editMenu.addSeparator();
        
        //findTextMI=new JMenuItem(findInTextAction);
        //findTextMI.setText("Find in text");
        //editMenu.add(findTextMI);
        
        //findNotesMI=new JMenuItem(findInNotesAction);
        //findNotesMI.setText("Find in notes (NI)");
        //editMenu.add(findNotesMI);
        
		//View Menu
        selectDraftM=new JMenu();
        selectDraftM.setText("Select Draft");
        viewMenu.add(selectDraftM);
        
        //showHideCommonNotesMI=new JMenuItem();
        //showHideCommonNotesMI.setText("Show/Hide Common Notes (NI)");
		//showHideCommonNotesMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,aKey));
        //viewMenu.add(showHideCommonNotesMI);
        
		//Markup Menu
        gradeDraftM=new JMenu();
		//gradeDraftM.setMnemonic(KeyStroke.getKeyStroke(KeyEvent.VK_G,aKey));
        gradeDraftM.setText("Grade Draft");
        markupMenu.add(gradeDraftM);
        
        markupMenu.addSeparator();
        
        scribbleModeMI=new JMenuItem(switchScribbleAction);
        scribbleModeMI.setText("Scribbles Mode");
        markupMenu.add(scribbleModeMI);
        
        notesModeMI=new JMenuItem(switchNotesAction);
        notesModeMI.setText("Notes Mode");
        markupMenu.add(notesModeMI);
        
        editModeMI=new JMenuItem(switchEditAction);
        editModeMI.setText("Edit Mode");
        markupMenu.add(editModeMI);
        
        spacesModeMI=new JMenuItem(switchInsertSpacesAction);
        spacesModeMI.setText("Insert Spaces Mode");
        markupMenu.add(spacesModeMI);
        
		//Scribbles Menu
        scribbleWidthM=new JMenu();
        scribbleWidthM.setText("Scribble Width");
        scribblesMenu.add(scribbleWidthM);
        for (int i=1 ; i<=15 ; i++) {
            item=new JMenuItem(new SetScribbleWidthAction(this, i));
            item.setText(""+i);
            scribbleWidthM.add(item);
        }
        
        //scribbleColorM=new JMenu();
        //scribbleColorM.setText("Scribble Color (NI)");
        //scribblesMenu.add(scribbleColorM);
        
		//Notes Menu
        //item=new JMenuItem();
        //item.setText("Insert New Note (NI)");
        //notesMenu.add(item);
        
        deleteSelectedMI=new JMenuItem(deleteSelectedAction);
        deleteSelectedMI.setText("Delete Selection");
        deleteSelectedMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0));
        notesMenu.add(deleteSelectedMI);
        
        notesMenu.addSeparator();
        
        noteColorM=new JMenu();
        noteColorM.setText("Note Color (NI)");
        notesMenu.add(noteColorM);
        
        notesMenu.addSeparator();
        
        //item=new JMenuItem();
        //item.setText("Expand Note (NI)");
        //notesMenu.add(item);
        
        //item=new JMenuItem();
        //item.setText("Collapse Note (NI)");
        //notesMenu.add(item);
        
		//help menu
        item=new JMenuItem(helpAction);
        item.setText("Blaise Grader Help");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_SLASH,aKey|InputEvent.SHIFT_MASK));//makes the "?"
            helpMenu.add(item);
            
            if(!Constants.isOSX()){
				///show the about menu on windows. (OSX places it in the Application menu)
                item=new JMenuItem(aboutAction);
                item.setText("About");
                helpMenu.add(item);
            }
    }
    
    /**
        * Returns this BlaiseGradingPane's menu structure.
     */
    public JMenuBar getMenuBar() {
        return menubar;
    }
    
    /*
     public JMenuBar initializeMenu(){
         String as[] = { "Open From File", "Mark Draft", "Save Markings", "Finalize Draft", "Finalize Paper", "Exit"};
         String as1[] = {"Undo Last Scribble", "Clear All Scribbles", "Delete Selected Scribble","Undo Last Gesture"};
         String as2[] = {"Scribble Mode", "Edit Mode", "Gesture Mode"};
         
         JMenuBar jmenubar = new JMenuBar();
         JMenu jmenu = new JMenu("File");
         JMenu jmenu1 = new JMenu("Edit");
         JMenu jmenu2 = new JMenu("Mode");
         for(int i = 0; i < as.length; i++){
             
             if(!parent.document.isEmpty() && i == 0)continue;
             JMenuItem jmenuitem = new JMenuItem(as[i]);
             jmenuitem.addActionListener(this);
             jmenu.add(jmenuitem);
         }
         
         formatScrib = new JMenuItem("Format Selected Scribble");
         formatScrib.addActionListener(this);
         jmenu1.add(formatScrib);
         formatScrib.setEnabled(false);
         for(int j = 0; j < as1.length; j++){
             JMenuItem jmenuitem1 = new JMenuItem(as1[j]);
             jmenuitem1.addActionListener(this);
             jmenu1.add(jmenuitem1);
         }
         
         for(int k = 0; k < as2.length; k++){
             JMenuItem jmenuitem2 = new JMenuItem(as2[k]);
             jmenuitem2.addActionListener(this);
             jmenu2.add(jmenuitem2);
         }
         
         jmenubar.add(jmenu);
         if(parent.current_draft != null){
             jmenubar.add(jmenu1);
             jmenubar.add(jmenu2);
         }
         
		 //parent.setJMenuBar(jmenubar);
         return jmenubar;
     }
     */
    
	//Initializes many of the GUI components
    private JToolBar initializeToolbar()
    {
       	String path = Constants.getIconPath();
        String scribbleIcon = "scribble.png";
        String editIcon = "edit.png";
        String gestureIcon = "gesture.png";
        String noteIcon = "noteicon.png";
        
		//Scribble Button
        scribbleButton = new JToggleButton(new ImageIcon(path + scribbleIcon));
        scribbleButton.setToolTipText("Scribble Mode");
        scribbleButton.setFocusable(false);
        scribbleButton.addActionListener(switchScribbleAction);
        scribbleButton.setSelected(true);
        
		//Edit Button
        editButton = new JToggleButton(new ImageIcon(path + editIcon));
        editButton.setToolTipText("Edit Mode");
        editButton.setFocusable(false);
        editButton.addActionListener(switchEditAction);
        
		//Spaces Button
        spacesButton = new JToggleButton(new ImageIcon(path + gestureIcon));
        spacesButton.setToolTipText("Insert Spaces Mode");
        spacesButton.setFocusable(false);
        spacesButton.addActionListener(switchInsertSpacesAction);
        
                //Notes Button
        notesButton = new JToggleButton(new ImageIcon(path + noteIcon));
        notesButton.setToolTipText("Add Notes Mode");
        notesButton.setFocusable(false);
        notesButton.addActionListener(switchNotesAction);
        /*
		//Gesture Button
         gestureButton = new JToggleButton(new ImageIcon(path + gestureIcon));
         gestureButton.setToolTipText("Gesture Mode");
         gestureButton.setFocusable(false);
         gestureButton.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent actionevent){
                 changeToGestureMode(actionevent);
             }
         });
         */
        
        //Create button group so that only one mode is active at a time
        ButtonGroup modeGroup=new ButtonGroup();
        modeGroup.add(scribbleButton);
        modeGroup.add(editButton);
        modeGroup.add(spacesButton);
        modeGroup.add(notesButton);
        
		//Add buttons to toolbar
        toolBar = new JToolBar();
        toolBar.setFloatable(false);
        
		//Mode buttons
        toolBar.addSeparator();
        toolBar.add(scribbleButton);
        toolBar.add(notesButton);
        toolBar.add(editButton);
        toolBar.add(spacesButton);
        /* toolBar.add(gestureButton); */
        toolBar.addSeparator();
        
		//Line formatting
        sizeBox = new JComboBox();
        String as[] = {
            "1", "2", "3", "4", "5", "6", "7", "8", "9", "10",
            "11", "12", "13", "14", "15"
        };
        for(int i = 0; i < as.length; i++)
            sizeBox.addItem(as[i]);
        
        sizeBox.setEditable(false);
        sizeBox.setSelectedIndex(width - 1);
        sizeBox.addItemListener(new ItemListener() {
            
            public void itemStateChanged(ItemEvent itemevent){
                try{
                    String value = (String)sizeBox.getSelectedItem();
                    if(value.equals("0")){
                        return;
                    }
                    width = Integer.parseInt(value);
                }
                catch(NumberFormatException numberformatexception) { }
            }
        });
        
        sizeBox.setPreferredSize(new Dimension(45,30));
        sizeBox.setMinimumSize(new Dimension(45,30));
        sizeBox.setMaximumSize(new Dimension(45,30));
        sizeBox.setSize(new Dimension(45,30));
        
        toolBar.add(sizeBox);
        toolBar.addSeparator();
        
		//Initialize color swatches
        c1 = new colorSwatch(Color.RED, true, thisPane);
        c2 = new colorSwatch(Color.ORANGE, false, thisPane);
        c3 = new colorSwatch(Color.YELLOW, false, thisPane);
        c4 = new colorSwatch(Color.GREEN, false, thisPane);
        c5 = new colorSwatch(Color.CYAN, false, thisPane);
        c6 = new colorSwatch(Color.BLUE, false, thisPane);
        c7 = new colorSwatch(new Color(114,0,203), false, thisPane);
        c8 = new colorSwatch(Color.MAGENTA, false, thisPane);
        c9 = new colorSwatch(Color.WHITE, false, thisPane);
        c10 = new colorSwatch(Color.WHITE, false, thisPane);
        
        toolBar.add(c1);
        toolBar.add(c2);
        toolBar.add(c3);
        toolBar.add(c4);
        toolBar.add(c5);
        toolBar.add(c6);
        toolBar.add(c7);
        toolBar.add(c8);
        toolBar.add(c9);
        toolBar.add(c10);
        
        return toolBar;
    }
    
    /*****************************************
        * MENU ACTIONS
        *****************************************/
    
    /**
        * Switches to Scribble Mode.
     */
    public class ScribbleAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="switch-scribble-mode";
        public ScribbleAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Switch to Scribble Mode");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            if(aSelection){
                scribbles.addElement(selected);
                aSelection = false;
            }
            layeredView.deselectAllIcons();
            state = CREATE_SCRIBBLE;
            scribbleButton.setSelected(true);
            myUIStateDelegate.updateUndoUI();
            repaint();
        }
    }
    
    /**
        * Switches to Edit Mode.
     */
    public class EditAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="switch-edit-mode";
        public EditAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Switch to Edit Mode");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            if(aSelection){
                scribbles.addElement(selected);
                aSelection = false;
            }
            layeredView.deselectAllIcons();
            state = EDIT_SCRIBBLE;
            editButton.setSelected(true);
            myUIStateDelegate.updateUndoUI();
            repaint();
        }
    }
    
    /**
        * Switches to Insert Spaces Mode.
     */
    public class InsertSpaceAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="switch-insert-space-mode";
        public InsertSpaceAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Switch to Insert Space Mode");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            if(aSelection){
                scribbles.addElement(selected);
                aSelection = false;
            }
            layeredView.deselectAllIcons();
            state = INSERT_SPACE;
            spacesButton.setSelected(true);
            myUIStateDelegate.updateUndoUI();
            repaint();
        }
    }
    
    /**
        * Switches to Notes Mode.
     */
    public class NotesAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="switch-notes-mode";
        public NotesAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Switch to Notes Mode");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            if(aSelection){
                scribbles.addElement(selected);
                aSelection = false;
            }
            state = CREATE_NOTES;
            notesButton.setSelected(true);
            myUIStateDelegate.updateUndoUI();
            repaint();
        }
    }
    
    /**
        * Handles the quitting of the application (standalone mode) or the closing of the main window (portal mode)
     */
    public class ExitAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="exit-application";
        public ExitAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Exit Blaise Grader");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            Thread thread=new Thread() {
                public void run() {
                    /*
                     int save=bgp.saveIfNeeded();
                     if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                         return;
                     }
                     else if(save==JOptionPane.YES_OPTION){
                         try{
                             bgp.saveThread.join();
                         }catch(InterruptedException e){
                         }
                     }
                     bgp.deleteExistingDir();//take care to close the paper.
                     */
                    
					// Call PortalToGraderInterface's done() method for clean-ups
                    parent.PtGI.done(parent.document.getPaperID() );
					//quit if standalone
                    if(STATEStandalone)
                        System.exit(0);
                    else
                        parent.done();
                }
            };
            thread.start();
        }
    }
    
    /**
	 * Handles the display of the Help pages
     */	
    public class HelpAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="help-action";
        public HelpAction(BlaiseGradingPane bgp) {
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Get help using the Blaise Grader");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            DocumentOpener docO=new DocumentOpener(new File("docs/grader/index.html"));
            if(!docO.open()){
                JOptionPane.showMessageDialog(bgp.parent, docO.getErrorMsg(), "Could not open help", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    /**
        * Linked to the Help->About or Application->About menu item
     */
    public class AboutAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="about-action";
        public AboutAction(BlaiseGradingPane bgp) {
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Find out about Blaise, and the Blaise Grader");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
            System.out.println("AboutAction");
        }
        
        public void actionPerformed(ActionEvent ae){
            new AboutDialog(bgp.parent,AboutDialog.GRADER_TYPE);
        }
    }
    
    /**
        * Basic support for undo/redo from O'Reilly's Java Swing (2nd ed) pg 828-830
     * and advanced (multiple undos from 658-660.
                     */
    public class UndoAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="undo-action";
        public UndoAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Undo the last performed action");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            try{
				// Depending on the mode, we will undo scribbles or normally
				// "Normal" Undo
				// bgp.undoManager.undo();
                if (state == CREATE_SCRIBBLE) {
					// Undo last written scribble
                    if(bgp.scribbles.size() > 0){
						//System.out.println("removeLastScribbles: ["+scribbles.size()+"/"+removedScribbles.size()+"]");
                        bgp.tempScribble = (Scribble)bgp.scribbles.remove(bgp.scribbles.size() - 1);
                        bgp.removedScribbles.add(bgp.tempScribble);
                        bgp.repaint();
                    } else {
						//System.out.println("Nothing to undo.");
                    }
                } else if (state == EDIT_SCRIBBLE) {					
                    bgp.undoManager.undo();
                } else if (state == INSERT_SPACE) {
                    if(bgp.spaces.size() > 0) {
                        HTMLEditorKit kit = (HTMLEditorKit)bgp.mainView.getEditorKit();
                        HTMLDocument doc = (HTMLDocument)bgp.mainView.getDocument();
                        Integer Caret;
                        
                        Caret = (Integer)(bgp.spaces.remove(bgp.spaces.size() - 1));
                        bgp.removedSpaces.add(Caret);
                        doc.remove(Caret.intValue(), 1);
                    } else {
						//System.out.println("Nothing to undo.");
                    }
                } else {
                    bgp.undoManager.undo();
                }
            }catch(CannotUndoException e){
                e.printStackTrace();
            }catch(Exception ex) {
                ex.printStackTrace();
            }
            finally{
                bgp.myUIStateDelegate.updateUndoUI();
            }
        }
    }
    
    public class RedoAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="redo-action";
        public RedoAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Redo the last undone action");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            try{
				// Depending on the mode, we will undo scribbles or normally
				// "Normal" Undo
				// bgp.undoManager.undo();
                if (state == CREATE_SCRIBBLE) {
					// Undo last written scribble
                    if(bgp.removedScribbles.size() > 0){
						//System.out.println("undoRemoveLastScribbles: ["+removedScribbles.size()+"/"+scribbles.size()+"]");
                        bgp.tempScribble = (Scribble)bgp.removedScribbles.remove(bgp.removedScribbles.size() - 1);
                        bgp.scribbles.add(tempScribble);
                        bgp.repaint();
                    } else {
						//System.out.println("Nothing to redo.");
                    }
                } else if (state == EDIT_SCRIBBLE) {					
                    bgp.undoManager.redo();
                } else if (state == INSERT_SPACE) {
                    if(bgp.removedSpaces.size() > 0) {
                        HTMLEditorKit kit = (HTMLEditorKit)bgp.mainView.getEditorKit();
                        HTMLDocument doc = (HTMLDocument)bgp.mainView.getDocument();
                        Integer prevCaret;		
                        
                        prevCaret = (Integer)(bgp.removedSpaces.remove(bgp.removedSpaces.size() - 1));
                        bgp.spaces.add(prevCaret);
                        kit.insertHTML(doc, prevCaret.intValue(), "<BR>", 0, 0, HTML.Tag.BR);
                    } else {
						//System.out.println("Nothing to redo.");
                    }
                } else {
                    bgp.undoManager.redo();
                }
            }catch(CannotRedoException e){
                e.printStackTrace();
            }catch(Exception ex){
                ex.printStackTrace();
            }
            finally{
                bgp.myUIStateDelegate.updateUndoUI();
            }
        }
    }
    
    /**
        * Linked to the File->Open menu item
     */
    public class OpenAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="open-paper";
        public OpenAction(BlaiseGradingPane bgpIN) {
            super(NAME);
            this.bgp=bgpIN;
            putValue(Action.SHORT_DESCRIPTION,"Open an existing Blaise Paper for editing");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
            System.out.println("OpenAction: "+NAME);
        }
        
        /**
            * Handles the opening of a file.  If the filename was received through other means
         * the actionCommand will be different from OpenAction.NAME
         */
        public void actionPerformed(final ActionEvent ae) {
            bgp.openThread=new Thread() {
                public void run() {
                    System.out.println("OpenAction.thread");
                    /* Save if needed
                    int save=bgp.saveIfNeeded();
                    if(save==JOptionPane.CANCEL_OPTION || save==JOptionPane.CLOSED_OPTION){//save if needed, return if cancelled.
                        return;
                    }
                    else if(save==JOptionPane.YES_OPTION){
                        try{
                            bgp.saveThread.join();
                        }catch(InterruptedException e){
                        }
                    }
                    bgp.deleteExistingDir();//take care to close the paper.
                    */
                    
                    System.out.println("ae.getActionCommand()");
                    String command=ae.getActionCommand();
                    File openFile;
                    if(command.equals(NAME)){
                        openFile = bgp.getFileFromChooser(bgp.parent, Constants.getUserDirectory(), JFileChooser.OPEN_DIALOG);
                    }
                    else{
						//Open file from the string.
                        openFile=new File(command);
                    }
                    if(openFile==null || (!openFile.exists())){//for whatever reason we don't have a filename... (they probably cancelled).
                        return;
                    }
                    if(DEBUG)   System.out.println("Opening Blaise Paper... (before try)");
                    try{
                        if(DEBUG)   System.out.println("Opening Blaise Paper... (after try)");
                        bgp.startProgView("Opening Blaise Paper...");
                        Blaise_Paper p=bgp.getBPFromFile(openFile);
                        if(p==null){
                            return;
                        }
                        bgp.parent.document=p;
                        
                        if(DEBUG)   System.out.println("Updating drafts window");
						//update the drafts pane and the bottom grading pane
                        SwingUtilities.invokeLater(new Runnable(){
							public void run(){
								bgp.parent.updateDraftsWindow();
								draftMessageLabel.setText("<html><h3>Please select the draft you would like to mark.</h3></html>");
								// If there was a paper being graded, remove the BCV and add the message label
								if (bgp.STATEMarkingModeActive) {
									bgp.remove(gradingView);
									bgp.add(draftMessageLabel);
									bgp.STATEMarkingModeActive = false;
								} else {
									System.out.println("****** STATEMarkingMode NOT ACTIVE ******");
								}
							}
						});
                        
						//bgp.parent.noActiveDraft();
                        bgp.parent.saveFile = openFile;
						//uistate stuff is thread safe...
                        bgp.myUIStateDelegate.setTitle(bgp.parent.saveFile.getAbsolutePath());
                        bgp.myUIStateDelegate.resetButtonsAndMenus();
                    }
                    catch(IOException e){
                        System.out.println("Uh oh, IOException...");
                        bgp.stopProgView();
                        JOptionPane.showMessageDialog(bgp.parent,"An Error Occurred that prevented the Blaise Document from opening.","Cannot Open Paper",JOptionPane.ERROR_MESSAGE);
                    }
                    finally{
                        bgp.stopProgView();
                    }
					
                }//end run
            };
            bgp.openThread.start();
        }
        
    }
    
    /**
        * Linked to File->Save Markings
     */
    public class SaveAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        public static final String NAME="save-paper";
        public SaveAction(BlaiseGradingPane bgpIN) {
            super(NAME);
            this.bgp=bgpIN;
            putValue(Action.SHORT_DESCRIPTION,"Saves changes made to the current Blaise Paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
            System.out.println("SaveAction: "+NAME);
        }
        
        public void actionPerformed(final ActionEvent ae) {
            bgp.saveThread=new Thread() {
                public void run() {
                    try {
                        // Start the progress dialog
                        bgp.startProgView("Saving Blaise Paper...");
                        
						// Add selected scribble back to scribble vector
                        if(bgp.aSelection) {
                            bgp.scribbles.addElement(selected);
                            bgp.aSelection = false;
                        }
                        
                        bgp.state = CREATE_SCRIBBLE;
                        
						// Sanity check
                        if(bgp.mainView == null) {
                            JOptionPane.showMessageDialog(bgp.parent
                                                          , "There was an error saving the draft.", "Save Error", JOptionPane.ERROR_MESSAGE);
                            return;
                        }
                        
						// Record the scribble canvas size
                        Dimension dimen = bgp.mainView.getSize();
                        
						// Get a Blaise_Comment to modify/create
                        Blaise_Comment comment = bgp.parent.current_draft.removeComment(bgp.graderName);
						System.out.println("Notes:"+bgp.notes);
                        if(comment == null) {
							/*leave this up to 
							/*GraderInfo gi=bgp.parent.PtGI.getGraderInfo();
							if(gi!=null){
								bgp.graderID=gi.getGraderID();
								bgp.graderName=gi.getName(false);
							}*/
							
                            comment = new Blaise_Comment("",
                                                         bgp.parent.document.getPaperID(),
                                                         bgp.graderID,
                                                         bgp.parent.current_draft.getDraftID(),
                                                         bgp.commentPrivate,
                                                         bgp.graderName,
                                                         bgp.assigned_grade,
                                                         Double.toString(dimen.getWidth()),
                                                         Double.toString(dimen.getHeight()),
                                                         bgp.mainView.getText(),
                                                         bgp.scribbles,
                                                         bgp.checkCorrections());
                            comment.setNotes(bgp.notes);
                        } else {
                            comment.setBody(bgp.mainView.getText());
                            comment.setAspectX(Double.toString(dimen.getWidth()));
                            comment.setAspectY(Double.toString(dimen.getHeight()));
                            comment.setScribbles(bgp.scribbles);
                            comment.setCorrections(bgp.checkCorrections());
                            comment.setAssignedGrade(bgp.assigned_grade);
                            comment.setNotes(bgp.notes);
                        }
                        bgp.parent.current_draft.addComment(comment);
                        bgp.parent.current_draft.setIsModified("YES");
                        //bgp.parent.updateDraftsWindow();
                        //bgp.parent.validate();
						
						// Write to file
                        if(bgp.parent.saveFile != null) {
							//Blaise_XML xml = new Blaise_XML();
							//xml.printPaperToFile(parent.saveFile.getAbsolutePath(), parent.document);
                            BlaiseFile bf=new BlaiseFile(bgp.parent.document,bgp.parent.saveFile,new GraderFileHandler());
                            bf.savePaper();
                            //bf.openPaper(); /* so the directory structure is updated. */
                            /*
                             bgp.myUIStateDelegate.setModified(false);
                             bgp.myUIStateDelegate.setTitle(saveFile.getAbsolutePath());
                             */
                        } else {
                            System.out.println("*** NO saveFile ***");
                        }
						
						bgp.openAction.actionPerformed(new ActionEvent(bgp,ActionEvent.ACTION_PERFORMED,bgp.parent.saveFile.getAbsolutePath()));
						try {
							bgp.openThread.join();						
						} catch (InterruptedException ie){
						}
					
						
                        /*
                         if(started){
                             parent.noActiveDraft();
                             parent.setStatus("  No Draft Selected For Marking");
                             parent.setResizable(true);
                         }
                         started = false;
                         */
                        
						// Remove and add BCV again (updates some variables)
                        //bgp.remove(gradingView);
                        //bgp.prepareBlaiseCommentView();
                        //bgp.add(gradingView);
                        
                    } catch(IOException e) {
                        bgp.stopProgView();
                        JOptionPane.showMessageDialog(bgp.parent,"An Error Occurred that prevented the Blaise Document from saving.","Cannot Save Paper",JOptionPane.ERROR_MESSAGE);
                        e.printStackTrace();
                    }
                    finally{
                        bgp.stopProgView();
                    }
                }//end run
            };
            bgp.saveThread.start();
        }
        
    }
    
    /**
        * Linked to Markup->Grade Draft menu item
     */
    public class GradeDraftAction extends AbstractAction {
        private BlaiseGradingPane bgp;
        private int selectedIndex;
        public static final String NAME="mark-draft-action";
        public GradeDraftAction(BlaiseGradingPane bgp) {
            this(bgp, -1);
        }		
        public GradeDraftAction(BlaiseGradingPane bgp, int index) {
            super(NAME);
            this.bgp = bgp;
            this.selectedIndex = index;
            putValue(Action.SHORT_DESCRIPTION, "Mark the specified draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(final ActionEvent ae){
            //Thread thread=new Thread(){
              //  public void run(){
                    if(bgp.parent.isSameDraft()){
                        return;
                    }
                    
                    if(bgp.parent.current_draft != null){
                        /* Need to finalize draft */
						//myThread = finishedDraftSession(actionevent);
                        if(DEBUG)	System.out.println("Do we need to save this draft?");
                    }
                    
                    if(selectedIndex == -1) {
                        selectedIndex = bgp.parent.draft_pane.getSelectedDraftIndex();
                    }
                    if(selectedIndex >= 0){
                        Blaise_Draft selectedDraft = bgp.parent.document.getDraft(selectedIndex);
                        if(bgp.parent.current_draft == null || !selectedDraft.equals(bgp.parent.current_draft)){
                            bgp.parent.current_draft = selectedDraft;
                            //bgp.parent.updateDraftsWindow();
                            
							// If there is a draft currently active, remove the BCV
                            if(STATEMarkingModeActive) {
                                bgp.remove(gradingView);
                                STATEMarkingModeActive = false;
                            }
							// Else, remove the message label
                            else {
                                bgp.remove(draftMessageLabel);
                            }
                            
							// Prepare BCV for marking
                            bgp.prepareBlaiseCommentView();
                            bgp.add(gradingView);
                            STATEMarkingModeActive = true;
                            
							// Enable marking mode menu items
							//bgp.myUIStateDelegate.markingMenuActivate();
							// Enable undo/redo menus
                            bgp.myUIStateDelegate.updateUndoUI();
                            
                            
                            bgp.parent.split.setDividerLocation((bgp.parent.getHeight())/3);
                            bgp.parent.split.validate();
                            
                            bgp.parent.setTitle("Blaise: "+bgp.parent.current_draft.getTitle());
                            bgp.parent.validate();
                            
                            System.out.println("----------");
                            System.out.println("heightAchiever:"+heightAchiever);
                            System.out.println("widthAchiever:"+widthAchiever);
                            System.out.println("tempPanel:"+tempPanel);
                            System.out.println("layeredView:"+layeredView);
                            System.out.println("scroll"+scroll);
                            System.out.println("gradingView:"+gradingView);
                            System.out.println("mainView:"+mainView);
                        }
                    }
                    
                //}
            //};
            //thread.start();
        }
    }
    
    /**
        * Linked to Markup->Add External Grade
     */
    public class AddExternalGradeAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="add-external-draft-action";
        public AddExternalGradeAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Set the specified external document as the current grading draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(final ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    //ask what file they'd like to attach.
                    File attachFile=getFileFromChooser(bgp.parent,null,JFileChooser.OPEN_DIALOG,false);//just need a number that isn't OPEN_DIALOG or SAVE_DIALOG
                    if(attachFile==null){
                        return;
                    }
                    
                    int answer = JOptionPane.showConfirmDialog(bgp.parent,"A copy of the selected file will be made and attached as this grading draft.\nAny new changes made to the original document will not be reflected in this copy." , "Confirm Attachment Request", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
                    if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.CLOSED_OPTION){
                        return;
                    }
                    String backupText=bgp.mainView.getText();//so we can undo if an IOException happens.
                        
                        File internalFile=null;
                        try{
                            bgp.startProgView("Attaching external draft as working body...");
                            String papDir=Constants.getClientPaperDirectory();
                            String attachFilename=attachFile.getName();
                            internalFile=new File(papDir+"WorkingDraft/Files/"+attachFilename);
							//copy the old file into the new file...
                            DocumentUtilities.duplicateFile(attachFile,internalFile);
                            Blaise_Draft draft=bgp.parent.document.getWorkingDraft();
                            draft.setExternalBodyLink(internalFile);
							//internalFile.setReadOnly();
                            
							//now update the gradingView stuff.
                            bgp.myUIStateDelegate.setExternalBody(true);
                        }
                        catch(IOException e){
							//an error occurred.  Tell the user, and then remove any external body...
                            JOptionPane.showMessageDialog(bgp.parent, e.toString(), "Could not attach document", JOptionPane.ERROR_MESSAGE);
                            Blaise_Draft draft=bgp.parent.document.getWorkingDraft();
                            draft.removeExternalBody();
                            bgp.myUIStateDelegate.setExternalBody(false);//undo any shiznit that changed in the UI.
                                bgp.myUIStateDelegate.setModified(false);//undo that.
                                    if(internalFile!=null && internalFile.exists()){
                                        internalFile.delete();
                                    }
                                    bgp.mainView.setText(backupText);
                        }
                        finally{
                            bgp.stopProgView();
                        }
                }
            };
            thread.start();
        }
    }
    
    public class RemoveExternalDraftAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="remove-external-draft-action";
        public RemoveExternalDraftAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Detach the external document from being the current grading draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(final ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    Blaise_Draft draft=bgp.parent.document.getWorkingDraft();
                    File internalFile=draft.getExternalBodyLink();
                    draft.removeExternalBody();
                    bgp.myUIStateDelegate.setExternalBody(false);
                    if(internalFile!=null && internalFile.exists()){
                        internalFile.delete();
                    }
                }
            };
            thread.start();
        }
    }
    
    public class ShowDraftAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        private int index;
        public static final String NAME="show-draft-action";
        public ShowDraftAction(BlaiseGradingPane bgp, int index){
            super(NAME);
            this.bgp=bgp;
            this.index=index;
            
            putValue(Action.SHORT_DESCRIPTION,"Show the selected graded draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            bgp.parent.draft_pane.selectDraft(index);
        }
    }
    
    public class FinishGradingDraftAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="finish-grading-draft-action";
        public FinishGradingDraftAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Finish grading the selected draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
					/* First, we want to save the current paper by calling saveAction's actionPerformed. */
					bgp.saveAction.actionPerformed(null);
					System.out.println("FinishGradingDraft - Saved.");
					
					/* We need to call the setStatusWorking in the current instance of 
						 PortalToGraderInterface, which can be accessed through the BlaiseSplitPane.
						 Of course, the BlaiseSplitPane can be accessed through 'parent'. */
					bgp.parent.PtGI.setStatusWorking(bgp.parent.document.getPaperID() );
					System.out.println("FinishGradingDraft - Portal notified of Working.");

					// Wait until the save thread finishes
					try {
						bgp.saveThread.join();						
					} catch (InterruptedException ie){
					}

					/* Set the displayed message to be the draft selection message, then remove the BCV from the bottom pane.
						 Adding the message back to the bottom pane will remove the draft and make things look nice.
						*/					
					draftMessageLabel.setText("<html><h3>Please select the draft you would like to mark.</h3></html>");
					if (bgp.STATEMarkingModeActive) {
						bgp.remove(gradingView);
						bgp.add(draftMessageLabel);
						bgp.STATEMarkingModeActive = false;
					}

					System.out.println("FinishGradingDraft - BCV removed.");
				}
    }
    
    public class FinishGradingPaperAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="finish-grading-paper-action";
        public FinishGradingPaperAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Finish grading the current paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
					/* First, we want to save the current paper by calling saveAction's actionPerformed. */
					bgp.saveAction.actionPerformed(null);
					
					/* We need to call the setStatusWorking in the current instance of 
					PortalToGraderInterface, which can be accessed through the BlaiseSplitPane.
					Of course, the BlaiseSplitPane can be accessed through 'parent'. */
					bgp.parent.PtGI.setStatusGraded(bgp.parent.document.getPaperID() );

					// Wait until the save thread finishes
					try {
						bgp.saveThread.join();						
					} catch (InterruptedException ie){
					}

					/* Set the displayed message to be the draft selection message, then remove the BCV from the bottom pane.
						Adding the message back to the bottom pane will remove the draft and make things look nice.
						*/
					draftMessageLabel.setText("<html><h3>Please select the draft you would like to mark.</h3></html>");
					if (bgp.STATEMarkingModeActive) {
						bgp.remove(gradingView);
						bgp.add(draftMessageLabel);
						bgp.STATEMarkingModeActive = false;
					}
					
					// We need to close the grader if this is from the portal, close the document if it's standalone.
					bgp.parent.done();
        }
    }
    
    public class PaperInformationAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="paper-information-action";
        public PaperInformationAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Get information about the current paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //To make this code a little more flexible in terms of where the containers
            //are above bgp, we'll enter into a loop to find a Frame.
            Container c=bgp;
            do{
                c=c.getParent();
            }while(!(c instanceof Frame));
            new PaperInfoDialog((Frame)c,bgp.parent.document);
        }
    }
    
    public class FindInTextAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="find-in-text-action";
        public FindInTextAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Find a string of text in the current draft");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //This only partially works--due to the modifications to the JTextPane
            //the text does not get scrolled to or highlighted, but a scan and find is
            //done...
            bgp.findFrame=FindFrame.showFind(bgp.mainView);
        }
    }
    
    public class FindInNotesAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="find-in-notes-action";
        public FindInNotesAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Find a string of text in the curret set of notes");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            
        }
    }
    
    public class ShowHideNotesAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="show-hide-notes-action";
        public ShowHideNotesAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Show or hide common notes");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            
        }
    }
    
    public class SetScribbleWidthAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        private int index;
        public static final String NAME="set-scribble-width-action";
        public SetScribbleWidthAction(BlaiseGradingPane bgp, int index){
            super(NAME);
            this.bgp=bgp;
            this.index=index;
            
            putValue(Action.SHORT_DESCRIPTION,"Set scribble to selected width");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            bgp.width = index;
            sizeBox.setSelectedIndex(bgp.width - 1);
            
        }
    }
    
    public class DeleteSelectedAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        private int index;
        public static final String NAME="delete-selected-action";
        public DeleteSelectedAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            this.index=index;
            
            putValue(Action.SHORT_DESCRIPTION,"If in Edit mode, delete any selected scribbles or notes");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            Vector selNotes=bgp.layeredView.getSelectedIconNotes();
            layeredView.removeSelectedNotes();
            int size=selNotes.size();

            for(int i=0;i<size;i++){
                bgp.notes.remove(selNotes.elementAt(i));
            }

            if(aSelection){//scribbles
                selected=null;
                aSelection = false;
            }
            bgp.repaint();
        }
    }
    
    public class SetModifiedAction extends AbstractAction{
        private BlaiseGradingPane bgp;
        public static final String NAME="set-modified-action";
        public SetModifiedAction(BlaiseGradingPane bgp){
            super(NAME);
            this.bgp=bgp;
            
            putValue(Action.SHORT_DESCRIPTION,"Set this document to be modified");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            bgp.myUIStateDelegate.setModified(true);
        }
    }
    
    
	///////////////////////////////////////////////////////////////////////////
	//
	// Actions
	//
	///////////////////////////////////////////////////////////////////////////
    Thread myThread = new Thread();
    boolean cancel = false;
    public void actionPerformed(final ActionEvent actionevent){
        
        //File Menu
        if(actionevent.getActionCommand().compareTo("Exit") == 0){
            if(parent.current_draft != null){
                myThread = saveMarks(actionevent);
            }
            
            Thread thread = new Thread () {
                public void run(){
                    try {
                        myThread.join();
                        if(parent.document != null){
                            int answer = JOptionPane.showConfirmDialog(thisPane, "Would you like to finalize the current paper?", "Would You Like To Finalize?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                            if(answer == 2){		//CANCEL
                                return;
                            }
                            else if(answer == 1){	//NO
                                if(!cancel)parent.done();
                            }
                            else{ //YES
                                actionPerformed(new ActionEvent(" ", 0, "Finalize Paper"));
                            }
                        }
                    }
                    catch (InterruptedException e){
                    }
                }
            };
            thread.start ();
        }
        else if(actionevent.getActionCommand().compareTo("Finalize Paper") == 0){
            cancel = false;
            if(parent.current_draft != null){
                myThread = finishedDraftSession(actionevent);
                
				//JOptionPane.showMessageDialog(parent, "Please finalize the current draft before finalizing the entire paper.", "Finalization Needed", JOptionPane.ERROR_MESSAGE);
				//return;
            }
            if(!cancel && myThread != null){
                Thread thread = new Thread () {
                    public void run(){
                        try {
                            myThread.join();
                            if(!cancel){
                                thisPane.finishedPaperSession(actionevent);
                                if(parent.saveFile != null){
                                    Blaise_XML xml = new Blaise_XML();
                                    xml.printPaperToFile(parent.saveFile.getAbsolutePath(), parent.document);
                                }
                                else{
                                    TabletPapers.removeUngraded(parent.document);
                                    TabletPapers.removeGraded(parent.document);
                                    TabletPapers.addGradedPaper(parent.document);
                                    TabletPapers.writeUngradedFile();
                                    TabletPapers.writeGradedFile();
                                    TabletPapers.reloadTables();
                                }
                                
                                JOptionPane.showMessageDialog(parent, "Paper was finalized successfully.", "Success", JOptionPane.INFORMATION_MESSAGE);
                                parent.done();
                            }
                        }
                        catch (InterruptedException e){
                        }
                    }
                };
                thread.start ();
            }
        }
        else if(actionevent.getActionCommand().compareTo("Mark Draft") == 0){
            
            if(parent.isSameDraft()){
                return;
            }
            
            cancel = false;
            if(parent.current_draft != null){
				//JOptionPane.showMessageDialog(parent, "Please finalize the current draft before starting to mark a new draft.", "Finalization Needed", JOptionPane.ERROR_MESSAGE);
                myThread = finishedDraftSession(actionevent);
            }
            
            if(!cancel && myThread != null){
                Thread thread = new Thread () {
                    public void run(){
                        try {
                            myThread.join();
                            parent.markDraft();
                        }
                        catch (InterruptedException e){}
                    }
                };
                thread.start ();
            }
            else{
                parent.markDraft();
            }
        }
        else if(actionevent.getActionCommand().compareTo("Finalize Draft") == 0){
            if(parent.current_draft != null){
                finishedDraftSession(actionevent);
            }
            else{
                System.out.println("Error"); //<MG 3/31/05> is this useful?  I don't know
            }
        }
        else if(actionevent.getActionCommand().compareTo("Save Markings") == 0){
            if(parent.current_draft != null){
                saveMarks(actionevent);
				//parent.updateDraftsWindow();
            }
        }
        else if(actionevent.getActionCommand().compareTo("Open From File") == 0){
            cancel = false;
            if(parent.current_draft != null){
                myThread = finishedDraftSession(actionevent);
                
                if(!cancel && myThread != null){
                    Thread thread = new Thread () {
                        public void run(){
                            try {
                                myThread.join();
                                
								//Prompt for file to open. Will only open .kjm
                                parent.saveFile = getFileFromChooser(parent, Constants.getUserDirectory(), JFileChooser.OPEN_DIALOG);
                                
								//If the file exists and is the proper format
                                if(parent.saveFile == null){
                                    return;
                                }
                                
                                if((parent.saveFile.getName()).endsWith(".kjm")){
                                    doThread(BlaiseGradingPane.OPEN, "Opening paper...");
                                }
                            }
                            catch (InterruptedException e){}
                        }
                    };
                    thread.start ();
                }
            }
            else{
				//Prompt for file to open. Will only open .kjm
                parent.saveFile = getFileFromChooser(parent, Constants.getUserDirectory(), JFileChooser.OPEN_DIALOG);
                
				//If the file exists and is the proper format
                if(parent.saveFile == null){
                    return;
                }
                
                if((parent.saveFile.getName()).endsWith(".kjm")){
                    doThread(BlaiseGradingPane.OPEN, "Opening paper...");
                    parent.standalone = false;
                }
            }
        }
        
        //Edit Menu
        else if(actionevent.getActionCommand().compareTo("Format Selected Scribble") == 0){
			//Opend FormatGesture Dialog ot format the line color and width
            FormatScribble format;
			//If a scribble is selected, format that one
            if(aSelection){
                format = new FormatScribble(this, selected.getWidth(), selected.getColor());
                format.setVisible(true);
                if(format.isValid()){
                    selected.setWidth(format.getLineWidth());
                    selected.setColor(format.getLineColor());
                    deactivateSwatches();
                }
            }
			//otherwise set the global width and color parameters
            else{
                format = new FormatScribble(this, width, color);
                format.setVisible(true);
                if(format.isValid()){
                    width = format.getLineWidth();
                    color = format.getLineColor();
                    sizeBox.setSelectedIndex(width-1);
                    deactivateSwatches();
                }
            }
            format.dispose();
        }
        else if(actionevent.getActionCommand().compareTo("Undo Last Gesture") == 0){
            
            if(lastGesture == NONE)return;
            
            if(lastGesture == SPELLING){
                CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(0);
                correctionset.removeLast();
            }
            else if(lastGesture == CAPITALIZE){
                CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(1);
                correctionset.removeLast();
            }
            else if(lastGesture == DELETE){
                CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(2);
                correctionset.removeLast();
            }
            
            lastGesture = NONE;
            repaint();
        }
        else if(actionevent.getActionCommand().compareTo("Undo Last Scribble") == 0)
            removeLastScribbles();
        else if(actionevent.getActionCommand().compareTo("Clear All Scribbles") == 0)
            removeAllScribbles();
        else if(actionevent.getActionCommand().compareTo("Delete Selected Scribble") == 0){
            aSelection = false;
            selected = new Scribble();
            repaint();
        }
        
        //Mode Menu
        else if(actionevent.getActionCommand().compareTo("Edit Mode") == 0)
            changeToEditMode(actionevent);
        else if(actionevent.getActionCommand().compareTo("Scribble Mode") == 0)
            changeToScribbleMode(actionevent);
        /*
         else if(actionevent.getActionCommand().compareTo("Gesture Mode") == 0)
         changeToGestureMode(actionevent);
         */
    }
    
    Thread temp_thread;
    
	//Called when a user wished to save their grading session
    private Thread saveMarks(ActionEvent actionevent){
        
        Thread thread = doThread(BlaiseGradingPane.SAVE, "Saving current draft...");
        return thread;
    }
    
    private Thread doThread(final byte type, String message){
        
    	// Start the indeterminate version.
    	progView = new ProgressView ((JFrame)parent, message, 0, true);
        
    	// Fire off the actual work as a thread:
    	Thread thread = new Thread () {
            public void run ()
        {
                
                if(type == BlaiseGradingPane.SAVE) doSave();
                else if(type == BlaiseGradingPane.OPEN) doOpen();
        }
    	};
    	//SwingUtilities.invokeLater (thread);
        thread.start();
        return thread;
    }
    
    private void doSave(){
        //Add selected scribble back to scribble vector
        if(aSelection){
            scribbles.addElement(selected);
            aSelection = false;
        }
        state = CREATE_SCRIBBLE;
        
        if(mainView == null){
            JOptionPane.showMessageDialog(this, "There was an error saving the draft.", "Error Saving", JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        Dimension dimen = mainView.getSize();
        
        Blaise_Comment comment = parent.current_draft.removeComment(graderName);
        if(comment == null){
            comment = new Blaise_Comment("", parent.document.getPaperID(), graderID, parent.current_draft.getDraftID(), commentPrivate, graderName, assigned_grade, Double.toString(dimen.getWidth()), Double.toString(dimen.getHeight()), mainView.getText(), scribbles, checkCorrections());
        }
        else{
            comment.setBody(mainView.getText());
            comment.setAspectX(Double.toString(dimen.getWidth()));
            comment.setAspectY(Double.toString(dimen.getHeight()));
            comment.setScribbles(scribbles);
            comment.setCorrections(checkCorrections());
            comment.setAssignedGrade(assigned_grade);
        }
        parent.current_draft.addComment(comment);
        parent.current_draft.setIsModified("YES");
        parent.updateDraftsWindow();
        parent.validate();
        //repaint();
        
        if(parent.saveFile != null){
            Blaise_XML xml = new Blaise_XML();
            xml.printPaperToFile(parent.saveFile.getAbsolutePath(), parent.document);
        }
        else{
            TabletPapers.removeUngraded(parent.document);
            TabletPapers.addUngradedPaper(parent.document);
            TabletPapers.writeUngradedFile();
        }
        
        progView.done();
        
        if(started){
            parent.noActiveDraft();
            parent.setStatus("  No Draft Selected For Marking");
            parent.setResizable(true);
        }
        started = false;
    }
    
    private void doOpen(){
        
		//Parse document XML into Blaise_Paper object
        Blaise_XML xml = new Blaise_XML();
        Vector paperVec = xml.parse(parent.saveFile.getAbsolutePath());
        
		//If no paper found, reject writer from opening document
        if(paperVec.size() == 0){
            JOptionPane.showMessageDialog(this, "You do not have permission to modify that file.", "Access Violation", JOptionPane.ERROR_MESSAGE);
            return;
        }
        else{
            
			//Extract paper
            parent.document = (Blaise_Paper)paperVec.elementAt(0);
            
			//Set the text of the JTextPane with the contents of the document just openned
            parent.updateDraftsWindow();
            parent.noActiveDraft();
        }
        
        progView.done();
    }
    
	//Determines if there are any corrections of value
    private Vector checkCorrections(){
        for(int i=0;i<corrections.size();i++){
            CorrectionSet set = (CorrectionSet)corrections.elementAt(i);
            if(set.size() > 0){
                return corrections;
            }
        }
        return new Vector();
    }
    
    private Thread finishedDraftSession(ActionEvent actionevent){
        
		//Get Grade (optional)
        assigned_grade = (String)JOptionPane.showInputDialog(this,"Please enter a grade for this draft (optional).", "Assign Grade", JOptionPane.QUESTION_MESSAGE, null, null, "");
        
		//If email is not valid, exit application
        if(assigned_grade == null){
            cancel = true;
            return new Thread();
        }
        
        commentPrivate = "NO";
		//Prompt to make marking public or not
        int answer = JOptionPane.showConfirmDialog(this, "Make markings on current draft visible to other graders?", "Make Current Draft Comments Private?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
        
		//if yes --> answer == 0
        if(answer == 0){
            commentPrivate = "YES";
        }
        else if(answer == 1){
            commentPrivate = "NO";
        }
        else if(answer == 2){
            cancel = true;
            return new Thread();
        }
        
        started = true;
        return saveMarks(actionevent);
    }
    
	//Called when a user is finished grading a document
    private void finishedPaperSession(ActionEvent actionevent){
        //Ensure all scribbles are present and not "selected"
        if(aSelection){
            scribbles.addElement(selected);
            aSelection = false;
        }
        state = CREATE_SCRIBBLE;
        repaint();
    }
    
	//Changes to scribble mode
    private void changeToScribbleMode(ActionEvent actionevent){
        if(aSelection){
            scribbles.addElement(selected);
            aSelection = false;
        }
        state = CREATE_SCRIBBLE;
        scribbleButton.setSelected(true);
        myUIStateDelegate.updateUndoUI();
        repaint();
    }
    /*
	//Changes to gesture mode
     private void changeToGestureMode(ActionEvent actionevent){
         if(aSelection){
             scribbles.addElement(selected);
             aSelection = false;
         }
         
         state = GESTURE_RECOGNITION;
         gestureButton.setSelected(true);
         repaint();
     }
     */
    
	//Changes to edit mode
    private void changeToEditMode(ActionEvent actionevent){
        if(aSelection){
            scribbles.addElement(selected);
            aSelection = false;
        }
        state = EDIT_SCRIBBLE;
        editButton.setSelected(true);
        myUIStateDelegate.updateUndoUI();
        repaint();
    }
    
	////////////////////////////////////////////////////////////
	//
	// Color Methods
	//
	///////////////////////////////////////////////////////////
	//Deactivates swatches
    public void deactivateSwatches(){
        c1.setActive(false);
        c2.setActive(false);
        c3.setActive(false);
        c4.setActive(false);
        c5.setActive(false);
        c6.setActive(false);
        c7.setActive(false);
        c8.setActive(false);
        c9.setActive(false);
        c10.setActive(false);
    }
    
	//Sets a swatch as active
    public void setActive(colorSwatch newSwatch){
        color = newSwatch.getColor();
        changeToScribbleMode(new ActionEvent(new Object(), 1, "Scribble"));
    }
    
	//Fills a swatch with a new color from FormatScribble's color selection modes
    public void getNewColor(colorSwatch colorswatch){
        FormatScribble format = new FormatScribble(this, width, color);
        format.setVisible(true);
        if(format.isValid()){
            width = format.getLineWidth();
            color = format.getLineColor();
            colorswatch.setColor(color);
        }
    }
    
    
    ///////////////////////////////////////////////////////////
    //
    // BlaiseCommentViewInterface Interface
    //
    //////////////////////////////////////////////////////////
    public Vector getScribbles(){ return scribbles;}
    public Vector getNotes(){return notes;}
    public Vector getCorrections(){ return corrections;}
    public boolean areScribblesVisible(){ return true;}
    public boolean areCorrectionsVisible(){ return true;}
    public Scribble getSelected(){
        if(aSelection){
            return selected;
        }
        else{
            return null;
        }
    }
    
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // ComponentListner Interface
    //
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
    public void componentResized(ComponentEvent e) {
        resize();
    }
    
    public void componentHidden(ComponentEvent e) {}
    public void componentMoved(ComponentEvent e) {}
    public void componentShown(ComponentEvent e) {}
    
    
    ///////////////////////////////////////////////////////////
    //
    // Utilities
    //
    //////////////////////////////////////////////////////////
    /**
        * Method for obtaining a File for input/output using a JFileChooser dialog
     *
     * @param mainView The JTextPane representing the contents of the document
     * @param startDir Directory where file chooser should begin
     * @param dialogType Type of file being chosen. [Open or Save]
     * @return File chosen by the writer
     */
    /*
     private File getFileFromChooser(JFrame mainView, String startDir, int dialogType) {
         
		 //Initialize new chooser
         JFileChooser jfileDialog = new JFileChooser(startDir);
         jfileDialog.setDialogType(dialogType);
         int optionSelected = JFileChooser.CANCEL_OPTION;
         
		 //Based on type, determine allowed file filters
         if(dialogType == JFileChooser.OPEN_DIALOG) {
             jfileDialog.setMultiSelectionEnabled(false);
             jfileDialog.addChoosableFileFilter(new GraderFileFilter("kjm","A Blaise Paper File (*.kjm)"));
             jfileDialog.addChoosableFileFilter(new GraderFileFilter("kjm","A Blaise Paper File (*.bls)"));
             optionSelected = jfileDialog.showOpenDialog(mainView);
         }
         else if(dialogType == JFileChooser.SAVE_DIALOG) {
             jfileDialog.setMultiSelectionEnabled(false);
             jfileDialog.addChoosableFileFilter(new GraderFileFilter("kjm","A Blaise Paper File (*.kjm)"));
             optionSelected = jfileDialog.showSaveDialog(mainView);
         }
         else {
             optionSelected = jfileDialog.showOpenDialog(mainView);
         }
         
		 //If approved operation is selected, return file
         if(optionSelected == JFileChooser.APPROVE_OPTION) {
             return jfileDialog.getSelectedFile();
         }
         return (File)null;
     }
     */
	//Determines if a given String is an end-of-word character
    private boolean isEnded(String s){
        return s.equals(" ") || s.equals(".") || s.equals(",") || s.equals("?") || s.equals("(") || s.equals(";") || s.equals("\n") || s.equals("\t");
    }
    
    public static void main(String args[]) throws Exception{
        
        Blaise_XML xml = new Blaise_XML();
		//Vector papers = xml.parse("E:\\Blaise\\data\\blaise2.kjm");
        Vector papers = xml.parse("D:\\Blaise\\data\\blaise_grade.kjm");
        new BlaiseSplitPane(BlaiseSplitPane.GRADER, (Blaise_Paper)papers.elementAt(0));
    }
    
    /**
        * Anything that changes the UIState should be done through this class.
     * Much of the time it just passes on the request, but sometimes it will check to see if
     * the request is valid for the current state, or perform other checking actions.
     *
     * @author Michael Gaiman
     * @modified Eisuke Arai
     * @version 1.2 7/26/2004
     */
    private class UIStateDelegate implements CaretListener, DraftViewListener, UndoableEditListener{
        BlaiseGradingPane bgp;
        public UIStateDelegate(BlaiseGradingPane bgpIN){
            this.bgp=bgpIN;
            System.out.println("In");
            /*
             bgp.mainView.addCaretListener(this);
             */
            System.out.println("Out");
            resetButtonsAndMenus();
        }
        /**
            * Unlike most UIStateDelegate methods, caretUpdate isn't threadsafe.
         */
        public void caretUpdate(final CaretEvent e){
        }
        /*
         public void caretUpdate(final CaretEvent e){
             int caret = e.getDot();
             int end = e.getMark();
             
             boolean selection = (caret!=end);
             bgp.cutMI.setEnabled(selection);
             bgp.cutB.setEnabled(selection);
             bgp.copyMI.setEnabled(selection);
             bgp.copyB.setEnabled(selection);
             
			 //want formatting to be from left side of selection
             if(end < caret){
                 caret = end;
             }
             AttributeSet a;
             if(caret==0){
                 a = ((JTextPane)(e.getSource())).getStyledDocument().getCharacterElement(caret).getAttributes();
             }
             else{
                 a = ((JTextPane)(e.getSource())).getStyledDocument().getCharacterElement(caret-1).getAttributes();
             }
             if(StyleConstants.isBold(a)){
                 bgp.boldTB.setSelected(true);
                 bgp.boldCBMI.setSelected(true);
             }
             else{
                 bgp.boldTB.setSelected(false);
                 bgp.boldCBMI.setSelected(false);
             }
             if(StyleConstants.isItalic(a)){
                 bgp.italicTB.setSelected(true);
                 bgp.italicCBMI.setSelected(true);
             }
             else{
                 bgp.italicTB.setSelected(false);
                 bgp.italicCBMI.setSelected(false);
             }
             
             if(StyleConstants.isUnderline(a)){
                 bgp.underlineTB.setSelected(true);
                 bgp.underlineCBMI.setSelected(true);
             }
             else{
                 bgp.underlineTB.setSelected(false);
                 bgp.underlineCBMI.setSelected(false);
             }
             
             int align = StyleConstants.getAlignment(a);
             if(align == StyleConstants.ALIGN_LEFT){
                 bgp.leftAlignTB.setSelected(true);
                 bgp.leftAlignCBMI.setSelected(true);
             }
             else if(align == StyleConstants.ALIGN_CENTER){
                 bgp.centerAlignTB.setSelected(true);
                 bgp.centerAlignCBMI.setSelected(true);
             }
             else if(align == StyleConstants.ALIGN_RIGHT){
                 bgp.rightAlignTB.setSelected(true);
                 bgp.rightAlignCBMI.setSelected(true);
             }
             
			 //update the finalizeDraftMI and submitMI
             if(bgp.mainView.isEmpty()){
                 bgp.finalizeDraftMI.setEnabled(false);
                 bgp.submitPaperMI.setEnabled(bgp.parent.document.hasDrafts());
             }
             else{
                 bgp.finalizeDraftMI.setEnabled(true);
                 bgp.submitPaperMI.setEnabled(true);
             }
         }
         */
        
        /**
            * Handles the updating of the draft and comment submenus.
         */
        public void draftViewChanged(DraftViewEvent e){
            System.out.println(e);
            
            int change=e.getChangeType();
            if(change==e.VIEW_UPDATE){
                populateGradeDraftMenu();
                populateSelectDraftMenu();
            }
            else if(change==e.DRAFT_SELECTED){
                populateGradeDraftMenu(e.getSelectedIndex());
                populateSelectDraftMenu(e.getSelectedIndex());
				 //populateShowGradedMenu(((BlaiseDraftView)e.getSource()).getSelectedCommentIndex());
            }
            /*
             else if(change==e.COMMENT_SELECTED){
                 System.out.println("COMMENT_SELECTED");
                 populateShowGradedMenu(e.getSelectedIndex());
                 populateSelectDraftMenu(e.getSelectedIndex());
             }
             */
        }
        
        /**
            * Takes care of the undo manager and updating UI.
         */
        public void undoableEditHappened(UndoableEditEvent e){
            System.out.println(e);
            bgp.undoManager.addEdit(e.getEdit());
            updateUndoUI();
        }
        
        public void updateUndoUI(){
            int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
            
            System.out.println("updateUndoUI()");
            bgp.undoMI.setText(bgp.undoManager.getUndoPresentationName());
            bgp.redoMI.setText(bgp.undoManager.getRedoPresentationName());
            
            if (state == CREATE_SCRIBBLE) {
                if(bgp.scribbles.size() > 0){
                    bgp.undoMI.setEnabled(true);
                    bgp.undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
                } else {
                    bgp.undoMI.setEnabled(false);
                }
                
                if(bgp.removedScribbles.size() > 0) {
                    bgp.redoMI.setEnabled(true);
                    bgp.redoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
                } else {
                    bgp.redoMI.setEnabled(false);
                }
            } else if (state == EDIT_SCRIBBLE) {					
                bgp.undoMI.setEnabled(bgp.undoManager.canUndo());
                bgp.undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
                bgp.redoMI.setEnabled(bgp.undoManager.canRedo());
                bgp.redoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
            } else if (state == INSERT_SPACE) {
                if(bgp.spaces.size() > 0){
                    bgp.undoMI.setEnabled(true);
                    bgp.undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
                } else {
                    bgp.undoMI.setEnabled(false);	
                }
                
                if(bgp.removedSpaces.size() > 0) {
                    bgp.redoMI.setEnabled(true);
                    bgp.undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
                } else {
                    bgp.redoMI.setEnabled(false);
                }
            } else {
                bgp.undoMI.setEnabled(bgp.undoManager.canUndo());
                bgp.undoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
                bgp.redoMI.setEnabled(bgp.undoManager.canRedo());
                bgp.redoMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, aKey));
            }
            
            
            /*
             bgp.undoMI.setText(bgp.undoManager.getUndoPresentationName());
             bgp.redoMI.setText(bgp.undoManager.getRedoPresentationName());
             bgp.undoMI.setEnabled(bgp.undoManager.canUndo());
             bgp.redoMI.setEnabled(bgp.undoManager.canRedo());
             */
        }
        
        /**
            * Single routine that consolidates updating general paper-related menu items
         * such as Save, Paper Information, etc.
         */
        public void updatePaperUI() {
            if(DEBUG)	System.out.println("updatePaperUI()");
            int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
            
            bgp.saveMI.setEnabled(true);
            
            bgp.scribbleWidthM.setEnabled(true);
            //bgp.scribbleColorM.setEnabled(true);
            bgp.noteColorM.setEnabled(true);
            
            bgp.scribbleModeMI.setEnabled(true);
            bgp.editModeMI.setEnabled(true);
            bgp.notesModeMI.setEnabled(true);
            bgp.spacesModeMI.setEnabled(true);
            
            bgp.finishGradingDraftMI.setEnabled(true);
            bgp.finishGradingPaperMI.setEnabled(true);
            
            bgp.paperInfoMI.setEnabled(true);
            
            //bgp.findTextMI.setEnabled(true);
            //bgp.findNotesMI.setEnabled(true);
            
            bgp.deleteSelectedMI.setEnabled(true);
            
            bgp.mainView.getDocument().addUndoableEditListener(this);
            setModified(false);
        }
        
        /**
            * Sets the title to "Blaise Editor: <title string>" or if title is null, the current document number.
         * Thread safe.
         */
        public void setTitle(final String title){
            if(title==null){
                setTitle("No Blaise documents opened");
            }
            else{
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        bgp.parent.setTitle("Blaise Grader: "+title);
                    }
                });
            }
        }
        
        private boolean hasExternal=false;
        private ExternalFileView efView=null;
        private Blaise_Draft efDraft=null;
        
        /**
            * Updates the UI to reflect the fact that the working body is either
         * internal or external.
         */
        public void setExternalBody(boolean hasExternal){
            if(hasExternal && this.hasExternal){//check to see if this new call is referring to a different draft
                if(efDraft==bgp.parent.document.getWorkingDraft()){
                    return;//not to worry...
                }
                else{
					//uninstall the old one...
                    efView.uninstall();
                }
            }
            else if(this.hasExternal==hasExternal){
				//ignore it.
                return;
            }
            this.hasExternal=hasExternal;
            efDraft=bgp.parent.document.getWorkingDraft();
            if(efView==null){
                efView=new ExternalFileView(bgp.parent);
            }
            
            setModified(true);
            bgp.mainView.setEditable(!hasExternal);
            /*
             bgp.addExternalGradeMI.setEnabled(!hasExternal);
             bgp.removeExternalGradeMI.setEnabled(hasExternal);
             */
            if(hasExternal){
                efView.install(bgp.mainView,efDraft);
            } else {
                efView.uninstall();
                /*
                 bgp.mainView.setEditable(true);
                 bgp.mainView.setText("");
                 bgp.mainView.setCaretPosition(0);
                 */
            }
        }
        /**
            * Resets the toggle buttons, and dynamic menus back to their initial state.
         */
        private void resetButtonsAndMenus() {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    System.out.println("updateUndoUI()");
                    
                    bgp.undoManager.discardAllEdits();//empty the undo manager.
                        
                        bgp.undoMI.setEnabled(false);
                        bgp.undoMI.setText(bgp.undoManager.getUndoPresentationName());//reset any text rename.
                            bgp.redoMI.setEnabled(false);
                            bgp.redoMI.setText(bgp.undoManager.getRedoPresentationName());
                            
                            bgp.cutMI.setEnabled(false);
                            bgp.copyMI.setEnabled(false);
                            bgp.pasteMI.setEnabled(false);
                            
                            bgp.saveMI.setEnabled(false);
                            if(bgp.STATEStandalone == false) {
                                bgp.fileMenu.remove(bgp.openMI);
                            }
                            bgp.paperInfoMI.setEnabled(true);
                            
                            bgp.scribbleModeMI.setEnabled(false);
                            bgp.notesModeMI.setEnabled(false);
                            bgp.editModeMI.setEnabled(false);
                            bgp.spacesModeMI.setEnabled(false);
                            
                            bgp.scribbleWidthM.setEnabled(false);
                            //bgp.scribbleColorM.setEnabled(false);
                            bgp.noteColorM.setEnabled(false);
                            bgp.selectDraftM.setEnabled(false);
                            bgp.gradeDraftM.setEnabled(false);
                            
					//bgp.findAgainMI.setEnabled(false);
                            /* NOT IMPLEMENTED */
                            //bgp.findTextMI.setEnabled(false);
                            //bgp.findNotesMI.setEnabled(false);
                            bgp.finishGradingPaperMI.setEnabled(false);
                            bgp.finishGradingDraftMI.setEnabled(false);
                            //bgp.showHideCommonNotesMI.setEnabled(false);
                            /* NOT IMPLEMENTED */
                            
                            bgp.deleteSelectedMI.setEnabled(false);
                            
                            populateGradeDraftMenu();
                            populateSelectDraftMenu();
                            
                            /*
                             bgp.toggleScribblesCBMI.setEnabled(false);
                             bgp.toggleNotesCBMI.setEnabled(false);
                             
                             bgp.expandNotesMI.setEnabled(false);
                             bgp.collapseNotesMI.setEnabled(false);
                             */
                            
					//boolean mvHasUseful = !bgp.mainView.isEmpty();
                            boolean haveDrafts=bgp.parent.document.hasDrafts();
                            boolean external=bgp.parent.document.getWorkingDraft().hasExternalBody();
					//bgp.finalizeDraftMI.setEnabled(mvHasUseful || external);
					//bgp.submitPaperMI.setEnabled(mvHasUseful || haveDrafts || external);
					//bgp.duplicateDraftMI.setEnabled(haveDrafts);
					//bgp.addExternalGradeMI.setEnabled(!external);
					//bgp.removeExternalGradeMI.setEnabled(external);
                            
                            setModified(false);
                }
            });
        }
        
        /**
            * All changes external programmatic changes to the enabledness of the menu or toolbar items
         * should be performed via this method.  In most cases it will just pass the request on, but
         * sometimes it will satisfy dependancies or handle related happenings.
         * Thread Safe
         * @param button The toolbar button or menu bar item to be enabled or disabled.
         */
        public void setEnabled(final AbstractButton button, final boolean enabled){
			//for now just pass it on.
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    button.setEnabled(enabled);
                }
            });
        }
        
        /**
            * Calls back to the BlaiseDraftView for a list of drafts and puts them
         * in the menu (with proper actions)
         */
        public void populateGradeDraftMenu(){
            populateGradeDraftMenu(bgp.parent.draft_pane.getSelectedDraftIndex());
        }
        
        private void populateGradeDraftMenu(int index){
			 //for the moment let's be lame about this and not optimize.
			 //with each call we'll remake the menu.  We could store some stuff
			 //and check to see if the menu has changed... but this is quicker and dirtier.
			 //fun stuff.
            if(DEBUG) System.out.println("populateGradeDraftMenu.index: "+index);
            
            /* If there's no file open, completely disable the menu */
            if (index == -1) {
                bgp.gradeDraftM.removeAll();
                JMenuItem item=new JMenuItem();
                item.setEnabled(false);
                item.setText("No Drafts");
                bgp.gradeDraftM.add(item);
                bgp.gradeDraftM.setEnabled(false);
            }
            /* Since a file is open, we want the menu enabled, even if there's no drafts to choose */
            else {
                String[] list=bgp.parent.draft_pane.getDraftList();
                JMenuItem item;
                int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
                
                bgp.gradeDraftM.removeAll();
                bgp.gradeDraftM.setEnabled(true);
                for(int i=0;i<list.length;i++){
                    item=new JMenuItem(new GradeDraftAction(bgp, i) );
                    item.setText(list[i]);
                    /*  KeyEvent.VK0-9 is the equivalent of ascii 0-9 (0x30-0x39) 
                        So, we'll assign shortcuts to drafts 1-9 by using the raw value.
                        Not pretty, but it works.
Note: Shortcuts start from 1, but indices start from 0. Offset by 1.
                        */
                    if(i >= 0 && i < 9) {
                        item.setAccelerator(KeyStroke.getKeyStroke((0x31 + i), aKey));
                    } else if (i == 9) {
                        item.setAccelerator(KeyStroke.getKeyStroke((0x30), aKey));
                    }
                    bgp.gradeDraftM.add(item);
                }
                
                if(list.length==0){
                    item=new JMenuItem();
                    item.setEnabled(false);
                    item.setText("No Drafts");
                    bgp.gradeDraftM.add(item);
                }
            }
        }
        
        /**
            * Calls back to the BlaiseDraftView for a list of comments and puts them
         * in the menu (with proper actions)
         */
        public void populateSelectDraftMenu(){
            populateSelectDraftMenu(bgp.parent.draft_pane.getSelectedDraftIndex());
        }
        
        private void populateSelectDraftMenu(int index){
			//for the moment let's be lame about this and not optimize.
			//with each call we'll remake the menu.  We could store some stuff
			//and check to see if the menu has changed... but this is quicker and dirtier.
			//fun stuff.
            
            /* If there's no file open, completely disable the menu */
            if (index == -1) {
                bgp.selectDraftM.removeAll();
                JMenuItem item=new JMenuItem();
                item.setEnabled(false);
                item.setText("No Drafts");
                bgp.selectDraftM.add(item);
                bgp.selectDraftM.setEnabled(false);
            }
            /* Since a file is open, we want the menu enabled, even if there's no drafts to choose */
            else {
                String[] list=bgp.parent.draft_pane.getDraftList();
                JMenuItem item;
                
                bgp.selectDraftM.removeAll();
                bgp.selectDraftM.setEnabled(true);
                for(int i=0;i<list.length;i++){
                    item=new JCheckBoxMenuItem(new ShowDraftAction(bgp, i) );
                    item.setText(list[i]);
                    bgp.selectDraftM.add(item);
                    if(i==index){
                        item.setSelected(true);
                    }
                }
                
                if(list.length==0){
                    item=new JMenuItem();
                    item.setEnabled(false);
                    item.setText("No Drafts");
                    bgp.selectDraftM.add(item);
                }
            }
        }
        /*
         public void populateShowGradedMenu(){
             int c=-1;
             if(bgp.parent.draft_pane.getSelectedDraftIndex()>=0){
                 c=bgp.parent.draft_pane.getSelectedCommentIndex();
             }
             populateShowGradedMenu(c);
         }
         
         private void populateShowGradedMenu(int index){
             String[] list=bgp.parent.draft_pane.getCommentList();
             JMenuItem item;
             
             bgp.showGradedM.removeAll();
             for(int i=0;i<list.length;i++){
                 item=new JCheckBoxMenuItem(new ShowDraftCommentAction(bep,false,i));
                 item.setText(list[i]);
                 bgp.showGradedM.add(item);
                 if(i==index){
                     item.setSelected(true);
                 }
             }
             
             if(list.length==0){
                 item=new JMenuItem();
                 item.setEnabled(false);
                 item.setText("No Graded Drafts");
                 bgp.showGradedM.add(item);
                 
				 //update toggle scribbles
                 bgp.toggleScribblesCBMI.setSelected(false);
                 bgp.toggleScribblesCBMI.setEnabled(false);
             }
             else{
                 bgp.toggleScribblesCBMI.setSelected(bgp.parent.draft_pane.areScribblesVisible());
                 bgp.toggleScribblesCBMI.setEnabled(true);
             }
             
         }
         */
        
        /**
            * Changes the internal mainView modified property.
         * Not thread safe (but no need).
         */
        public void setModified(boolean modified){
            if (bgp.mainView != null) {
                bgp.mainView.setModified(modified);
            } else {
                if(DEBUG) System.out.println("No BlaiseCommentView enabled.");
            }
        }
    }//end UIStateDelegate
}

//This class represents a color swatch
class colorSwatch extends JPanel implements MouseListener {
    
    Color color;		//the color
    boolean active;		//if it is active or not
    BlaiseGradingPane parent;// the parent
        
		//Constructor
        public colorSwatch(Color color, boolean flag, BlaiseGradingPane sheet){
            addMouseListener(this);
            setLayout(new BorderLayout());
            
			//Initialize variables
            this.color = color;
            parent = sheet;
            
			//Set size
            setPreferredSize(new Dimension(30, 30));
            setMinimumSize(new Dimension(30, 30));
            setMaximumSize(new Dimension(30, 30));
            setSize(new Dimension(30,30));
            
            setActive(flag); //set active flag
        }
        
        public void setActive(boolean flag){
            active = flag;
            if(active){
                setBorder(new LineBorder(Color.BLACK, 3));
            }
            else{
                setBorder(new LineBorder(Color.lightGray));
            }
        }
        
		//Mutator for color
        public void setColor(Color color1){
            color = color1;
        }
        
		//Accessor for color
        public Color getColor(){
            return color;
        }
        
        public void paintComponent(Graphics g){
            super.paintComponent(g);
            
            this.setBackground(color);
        }
        
		//When pressed, set as active. If pressed twice (double) - get new color from parent
        public void mousePressed(MouseEvent mouseevent){
            
            if(mouseevent.getClickCount() >= 2){
                parent.getNewColor(this);
            }
            
            parent.deactivateSwatches();
            parent.setActive(this);
            setActive(true);
        }
        
		//Necessary for abstract class
        public void mouseClicked(MouseEvent m){}
        public void mouseEntered(MouseEvent mouseevent){}
        public void mouseExited(MouseEvent mouseevent){}
        public void mouseReleased(MouseEvent mouseevent){}
        
}