package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import com.apple.eawt.*;
import java.io.File;

/**
 * BlaiseGradingPortal is the entry point to the new Blaise Grading Portal
 * application.  (In version 1, this was done in the Blaise_Grader class).
 * 
 * @author Michael Gaiman
 * @version 1 6/6/2004
 */
public class BlaiseGradingPortal extends JFrame implements ComponentListener{

    /**
     * This user was logged in via cached, offline information.
     */
    public static final int CACHED=626;
    /**
     * This user was logged in via the blaise backend, and has fresh info.
     */
    public static final int LIVE=929;
    
    /**
     * A switch to set whether this build of the BlaiseGradingPortal is for
     * writers or for graders
     * equals TransmissionHandler.GRADER_TYPE or TransmissionHandler.WRITER_TYPE depending...
     */
    public static final int USER_TYPE=TransmissionHandler.GRADER_TYPE;

    private static int loginType=LIVE;//LIVE or CACHED
    private JTabbedPane tabbedPane;
    private CheckoutPane checkoutPane;
    private GradingPane gradingPane;
    private CourseAdministrationPane adminPane;
    private GraderInfo graderInfo;
    private String userID;
    private Application app; //OSX stuff.
    
    private Boolean isOnline=Boolean.TRUE;
    private OnlineCheckerThread oct;
    
    public BlaiseGradingPortal(GraderInfo graderInfo){
        if(Constants.isOSX()){
            app=new Application();
            //OSX double buffers natively, so swing doesn't need to.
            RepaintManager currentManager = RepaintManager.currentManager(null);
            currentManager.setDoubleBufferingEnabled(false);
            
            app.setEnabledPreferencesMenu(false); //none yet
            app.addApplicationListener(new com.apple.eawt.ApplicationAdapter(){
                public void handleQuit(ApplicationEvent e) {
                    quitAction();
                }
                public void handleAbout(ApplicationEvent e){
                    e.setHandled(true);
                    new AboutDialog(BlaiseGradingPortal.this,AboutDialog.PORTAL_TYPE);
                }
            });
        }
        
        this.graderInfo=graderInfo;
        if(USER_TYPE==TransmissionHandler.GRADER_TYPE){
            userID=graderInfo.getGraderID();
        }
        else{
            userID=graderInfo.getWriterID();
        }
        
        //spawn the a thread that will check online status
        oct=new OnlineCheckerThread();
        oct.start();
                
        //set to system UI
        try{
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }catch(Exception e){}
        //create the UI
        javax.swing.SwingUtilities.invokeLater(new Runnable(){
            public void run() {
                createGUI();
                //show the UI
                WindowOpener.open(BlaiseGradingPortal.this,"GradingPortal");
            }
        });
        
        addComponentListener(this);
    }
    
    public CheckoutPane getCheckoutPane(){
        return checkoutPane;
    }
    
    public GradingPane getGradingPane(){
        return gradingPane;
    }
    
    public CourseAdministrationPane getAdminPane(){
        return adminPane;
    }
    
    public GraderInfo getGraderInfo(){
        return graderInfo;
    }
    
    public String getUserID(){
        return userID;
    }
    
    /**
     * A convience method.  This is slightly unsafe, as in the future
     * there could be more than two users, in which cas a boolean wouldn't do,
     * but for the moment this makes life easier.
     */
    public static boolean isGrader(){
        if(USER_TYPE==TransmissionHandler.GRADER_TYPE){
            return true;
        }
        return false;
    }
    
    /**
     * Intelligently sets the title (depending upon USER_TYPE and loginType)
     */
    void setTitle(){
        StringBuffer title=new StringBuffer("Blaise Grading Portal");
        if(USER_TYPE==TransmissionHandler.WRITER_TYPE){
            title.append(" Student Edition");
        }
        if(loginType==CACHED){
            title.append(" --  Working Offline");
        }
        
        setTitle(title.toString());
    }
    
	public class HelpAction extends AbstractAction {
        public static final String NAME="help-action";
		private BlaiseGradingPortal bgp;
        public HelpAction(BlaiseGradingPortal bgp) {
            super(NAME);
			this.bgp=bgp;
            putValue(Action.SHORT_DESCRIPTION,"Get help using the Blaise Grading Portal");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae) {
            DocumentOpener docO=new DocumentOpener(new File("docs/gp/index.html"));
            if(!docO.open()){
                JOptionPane.showMessageDialog(bgp, docO.getErrorMsg(), "Could not open help", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
	
    /**
	 * Creates the GUI by instantiating and setting everything up.
     * For thread safety, this method should be invoked from the
     * event-dispatching thread.
     */
    private void createGUI(){
        setTitle();
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
                quitAction();
            }
        });
        
        tabbedPane=new JTabbedPane();
        
        checkoutPane=new CheckoutPane(this);
        tabbedPane.add("Paper Checkout",checkoutPane);
        
        gradingPane=new GradingPane(this);
        tabbedPane.add("Grade Papers",gradingPane);
        
        //adminPane=new CourseAdministrationPane(this);
        //tabbedPane.add("Course Administration",adminPane);
        
        getContentPane().add(tabbedPane);
        
        int aKey=Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(); //platform independent
        JMenuBar menuBar = new JMenuBar();
        JMenuItem item;
        //Only option is to exit
        if(!Constants.isOSX()){
            JMenu fileMenu = new JMenu("File");
            item = new JMenuItem("Exit");
            item.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e){
                    quitAction();
                }
            });
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,InputEvent.ALT_MASK));
            fileMenu.add(item);
            menuBar.add(fileMenu);
        }
        JMenu helpMenu = new JMenu("Help");
        item=new JMenuItem(new HelpAction(this));
		item.setText("Grading Portal Help");
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_SLASH,aKey|InputEvent.SHIFT_MASK)); //makes the "?"
        helpMenu.add(item);
        item=new JMenuItem("About");
        item.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                new AboutDialog(BlaiseGradingPortal.this,AboutDialog.PORTAL_TYPE);
            }
        });
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);
        
        //set default size (though WindowOpener may override this)
        setSize(650,500);
    }
    
    private static final int MIN_HEIGHT=340;
    private static final int MIN_WIDTH=620;
    
    public void componentResized(ComponentEvent e) {
        int width = getWidth();
        int height = getHeight();

        //we check if either the width
        //or the height are below minimum

        boolean resize = false;

        if (width < MIN_WIDTH) {
            resize = true;
            width = MIN_WIDTH;
        }
        if (height < MIN_HEIGHT) {
            resize = true;
            height = MIN_HEIGHT;
        }
        if (resize) {
            setSize(width, height);
        }
    }
    public void componentMoved(ComponentEvent e) {}
    public void componentShown(ComponentEvent e) {}
    public void componentHidden(ComponentEvent e) {}
    
    /**
	 * Causes the grading portal to exit (and performs closing related housework).
     */
    public void quitAction(){
        System.exit(0);
    }
    
    private ProgressView progView=null;
    
    /**
     * Starts a progress view with the supplied properties. (replacing an existing one if needed)
     */
    void startProgView(String title, int timeSeconds, boolean isIndeterminate){
        //starting progress view is thread safe.
        if(progView==null){
            progView=new ProgressView(this,title,timeSeconds,isIndeterminate);
        }
        else{
            progView.done();
            progView=new ProgressView(this,title,timeSeconds,isIndeterminate);
        }
    }
    
    /**
     * Starts an indeterminate progress view, with the supplied title. (replacing an existing one if needed)
     */
    void startProgView(String title){
        startProgView(title,0,true);
    }
    
    /**
     * Stops the progress view (if it exists)
     */
    void stopProgView(){
        if(progView!=null){
            //done is thread safe
            progView.done();
            progView=null;
        }
    }
    
    boolean isOnline(){
        synchronized(isOnline){
            return isOnline.booleanValue();
        }
    }
    
    synchronized void setIsOnline(boolean ol){
        synchronized(isOnline){
            isOnline=(ol)?(Boolean.TRUE):(Boolean.FALSE);
        }
    }
    
    /**used by offerOnline (sometimes)*/
    private boolean cancelled;
    private boolean haveAnswer;
    /**
     * Depending on the online status, it will return true if the user
     * decides to go online, false if they stay offline.  It will also return
     * true if the user is already online (without asking the user anything)
     * and it will return false if the user cannot go online (after telling the user
     * that the selected action requires an internet connection).
     * (Dont's call this method with the event thread, because it blocks if the user decides to go online
     * until they've logged in.)
     */
    public synchronized boolean offerOnline(){
        if(isOnline()){
            if(loginType==LIVE){
                //already online and logged in.
                return true;
            }
            else if(loginType==CACHED){
                //ask them if they'd like to go online, and then proceed as they direct.
                haveAnswer=false;
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        int answer=JOptionPane.showConfirmDialog(BlaiseGradingPortal.this,"The requested action requires you to be online.\nWould you like to go online now?","Go Online",JOptionPane.YES_NO_CANCEL_OPTION);
                        if(answer==JOptionPane.CANCEL_OPTION || answer==JOptionPane.NO_OPTION || answer==JOptionPane.CLOSED_OPTION){
                            cancelled=true;
                        }
                        else{
                            cancelled=false;
                        }
                        haveAnswer=true;
                    }
                });
                
                //busy wait...annoying.
                try{
                    while(!haveAnswer){
                        Thread.sleep(100);
                    }
                }catch(InterruptedException e){}
                if(cancelled){
                    return false;
                }
                
                //try to go online...

                cancelled=false;//cancelled is declared right above offerOnline.
                PortalLoginHandler handler=new PortalLoginHandler(true,USER_TYPE){
                    //extend to override the cancel handling.
                    public void doCancel(){
                        cancelled=true;
                        done=true;
                    }
                };
                
                new NewLoginUtility(this,"Blaise Grading Portal Login","Please re-enter your username and password.",handler);
                graderInfo=handler.getGraderInfo();
                if(cancelled){
                     return false;
                }
                else{
                    int oldLT=loginType;
                    loginType=LIVE;
                    setTitle();
                    firePropertyChange("onlineStatus",oldLT,loginType);
                    return true;
                }                   
            }
            else{
                throw new IllegalStateException("loginType is neither LIVE nor CACHED");
            }
        }
        else{
            if(loginType==LIVE){
                //give them an error message
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        JOptionPane.showMessageDialog(BlaiseGradingPortal.this,"Cannot connect to Blaise servers.  Please check your connection and try again.","Cannot Connect",JOptionPane.ERROR_MESSAGE);
                    }
                });
                return false;
            }
            else if(loginType==CACHED){
                //give them an error message
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        JOptionPane.showMessageDialog(BlaiseGradingPortal.this,"The requested action requires an internet connection.","Cannot Find Connection",JOptionPane.ERROR_MESSAGE);
                    }
                });
                return false;
            }
            else{
                throw new IllegalStateException("loginType is neither LIVE nor CACHED");
            }
        }
    }
    
    /**
     * Returns the login type that the user is currently logged in via.
     */
    public static int getLoginType(){
        return loginType;
    }
    
    public static void main(String[] argv){
        if(Constants.isOSX()){
            if(MacInstall.installNeeded()){//move user dirs out of app.
                MacInstall.doMacInstall();
            }
        }
        
        try{
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }catch(Exception e){}
        
        //get the user's graderID (either locally, or from the database)
        
        GraderInfo info=null;
        PortalLoginHandler plHandler;
        if(TransmissionHandler.isOnline()){
            plHandler=new PortalLoginHandler(true,USER_TYPE);
            new NewLoginUtility(null,"Blaise Grading Portal Login","Users of the Blaise Grading Portal must login using their Blaise credentials.",plHandler);
            
            loginType=LIVE;
            info=plHandler.getGraderInfo();
        }
        else{
            //login offline
            System.out.println("OFFLINE");
            plHandler=new PortalLoginHandler(false,USER_TYPE);
            new NewLoginUtility(null,"Blaise Grading Portal Cached Login","An internet connection could not be found.  In cached mode users can only grade papers already checked out.",plHandler);
            
            info=plHandler.getGraderInfo();
            loginType=CACHED;
        }
        
        //temp for testing:
        //loginType=LIVE;
        //info=tempLogin();//fills in a graderInfo object with a local temp grader
        
        try{
            String file = "gpsplash.jpg";
            SplashScreen screen = new SplashScreen(file);
            screen.setVisible(true);
            //we should have enough work to do, that it is actually useful now.
            /*
             try{
                 Thread.sleep(Constants.getDelay());
             } catch (Exception e) {}
             */
            
            new BlaiseGradingPortal(info);
            screen.dispose();
        }
        catch(Exception e){}
    }
    
    /**
     * presents a Login dialog, logs in, fills in GraderInfo, returns it.
     */
    private static GraderInfo loginOnline(){
        boolean notIn=true;
        GraderLoginQuery glq=null;
        WriterLoginQuery wlq=null;
        while(notIn){
            LoginUtility login;
            if(USER_TYPE==TransmissionHandler.GRADER_TYPE){
                login=new LoginUtility(null,LoginUtility.GRADER);
            }
            else{
                login=new LoginUtility(null,LoginUtility.CLIENT);
            }
            login.confirmLogin();
            Thread lThread=login.getLoginThread();
            try{
                while(lThread==null){
                    if(login.getOutput()==login.CANCEL_OPTION){
                        System.exit(0);//they cancelled, quit.
                    }
                    //waiting for the LU to try to login, then we can just wait for it to be done (without a busy wait)
                    Thread.sleep(100);
                    lThread=login.getLoginThread();
                }
                //now wait for the thread the submit button spawned to finish.
                lThread.join();
            }catch(InterruptedException e){}
            Object result=login.getOutput();
            login.dispose();
            if(result == login.CANCEL_OPTION){
                System.exit(0);//they cancelled, quit
            }
            else if(result instanceof String){
                JOptionPane.showMessageDialog(null,result, "Login Failure", JOptionPane.ERROR_MESSAGE);
                //continue now...
            }
            else{
                notIn=false;
                if(USER_TYPE==TransmissionHandler.GRADER_TYPE){
                    glq=(GraderLoginQuery)result;
                }
                else{
                    wlq=(WriterLoginQuery)result;
                }
            }
        }
        if(USER_TYPE==TransmissionHandler.GRADER_TYPE){
            java.util.Vector courses=glq.getCourses();
            Course[] ca=new Course[courses.size()];
            ca=(Course[])courses.toArray(ca);
            return new GraderInfo(glq.getGrader(),ca);
        }
        else{
            java.util.Vector courses=wlq.getCourses();
            Course[] ca=new Course[courses.size()];
            ca=(Course[])courses.toArray(ca);
            return new GraderInfo(wlq.getWriter(),ca);
        }
    }
    
    
    /**
     * Done in its own thread so that the UI doesn't need to be held up.
     */
    private class OnlineCheckerThread extends Thread{
        private boolean stop=false;
        
        public OnlineCheckerThread(){
            setDaemon(true);
        }
        
        public void stopThread(){
            stop=true;
        }
        
        public void run(){
            boolean old;
            boolean neww;
            while(!stop){
                old=isOnline();
                neww=TransmissionHandler.isOnline();
                if(neww!=old){
                    System.out.println("Online status changed to "+neww);
                    setIsOnline(neww);
                }
                try{
                    sleep(100);
                }
                catch(InterruptedException e){}
            }
        }
    }
    
    /**
     * Returns a fictional GraderInfo for testing...
     */
    /*private static GraderInfo tempLogin(){
        Grader grader=new Grader(123L,new java.util.Date(),1,"John","Doe","jd","jd@gwu.edu","cool man",new java.util.Date());
        Course[] courses=new Course[2];
        Vector assignments=new Vector();
        Assignment a=new Assignment(1234L,911L,"Paper 1 Outline",1,1,new Date(),null);
        assignments.addElement(a);
        a=new Assignment(2341L,911L,"Paper 1 Draft 1",1,2,new Date(),null);
        assignments.addElement(a);
        a=new Assignment(2342L,911L,"Paper 1 Draft 2",1,3,new Date(),null);
        assignments.addElement(a);
        courses[0]=new Course(911L,"call me",1,"MATH","101","A","Fall",2004,1,assignments);
        assignments=new Vector();
        a= new Assignment(5550L,811L,"Sketches Essay",2,1,new Date(),null);
        assignments.addElement(a);
        a=new Assignment(5551L,811L,"Mona Lisa Essay",1,2,new Date(),null);
        assignments.addElement(a);
        courses[1]=new Course(811L,"none",1,"ART","2","MV","Fall",2004,1,assignments);
        return new GraderInfo(grader,courses);
    }*/
}


/**
 * Handles the logging in for the portal.
 * It handles three different login cases:
 * 1) Grader online
 * 2) Writer online
 * 3) Cached login
 */
class PortalLoginHandler implements LoginHandler{
    private Integer status=new Integer(UNKNOWN);
    private boolean online;
    private GraderInfo gi=null;
    protected Object GISync=new Object();
    /**Subclasses can set done to true to unblock getGraderInfo()*/
    protected boolean done=false;
    private String message;
    private int USER_TYPE;
    private CachedLoginStore cls;
    
    public PortalLoginHandler(boolean online,int user_type){
        USER_TYPE=user_type;
        this.online=online;
    }
    
    public void doLogin(String username,String password){
        synchronized(status){//reset for each call.
            status=new Integer(UNKNOWN);
        }
        Thread thread;
        if(online){
            thread=getOnlineThread(username,password);
        }
        else{
            thread=getCachedThread(username,password);
        }
        thread.start();
    }
    
    /**
     * Returns a login thread for use will an internet connection.
     */
    private Thread getOnlineThread(final String username,final String password){
        return new Thread(){
            public void run(){
                if(USER_TYPE==TransmissionHandler.GRADER_TYPE){
                    GraderLoginQuery glq = TransmissionHandler.getGraderLogin(username,password,Constants.getUniversityID());
                    if(glq.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                        java.util.Vector courses=glq.getCourses();
                        Course[] ca=new Course[courses.size()];
                        ca=(Course[])courses.toArray(ca);
                        synchronized(GISync){
                            gi=new GraderInfo(glq.getGrader(),ca);
                        }
                        synchronized(status){
                            status=new Integer(SUCCESS);
                        }
                        updateCLS();
                        return;
                    }
                    else{
                        message=glq.getMessage();
                        synchronized(status){
                            status=new Integer(FAILURE);
                        }
                        return;
                    }
                }
                else{//writer login
                    WriterLoginQuery wlq = TransmissionHandler.getWriterLogin(username, password, Constants.getUniversityID());
                    if(wlq.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                        java.util.Vector courses=wlq.getCourses();
                        Course[] ca=new Course[courses.size()];
                        ca=(Course[])courses.toArray(ca);
                        synchronized(GISync){
                            gi=new GraderInfo(wlq.getWriter(),ca);
                        }
                        synchronized(status){
                            status=new Integer(SUCCESS);
                        }
                        updateCLS();
                        return;
                    }
                    else{
                        message=wlq.getMessage();
                        synchronized(status){
                            status=new Integer(FAILURE);
                        }
                        return;
                    }
                }
            }
        };
    }
    
    /**
     * In a new thread, the CachedLoginStore gets updated to the latest
     * info about the user who just logged in.
     */
    private void updateCLS(){
        Thread thread=new Thread(){
            public void run(){
                System.out.println("adding cls user");
                cls=CachedLoginStore.getInstance();
                cls.addUser(TransmissionHandler.getUserName(),TransmissionHandler.getEPass(),TransmissionHandler.getUniversityID(),gi);
                System.out.println("done adding cls user");
            }
        };
        thread.start();
    }
    
    /**
     * returns a thread that logs in using cached data.
     */
    private Thread getCachedThread(final String username,final String password){
        return new Thread(){
            public void run(){
                cls=CachedLoginStore.getInstance();
                synchronized(GISync){
                    gi=cls.getUser(username,Encryption.encryptPassword(password),Constants.getUniversityID());
                    if(gi==null){//failure
                        message="Login failure.  Invalid username/password combination or user not cached.";
                        synchronized(status){
                            status=new Integer(FAILURE);
                        }
                        return;
                    }
                    else{
                        synchronized(status){
                            status=new Integer(SUCCESS);
                        }
                        return;
                    }
                }
            }
        };
    }
    
    public void doCancel(){
        System.exit(0);
    }
    
    public int getLoginStatus(){
        synchronized(status){
            return status.intValue();
        }
    }
    
    public String getMessage(){
        return message;
    }
    
    /**
     * Returns the grader info object that was made with the login stuff.
     * (Blocks until it has a grader info object to return)
     */
    public GraderInfo getGraderInfo(){
        System.out.println("getGraderInfo called with event thread: "+SwingUtilities.isEventDispatchThread());
        while(true){
            synchronized(GISync){
                if(gi!=null || done){
                    break;
                }
            }
            try{
                Thread.sleep(100);
            }catch(InterruptedException e){
            }
        }
        
        return gi;
    }
    
}
