//LOC:180
package edu.gwu.blaise.version2.grader;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import java.util.*;
import java.io.*;

import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;

/**
 * Briefcase pane allows user to view documents in briefcase, remove selections from briefcase, and download contents of briefcase.
 * It reads the documents from the static BriefCase class
 */
public class BriefCasePane extends JPanel{

	private boolean debug = false;	//debugging purposes

	JFrame parent;
	ProgressView progView;
	BriefCasePane thisPane;

	//Table, table column, and table model setup
	BlaiseTableModel rt;
	BlaiseTableColumnModel ft;
	JTable jt;

	Vector paperVector;	//vector of papers within the briefcase

	/**
	 * Constructor
	 *
	 * @param ident The grader identification number
	 */
	public BriefCasePane(JFrame par){
		this.setLayout(new BorderLayout());

		this.parent = par;
		thisPane = this;

		//Initialize table
		UIManager.put("Table.focusCellHighlightBorder", new LineBorder(new Color(49,106,197), 0 ) );
		rt = new BlaiseTableModel();
		ft = new BlaiseTableColumnModel();
		jt = new JTable(rt,ft);
		jt.createDefaultColumnsFromModel();

		//Add table to scrollpane
		JScrollPane scroll = new JScrollPane(jt);

		//Establish button panel
		JPanel buttons = new JPanel();

		//Button setup
		JButton clearButton = new JButton("Remove Selected");
		JButton downloadButton = new JButton("Download Briefcase");
		JButton selectAllButton = new JButton("Select All");
		JButton selectNoneButton = new JButton("Select None");

		//On selectAll, select all
		selectAllButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){
				jt.selectAll();
			}
		});

		//On selectAll, select all
		selectNoneButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){
				jt.clearSelection();
			}
		});

		//On clear, remove selected papers from vector and refresh table
		clearButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){
				int[] selected = jt.getSelectedRows();

				if(selected.length == 0)return;

				//remove papers selected from briefcase
				for(int i=selected.length-1;i>=0;i--){
					BriefCase.removePaper(selected[i]);
				}

				rt.setTable(BriefCase.getPapers());	//set table to papers currently in briefcase
			}
		});

		//On download, create XML file of all papers in brief-case
		//Save file using a name that will be known to grader.
		//Once file is saved, move papers from ungraded to pending table
		downloadButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){

				//If no papers present
				if(BriefCase.numberInBrief() == 0){
					return;
				}

				if(!TransmissionHandler.isOnline()){
					JOptionPane.showMessageDialog(parent, "You are not connected to the Internet.", "Internet Connection Not Found", JOptionPane.ERROR_MESSAGE);
					return;
				}

				thisPane.doThread("Receiving papers from Blaise system...");
			}
		});

		//Add buttons
		buttons.add(selectAllButton);
		buttons.add(selectNoneButton);
		buttons.add(clearButton);
		buttons.add(downloadButton);

		this.add(scroll,BorderLayout.CENTER);
		this.add(buttons, BorderLayout.SOUTH);
	}

	private void doThread(String message){

		// Start the indeterminate version.
		progView = new ProgressView ((JFrame)parent, message, 0, true);

		// Fire off the actual work as a thread:
		Thread thread = new Thread () {
			public void run ()
			{
				doTransmit();
			}
		};
		SwingUtilities.invokeLater (thread);
	}

	private void doTransmit(){
		/*
		CheckOutPapersQuery query = TransmissionHandler.getCheckOutPapers(BriefCase.getPaperHeaders());
		if(!query.transmit()){
			progView.done();
			JOptionPane.showMessageDialog(parent, query.getMessage(), "Transmission Failure", JOptionPane.ERROR_MESSAGE);
		}
		else{
			Vector incoming = query.getPapers();

			//Extract Papers already on tablet
			File old = new File(Constants.getUngradedFile());
			if(debug)System.out.println("BriefCasePane -- Got Ungraded Paper File: "+old);

			//Fill vector of existing papers
			Vector existingPapers;
			Blaise_XML xml = new Blaise_XML();
			if(old.exists()){
				existingPapers = xml.parse(old.getAbsolutePath());
			}
			else{
				existingPapers = new Vector();
			}

			//Combine old and new documents into one Vector
			Vector total_papers = new Vector();
			for(int i=0;i<existingPapers.size();i++){
				total_papers.add((Blaise_Paper)existingPapers.elementAt(i));
			}
			for(int i=0;i<incoming.size();i++){
				Blaise_Paper curr = (Blaise_Paper)incoming.elementAt(i);
				curr.clearGraderInfo(Blaise_Grader.graderID, Blaise_Grader.username); //set all incoming drafts with this graders id and pass to ""
				total_papers.add(curr);
			}

			//Generate new XML and write to file
			xml.printPaperToFile(old.getAbsolutePath(), total_papers);

			BriefCase.clearBriefCase();			//clear the briefcase
			rt.setTable(BriefCase.getPapers());	//update table
			TabletPapers.reloadTables();
		}

		progView.done();
		 */
	}


	/**
	 * Called to have the Briefcase Pane query the Briefcase for documents found within
	 */
	public void updateBriefCase(){
		rt.setTable(BriefCase.getPapers());
	}
}
