package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.util.Constants;
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
/**
 * CachedLoginStore holds the information about a Grader or Writer (as grader)
 * that is needed to login and use the BlaiseGradingPortal.  This allows
 * the portal to be used without an internet connection.
 *
 * Note: the CachedLoginStore currently does not cache past courses.
 *
 * @author Michael Gaiman
 * @version 1 7/1/2004
 */
public class CachedLoginStore{
    private Hashtable store;
    /**Makes the Read and the Write wait until the other is done.*/
    private Object RWControl;
    
    private static CachedLoginStore instance;

    private CachedLoginStore(){
        store=new Hashtable();
        RWControl=new Object();
        readStore();
    }
    
    /**
     * CachedLoginStore is a singleton class.
     */
    public static CachedLoginStore getInstance(){
        if(instance==null){
            instance=new CachedLoginStore();
        }
        return instance;
    }
    
    private void readStore(){
        synchronized(RWControl){
            File f=new File(Constants.getCachedLoginStore());
            if(!f.exists()){
                System.out.println("Couldn't find cls.");//not a big deal--probably just means this is the first time running...
                return;
            }
            Properties props=new Properties();
            try{
                InputStream in=new BufferedInputStream(new FileInputStream(f));
                props.load(in);
                in.close();
            }catch(IOException e){
                System.out.println("Could not read cls Exception:"+e);
                return;
            }
            
            String key,base,base2,base3,base4;
            SimpleDateFormat dateFormat=new SimpleDateFormat(TransmissionConstants.dateFormat);
            GraderInfo gi;
            edu.gwu.blaise.version2.transmission.Writer writer=null;
            Grader grader=null;
            Vector v,v2;
            Course[] courses;
            Assignment assign;
            DraftStructure ds;
            int size,size2,j,k,l;
            try{
                int numEntries=Integer.parseInt(props.getProperty("numEntries"));
                for(int i=0;i<numEntries;i++){
                    key=props.getProperty(Integer.toString(i));
                    //assemble the writer/grader object
                    base=new StringBuffer().append(i).append('.').toString();
                    if(props.getProperty(base+"type").equals("G")){
                        grader=new Grader(
                            Long.parseLong(props.getProperty(base+"graderID")),
                            dateFormat.parse(props.getProperty(base+"creationDate")),
                            Integer.parseInt(props.getProperty(base+"universityID")),
                            props.getProperty(base+"firstName"),
                            props.getProperty(base+"lastName"),
                            props.getProperty(base+"userName"),
                            props.getProperty(base+"email"),
                            props.getProperty(base+"status"),
                            ((props.getProperty(base+"simCheckDate")==null)?(null):(dateFormat.parse(props.getProperty(base+"simCheckDate"))))
                        );
                    }
                    else{
                        //get the wag vector first
                        v=new Vector();
                        size=Integer.parseInt(props.getProperty(base+"wagSize"));
                        base2=base+"wag";
                        for(j=0;j<size;j++){
                            v.addElement(new Long(props.getProperty(base2+j)));
                        }
                        writer=new edu.gwu.blaise.version2.transmission.Writer(
                            Long.parseLong(props.getProperty(base+"writerID")),
                            Integer.parseInt(props.getProperty(base+"universityID")),
                            props.getProperty(base+"studentIdent"),
                            props.getProperty(base+"firstName"),
                            props.getProperty(base+"lastName"),
                            props.getProperty(base+"userName"),
                            props.getProperty(base+"email"),
                            props.getProperty(base+"UNType"),
                            dateFormat.parse(props.getProperty(base+"creationDate")),
                            v
                        );
                    }
                    //put together the course info
                    size=Integer.parseInt(props.getProperty(base+"courseSize"));
                    courses=new Course[size];
                    for(j=0;j<courses.length;j++){
                        base2=base+"course"+j+'.';
                        //put together the Assignment vector for this course first
                        v=new Vector();
                        size=Integer.parseInt(props.getProperty(base2+"assignSize"));
                        for(k=0;k<size;k++){
                            base3=base2+"assign"+k+'.';
                            v2=new Vector();
                            size2=Integer.parseInt(props.getProperty(base3+"dsSize"));
                            for(l=0;l<size2;l++){
                                base4=base3+"ds"+l+'.';
                                ds=new DraftStructure(
                                    Long.parseLong(props.getProperty(base4+"dsID")),
                                    Integer.parseInt(props.getProperty(base4+"dsDraftNum")),
                                    Long.parseLong(props.getProperty(base3+"assignID")),
                                    props.getProperty(base4+"dsGradingType"),
                                    dateFormat.parse(props.getProperty(base4+"dsNoChangeDate"))
                                );
                                v2.addElement(ds);
                            }
                            assign=new Assignment(
                                Long.parseLong(props.getProperty(base3+"assignID")),
                                Long.parseLong(props.getProperty(base3+"courseID")),
                                props.getProperty(base3+"name"),
                                Integer.parseInt(props.getProperty(base3+"numDrafts")),
                                Integer.parseInt(props.getProperty(base3+"assignNum")),
                                v2
                            );
                            v.addElement(assign);
                        }
                        //get the course if for this course
                        courses[j]=new Course(
                            Long.parseLong(props.getProperty(base2+"courseID")),
                            props.getProperty(base2+"contactInfo"),
                            Integer.parseInt(props.getProperty(base2+"universityID")),
                            props.getProperty(base2+"dept"),
                            props.getProperty(base2+"number"),
                            props.getProperty(base2+"section"),
                            props.getProperty(base2+"semester"),
                            Integer.parseInt(props.getProperty(base2+"year")),
                            v,
                            props.getProperty(base2+"isCurrent")
                        );
                    }
                    //make the grader info object and add it to the store.
                    if(grader==null){
                        gi=new GraderInfo(writer,courses);
                    }
                    else{
                        gi=new GraderInfo(grader,courses);
                    }
                    store.put(key,gi);
                }
            }catch(java.text.ParseException e){
                System.out.println(e);
            }
        }
    }
    
    private void writeStore(){
        synchronized(RWControl){
            Properties props=new Properties();
            props.setProperty("numEntries",Integer.toString(store.size()));
            String base,base2,base3,base4;
            GraderInfo gi;
            edu.gwu.blaise.version2.transmission.Writer writer;
            Grader grader;
            Vector v,v2;
            Course[] courses;
            Assignment assign;
            DraftStructure ds;
            SimpleDateFormat dateFormat=new SimpleDateFormat(TransmissionConstants.dateFormat);
            
            int i=0,j,k,l,size,size2;
            
            for(Enumeration e=store.keys();e.hasMoreElements();i++){
                base=(String)e.nextElement();
                
                gi=(GraderInfo)store.get(base);
                props.setProperty(Integer.toString(i),base);
                base=new StringBuffer().append(i).append('.').toString();
                //write out the Grader/Writer object
                if(gi.getType()==TransmissionHandler.GRADER_TYPE){
                    grader=gi.getGrader();
                    props.setProperty(base+"type","G");
                    props.setProperty(base+"graderID",Long.toString(grader.getGraderID()));
                    props.setProperty(base+"creationDate",dateFormat.format(grader.getCreationDate()));
                    props.setProperty(base+"universityID",Integer.toString(grader.getUniversityID()));
                    props.setProperty(base+"firstName",grader.getFirstName());
                    props.setProperty(base+"lastName",grader.getLastName());
                    props.setProperty(base+"userName",grader.getUserName());
                    props.setProperty(base+"email",grader.getEmail());
                    props.setProperty(base+"status",grader.getStatus());
                    if(grader.getSimilarityCheckedDate()!=null){
                        props.setProperty(base+"simCheckDate",dateFormat.format(grader.getSimilarityCheckedDate()));
                    }
                }
                else{
                    writer=gi.getWriter();
                    props.setProperty(base+"type","W");
                    props.setProperty(base+"writerID",Long.toString(writer.getWriterID()));
                    props.setProperty(base+"universityID",Integer.toString(writer.getUniversityID()));
                    props.setProperty(base+"studentIdent",writer.getStudentIdent());
                    props.setProperty(base+"firstName",writer.getFirstName());
                    props.setProperty(base+"lastName",writer.getLastName());
                    props.setProperty(base+"userName",writer.getUserName());
                    props.setProperty(base+"email",writer.getEmail());
                    props.setProperty(base+"UNType",writer.getUserNameType());
                    props.setProperty(base+"creationDate",dateFormat.format(writer.getCreationDate()));
                    //do the vector of wag ids.
                    v=writer.getWriterAsGraderIDs();
                    size=v.size();
                    props.setProperty(base+"wagSize",Integer.toString(size));
                    base2=base+"wag";
                    for(j=0;j<size;j++){
                        props.setProperty(base2+j,v.elementAt(j).toString());
                    }
                }
                //now do the courses
                courses=gi.getCourses();
                props.setProperty(base+"courseSize",Integer.toString(courses.length));
                for(j=0;j<courses.length;j++){
                    base2=base+"course"+j+'.';
                    props.setProperty(base2+"courseID",Long.toString(courses[j].getCourseID()));
                    props.setProperty(base2+"contactInfo",courses[j].getContactInfo());
                    props.setProperty(base2+"universityID",Integer.toString(courses[j].getUniversityID()));
                    props.setProperty(base2+"dept",courses[j].getDept());
                    props.setProperty(base2+"number",courses[j].getNumber());
                    props.setProperty(base2+"section",courses[j].getSection());
                    props.setProperty(base2+"semester",courses[j].getSemester());
                    props.setProperty(base2+"year",Integer.toString(courses[j].getYear()));
                    props.setProperty(base2+"isCurrent",courses[j].getIsCurrent());
                    //do the assignments
                    v=courses[j].getAssignments();
                    size=v.size();
                    props.setProperty(base2+"assignSize",Integer.toString(size));
                    for(k=0;k<size;k++){
                        base3=base2+"assign"+k+'.';
                        assign=(Assignment)v.elementAt(k);
                        props.setProperty(base3+"assignID",Long.toString(assign.getAssignID()));
                        props.setProperty(base3+"courseID",Long.toString(assign.getcourseID()));
                        props.setProperty(base3+"name",assign.name());
                        props.setProperty(base3+"numDrafts",Integer.toString(assign.numDrafts()));
                        props.setProperty(base3+"assignNum",Integer.toString(assign.assignNum()));
                        v2=assign.getDraftStructure();
                        size2=v2.size();
                        props.setProperty(base3+"dsSize",Integer.toString(size2));
                        for(l=0;l<size2;l++){
                            base4=base3+"ds"+l+'.';
                            ds=(DraftStructure)v2.elementAt(l);
                            props.setProperty(base4+"dsID",Long.toString(ds.getDraftStructureID()));
                            props.setProperty(base4+"dsDraftNum",Integer.toString(ds.getDraftNum()));
                            props.setProperty(base4+"dsGradingType",ds.getGradingType());
                            props.setProperty(base4+"dsNoChangeDate",dateFormat.format(ds.noChangeDate()));
                        }
                    }
                }
            }
            
            //write out the props file.
            
            try{
                File f=new File(Constants.getCachedLoginStore());
                if(!f.exists()){
                    File p=f.getParentFile();
                    if(p!=null){
                        p.mkdirs();
                    }
                    f.createNewFile();
                }
                OutputStream out=new BufferedOutputStream(new FileOutputStream(f));
                props.store(out,null);
                out.flush();
                out.close();
            }catch(IOException e){
                System.out.println("Could not write cls file.  Exception:"+e);
            }

        }
    }
    
    /**
     * adds a user to the store, or if the user is already in the store, 
     * it will update the GraderInfo of the user.
     */
    public void addUser(String username, String ePass, String universityID, GraderInfo gi){
        synchronized(store){
            store.put(getKey(username,ePass,universityID),gi);
        }
        writeStore();
    }
    
    /**
     * Get a user's GraderInfo, or null if the username/ePass/universityID combo isn't in the store.
     */
    public GraderInfo getUser(String username, String ePass, String universityID){
        return (GraderInfo)store.get(getKey(username,ePass,universityID));
    }
    
    /**
     * The store key is build from a combination username, ePass, and universityID.
     */
    private static String getKey(String username,String ePass, String universityID){
        return username+','+ePass+','+universityID;
    }
    
}
