package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.util.Constants;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.IOException;
import java.io.File;
/**
 * Presents a dialog with various checking in options to the user
 * and then checks in the selected papers.
 * As implemented, only basic information is provided to the user,
 * but this class has access to extended information (via BlaiseGradingPortal)
 * if additional information is deemed necessary.
 * @author Michael Gaiman
 * @version 1 6/27/2004
 */
public class CheckinDialog extends JDialog{
    private BlaiseGradingPortal parent;
    private Vector entries;
    private JCheckBox stageCB;
    private CheckinCallBack checkinCallback;
        
    private static final int WIDTH=270;
    private static final int HEIGHT=120;
    
    /**
     * @param entries A Vector of PaperStore.Entry objects
     * @param cicb a object that will be called each time a paper is successfully checked in.
     */
    public CheckinDialog(BlaiseGradingPortal parent,Vector entries,CheckinCallBack cicb){
        super(parent,"Check in Papers",true);
        this.parent=parent;
        this.entries=entries;
        checkinCallback=cicb;
        
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private void createAndShowGUI(){
        Container cont=getContentPane();
        cont.setLayout(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        int size=entries.size();
        JLabel infoL=new JLabel("You have selected "+size+((size==1)?" paper":" papers")+" to check in.\n");
        
        stageCB=new JCheckBox("Stage checked in papers until later");
        stageCB.setSelected(true);
        
        JButton cancelB=new JButton("Cancel");
        cancelB.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                setVisible(false);
                dispose();
            }
        });
        
        JButton checkinB=new JButton(new DoCheckinAction());
        checkinB.setText("Check in Papers");
        
        c.weightx=1;
        c.weighty=.3;
        c.gridwidth=2;
        cont.add(infoL,c);
        
        c.gridy=1;
        c.weighty=.4;
        cont.add(stageCB,c);
        
        c.gridy=2;
        c.weighty=.3;
        c.gridwidth=1;
        if(Constants.isOSX()){
            cont.add(cancelB,c);
        }
        else{
            cont.add(checkinB,c);
        }
        c.gridx=1;
        if(Constants.isOSX()){
            cont.add(checkinB,c);
        }
        else{
            cont.add(cancelB,c);
        }
        
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.getX();
            pY=parent.getY();
            pW=parent.getWidth();
            pH=parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
    
        setSize(WIDTH,HEIGHT);
        setLocation(myX,myY);
        
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setResizable(false);
        getRootPane().setDefaultButton(checkinB);
        setVisible(true);
    }
    
    private class DoCheckinAction extends AbstractAction{
        public static final String NAME="do-checkin-paper-action";
        public DoCheckinAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Checkin the currently selected papers.");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
		private void updateGraderName(Blaise_Paper paper){
			Vector drafts=paper.getDrafts(), comments;
			int dSize=drafts.size(),cSize,i,j;
			Blaise_Draft d;
			Blaise_Comment c;
			String gID;
			for(i=0;i<dSize;i++){
				d=(Blaise_Draft)drafts.elementAt(i);
				comments=d.getComments();
				cSize=comments.size();
				for(j=0;j<cSize;j++){
					c=(Blaise_Comment)comments.elementAt(j);
					gID=c.getGraderID();
					if(gID==null || gID.equals("")){
						c.setGraderID(parent.getUserID());
					}
					if(c.getGraderID().equals(parent.getUserID())){
						c.setGraderName(parent.getGraderInfo().getName(false));
					}
				}
			}
		}
		
        public void actionPerformed(ActionEvent ae){
            final boolean stage=stageCB.isSelected();
            setVisible(false);
            Thread thread=new Thread(){
                public void run(){
                    try{
                        parent.startProgView("Checking in papers...");
                        TransmissionQuery tq;
                        PaperStore.Entry entry;
                        Blaise_Paper paper;
                        BackendFileHandler handler=new BackendFileHandler();
                        BlaiseFile bf=new BlaiseFile();
                        int size=entries.size();
                        for(int i=0;i<size;i++){
                            entry=(PaperStore.Entry)entries.elementAt(i);
                            paper=new Blaise_Paper();
                            bf.newJob(paper,new File(entry.getPaperFilePath()),handler);
                            try{
                                bf.openPaper();
                            }catch(IOException e){
                                //failure of some sort...tell user
                                parent.stopProgView();
                                System.out.println(e);
                                JOptionPane.showMessageDialog(parent,"Could not open Blaise Paper for transmission.\n(Exception:"+e+')',"An Error Occurred",JOptionPane.ERROR_MESSAGE);
                                checkinCallback.checkinDone(false);
                                return;
                            }
							updateGraderName(paper);
                            if(stage){
                                tq=TransmissionHandler.getStagePapers(paper,parent.USER_TYPE);
                            }
                            else{
                                tq=TransmissionHandler.getCheckInPapers(paper,parent.USER_TYPE);
                            }
                            if(!tq.transmit()){
                                //failure of some sort...tell user
                                parent.stopProgView();
                                JOptionPane.showMessageDialog(parent,tq.getMessage(),"Transmission Failure",JOptionPane.ERROR_MESSAGE);
                                checkinCallback.checkinDone(false);
                                return;
                            }
                            checkinCallback.handleEntry(entry);
                        }
                    }
                    finally{
                        checkinCallback.checkinDone(true);//successful.
                        parent.stopProgView();
                    }
                }
            };
            thread.start();
        }
    }
}
