package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.util.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
/**
 * CheckoutPane allows users to perform paper searches and checkouts from
 * the blaise database.
 * @author Michael Gaiman
 * @version 1 6/6/2004
 */
public class CheckoutPane extends AbstractPortalPane{
    private static final boolean debug=true;
    private BlaiseGradingPortal parent;
    
    private JComboBox courseCB;
    private JComboBox assignmentCB;
    private JTextField nameTF;
    private JCheckBox gradedCB;
    private JTable table;
    private CheckoutTableModel tableModel;
    private JButton viewInfoB; //"global" so that it can be disabled when more than one row is selected.
    private JButton checkoutB;
    private SearchAction searchA;
    
    public CheckoutPane(BlaiseGradingPortal parent){
        super(parent);
        this.parent=parent;
        searchA=new SearchAction();//used by both the search button and by the name field.
    }
    
    protected void onlineStatusChange(int loginType){
        if(loginType==parent.LIVE){
            //switched to live status... update course and assignment list
            fillInCourses();
            fillInAssignments();
        }
    }
    
    /**
     * Added to combo boxes.  Stores a string and a Course or Course and Assignment.
     */
    private class PairObject{
        public Course course;
        public Assignment assignment;
        public String toString;
        
        public PairObject(Course course, String toString){
            this.course=course;
            this.toString=toString;
            this.assignment=null;
        }
        
        public PairObject(Assignment assignment, Course course, String toString){
            this.course=course;
            this.toString=toString;
            this.assignment=assignment;
        }
        
        public String toString(){
            return toString;
        }
    }
    
    private void fillInCourses(){
        courseCB.removeAllItems();
        courseCB.addItem(new PairObject(null,"All Courses"));
        GraderInfo gi=parent.getGraderInfo();
        Course[] courses=gi.getCourses();
        String name;
        for(int i=0;i<courses.length;i++){
            name=gi.getCourseNameForID(courses[i].getCourseID());
            courseCB.addItem(new PairObject(courses[i],name));
        }
        courseCB.setPreferredSize(nameTF.getPreferredSize());
    }
    
    private void fillInAssignments(){
        assignmentCB.removeAllItems();
        PairObject course=(PairObject)courseCB.getSelectedItem();
        if(course.course==null){
            assignmentCB.addItem(new PairObject(null,null,"All Assignments"));
            //if we wanted to make assignmentCB list all assignments, by course,
            //here's where it would go.
        }
        else{
            Assignment[] assigns=parent.getGraderInfo().getAssignments(course.course);
            String name;
            assignmentCB.addItem(new PairObject(null,course.course,"All Assignments"));
            if(assigns==null){
                return;
            }
            for(int i=0;i<assigns.length;i++){
                name=Integer.toString(assigns[i].assignNum())+' '+assigns[i].name();
                assignmentCB.addItem(new PairObject(assigns[i],course.course,name));
            }
        }
        
        assignmentCB.setPreferredSize(nameTF.getPreferredSize());
    }
    
    protected JLabel getHeader(){
        return new JLabel("Search for papers to grade with three criteria.  From there checkout the papers to grade them.",JLabel.LEFT);
    }
    
    /**
     * Return the search panel.
     */
    protected Component getUpperPanel(){
        JPanel criteria=new JPanel(new GridBagLayout());
        GridBagConstraints criteriaC = new GridBagConstraints();
        
        criteria.setBorder(BorderFactory.createTitledBorder("Search Criteria"));
        criteria.add(Box.createVerticalGlue());
        
        JLabel courseL = new JLabel("Course",JLabel.RIGHT);
        JLabel assignmentL = new JLabel("Assignment",JLabel.RIGHT);
        JLabel nameL = new JLabel("Student Last Name",JLabel.RIGHT);
        
        nameTF = new JTextField(15);
        Dimension nameD = nameTF.getPreferredSize();
        //if(debug)System.out.println("nameD:"+nameD);
        nameTF.setMinimumSize(nameD);
        
        courseCB = new JComboBox();
        courseCB.setEditable(false);
        //courseCB.setPreferredSize(nameD);
        courseCB.setFocusable(false);
        //add a listener that will update the assignmentCB when courseCB changes
        fillInCourses();
        courseCB.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e){
                if(e.getStateChange()==ItemEvent.SELECTED){
                    fillInAssignments();
                }
            }
        });
        
        assignmentCB = new JComboBox();
        assignmentCB.setEditable(false);
        //assignmentCB.setPreferredSize(nameD);
        assignmentCB.setFocusable(false);
        fillInAssignments();
        
        //add text field action
        nameTF.addActionListener(searchA);
        
        criteriaC.gridx=0;
        criteriaC.gridy=0;
        Insets criteriaI=new Insets(0,0,0,5);
        Insets defaultI=new Insets(0,0,0,0);
        criteriaC.insets=criteriaI;
        criteriaC.anchor=GridBagConstraints.LINE_END;
        criteria.add(courseL,criteriaC);
        criteriaC.gridx=1; 
        criteriaC.insets=defaultI;
        criteria.add(courseCB,criteriaC);
        criteriaC.gridx=0;
        criteriaC.gridy=1;
        criteriaC.insets=criteriaI;
        criteria.add(assignmentL,criteriaC);
        criteriaC.gridx=1;
        criteriaC.insets=defaultI;
        criteria.add(assignmentCB,criteriaC);
        criteriaC.gridx=0;
        criteriaC.gridy=2;
        criteriaC.insets=criteriaI;
        criteria.add(nameL,criteriaC); 
        criteriaC.gridx=1;
        criteriaC.insets=defaultI;
        criteria.add(nameTF,criteriaC);
        
        JPanel options=new JPanel();
        options.setLayout(new GridBagLayout());
        GridBagConstraints optionsC=new GridBagConstraints();
        options.setBorder(BorderFactory.createTitledBorder("Search Options"));
        
        gradedCB = new JCheckBox("Ungraded Papers Only", true);
        gradedCB.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e){
                int state=e.getStateChange();
                if(state==e.SELECTED){
                    possiblyEnableCheckoutB();//if all other constraints are satisfied.
                }
                else if(state==e.DESELECTED){
                    checkoutB.setEnabled(false);
                }
            }
        });
        optionsC.gridx=0;
        optionsC.gridwidth=GridBagConstraints.REMAINDER;
        optionsC.gridy=0;
        optionsC.weighty=.6;
        options.add(gradedCB,optionsC);
        
        JButton searchB = new JButton(searchA);
        searchB.setText("Search");
        
        searchB.setFocusable(false);
        JButton resetB = new JButton(new ResetAction());
        resetB.setText("Reset");
        resetB.setFocusable(false);
        optionsC.gridx=0;
        optionsC.gridy=1;
        optionsC.gridwidth=GridBagConstraints.RELATIVE;
        optionsC.weighty=.4;
        optionsC.anchor=GridBagConstraints.SOUTH;
        //optionsC.anchor=GridBagConstraints.LAST_LINE_START;
        options.add(searchB,optionsC);
        optionsC.gridx=1;
        optionsC.gridwidth=GridBagConstraints.REMAINDER;
        //optionsC.anchor=GridBagConstraints.LAST_LINE_END;
        options.add(resetB,optionsC);
        
        //JPanel upper = new JPanel(new GridLayout(1,2));
        JPanel upper = new JPanel(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        c.weightx=.55;
        c.weighty=1;
        upper.add(criteria,c);
        c.weightx=.45;
        upper.add(options,c);
        return upper;
        
    }
    
    protected Component getLowerPanel(){
        tableModel=new CheckoutTableModel();
        TableSorter sorter=new TableSorter(tableModel);
        table=new JTable(sorter,new CheckoutColumnModel());
        table.createDefaultColumnsFromModel();
        sorter.setTableHeader(table.getTableHeader());
        table.setDefaultRenderer(Object.class,new BoldingCellRenderer());
        table.setShowHorizontalLines(true);
        table.setShowVerticalLines(true);
        table.setFocusable(false);
        
        return new JScrollPane(table);
    }
    
    protected AbstractButton[] getBottomButtons(){
        JButton[] buttons=new JButton[3];
        buttons[0]=new JButton(new SelectAllAction(table));
        buttons[0].setText("Select All");
        buttons[0].setFocusable(false);
        
        buttons[1]=new JButton(new ViewInfoAction());
        buttons[1].setText("View Paper Details");
        buttons[1].setEnabled(false);//no papers selected initially.
        buttons[1].setFocusable(false);
        viewInfoB=buttons[1];
        
        
        buttons[2]=new JButton(new CheckoutAction());
        buttons[2].setText("Check out Papers");
        buttons[2].setEnabled(false);//no papers selected initially.
        buttons[2].setFocusable(false);
        checkoutB=buttons[2];
        
        return buttons;
    }
    
    /**
     * If all the constraints are satisfied, the checkout button
     * will be enabled.
     * @returns true if the bottom is enabled, false otherwise.
     */
    private boolean possiblyEnableCheckoutB(){
        if(gradedCB.isSelected()){
            int numSelected=table.getSelectedRowCount();
            if(numSelected>0){
                checkoutB.setEnabled(true);
                return true;
            }
        }
        checkoutB.setEnabled(false);
        return false;
    }
    
    private class SearchAction extends AbstractAction{
        public static final String NAME="search-action";
        public SearchAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Search for papers that match the requirements selected");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            
            //get the search values in the event thread
            final PairObject selectedCourse=(PairObject)courseCB.getSelectedItem();
            final PairObject selectedAssignment=(PairObject)assignmentCB.getSelectedItem();
            final boolean ungradedOnly=gradedCB.isSelected();
            final String name=nameTF.getText();
            
            Thread thread=new Thread(){
                public void run(){
                    //go online if needed (and able)
                    if(!parent.offerOnline()){
                        //unable to go online...
                        System.out.println("offer online failed");
                        return;
                    }
                    System.out.println("gonna perform search (thread started)");
                    parent.startProgView("Performing Search...");
                    try{
                        System.out.println("back from startProgView");
                        PaperSearchQuery psq;
                        //prepare assignment id vector
                        Vector assignV=null;
                        if(selectedAssignment.assignment!=null){
                            assignV=new Vector();
                            assignV.add(Long.toString(selectedAssignment.assignment.getAssignID()));
                        }
                        System.out.println("did assignment thingie");
                        if(parent.isGrader()){
                            psq=TransmissionHandler.getPaperSearch(
                            ungradedOnly,name,null,null,selectedCourse.course,parent.USER_TYPE,null,assignV,null);
                            System.out.println("1");
                        }
                        else{
                            psq=TransmissionHandler.getPaperSearch(
                            ungradedOnly,name,null,null,selectedCourse.course,parent.USER_TYPE,parent.getGraderInfo().getWriter().getWriterAsGraderIDs(),assignV,null);
                            System.out.println("2");
                        }
                        System.out.println("9");
                        if(psq.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                            System.out.println("3");
                            tableModel.setPapers(psq.getPapers());
                            System.out.println("4");
                        }
                        else{
                            //failure of some sort...tell user
                            System.out.println("5");
                            parent.stopProgView();
                            JOptionPane.showMessageDialog(parent,psq.getMessage(),"Transmission Failure",JOptionPane.ERROR_MESSAGE);
                            System.out.println("6");
                            return;
                        }
                    }
                    finally{
                        System.out.println("7");
                        parent.stopProgView();
                    }
                    System.out.println("8");
                }
            };
            thread.start();
        }
    }
    
    private class ViewInfoAction extends AbstractAction{
        public static final String NAME="view-info-action";
        public ViewInfoAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Displays a dialog with the details of the selected paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            new PaperInfoDialog(parent,tableModel.getPaperAt(table.getSelectedRow()));
        }
    }
    
    private class ResetAction extends AbstractAction{
        public static final String NAME="reset-search-action";
        public ResetAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Reset search parameters to initial state");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            nameTF.setText("");
            courseCB.setSelectedIndex(0);
            assignmentCB.setSelectedIndex(0);
        }
    }
    
    private class CheckoutAction extends AbstractAction{
        public static final String NAME="checkout-action";
        public CheckoutAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Download the selected papers to be graded");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            final int[] selected=table.getSelectedRows();
            if(debug)System.out.println(selected.length+" selected papers to checkout");
            Thread thread=new Thread(){
                public void run(){
                    //go online if needed (and able)
                    if(!parent.offerOnline()){
                        //unable to go online...
                        return;
                    }
                    parent.startProgView("Checking out papers...");
                    try{
                        Vector ids=new Vector();
                        for(int i=0; i<selected.length;i++){
                            ids.addElement(new Long(tableModel.getPaperID(selected[i])));
                        }
                        CheckOutPapersQuery cpq=TransmissionHandler.getCheckOutPapers(ids,parent.USER_TYPE);
                        if(cpq.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                            GradingPane gp=parent.getGradingPane();
                            final Vector papers=cpq.getPapers();
                            gp.addCheckedOutPapers(papers);
                            SwingUtilities.invokeLater(new Runnable(){
                                public void run(){
                                    tableModel.removePapers(papers);
                                }
                            });
                        }
                        else{
                            //failure of some sort...tell user
                            parent.stopProgView();
                            JOptionPane.showMessageDialog(parent,cpq.getMessage(),"Transmission Failure",JOptionPane.ERROR_MESSAGE);
                            //because checkout papers is a compound operation that effects the backend,
                            //get papers and see if any where successfully checked out.  If they were,
                            //add them to the grading pane (and remove them from checkout table)
                            final Vector papers=cpq.getPapers();
                            if(papers==null || papers.size()==0){
                                return;
                            }
                            else{
                                //something to do.
                                GradingPane gp=parent.getGradingPane();
                                gp.addCheckedOutPapers(papers);
                                SwingUtilities.invokeLater(new Runnable(){
                                    public void run(){
                                        tableModel.removePapers(papers);
                                    }
                                });
                            }
                            return;
                        }
                    }
                    finally{
                        parent.stopProgView();
                    }
                }
            };
            thread.start();
        }
    }
    
    /**
     * A TableModel for use with Blaise_Papers.
     */
    private class CheckoutTableModel extends AbstractTableModel{
        private final String[] columns={"Course","Assignment","Student","Draft Count","Turn-in Date"};
        private Vector papers;
        
        public CheckoutTableModel(){
            papers=new Vector();
        }
        
        public int getRowCount(){
            return papers.size();
        }
        public String getColumnName(int i){
            return columns[i];
        }
        public int getColumnCount(){
            return columns.length;
        }
        public Object getValueAt(int row, int column){
            Blaise_Paper bp=(Blaise_Paper)papers.elementAt(row);
            switch(column){
                case 0:
                    return parent.getGraderInfo().getCourseNameForID(Long.parseLong(bp.getCourseID()));
                case 1:
                    return bp.getAssignmentNumber();
                case 2:
                    return bp.getFirst()+' '+bp.getLast();
                case 3:
                    return Integer.toString(bp.getDrafts().size());
                case 4:
                    return bp.getLastDraft().getTurninDate();
                default:
                    return "Invalid";
            }
        }
        
        public Blaise_Paper getPaperAt(int row){
            if(row<0||row>getRowCount()){
                throw new IllegalArgumentException("Invalid Row");
            }
            return (Blaise_Paper)papers.elementAt(row);
        }
        
        /**
         * returns the paperID for the paper at row row
         */
        public String getPaperID(int row){
            if(row<0 || row>getRowCount()){
                throw new IllegalArgumentException("Invalid Row");
            }
            return ((Blaise_Paper)papers.elementAt(row)).getPaperID();
        }
        
        /**
         * Given a vector of Blaise_Paper objects (either "full" or not)
         * it will remove all papers for the table that have a matching paperID.
         */
        public void removePapers(Vector toRm){
            //order them for easier removal (last first)
            int size=toRm.size(),index;
            for(int i=0;i<size;i++){
                //using Blaise_Paper's equals method, it find the index and then removes the paper at that index.
                index=papers.indexOf(toRm.get(i));
                papers.remove(index);
            }
            fireTableDataChanged();
        }
        
        public void setPapers(Vector papers){
            this.papers=papers;
            fireTableDataChanged();
        }
    }
    
    /**
     * Sets the default column spacing.
     */
    private class CheckoutColumnModel extends DefaultTableColumnModel{
        private int charWidth=0;
        private BoldingCellRenderer rightBCR;
        
        public CheckoutColumnModel(){
            rightBCR=new BoldingCellRenderer();
            rightBCR.setHorizontalAlignment(SwingConstants.RIGHT);
            //update the buttons depending on selection status.
            selectionModel.addListSelectionListener(new ListSelectionListener(){
                public void valueChanged(ListSelectionEvent e){
                    boolean some=!selectionModel.isSelectionEmpty();
                    viewInfoB.setEnabled(some);
                    if(some){
                        possiblyEnableCheckoutB();
                    }
                    else{
                        checkoutB.setEnabled(false);
                    }
                }
            });
        }
        
        public void addColumn(TableColumn tc){
            if (charWidth == 0) {//look up character width, to make it easier to figure out column widths
                Font f=UIManager.getLookAndFeelDefaults().getFont("Table.font");
                //System.out.println("table font: "+f);
                FontMetrics metrics = getFontMetrics(f);
                charWidth = metrics.charWidth('m');
                //System.out.println("charWidth:"+charWidth);
            }
        
            String name = (String)tc.getHeaderValue();
            if(name.equals("Course")){
                tc.setPreferredWidth(11*charWidth);
            }
            else if(name.equals("Assignment")){
                tc.setPreferredWidth(20*charWidth);
            }
            else if(name.equals("Student")){
                tc.setPreferredWidth(20*charWidth);
            }
            else if(name.equals("Draft Count")){
                tc.setPreferredWidth(11*charWidth);//11: "Draft Count" (otherwise it would be 1 or 2)
                tc.setCellRenderer(rightBCR);
            }
            else if(name.equals("Turn-in Date")){
                tc.setPreferredWidth(17*charWidth);
            }
            
            super.addColumn(tc);
        }
    }
    
    /**
     * Makes rows that have UNGRADED_STATUS bold. (Different than GradingPane's BoldingCellRenderer in
     * that all cells are bold)
     */
    private class BoldingCellRenderer extends DefaultTableCellRenderer{
        private Font fontB=null;
            
        public Component getTableCellRendererComponent(JTable table,Object value,boolean isSelected,boolean hasFocus,int row,int column){
            //in super.getTableCellRendererComponent the font gets set to the table's font, and
            //the string get set (among other things).  It then returns itself.
            //we're going to catch that and override the font.
            
            super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);
            if(fontB==null){
                fontB=getFont().deriveFont(Font.BOLD);
            }
            setFont(fontB);
            return this;
        }
    }
}
