//LOC:209
package edu.gwu.blaise.version2.grader;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.colorchooser.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

/**
 * This class extends JDialog and is used by the GradingSheet to customize scribble line width and color
 */
public class FormatScribble extends JDialog {

	//Current color
	public Color color = Color.RED;
	public int width = 1;

	private boolean isValid = false;

	//Components
	JComboBox sizeBox;
	JColorChooser chooser;

	LinePreview preview;

	/**
	 * Constructor
	 *
	 * @param parent JFrame parent
	 * @param wid Current width of line
	 * @param col Current color of line
	 */
	public FormatScribble(BlaiseGradingPane parent, int wid, Color col){
		super(parent.parent, "Format Scribble", true);

		setSize(600,500);
		setResizable(false);

		//Set variables
		width = wid;
		color = col;

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e){
				cancel();
			}
		});

		//Create and add all SWING components
		Container c = getContentPane();
		c.setLayout(new BorderLayout());
		sizeBox = new JComboBox();
		JPanel sizePanel = new JPanel();
		String[] sizes = {"1","2","3","4","5","6","7","8","9","10","11","12","13","14","15"};
		for(int i=0;i<sizes.length;i++){
			sizeBox.addItem(sizes[i]);
		}
		sizeBox.setEditable(false);
		sizeBox.setSelectedIndex(wid-1);
		sizeBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e){
				updateLine();
			}
		});

		JLabel sizeLabel = new JLabel(" Size: ");

		sizePanel.add(sizeLabel);
		sizePanel.add(sizeBox);
		c.add(sizePanel, BorderLayout.NORTH);

		chooser = new JColorChooser(color);
		chooser.setPreviewPanel(new JPanel());
		chooser.getSelectionModel().addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e){
				color = chooser.getColor();
				updateLine();
			}
		});

		preview = new LinePreview(width, color);
		preview.setMinimumSize(new Dimension(50,100));
		preview.setPreferredSize(new Dimension(50,100));

		//Initialize the panel to provide the line preview
		JPanel color_preview = new JPanel(new BorderLayout());
		color_preview.add(chooser, BorderLayout.CENTER);
		color_preview.add(preview, BorderLayout.SOUTH);

		c.add(color_preview, BorderLayout.CENTER);

		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				accept();
			}
		});

		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				cancel();
			}
		});

		JPanel buttonP = new JPanel(new FlowLayout());
		buttonP.add(ok);
		buttonP.add(cancel);

		c.add(buttonP, BorderLayout.SOUTH);

	}

	//This method updates a line width and color
	private void updateLine(){

		try{
			String w = (String)sizeBox.getSelectedItem();
			if(w.equals("0")){
				preview.clear();
				return;
			}

			preview.setValues(Integer.parseInt(w), chooser.getColor());
			preview.repaint();
		}
		catch(NumberFormatException e){
			preview.clear();
			isValid = false;
		}
	}

	private void cancel(){
		setVisible(false);
		isValid = false;
	}

	//Accepts the new format for the line
	private void accept(){
		try{
			String w = (String)sizeBox.getSelectedItem();
			if(!w.equals("")){
				width = Integer.parseInt(w);
				color = chooser.getColor();
				isValid = true;
			}
		}
		catch(NumberFormatException e){
			isValid = false;
		}

		setVisible(false);
	}

	//Accessors
	public Color getLineColor(){
		return color;
	}

	public int getLineWidth(){
		return width;
	}

	public boolean isValid(){
		return isValid;
	}
}

/**
 * Extends JPanel and provides a preview of line formatting
 */
class LinePreview extends JPanel {

	int width;
	Color color;

	public LinePreview(int w, Color c){
		width = w;
		color = c;
	}

	public void setValues(int w, Color c){
		width = w;
		color = c;
	}

	public void paintComponent(Graphics gr){
		super.paintComponent(gr);
		Graphics2D g = (Graphics2D)gr;

		if(width != 0){
			g.setPaint(color);
			g.setStroke(new BasicStroke(width));
			Line2D.Double line = new Line2D.Double(0,50,600,50);
			g.draw(line);
		}
		else{
			g.clearRect(0,0,600,600);
		}
	}

	public void clear(){
		setValues(0,Color.white);
		this.repaint();
	}
}