package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.transmission.*;
import java.util.*;
/**
 * GraderInfo stores all useful information about the current user of the BlaiseGradingPortal.
 *
 * @author Michael Gaiman
 * @version 1 6/13/2004
 */
public class GraderInfo {
    //grader specific
    private String graderID=null;
    private Grader grader=null;
    
    //writer specific
    private String writerID=null;
    private Writer writer=null;
    
    //needed by both
    
    private int type;
    private Course[] courses=null;
    private Hashtable c2a=new Hashtable();//course to assignments
    private Hashtable cNames=new Hashtable();
    private Hashtable aNames=new Hashtable();
    
    public GraderInfo(Grader grader, Course[] courses){
        type=TransmissionHandler.GRADER_TYPE;
        this.grader=grader;
        this.courses=courses;
        graderID=Long.toString(grader.getGraderID());
        populateAssignments();
        makeCourseNames();
    }
    
    public GraderInfo(Writer writer, Course[] courses){
        type=TransmissionHandler.WRITER_TYPE;
        this.writer=writer;
        this.courses=courses;
        writerID=Long.toString(writer.getWriterID());
        populateAssignments();
        makeCourseNames();
    }
    
    /**
     * convert the Vector of Assignments in each Course to an Assignment[]
     * indexed by the course in the hashtable.
     */
    private void populateAssignments(){
        if(courses==null || courses.length==0){
            return;
        }
        
        Assignment[] type=new Assignment[0];
        Assignment[] assigns;
        Vector aVec;
        int i,j;
        for(i=0; i<courses.length;i++){
            aVec=courses[i].getAssignments();
            if(aVec!=null){
                assigns=(Assignment[])aVec.toArray(type);
                c2a.put(courses[i],assigns);
                for(j=0;j<assigns.length;j++){//add the names
                    aNames.put(Long.toString(assigns[j].getAssignID()),assigns[j].name());
                }
            }
        }
    }
    
    /**
     * Makes a human readable course names for each course.
     * Storred in a hashtable indexed by courseID.
     */
    private void makeCourseNames(){
        if(courses==null || courses.length==0){
            return;
        }
        
        String name;
        for(int i=0;i<courses.length;i++){
            name=courses[i].getDept()+' '+courses[i].getNumber()+'-'+courses[i].getSection();
            cNames.put(new Long(courses[i].getCourseID()),name);
        }
    }
    
    /**
     * returns TransmissionHandler.WRITER_TYPE or TransmissionHandler.GRADER_TYPE
     */
    public int getType(){
        return type;
    }
    
    public String getWriterID(){
        return writerID;
    }
    
    public Writer getWriter(){
        return writer;
    }
    
    public String getGraderID(){
        return graderID;
    }
    
    public Grader getGrader(){
        return grader;
    }
    
    public String getUsername(){
        if(grader!=null){
            return grader.getUserName();
        }
        else if(writer!=null){
            return writer.getUserName();
        }
        else{
            return null;
        }
    }
    
    public String getName(boolean lastNameFirst){
        if(grader!=null){
            if(lastNameFirst){
                return grader.getLastName()+", "+grader.getFirstName();
            }
            else{
                return grader.getFirstName()+' '+grader.getLastName();
            }
        }
        else if(writer!=null){
            if(lastNameFirst){
                return writer.getLastName()+", "+writer.getFirstName();
            }
            else{
                return writer.getFirstName()+' '+writer.getLastName();
            }
        }
        else{
            return null;
        }
    }
    
    /**
     * Treat this as read only.
     */
    public Course[] getCourses(){
        return courses;
    }
    
    public Course getCourseAt(int index){
        return courses[index];
    }
    
    public Course getCourseForID(long courseID){
        //courses are usually in courseID, but that isn't an absolute guarentee,
        //so just to an exhaustive search
        for(int i=0;i<courses.length;i++){
            if(courses[i].getCourseID()==courseID){
                return courses[i];
            }
        }
        return null;
    }
    
    /**
     * Returns a nicely formatted name from the courseID.
     */
    public String getCourseNameForID(long courseID){
        return (String)cNames.get(new Long(courseID));
    }
    
    /**
     * Returns the assignments of the course at courseIndex as an array.
     */ 
    public Assignment[] getAssignments(int courseIndex){
        if(courses==null || courseIndex>=courses.length){
            return null;
        }
        return (Assignment[])c2a.get(courses[courseIndex]);
    }
    
    /**
     * Returns the assignments in Course c as an array.
     */
    public Assignment[] getAssignments(Course c){
        return (Assignment[])c2a.get(c);
    }
    
    public String getAssignmentNameForID(long aID){
        return (String)aNames.get(Long.toString(aID));
    }
    
    public String getAssignmentNameForID(String aID){
        return (String)aNames.get(aID);
    }
    
}
