package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.util.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.table.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.io.File;
import java.io.IOException;
/**
 * GradingPane is where graders initiate viewing, grading and then checkin of papers.
 * @author Michael Gaiman
 * @version 1 6/6/2004
 */
public class GradingPane extends AbstractPortalPane{
    private BlaiseGradingPortal parent;
    private JComboBox courseCB;
    private JComboBox assignmentCB;
    private JLabel statusL;
    private JTable table;
    private GradingTableModel tableModel;
    private JButton gradeB;
    private JButton checkinB;
    private JButton viewInfoB;
    private JButton stagingB;
    private PaperStore paperStore;
    
    public static final String UNGRADED_STATUS="U";
    public static final String GRADED_STATUS="G";
    public static final String WORKING_STATUS="W";
    
    public GradingPane(BlaiseGradingPortal parent){
        super(parent);
        this.parent=parent;
        paperStore=new PaperStore(parent.getUserID());
    }
    
    protected void onlineStatusChange(int loginType){
        if(loginType==parent.LIVE){
            //switched to live status... update course and assignment list
            fillInCourses();
            fillInAssignments();
            stagingB.setEnabled(true);
        }
    }
    
    /**
     * Added to combo boxes.  Stores a string and a Course or Course and Assignment.
     */
    private class PairObject{
        public Course course;
        public Assignment assignment;
        public String toString;
        
        public PairObject(Course course, String toString){
            this.course=course;
            this.toString=toString;
            this.assignment=null;
        }
        
        public PairObject(Assignment assignment, Course course, String toString){
            this.course=course;
            this.toString=toString;
            this.assignment=assignment;
        }
        
        public String toString(){
            return toString;
        }
    }
    
    private void fillInCourses(){
        courseCB.removeAllItems();
        courseCB.addItem(new PairObject(null,"All Courses"));
        GraderInfo gi=parent.getGraderInfo();
        Course[] courses=gi.getCourses();
        String name;
        for(int i=0;i<courses.length;i++){
            name=gi.getCourseNameForID(courses[i].getCourseID());
            courseCB.addItem(new PairObject(courses[i],name));
        }
    }
    
    private void fillInAssignments(){
        assignmentCB.removeAllItems();
        PairObject course=(PairObject)courseCB.getSelectedItem();
        if(course.course==null){
            assignmentCB.addItem(new PairObject(null,null,"All Assignments"));
            //if we wanted to make assignmentCB list all assignments, by course,
            //here's where it would go.
        }
        else{
            Assignment[] assigns=parent.getGraderInfo().getAssignments(course.course);
            String name;
            assignmentCB.addItem(new PairObject(null,course.course,"All Assignments"));
            if(assigns==null){
                return;
            }
            for(int i=0;i<assigns.length;i++){
                name=Integer.toString(assigns[i].assignNum())+' '+assigns[i].name();
                assignmentCB.addItem(new PairObject(assigns[i],course.course,name));
            }
        }
    }
    
    protected JLabel getHeader(){
        return new JLabel("In the grading tab select papers to grade and papers to check in.");
    }
    
    protected Component getUpperPanel(){
        JPanel p=new JPanel();
        p.add(new JLabel("Show",JLabel.RIGHT));
        courseCB=new JComboBox();
        courseCB.setEditable(false);
        fillInCourses();
        courseCB.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e){
                if(e.getStateChange()==ItemEvent.SELECTED){
                    fillInAssignments();
                }
            }
        });
        courseCB.setFocusable(false);
        p.add(courseCB);
        assignmentCB=new JComboBox();
        assignmentCB.setEditable(false);
        fillInAssignments();
        assignmentCB.setFocusable(false);
        p.add(assignmentCB);
        return p;
    }
    
    protected Component getLowerPanel(){
        statusL=new JLabel();//instantiated first because tableModel sets it.
        tableModel=new GradingTableModel();
        TableSorter sorter=new TableSorter(tableModel);
        table=new JTable(sorter,new GradingColumnModel());
        //table=new JTable(tableModel,new GradingColumnModel());
        table.createDefaultColumnsFromModel();
        sorter.setTableHeader(table.getTableHeader());
        table.setDefaultRenderer(Object.class,new BoldingCellRenderer());
        table.setShowHorizontalLines(true);
        table.setShowVerticalLines(true);
        table.setFocusable(false);
        
        //table.getInputMap(table.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_A,Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(),), "select-all-action");
        //table.getActionMap().put("select-all-action", new SelectAllAction(table));
        
        //done as a work around to a bug in the table selection listener.
        table.addMouseListener(new MouseAdapter(){
            public void mouseReleased(MouseEvent e){
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        //System.out.println("there are "+table.getSelectedRowCount()+" selected rows");
                        int count=table.getSelectedRowCount();
                        boolean some=(count>0);
                        boolean single=(count==1);
                        gradeB.setEnabled(some && single);
                        checkinB.setEnabled(some);
                        viewInfoB.setEnabled(single);
                    }
                });
            }
        });
        
        JPanel panel=new JPanel(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        c.weighty=.999;
        c.weightx=1;
        c.fill=c.BOTH;
        panel.add(new JScrollPane(table),c);
        c.fill=c.HORIZONTAL;
        c.gridy=1;
        c.weighty=.001;
        panel.add(statusL,c);
        
        return panel;
    }
    
    protected AbstractButton[] getBottomButtons(){
    
        JButton[] buttons=new JButton[5];
        buttons[0]=new JButton(new SelectAllAction(table));
        buttons[0].setText("Select All");
        buttons[0].setFocusable(false);
        
        buttons[1]=new JButton(new ViewInfoAction());
        buttons[1].setText("View Paper Details");
        buttons[1].setEnabled(false);//no papers selected initially.
        buttons[1].setFocusable(false);
        viewInfoB=buttons[1];
        
        buttons[2]=new JButton(new GradeAction());
        buttons[2].setText("Grade Paper");
        buttons[2].setEnabled(false);//no papers selected initially.
        buttons[2].setFocusable(false);
        gradeB=buttons[2];
        
        buttons[3]=new JButton(new CheckinAction());
        buttons[3].setText("Check in Papers");
        buttons[3].setEnabled(false);//no papers selected initially.
        buttons[3].setFocusable(false);
        checkinB=buttons[3];
        
        buttons[4]=new JButton(new StagingAction());
        buttons[4].setText("Send Notifications");
        buttons[4].setEnabled(TransmissionHandler.isOnline());
        buttons[4].setFocusable(false);
        stagingB=buttons[4];
        
        return buttons;
    }
    
    private void setStatusText(int numU,int numW,int numG){
        StringBuffer sb=new StringBuffer("Showing: ");
        boolean u=(numU>0),w=(numW>0),g=(numG>0);
        if(u){
            sb.append(numU).append(" Ungraded");
        }
        if(w){
            if(u && !g){
                sb.append(" and ");
            }
            else if(u){
                sb.append(", ");
            }
            sb.append(numW).append(" Working");
        }
        if(g){
            if(u || w){
                sb.append(" and ");
            }
            sb.append(numG).append(" Graded");
        }
        if(u || w || g){
            int total=numU+numW+numG;
            if(total==1){
                sb.append(" paper");
            }
            else{
                sb.append(" papers");
            }
        }
        else{
            sb.append("0 papers");
        }

        statusL.setText(sb.toString());
    }
    
    /**
     * Adds an Blaise_Papers in the supplied vector to the table
     * It does this by saving the papers onto the filesystem,
     * adding them to the paper store, and then finally adding the
     * PaperStore.Entry objects to the table.
     * (This method is called by the checkoutpane to add papers to this table.)
     */
    void addCheckedOutPapers(final Vector papers){
        String filename;
        Blaise_Paper bp;
        GraderInfo gi=parent.getGraderInfo();
        final int size=papers.size();
        String gName=gi.getUsername();
        String cName;
        String dir=Constants.getGraderPaperDirectory()+parent.getUserID()+File.separator;
        BlaiseFile bf=new BlaiseFile();
        BackendFileHandler bfh=new BackendFileHandler();
        File file;
        for(int i=0;i<size;i++){
            bp=(Blaise_Paper)papers.elementAt(i);
            filename=dir+bp.getPaperID()+".bls";
            cName=gi.getCourseNameForID(Long.parseLong(bp.getCourseID()));
            paperStore.addPaper(bp,cName,gName,filename,UNGRADED_STATUS);
            try{
                file=new File(filename);
                if(!file.exists()){
                    DocumentUtilities.makeDirs(file);
                    file.createNewFile();
                }
                bf.newJob(bp,file,bfh);
                bf.savePaper();
            }
            catch(IOException e){
                System.out.println("Could not save paper with paperID:"+bp.getPaperID()+
                    " due to exception: "+e);
                class MyRunnable implements Runnable{
                    private Blaise_Paper bp;
                    public MyRunnable(Blaise_Paper bp){
                        this.bp=bp;
                    }
                    public void run(){
                        JOptionPane.showMessageDialog(parent,"An error occurred while saving a checked out paper.\nPlease consult a Blaise Administrator for support.\n(pID:"+bp.getPaperID()+" gID:"+parent.getUserID()+")","Checkout Error",JOptionPane.ERROR_MESSAGE);
                    }
                }
                SwingUtilities.invokeLater(new MyRunnable(bp));
            }
        }
        //this will probably be called from a different thread,
        //so do the GUI related stuff in the event thread.
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                for(int i=0;i<size;i++){
                    tableModel.addPaper((Blaise_Paper)papers.elementAt(i));
                }
            }
        });
    }
    
    private class ViewInfoAction extends AbstractAction{
        public static final String NAME="view-info-action";
        public ViewInfoAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Displays a dialog with the details of the selected paper");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            Thread thread=new Thread(){
                public void run(){
                    PaperStore.Entry entry=tableModel.getEntryAt(table.getSelectedRow());
                    Blaise_Paper bp=new Blaise_Paper();
                    BlaiseFile bf=new BlaiseFile(bp,new File(entry.getPaperFilePath()),new BackendFileHandler());
                    try{
                        parent.startProgView("Preparing Paper");
                        bf.openPaper();
                    }catch(IOException e){
                        System.out.println(e);
                        JOptionPane.showMessageDialog(parent,"Could not show paper properties.\n(Exception:"+e+")","An Error Occurred",JOptionPane.ERROR_MESSAGE);
                        bp=null;
                    }
                    finally{
                        parent.stopProgView();
                    }
                    if(bp!=null){
                        final Blaise_Paper paper=bp;
                        SwingUtilities.invokeLater(new Runnable(){
                            public void run(){
                                new PaperInfoDialog(parent,paper);
                            }
                        });
                    }
                }
            };
            thread.start();
        }
    }
    
    private class StagingAction extends AbstractAction{
        public static final String NAME="staging-action";
        public StagingAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Send students notifications that their papers are graded.");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            //because offerOnline needs to be called from a different thread,
            //put this in a different thread.  Annoying, yes, but needed.
            Thread thread=new Thread(){
                public void run(){
                    //go online if needed (and able)
                    if(!parent.offerOnline()){
                        //unable to go online...
                        return;
                    }
                    new StagingDialog(parent,parent.getGraderInfo());
                }
            };
            thread.start();
        }
    }
    
    private class GradeAction extends AbstractAction{
        public static final String NAME="grade-paper-action";
        public GradeAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"View and grade the currently selected paper.");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            final Blaise_Paper paper=getSelectedPaper();
            final File file=getSelectedPaperFile();
			
			//before we hand the paper off the grader, we need to go through
			//and set any existing comments by the logged in grader to be
			//"My Marks"
			//we'll need to undo this just before we submit this to the server
			Vector drafts=paper.getDrafts(), comments;
			int dSize=drafts.size(),cSize,i,j;
			Blaise_Draft d;
			Blaise_Comment c;
			for(i=0;i<dSize;i++){
				d=(Blaise_Draft)drafts.elementAt(i);
				comments=d.getComments();
				cSize=comments.size();
				for(j=0;j<cSize;j++){
					c=(Blaise_Comment)comments.elementAt(j);
					if(c.getGraderID().equals(parent.getUserID())){
						c.setGraderName("My Marks");
					}
				}
			}
			
            Thread thread=new Thread(){
                public void run(){
                    PortalToGraderInterface ptgi=new PortalToGraderInterface(){
                        public Blaise_Paper getPaper(){
                            return paper;
                        }
                        public GraderInfo getGraderInfo(){
                            return parent.getGraderInfo();
                        }
                        public void setStatusWorking(String paperID){
                            tableModel.setStatus(paperID,WORKING_STATUS);
                        }
                        public void setStatusGraded(String paperID){
                            tableModel.setStatus(paperID,GRADED_STATUS);
                        }
                        public void setStatusUngraded(String paperID){
                            tableModel.setStatus(paperID,UNGRADED_STATUS);
                        }
                        public void done(String paperID){}
                        public File getPaperFile(){
                            return file;
                        }
                    };
                    new Blaise_Document_Grader(ptgi);
                }
            };
            thread.start();
        }
        
        /**
         * Returns the currently selected paper.
         */
        private Blaise_Paper getSelectedPaper(){
            Blaise_Paper paper=new Blaise_Paper();
            BlaiseFile bf=new BlaiseFile(paper,getSelectedPaperFile(),new GraderFileHandler());
            try{
                bf.openPaper();
            }catch(IOException e){
                paper=null;
                JOptionPane.showMessageDialog(parent,"Could not open paper (Exception:"+e+')',"An Error Occurred",JOptionPane.ERROR_MESSAGE);
            }
            return paper;
        }
        
        private File getSelectedPaperFile(){
            PaperStore.Entry entry=tableModel.getEntryAt(table.getSelectedRow());
            return new File(entry.getPaperFilePath());
        }
    }
    
    private class CheckinAction extends AbstractAction implements CheckinCallBack{
        public static final String NAME="checkin-paper-action";
        public CheckinAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Checkin the currently selected papers.");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            final int[] rows=table.getSelectedRows();
            Thread thread=new Thread(){
                public void run(){
                    //go online if needed (and able)
                    if(!parent.offerOnline()){
                        //unable to go online...
                        return;
                    }
                    
                    Vector entries=new Vector();
                    for(int i=0;i<rows.length;i++){
                        entries.addElement(tableModel.getEntryAt(rows[i]));
                    }
    
                    new CheckinDialog(parent,entries,CheckinAction.this);
                }
            };
            thread.start();
        }
        
        /**
         * CheckinDialog calls this with each successful entry.
         */
        public void handleEntry(final PaperStore.Entry entry){
            //delete file
            File f=new File(entry.getPaperFilePath());
            if(!f.delete()){
                System.out.println("Could not delete file "+f);
            }
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    //remove from table
                    tableModel.removeEntry(entry.getPaperID());
                    //remove from paperstore
                    //done after it is removed from the table to avoid a null pointer exception.
                    synchronized(paperStore){
                        paperStore.removeEntry(entry);
                    }
                }
            });
        }
        
        public void checkinDone(boolean success){
            try{
                paperStore.writeStore();
            }catch(IOException e){
                //let's just be hopeful that it will write out successfully on shutdown.
                System.out.println("PaperStore could not serialize changes.  (Error: "+e+")");
            }
        }
    }
    
    /**
     * A TableModel for use with PaperStore.Entries.
     */
    private class GradingTableModel extends AbstractTableModel implements ItemListener{
        private final String[] columns={"Status","Course","Assignment","Student","Draft Count","Turn-in Date"};
        private Vector ids;
        private Course currentCourse;
        private Assignment currentAssignment;
        private int numU=0,numG=0,numW=0;
        public GradingTableModel(){
            //register to know about changes to the controlling combo boxes.
            courseCB.addItemListener(this);
            assignmentCB.addItemListener(this);
            
            //initially course and assignments are set to all, and are thus null.
            currentCourse=null;
            currentAssignment=null;
        
            //get a Vector of all of the paperIDs, this will allow us to make
            //a row to Entry mapping.
            ids=new Vector();
            Enumeration e=paperStore.ids();
            for(;e.hasMoreElements();){
                ids.add(e.nextElement());
            }
            updateStatusText();
        }
        
        /**
         * respond to changes in the show combo boxes
         */
        public void itemStateChanged(ItemEvent e){
            if(e.getStateChange()==ItemEvent.SELECTED){
                //get current course and assignment
                currentCourse=((PairObject)courseCB.getSelectedItem()).course;
                currentAssignment=((PairObject)assignmentCB.getSelectedItem()).assignment;
                
                Vector v;
                if(currentAssignment!=null){
                    v=paperStore.getEntriesByAssignment(Long.toString(currentAssignment.getAssignID()));
                }
                else if(currentCourse!=null){
                    v=paperStore.getEntriesByCourse(Long.toString(currentCourse.getCourseID()));
                }
                else{//all papers
                    v=paperStore.getPaperIDs();
                }
                
                if(v==null){
                    ids.clear();
                }
                else{
                    //Note: makes copies of the Vectors returned because the ids vector gets added to
                    //when new papers are checked out, and vectors from paperStore should be thought of
                    //as read only.
                    ids=new Vector(v);
                }
                fireTableDataChanged();
                updateStatusText();
            }
        }
        
        private void updateStatusText(){
            numU=numW=numG=0;
            int size=ids.size();
            PaperStore.Entry entry;
            String status;
            for(int i=0;i<size;i++){
                entry=paperStore.getEntry((String)ids.elementAt(i));
                status=entry.getStatus();
                if(status.equals(UNGRADED_STATUS)){
                    numU++;
                }
                else if(status.equals(WORKING_STATUS)){
                    numW++;
                }
                else{
                    numG++;
                }
            }
            setStatusText(numU,numW,numG);
        }
        
        /**
         * updates the status text due to the insertion of a single paper (paperID).
         */
        private void updateStatusText(String paperID){
            String status=paperStore.getEntry(paperID).getStatus();
            if(status.equals(UNGRADED_STATUS)){
                numU++;
            }
            else if(status.equals(WORKING_STATUS)){
                numW++;
            }
            else{
                numG++;
            }
            setStatusText(numU,numW,numG);
        }
        
        public int getRowCount(){
            int size;
            synchronized(ids){
                size=ids.size();
            }
            return size;
        }
        public String getColumnName(int i){
            return columns[i];
        }
        public int getColumnCount(){
            return columns.length;
        }
        public Object getValueAt(int row, int column){
            PaperStore.Entry entry;
            String pid;
            synchronized(ids){
                pid=(String)ids.elementAt(row);
            }
            synchronized(paperStore){
                entry=paperStore.getEntry(pid);
            }
            switch(column){
                case 0:
                    return entry.getStatus();
                case 1:
                    return entry.getCourse();
                case 2:
                    if(currentAssignment==null){
                        return parent.getGraderInfo().getAssignmentNameForID(entry.getAssignmentID());
                    }
                    else{
                        return currentAssignment.name();
                    }
                case 3:
                    return entry.getFirstName()+' '+entry.getLastName();
                case 4:
                    return entry.getDraftCount();
                case 5:
                    return entry.getTurninDate();
                default:
                    return "Invalid";
            }
        }
        
        public PaperStore.Entry getEntryAt(int row){
            return paperStore.getEntry((String)ids.elementAt(row));
        }
        
        /**
         * Adds the paper to the current table view (if it matches the user's settings)
         */
        public void addPaper(Blaise_Paper bp){
            String paperID=bp.getPaperID();
            int index=ids.indexOf(paperID);
            if(index>=0){
                //already in the table (but this one has new drafts or something)
                //so just update it's row.
                fireTableRowsUpdated(index,index);
                updateStatusText();
                return;
            }
            
            if(currentCourse==null){//simple case--all courses
                ids.addElement(paperID);
                int r=ids.size()-1;
                fireTableRowsInserted(r,r);
                updateStatusText(paperID);
                return;
            }
            
            if(currentCourse.getCourseID()==Long.parseLong(bp.getCourseID())){
                if(currentAssignment==null){
                    ids.addElement(paperID);
                    int r=ids.size()-1;
                    fireTableRowsInserted(r,r);
                    updateStatusText(paperID);
                    return;
                }
                
                if(currentAssignment.getAssignID()==Long.parseLong(bp.getAssignmentID())){
                    ids.addElement(paperID);
                    int r=ids.size()-1;
                    fireTableRowsInserted(r,r);
                    updateStatusText(paperID);
                    return;
                }
            }
        }
        
        /**
         * Updates the table for the correct entry
         */
        public void entryModified(String paperID){
            int row=ids.indexOf(paperID);
            if(row!=-1){
                fireTableRowsUpdated(row,row);
                updateStatusText();
            }
        }
        
        public void removeEntry(String paperID){
            synchronized(ids){
                int row=ids.indexOf(paperID);
                if(row!=-1){
                    ids.removeElementAt(row);
                    fireTableRowsDeleted(row,row);
                    //fireTableDataChanged();
                    updateStatusText();
                }
            }
        }
        
        /**
         * Adjusts the status of an entry.
         */
        public void setStatus(String paperID, String status){
            paperStore.modifyEntry(paperID,null,status);
            entryModified(paperID);
        }
    }
    
    /**
     * Sets the default column spacing.
     */
    private class GradingColumnModel extends DefaultTableColumnModel{
        private int charWidth=0;
        private BoldingCellRenderer rightBCR;
        
        public GradingColumnModel(){
            rightBCR=new BoldingCellRenderer();
            rightBCR.setHorizontalAlignment(SwingConstants.RIGHT);
            
            //update the buttons depending on selection status.
            selectionModel.addListSelectionListener(new ListSelectionListener(){
                public void valueChanged(ListSelectionEvent e){
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            //System.out.println("there are "+table.getSelectedRowCount()+" selected rows");
                            int count=table.getSelectedRowCount();
                            boolean some=(count>0);
                            boolean single=(count==1);
                            gradeB.setEnabled(some && single);
                            checkinB.setEnabled(some);
                            viewInfoB.setEnabled(single);
                        }
                    });
                }
            });
        }
        
        public void addColumn(TableColumn tc){
            if (charWidth == 0) {//look up character width, to make it easier to figure out column widths
                Font f=UIManager.getLookAndFeelDefaults().getFont("Table.font");
                //System.out.println("table font: "+f);
                FontMetrics metrics = getFontMetrics(f);
                charWidth = metrics.charWidth('m');
                //System.out.println("charWidth:"+charWidth);
            }
        
            String name = (String)tc.getHeaderValue();
            if(name.equals("Status")){
                tc.setPreferredWidth(6*charWidth);//6: "Status"
            }
            else if(name.equals("Course")){
                tc.setPreferredWidth(11*charWidth);
            }
            else if(name.equals("Assignment")){
                tc.setPreferredWidth(20*charWidth);
            }
            else if(name.equals("Student")){
                tc.setPreferredWidth(20*charWidth);
            }
            else if(name.equals("Draft Count")){
                tc.setPreferredWidth(11*charWidth);//11: "Draft Count" (otherwise it would be 1 or 2)
                tc.setCellRenderer(rightBCR);
            }
            else if(name.equals("Turn-in Date")){
                tc.setPreferredWidth(17*charWidth);
            }
            
            super.addColumn(tc);
        }
    }
    
    /**
     * Makes rows that have UNGRADED_STATUS bold.
     */
    private class BoldingCellRenderer extends DefaultTableCellRenderer{
        private Font fontB=null;
        private Font fontP=null;
    
        public Component getTableCellRendererComponent(JTable table,Object value,boolean isSelected,boolean hasFocus,int row,int column){
            //check the status of the row.  If it is UNGRADED_STATUS then make this cell bold.
            String status=(String)tableModel.getValueAt(row,0);//0 is status
            
            //in super.getTableCellRendererComponent the font gets set to the table's font, and
            //the string get set (among other things).  It then returns itself.
            //we're going to catch that and override the font.
            
            super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);
            
            if(status.equals(UNGRADED_STATUS)){
                if(fontB==null){
                    fontB=getFont().deriveFont(Font.BOLD);
                }
                setFont(fontB);
            }
            else{
                if(fontP==null){
                    fontP=getFont().deriveFont(Font.PLAIN);
                }
                setFont(fontP);
            }
            return this;
        }
    }
}
