package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.documents.Blaise_Paper;
import java.io.File;
/**
 * PortalToGraderInterface is the interface that allows the
 * Grading Portal and the Grader to communicate.
 * An instantiated copy of this class is passed to the Grader on
 * invocation, and the Grader when actions are performed by the user calls
 * back to the Portal to update the status of papers, etc.
 *
 * All of the methods (except getPaper()) have a paperID argument.  This
 * is because the Portal supports having multiple Grading documents open at at time
 * and the PortalToGraderInterface object is shared between Grader instances.
 *
 * @author Michael Gaiman
 * @version 1 7/19/2004
 */
public interface PortalToGraderInterface {
    /**
     * Called by the Grader to get the paper to grade.
     * @returns a Blaise_Paper object (which was opened via the GraderFileHandler)
     */
    public Blaise_Paper getPaper();
    
    /**
     * Called by the Grader to get the File of the Blaise_Paper.
     */
    public File getPaperFile();
    
    /**
     * Returns the associated GraderInfo object that the current user has.
     * (Note: GraderInfo is sort of a misnomer, it is really for both the
     * Grader and/or Writer (writer as grader).
     */
    public GraderInfo getGraderInfo();
    
    /**
     * Every checked out paper has a status.  There are three status values:
     * ungraded, working and graded.
     * Working status means that the user is in the process of grading this paper
     * This is true under a few circumstances: they are actually working on it right now,
     * they have chosen to save their work but not finalize the paper or draft, or
     * they have finalized some drafts but there are other drafts that are ungraded still.
     */
    public void setStatusWorking(String paperID);
    
    /**
     * Every checked out paper has a status.  There are three status values:
     * ungraded, working and graded.
     * Graded status means that the user has finished grading a paper.  This occurs in
     * two cases: they have graded every draft in a paper, or they have clicked "finalize paper"
     * and are thus finished grading the paper for this checkout cycle.
     */
    public void setStatusGraded(String paperID);
    
    /**
     * Every checked out paper has a status.  There are three status values:
     * ungraded, working and graded.
     * Ungraded status means that they have not began to grade the paper yet.
     * This is the case until they first click "Grade Draft..."
     */
    public void setStatusUngraded(String paperID);
    
    /**
     * Called when the Grader is exiting to let the Portal know to do any housekeeping
     * that may need to be done.
     */
    public void done(String paperID);
}
