package edu.gwu.blaise.version2.grader;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import java.util.*;
import java.io.*;

import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.documents.*;

/**
 * The SearchPane is used by a grader to search for documents based on certain criteria, move documents to their briefcase
 *
 */
public class SearchPane extends JPanel{

	public static byte SEARCH = 0;
	public static byte VIEW = 1;

	private boolean debug = false;	//for debugging

	JFrame parent;			//parent Frame
	ProgressView progView;

	//Panel components
	JTextField lastField;
	JTextField ssnField;
	JComboBox coursesBox;
	JCheckBox onlyUngradedCheck;
	JButton addToBrief;

	//Table model and JTable
	BlaiseTableModel rt;
	BlaiseTableColumnModel ft;
	JTable jt;

	Vector paperVector;		//papers currently being displayed by the table
	BriefCasePane briefP;	//briefcase

	boolean coursesFilled = false;

	/**
	 * Constructor
	 *
	 * @param par What the SearchPanel is contained within
	 * @param briefPane The briefcase pane object
	 * @param cs Courses grader can download from
	 * @param identifier The identification of the grader (grader id)
	 */
	public SearchPane(JFrame parentFrame, BriefCasePane briefPane){
		//Initialize variables
		briefP = briefPane;
		parent = parentFrame;

		//Crate table and column models and create JTable to display results
		rt = new BlaiseTableModel();
		ft = new BlaiseTableColumnModel();
		jt = new JTable(rt,ft);
		jt.createDefaultColumnsFromModel();
		jt.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		UIManager.put("Table.focusCellHighlightBorder", new LineBorder(new Color(49,106,197), 0 ) );

		JScrollPane scroll = new JScrollPane(jt);	//add table to scrollpane

		//upper half panel and overall layout manager
		JPanel upper = new JPanel(new BorderLayout());
		JPanel lower = new JPanel(new BorderLayout());
		this.setLayout(new BorderLayout());

		//Set up border
		TitledBorder title = BorderFactory.createTitledBorder("Search Criteria");
		upper.setBorder(title);

		//Set up sub-panels
		JPanel buttonPanel = new JPanel();
		JPanel fieldPanel = new JPanel(new GridLayout(3,2,4,4));
		JPanel bottomButtons = new JPanel();

	/*Field Panel Setup*/
		//Set up labels
		JLabel lastLabel = new JLabel("Last Name:",JLabel.CENTER);
		JLabel ssnLabel = new JLabel("Student Number:",JLabel.CENTER);
		JLabel courseLabel = new JLabel("Course:",JLabel.CENTER);

		//Set up fields
		lastField = new JTextField(20);
		ssnField = new JTextField(20);

		//Set up Combo Box
		coursesBox = new JComboBox();
		coursesBox.setEditable(false);
		coursesBox.addItem("All My Courses");

		if(Blaise_Grader.loggedIn){
			if((Blaise_Grader.courses).size() != 0 && !coursesFilled){
				for(int i=0;i<Blaise_Grader.courses.size();i++){
					Course current = (Course)Blaise_Grader.courses.elementAt(i);
					String build = (i+1) + " "+current.getDept() + " " + current.getNumber() + "-" + current.getSection();
					coursesBox.addItem(build);
				}
				coursesFilled = true;
			}
		}

		//Add labels and fields to panel
		fieldPanel.add(lastLabel);
		fieldPanel.add(lastField);
		fieldPanel.add(ssnLabel);
		fieldPanel.add(ssnField);
		fieldPanel.add(courseLabel);
		fieldPanel.add(coursesBox);


	/*BUTTON SETUP*/
		//Set up buttonPanel
		JButton searchButton = new JButton("Search");
		JButton clearButton = new JButton("Reset Fields");
		onlyUngradedCheck = new JCheckBox("Only Ungraded Papers", true);
		buttonPanel.add(searchButton);
		buttonPanel.add(clearButton);
		buttonPanel.add(onlyUngradedCheck);

		//On clear, clear all forms
		clearButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){
				lastField.setText("");
				lastField.setEditable(true);
				ssnField.setText("");
				ssnField.setEditable(true);
				coursesBox.setSelectedIndex(0);
			}
		});

		//On search, query server for values of fields
		searchButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){

				if(debug)System.out.println("SearchPane -- Beginning Searching");

				paperVector = new Vector();

				//Activate or deactivate add to brieface button based on onlyUngradedCheck
				if(!onlyUngradedCheck.isSelected()){
					addToBrief.setEnabled(false);
				}
				else{
					addToBrief.setEnabled(true);
				}

				if(!Blaise_Grader.loggedIn){
					Blaise_Grader.doLogin(parent);
					if(Blaise_Grader.loggedIn){
						if((Blaise_Grader.courses).size() != 0 && !coursesFilled){
							for(int i=0;i<Blaise_Grader.courses.size();i++){
								Course current = (Course)Blaise_Grader.courses.elementAt(i);
								String build = (i+1) + " "+current.getDept() + " " + current.getNumber() + "-" + current.getSection();
								coursesBox.addItem(build);
							}
							coursesFilled = true;
						}
					}
					else{
						return;
					}
				}

				if(!TransmissionHandler.isOnline()){
					JOptionPane.showMessageDialog(parent, "Internet Connection Not Found.\nYou Must Be Online To Access The Blaise Server.", "FAILURE", JOptionPane.ERROR_MESSAGE);
					return;
				}


				//Determine if Course has been selected to search by
				int desired_index = coursesBox.getSelectedIndex()-1; //must be -1 b/c the first entry is "All My Courses"
				PaperSearchQuery query;
				if(desired_index >= 0){
					query = TransmissionHandler.getPaperSearch(onlyUngradedCheck.isSelected(),lastField.getText(),ssnField.getText(), "", (Course)Blaise_Grader.courses.elementAt(desired_index));
				}
				else{
					query = TransmissionHandler.getPaperSearch(onlyUngradedCheck.isSelected(),lastField.getText(),ssnField.getText(), "", null);
				}


				doThread(SearchPane.SEARCH, "Performing Search...", query);
			}
		});

	/*Bottom Button SETUP*/
		JButton selectAll = new JButton("Select All");
		JButton selectNone = new JButton("Select None");
		addToBrief = new JButton("Add Selected To Brief-Case");
		JButton viewSelected = new JButton("View Selected Paper");
		bottomButtons.add(selectAll);
		bottomButtons.add(selectNone);
		bottomButtons.add(addToBrief);
		bottomButtons.add(viewSelected);

		//On select all, select all papers from table
		selectAll.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){
				jt.selectAll();
			}
		});

		//On select none, deselect any selected papers
		selectNone.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){
				jt.clearSelection();
			}
		});

		//On addToBrief, get indexes of papers and add to Vector representing BriefCase
		addToBrief.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){

				//get indexes of selected papers
				int[] selected = jt.getSelectedRows();

				if(selected.length != 0){
					//add papers selected to briefcase...done backwards other elements aren't removed
					for(int i=selected.length-1;i>=0;i--){
						Blaise_Paper current = (Blaise_Paper)paperVector.elementAt(selected[i]);
						BriefCase.addPaper(current);
						paperVector.removeElementAt(selected[i]);
					}

					rt.setTable(paperVector);	//set table with new paper vector
					briefP.updateBriefCase();	//update the briefcase panel
				}
			}
		});

		//On select none, deselect any selected papers
		viewSelected.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent rv){

				//Go to server and extract values
				int[] selected = jt.getSelectedRows();

				if(selected.length == 0)return;

				Blaise_Paper current = (Blaise_Paper)paperVector.elementAt(selected[0]);
				Vector IDs = new Vector();
				IDs.add(new Long(current.getPaperID()));

				PaperDownloadQuery query = TransmissionHandler.getPaperDownload(IDs, TransmissionHandler.GRADER_TYPE);
				doThread(SearchPane.VIEW, "Retrieving Paper...", query);
			}
		});

		//Add all upper and lower components to the proper panel
		upper.add(fieldPanel, BorderLayout.NORTH);
		upper.add(buttonPanel, BorderLayout.CENTER);

		lower.add(bottomButtons, BorderLayout.SOUTH);
		lower.add(scroll, BorderLayout.CENTER);

		//Add upper and lower panels to this frame
		this.add(upper,BorderLayout.NORTH);
		this.add(lower, BorderLayout.CENTER);
	}

    private void doThread(final byte type, String message, final Object query){

    	// Start the indeterminate version.
    	progView = new ProgressView(parent, message, 0, true);

    	// Fire off the actual work as a thread:
    	Thread thread = new Thread () {
        	public void run ()
        	{
        	  if(type == SearchPane.SEARCH) doSearch(query);
        	  else if(type == SearchPane.VIEW) doView(query);
        	}
    	};
    	//SwingUtilities.invokeLater (thread);
		thread.start();
	}

	private void doSearch(Object obj){

		PaperSearchQuery query = (PaperSearchQuery)obj;

		if(query.transmit()){
			paperVector = query.getPapers();
		}
		else{
			JOptionPane.showMessageDialog(parent, query.getMessage(), "Failure", JOptionPane.ERROR_MESSAGE);
		}

		rt.setTable(paperVector);	//set table with new paper vector

		progView.done();
	}

	private void doView(Object obj){

		PaperDownloadQuery query = (PaperDownloadQuery)obj;

		if(!query.transmit()){
			JOptionPane.showMessageDialog(parent, query.getMessage(), "Failure", JOptionPane.ERROR_MESSAGE);
		}
		else{
			Vector papers = query.getPapers();
			Blaise_Paper paper = (Blaise_Paper)papers.elementAt(0);
			BlaiseSplitPane pane = new BlaiseSplitPane(BlaiseSplitPane.VIEWER, paper);
			pane.setSize(500,500);
		}

		progView.done();
	}
}