package edu.gwu.blaise.version2.grader;
import edu.gwu.blaise.version2.transmission.*;
import edu.gwu.blaise.version2.util.Constants;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
/**
 * A dialog to present a list of courses to flush
 * the papers that are staged.
 * @author Michael Gaiman
 * @version 1 6/25/2004
 */
public class StagingDialog extends JDialog{
    private BlaiseGradingPortal parent;
    private GraderInfo graderInfo;
    private JComboBox courseCB;
    
    private static final int WIDTH=270;
    private static final int HEIGHT=160;
    
    /**
     * Assumes that the user is online.  Don't display the dialog if they're not.
     */
    public StagingDialog(BlaiseGradingPortal parent,GraderInfo graderInfo){
        super(parent,"Staged Paper Return",true);
        this.parent=parent;
        this.graderInfo=graderInfo;
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private void createAndShowGUI(){
        Container cont=getContentPane();
        cont.setLayout(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        
        JLabel jta=new JLabel("<html><center><p>To notify students that their papers have been graded simply choose a course.</p></center><html>");
        
        courseCB=new JComboBox();
        courseCB.addItem(new PairObject(null,"All Courses"));
        Course[] courses=graderInfo.getCourses();
        String name;
        for(int i=0;i<courses.length;i++){
            name=graderInfo.getCourseNameForID(courses[i].getCourseID());
            courseCB.addItem(new PairObject(courses[i],name));
        }
        
        JButton cancelB=new JButton("Cancel");
        cancelB.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                setVisible(false);
                dispose();
            }
        });
        
        JButton notifyB=new JButton(new DoStagingAction());
        notifyB.setText("Notify Students");
        
        c.fill=c.BOTH;
        c.weightx=1;
        c.gridwidth=2;
        c.gridx=0;
        c.gridy=0;
        c.weighty=.6;
        cont.add(jta,c);
        
        c.gridy=1;
        c.weighty=.2;
        c.fill=c.NONE;
        cont.add(courseCB,c);
        
        c.gridy=2;
        c.gridwidth=1;
        if(Constants.isOSX()){
            cont.add(cancelB,c);
        }
        else{
            cont.add(notifyB,c);
        }
        c.gridx=1;
        if(Constants.isOSX()){
            cont.add(notifyB,c);
        }
        else{
            cont.add(cancelB,c);
        }
        
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.getX();
            pY=parent.getY();
            pW=parent.getWidth();
            pH=parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
    
        setSize(WIDTH,HEIGHT);
        setLocation(myX,myY);
        
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        getRootPane().setDefaultButton(notifyB);
        setResizable(false);
        setVisible(true);
    }
    
    private class DoStagingAction extends AbstractAction{
        public static final String NAME="do-staging-action";
        public DoStagingAction(){
            super(NAME);
            putValue(Action.SHORT_DESCRIPTION,"Send students notifications that their papers are graded.");
            putValue(Action.ACTION_COMMAND_KEY,NAME);
        }
        
        public void actionPerformed(ActionEvent ae){
            final Course course=((PairObject)courseCB.getSelectedItem()).course;
            setVisible(false);
            Thread thread=new Thread(){
                public void run(){
                    try{
                        parent.startProgView("Sending notification request...");
                        Vector cIDs=new Vector();
                        if(course==null){
                            //all courses.
                            Course[] courses=graderInfo.getCourses();
                            for(int i=0;i<courses.length;i++){
                                cIDs.addElement(new Long(courses[i].getCourseID()));
                            }
                        }
                        else{
                            cIDs.addElement(new Long(course.getCourseID()));
                        }
                        TransmissionQuery tq=TransmissionHandler.getSendStagedPapers(cIDs,parent.USER_TYPE);
                        if(!tq.transmit()){
                            //failure of some sort...tell user
                            parent.stopProgView();
                            JOptionPane.showMessageDialog(parent,tq.getMessage(),"Transmission Failure",JOptionPane.ERROR_MESSAGE);
                            return;
                        }
                    }
                    finally{
                        parent.stopProgView();
                    }
                }
            };
            thread.start();
        }
    }
    
    
    private class PairObject{
        public Course course;
        public String toString;
        public PairObject(Course course,String toString){
            this.course=course;
            this.toString=toString;
        }
        public String toString(){
            return toString;
        }
    }
}
