//LOC:225
package edu.gwu.blaise.version2.grader;

import java.util.*;
import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;

/**
 * TabletPapers serves as a repository for papers on the tablet.
 */
public class TabletPapers{

	//Each type has it's own vector
	public static Vector ungraded = new Vector();
	public static Vector graded = new Vector();

	public static String ungradedName;
	public static String gradedName;

	/**
	 * Initializes ungraded list
	 *
	 * @param papers Papers that should be added to the ungraded vector
	 */
	public static void initializeUngraded(String name, Vector papers){
		ungraded = papers;
		ungradedName = name;
	}

	/**
	 * Initializes graded list
	 *
	 * @param papers Papers that should be added to the graded vector
	 */
	public static void initializeGraded(String name, Vector papers){
		graded = papers;
		gradedName = name;
	}

	/**
	 * Adds paper to ungraded list
	 *
	 * @param paper Paper that should be added to the ungraded vector
	 */
	public static void addUngradedPaper(Blaise_Paper paper){
		ungraded.addElement(paper);
	}

	/**
	 * Adds paper to graded list
	 *
	 * @param paper Paper that should be added to the graded vector
	 */
	public static void addGradedPaper(Blaise_Paper paper){
		graded.addElement(paper);
	}

	/**
	 * Removes a paper form the ungraded list
	 *
	 * @param i Index of paper that should be removed from ungraded vector
	 */
	public static void removeUngraded(int i){
		ungraded.removeElementAt(i);
	}

	/**
	 * Removes a paper form the graded list
	 *
	 * @param i Index of paper that should be removed from graded vector
	 */
	public static void removeGraded(int i){
		graded.removeElementAt(i);
	}

	/**
	 * Gets the ungraded list
	 *
	 * @return <code>Vector</code> Ungraded Vector
	 */
	public static Vector getUngraded(){
		return ungraded;
	}

	/**
	 * Gets the graded list
	 *
	 * @return <code>Vector</code> Graded Vector
	 */
	public static Vector getGraded(){
		return graded;
	}

	/**
	 * Gets the number of ungraded papers
	 *
	 * @return <code>int</code> Number of ungraded papers
	 */
	public static int getUngradedSize(){
		return ungraded.size();
	}

	/**
	 * Gets the number of graded papers
	 *
	 * @return <code>int</code> Number of graded papers
	 */
	public static int getGradedSize(){
		return graded.size();
	}

	/**
	 * Gets the ungraded paper at a specific location
	 *
	 * @param i Index of desired paper
	 * @return <code>Blaise_Paper</code> Specified paper
	 */
	public static Blaise_Paper getUngradedAt(int i){
		return (Blaise_Paper)ungraded.elementAt(i);
	}

	/**
	 * Gets the graded paper at a specific location
	 *
	 * @param i Index of desired paper
	 * @return <code>Blaise_Paper</code> Specified paper
	 */
	public static Blaise_Paper getGradedAt(int i){
		return (Blaise_Paper)graded.elementAt(i);
	}

	/**
	 * Clears all papers from the graded list
	 */
	public static void clearGraded(){
		graded = new Vector();
	}

	/**
	 * Returns the total number of papers on tablet
	 *
	 * @return <code>int</code> The number of papers on the tablet
	 */
	public static int totalPapers(){
		return(ungraded.size() + graded.size());
	}


	/**
	 * Removes a paper from the ungraded list
	 *
	 * @param paper Paper to be removed
	 */
	public static void removeUngraded(Blaise_Paper paper){
		for(int i=0;i<ungraded.size();i++){
			if(paper.equals((Blaise_Paper)ungraded.elementAt(i))){
				ungraded.removeElementAt(i);
			}
		}
	}

	/**
	 * Removes a paper from the graded list
	 *
	 * @param paper Paper to be removed
	 */
	public static boolean removeGraded(Blaise_Paper paper){
		for(int i=0;i<graded.size();i++){
			if(paper.equals((Blaise_Paper)graded.elementAt(i))){
				graded.removeElementAt(i);
				return true;
			}
		}

		return false;
	}

	public static void writeUngradedFile(){
		Blaise_XML xml = new Blaise_XML();
		xml.printPaperToFile(ungradedName, ungraded);
	}

	public static void writeGradedFile(){
		Blaise_XML xml = new Blaise_XML();
		xml.printPaperToFile(gradedName, graded);
	}

	public static void reloadTables(){

		Blaise_XML xml = new Blaise_XML();
		if(!ungradedName.equals("")){
			TabletPapers.initializeUngraded(ungradedName, xml.parse(ungradedName));
		}
		if(!gradedName.equals("")){
			TabletPapers.initializeGraded(gradedName, xml.parse(gradedName));
		}
	}
}
