//LOC:147
package edu.gwu.blaise.version2.recognizer;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * This is an extension of a JPanel that will display a given gesture at the proper depending on it's drawing location.
 *
 * @author Sean T. Hanlon
 */
public class Gesture_Panel extends JPanel{

	boolean debug = true;

	boolean isHalf;				//if being drawn on half of available space (as in when testing recognition)
	Vector gest;				//the gesture itself
	Rectangle2D.Double[] rects;	//the rectangles representing the grid

	double width;
	double height;

	/**
	 * Constructor
	 *
	 * @param gest Gesture to be displayed
	 * @param isHalf Flag for whether or not to draw gesture on half the available space
	 */
	public Gesture_Panel(Vector gest, boolean isHalf){
		this.gest = gest;
		this.rects = new Rectangle2D.Double[2];
		rects[0] = new Rectangle2D.Double(0,0,0,0);
		rects[1] = new Rectangle2D.Double(1,1,1,1);
		this.isHalf = isHalf;
		calculateBounds();
	}

	/**
	 * Constructor
	 *
	 * @param gest Gesture to be displayed
	 * @param rects Rectangles representing the grid
	 */
	public Gesture_Panel(Vector gest, Rectangle2D.Double[] rects){
		this.gest = gest;
		this.rects = rects;
		isHalf = false;
		calculateBounds();
	}

	/**
	 * Constructor
	 *
	 * @param gest Gesture to be displayed
	 * @param rects Rectangles representing the grid
	 * @param isHalf Flag for whether or not to draw gesture on half the available space
	 */
	public Gesture_Panel(Vector gest, Rectangle2D.Double[] rects, boolean isHalf){
		this(gest,rects);
		this.isHalf = isHalf;
	}

	//Mutator for isHalf
	public void setHalf(boolean in){
		isHalf = in;
	}

	//Calculates the width and height and size to draw gesture
	private void calculateBounds(){

		//If rects are not present, default to 1
		if(rects.length <= 0){
			width = 1.0;
			height = 1.0;
			return;
		}

		//Use top left and bottom right corners of rects
		Rectangle2D.Double top_left = rects[0];
		Rectangle2D.Double bottom_right = rects[rects.length-1];

		double min_x = top_left.getX();
		double min_y = top_left.getY();
		double max_x = bottom_right.getX() + bottom_right.getWidth();
		double max_y = bottom_right.getY() + bottom_right.getHeight();

		width = max_x - min_x;
		height = max_y - min_y;
	}

	/**
	 * Paints the component
	 *
	 * @param gr Graphics object of JPanel
	 */
	public void paintComponent(Graphics gr){
		super.paintComponent(gr);
		Graphics2D g = (Graphics2D)gr;

		Component comp = this.getParent();

		//determine scaling factor
		double scale = 1.0;
		double w = -1.0;
		double h = (double)(comp.getHeight()-50)/(double)height;
		if(isHalf){
			w = (double)((comp.getWidth()-20)/2)/(double)width;
		}
		else{
			w = (double)(comp.getWidth()-20)/(double)width;
		}

		if(w < h){
			scale = w;
		}
		else{
			scale = h;
		}

		//if debug, draw the grid
		if(debug){
			if(gest.size() > 0){
				for(int i=0;i<rects.length;i++){
					Rectangle2D.Double r = rects[i];
					g.draw(new Rectangle2D.Double(r.getX()*scale, r.getY()*scale, r.getWidth()*scale, r.getHeight()*scale));
				}
			}
		}
		g.setColor(Color.RED);

		for(int i=0;i<gest.size();i++){
			Line2D.Double temp = (Line2D.Double)gest.elementAt(i);
			Line2D.Double line = new Line2D.Double(temp.getX1()*scale, temp.getY1()*scale,temp.getX2()*scale, temp.getY2()*scale);
			g.draw(line);
		}

		if(gest.size() > 0){
			//mark starting point of gesture
			Line2D.Double l = (Line2D.Double)gest.elementAt(0);
			Line2D.Double line = new Line2D.Double(l.getX1()*scale, l.getY1()*scale,l.getX1()*scale, l.getY1()*scale);
			g.setStroke(new BasicStroke(7));
			g.draw(line);
		}
	}
}