//LOC:572
package edu.gwu.blaise.version2.recognizer;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import edu.gwu.blaise.version2.util.*;

/**
 * The Recognizer_GUI is used by a user to create, modify, and test gesture sets
 *
 * @author Sean T. Hanlon
 */
public class Recognizer_GUI extends JFrame implements ActionListener, ListSelectionListener{

	boolean debug = true;

	Recognizer rec;	//the "brain" instance

	Vector names;	//names of gestures
	Vector samples;	//samples of gestures
	Vector rects;	//rectangles of gestures

	JList list;				//list of gestures
	JScrollPane scroll;		//scrollpane containing the list
	int longest_name = 1;	//longest named gesture - used to size JList width

	JPanel content;					//everything
	Gesture_Panel currentGesture;	//currently selected gesture display
	JPanel buttonPanel;				//buttons
	JButton deleteButton;			//deletes the selected gesture from the set

	String filename;				//filename where the set is sae into and read from

	JMenuItem testSet;				//used to test set

	/**
	 * Constructor
	 */
	public Recognizer_GUI(){

		createMenu();

		this.setIconImage(new ImageIcon(Constants.getIconPath()+"icon.gif").getImage());

		//Initialize the data
		names = new Vector();
		samples = new Vector();
		rects = new Vector();
		rec = new Recognizer(new File("default"));
		filename = "default";

		//Initialize the Frame
		this.getContentPane().setLayout(new BorderLayout());

		content = new JPanel(new BorderLayout());
		deleteButton = new JButton("Delete");
		deleteButton.setEnabled(false);
		deleteButton.addActionListener(this);

		buttonPanel = new JPanel(new FlowLayout());
		buttonPanel.add(deleteButton);

		content.add(buttonPanel, BorderLayout.SOUTH);

		this.getContentPane().add(content, BorderLayout.CENTER);
		this.setTitle(filename);
		this.setSize(600,600);
	}

	/**
	 * Performs an action
	 *
	 * @param ae The action to be performed
	 */
    public void actionPerformed(ActionEvent ae){
        String command = ae.getActionCommand();

		if(command.equals("Exit")){
			//Prompt to sae changes
			if(names.size() > 0){
				int answer = JOptionPane.showConfirmDialog(this, "Would you like to save any changes made to this file?", "Discard Changes?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);

				//if yes --> answer == 0
				if(answer == 0){
					actionPerformed(new ActionEvent(" ", 0, "Save Gesture Set"));
				}

				if(answer != 2) { //CANCEL
					System.exit(0);
				}
			}
			else{
				System.exit(0);
			}
		}
		else if(command.equals("New Gesture Set")){
			//Prompt to save changes and then create a new set
			if(names.size() > 0){
				int answer = JOptionPane.showConfirmDialog(this, "Would you like to save any changes made to this file?", "Discard Changes?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);

				//if yes --> answer == 0
				if(answer == 0){
					actionPerformed(new ActionEvent(" ", 0, "Save Gesture Set"));
				}

				if(answer != 2) { //CANCEL
					filename = "default";
					rec.newSet();
					names = new Vector();
					samples = new Vector();
					rects = new Vector();
					updateList();
					this.setTitle(filename);
				}
			}
		}
		else if(command.equals("Open Existing Gesture Set")){
			//Prompt for a file to open and open it
			File f = getFileFromChooser(Constants.getSystemDirectory(), JFileChooser.OPEN_DIALOG);
			filename = f.getAbsolutePath();
			this.setTitle(filename);
			if(f != null && f.exists()){
				rec = new Recognizer(f);

				names = rec.getNames();
				samples = rec.getSamples();
				rects = rec.getRects();

				for(int i=0;i<names.size();i++){
					String current = (String)names.elementAt(i);
					if(current.length() > longest_name){
						longest_name = current.length();
					}
				}

				updateList();
			}
		}
		else if(command.equals("Save Gesture Set")){
			//Save the current gesture set
			if(names.size() > 0){
				File saveFile = null;

				//if nameOfFile is 'default', prompt for new name
				if(filename.equals("default")){
					saveFile = getFileFromChooser(Constants.getSystemDirectory(), JFileChooser.SAVE_DIALOG);
				}
				else{
					saveFile = new File(filename);
				}

				if(saveFile != null){
					if(!(saveFile.getName()).endsWith(".ges")){
						saveFile = new File(saveFile.getAbsolutePath()+".ges");
					}

					rec.writeToFile(saveFile);
					filename = saveFile.getAbsolutePath();
					this.setTitle(filename);
				}
			}
		}
		else if(command.equals("Save Gesture Set As")){
			filename = "default";
			actionPerformed(new ActionEvent(new Object(), 0, "Save Gesture Set"));
		}
		else if(command.equals("Add Gesture To Set")){

			//Prompt for new name
			String name = (String)JOptionPane.showInputDialog(this,"Please enter a name for the new gesture", "New Gesture Creation", JOptionPane.QUESTION_MESSAGE, null, null, "");
			if(name != null && !name.equals("")){
				name = name.trim();
				name = name.replace(' ','_');

				//Create gesture
				GestureCreator creator = new GestureCreator(this, ("Create "+name));
				if(creator.isDone()){
					Vector sample = creator.getGesture();

					if(rec.addGesture(name,sample)){
						names = rec.getNames();
						samples = rec.getSamples();
						rects = rec.getRects();

						if(name.length() > longest_name){
							longest_name = name.length();
						}

						updateList();
					}
					else{
						int answer = JOptionPane.showConfirmDialog(this, "Cannot Add Gesture To Gesture Set. Too Similar To Another Gesture", "Error", JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE);
					}
				}
			}

			this.setTitle(filename);
		}
		else if(command.equals("Test Recognition")){
			new testRecognizer();
		}
		else if(command.equals("Delete")){
			//Delete the selected gesture from the set
			rec.removeEntry((String)names.elementAt(list.getSelectedIndex()));
			names = rec.getNames();
			samples = rec.getSamples();
			rects = rec.getRects();

			updateList();
		}
	}

	//Called when the list changes
	public void valueChanged(ListSelectionEvent e){
		updateContent();
	}

	//Updates the list and display to reflect the currently selected item
	private void updateContent(){

		if(names.size() > 0){
			deleteButton.setEnabled(true);
			if(currentGesture != null){
				content.remove(currentGesture);
			}
			currentGesture = new Gesture_Panel((Vector)samples.elementAt(list.getSelectedIndex()), (Rectangle2D.Double[])rects.elementAt(list.getSelectedIndex()));
			content.add(currentGesture, BorderLayout.CENTER);
		}
		else{
			deleteButton.setEnabled(false);
			content.remove(currentGesture);
		}

		this.validate();
	}

	//Updates list
	private void updateList(){

		list = new JList(names);
		if(scroll != null){
			this.getContentPane().remove(scroll);
		}

		if(names.size() > 0){
			list.setSelectedIndex(0);
			(list.getSelectionModel()).setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			list.setCellRenderer(new listRenderer());
			list.addListSelectionListener(this);
			testSet.setEnabled(true);

			scroll = new JScrollPane(list);
			this.getContentPane().add(scroll, BorderLayout.WEST);
		}
		else{
			list.setSelectedIndex(-1);
			testSet.setEnabled(false);
		}

		updateContent();
		this.validate();
	}

	//Creates a menu for this application
	private void createMenu(){

		//File Menu
		JMenu jMenuFile = new JMenu("File");

		JMenuItem newSet = new JMenuItem("New Gesture Set");
		newSet.addActionListener(this);
		jMenuFile.add(newSet);

		JMenuItem open = new JMenuItem("Open Existing Gesture Set");
		open.addActionListener(this);
		jMenuFile.add(open);

		JMenuItem save = new JMenuItem("Save Gesture Set");
		save.addActionListener(this);
		jMenuFile.add(save);

		JMenuItem saveAs = new JMenuItem("Save Gesture Set As");
		saveAs.addActionListener(this);
		jMenuFile.add(saveAs);

		JMenuItem exit = new JMenuItem("Exit");
		exit.addActionListener(this);
		jMenuFile.add(exit);

		//Actions Menu
		JMenu jMenuActions = new JMenu("Actions");

		JMenuItem addGest = new JMenuItem("Add Gesture To Set");
		addGest.addActionListener(this);
		jMenuActions.add(addGest);

		testSet = new JMenuItem("Test Recognition");
		testSet.addActionListener(this);
		testSet.setEnabled(false);
		jMenuActions.add(testSet);

        JMenuBar jMenuBar = new JMenuBar();
        jMenuBar.add(jMenuFile);
        jMenuBar.add(jMenuActions);

        this.setJMenuBar(jMenuBar);
	}

    /** Method for obtaining a File for input/output using a JFileChooser dialog
      */
    private File getFileFromChooser(String startDir, int dialogType) {
        JFileChooser jfileDialog = new JFileChooser(startDir);
        jfileDialog.setDialogType(dialogType);
        int optionSelected = JFileChooser.CANCEL_OPTION;
        if(dialogType == JFileChooser.OPEN_DIALOG) {
			jfileDialog.setMultiSelectionEnabled(false);
			jfileDialog.addChoosableFileFilter(new fileFilter("ges","A Gesture Dictionary (*.ges)"));
            optionSelected = jfileDialog.showOpenDialog(this);
        }
        else if(dialogType == JFileChooser.SAVE_DIALOG) {
            jfileDialog.setMultiSelectionEnabled(false);
			jfileDialog.addChoosableFileFilter(new fileFilter("ges","A Gesture Dictionary (*.ges)"));
            optionSelected = jfileDialog.showSaveDialog(this);
        }
        else {// default to an OPEN_DIALOG
            optionSelected = jfileDialog.showOpenDialog(this);
        }
        if(optionSelected == JFileChooser.APPROVE_OPTION) {
            return jfileDialog.getSelectedFile();
        }
        return (File)null;
    }

	public static void main(String[] argv){
		new Recognizer_GUI();
	}

	//Renders the list
	class listRenderer extends JLabel implements ListCellRenderer{

		Color highlight = new Color(0,0,128);

		listRenderer(){
			setOpaque(true);
		}

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus){
			if(index == -1){
				int selected = list.getSelectedIndex();
				if(selected == -1){
					return this;
				}
				else{
					index = selected;
				}
			}

			//Create string of content
			StringBuffer buffer = new StringBuffer("  ");
			int diff = longest_name - ((String)names.elementAt(index)).length();
			buffer.append((String)names.elementAt(index));
			for(int i=0;i<diff;i++){
				buffer.append(" ");
			}
			buffer.append("    ");
			setText(buffer.toString());

			if(isSelected){
				setBackground(highlight);
				setForeground(Color.WHITE);
			}
			else{
				setBackground(Color.WHITE);
				setForeground(Color.BLACK);
			}

			return this;
		}
	}

	//Filters file-type when opening or saving a file
	class fileFilter extends javax.swing.filechooser.FileFilter{

		private String extension;
		private String description;

		public fileFilter(String ext){
			extension = ext;
		}

		public fileFilter(String ext, String desc){
			extension = ext;
			description = desc;
		}

		public boolean accept(File f){
			if(f.isDirectory()){
				return true;
			}

			String ext = f.getName().toLowerCase();
			if(ext.endsWith(extension)){
				return true;
			}
			else{
				return false;
			}
		}

		public String getDescription(){
			return description;
		}
	}

	//Used to test the recognition capabilities of the gesture set
	class testRecognizer extends JFrame implements ActionListener{

		JPanel main;			//contains the result and input
		Gesture_Panel result;	//contains the result
		JPanel query;			//where user enters test gesture

		//Components
		JButton clear;
		JButton done;
		JLabel matchedValue;
		JLabel inputValue;

		Vector gestu;
		int current_x = 0;
		int current_y = 0;

		testRecognizer(){
			super("Test Recognition");

			main = new JPanel(new GridLayout(1,2));

			//Setup main window
			JPanel buttons = new JPanel(new FlowLayout());
			clear = new JButton("Clear");
			done = new JButton("Done");
			clear.addActionListener(this);
			done.addActionListener(this);
			buttons.add(clear);
			buttons.add(done);

			JPanel labels = new JPanel(new GridLayout(1,2));
			matchedValue = new JLabel("");
			inputValue = new JLabel("");
			labels.add(matchedValue);
			labels.add(inputValue);

			this.getContentPane().add(labels, BorderLayout.NORTH);
			this.getContentPane().add(buttons, BorderLayout.SOUTH);

			init();

			this.setSize(500,400);
			this.setResizable(true);
			this.setVisible(true);
		}

		private void init(){
			//Set up panel to capture test gesture
			query = new JPanel();
			query.addMouseListener ( new MouseAdapter (){
				public void mousePressed (MouseEvent m)
				{
					query.requestFocus();
			        Graphics2D g = (Graphics2D)query.getGraphics();
					g.setColor(Color.BLACK);
					gestu = new Vector();
					int x = m.getX();
					int y = m.getY();
					current_x = x;
					current_y = y;
				}
			});

			query.addMouseMotionListener (new MouseMotionAdapter () {
			    public void mouseDragged (MouseEvent m)
			    {
			        int x = m.getX();
			        int y = m.getY();
			        Graphics2D g = (Graphics2D)query.getGraphics();

					Line2D.Double line = new Line2D.Double((double)current_x, (double)current_y, (double)x, (double)y);
					g.setColor(Color.BLACK);
					g.draw(line);
					gestu.add(line);

			        current_x = x;
			        current_y = y;
			    }
		   	});

			query.addMouseListener (new MouseAdapter () {
			    public void mouseReleased (MouseEvent m)
			    {
					match();
				}
		    });

			query.setBorder(BorderFactory.createLineBorder(Color.BLACK));
			query.setBackground(Color.WHITE);
			main.removeAll();
			main.add(query);
			main.validate();

			matchedValue.setText("  Please enter your query");
			inputValue.setText("");
			this.getContentPane().add(main, BorderLayout.CENTER);
			this.getContentPane().validate();
		}

	    public void actionPerformed(ActionEvent ae){
	        String command = ae.getActionCommand();

			if(command.equals("Clear")){
				main.removeAll();
				init();
			}
			else if(command.equals("Done")){
				this.dispose();
			}
		}

		private void match(){

			//Match input gesture, if not recognized, inform user
			String matched = rec.match(gestu);
			if(matched.equals("_++++_+_+_+__unknown_++++_+_+_+__")){
				result = new Gesture_Panel(new Vector(), true);
				matched = "UNKNOWN";
			}
			else{
				//Otherwise, display the matched gesture and the input gesture side-by-side
				Vector found = new Vector();
				Rectangle2D.Double[] rectan = new Rectangle2D.Double[0];
				for(int i=0;i<names.size();i++){
					if(((String)names.elementAt(i)).equals(matched)){
						found = (Vector)samples.elementAt(i);
						rectan = (Rectangle2D.Double[])(rects.elementAt(i));
						break;
					}
				}

				result = new Gesture_Panel(found,rectan,true);
			}

			Gesture_Panel input = Recognizer.getPanel(gestu);
			input.setHalf(true);

			main.removeAll();
			main.add(result);
			main.add(input);
			main.validate();

			matchedValue.setText("  Matched: "+matched);
			inputValue.setText("  The value you entered");
			this.getContentPane().add(main, BorderLayout.CENTER);
			this.getContentPane().validate();

			query.setForeground(Color.BLACK);
		}
	}
}
