package edu.gwu.blaise.version2.transmission;
import java.io.*;
import java.util.*;
/**
 * Performs encryption on data for transmission across the internet.
 * This is not military grade encryption, rather it is intended to just mask
 * the passwords, etc a little.
 * @author Michael Gaiman
 * @version 2 1/25/2004
 */
public class Encryption {
    static boolean useNums=true;            //set to true if want encrypted string to be the actual numbers, or false if characters are to be used
    static String k="MySixteenByteKey";     //can use a different key if need be, or have a function to set the key
    static byte[] eKey=k.getBytes();
    
    /**
     * Given a password this method will encrypt it, returning the encrypted form.
     */
    public static String encryptPassword(String password){   
        if(password==null){
            return null;
        }
        if(password.length()%2!=0){ 
            password+=' ';
        } // Make sure array is even in length.
        byte[] dataBytes=null;
        try{
            dataBytes=password.getBytes("UTF-8");
        }catch(UnsupportedEncodingException e){
            return null;
        }

        StringBuffer strByte=new StringBuffer();
        StringBuffer cipher=new StringBuffer();;
        int[] tempData=new int[2];
        for(int i=0; i<dataBytes.length; i+=2)
        {
            tempData[0]=dataBytes[i];
            tempData[1]=dataBytes[i+1];
            tempData=encode(tempData, eKey);
            cipher.append(intToString(tempData[0]));
            cipher.append(intToString(tempData[1]));
            strByte.append(tempData[0]).append("_");
            strByte.append(tempData[1]).append("_");
        }
        //strByte = strByte.substring(0, strByte.length()-1); 
        if(!useNums){
            return cipher.toString();
        }
        return strByte.toString();
    }
    
    /**
     * Given an encrypted password it will return the plaintext form.
     */
    public static String decryptPassword(String ePass) throws NoSuchElementException{
        if(ePass==null){
            return null;
        }
        StringTokenizer tok=new StringTokenizer(ePass, "_", false);
        int x=0;
        int[] tempData=new int[2];        
        byte[] dataBytes=new byte[ePass.length()/4];
        if(!useNums){
            for(int i=0; i<ePass.length(); i+=8){
                tempData[0]=stringToInt(ePass.substring(i, i+4));
                tempData[1]=stringToInt(ePass.substring(i+4, i+8));
                tempData=decode(tempData, eKey);
                dataBytes[x++]=(byte)tempData[0];
                dataBytes[x++]=(byte)tempData[1];
            }
        }
        else{
            String zero,one;
            while(tok.hasMoreElements()){
                zero=(String)tok.nextElement();
                one=(String)tok.nextElement();
                try{
                    tempData[0]=Integer.parseInt(zero);
                    tempData[1]=Integer.parseInt(one);
                }catch(NumberFormatException e){
                    return null;
                }
                    tempData=decode(tempData, eKey);
                    dataBytes[x++]=(byte)tempData[0];
                    dataBytes[x++]=(byte)tempData[1];
            }
        }
        String decipheredString=null;
        try{
            decipheredString=new String(dataBytes,0,dataBytes.length-1,"UTF-8");
        }catch(UnsupportedEncodingException e){
            return null;
        }

        return decipheredString.trim();
    }
    
    /**
     * Performs the byte level decoding
     */
    private static int[] decode (int[] data, byte[] key){
         int n=32;
         int sum;
         int y=data[0];
         int z=data[1];
         int delta=0x9e3779b9;

         sum=delta<<5 ;
         
         while(n-->0){   /* start cycle */
             z-=(y<<4)+key[2]^y+sum^(y>>5)+key[3]; 
             y-=(z<<4)+key[0]^z+sum^(z>>5)+key[1];
             sum-=delta ;  
         }                 /* end cycle */
         data[0]=y ;
         data[1]=z ; 
         return data;
    }
    
    /**
     * Performs the byte level encoding
     */
    private static int[] encode (int[] data, byte[] key){
        int y=data[0];  /* set up a key schedule constant*/
        int z=data[1];
        int sum=0;   
        int delta=0x9e3779b9;
        int n=32 ;            
        while(n-->0){                       /* basic cycle start */
            sum+=delta ;
            y+=(z<<4)+key[0]^z+sum^(z>>5)+key[1];
            z+=(y<<4)+key[2]^y+sum^(y>>5)+key[3];   /* end cycle */
        } 
        data[0]=y; 
        data[1]=z;   
        return data;
    }

    /**
     * Converts an integer to a String for use in decoding
     */
    private static String intToString(int Input){
        StringBuffer output=new StringBuffer();
        output.append((char)((Input & 0xFF)));
        output.append((char)((Input >> 8) & 0xFF));
        output.append((char)((Input >> 16) & 0xFF));
        output.append((char)((Input >> 24) & 0xFF));
        return output.toString();
    }

    /**
     * Converts a String to an integer, for use in encoding.  Only works with four character Strings.
     */
    private static int stringToInt(String Input){
        int output=((int)Input.charAt(0));
        output+=((int)Input.charAt(1) << 8);
        output+=((int)Input.charAt(2) << 16);
        output+=((int)Input.charAt(3) << 24);
        return output;
    }
    
    /* test program */
    public static void main(String arg[]){
      boolean decrypt=false;
      String encrStr;
      String decrStr="";
      if(arg.length==0){
        encrStr="quidditch";
        System.out.println("Encrypting '"+encrStr+"'");
        try{
          encrStr=Encryption.encryptPassword(encrStr);
        }catch(Exception e){
          System.out.println(e);
        }
        System.out.println("Encrypted password: '"+encrStr+"'");
        try{
          decrStr=Encryption.decryptPassword(encrStr);
        }catch(Exception e){
          System.out.println(e);
        }        
        System.out.println("Decrypted password: '"+decrStr+"'");
      }
      else if(arg.length>1&& arg[1].equalsIgnoreCase("encrypt")){
        encrStr=arg[0];
        try{
          encrStr=Encryption.encryptPassword(encrStr);
        }catch(Exception e){
          System.out.println(e);
        }
        System.out.println("Encrypted password: '"+encrStr+"'");        
      }
      else if(arg.length>1&&arg[1].equalsIgnoreCase("decrypt")){
        encrStr=arg[0];
        try{
          decrStr=Encryption.decryptPassword(encrStr);
        }catch(Exception e){
          System.out.println(e);
        }        
        System.out.println("Decrypted password: '"+decrStr+"'");        
      }
      else{
        encrStr=arg[0];
        try{
          encrStr=Encryption.encryptPassword(encrStr);
        }catch(Exception e){
          System.out.println(e);
        }
        System.out.println("Encrypted password: '"+encrStr+"'");
        try{
          decrStr=Encryption.decryptPassword(encrStr);
        }catch(Exception e){
          System.out.println(e);
        }        
        System.out.println("Decrypted password: '"+decrStr+"'");        
      }
    }
}
