package edu.gwu.blaise.version2.transmission;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import java.io.*;
/**
 * Performs encryption on data for transmission across the internet.
 * This is not military grade encryption, rather it is intended to just mask
 * the passwords, etc a little.
 * @author Michael Gaiman
 * @version 2 1/25/2004
 */
public class EncryptionWithCrypto {
    
    /**
     * Given a password it will encrypt it, returning the encrypted form.
     */
    public static byte[] encryptPassword(String password) throws Exception{
       // get the secret key spec and initialize the cipher
       SecretKeySpec skeySpec = getSKS();
       Cipher cipher = Cipher.getInstance("AES");
       cipher.init(Cipher.ENCRYPT_MODE, skeySpec);
       
       //actual encryption done here
       byte[] encrypted = cipher.doFinal(password.getBytes());

       //some debug stuff
       System.out.print("Byte stream: ");
       printBytes(password.getBytes());
       System.out.print("Encrypted byte stream: ");
       printBytes(encrypted);
       String str = new String(encrypted);
       
       return encrypted;
    }
    
    /**
     * Given an encrypted password it will return the plaintext form.
     */
    public static byte[] decryptPassword(byte[] ePass) throws Exception{
       //get the secret key spec and initialize the cipher
       SecretKeySpec skeySpec = getSKS();
       Cipher cipher = Cipher.getInstance("AES");
       cipher.init(Cipher.DECRYPT_MODE, skeySpec);

       //actual decryption done here
       System.out.print("epass: ");
       printBytes(ePass);
       byte[] original = cipher.doFinal(ePass);
       System.out.print("original: ");
       printBytes(original);
       return original;
    }

    /**
     *  Returns a secret key spec object for use by the cipher
     */
     private static SecretKeySpec getSKS() throws Exception{
       // Get the KeyGenerator
       KeyGenerator kgen = KeyGenerator.getInstance("AES");
       kgen.init(128); // 192 and 256 bits may not be available

       // Generate the secret key specs.
       SecretKey skey = kgen.generateKey();
       byte[] raw = skey.getEncoded();
       SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
       return skeySpec;
     }
     
    // tests the Encryption class 
    public static void main(String arg[]){
     String encrStr = "wrong";
     String decrStr = "not the password";
     byte[] encr = {0};
     byte[] decr = {0};
     try{
         encr = EncryptionWithCrypto.encryptPassword("Voip");
     }catch(Exception e){
         System.out.println(e);
     }
     //next two lines are for debug
     System.out.print("Encrypted byte stream (main method): ");
     printBytes(encr);
     try{
         decr = EncryptionWithCrypto.decryptPassword(encr);
     }catch(Exception e){
         System.out.println(e);
     }        
     //next two lines are for debug
     System.out.print("Decrypted byte stream: ");
     printBytes(decr);
    }
    
    /** 
     *Prints the bytes of a given byte array.  To be used for debugging purposes
     */
    private static void printBytes(byte[] b){
        for(int i = 0; i < b.length; i++){
            System.out.print(b[i] + " ");
        }
        System.out.println("");
    }
   }
