package edu.gwu.blaise.version2.transmission;
import java.util.*;
/**
 * Takes a query result String from a getXXXIDQuery and parses it into a form that can
 * be read easily.
 * @author Michael Gaiman
 * @version 2 1/25/2004
 */
public class IDResultReader extends ResultReader{
    private static boolean debug=false;
    Vector IDs;
    String currentID;
    String IDType;
    int index=0;


    public IDResultReader(String result){
        super(result);
    }

    /**
     * Parses the contents of the result into status, format, and body field.
     * If the format contains a row and column spec, then the body is parsed into table form.
     * @throws ParseResultException if the result is not in the proper form.
     */
    public void parse() throws ParseResultException{
        //parse the basic stuff first.
        super.parse();

        if(status==true){
            //get the IDType
            String[] fBits=format.split(TransmissionConstants.ID_SEPARATOR_RX);
            if(fBits[0].equals(format)){
                throw new ParseResultException("Invalid format field in result.");
            }
            IDType=fBits[0];

            if(debug)System.out.println("IDType:"+IDType+" from format:"+format);

            //get the IDs
            String[] ids=body.split(TransmissionConstants.ID_SEPARATOR_RX);
            int size=ids.length;
            IDs=new Vector();
            for(int i=0;i<size;i++){
                IDs.add(ids[i]);
            }

            if(debug)System.out.println("IDs:"+IDs+" from body:"+body);
        }
    }

    /**
     * If there is a next ID in the list, the current ID will be incremented to it.
     * The current ID is initially before the start of the list, so this method should be called at least
     * once, initially.
     * @return <code>true</code> if there is a next ID, <code>false</code> otherwise.
     */
    public boolean next(){
        //check for any empty body, if that's the case then there are no IDs
        if(body.equals("") || status==false){
            return false;
        }
        if((index+1)<=IDs.size()){
            currentID=(String)IDs.get(index);
            index++;
            return true;
        }
        else{
            return false;
        }
    }

    /**
     * If the result has been parsed, this will return a String with the ID type that was parsed.
     * (<code>null</code> if the result hasn't been parsed.)
     */
    public String getIDType(){
        return IDType;
    }

    /**
     * Retrieves the current ID in String form.
     * @see #next
     */
    public String getCurrentID(){
        return currentID;
    }

    /**
     * Retrieves the ID specified by the index.
     * @param i The index value.  Valid range from 0 (inclusive) to getNumberOfIDs() (exclusive).
     * @return The ID in String form or <code>null</code> if the index was invalid.
     */
    public String getID(int i){
        if(IDs==null || i>=IDs.size() || i<0){
            return null;
        }
        return (String)IDs.get(i);
    }

    /**
     * returns the number of IDs.
     */
    public int getNumberOfIDs(){
        if(IDs==null){
            return 0;
        }
        return IDs.size();
    }

    public static void main(String[] argv){
        String s="SUCCESS\npaperID,,,\n1,,,2,,,3,,,5";
        //String s="FAILURE\n\nYou Suck.";
        IDResultReader rr=new IDResultReader(s);
        try{
            rr.parse();
        }catch(ParseResultException e){
            System.out.println(e);
        }

        String type=rr.getIDType();
        while(rr.next()){
            System.out.println(type+":"+rr.getCurrentID());
        }

        int num=rr.getNumberOfIDs();
        System.out.println("number of IDs:"+num);
        for(int i=0;i<num;i++){
            System.out.println(type+"("+i+"):"+rr.getID(i));
        }
    }
}
