package edu.gwu.blaise.version2.transmission;
import java.util.*;

/**
 * This is the parent class of all the ResultReaders.  A specific ResultReader is
 * expected to be defined for each type of result (getID query, getQuery, etc).
 * @author Michael Gaiman
 * @version 2 1/25/2004
 */
public class ResultReader {
    static boolean debug=false;
    String result=null;
    boolean status;
    String format=null;
    String body=null;

    public ResultReader(String result){
        this.result=result;
    }

    /**
     * Performs the parsing of the result String
     */
    public void parse() throws ParseResultException{
        //pull out the success and then the format first.
        String[] bits=result.split(TransmissionConstants.BREAK_RX);

        if(bits[0].equals(TransmissionConstants.SUCCESS)){
            status=true;
        }
        else if(bits[0].equals(TransmissionConstants.FAILURE)){
            status=false;
        }
        else{
            throw new ParseResultException("Invalid status field in result.");
        }
        if(debug)System.out.println("status:"+status);

        if(bits.length<2){
            throw new ParseResultException("Invalid format field in result.");
        }

        format=bits[1];

        if(debug)System.out.println("format:"+format);

        //now pull the body from the result
        StringBuffer sb=new StringBuffer();
        for(int i=2;i<bits.length;i++){
            sb.append(bits[i]);
            if(i!=bits.length-1){
                sb.append(TransmissionConstants.BREAK);
            }
        }
        body=sb.toString();
        if(debug)System.out.println("body:"+body);
    }

    /**
     * Returns the status of the result.  This method only produces correct
     * returns after the {@link #parse} method is called.
     */
    public boolean getStatus(){
        return status;
    }

    /**
     * Returns the format of the result.  (<code>null</code> if called before {@link #parse})
     */
    public String getFormat(){
        return format;
    }

    /**
     * Returns the body of the result.  (<code>null</code> if called before {@link #parse})
     */
    public String getBody(){
        return body;
    }

    /**
     * Returns the original result String.
     */
    public String getResult(){
        return result;
    }

    public static void main(String[] argv){

        String s="FAILURE\n\nExacly one graderID required.";
        ResultReader rr=new ResultReader(s);
        try{
            rr.parse();
        }catch(ParseResultException e){
            System.out.println(e);
        }

        System.out.println("status:"+rr.getStatus());
        System.out.println("format:"+rr.getFormat());
        System.out.println("body:"+rr.getBody());
    }

}
