package edu.gwu.blaise.version2.transmission;
import java.util.*;
/**
 * Takes a query result String from a submitPaperQuery and parses it into a form that can
 * be read easily.  The response from a submitPaperQuery is semi-nonstandard,
 * it is easier to parse it specially.
 * @author Michael Gaiman
 * @version 2 1/25/2004
 */
public class SPResultReader extends ResultReader{
    private static boolean debug=false;
    Vector dIDs=null;
    String pID=null;
    String currentID=null;
    int index=0;

    public SPResultReader(String result){
        super(result);
    }

    /**
     * Parses the contents of the result into status, format, and body field.
     * If the format contains a row and column spec, then the body is parsed into table form.
     * @throws ParseResultException if the result is not in the proper form.
     */
    public void parse() throws ParseResultException{
        //parse the basic stuff first.
        super.parse();

        if(status==true){
            //split the format
            String[] fBits=format.split(TransmissionConstants.ID_SEPARATOR_RX);
            if(fBits[0].equals(format)){
                throw new ParseResultException("Invalid format field in result.");
            }

            //split the body
            String[] ids=body.split(TransmissionConstants.ID_SEPARATOR_RX);
            int startIndex=0;
            if(fBits[0].equals("paperID")){
                pID=ids[0];
                startIndex++;//start from the second position
                if(debug){System.out.println("PaperID found:"+pID);}
            }

            int size=ids.length;
            dIDs=new Vector();
            for(int i=startIndex;i<size;i++){
                if(!ids[i].equals("")){
                    dIDs.add(ids[i]);
                }
            }

            if(debug){
                System.out.println(
                "From format:"+format+" and body:"+body+" pID:"+pID+" and dIDs:"+dIDs+" were assembled.");
            }
        }

    }

    /**
     * if the paper that was submitted was new, then a paperID was included in the response.
     * this returns that paperID, or null if the paper was not new.
     */
    public String getPaperID(){
        return pID;
    }

    /**
     * If there is a next draftID in the list, the current draftID will be incremented to it.
     * The current draftID is initially before the start of the list, so this method should be called at least
     * once, initially.
     * @return <code>true</code> if there is a next draftID, <code>false</code> otherwise.
     */
    public boolean next(){
        //check for any empty body, if that's the case then there are no IDs
        if(body.equals("") || status==false){
            return false;
        }
        if((index+1)<=dIDs.size()){
            currentID=(String)dIDs.get(index);
            index++;
            return true;
        }
        else{
            return false;
        }
    }

    /**
     * Retrieves the current draftID in String form.
     * @see #next
     */
    public String getCurrentDraftID(){
        return currentID;
    }

    /**
     * Retrieves the draftID specified by the index.
     * @param i The index value.  Valid range from 0 (inclusive) to getNumberOfDraftIDs() (exclusive).
     * @return The ID in String form or <code>null</code> if the index was invalid.
     */
    public String getDraftID(int i){
        if(dIDs==null || i>=dIDs.size() || i<0){
            return null;
        }
        return (String)dIDs.get(i);
    }

    /**
     * returns the number of draftIDs.
     */
    public int getNumberOfDraftIDs(){
        if(dIDs==null){
            return 0;
        }
        return dIDs.size();
    }

    public static void main(String[] argv){
        String s="SUCCESS\npaperID,,,draftID,,,\n8,,,10,,,";
        //String s="SUCCESS\ndraftID,,,\n8,,,10,,,";
        //String s="SUCCESS\ndraftID,,,\n8,,,";
        //String s="FAILURE\n\nDon't worry, I don't think you're a FAILURE.";

        SPResultReader srr=new SPResultReader(s);
        try{
            srr.parse();
        }catch(ParseResultException e){
            System.out.println(e);
        }

        System.out.println("paperID:"+srr.getPaperID());
        while(srr.next()){
            System.out.println("draftID:"+srr.getCurrentDraftID());
        }

        int num=srr.getNumberOfDraftIDs();
        System.out.println("number of draftIDs:"+num);
        for(int i=0;i<num;i++){
            System.out.println("draftID ("+i+"):"+srr.getDraftID(i));
        }
    }
}
