package edu.gwu.blaise.version2.transmission;
import java.util.*;
import java.net.*;
import java.io.*;
import java.text.*;
import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;

/**
 * From TransmissionHandler you can get all of the transmission query objects.
 * TransmissionHandler holds all specific data needed to perform a transmission
 * with the backend.
 * @author Michael Gaiman
 * @author Jared Kiraly
 * @version 2 6/3/2004
 */
public class TransmissionHandler{
    static final boolean debug=true;
    static String ID=null;
    static String userName=null;
    static String ePass=null;
    static String universityID=null;

    /** Used with PaperSearch*/
    public static final int WRITER_TYPE=101;
    /** Used with PaperSearch*/
    public static final int GRADER_TYPE=202;

    /**
     * Added by mgaiman to support caching.
     */
    public static String getEPass(){
        return ePass;
    }
    
    /**
     * Added by mgaiman to support caching
     */
    public static String getUserName(){
        return userName;
    }
    
    /**
     * Added by mgaiman to support caching
     */
    public static String getUniversityID(){
        return universityID;
    }

    /**
     * Returns a GraderLoginQuery object that is ready to perform the query and return
     * results.  It performs a validateGrader query followed by getGrader, and getCourses.
     * It will perform a check to see if new similarity results are available.
     * @param user The username of the grader who is performing the query.
     * @param password (suprise) the grader's password.
     * @param uniID is the universityID of the grader.
     */
    public static GraderLoginQuery getGraderLogin(String user, String password, String uniID){
        ePass=Encryption.encryptPassword(password);
        userName=user;
        universityID=uniID;
        return new GraderLoginQuery(){
            private String message="";
            private Vector courses=null;
            private Vector pastCourses=null;
            private Vector currentCourses=null;
            private Vector assignments=null;
            private DraftStructure ds=null;
            private Grader grader=null;
            private String fullText=null;
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                //prepare the validateGraderLogin query first
                Properties p=new Properties();
                p.setProperty("command",TransmissionConstants.validateGraderLogin);
                TransmissionHandler.addDefaultGraderProperties(p);
                p.setProperty("userName",userName);
                p.setProperty("password",ePass);
                p.setProperty("universityID",universityID);
                //perform the query
                SubmitQuery sq=new SubmitQuery(p,graderAddress);
                String s=null;
                StringBuffer text=new StringBuffer();

                try{
                    s=sq.submit();
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During u/p validation.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During u/p validation.)").toString();
                    return false;
                }

                //parse it
                ResultReader rr=new ResultReader(s);

                try{
                    rr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message="Validation query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Validation query returned in an unknown format.  Please notify an administrator.").toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Validation query returned in an unknown format.");
                    return false;
                }

                boolean status=rr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=rr.getBody();
                    fullText=text.toString();
                    return false;
                }
                //if the status was true, then continue and get the grader's info.
                TransmissionHandler.ID=rr.getBody();

                p=new Properties();
                TransmissionHandler.addDefaultGraderProperties(p);
                p.setProperty("command",TransmissionConstants.getGrader);
                p.setProperty("graderID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                sq=new SubmitQuery(p,graderAddress);
                try{
                    s=sq.submit();
                    text.append("\n");
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During get grader info.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During get grader info.)")
                    .toString();
                    return false;
                }

                //parse the grader's info
                TableResultReader trr=new TableResultReader(s);
                try{
                    trr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Grader information query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Grader information query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Grader information query returned in an unknown format.");
                    return false;
                }

                status=trr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=trr.getBody();
                    fullText=text.toString();
                    return false;
                }
                //if the status was true, then get the Grader's info from the reader.
                trr.next();
                long gID=0L;
                Date cDate=null;
                int uID=0;
                String firstName=null;
                String lastName=null;
                String uName=null;
                String email=null;
                String stat=null;
                Date sDate=null;
                
//***********                
                while(!trr.getCell("userName").equals(TransmissionHandler.userName)){
					trr.next();
				}
                
                try{
                    gID=Long.parseLong(trr.getCell("graderID"));
                    SimpleDateFormat sdf=new SimpleDateFormat(TransmissionConstants.dateFormat);
                    cDate=sdf.parse(trr.getCell("creationDate"));
                    uID=Integer.parseInt(trr.getCell("universityID"));
                    firstName=trr.getCell("firstName");
                    lastName=trr.getCell("lastName");
                    uName=trr.getCell("userName");
                    email=trr.getCell("email");
                    stat=trr.getCell("status");
                    s=trr.getCell("similarityCheckedDate");
                    if(!s.equals("null")){
                        sDate=sdf.parse(s);
                    }

                }catch(NumberFormatException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Grader information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                    fullText=new ResultWriter(false,"",
                    "Grader information query returned in an unknown format.  Please notify an administrator. (Number formats)")
                    .toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Grader information query returned in an unknown number format.");
                    return false;
                }catch(ParseException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Grader information query returned in an unknown format.  Please notify an administrator. (Date formats)";
                    fullText=new ResultWriter(false,"",
                    "Grader information query returned in an unknown format.  Please notify an administrator. (Date formats)")
                    .toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Grader information query returned in an unknown date format.");
                    return false;
                }
                grader=new Grader(gID,cDate,uID,firstName,lastName,uName,email,stat,sDate);

                //finally, go and get all the courses that the grader grades in
                //and put it into a Vector of Course objects.

                //(we do this by first getting courseIDs and then getCourse)
                p=new Properties();
                TransmissionHandler.addDefaultGraderProperties(p);
                p.setProperty("command",TransmissionConstants.getCourseID);
                p.setProperty("graderID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                sq=new SubmitQuery(p,graderAddress);
                try{
                    s=sq.submit();
                    text.append("\n");
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During course ID lookup.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During course ID lookup.)")
                    .toString();
                    return false;
                }

                IDResultReader IDrr=new IDResultReader(s);
                try{
                    IDrr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Course ID query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Course ID query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Course ID query returned in an unknown format.");
                    return false;
                }

                status=IDrr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=IDrr.getBody();
                    fullText=text.toString();
                    return false;
                }
                p=new Properties();
                TransmissionHandler.addDefaultGraderProperties(p);
                p.setProperty("command",TransmissionConstants.getCourse);
                p.setProperty("graderID",TransmissionHandler.ID);

                p.setProperty("courseID",IDrr.getBody());//formatted all nicely already.

                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                sq=new SubmitQuery(p,graderAddress);
                try{
                    text.append("\n");
                    s=sq.submit();
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During course lookup.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During course lookup.)").toString();
                    return false;
                }


                trr=new TableResultReader(s);
                try{
                    trr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Course information query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Course information query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Course information query returned in an unknown format.");
                    return false;
                }
                status=trr.getStatus();
                if(status==false){
                    message=trr.getBody();
                    fullText=text.toString();
                    return false;
                }

                try{
                    courses=TransmissionHandler.getCourses(trr,TransmissionHandler.GRADER_TYPE);
                }catch(NumberFormatException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Course information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                    fullText=new ResultWriter(false,"",
                    "Course information query returned in an unknown format.  Please notify an administrator. (Number formats)").toString();
                    sendMessageToDebugFile("Error in getGraderLogin", 
                                           "Course information query returned an unknown number format.");
                    return false;
                }
                
                //now put the assignments into the courses
                pastCourses=new Vector();
                currentCourses=new Vector();
                for(int i=0;i<courses.size();i++){
                    Course c=(Course)courses.elementAt(i);
                    if(c.getIsCurrent().equals("Y")){
                        currentCourses.add(c);
                    }
                    else if(c.getIsCurrent().equals("N")){
                        pastCourses.add(c);
                    }
                    p=new Properties();
                    //first get the assignIDs
                    TransmissionHandler.addDefaultGraderProperties(p);
                    p.setProperty("courseID",Long.toString(c.getCourseID()));
                    p.setProperty("command",TransmissionConstants.getAssignID);
                    p.setProperty("graderID",TransmissionHandler.ID); 
                    p.setProperty("myUserName",TransmissionHandler.userName);
                    p.setProperty("myEPass",TransmissionHandler.ePass);
                    p.setProperty("myUniversityID",TransmissionHandler.universityID);
                    sq=new SubmitQuery(p,graderAddress);
                    try{
                        s=sq.submit();
                        text.append("\n");
                        text.append(s);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message="An error occurred when trying to connect to the server. (During assignment ID lookup.)";
                        fullText=new
                        ResultWriter(false,"",
                        "An error occurred when trying to connect to the server. (During assignment ID lookup.)")
                        .toString();
                        return false;
                    }
    
                    IDrr=new IDResultReader(s);
                    try{
                        IDrr.parse();
                    }catch(ParseResultException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Assignment ID query returned in an unknown format.  Please notify an administrator.";
                        fullText=new ResultWriter(false,"",
                        "Assignment ID query returned in an unknown format.  Please notify an administrator.")
                        .toString();
                        sendMessageToDebugFile("Error in getGraderLogin", 
                                               "Assignment ID query returned in an unknown format.");
                        return false;
                    }
    
                    status=IDrr.getStatus();
                    //if the status was false, then don't continue.
                    if(status==false){
                        message=IDrr.getBody();
                        fullText=text.toString();
                        return false;
                    }
                    p=new Properties();
                    TransmissionHandler.addDefaultGraderProperties(p);
                    p.setProperty("command",TransmissionConstants.getAssignment);
                    p.setProperty("graderID",TransmissionHandler.ID);    
                    p.setProperty("assignID",IDrr.getBody());//formatted all nicely already.    
                    p.setProperty("myUserName",TransmissionHandler.userName);
                    p.setProperty("myEPass",TransmissionHandler.ePass);
                    p.setProperty("myUniversityID",TransmissionHandler.universityID);

                    sq=new SubmitQuery(p,graderAddress);
                    try{
                        text.append("\n");
                        s=sq.submit();
                        text.append(s);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message="An error occurred when trying to connect to the server. (During assignment lookup.)";
                        fullText=new
                        ResultWriter(false,"",
                        "An error occurred when trying to connect to the server. (During assignment lookup.)").toString();
                        sendMessageToDebugFile("Error in getGraderLogin", 
                                               message);
                        return false;
                    }


                    trr=new TableResultReader(s);
                    try{
                        trr.parse();
                    }catch(ParseResultException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Assignment information query returned in an unknown format.  Please notify an administrator.";
                        fullText=new ResultWriter(false,"",
                        "Assignment information query returned in an unknown format.  Please notify an administrator.")
                        .toString();
                        sendMessageToDebugFile("Error in getGraderLogin", 
                                               "Assignment information query returned in an unknown format.");
                        return false;
                    }
                    status=trr.getStatus();
                    if(status==false){
                        message=trr.getBody();
                        fullText=text.toString();
                        return false;
                    }

                    try{
                        assignments=TransmissionHandler.getAssignments(trr,TransmissionHandler.GRADER_TYPE);
                    }catch(NumberFormatException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Assignment information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                        fullText=new ResultWriter(false,"",
                        "Assignment information query returned in an unknown format.  Please notify an administrator. (Number formats)").toString();
                        sendMessageToDebugFile("Error in getGraderLogin", 
                                               "Assignment information query returned in an unknown number format.");
                        return false;
                    }
                    //now get the draftStructures for every assignment
                    for(int j=0;j<assignments.size();j++){
                        Assignment a=(Assignment)assignments.elementAt(j);
                        p=new Properties();
                        //first get the draftStructureIDs
                        TransmissionHandler.addDefaultGraderProperties(p);
                        p.setProperty("assignID",Long.toString(a.getAssignID()));
                        p.setProperty("command",TransmissionConstants.getDraftStructureID);
                        p.setProperty("graderID",TransmissionHandler.ID); 
                        p.setProperty("myUserName",TransmissionHandler.userName);
                        p.setProperty("myEPass",TransmissionHandler.ePass);
                        p.setProperty("myUniversityID",TransmissionHandler.universityID);
                        sq=new SubmitQuery(p,graderAddress);
                        try{
                            s=sq.submit();
                            text.append("\n");
                            text.append(s);
                        }catch(IOException e){
                            if(debug){System.out.println(e);}
                            message="An error occurred when trying to connect to the server. (During draft structure ID lookup.)";
                            fullText=new
                            ResultWriter(false,"",
                            "An error occurred when trying to connect to the server. (During draft structure ID lookup.)")
                            .toString();
                            return false;
                        }
        
                        IDrr=new IDResultReader(s);
                        try{
                            IDrr.parse();
                        }catch(ParseResultException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Draft Structure ID query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Draft Structure ID query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getGraderLogin", 
                                                   "Draft Structure ID query returned in unknown format.");
                            return false;
                        }
        
                        status=IDrr.getStatus();
                        //if the status was false, then don't continue.
                        if(status==false){
                            message=IDrr.getBody();
                            fullText=text.toString();
                            return false;
                        }
                        //now get the draft structures
                        p=new Properties();
                        Vector vds=null;
                        TransmissionHandler.addDefaultGraderProperties(p);
                        p.setProperty("command",TransmissionConstants.getDraftStructure);
                        p.setProperty("graderID",TransmissionHandler.ID);    
                        p.setProperty("draftStructureID",IDrr.getBody());//formatted all nicely already.    
                        p.setProperty("myUserName",TransmissionHandler.userName);
                        p.setProperty("myEPass",TransmissionHandler.ePass);
                        p.setProperty("myUniversityID",TransmissionHandler.universityID);

                        sq=new SubmitQuery(p,graderAddress);
                        try{
                            text.append("\n");
                            s=sq.submit();
                            text.append(s);
                        }catch(IOException e){
                            if(debug){System.out.println(e);}
                            message="An error occurred when trying to connect to the server. (During draft structure lookup.)";
                            fullText=new
                            ResultWriter(false,"",
                            "An error occurred when trying to connect to the server. (During draft structure lookup.)").toString();
                            return false;
                        }


                        trr=new TableResultReader(s);
                        try{
                            trr.parse();
                        }catch(ParseResultException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Draft structure information query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Draft structure information query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getGraderLogin", 
                                                   "Draft structure information query returned in unknown format.");
                            return false;
                        }
                        status=trr.getStatus();
                        if(status==false){
                            message=trr.getBody();
                            fullText=text.toString();
                            return false;
                        }
    
                        try{
                            vds=TransmissionHandler.getDraftStructure(trr,TransmissionHandler.GRADER_TYPE);
                        }catch(NumberFormatException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Draft Structure information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                            fullText=new ResultWriter(false,"",
                            "Draft Structure information query returned in an unknown format.  Please notify an administrator. (Number formats)").toString();
                            sendMessageToDebugFile("Error in getGraderLogin", 
                                                   "Draft Structure information query returned in an unknown number format.");
                            return false;
                        }
                        a.setDraftStructure(vds);
                        c.addAssignment(a);
                    }
                }
                
                //finally...we're done.  It has been a long ride, and I'd like to thank you.
                fullText=text.toString();
                return status;
            }
            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
            //only returns current courses
            public Vector getCourses(){
                return currentCourses;
            }
            //only returns past courses
            public Vector getPastCourses(){
                return pastCourses;
            }
            //returns all courses
            public Vector getAllCourses(){
                return courses;
            }
            public Grader getGrader(){
                return grader;
            }
        };
    }

    /**
     * Grader version of getPaperSearch.
     */
    public static PaperSearchQuery getPaperSearch(final boolean ungradedOnly, final String lastName, final String studentIdent, final String email, final Course course){
        return getPaperSearch(ungradedOnly,lastName,studentIdent,email,course,GRADER_TYPE,null,null,null);
    }
    /**
     * Returns a PaperSearchQuery object that performs searches to find papers that match the
     * parameters.  Only paper headers are returned by this query.  To get a full paper, the Grader
     * must check it out via {@link #getCheckOutPapers}
     * @param ungradedOnly if <code>true</code> the results are limited to only ungraded papers.
     * @param lastName an optional search parameter.  If unneeded, set to <code>null</code>.
     * @param studentIdent an optional search parameter.  If unneeded, set to <code>null</code>.
     * @param email an optional search parameter.  If unneeded, set to <code>null</code>.
     * @param course an optional search parameter.  If unneeded, set to <code>null</code>.
     * @param type {@link #WRITER_TYPE} or {@link #GRADER_TYPE}
     * @param wagID the rowIDs from writerAsGrader, provided so the writer can checkout papers
     * @param aID an optional search parameter to search by assignments.  If unneeded, set to <code>null</code>
     * @param completed an optional search parameter used with peer grading searches, telling if said peer grading has been done
     */
    public static PaperSearchQuery getPaperSearch(final boolean ungradedOnly, final String lastName, final String studentIdent, final String email, final Course course, final int type, final Vector wagID, final Vector aID,final String completed){
        if(type!=WRITER_TYPE && type!=GRADER_TYPE){
            return null;
        }
        return new PaperSearchQuery(){
            private String message="";
            private String fullText=null;
            private Vector papers=null;
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                String complete=completed;
                //perform getPaperID query first
                Properties p=new Properties();
                if(type==WRITER_TYPE){
                    TransmissionHandler.addDefaultWriterProperties(p);
                }
                else if(type==GRADER_TYPE){
                    TransmissionHandler.addDefaultGraderProperties(p);
                }
                if(ungradedOnly){
                    p.setProperty("command",TransmissionConstants.getPaperUngradedID);
                }else{
                    p.setProperty("command",TransmissionConstants.getPaperID);
                }
                if(type==GRADER_TYPE){
                    p.setProperty("graderID",TransmissionHandler.ID);
                }
                else if(type==WRITER_TYPE){
                    p.setProperty("writerID",TransmissionHandler.ID);
                }
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                StringBuffer pID=new StringBuffer();
                if(wagID!=null){
                    for(int i=0;i<wagID.size();i++){
                        pID.append(wagID.get(i));
                        if(i!=wagID.size()-1){//not last one
                            pID.append(TransmissionConstants.ID_SEPARATOR);
                        }
                    }
                    p.setProperty("writerAsGraderID",pID.toString());
                }
                if(lastName!=null){
                    p.setProperty("lastName",lastName);
                }
                if(studentIdent!=null){
                    p.setProperty("studentIdent",studentIdent);
                }
                if(email!=null){
                    p.setProperty("email",email);
                }
                if(course!=null){
                    p.setProperty("courseID",Long.toString(course.getCourseID()));
                }
                //if the search is for ungraded papers, then completed must be false.  otherwise, it can be specified here
                if(ungradedOnly){
                    complete="N";
                }
                if(completed!=null){
                    p.setProperty("completed",complete);
                }
                pID=new StringBuffer();
                if(aID!=null){
                    for(int i=0;i<aID.size();i++){
                        pID.append(aID.get(i));
                        if(i!=aID.size()-1){//not last one
                            pID.append(TransmissionConstants.ID_SEPARATOR);
                        }
                    }
                    p.setProperty("assignID",pID.toString());
                }


                SubmitQuery sq=null;
                if(type==GRADER_TYPE){
                    sq=new SubmitQuery(p,graderAddress);
                }
                else if(type==WRITER_TYPE){
                    sq=new SubmitQuery(p,writerAddress);
                }
                StringBuffer text=new StringBuffer();
                String s;
                try{
                    s=sq.submit();
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During get paper IDs.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During get paper IDs.)")
                    .toString();
                    return false;
                }

                boolean status;
                IDResultReader IDrr=new IDResultReader(s);
                try{
                    IDrr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Paper ID query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Paper ID query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getPaperSearch", 
                                           "Paper ID query returned in unknown format.");
                    return false;
                }
                status=IDrr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=IDrr.getBody();
                    fullText=text.toString();
                    return false;
                }

                String paperIDString=IDrr.getBody();
                if(paperIDString.equals("")){
                    //there were no paperIDs that matched the requirements.
                    fullText=text.toString();
                    papers=new Vector();//empty
                    return true;
                }
                //next perform getPaperHeader query
                p=new Properties();
                if(type==WRITER_TYPE){
                    TransmissionHandler.addDefaultWriterProperties(p);
                }
                else if(type==GRADER_TYPE){
                    TransmissionHandler.addDefaultGraderProperties(p);
                }
                p.setProperty("command",TransmissionConstants.getPaperHeader);
                if(type==GRADER_TYPE){
                    p.setProperty("graderID",TransmissionHandler.ID);
                }
                else if(type==WRITER_TYPE){
                    p.setProperty("writerID",TransmissionHandler.ID);
                }
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                p.setProperty("paperID",IDrr.getBody());
                byte[] sbq=null;
                
                if(type==GRADER_TYPE){
                    sq=new SubmitQuery(p,graderAddress);
                }
                else if(type==WRITER_TYPE){
                    sq=new SubmitQuery(p,writerAddress);
                }
                try{
                    sbq=sq.submitBinary(null);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During get paper header.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During get paper header.)")
                    .toString();
                    return false;
                }
                //this call to SubmitQuery will only return a ResultWriter if something has gone wrong
                byte[] pro = DualByteArray.extractFirst(sbq);
                Properties props = DualByteArray.toProperties(pro);
                if(!props.getProperty("status").equals(TransmissionConstants.SUCCESS)){
                    s=new String(props.getProperty("message"));
                    ResultReader rr=new ResultReader(s);
                    try{
                        rr.parse();
                    }catch(ParseResultException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Paper Header query returned in an unknown format.  Please notify an administrator.";
                        fullText=new ResultWriter(false,"",
                        "Paper Header query returned in an unknown format.  Please notify an administrator.")
                        .toString();
                        sendMessageToDebugFile("Error in getPaperSearch", 
                                               "Paper Header query returned in an unknown format.");
                        return false;
                    }
                    message=rr.getBody();
                    fullText=text.toString();
                    return false;
                }
                
                Blaise_Paper pap=null;
                byte[] pape=null;
                int numEntries=DualByteArray.getNumEntries(sbq);
                papers=new Vector();
                for(int i=1;i<=numEntries;i++){
                    pape=DualByteArray.extractNth(sbq, i+1);
                    try{
                        pap=BlaiseFile.byteArrayToPaper(pape);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Paper Header query returned in an unknown format.  Please notify an administrator.";
                        fullText=new ResultWriter(false,"",
                        "Paper Header query returned in an unknown format.  Please notify an administrator.")
                        .toString();
                        sendMessageToDebugFile("Error in getPaperSearch", 
                                               "Paper Header query returned in an unknown format.");
                        return false;                        
                    }
                    papers.add(pap);
                }
                return true;
            }
            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
            public Vector getPapers(){
                return papers;
            }
        };
    }

    /**
     * Returns a CheckOutPapersQuery object that performs checkouts of papers that will be graded.
     * @param paperIDs The papers the grader would like to checkout to grade. (Vector of Longs).
     */
    public static CheckOutPapersQuery getCheckOutPapers(final Vector paperIDs,final int type){
        return new CheckOutPapersQuery(){
            String fullText=null;
            String message="";
            Vector papers=null;
            Long ID=null;
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                //perform checkOutPaper query for every paper ID in the vector
                papers=new Vector();
                for(int j=0;j<paperIDs.size();j++){
                    Properties p=new Properties();
                    if(type==GRADER_TYPE){
                        TransmissionHandler.addDefaultGraderProperties(p);
                    }
                    else if(type==WRITER_TYPE){
                        TransmissionHandler.addDefaultWriterProperties(p);
                    }
                    else{
                        message=
                        "An invalid type was sent in. (During check out papers: paper"+j+" out of "+paperIDs.size()+")";
                        fullText=new
                        ResultWriter(false,"",
                        "An invalid type was sent in. (During check out papers: paper"+j+" out of "+paperIDs.size()+")")
                        .toString();
                        sendMessageToDebugFile("Error in getCheckoutPapers", 
                                               message);
                        return false;                         
                    }
                    p.setProperty("command",TransmissionConstants.checkOutPaper);
                    if(type==GRADER_TYPE){
                        p.setProperty("graderID",TransmissionHandler.ID);
                    }
                    else if(type==WRITER_TYPE){
                        p.setProperty("writerID",TransmissionHandler.ID);
                    }
                    p.setProperty("myUserName",TransmissionHandler.userName);
                    p.setProperty("myEPass",TransmissionHandler.ePass);
                    p.setProperty("myUniversityID",TransmissionHandler.universityID);
                    /*  This block of code used in the event of getting multiple papers with one query
                        For the time being, we get one paper at a time.
                    StringBuffer pID=new StringBuffer();
                    for(int i=0;i<paperIDs.size();i++){
                        pID.append(paperIDs.get(i));
                        if(i!=paperIDs.size()-1){//not last one
                            pID.append(TransmissionConstants.ID_SEPARATOR);
                        }
                    }   */
                    ID=(Long)paperIDs.get(j);
                    p.setProperty("paperID",ID.toString());
                    SubmitQuery sq=null;
                    if(type==GRADER_TYPE){
                        sq=new SubmitQuery(p,graderAddress);
                    }
                    else if(type==WRITER_TYPE){
                        sq=new SubmitQuery(p,writerAddress);
                    }
                    byte[] sbq=null;
                    try{
                        sbq=sq.submitBinary(null);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message=
                        "An error occurred when trying to connect to the server. (During check out papers: paper"+j+" out of "+paperIDs.size()+")";
                        fullText=new
                        ResultWriter(false,"",
                        "An error occurred when trying to connect to the server. (During check out papers: paper"+j+" out of "+paperIDs.size()+")")
                        .toString();
                        return false;
                    }
    
                    //this call to SubmitQuery will only return a ResultWriter if something has gone wrong
                    byte[] pro = DualByteArray.extractFirst(sbq);
                    Properties props = DualByteArray.toProperties(pro);
                    if(!props.getProperty("status").equals(TransmissionConstants.SUCCESS)){
                        String s=props.getProperty("message");
                        ResultReader rr=new ResultReader(s);
                        try{
                            rr.parse();
                        }catch(ParseResultException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Check out paper query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Check out paper query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getCheckoutPapers", 
                                                   "Check out paper query returned in an unknown format.");
                            return false;
                        }
                        message=rr.getBody();
                        //full text gotten above
                        return false;
                    }
    
                    Blaise_Paper pap=null;
                    byte[] pape=null;
                    int numEntries=DualByteArray.getNumEntries(sbq);
                    for(int i=1;i<=numEntries;i++){
                        pape=DualByteArray.extractNth(sbq, i+1);
                        try{
                            pap=BlaiseFile.byteArrayToPaper(pape);
                        }catch(IOException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Paper Header query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Paper Header query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getCheckoutPapers", 
                                                   "Paper Header query returned in an unknown format.");
                            return false;                        
                        }
                        papers.add(pap);
                    }
                }
                return true;
            }
            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
            public Vector getPapers(){
                return papers;
            }
        };
    }

    /**
     * Returns a CheckInPapersQuery object that performs a comment (grade) checkin.
     * @param paper a Blaise_Paper object to submit.
     */
    public static TransmissionQuery getCheckInPapers(final Blaise_Paper paper,final int type){
        return new TransmissionQuery(){
            String fullText=null;
            String message="";
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                //perform checkInPaper query
                Properties p=new Properties();
                if(type==TransmissionHandler.GRADER_TYPE){
                    TransmissionHandler.addDefaultGraderProperties(p);
                }
                else if(type==TransmissionHandler.WRITER_TYPE){
                    TransmissionHandler.addDefaultWriterProperties(p);
                }
                else{
                    message=
                    "An invalid type was sent in. (During check in paper)";
                    fullText=new
                    ResultWriter(false,"",
                    "An invalid type was sent in. (During check in paper)")
                    .toString();
                    return false;                         
                }
                p.setProperty("command",TransmissionConstants.checkInPaper);
                if(type==GRADER_TYPE){
                    p.setProperty("graderID",TransmissionHandler.ID);
                }
                else if(type==WRITER_TYPE){
                    p.setProperty("writerID",TransmissionHandler.ID);
                }
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                //currently it sends the whole paper.  To optimize this, only the changed/new comments by
                //this grader should be sent.

                SubmitQuery sq=null;
                if(type==TransmissionHandler.GRADER_TYPE){
                    sq=new SubmitQuery(p,graderAddress);
                }
                else if(type==TransmissionHandler.WRITER_TYPE){
                    sq=new SubmitQuery(p,writerAddress);
                }

                byte[] sbq=null;
                try{
                    sbq=sq.submitBinary(paper);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During check in papers.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During check in papers.)")
                    .toString();
                    return false;
                }

                boolean status;
                byte[] b=DualByteArray.extractFirst(sbq);
                p=DualByteArray.toProperties(b);
                String s=p.getProperty("message");
                ResultReader rr=new ResultReader(s);
                try{
                    rr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Check in paper query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Check in paper query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getCheckinPapers", 
                                           "Check in paper query returned in an unknown format.");
                    return false;
                }
                status=rr.getStatus();
                message=rr.getBody();

                return status;
            }


            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
        };
    }

    /**
     * Returns a WriterLoginQuery object that is ready to perform the query and return
     * results.  It performs a validateWriter query followed by getWriter, and getCourses.
     * @param user The username of the writer who is performing the query.
     * @param password (suprise) the writer's password.
     * @param uniID is the universityID of the writer.
     */
    public static WriterLoginQuery getWriterLogin(String user, String password, String uniID){
        ePass=Encryption.encryptPassword(password);
        userName=user;
        universityID=uniID;
        return new WriterLoginQuery(){
            private String message="";
            private Vector courses=null;
            private Vector pastCourses=null;
            private Vector currentCourses=null;
            private Writer writer=null;
            private String fullText=null;
            private Vector wags=null;
            private Vector assignments=null;
            private DraftStructure ds=null;
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                //prepare the validateWriterLogin query first
                Properties p=new Properties();
                p.setProperty("command",TransmissionConstants.validateWriterLogin);
                TransmissionHandler.addDefaultWriterProperties(p);
                p.setProperty("userName",userName);
                p.setProperty("password",ePass);
                p.setProperty("universityID",universityID);
                //perform the query
                SubmitQuery sq=new SubmitQuery(p,writerAddress);
                String s=null;
                StringBuffer text=new StringBuffer();

                try{
                    s=sq.submit();
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During u/p validation.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During u/p validation.)").toString();
                    return false;
                }

                //parse it
                ResultReader rr=new ResultReader(s);

                try{
                    rr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message="Validation query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Validation query returned in an unknown format.  Please notify an administrator.").toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Validation query returned in an unknown format.");
                    return false;
                }

                boolean status=rr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=rr.getBody();
                    fullText=text.toString();
                    return false;
                }
                //if the status was true, then continue and get the writer's info.
                TransmissionHandler.ID=rr.getBody();

                p=new Properties();
                TransmissionHandler.addDefaultWriterProperties(p);
                p.setProperty("command",TransmissionConstants.getWriter);
                p.setProperty("writerID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                sq=new SubmitQuery(p,writerAddress);
                try{
                    s=sq.submit();
                    text.append("\n");
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During get writer info.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During get writer info.)")
                    .toString();
                    return false;
                }

                //parse the writer's info
                TableResultReader trr=new TableResultReader(s);
                try{
                    trr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Writer information query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Writer information query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Writer information query returned in unknown format.");
                    return false;
                }

                status=trr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=trr.getBody();
                    fullText=text.toString();
                    return false;
                }
                //if the status was true, then get the Writer's info from the reader.
                trr.next();
                long wID=0L;
                int uID=0;
                String studentIdent=null;
                String firstName=null;
                String lastName=null;
                String uName=null;
                String email=null;
                String uNameType=null;
                Date cDate=null;
                
//******                
				while(!trr.getCell("userName").equals(TransmissionHandler.userName)){
					trr.next();
				}
                
                try{
                    wID=Long.parseLong(trr.getCell("writerID"));
                    uID=Integer.parseInt(trr.getCell("universityID"));
                    SimpleDateFormat sdf=new SimpleDateFormat(TransmissionConstants.dateFormat);
                    cDate=sdf.parse(trr.getCell("creationDate"));
                    firstName=trr.getCell("firstName");
                    lastName=trr.getCell("lastName");
                    uName=trr.getCell("userName");
                    email=trr.getCell("email");
                    studentIdent=trr.getCell("studentIdentifier");
                    uNameType=trr.getCell("userNameType");
                }catch(NumberFormatException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Writer information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                    fullText=new ResultWriter(false,"",
                    "Writer information query returned in an unknown format.  Please notify an administrator. (Number formats)")
                    .toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Writer information query returned in unknown number format.");
                    return false;
                }catch(ParseException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Writer information query returned in an unknown format.  Please notify an administrator. (Date formats)";
                    fullText=new ResultWriter(false,"",
                    "Writer information query returned in an unknown format.  Please notify an administrator. (Date formats)")
                    .toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Writer information query returned in unknown Date format.");
                    return false;
                }
                
                //before we create the Writer, must get the writer's writer as grader IDs
                //more specifically, these are the rowIDs from the writerAsGrader table, 
                //for which this writer is the grader
                p=new Properties();
                TransmissionHandler.addDefaultWriterProperties(p);
                p.setProperty("command",TransmissionConstants.getWriterAsGraderID);
                p.setProperty("writerID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                sq=new SubmitQuery(p,writerAddress);
                try{
                    s=sq.submit();
                    text.append("\n");
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During writer as grader ID lookup.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During writer as grader ID lookup.)")
                    .toString();
                    return false;
                }

                IDResultReader IDrr=new IDResultReader(s);
                try{
                    IDrr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "writer as grader ID query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "writer as grader ID query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Writer as grader ID query returned in unknown format.");
                    return false;
                }

                status=IDrr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=IDrr.getBody();
                    fullText=text.toString();
                    return false;
                }

                
                Vector wagIDs=new Vector();
                while(IDrr.next()){
                    wagIDs.add(new Long(IDrr.getCurrentID()));
                }
                //now create the Writer
                writer=new Writer(wID,uID,studentIdent,firstName,lastName,uName,email,uNameType,cDate,wagIDs);
                //finally, go and get all the courses that the writer is in
                //and put it into a Vector of Course objects.

                //(we do this by first getting courseIDs and then getCourse)
                p=new Properties();
                TransmissionHandler.addDefaultWriterProperties(p);
                p.setProperty("command",TransmissionConstants.getCourseID);
                p.setProperty("writerID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                sq=new SubmitQuery(p,writerAddress);
                try{
                    s=sq.submit();
                    text.append("\n");
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During course ID lookup.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During course ID lookup.)")
                    .toString();
                    return false;
                }

                IDrr=new IDResultReader(s);
                try{
                    IDrr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "course ID query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "course ID query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "course ID query returned in unknown format.");
                    return false;
                }

                status=IDrr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=IDrr.getBody();
                    fullText=text.toString();
                    return false;
                }
                                
                p=new Properties();
                TransmissionHandler.addDefaultWriterProperties(p);
                p.setProperty("command",TransmissionConstants.getCourse);
                p.setProperty("writerID",TransmissionHandler.ID);

                p.setProperty("courseID",IDrr.getBody());//formatted all nicely already.

                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                sq=new SubmitQuery(p,writerAddress);
                try{
                    text.append("\n");
                    s=sq.submit();
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During course lookup.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During course lookup.)").toString();
                    return false;
                }


                trr=new TableResultReader(s);
                try{
                    trr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Course information query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Course information query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Course information query returned in unknown format.");
                    return false;
                }
                status=trr.getStatus();
                if(status==false){
                    message=trr.getBody();
                    fullText=text.toString();
                    return false;
                }

                try{
                    courses=TransmissionHandler.getCourses(trr,TransmissionHandler.WRITER_TYPE);
                }catch(NumberFormatException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Course information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                    fullText=new ResultWriter(false,"",
                    "Course information query returned in an unknown format.  Please notify an administrator. (Number formats)").toString();
                    sendMessageToDebugFile("Error in getWriterLogin", 
                                           "Course information query returned in unknown number format");
                    return false;
                }

                //now put the assignments into the courses
                currentCourses=new Vector();
                pastCourses=new Vector();
                for(int i=0;i<courses.size();i++){
                    Course c=(Course)courses.elementAt(i);
                    if(c.getIsCurrent().equals("Y")){
                        currentCourses.add(c);
                    }
                    else if(c.getIsCurrent().equals("N")){
                        currentCourses.add(c);
                    }
                    p=new Properties();
                    //first get the assignIDs
                    TransmissionHandler.addDefaultWriterProperties(p);
                    p.setProperty("courseID",Long.toString(c.getCourseID()));
                    p.setProperty("command",TransmissionConstants.getAssignID);
                    p.setProperty("writerID",TransmissionHandler.ID); 
                    p.setProperty("myUserName",TransmissionHandler.userName);
                    p.setProperty("myEPass",TransmissionHandler.ePass);
                    p.setProperty("myUniversityID",TransmissionHandler.universityID);
                    sq=new SubmitQuery(p,writerAddress);
                    try{
                        s=sq.submit();
                        text.append("\n");
                        text.append(s);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message="An error occurred when trying to connect to the server. (During assignment ID lookup.)";
                        fullText=new
                        ResultWriter(false,"","An error occurred when trying to connect to the server. (During assignment ID lookup.)").toString();
                        return false;
                    }
    
                    IDrr=new IDResultReader(s);
                    try{
                        IDrr.parse();
                    }catch(ParseResultException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Assignment ID query returned in an unknown format.  Please notify an administrator.";
                        fullText=new ResultWriter(false,"",
                        "Assignment ID query returned in an unknown format.  Please notify an administrator.")
                        .toString();
                        sendMessageToDebugFile("Error in getWriterLogin", 
                                               "Assignment ID query returned in unknown format.");
                        return false;
                    }
    
                    status=IDrr.getStatus();
                    //if the status was false, then don't continue.
                    if(status==false){
                        message=IDrr.getBody();
                        fullText=text.toString();
                        sendMessageToDebugFile("Error in getWriterLogin", 
                                               message);
                        return false;
                    }
                    p=new Properties();
                    TransmissionHandler.addDefaultWriterProperties(p);
                    p.setProperty("command",TransmissionConstants.getAssignment);
                    p.setProperty("writerID",TransmissionHandler.ID);    
                    p.setProperty("assignID",IDrr.getBody());//formatted all nicely already.    
                    p.setProperty("myUserName",TransmissionHandler.userName);
                    p.setProperty("myEPass",TransmissionHandler.ePass);
                    p.setProperty("myUniversityID",TransmissionHandler.universityID);

                    sq=new SubmitQuery(p,writerAddress);
                    try{
                        text.append("\n");
                        s=sq.submit();
                        text.append(s);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message="An error occurred when trying to connect to the server. (During assignment lookup.)";
                        fullText=new
                        ResultWriter(false,"",
                        "An error occurred when trying to connect to the server. (During assignment lookup.)").toString();
                        return false;
                    }


                    trr=new TableResultReader(s);
                    try{
                        trr.parse();
                    }catch(ParseResultException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Assignment information query returned in an unknown format.  Please notify an administrator.";
                        fullText=new ResultWriter(false,"",
                        "Assignment information query returned in an unknown format.  Please notify an administrator.")
                        .toString();
                        sendMessageToDebugFile("Error in getWriterLogin", 
                                               "Assignment information query returned in unknown format.");
                        return false;
                    }
                    status=trr.getStatus();
                    if(status==false){
                        message=trr.getBody();
                        fullText=text.toString();
                        return false;
                    }

                    try{
                        assignments=TransmissionHandler.getAssignments(trr,TransmissionHandler.WRITER_TYPE);
                    }catch(NumberFormatException e){
                        if(debug){System.out.println(e);}
                        message=
                        "Assignment information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                        fullText=new ResultWriter(false,"",
                        "Assignment information query returned in an unknown format.  Please notify an administrator. (Number formats)").toString();
                        sendMessageToDebugFile("Error in getWriterLogin", 
                                               "Assignment information query returned in unknown number format");
                        return false;
                    }
                    //now get the draftStructures for every assignment
                    for(int j=0;j<assignments.size();j++){
                        Assignment a=(Assignment)assignments.elementAt(j);
                        p=new Properties();
                        //first get the assignIDs
                        TransmissionHandler.addDefaultWriterProperties(p);
                        p.setProperty("assignID",Long.toString(a.getAssignID()));
                        p.setProperty("command",TransmissionConstants.getDraftStructureID);
                        p.setProperty("writerID",TransmissionHandler.ID); 
                        p.setProperty("myUserName",TransmissionHandler.userName);
                        p.setProperty("myEPass",TransmissionHandler.ePass);
                        p.setProperty("myUniversityID",TransmissionHandler.universityID);
                        sq=new SubmitQuery(p,writerAddress);
                        try{
                            s=sq.submit();
                            text.append("\n");
                            text.append(s);
                        }catch(IOException e){
                            if(debug){System.out.println(e);}
                            message="An error occurred when trying to connect to the server. (During draft structure ID lookup.)";
                            fullText=new
                            ResultWriter(false,"",
                            "An error occurred when trying to connect to the server. (During draft structure ID lookup.)")
                            .toString();
                            return false;
                        }
                        IDrr=new IDResultReader(s);
                        try{
                            IDrr.parse();
                        }catch(ParseResultException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Draft Structure ID query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Draft Structure ID query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getWriterLogin", 
                                                   "Draft Structure ID query returned in unknown format.");
                            return false;
                        }
        
                        status=IDrr.getStatus();
                        //if the status was false, then don't continue.
                        if(status==false){
                            message=IDrr.getBody();
                            fullText=text.toString();
                            return false;
                        }
                        //now get the draft structures
                        Vector vds=new Vector();
                        p=new Properties();
                        TransmissionHandler.addDefaultWriterProperties(p);
                        p.setProperty("command",TransmissionConstants.getDraftStructure);
                        p.setProperty("writerID",TransmissionHandler.ID);    
                        p.setProperty("draftStructureID",IDrr.getBody());//formatted all nicely already.    
                        p.setProperty("myUserName",TransmissionHandler.userName);
                        p.setProperty("myEPass",TransmissionHandler.ePass);
                        p.setProperty("myUniversityID",TransmissionHandler.universityID);

                        sq=new SubmitQuery(p,writerAddress);
                        try{
                            text.append("\n");
                            s=sq.submit();
                            text.append(s);
                        }catch(IOException e){
                            if(debug){System.out.println(e);}
                            message="An error occurred when trying to connect to the server. (During draft structure lookup.)";    
                            fullText=new
                            ResultWriter(false,"",
                            "An error occurred when trying to connect to the server. (During draft structure lookup.)").toString();
                            return false;
                        }


                        trr=new TableResultReader(s);
                        try{
                            trr.parse();
                        }catch(ParseResultException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Draft structure information query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Draft structure information query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getWriterLogin", 
                                                   "Draft structure information query returned in unknown format");
                            return false;
                        }
                        status=trr.getStatus();
                        if(status==false){
                            message=trr.getBody();
                            fullText=text.toString();
                            return false;
                        }
                        
                        try{
                            vds=TransmissionHandler.getDraftStructure(trr,TransmissionHandler.WRITER_TYPE);
                        }catch(NumberFormatException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Draft structure information query returned in an unknown format.  Please notify an administrator. (Number formats)";
                            fullText=new ResultWriter(false,"",
                            "Draft structure information query returned in an unknown format.  Please notify an administrator. (Number formats)").toString();
                            sendMessageToDebugFile("Error in getWriterLogin", 
                                                   "Draft structure information query returned in unknown number format.");
                            return false;
                        }
                        a.setDraftStructure(vds);
                        c.addAssignment(a);
                    }
                }

                //finally...we're done.  It has been a long ride, and I'd like to thank you.
                fullText=text.toString();
                return status;
            }
            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
            //only returns current courses
            public Vector getCourses(){
                return currentCourses;
            }
            //only returns past courses
            public Vector getPastCourses(){
                return pastCourses;
            }
            //returns all courses
            public Vector getAllCourses(){
                return courses;
            }
            public Writer getWriter(){
                return writer;
            }
        };
    }


    /**
     * Returns a SubmitPaperQuery object that performs a paper submission.
     * @param bp The paper to submit.
     */
    public static SubmitPaperQuery getSubmitPaper(final Blaise_Paper bp){
        return new SubmitPaperQuery(){
            Blaise_Paper paper=null;
            private String message="";
            private String fullText=null;
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                Properties p=new Properties();
                TransmissionHandler.addDefaultWriterProperties(p);
                p.setProperty("command",TransmissionConstants.submitPaper);
                p.setProperty("writerID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                byte[] sbq=null;
                SubmitQuery sq=new SubmitQuery(p,writerAddress);
                String s=null;
                try{
                    sbq=sq.submitBinary(bp);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During submit paper.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During submit paper.)")
                    .toString();
                    return false;
                }

                boolean status;
                byte[] b=DualByteArray.extractFirst(sbq);
                p=DualByteArray.toProperties(b);
                s=p.getProperty("message");
                SPResultReader SPrr=new SPResultReader(s);
                try{
                    SPrr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Submit paper query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Submit paper query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getSubmitPaper", 
                                           "Submit paper query returned in unknown format.");
                    return false;
                }

                if(SPrr.getStatus()==false){
                    message=SPrr.getBody();
                    return false;
                }

                //Now attach the new IDs to the paper.

                paper=bp;//set this paper to be the one returned.

                String pID=SPrr.getPaperID();
                if(pID!=null){
                    paper.setPaperID(pID);
                }
                Vector drafts=paper.getDrafts();
                Blaise_Draft d;

                int numDrafts=drafts.size();
                //first count the number of drafts with empty draftIDs
                //and compare it to the amount of draftIDs provided
                //if they don't equal, get upset.
                int emptyCount=0;
                for(int i=0;i<numDrafts;i++){
                    d=(Blaise_Draft)drafts.get(i);
                    if(d.getDraftID().equals("")){
                        emptyCount++;
                    }
                }
                if(emptyCount!=SPrr.getNumberOfDraftIDs()){
                    if(debug){System.out.println("emptyCount:"+emptyCount+" numberOfDraftIDs():"+SPrr.getNumberOfDraftIDs());}
                    message=
                    "An error occurred during paper submission (draft counts don't equal).  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "An error occurred during paper submission (draft counts don't equal).  Please notify an administrator.").toString();
                    sendMessageToDebugFile("Error in getSubmitPaper", 
                                           "An error occurred during paper submission: draft counts don't equal.");
                    return false;
                }
                //then, if they do equal, add the draftIDs to the drafts.
                int currentIDIndex=0;
                for(int i=0;i<numDrafts;i++){
                    d=(Blaise_Draft)drafts.get(i);
                    if(d.getDraftID().equals("")){
                        d.setDraftID(SPrr.getDraftID(currentIDIndex));
                        currentIDIndex++;
                    }
                    if(pID!=null){//this was a new paper, so set the paperIDs in the drafts, too.
                        d.setPaperID(pID);
                    }
                }

                //all the IDs have been updated...
                return true;
            }
            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
            public Blaise_Paper getUpdatedPaper(){
                return paper;
            }
        };
    }

    /**
     * Allows the user to download papers.  If the user is a writer, the papers are limited
     * to papers that they have authored.  If the user is a grader, the papers are limited to
     * papers from their classes.
     * @param paperIDs a Vector containing all of the paperIDs (in Long objects)
     * @param type {@link #WRITER_TYPE} or {@link #GRADER_TYPE}.
     */
     public static PaperDownloadQuery getPaperDownload(final Vector paperIDs, final int type){
        if(type!=WRITER_TYPE && type!=GRADER_TYPE){
            return null;
        }
        return new PaperDownloadQuery(){
            Vector papers=null;
            private String message="";
            private String fullText=null;
            public Long theID=null;
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                papers=new Vector();
                for(int j=0;j<paperIDs.size();j++){
                    Properties p=new Properties();

                    p.setProperty("command",TransmissionConstants.getPaperFull);
                    if(type==GRADER_TYPE){
                        TransmissionHandler.addDefaultGraderProperties(p);
                        p.setProperty("graderID",TransmissionHandler.ID);
                    }
                    else if(type==WRITER_TYPE){
                        TransmissionHandler.addDefaultWriterProperties(p);
                        p.setProperty("writerID",TransmissionHandler.ID);
                    }
                    p.setProperty("myUserName",TransmissionHandler.userName);
                    p.setProperty("myEPass",TransmissionHandler.ePass);
                    p.setProperty("myUniversityID",TransmissionHandler.universityID);
                    /*Once again, this is only used to get multiple papers at once
                    StringBuffer sb=new StringBuffer();
                    int size=paperIDs.size();
                    for(int i=0;i<size;i++){
                        sb.append(((Long)paperIDs.get(i)).toString());
                        if(i<size-1){
                            sb.append(TransmissionConstants.ID_SEPARATOR);
                        }
                    }
                    p.setProperty("paperID",sb.toString());
                    */    
                    theID=(Long)paperIDs.get(j);
                    p.setProperty("paperID",theID.toString());
                    SubmitQuery sq=null;
                    if(type==GRADER_TYPE){
                        sq=new SubmitQuery(p,graderAddress);
                    }
                    else if(type==WRITER_TYPE){
                        sq=new SubmitQuery(p,writerAddress);
                    }
                    byte[] sbq=null;
                    try{
                        sbq=sq.submitBinary(null);
                    }catch(IOException e){
                        if(debug){System.out.println(e);}
                        message="An error occurred when trying to connect to the server during download of: paper"+j+" out of "+paperIDs.size();
                        fullText=new
                        ResultWriter(false,"",
                        "An error occurred when trying to connect to the server during download of: paper"+j+" out of "+paperIDs.size())
                        .toString();
                        return false;
                    }
                    byte[] firstPart=DualByteArray.extractFirst(sbq);
                    Properties prop=DualByteArray.toProperties(firstPart);
                    //this SubmitQuery will only return a ResultWriter when something has gone wrong
                    if(!prop.getProperty("status").equals(TransmissionConstants.SUCCESS)){
                        ResultReader rr=new ResultReader(prop.getProperty("message"));
                        try{
                            rr.parse();
                        }catch(ParseResultException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Get paper query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Get paper query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getPaperDownload", 
                                                   "Get paper query returned in unknown format.");
                            return false;
                        }
    
                        message=rr.getBody();
                        return false;
                    }
                    Blaise_Paper pap=null;
                    byte[] pape=null;
                    int numEntries=DualByteArray.getNumEntries(sbq);
                    for(int i=1;i<=numEntries;i++){
                        pape=DualByteArray.extractNth(sbq, i+1);
                        try{
                            pap=BlaiseFile.byteArrayToPaper(pape);
                        }catch(IOException e){
                            if(debug){System.out.println(e);}
                            message=
                            "Paper Header query returned in an unknown format.  Please notify an administrator.";
                            fullText=new ResultWriter(false,"",
                            "Paper Header query returned in an unknown format.  Please notify an administrator.")
                            .toString();
                            sendMessageToDebugFile("Error in getPaperDownload", 
                                                   "Paper header query returned in unknown format.");
                            return false;                        
                        }
                        papers.add(pap);
                    }
                }
                return true;
            }
            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
            public Vector getPapers(){
                return papers;
            }
        };
     }

    /**
     * Returns a StagingQuery object that performs a comment (grade) checkin.
     * @param paper a Blaise_Paper object to submit.
     */
    public static TransmissionQuery getStagePapers(final Blaise_Paper paper, final int type){
        return new TransmissionQuery(){
            String fullText=null;
            String message="";
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                //perform stagePaper query
                Properties p=new Properties();
                if(type==TransmissionHandler.GRADER_TYPE){
                    TransmissionHandler.addDefaultGraderProperties(p);
                }
                else if(type==TransmissionHandler.WRITER_TYPE){
                    TransmissionHandler.addDefaultWriterProperties(p);
                }
                p.setProperty("command",TransmissionConstants.stagePaper);
                p.setProperty("graderID",TransmissionHandler.ID);
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);

                SubmitQuery sq=null;
                if(type==TransmissionHandler.GRADER_TYPE){
                    sq=new SubmitQuery(p,graderAddress);
                }
                else if(type==TransmissionHandler.WRITER_TYPE){
                    sq=new SubmitQuery(p,writerAddress);
                }
                byte[] sbq=null;
                try{
                    sbq=sq.submitBinary(paper);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During stage papers.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During stage papers.)")
                    .toString();
                    return false;
                }

                boolean status;
                byte[] b=DualByteArray.extractFirst(sbq);
                p=DualByteArray.toProperties(b);
                String s=p.getProperty("message");
                ResultReader rr=new ResultReader(s);
                try{
                    rr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Stage paper query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Stage paper query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getStagePapers", 
                                           "Stage paper query returned in unknown format");
                    return false;
                }
                status=rr.getStatus();
                message=rr.getBody();

                return status;
            }


            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
        };
    }

    /**
     * Returns a StagingQuery object that performs a comment (grade) checkin.
     * @param courseIDs a vector of the courseIDs of the papers to send
     */
    public static TransmissionQuery getSendStagedPapers(final Vector courseIDs, final int type){
        return new TransmissionQuery(){
            String fullText=null;
            String message="";
            public boolean transmit(){
                return this.transmit(TransmissionConstants.GRADER_FORWARD_ADDRESS, TransmissionConstants.WRITER_FORWARD_ADDRESS);
            }
            public boolean transmit(String graderAddress, String writerAddress){
                //first get the stagingIDs of the papers to send
                StringBuffer sb=new StringBuffer();
                Properties p=new Properties();
                if(type==TransmissionHandler.GRADER_TYPE){
                    TransmissionHandler.addDefaultGraderProperties(p);
                }
                else if(type==TransmissionHandler.WRITER_TYPE){
                    TransmissionHandler.addDefaultWriterProperties(p);
                }
                int size=courseIDs.size();
                StringBuffer text=new StringBuffer();
                for(int i=0;i<size;i++){
                    sb.append(((Long)courseIDs.get(i)).toString());
                    if(i<size-1){
                        sb.append(TransmissionConstants.ID_SEPARATOR);
                    }
                }
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                p.setProperty("command",TransmissionConstants.getStagingID);
                p.setProperty("courseID",sb.toString());
                if(type==GRADER_TYPE){
                    p.setProperty("graderID",TransmissionHandler.ID);
                }
                else if(type==WRITER_TYPE){
                    p.setProperty("writerID",TransmissionHandler.ID);
                }
                p.setProperty("status",TransmissionConstants.INCOMPLETE);
                SubmitQuery sq=null;
                if(type==GRADER_TYPE){
                    sq=new SubmitQuery(p,graderAddress);
                }
                else if(type==WRITER_TYPE){
                    sq=new SubmitQuery(p,writerAddress);
                }
                String s=new String();
                try{
                    s=sq.submit();
                    text.append(s);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During send staged papers.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During send staged papers.)")
                    .toString();
                    return false;
                }
                
                boolean status;
                IDResultReader IDrr=new IDResultReader(s);
                try{
                    IDrr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Staging ID query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Staging ID query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getSendStagedPapers", 
                                           "Staging ID query returned in unknown format");
                    return false;
                }
                status=IDrr.getStatus();
                //if the status was false, then don't continue.
                if(status==false){
                    message=IDrr.getBody();
                    fullText=text.toString();
                    return false;
                }

                Vector stage=null;
                String paperIDString=IDrr.getBody();
                if(paperIDString.equals("")){
                    //there were no stagingIDs that matched the requirements.
                    fullText=text.toString();
                    stage=new Vector();//empty
                    return true;
                }

                p=new Properties();
                if(type==TransmissionHandler.GRADER_TYPE){
                    TransmissionHandler.addDefaultGraderProperties(p);
                }
                else if(type==TransmissionHandler.WRITER_TYPE){
                    TransmissionHandler.addDefaultWriterProperties(p);
                }
                p.setProperty("myUserName",TransmissionHandler.userName);
                p.setProperty("myEPass",TransmissionHandler.ePass);
                p.setProperty("myUniversityID",TransmissionHandler.universityID);
                p.setProperty("command",TransmissionConstants.sendStagedPaper);
                if(type==GRADER_TYPE){
                    p.setProperty("graderID",TransmissionHandler.ID);
                }
                else if(type==WRITER_TYPE){
                    p.setProperty("writerID",TransmissionHandler.ID);
                }
                p.setProperty("stagingID",IDrr.getBody());
                sb=new StringBuffer();
                if(type==GRADER_TYPE){
                    sq=new SubmitQuery(p,graderAddress);
                }
                else if(type==WRITER_TYPE){
                    sq=new SubmitQuery(p,writerAddress);
                }
                byte[] sbq=null;
                try{
                    sbq=sq.submitBinary(null);
                }catch(IOException e){
                    if(debug){System.out.println(e);}
                    message="An error occurred when trying to connect to the server. (During send staged papers.)";
                    fullText=new
                    ResultWriter(false,"",
                    "An error occurred when trying to connect to the server. (During send staged papers.)")
                    .toString();
                    return false;
                }

                byte[] b=DualByteArray.extractFirst(sbq);
                p=DualByteArray.toProperties(b);
                s=p.getProperty("message");
                ResultReader rr=new ResultReader(s);
                try{
                    rr.parse();
                }catch(ParseResultException e){
                    if(debug){System.out.println(e);}
                    message=
                    "Send staged paper query returned in an unknown format.  Please notify an administrator.";
                    fullText=new ResultWriter(false,"",
                    "Send staged paper query returned in an unknown format.  Please notify an administrator.")
                    .toString();
                    sendMessageToDebugFile("Error in getSendStagedPapers", 
                                           "Send staged paper query returned in an unknown format.");
                    return false;
                }
                status=rr.getStatus();
                message=rr.getBody();

                return status;
            }


            public String getMessage(){
                return message;
            }
            public String getFullText(){
                return fullText;
            }
        };
    }

    /**
     * Checks to see if connection to the server can be made.
     * @return <code>true</code> if a connection was made, <code>false</code> otherwise.
     */
    public static boolean isOnline(){
        try{
            HttpURLConnection c=(HttpURLConnection)(new URL(TransmissionConstants.GRADER_FORWARD_ADDRESS)).openConnection();
            c.setUseCaches(false);
            c.getContent();
            c.disconnect();
        }catch(IOException e){
            return false;
        }
        return true;
    }

    /**
     * Sets the properties that every grader query needs, that don't change.  Including version and passCode.
     */
    static void addDefaultGraderProperties(Properties p){
        p.setProperty("version","2");
        p.setProperty("passCode",TransmissionConstants.GRADER_PASS_CODE);
    }

    /**
     * Sets the properties that every writer query needs, that don't change.  Including version and passCode.
     */
    static void addDefaultWriterProperties(Properties p){
        p.setProperty("version","2");
        p.setProperty("passCode",TransmissionConstants.WRITER_PASS_CODE);
    }

    /**
     * When given a TableResultReader with a successful getCourse query, it will
     * pull all the courses from the reader and return a Vector full of Course objects.
     */
    static Vector getCourses(TableResultReader trr, final int type) throws NumberFormatException{
        Vector v=new Vector();

        long courseID;
        String contactInfo;
        int universityID;
        String dept;
        String number;
        String section;
        String semester;
        int year;
        String isCurrent;

        Course c;

        String s;
        while(trr.next()){
            s=trr.getCell("courseID");
            courseID=Long.parseLong(s);
            contactInfo=trr.getCell("contactInfo");
            s=trr.getCell("universityID");
            universityID=Integer.parseInt(s);
            dept=trr.getCell("dept");
            number=trr.getCell("number");
            section=trr.getCell("section");
            semester=trr.getCell("semester");
            s=trr.getCell("year");
            year=Integer.parseInt(s);
            s=trr.getCell("numberGraders");
            isCurrent=trr.getCell("isCurrent");

            c=new Course(courseID,contactInfo,universityID,dept,number,section,semester,year,null,isCurrent);
            v.add(c);
        }
        return v;
    }
    
    /**
     * When given a courseID, this returns all of the assignments for that course
     */
    static Vector getAssignments(TableResultReader trr,final int type){
        Vector v=new Vector();
        Assignment a=null;
        long aID=0L;
        long cID=0L;
        String name=null;
        int numDrafts=0;
        int assignNum=0;
        
        //get the Writer's info from the reader.
        while(trr.next()){
            aID=0L;
            cID=0L;
            name=null;
            numDrafts=0;
            assignNum=0;
            try{
                aID=Long.parseLong(trr.getCell("assignID"));
                cID=Long.parseLong(trr.getCell("courseID"));
                SimpleDateFormat sdf=new SimpleDateFormat(TransmissionConstants.dateFormat);
                name=trr.getCell("name");
                numDrafts=Integer.parseInt(trr.getCell("numDrafts"));
                assignNum=Integer.parseInt(trr.getCell("assignNum"));
            }catch(NumberFormatException e){
                if(debug){System.out.println(e);}
                return null;
            }        
            a=new Assignment(aID,cID,name,numDrafts,assignNum,null);
            v.add(a);
        }
        return v;
    }
    
    /**
     * When given an assignID, this method returns the corresponding DraftStructure
     */
    static Vector getDraftStructure(TableResultReader trr,final int type){
        Vector v=new Vector();
        DraftStructure ds=null;
        SimpleDateFormat sdf=new SimpleDateFormat(TransmissionConstants.dateFormat);
        //get the Writer's info from the reader.
        long dsID=0L;
        int dNum=0;
        long aID=0L;
        String gradingType=null;
        Date ncDate=null;
        while(trr.next()){
            dsID=0L;
            dNum=0;
            aID=0L;
            gradingType=null;
            ncDate=null;
            try{
                dsID=Long.parseLong(trr.getCell("draftStructureID"));
                dNum=Integer.parseInt(trr.getCell("draftNum"));
                ncDate=sdf.parse(trr.getCell("noChangeDate"));
                aID=Long.parseLong(trr.getCell("assignID"));
                gradingType=trr.getCell("gradingType");
            }catch(NumberFormatException e){
                if(debug){System.out.println(e);}
                return null;
            }catch(ParseException e){
                if(debug){System.out.println(e);}
                return null;
            }
            ds=new DraftStructure(dsID,dNum,aID,gradingType,ncDate);
            v.add(ds);
        }
        return v;
    }
    
    //writes the specified message to a debug file on the server
    //currently, it gets back success or failure in string form, 
    //with which it currently does nothing
    public static void sendMessageToDebugFile(final String header,final String body){
        Properties p=new Properties();
        p.setProperty("header",header);
        p.setProperty("body",body);
        SubmitQuery sq=new SubmitQuery(p,TransmissionConstants.DEBUG_FORWARD_ADDRESS);
        String s=null;
        try{
            s=sq.submit();
        }catch(IOException e){
            if(debug){System.out.println(e);}
        }
    }

    /**
     * Runs a series of tests to determine if everything is working (db specific).
     */
    public static void main(String[] argv){
        if(argv.length!=1){
            System.out.println("usage TransmissionHandler writer|grader");
            return;
        }
        Blaise_Paper paper=null;
        FileInputStream fis=null;
        if(argv[0].equalsIgnoreCase("grader")){
            
            GraderLoginQuery glq=TransmissionHandler.getGraderLogin("mcGon","transfigure","1");
            System.out.println("GraderLoginQuery");
            boolean status=glq.transmit();
            System.out.println("fullText="+glq.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+glq.getMessage());
            System.out.println("Grader="+glq.getGrader());
            System.out.println("Courses="+glq.getCourses());

            PaperSearchQuery psq=TransmissionHandler.getPaperSearch(false,null,null,null,null);
            System.out.println("PaperSearchQuery (all)");
            status=psq.transmit();
            System.out.println("fullText="+psq.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+psq.getMessage());
            Vector p=psq.getPapers();
            for(int i=0;i<p.size();i++){
                ((Blaise_Paper)p.get(i)).print();
            }
            Vector courses=glq.getCourses();
            psq=TransmissionHandler.getPaperSearch(true,null,null,null,(Course)courses.get(2));
            System.out.println("PaperSearchQuery (ungraded)");
            status=psq.transmit();
            System.out.println("fullText="+psq.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+psq.getMessage());
            p=psq.getPapers();
            for(int i=0;i<p.size();i++){
                ((Blaise_Paper)p.get(i)).print();
            }

            p.clear();
            p.add(new Long(5L));
            p.add(new Long(1L));
            CheckOutPapersQuery cpq=TransmissionHandler.getCheckOutPapers(p,GRADER_TYPE);
            System.out.println("CheckOutPaperQuery");
            status=cpq.transmit();
            System.out.println("fullText="+cpq.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+cpq.getMessage());
            p=cpq.getPapers();
            for(int i=0;i<p.size();i++){
                ((Blaise_Paper)p.get(i)).print();
            }
            
            try{
                fis=new FileInputStream("checkin.txt");
            }catch(FileNotFoundException e){
                System.out.println("checkin.txt not found: "+e);
            }
            try{
                paper=BlaiseFile.inputStreamToPaper(fis);
            }catch(IOException e){
                System.out.println("Input stream could not be converted to paper: "+e);
                return;
            }
            TransmissionQuery cip=TransmissionHandler.getCheckInPapers(paper,GRADER_TYPE);
            System.out.println("CheckInPaperQuery");
            status=cip.transmit();
            System.out.println("fullText="+cip.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+cip.getMessage());
        }
        else if(argv[0].equalsIgnoreCase("writer")){
            WriterLoginQuery wlq=TransmissionHandler.getWriterLogin("potter","quidditch","1");
            System.out.println("WriterLoginQuery");
            boolean status=wlq.transmit();
            System.out.println("fullText="+wlq.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+wlq.getMessage());
            System.out.println("Writer="+wlq.getWriter());
            System.out.println("Courses="+wlq.getCourses());

            System.out.println("SubmitPaperQuery");
            try{
                fis=new FileInputStream("paper.txt");
            }catch(FileNotFoundException e){
                System.out.println("paper.txt not found: "+e);
            }
            try{
                paper=BlaiseFile.inputStreamToPaper(fis);
            }catch(IOException e){
                System.out.println("Input stream could not be converted to paper: "+e);
                return;
            }
            SubmitPaperQuery spq=TransmissionHandler.getSubmitPaper(paper);
            status=spq.transmit();
            System.out.println("fullText="+spq.getFullText());
            System.out.println("status="+status);
            System.out.println("message="+spq.getMessage());
            Blaise_Paper bp=spq.getUpdatedPaper();
            System.out.println("Updated paper=");
            bp.print();
        }
    }
}
