package edu.gwu.blaise.version2.transmission;
/**
 * This is the base from which all of the other TransmissionQuery interfaces extend.
 * It provides basic functionality that is available in all of the interfaces.
 * @author Michael Gaiman
 * @version 2 1/25/2004
 */
public interface TransmissionQuery {

    /**
     * Actually causes the transmission to be sent.
     * @return <code>true</code> if the transmission and query were successful.
     * <code>false</code> otherwise.
     */
    public boolean transmit();
    
    /**
     * Actually causes the transmission to be sent, separate addresses are used for different query types.
     * @return <code>true</code> if the transmission and query were successful.
     * <code>false</code> otherwise.
     */
    public boolean transmit(String graderAddress, String writerAddress);

    
    /**
     * If the transmission was a failure, this method will provide the supplied reason.
     * It is appriopriate to display to the user.
     * If the transmission was successfull, this method may or may not provide extra information
     * depending on the type of query.
     */
    public String getMessage();
    
    /**
     * @return The full text of what was recieved in response.  If there Query performed multiple
     * transactions with the backend, it will include all of the concatinated one after the other in
     * the order recieved.
     */
    public String getFullText();
}
