package edu.gwu.blaise.version2.util;
import javax.swing.*;
import java.awt.*;
/**
* AboutDialog is a JDialog that displays information about the
 * Blaise Client currently being ran and about the java platform
 * that is running it.
 * @author Michael Gaiman
 * @version 1 7/15/2004
 */
public class AboutDialog extends JDialog{
	public static final int EDITOR_TYPE=-777;
	public static final int PORTAL_TYPE=-778;
	public static final int GRADER_TYPE=-779;
	
	private JFrame parent;
	private int type;
	
	public AboutDialog(JFrame parent,int type){
		super(parent,"About",false);//non model
		this.parent=parent;
		this.type=type;
		
				System.out.println("Creating AboutDialog...");
				
        SwingUtilities.invokeLater(new Runnable(){
					public void run(){
						System.out.println("Running AboutDialog...");
						createAndShowGUI();
					}
        });
	}
	
	private static final int WIDTH=400;  // 200
	private static final int HEIGHT=400; // 300
	
	private void createAndShowGUI(){
		setSize(WIDTH,HEIGHT);
		setTitle("About "+getClientName(type));
		Container cont=getContentPane();
		cont.setLayout(new GridBagLayout());
		GridBagConstraints c=new GridBagConstraints();
		
		JLabel icon=new JLabel(new ImageIcon(Constants.getIconPath()+"about.png"));
		c.weightx=1;
		c.gridwidth=c.REMAINDER;
		c.weighty=.3;
		cont.add(icon,c);
		
		JEditorPane pane=new JEditorPane("text/html",getAboutText());
		pane.setEditable(false);
		pane.setCaretPosition(0);
		//pane.setEditorKit(new javax.swing.text.html.HTMLEditorKit());
		//pane.setText(getAboutText());
		
		c.weightx=1;
		c.weighty=.7;
		c.fill=c.BOTH;
		JScrollPane scroll=new JScrollPane(pane);
		cont.add(scroll,c);
		
		//figure out the location to open on the screen.
		int myX;
		int myY;
		int pX,pY,pW,pH;
		if(parent==null){
			//open it in the middle of the screen.
			Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
			pX=0;
			pY=0;
			pW=screen.width;
			pH=screen.height;
		}
		else{
			pX=parent.getX();
			pY=parent.getY();
			pW=parent.getWidth();
			pH=parent.getHeight();
		}
		
		myX=pX+(pW/2)-(WIDTH/2);
		myY=pY+(pH/2)-(HEIGHT/2);
		setLocation(myX,myY);
		
		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setVisible(true);
	}
	
	private String getClientName(int type){
		switch(type){
			case EDITOR_TYPE:
				return "Blaise Editor";
			case PORTAL_TYPE:
				return "Blaise Grading Portal";
			case GRADER_TYPE:
				return "Blaise Grader";
			default:
				return "";
		}
	}
	
	private String getClientVersion(){
		switch(type){
			case EDITOR_TYPE:
				return "2 (b5)";
			case PORTAL_TYPE:
				return "2 (b5)";
			case GRADER_TYPE:
				return "2 (b5)";
			default:
				return "";
		}
	}

	/**
	 * Generates a static string of text for the About Box. The text is an HTML code which is contained
	 * within a table to allow for indentation and individualised line centering.
	 */
	private String getAboutText(){
		//start with version of client
		StringBuffer text=new StringBuffer("<html><body><table border=0 cellpadding=3><tr><th colspan=2 align=left>Version:</th></tr>");
		
		text.append("<tr><td>&nbsp;&nbsp;&nbsp;&nbsp;</td><td>"+getClientName(type));
		text.append(getClientVersion()+"</td></tr>");
		
		text.append("<tr><th colspan=2 align=left><br>Java Version:</th></tr>");
		
		text.append("<tr><td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				text.append("<td>"+System.getProperty("java.version")+"</td></tr>");
				
        text.append("<tr><th colspan=2 align=left><br>Credits:</th></tr>");
				
        text.append("<tr><td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				text.append("<td align=center>Eisuke Arai, Michael Gaiman, Sean Hanlon, Kunal Joher, Jared Kiraly, Rizwan Sattar, Rahul Simha</td>");
        text.append("</table></body></html>");
        return text.toString();
	}
}
