package edu.gwu.blaise.version2.util;

import edu.gwu.blaise.version2.documents.Scribble;
import edu.gwu.blaise.version2.documents.ScribbleLine;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;

import java.util.*;

public class BlaiseCommentView extends JTextPane implements DocumentListener{
    
    BlaiseCommentViewInterface source;
    public boolean isOriginal;
    public int specialWidth;
    private boolean modified;
    
    public BlaiseCommentView(BlaiseCommentViewInterface source){
        super();
        this.source = source;
        isOriginal = false;
        specialWidth = -1;
        
        try {
            //Create the HTML Editor Kit and set the style sheet
            HTMLEditorKit htmlKit = new HTMLEditorKit();
            StyleSheet ss=new StyleSheet();
			ss.addRule("body {font: 15pt Helvetica;}\np {display: list-item;}");
			//htmlKit.setStyleSheet(htmlKit.getStyleSheet());
            htmlKit.setStyleSheet(ss);
            
            this.setEditorKit(htmlKit);	//assign editor kit to the textpane
            this.getDocument().putProperty("IgnoreCharsetDirective", Boolean.TRUE);

            this.setEditable(false);
            
            BufferedReader in = new BufferedReader(new StringReader(""));
            htmlKit.read(in, this.getDocument(),0);
        }
        catch (Exception e) {
            System.out.println("exception in BlaiseCommentView constructor");
        }
        
        //top,left,bottom,right margin defaults
        this.setMargin(new Insets(15,30,15,30));
        this.getCaret().deinstall(this);
    }
    
    public void setDimensions(int xAspect, int yAspect){
        Dimension desired = new Dimension(xAspect, yAspect);
        this.setPreferredSize(desired);
        this.setSize(desired);
        this.revalidate();
    }
    
    public Dimension getPreferredSize(){
        Dimension size = super.getPreferredSize();
        /*if(isOriginal){
            size.width = 450;
        }*/
        
        if(specialWidth != -1){
            size.width = specialWidth;
        }
        
        return size;
    }
    
    public void paintComponent(Graphics g){
        super.paintComponent(g);
                
        try{
            Graphics2D g2 = (Graphics2D)g;		//get graphics2D instance
            g2.setColor(Color.BLACK);			//reset to Black
            
			//Draw scribbles
            if(source.areScribblesVisible()){
                Vector scribbles = source.getScribbles();
                for(int j = 0; j < scribbles.size(); j++){
                    Scribble scrib = (Scribble)scribbles.elementAt(j);
                    Rectangle rectangle = this.modelToView(scrib.getOffset());
                    scrib.fixPosition(rectangle.getX() + scrib.getXDistanceFromOffset(), rectangle.getY());
                    g2.setPaint(scrib.getColor());
                    g2.setStroke(new BasicStroke(scrib.getWidth()));
                    for(int l1 = 0; l1 < scrib.size(); l1++){
                        ScribbleLine line = scrib.getLine(l1);
                        g2.draw(line);
                    }
                }
                g2.setStroke(new BasicStroke(1));	//reset stroke width to 1
                g2.setColor(Color.BLACK);			//reset to Black
            }
            
			//Draw corrections
            if(source.areCorrectionsVisible()){
                Vector corrections = source.getCorrections();
                for(int ind = 0; ind < corrections.size(); ind++){
                    CorrectionSet correctionset = (CorrectionSet)corrections.elementAt(ind);
                    Vector vector = correctionset.getStarts();
                    Vector vector1 = correctionset.getEnds();
                    g2.setPaint(correctionset.getColor());
                    for(int j = 0; j < vector.size(); j++){
                        Rectangle rectangle4 = this.modelToView(((Integer)vector.elementAt(j)).intValue());
                        Rectangle rectangle5 = this.modelToView(((Integer)vector1.elementAt(j)).intValue());
                        java.awt.geom.Rectangle2D.Double double3 = new java.awt.geom.Rectangle2D.Double();
                        double3.setRect(rectangle4.getX(), rectangle4.getY(), (rectangle5.getX() - rectangle4.getX()) + 1.0D, rectangle4.getHeight());
                        g2.draw(double3);
                    }
                }
                g2.setStroke(new BasicStroke(1));	//reset stroke width to 1
                g2.setColor(Color.BLACK);			//reset to Black
            }
            
			//If a scribble is selected, draw as blue with black outline around it
            Scribble selected = source.getSelected();
            if(selected != null){
                java.awt.geom.Rectangle2D.Double double1 = selected.getRect();
                g2.setPaint(Color.BLACK);
                g2.draw(double1);
                g2.setPaint(Color.BLUE);
                g2.setStroke(new BasicStroke(selected.getWidth()));
                for(int j1 = 0; j1 < selected.size(); j1++){
                    ScribbleLine line = selected.getLine(j1);
                    g2.draw(line);
                }
            }
        }
        catch(BadLocationException badlocationexception){}
    }
    
    public boolean isEmpty(){
        Document doc=getDocument();
        int length=doc.getLength();
        return (length==0);
    }
    
    public boolean isModified(){
        return modified;
    }
    
    public  void setModified(boolean m){
        if(Constants.isOSX() && m!=modified){
            //getRootPane().putClientProperty("windowModified", (m)?Boolean.TRUE:Boolean.FALSE);
        }
        modified=m;
    }
    
    public void changedUpdate(DocumentEvent e){
        setModified(true);
    }
    public void insertUpdate(DocumentEvent e){
        setModified(true);
    }
    public void removeUpdate(DocumentEvent e){
        setModified(true);
    }	
}
