package edu.gwu.blaise.version2.util;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.BorderFactory.*;
import javax.swing.event.*;
import edu.gwu.blaise.version2.documents.*;

/**
 * BlaiseDraftView, along with a couple inner classes handles the upper part
 * of the split pane.
 * @author Sean T. Hanlon
 * @author Michael Gaiman
 */
public class BlaiseDraftView extends JPanel {
    
    private static final boolean debug=false;
    private BlaiseDraftView thisSheet;
    private BlaiseSplitPane parent;
    private JTabbedPane tab_pane;
    private Vector listeners;
    
    private Vector tabs;
    private boolean fireComments=true;
    
    public BlaiseDraftView(BlaiseSplitPane parent){
        this.parent = parent;
        thisSheet = this;
        this.setLayout(new BorderLayout());
        listeners=new Vector();
        updateView(-1);
    }
    
    /**
        * Add a listener.
     */
    public void addDraftViewListener(DraftViewListener dlv){
        listeners.addElement(dlv);
    }
    
    /**
        * remove a specified listener.
     */
    public void removeDraftViewListener(DraftViewListener dlv){
        listeners.remove(dlv);
    }
    
    /**
     * Updates the view to reflect changes in the draft structure.
     * @param index if parent is Grader, it will turn the index tab color to red.
     */
    public void updateView(int index){
        if(parent.document == null || parent.document.getDrafts().size() == 0){
            this.removeAll();
            tabs = new Vector();
            //tab_pane = new JTabbedPane(JTabbedPane.TOP);
            //this.add(tab_pane, BorderLayout.CENTER);
            //this.validate();
        }
        
        int selected = 0;
        tab current;
        int selected_comment = 0;
        if(tab_pane != null && tabs.size() != 0){
            selected = tab_pane.getSelectedIndex();
            current = (tab)tabs.elementAt(selected);
            selected_comment = current.current_index;
        }
        
        this.removeAll();
        tabs = new Vector();
        
        tab_pane = new JTabbedPane(JTabbedPane.TOP);
        tab_pane.addChangeListener(new ChangeListener(){
                public void stateChanged(ChangeEvent e){
                    int selected=/*BlaiseDraftView.this.*/tab_pane.getSelectedIndex();
                    if(debug)System.out.println("Event point 1");
                    fireViewChanged(new DraftViewEvent(thisSheet,DraftViewEvent.DRAFT_SELECTED,selected));
                }
            });
        Vector drafts = parent.document.getDrafts();
        fireComments=false;//while creating the tabs, don't allow them to fire view events.
        for(int i=0;i<drafts.size();i++){
            tab t = createTab((Blaise_Draft)drafts.elementAt(i));
            
            if(i == selected){
				//t.setSelectedGraderIndex(selected_comment);
                t.setSelectedGraderIndex(t.current_index);
            }
        }
        fireComments=true;
        
        this.add(tab_pane, BorderLayout.CENTER);
        
        if(drafts.size() > 0) tab_pane.setSelectedIndex(selected);
        if(index != -1 && parent.mode == BlaiseSplitPane.GRADER) tab_pane.setForegroundAt(index, Color.RED);
        this.validate();
        
        //notify the listeners.
        if(debug)System.out.println("Event point 2");
        fireViewChanged(new DraftViewEvent(this,DraftViewEvent.VIEW_UPDATE));
    }
    
    /**
     * Titles longer than this will be truncated.
     */
    public static final int DRAFT_LIST_MAX_TITLE=31;
    
    /**
     * Gets a list of all of the drafts formatted in
     * "draft#" : "title" manner.  The array is organized in index order.
     */
    public String[] getDraftList(){
        Vector drafts=parent.document.getDrafts();
        String[] list=new String[drafts.size()];
        String title;
        int size;
        StringBuffer temp;
        int sub,odd;
        for(int i=0;i<list.length;i++){
            title=((Blaise_Draft)drafts.elementAt(i)).getTitle();
            size=title.length();
            if(size>DRAFT_LIST_MAX_TITLE){
                if(Constants.isOSX()){//OSX does text...text
                    odd=(size-DRAFT_LIST_MAX_TITLE+3);
                    sub=odd/2;
                    temp=new StringBuffer();
                    temp.append((i+1));
                    temp.append(": ");
                    temp.append(title.substring(0,sub));
                    temp.append("...");
                    if(odd%2!=0){//odd so add on one more ch.
                        sub++;
                    }
                    temp.append(title.substring((size-sub),size));
                    list[i]=temp.toString();
                }
                else{//windows does text...
                    title=title.substring(0,DRAFT_LIST_MAX_TITLE-3);
                    list[i]=(i+1)+": "+title+"...";
                }
            }
            else{
                list[i]=(i+1)+": "+title;
            }
        }
        return list;
    }
    
    /**
     * Returns a list of all of the graders names for the comments of the
     * selected draft.
     */
    public String[] getCommentList(){
        int sel=tab_pane.getSelectedIndex();
        if(sel<0){
            return new String[0];
        }
        return ((tab)tabs.elementAt(sel)).getCommentList();
    }
    
	//Returns the index of the selected draft
    public int getSelectedDraftIndex(){
        return tab_pane.getSelectedIndex();
    }
    
    /**
     * For the currently selected draft, are the scribbles visible
     * if there are no drafts, returns false.
     */
    public boolean areScribblesVisible(){
        int sel=tab_pane.getSelectedIndex();
        if(sel>=0){
            return ((tab)tabs.elementAt(sel)).areScribblesVisible();
        }
        else{//no drafts
            return false;
        }
    }
    
    /**
     * For the currently selected draft, set the scribbles visible
     */
    public void setScribblesVisible(boolean visible){
        int sel=tab_pane.getSelectedIndex();
        if(sel>=0){
            ((tab)tabs.elementAt(sel)).setScribblesVisible(visible);
        }
        else{//no drafts
            throw new IllegalStateException("No Drafts--cannot set scribbles visible");
        }
    }
    
    public int getSelectedCommentIndex(){
        int sel=tab_pane.getSelectedIndex();
        if(sel>=0){
            return ((tab)tabs.elementAt(sel)).getSelectedGraderIndex();
        }
        else{
            return -1;
        }
    }
    
    /**
     * Selects the specified tab.
     */
    public void selectDraft(int index){
        tab_pane.setSelectedIndex(index);
        if(debug)System.out.println("Event point 3");
        fireViewChanged(new DraftViewEvent(thisSheet,DraftViewEvent.DRAFT_SELECTED,index));
    }
    
    /**
        * Selects the specified comment.
     */
    public void selectComment(int index){
        int sel=tab_pane.getSelectedIndex();
        ((tab)tabs.elementAt(sel)).setSelectedGraderIndex(index);
    }
    
    /**
        * calls all registered listeners with the specified event.
     */
    private void fireViewChanged(DraftViewEvent e){
        //updateView recreates the tabs, and the selected one wants to fire
        //off a comment selected event.  This behavior isn't wanted so updateView
        //turns off comment fires while recreating the tabs...
        if(!fireComments && e.getChangeType()==e.COMMENT_SELECTED){
            return;
        }
        int size=listeners.size();
        for(int i=0;i<size;i++){
            ((DraftViewListener)listeners.elementAt(i)).draftViewChanged(e);
        }
    }
    
    private tab createTab(Blaise_Draft draft){
        tab current = new tab(draft, parent, this);
        tabs.add(current);
        tab_pane.add(draft.getDraftNumber(), current);
        return current;
    }
    
    /*public static void main(String[] argv){
        Blaise_XML xml = new Blaise_XML();
    Vector papers = xml.parse("D:\\Blaise\\data\\blaise_grade.kjm");
		//Vector papers = xml.parse("C:\\Archives\\Misc\\Blaise\\data\\blaise_grade.kjm");
    new BlaiseSplitPane(BlaiseSplitPane.GRADER, (Blaise_Paper)papers.elementAt(0));
    }*/
    
    
    class tab extends JPanel implements BlaiseCommentViewInterface, ComponentListener,java.beans.PropertyChangeListener{
        
        int current_index;
        comment current_comment;
        boolean scribble_visibility;
        boolean correction_visibility;
        String title;
        String turninDate;
        String original_body;
        JComboBox graders_box;
        //JCheckBox scribbles_check;
        //JCheckBox corrections_check;
        BlaiseCommentView text;
        JScrollBar bar;
        JScrollPane scroll;
        ExternalFileView efView;
        Blaise_Draft draft;
        BlaiseDraftView draftView;
        
        Vector comments;
        
        BlaiseSplitPane parent;
        JPanel tempPanel;
        BlaiseLayeredCommentPane layeredView;
        JPanel widthAchiever;
        JPanel heightAchiever;
        
		int originalSize = -1;
        int graders_index = -1;
        
        public tab(Blaise_Draft draft, BlaiseSplitPane parent, BlaiseDraftView draftView){
            
            this.parent = parent;
            this.draft = draft;
            this.draftView = draftView;
            
            title = draft.getTitle();
            turninDate = draft.getTurninDate();
            original_body = draft.getBody();
            text = new BlaiseCommentView(this);
            efView=new ExternalFileView(parent);
            comments = new Vector();
            
            graders_box = new JComboBox();
            graders_box.setEditable(false);
            
            current_index = 0;
            current_comment = null;
            Vector coms = draft.getComments();
            
            //See if this grader's comments are present
            for(int i=0;i<coms.size();i++){
                Blaise_Comment bComment = (Blaise_Comment)coms.elementAt(i);
                if(bComment.getGraderName().equals("My Marks")){
                    graders_index = i;
                    break;
                }
            }
            
            if(graders_index == -1){
                graders_box.addItem("Original Submission");
            }
            else{
                Blaise_Comment bComment = (Blaise_Comment)coms.elementAt(graders_index);
                comment current = new comment(bComment);
                comments.add(current);
                current_comment = current;
                graders_box.addItem(current.getBoxHeading());
                graders_box.addItem("Original Submission");
            }
            
            for(int i=0;i<coms.size();i++){
                if(i == graders_index)continue;
                Blaise_Comment bComment = (Blaise_Comment)coms.elementAt(i);
                comment current = new comment(bComment);
                
			//If a grading instance, hide hidden comments
			//**We're going to show all of them, as IsPublic isn't really around anymore**
                /*if(parent.mode == BlaiseSplitPane.GRADER || parent.mode == BlaiseSplitPane.VIEWER){
                    if(bComment.getIsPublic().equals("NO")){
					//Ensure comments by active grader are shown regardless
                        if(!bComment.getGraderName().equals("My Marks")){
                            continue;
                        }
                    }
                }*/
                comments.add(current);
                graders_box.addItem(current.getBoxHeading());
            }
            graders_box.setPreferredSize(new Dimension(200,30));
            graders_box.setMinimumSize(new Dimension(200,30));
            graders_box.setMaximumSize(new Dimension(200,30));
            graders_box.setSize(new Dimension(200,30));
            
            //Panel set-up
            this.setLayout(new BorderLayout());
            this.add(createHeaderPanel(), BorderLayout.NORTH);
            this.add(createBodyPanel(), BorderLayout.CENTER);
            
            final BlaiseDraftView bdv=draftView;
            graders_box.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e) {
					originalSize=-1;
                    current_index = graders_box.getSelectedIndex();
                    updateText();
                    if(debug)System.out.println("Event Point 4");
                    bdv.fireViewChanged(new DraftViewEvent(bdv,DraftViewEvent.COMMENT_SELECTED,current_index));
                }
            });
            
            updateText();
            
            parent.addComponentListener(this);
			parent.split.addPropertyChangeListener("dividerLocation",this);
        }
        
        public void setSelectedGraderIndex(int index){
		//index++;
            current_index = index;
            graders_box.setSelectedIndex(index);
            updateText();
        }
        
        public int getSelectedGraderIndex(){
            return current_index;
        }
        
        public String[] getCommentList(){
            
            String[] clist=new String[comments.size()];
            for(int i=0;i<clist.length;i++){
                clist[i]=((comment)comments.elementAt(i)).grader_name;
            }
            return clist;
        }
        
        private void updateText(){
            if((current_index != 0 && graders_index == -1) || (graders_index != -1 && current_index != 1)){
                text.isOriginal = false;
                
                
                if(current_index == 0){
                    current_comment = (comment)comments.elementAt(current_index);
                }
                else{
                    current_comment = (comment)comments.elementAt(current_index-1);
                }
                
                efView.uninstall(); //just incase.
                if(current_comment.external){
                    efView.install(text,current_comment.comment);
                }
                else{
                    text.setText(current_comment.body);
                }
                
				text.setDimensions(current_comment.aspectX, current_comment.aspectY);
				
                Dimension dimen = text.getSize();
                Dimension panelSize = parent.getSize();
                
			//If text was marked at a larger size, stretch
                if(dimen.getWidth() > panelSize.getWidth()){
                    scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    
                    Dimension wide_dimen = new Dimension((int)dimen.getWidth(),1);
                    widthAchiever.setPreferredSize(wide_dimen);
                    widthAchiever.setMaximumSize(wide_dimen);
                    widthAchiever.setMinimumSize(wide_dimen);
                    widthAchiever.setSize(wide_dimen);
                    
                    /*
                     System.out.println("-Text was marked at a LARGER size----------------");
                     System.out.println("text: "+dimen);
                     System.out.println("panel: "+this.getSize());
                     System.out.println("widthAchiever: "+widthAchiever.getSize());
                     System.out.println();
                     */
                }
			//Else If text was marked at a smaller sizw, use height achiever to shrink size
                else if(dimen.getWidth() < panelSize.getWidth()){
                    scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
                    
                    Dimension high_dimen = new Dimension(((int)panelSize.getWidth() - (int)dimen.getWidth()),1);
                    heightAchiever.setPreferredSize(high_dimen);
                    heightAchiever.setMaximumSize(high_dimen);
                    heightAchiever.setMinimumSize(high_dimen);
                    heightAchiever.setSize(high_dimen);
                    
                    /*
                     System.out.println("-Text was marked at a SMALLER size----------------");
                     System.out.println("text: "+dimen);
                     System.out.println("panel: "+this.getSize());
                     System.out.println("heightAchiever: "+heightAchiever.getSize());
                     System.out.println();
                     */
                }
                else{
                    scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
                }
				
				sizeGraded();
                
			//System.out.println("Size of the DraftView: "+dimen);
                
			//If comment has no scribbles or corrections, disable appropriate check box
                /*if(current_comment.hasScribbles()){
                    scribbles_check.setEnabled(true);
                }
                else{
                    scribbles_check.setEnabled(false);
                }
                
                if(current_comment.hasCorrections()){
                    corrections_check.setEnabled(true);
                }
                else{
                    corrections_check.setEnabled(false);
                }*/
                
                Vector notes=getNotes();
                int i,size=notes.size();
                for(i=0;i<size;i++){
                    ((Note)notes.elementAt(i)).display(layeredView,false);//causes the note to get added to the layeredPane
                }
            }
            else{
                if(draft.hasExternalBody()){
                    efView.uninstall();//just incase...
                    efView.install(text,draft);
                }
                else{
                    text.setText(original_body);
                    sizeOriginal();
                }
                text.isOriginal = true;
                Vector notes=getNotes();
                int i,size=notes.size();
                for(i=0;i<size;i++){
                    ((Note)notes.elementAt(i)).remove();//causes the note to get removed from the layered pane
                }
                current_comment = null;
                //scribbles_check.setEnabled(false);
                //corrections_check.setEnabled(false);
            }
            
            //this.revalidate();
        }
		
		int firstSizing=-1;
		
		void sizeOriginal(){
			if(!text.isOriginal){ return;}
			
			//Dimension panelSize = scroll.getViewport().getViewSize();
			Dimension panelSize = scroll.getSize();
			Dimension textD=text.getPreferredSize();
			Dimension parentSize = parent.getSize();
			int h;
			if(originalSize<0 || firstSizing<1){
				originalSize=textD.height;
			}
			h=(panelSize.height>originalSize)?panelSize.height:originalSize;
			//text.setDimensions(panelSize.width, textD.height);
			int deltaW;
			if(panelSize.width==scroll.getViewport().getViewSize().width){
				deltaW=0;
			}
			else{
				deltaW=(Constants.isOSX())?54:44;//platform dependent
			}
			text.setDimensions(parentSize.width-deltaW,h);
			if(firstSizing<1){
				firstSizing++;
			}
			
			System.out.println("-------panelSize:"+panelSize+" textD:"+textD+" wholePanelSize:"+scroll.getSize());
			
		}
		
		void sizeGraded(){
			if(text.isOriginal){ return;}
			//most of the work for size graded has already been done in updateText()
			//this method just makes sure it is long enough.
			Dimension panelSize = scroll.getSize();
			int h;
			h=(panelSize.height>current_comment.aspectY)?panelSize.height:current_comment.aspectY;
			text.setDimensions(current_comment.aspectX,h);
			System.out.println("----------panelSize:"+panelSize);
		}
        
        public void setScribblesVisible(boolean visible){
            scribble_visibility = visible;
            text.repaint();
        }
        
        private JPanel createHeaderPanel(){
            
            JPanel header = new JPanel(new BorderLayout());
            JToolBar menu = new JToolBar();
            JToolBar titleBar = new JToolBar();
            
            menu.setFloatable(false);
            menu.addSeparator();
            menu.add(graders_box);
            scribble_visibility = true;
            
            //Moved to menu bar item in the editor and grader instead.
            /*scribbles_check = new JCheckBox("Scribbles", true);
            scribble_visibility = true;
            scribbles_check.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e) {
                    setScribblesVisible(!scribbles_visibility);
                }
            });
            corrections_check = new JCheckBox("Corrections", true);
            correction_visibility = true;
            corrections_check.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e) {
                    correction_visibility = !correction_visibility;
                    text.repaint();
                }
            });*/
            
            //menu.add(scribbles_check);
            //menu.add(corrections_check);
            
            titleBar.setFloatable(false);
            titleBar.addSeparator();
            titleBar.add(new JLabel(title));
            
            header.add(menu, BorderLayout.SOUTH);
            header.add(titleBar, BorderLayout.NORTH);
            
            return header;
        }
        
        private JScrollPane createBodyPanel(){
            bar = new JScrollBar();
            bar.setOrientation(JScrollBar.VERTICAL);
            bar.addAdjustmentListener ( new AdjustmentListener() {
                public void adjustmentValueChanged(AdjustmentEvent e){
                    text.repaint();
                }
            });
            
            tempPanel = new JPanel(new BorderLayout());
            layeredView=new BlaiseLayeredCommentPane(text);
            tempPanel.add(layeredView, BorderLayout.CENTER);
            
            widthAchiever = new JPanel();
            widthAchiever.setBackground(Color.WHITE);
            heightAchiever = new JPanel();
            heightAchiever.setBackground(Color.WHITE);
            tempPanel.add(widthAchiever, BorderLayout.SOUTH);
            tempPanel.add(heightAchiever, BorderLayout.EAST);
            
            scroll = new JScrollPane(tempPanel,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            
        //scroll = new JScrollPane(text,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            scroll.setVerticalScrollBar(bar);
            
            return scroll;
        }
        
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// BlaiseCommentViewInterface Interface
//    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        public Vector getScribbles(){
            if(current_comment != null){
                return current_comment.scribbles;
            }
            else{
                return new Vector();
            }
        }
        public Vector getNotes(){
            if(current_comment != null){
                return current_comment.notes;
            }
            else{
                return new Vector();
            }
        }
        public Vector getCorrections(){
            if(current_comment != null){
                return current_comment.corrections;
            }
            else{
                return new Vector();
            }
        }
        public boolean areScribblesVisible(){
            return scribble_visibility;
        }
        public boolean areCorrectionsVisible(){
            return correction_visibility;
        }
        public Scribble getSelected(){
            return null;
        }
        
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //
    // ComponentListner Interface
    //
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        public void componentResized(ComponentEvent e) {
            updateText();
        }
        
        public void componentHidden(ComponentEvent e) {}
        public void componentMoved(ComponentEvent e) {}
        public void componentShown(ComponentEvent e) {
			sizeOriginal();
			sizeGraded();
		}
		
		public void propertyChange(java.beans.PropertyChangeEvent evt) {
			//System.out.println("event:"+evt);
			sizeOriginal();
			sizeGraded();
		}
    }
    
    class comment{
        
        String grader_name;
	//String assigned_grade;
        Vector scribbles;
        Vector corrections;
        Vector notes;
        String body=null;
        Blaise_Comment comment=null;
        boolean external;
        int aspectX;
        int aspectY;
        
        public comment(Blaise_Comment comment){
            
            grader_name = comment.getGraderName();
		//assigned_grade = comment.getAssignedGrade();
            scribbles = comment.getScribbles();
            corrections = comment.getCorrections();
            notes = comment.getNotes();
            if(comment.hasExternalBody()){
                this.comment=comment;
                external=true;
            }
            else{
                body = comment.getBody();
                external=false;
            }
            aspectX = (int)Double.parseDouble(comment.getAspectX());
            aspectY = (int)Double.parseDouble(comment.getAspectY());
        }
        
        public String getBoxHeading(){
		//if(assigned_grade.equals("")){
            return grader_name;
            /*}
            else{
                return (grader_name+" : "+assigned_grade);
            }*/
        }
        
        public boolean hasScribbles(){
            if(scribbles.size() == 0){
                return false;
            }
            else{
                return true;
            }
        }
        public boolean hasCorrections(){
            if(corrections.size() == 0){
                return false;
            }
            else{
                return true;
            }
        }
        public boolean hasNotes(){
            if(notes.size()==0){
                return false;
            }
            else{
                return true;
            }
        }
    }
    
}