//LOC:57
package edu.gwu.blaise.version2.util;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;

import java.util.*;

/**
 * The JTextPane that displays a document
 *
 * @author Sean T. Hanlon
 * @author Rahul Simha
 * @author Rizwan Sattar
 * @author Michael Gaiman
 */
public class BlaiseJTextPane extends JTextPane implements DocumentListener{
    
    private boolean modified = false;
    
    private BlaiseJTextPane thisPane;
    
    private HTMLEditorKit htmlKit;
    
    private String ctext;
    private int start;
    private int end;
    private AttributeSet[] ctextAttribute;
    private HTML_Parser parser;
    
    public BlaiseJTextPane(){
        this("");
    }
    
    public BlaiseJTextPane(String body){
        super();
        thisPane = this;
        parser = new HTML_Parser(HTML_Parser.PASTE);
        ctext = "";
        
		//Create the HTML Editor Kit and set the style sheet
        htmlKit = new HTMLEditorKit();
        htmlKit.setStyleSheet(htmlKit.getStyleSheet());
        this.setEditorKit(htmlKit);	//assign editor kit to the textpane
        this.getDocument().putProperty("IgnoreCharsetDirective", Boolean.TRUE);
        this.setEditable(true);
        
        try{
			//Create buffered reader for text and read into textpane
            BufferedReader in = new BufferedReader(new StringReader(body));
            htmlKit.read(in, this.getDocument(),0);
        }catch(Exception e){}
        
        //this.addKeyListener(this);
        getDocument().addDocumentListener(this);
		
		//this is a fix for a subtle spelling bug where the misspelled text
		//turns invisible.
		this.setSelectedTextColor(Color.black);
    }
    
    public boolean isEmpty(){
        Document doc=getDocument();
        int length=doc.getLength();
        return (length==0);
    }
    
	// Override cut
    public void cut (){
        ctext = getSelectedText();
        
        start = getSelectionStart();
        end = getSelectionEnd();
        setAttributeArray();
        
        setSelectionStart(start);
        setSelectionEnd(end);
        
        super.cut ();
    }
    
    public void copy () {
        ctext = getSelectedText();
        start = getSelectionStart();
        end = getSelectionEnd();
        setAttributeArray();
        
        setSelectionStart(start);
        setSelectionEnd(end);
        
        super.copy ();
    }
    
    private void setAttributeArray(){
        ctextAttribute = new AttributeSet[end-start+1];
        int temp = start;
        for(int i=0;i<end-start+1;i++){
            setCaretPosition(temp);
            ctextAttribute[i] = getCharacterAttributes();
            temp++;
        }
    }
    
    public void paste () {
        System.out.println("BlaiseJTextPane paste command called.");
        if (ctext != null) {
            try {
                Document doc = this.getDocument();
                int count = end-start;
                int paste_local = getSelectionStart();
                int other_end = getSelectionEnd();
                
                if(other_end < paste_local){
                    paste_local = other_end;
                }
                
			//Delete any selection
                this.replaceSelection("");
                
                for(int i=0;i<count;i++){
                    doc.insertString(paste_local, ctext.substring(i,i+1), ctextAttribute[i]);
                    paste_local++;
                }
            }
            catch (BadLocationException e) {
			//e.printStackTrace();
            }
        }
    }
    
    /**
     * Return a JTextPane from the given String with an HTML Editor Kit
     *
     * @param text The text to be displayed in the JTextPane
     * @return <code>JTextPane</code> representation of the text
     */
    public static BlaiseJTextPane createContent(String text) {
        
        BlaiseJTextPane messageView = null;
        
        try {
            BlaiseJTextPane tp = new BlaiseJTextPane(text);
            messageView = tp;
            
        }
        catch (Exception e) {}
        
		//top,left,bottom,right margin defaults
        messageView.setMargin(new Insets(15,15,15,15));
        
        return messageView;
    }
    
    public boolean isModified(){
        return modified;
    }
    
    public  void setModified(boolean m){
        if(Constants.isOSX() && m!=modified){
            getRootPane().putClientProperty("windowModified", (m)?Boolean.TRUE:Boolean.FALSE);
        }
        modified=m;
    }
    
    public void changedUpdate(DocumentEvent e){
        setModified(true);
    }
    public void insertUpdate(DocumentEvent e){
        setModified(true);
    }
    public void removeUpdate(DocumentEvent e){
        setModified(true);
    }
}