package edu.gwu.blaise.version2.util;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import edu.gwu.blaise.version2.documents.Note;
import java.util.*;
import java.beans.PropertyVetoException;
/**
 * BlaiseLayeredCommentPane is the parent of BlaiseCommentView.
 * Putting the BlaiseCommentView in a layered pane allows the display
 * of Notes on a layer higher than the document and scribbles.
 * In addition, BlaiseLayeredCommentPane implements NoteControllerInterface
 * so manage the interaction between Notes.
 * @author Michael Gaiman
 * @version 1 10/24/2004
 */
public class BlaiseLayeredCommentPane extends JLayeredPane implements NoteControllerInterface{
    private BlaiseCommentView bcv;
    private Vector openFrames;
    private Vector allFrames;
    private Vector selectedIcons;
    private Hashtable icons2frames;
    private EditModeManager emm;

    public BlaiseLayeredCommentPane(BlaiseCommentView bcv){
        this(bcv,null);
    }
    
    public BlaiseLayeredCommentPane(BlaiseCommentView bcv,EditModeManager emm){
        this.bcv=bcv;
        this.emm=emm;
		
        add(bcv, JLayeredPane.DEFAULT_LAYER,0);
        openFrames=new Vector();
        allFrames=new Vector();
        selectedIcons=new Vector();
        icons2frames=new Hashtable();
    }
    
    /**
     * To better represent the size, if possible return the comment view's
     * preferred size.
     */
    public Dimension getPreferredSize(){
        if(bcv==null){
            return super.getPreferredSize();
        }
        else{
			Dimension size=bcv.getPreferredSize();
			System.out.println("getPreferredSize:"+size);
			bcv.setSize(size);
            return size;
        }
    }
    
    /**
     * To better represent the size, if possible return the comment view's
     * minimum size.
     */
    public Dimension getMinimumSize(){
        if(bcv==null){
            return super.getMinimumSize();
        }
        else{
            return bcv.getMinimumSize();
        }
    }
    
    /**
     * To better represent the size, if possible return the comment view's
     * maximum size.
     */
    public Dimension getMaximumSize(){
        if(bcv==null){
            return super.getMaximumSize();
        }
        else{
            return bcv.getMaximumSize();
        }
    }
	
	public void setSize(Dimension d){
		System.out.println("BLCP.setSize:"+d);
		super.setSize(d);
	}
	
	public void setSize(int x, int y){
		System.out.println("BLCP.setSize:["+x+", "+y+"]");
		super.setSize(x,y);
	}
	
	public void setBounds(Rectangle r){
		System.out.println("BLCP.setBounds:"+r);
		super.setBounds(r);
	}
	
	public void setBounds(int x,int y,int z, int q){
		System.out.println("BLCP.setBounds:["+x+" "+y+" "+z+" "+q+"]");
		if(bcv!=null){
			bcv.setBounds(x,y,z,q);
		}
		super.setBounds(x,y,z,q);
	}
    
    /**
     * Returns this CommentPane's CommentView.
     */
    public BlaiseCommentView getBCV(){
        return bcv;
    }
        
    /**
     * adds the specified NoteIcon to the icon layer.
     */
    private void addNoteIcon(final Note.NoteIcon icon){
        this.add(icon, JLayeredPane.PALETTE_LAYER,0);//on top of default layer.
        MouseInputAdapter noteMA=new MouseInputAdapter(){
            private int xoffset,yoffset;
            
            private void selectThis(){
                deselectAllIcons();
                emm.deselectAll();
                selectedIcons.addElement(icon);
                icon.select();
            }
            
            public void mouseClicked(MouseEvent event){
                int count=event.getClickCount();
                if(count==1){
                    if(emm!=null && emm.getEditMode()){
                        selectThis();
                    }
                }
                else if(count==2){
                    Note.NoteFrame f=null;
                    f=(Note.NoteFrame)icons2frames.get(icon);
                    if(f.isVisible()){
                        selectedAndToFront(f);
                    }
                    else{
                        f.show();
                    }
                }
            }
            public void mousePressed(MouseEvent event){
                if(emm!=null && emm.getEditMode()){
                    Point p=event.getPoint();
                    xoffset=p.x;
                    yoffset=p.y;
                    selectThis();
                }
            }
            public void mouseDragged(MouseEvent event){
                if(emm!=null && emm.getEditMode()){
                    Point p=event.getPoint();
                    
                    p.translate(icon.getX()-xoffset,icon.getY()-yoffset);
                    icon.setLocation(p);
                }
            }
        };
        icon.addMouseListener(noteMA);
        icon.addMouseMotionListener(noteMA);
    }
    
    /**
     * adds the specified NoteFrame to the note frame layer.
     */
    private void addFrameIcon(Note.NoteFrame frame){
        this.add(frame,JLayeredPane.MODAL_LAYER,0);//infront.
    }
    
    
    //NOTECONTROLLERINTERFACE METHODS:
    /**
     * Called by Note to display its note frame and icon in the layered pane.
     */
    public void addNoteToPane(Note.NoteFrame frame, Note.NoteIcon icon){
        System.out.println("Adding note to pane...");
        addNoteIcon(icon);
        addFrameIcon(frame);
        //keep a reference to the frame around for openAll()
        allFrames.addElement(frame);
        icons2frames.put(icon,frame);
        revalidate();
        System.out.println("icon:"+icon);
    }
    
    /**
     * Called by Note to stop its note frame and icon from being displayed
     * in the layered pane.
     */
    public void removeNoteFromPane(Note.NoteFrame frame, Note.NoteIcon icon){
        this.remove(icon);
        this.remove(frame);
        allFrames.removeElement(frame);
        openFrames.removeElement(frame);//if open...
        icons2frames.remove(icon);
    }
    
    public void removeSelectedNotes(){
        Note.NoteIcon icon=null;
        while(!selectedIcons.isEmpty()){
            icon=(Note.NoteIcon)selectedIcons.elementAt(0);
            removeNoteFromPane((Note.NoteFrame)icons2frames.get(icon),icon);
            selectedIcons.removeElement(icon);
        }
    }
    
    /**
     * Closes all open Notes
     */
    public void closeAll(){
        Iterator i=openFrames.iterator();
        Note.NoteFrame f;
        while(i.hasNext()){
            f=(Note.NoteFrame)i.next();
            f.hide();
            i.remove();
        }
    }
    
    /**
     * Opens all Notes
     */
    public void openAll(){
        int i,size=allFrames.size();
        for(i=0;i<size;i++){
            ((Note.NoteFrame)allFrames.elementAt(i)).show();
        }
    }
    
    /**
     * Draws a selection rectangle around all note icons.
     */
    public void selectAllIcons(){
        Enumeration e=icons2frames.keys();
        Note.NoteIcon i=null;
        for(;e.hasMoreElements();i=(Note.NoteIcon)e.nextElement()){
            selectedIcons.addElement(i);
            i.select();
        }
    }
    
    /**
     * Returns a Vector containing the Note objects of all selected Icons.
     */
    public Vector getSelectedIconNotes(){
        Vector v=new Vector();
        int size=selectedIcons.size();
        for(int i=0;i<size;i++){
            v.addElement(((Note.NoteIcon)selectedIcons.elementAt(i)).getParentNote());
        }
        return v;
    }
    
    /**
     * Stops drawing a selection rectangle around all note icons.
     */
    public void deselectAllIcons(){
        int i,size=selectedIcons.size();
        if(size>0){
            for(i=0;i<size;i++){
                ((Note.NoteIcon)selectedIcons.elementAt(i)).deselect();
            }
            selectedIcons.clear();
        }
    }
    
    /**
     * Selects the caller Note while deselecting all
     * other open notes.
     */
    public void selectedAndToFront(Note.NoteFrame caller){
        try{
            Note.NoteFrame f;
            int size=openFrames.size(), i;
            for(i=0;i<size;++i){
                f=(Note.NoteFrame)openFrames.elementAt(i);
                if(f==caller){
                    continue;//do nothing to self.
                }
                f.setSelected(false);
            }
            caller.setSelected(true);
            caller.toFront();
        }catch(PropertyVetoException pve){
            System.out.println(pve);
        } 
    }
    
    /**
     * Called to notify the controller that the user opened
     * caller Note.
     */
    public void noteOpened(Note.NoteFrame caller){
        openFrames.addElement(caller);
    }
    
    /**
     * Called to notify the controller that the user closed
     * caller Note.
     */
    public void noteClosed(Note.NoteFrame caller){
        openFrames.remove(caller);
    }
    
    /**
     * Sets any selected Notes to have the supplied color as their background.
     */
    public void setSelectedNotesToColor(Color c){
        Vector v=getSelectedIconNotes();
        int size=v.size();
        for(int i=0;i<size;i++){
            ((Note)v.elementAt(i)).setNoteColor(c);
        }
    }
}
