//LOC:277
package edu.gwu.blaise.version2.util;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import java.awt.geom.*;

import java.util.*;
import edu.gwu.blaise.version2.documents.*;

//A specific graders marks
class GradeSet{

	private Vector scribbles;		//Vector of Scribble
	private Vector corrections;		//Vector of CorrectionSet
	private boolean visible;

	public GradeSet(Vector scrib, Vector correct){
		scribbles = scrib;
		corrections = correct;

		visible = true;
	}

	//Accessors and mutators
	public Vector getScribbles(){
		return scribbles;
	}

	public Vector getCorrections(){
		return corrections;
	}

	//Visibility control
	public void flipVisible(){
		visible = !visible;
	}

	public void setVisible(boolean value){
		visible = value;
	}

	public boolean isVisible(){
		return visible;
	}
}

public class BlaisePaperViewerPane extends JPanel {

	Vector grades;			//local GradeSets
	Vector draft_grades;	//actual Blaise_Comment objects

	//Swing Components
	JScrollPane scroll;
	JScrollBar bar;

	JTextPane text;
	boolean edit;

	BlaisePaperViewerPane thisSheet;	//this object

	public BlaisePaperViewerPane(Blaise_Draft draft){

		this.setLayout(new BorderLayout());
		thisSheet = this;
		draft_grades = new Vector();
		grades = new Vector();

		//If the paper has grades
		if(draft.hasComments()){
			draft_grades = draft.getComments();

			//Extract each unique set of graders' markings and store in a GradeSet object
			for(int i=0;i<draft_grades.size();i++){
				Blaise_Comment current = (Blaise_Comment)draft_grades.elementAt(i);
				GradeSet info = new GradeSet(current.getScribbles(), current.getCorrections());
				grades.addElement(info);
			}
		}

		//Display body of draft
		text = BlaiseJTextPane.createContent(draft.getBody());
		text.getCaret().deinstall(text);
		text.setEditable(false);

		/////////////////////////
		//Set aspect here
		/////////////////////////

		this.add(text, BorderLayout.CENTER);

	}

	public void paint(Graphics g){
		super.paint(g);

		//draw each guesture
		for(int i=0;i<grades.size();i++){
			GradeSet at = (GradeSet)grades.elementAt(i);
			//If the GradeSet is visible, draw markings
			if(at.isVisible()){
				Vector scribs = at.getScribbles();
				Vector correct = at.getCorrections();
				Graphics2D g2 = (Graphics2D)text.getGraphics();

				try{
					//draw each scribble
					for(int j=0;j<scribs.size();j++){
						Scribble scrib = (Scribble)scribs.elementAt(j);
						Rectangle position = text.modelToView(scrib.getOffset());
						scrib.fixPosition(position.getX() + scrib.getXDistanceFromOffset(), position.getY());
						g2.setPaint(scrib.getColor());
						g2.setStroke(new BasicStroke(scrib.getWidth()));
						for(int k=0;k<scrib.size();k++){
							ScribbleLine line = scrib.getLine(k);
							g2.draw(line);
						}
					}

					//draw correction sets
					for(int j=0;j<correct.size();j++){
						CorrectionSet set = (CorrectionSet)correct.elementAt(j);
						Vector starts = set.getStarts();
						Vector ends = set.getEnds();
						g2.setPaint(set.getColor());

						for(int k=0;k<starts.size();k++){
							Rectangle Lposition = text.modelToView(((Integer)starts.elementAt(k)).intValue());
							Rectangle Rposition = text.modelToView(((Integer)ends.elementAt(k)).intValue());

							Rectangle2D.Double rect = new Rectangle2D.Double();
							rect.setRect(Lposition.getX(), Lposition.getY(),(Rposition.getX() - Lposition.getX() + 1),Lposition.getHeight());
							g2.setStroke(new BasicStroke(1));
							g2.draw(rect);
						}
					}
				}
				catch(BadLocationException e){
					System.out.println("DisplayPaper - BadNumberException: "+e);
				}
			}
		}
	}

/*
	//Provides a mechanism for the viewer to toggle the markings of the different graders
	private void displayInfo(){

		final JFrame f = new JFrame();
		JPanel overall = new JPanel(new BorderLayout());

		JPanel all = new JPanel(new GridLayout(paper_grades.size(), 1));

		JLabel info = new JLabel("Grade Summary - Please select what markings you want to view:", JLabel.CENTER);
		overall.add(info, BorderLayout.NORTH);

		String value = "";
		JToggleButton but = new JToggleButton("");
		final String[] graders = new String[paper_grades.size()];

		for(int i=0;i<paper_grades.size();i++){
			PGS_Grade at = (PGS_Grade)paper_grades.elementAt(i);
			//Initialize label of button
			value = ("<html><body>"+at.getUser()+" -- "+at.getGrade());
			value += "<br>Spelling - Green";
			value += "<br>Delete - Red";
			value += "<br>Capitalize - Purple";
			value += "</body></html>";

			graders[i] = at.getUser();
			but = new JToggleButton(value, ((GradeSet)grades.elementAt(i)).isVisible());
			but.setBorder(BorderFactory.createLineBorder(Color.black));
			//Each time the button is pressed, flip the visibility of the GradeSet
			but.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent action){
					for(int j=0;j<graders.length;j++){
						if((action.getActionCommand()).startsWith("<html><body>"+graders[j])){
							((GradeSet)grades.elementAt(j)).flipVisible();
						}
					}
					thisSheet.repaint();
				}
			});

			all.add(but);
		}

		//add ok button
		JButton ok = new JButton("Ok");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent action){
				f.dispose();
			}
		});

		JPanel button = new JPanel(new FlowLayout());
		button.add(ok);

		overall.add(button, BorderLayout.SOUTH);

		JScrollPane scroll = new JScrollPane(all, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		overall.add(scroll, BorderLayout.CENTER);


		f.getContentPane().add(overall);

		Dimension parentSize = this.getSize();
		Point start = this.getLocation();

		f.setLocation((int)(parentSize.getWidth() + start.getX()), (int)start.getY());

		f.setSize(new Dimension(400,400));
		f.setVisible(true);
	}
*/
}