package edu.gwu.blaise.version2.util;

import java.util.*;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import edu.gwu.blaise.version2.client.*;
import edu.gwu.blaise.version2.grader.*;
import edu.gwu.blaise.version2.documents.*;

public class BlaiseSplitPane extends JFrame {

	private BlaiseSplitPane thisFrame;
	private int my_minimum_width;
	public JSplitPane split;
	private JPanel bottom_pane;
	private JMenuBar jmenubar;
	private JLabel statusLabel;
	private JPanel statusPane;

	private static int MIN_EDITOR_WIDTH = 500;
	private static int MIN_GRADER_WIDTH = 500;
	public static final byte EDITOR = 0;
	public static final byte GRADER = 1;
	public static final byte VIEWER = 2;

	public /*static*/ Blaise_Paper document;
	public /*static*/ Blaise_Draft current_draft;
	public /*static*/ BlaiseDraftView draft_pane;
	public /*static*/ File saveFile;
	public int selected_index = -1;
	public byte mode;
	public boolean standalone = false;

	public String graderID = "";
	public String graderName = "";
	public String universityID = "";

	public PortalToGraderInterface PtGI = null;
	
	public BlaiseSplitPane(byte type, Blaise_Paper paper){
		this(type, paper, false);
	}

	public BlaiseSplitPane(byte type, Blaise_Paper paper, boolean onlyGr, PortalToGraderInterface pti) {
		// Set up a reference to the PortalToGraderInterface
		this(type, paper, onlyGr);
		this.PtGI = pti;
	}

	public BlaiseSplitPane(byte type, Blaise_Paper paper, boolean onlyGr){
		thisFrame = this;
		document = paper;

		standalone = onlyGr;

		this.setIconImage(new ImageIcon(Constants.getIconPath()+"icon.gif").getImage());

		mode = type;
		split = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);
		draft_pane = new BlaiseDraftView(this);
		bottom_pane = new JPanel();
		current_draft = null;
		
		// Initialize the minimum window width to satisfy the compiler
		my_minimum_width = 100;
		if(type == EDITOR){
			bottom_pane = new BlaiseEditorPane(this);
			my_minimum_width = MIN_EDITOR_WIDTH;
			jmenubar = ((BlaiseEditorPane)bottom_pane).getMenuBar();
		}
		else if(type == GRADER){
			bottom_pane = new BlaiseGradingPane(this, standalone);
			my_minimum_width = MIN_GRADER_WIDTH;
			jmenubar = ((BlaiseGradingPane)bottom_pane).getMenuBar();

			// Add a status bar to the bottom of the window
			this.getContentPane().add(initializeStatusBar(), BorderLayout.SOUTH);
			if(!paper.isEmpty()){
				this.setTitle("Blaise: No Draft Selected For Marking");
			}
			else{
				this.setTitle("Blaise: No Paper Selected For Marking");
			}
		}
		/*
		else if(type == VIEWER){
			bottom_pane = new JPanel();

			JMenuItem item = new JMenuItem("Exit");
			item.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent rv){
					thisFrame.dispose();
				}
			});

			jmenubar = new JMenuBar();
			JMenu menu = new JMenu("File");
			menu.add(item);
			jmenubar.add(menu);

			this.setTitle("Blaise: Paper View");
		}
		 */

		//this.getContentPane().add(jmenubar, BorderLayout.NORTH);
                setJMenuBar(jmenubar);

		//Add component listener to ensure that the minimum width is never exceeded
		this.addComponentListener(
			new java.awt.event.ComponentAdapter(){
				public void componentResized(ComponentEvent event) {
					if (thisFrame.getWidth() < my_minimum_width) {
						thisFrame.setSize(my_minimum_width, thisFrame.getHeight());
					}
				}
			}
		);

		if(type == EDITOR || type == GRADER){
			split.setTopComponent(draft_pane);
			split.setBottomComponent(bottom_pane);
			split.setOneTouchExpandable(true);

			this.getContentPane().add(split);
		}
		else{
			this.getContentPane().add(draft_pane);
		}

                //Sets the size for the first time this is ran.  After that,
                //the WindowOpener opening saved stuff takes over.
                
		//get properties of system and set size accordingly
                
		//this.setSize(monitor.width,monitor.height-100);
		Toolkit kit = Toolkit.getDefaultToolkit();
		Dimension monitor = kit.getScreenSize();
		int dpi = kit.getScreenResolution();
		int wide = 600;
		if(dpi < 90) wide = 700;
		if(dpi < 70) wide = 800;

		this.setSize(wide, (int)(monitor.getHeight()-100));

		if(split != null) split.setDividerLocation((monitor.height/2)-150);
		
                //open the Blaise_Paper properly...create the dir. structure.
                if(type==EDITOR){
                    BlaiseFile bf=new BlaiseFile(document,new File(Constants.getTemplate()),new ClientFileHandler());
                    try{
                        bf.openPaper();
                    }catch(IOException e){
                        System.out.println(e +" (template is: '"+Constants.getTemplate()+"')");
                    }
                }
                WindowOpener.open(this,"BlaiseSplitPane");
	}

	public void updateDraftsWindow(){
		draft_pane.updateView(selected_index);
		//a stupid hack that makes comments in
		//the drafts window take up the whole space
		Dimension d=this.getSize();
		d.height+=1;
		this.setSize(d);
		d.height-=1;
		this.setSize(d);
	}

	public boolean isSameDraft(){
		int temp_selected_index = draft_pane.getSelectedDraftIndex();
		if(temp_selected_index == selected_index){
			return true;
		}
		else{
			return false;
		}
	}

	public void markDraft(){

		selected_index = draft_pane.getSelectedDraftIndex();
		if(selected_index >= 0){
			Blaise_Draft selected_draft = document.getDraft(selected_index);
			if(current_draft == null || !selected_draft.equals(current_draft)){
				current_draft = selected_draft;
				updateDraftsWindow();

				split.remove(bottom_pane);
				bottom_pane = new BlaiseGradingPane(this);

				this.getContentPane().remove(jmenubar);
				jmenubar = ((BlaiseGradingPane)bottom_pane).getMenuBar();
				this.getContentPane().add(jmenubar, BorderLayout.NORTH);

				split.setBottomComponent(bottom_pane);
				split.setDividerLocation((this.getHeight())/2);
				split.validate();

				this.setTitle("Blaise: "+current_draft.getTitle());
				this.validate();
			}
		}
	}

	public void noActiveDraft(){

		current_draft = null;
		split.remove(bottom_pane);
		bottom_pane = new BlaiseGradingPane(this);

		//this.getContentPane().remove(jmenubar);
		//jmenubar = ((BlaiseGradingPane)bottom_pane).getMenuBar();
		//this.getContentPane().add(jmenubar, BorderLayout.NORTH);

		
		split.setBottomComponent(bottom_pane);
		split.setDividerLocation(this.getHeight()/3);
		split.validate();

		selected_index = -1;
		updateDraftsWindow();

		this.setTitle("No Draft Selected For Marking");
		this.validate();
	}

	public JPanel initializeStatusBar(){

		statusLabel = new JLabel("  No Draft Selected For Marking");

		statusPane = new JPanel(new BorderLayout());
		statusPane.setSize(this.getSize());
		statusPane.add(statusLabel, BorderLayout.WEST);
		statusPane.setBorder(BorderFactory.createEtchedBorder());

		return statusPane;
	}

	public void setStatus(String new_label){
		statusLabel.setText(new_label);
		this.validate();
	}

	public void done(){
		if(standalone){
			System.exit(0);
		}
		else{
			this.dispose();
		}
	}
        
        /**
         * Added by mgaiman for mac port.  Gives access to bottom_panel.
         */
        public JPanel getBottom(){
            return bottom_pane;
        }
        
        /**
         * Added by mgaiman for mac port.  Gives access to mode, so that the type of bottom_panel can be determined.
         */
        public byte getMode(){
            return mode;
        }

	public static void main(String[] argv){
		//new BlaiseSplitPane(BlaiseSplitPane.GRADER, new Blaise_Paper());

		Blaise_XML xml = new Blaise_XML();
		Vector papers = xml.parse("D:\\Blaise\\data\\blaise_grade.kjm");
		//Vector papers = xml.parse("C:\\Archives\\Misc\\Blaise\\data\\blaise_grade.kjm");
		new BlaiseSplitPane(BlaiseSplitPane.GRADER, (Blaise_Paper)papers.elementAt(0));

	}
}