//LOC:271
package edu.gwu.blaise.version2.util;

import java.io.*;
import java.util.*;
import edu.gwu.blaise.version2.transmission.TransmissionConstants;

/**
 * This is the constants file that reads system parameters from a "blaise.props" file.
 *
 * @author Sean T. Hanlon
 * @author Michael R. Gaiman
 */
public class Constants {

    public static String file = "blaise.props";

    private static Properties p;
    private static String version = "";
    private static boolean initialized = false;

    private static String gestureFile = "";

    private static String userDataDirectory = "";
    private static String systemDataDirectory = "";
    private static String iconDirectory = "";
    private static String clientPaperDirectory = "";
    private static String graderPaperDirectory = "";
    
    private static String paperStoreBase = "";
    
    private static String cachedLoginStore = "";
    
    private static String dictionaryDir = "";
    private static String dictionaryFile = "";
    private static String phonetFile = "";
    
    private static String universityID= "1";//default.
    
    private static String windowOpenerFile = "";
    
    //they both default to transmission constants stuff (in parse if needed)
    private static String writerAddress = "";
    private static String graderAddress = "";
    
    private static String template = "";
    
    private static String propUser = "";

    private static int splashDelay = 2500;
    
    private static final boolean isOSX=(System.getProperty("mrj.version") != null);
    
    private static boolean windowsKnown=false;
    private static boolean isWindows;

    private static String gestureDTD = "<!ELEMENT GESTURE_FILE (GESTURE*)><!ELEMENT GESTURE (name, sample)><!ELEMENT name (#PCDATA)><!ELEMENT sample (#PCDATA)>";
    //private static String[] depts = {"","ACCY", "AH", "AMST", "ANAT","ANTH", "APSC", "ARAB", "ARTH","ASTRO", "BADM", "BIOC", "BISC", "BMSC", "CCAS", "CE", "CHEM", "CHIN", "CLAS", "CNSL", "COMM","CSCI", "ECE", "ECON", "EDUC", "EES", "EFL", "EMDA", "EMED", "EMSE", "ENGL", "EXSA", "EXSC", "FA", "FILM", "FINA", "FORS", "FREN", "GEOB", "GEOG", "GER", "GNET", "GREK", "HCS", "HDEV", "HEBR", "HIST", "HMN", "HMSC", "HMSR", "HOMP", "HONR", "HRD", "HSCI", "HSMP", "IAFF", "IBUS", "IMMU", "ITAL", "JAPN", "JOUR", "JUST", "KOR", "LATN", "LING", "MAE", "MATH", "MBAD", "MGT", "MICR", "MKTG", "MSTD", "MUS", "NEUS", "NSC", "ONCO", "PAD", "PATH", "PCM", "PHAR", "PHIL", "PHHS", "PHYL", "PHYS", "PMGT", "PORT", "PPOL", "PPSY", "PSC", "PSTD", "PSYC", "PUBH", "RAD", "REL", "ROM", "SEAS", "SLAV", "SLP", "SMPA", "SMPP", "SOC", "SOCL", "SPAN", "SPED", "SPHR", "STAT", "TRED", "TRDA", "TSTD", "UNIV", "VIET", "WLP","WSTU" };

    private static int meshSize = 0;

    /**
     * Constructor
     *
     * @param propsFile The props file to be parsed
     */
    public Constants(String propsFile){
        parsePropsFile(propsFile);
    }
    
    /**
     * @return <code>true</code> if currently running on MacOSX.
     */
    public static boolean isOSX(){
        return isOSX;
    }
    
    public static boolean isWindows(){
        if(!windowsKnown){
            String name=System.getProperty("os.name").toLowerCase();
            isWindows=(name.indexOf("windows")>=0);
            windowsKnown=true;
        }
        return isWindows;
    }

    /**
     * Gets the directory of Blaise System data
     *
     * @return <code>String</code> The data directory path
     */
    public static String getSystemDirectory(){
        parseIfNeeded();
        return systemDataDirectory;
    }

    /**
     * Gets the directory of Blaise System data
     *
     * @return <code>String</code> The data directory path
     */
    public static String getUserDirectory(){
        parseIfNeeded();
        return userDataDirectory;
    }

    public static String getVersion(){
        parseIfNeeded();
        return version;
    }
    
    public static String getClientPaperDirectory(){
        parseIfNeeded();
        return clientPaperDirectory;
    }
    
    public static String getGraderPaperDirectory(){
        parseIfNeeded();
        return graderPaperDirectory;
    }
    
    /**
     * The template it the filename for the client's template file.
     * The template file is a barebones bls document that get's opened to create
     * a starting directory structure.
     */
    public static String getTemplate(){
        parseIfNeeded();
        return template;
    }

    /**
     * The path and first part of the file name for paper store properties
     * files.  The second half is <graderID>.props.
     */
    public static String getPaperStoreBase(){
        parseIfNeeded();
        return paperStoreBase;
    }
    
    /**
     * The path and filename to the cached file store file.
     */
    public static String getCachedLoginStore(){
        parseIfNeeded();
        return cachedLoginStore;
    }
    
    public static String getDictionaryDir(){
        parseIfNeeded();
        return dictionaryDir;
    }

    /**
     * Gets the dictionary file
     *
     * @return <code>String</code> The dictionary file path
     */
    public static String getDictionaryFile(){
        parseIfNeeded();
        return dictionaryFile;
    }
    
    public static String getPhonetFile(){
        parseIfNeeded();
        return phonetFile;
    }
    
    public static String getUniversityID(){
        parseIfNeeded();
        return universityID;
    }

    /**
     * Gets the gesture DTD
     *
     * @return <code>String</code> The gesture DTD
     */
    public static String getGestureDTD(){
        return gestureDTD;
    }

    /**
     * Gets the gesture file
     *
     * @return <code>String</code> The gesture file path
     */
    public static String getGestureFile(){
        parseIfNeeded();
        return gestureFile;
    }

    /**
     * Gets the icon directory
     *
     * @return <code>String</code> The icon directory file path
     */
    public static String getIconPath(){
        parseIfNeeded();
        return iconDirectory;
    }

    /**
     * Gets the delay to show the splash screens
     *
     * @return <code>int</code> The delay for the splash screens
     */
    public static int getDelay(){
        parseIfNeeded();
        return splashDelay;
    }

    /**
     * Gets the mesh size
     *
     * @return <code>int</code> The mesh size
     */
    public static int getMeshSize(){
        parseIfNeeded();
        return meshSize;
    }
    
    public static String getWindowOpenerFile(){
        parseIfNeeded();
        return windowOpenerFile;
    }
    
    /**
     * The address of the pass through for the writer servlet.
     */
    public static String getWriterAddress(){
        parseIfNeeded();
        return writerAddress;
    }
    
    /**
     * The address of the pass through for the grader servlet.
     */
    public static String getGraderAddress(){
        parseIfNeeded();
        return graderAddress;
    }
    
    public static String getUser(){
        parseIfNeeded();
        return propUser;
    }
    
    /**
     * Checks to see if the Constants have been initialized,
     * and parse the prop file them if not.
     */
    private static void parseIfNeeded(){
        if(!initialized){
            initialized = true;
            parsePropsFile(file);
        }
    }

    /**
     * Parses the props file
     */
    public static void parsePropsFile(String file){
        p = new Properties();

        try{
            FileInputStream f = new FileInputStream (file);
            p.load (f);
        }
        catch (IOException e){
            System.out.println ("Error Loading Constants File.");
            File f = new File(".");
            System.out.println(f.getAbsolutePath());

            //System.exit(0);
        }

        char sep = File.separatorChar;

        userDataDirectory =  p.getProperty ("userDataDirectory") + sep;
        userDataDirectory = ".." + sep + userDataDirectory;

        systemDataDirectory =  p.getProperty ("systemDataDirectory") + sep;
        
        clientPaperDirectory= p.getProperty("clientPaperDirectory")+sep;
        graderPaperDirectory= p.getProperty("graderPaperDirectory")+sep;
        
        paperStoreBase=p.getProperty("paperStoreBase");
        cachedLoginStore=p.getProperty("cls");
        
        dictionaryDir= p.getProperty("dictDirectory") + sep;
        
        template=p.getProperty("template");
        
        dictionaryFile = dictionaryDir + p.getProperty ("dictionaryFile");
        phonetFile = dictionaryDir + p.getProperty ("phonetFile");
        gestureFile =  systemDataDirectory + p.getProperty ("gestureFile");
        iconDirectory = p.getProperty ("iconDirectory") + sep;
        version = p.getProperty ("version");
        
        windowOpenerFile = systemDataDirectory + p.getProperty ("windowOpenerFile");
        
        propUser=p.getProperty("user.name");
        
        universityID= p.getProperty("universityID");

        String temp_splash = p.getProperty("splashDelay");
        String temp_mesh = p.getProperty("meshSize");
        try{
            splashDelay = Integer.parseInt(temp_splash);
        }
        catch(Exception e){
            splashDelay = 2000;
        }

        try{
            meshSize = Integer.parseInt(temp_mesh);
        }
        catch(Exception e){
            meshSize = 10;
        }
        
        //default to the constant value...
        writerAddress = p.getProperty("writerAddress",TransmissionConstants.WRITER_FORWARD_ADDRESS);
        graderAddress = p.getProperty("graderAddress",TransmissionConstants.GRADER_FORWARD_ADDRESS);
    }
    
    /**
     * Rewrites the properties file so that all the directories and files 
     * are located in a different directory.
     * (include ending /)
     */
    public static void reWritePropsFile(String newCD){
        parseIfNeeded();
        Properties props=new Properties();
        //first properties that don't change:
        props.setProperty("version",p.getProperty("version"));
        props.setProperty("splashDelay",p.getProperty("splashDelay"));
        props.setProperty("meshSize",p.getProperty("meshSize"));
        props.setProperty("gestureFile",p.getProperty("gestureFile"));
        props.setProperty("dictionaryFile",p.getProperty("dictionaryFile"));
        props.setProperty("phonetFile",p.getProperty("phonetFile"));
        props.setProperty("universityID",p.getProperty("universityID"));
        props.setProperty("iconDirectory",p.getProperty("iconDirectory"));
        String temp=p.getProperty("writerAddress");
        if(temp!=null){
            props.setProperty("writerAddress",temp);
        }
        temp=p.getProperty("graderAddress");
        if(temp!=null){
            props.setProperty("graderAddress",temp);
        }
        props.setProperty("systemDataDirectory",p.getProperty("systemDataDirectory"));
        props.setProperty("template",p.getProperty("template"));
        props.setProperty("dictDirectory",p.getProperty("dictDirectory"));
        props.setProperty("windowOpenerFile",p.getProperty("windowOpenerFile"));
        
        //now properties that do
        props.setProperty("userDataDirectory",newCD+p.getProperty("userDataDirectory"));
        
        props.setProperty("clientPaperDirectory",newCD+p.getProperty("clientPaperDirectory"));
        props.setProperty("graderPaperDirectory",newCD+p.getProperty("graderPaperDirectory"));
        props.setProperty("paperStoreBase",newCD+p.getProperty("paperStoreBase"));
        props.setProperty("cls",newCD+p.getProperty("cls"));
        
        props.setProperty("user.name",System.getProperty("user.name"));//for MacInstall the user this is for.
        
        try{
            FileOutputStream f = new FileOutputStream (file);
            props.store(f,"Blaise Properties File (rewritten)");
        }catch(IOException e){
            System.out.println("Could not rewrite constants file.");
            e.printStackTrace();
        }
    }

    public static void main (String[] argv){
            Constants c = new Constants("blaise.props");
    }
}