package edu.gwu.blaise.version2.util;
import java.util.EventObject;
/**
 * The object passed by BlaiseDraftView to interested parties that
 * implement the DraftViewListener interface (and register).
 * @author Michael Gaiman
 * @version 1 5/31/2004
 */
public class DraftViewEvent extends EventObject{
    private BlaiseDraftView source;
    private int changeType;
    private int selectedIndex;
    
    /**
     * VIEW_UPDATE happens when the updateView() method is called.
     * Usually this means that the Blaise_Paper object has changed, 
     * a new draft or something...
     */
    public static final int VIEW_UPDATE=55;
    
    /**
     * a draft has been selected by the user.
     */
    public static final int DRAFT_SELECTED=66;
    
    /**
     * A comment has been selected by the user.
     */
    public static final int COMMENT_SELECTED=77;

    DraftViewEvent(BlaiseDraftView source, int changeType){   
        this(source,changeType,-1);
    }
    
    DraftViewEvent(BlaiseDraftView source, int changeType, int selectedIndex){
        super(source);
        this.source=source;
        this.changeType=changeType;
        this.selectedIndex=selectedIndex;
    }
    
    /**
     * Returns the change type.  These will be one of VIEW_UPDATE, DRAFT_SELECTED,
     * or COMMENT_SELECTED.
     */
    public int getChangeType(){
        return changeType;
    }
    
    /**
     * If this event was a DRAFT_ or COMMENT_ SELECTED event, this
     * method will return the index of the selected draft or comment (starting from 0)
     * If this event was some other type, this method will return -1.
     */ 
    public int getSelectedIndex(){
        return selectedIndex;
    }
    
    /**
     * Calls back to the source to get a list of all of the drafts formatted in
     * "draft#" : "title" manner.  The array is organized in index order.
     */
    public String[] getDraftList(){
        return source.getDraftList();
    }
    
    /**
     * Calls back to the source to get a list of all of the comments for the
     * currently selected draft.  They array is organized in index order.
     */
    public String[] getCommentList(){
        return source.getCommentList();
    }
    
    public String toString(){
        StringBuffer sb=new StringBuffer("DraftViewEvent [Source:");
        sb.append(source);
        sb.append(" changeType:");
        if(changeType==VIEW_UPDATE){
            sb.append("VIEW_UPDATE");
        }
        else if(changeType==DRAFT_SELECTED){
            sb.append("DRAFT_SELECTED");
        }
        else if(changeType==COMMENT_SELECTED){
            sb.append("COMMENT_SELECTED");
        }
        else{
            sb.append("UNKNOWN");
        }
        if(selectedIndex!=-1){
            sb.append(" selectedIndex:");
            sb.append(selectedIndex);
        }
        sb.append("]");
        return sb.toString();
    }
}
