package edu.gwu.blaise.version2.util;
import javax.swing.event.*;
import javax.swing.*;
import edu.gwu.blaise.version2.documents.*;
import java.awt.*;
import java.io.*;
/**
 * ExternalFileView is a utility class that, given JTextPane and a
 * ExternalFileContainer will format the text pane so that it displays
 * a link to the external file.  Once the link is clicked, the external file
 * will be opened in its default editor/viewer.
 * @author Michael Gaiman
 * @version 5/27/2004
 */
public class ExternalFileView implements HyperlinkListener{
    private JTextPane view=null;
    private ExternalFileContainer efc=null;
    private Component parent=null;
    
    /**
     * specify the parent incase an error occurs so that a dialog can be shown.
     */
    public ExternalFileView(Component parent){
        this.parent=parent;
    }
    
    /**
     * Install this external file view on this text pane/container set
     * The text in the text pane will be erased and replaced with a link
     * to the external file.
     * A hyperlink listener will also be installed and will be used to active
     * the program once the link is clicked.
     */
    public void install(JTextPane view, ExternalFileContainer efc){
        if(this.view!=null || this.efc!=null){
            throw new IllegalStateException("ExternalFileView can only be used on one JTextPane/ExternalFileContainer combination at a time.");
        }
        
        this.view=view;
        this.efc=efc;
        view.addHyperlinkListener(this);
        view.setText("<html><body><p></p><p>This draft is an external document. Click on the link to view the document <a href=\"file://"+efc.getExternalBodyLink()+"\">"+efc.getExternalBodyFilename()+"</a>.</p></body></html>");
    }
    
    /**
     * Causes the ExternalFileView to be removed from the previous textpane/container
     * combination.
     */
    public void uninstall(){
        if(view!=null){
            view.removeHyperlinkListener(this);
            view=null;
        }
        efc=null;
    }
    
    /**
     * Method from HyperlinkListener.  Causes the file to be opened.
     */
    public void hyperlinkUpdate(HyperlinkEvent e){
        if(e.getEventType()==HyperlinkEvent.EventType.ACTIVATED){
            DocumentOpener opener=new DocumentOpener(efc.getExternalBodyLink());
            if(!opener.open()){
                JOptionPane.showMessageDialog(parent,"An Error Occurred that prevented Blaise from opening the external document.\n("+opener.getErrorMsg()+")","Cannot Open Document",JOptionPane.ERROR_MESSAGE);
            }
        }
    }
}
