//LOC:66
package edu.gwu.blaise.version2.util;

import java.io.*;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.parser.ParserDelegator;

/**
 * The HTML_Parser elminates HTML tags
 *
 * @author Sean T. Hanlon
 */
public class HTML_Parser extends HTMLEditorKit.ParserCallback{

	public static byte PASTE;
	public static byte COUNT;

	String output = "";	//The string output
	int start;			//start of a paragraph
	boolean gotText;	//if we got text
	boolean isPaste;	//if using parser to determine legal paste values
	boolean isWordCount;//if counting words

	/**
	 * Constructor
	 */
	public HTML_Parser(){
		output = "";
		start = 0;
		gotText = false;
		isPaste = false;
		isWordCount = false;
	}

	public HTML_Parser(byte type){
		this();

		if(type == PASTE) isPaste = true;
		else if(type == COUNT) isWordCount = true;
	}

	/**
	 * Parses a given html-file
	 *
	 * @param file The file to be parsed
	 */
	public String parse(String file){

		output = "";
		try{
			Reader reader;
				if(isPaste || isWordCount){
					reader = new StringReader(file);
				}
				else{
					reader = new FileReader(file);
				}
			new ParserDelegator().parse(reader, this, false);
		}
		catch (IOException e){
			System.out.println(e);
		}

		return output;
	}

	//When a <p> is found, set index
	public void handleStartTag(HTML.Tag tag, MutableAttributeSet a, int pos)	{
		if(isPaste)return;
		if (tag.equals(HTML.Tag.P)){
			if(isWordCount)output += " ";
			else{
				start = pos;
				gotText = false;
			}
		}
	}

	//When a </p> is found, insert a new line to signify a break in paragraph
	public void handleEndTag(HTML.Tag tag, int pos)	{
		if(isPaste)return;
		if (tag.equals(HTML.Tag.P)){
			int value = pos - start;
			if((value > 5) && gotText){
				output += "\n";
				gotText = false;
			}
		}
	}

	//When text is encountered: append as is
	public void handleText(char[] data, int pos)	{
		gotText = true;
		output += new String(data);
	}

	//Not used
	public void handleComment(char[] data, int pos)	{}
	public void handleEndOfLineString(String eol)	{}
	public void handleError(String errorMsg, int pos){}
	public void handleMutableTag(HTML.Tag tag, MutableAttributeSet a, int pos){}
	public void handleSimpleTag(HTML.Tag tag, MutableAttributeSet a, int pos){}


	public static void main(String[] argv){

		File f = new File("E:\\SeniorDesign\\Versions\\PGS\\data\\test.html");
		HTML_Parser parse = new HTML_Parser();

		String out = parse.parse(f.getAbsolutePath());
		System.out.println(out);
	}
}