package edu.gwu.blaise.version2.util;

/**
 * LoginHandler performs the actual login work for NewLoginUtility.
 * This helps keep NewLoginUtility generic.
 * @author Michael Gaiman
 * @version 1 6/30/2004
 */
public interface LoginHandler {
    /**
     * Returned by getLoginStatus if logged in successfully.
     */
    public static final int SUCCESS=1;
    /**
     * Returned by getLoginStatus if there was a failure logging in.
     */
    public static final int FAILURE=2;
    /**
     * Returned by getLoginStatus if the login query hasn't finished yet.
     */
    public static final int UNKNOWN=3;

    /**
     * Called first by NewLoginUtility.  This method should start off
     * the login process. (The logging in should be done in a different thread
     * as this method is called from the event thread).
     */
    public void doLogin(String username,String password);
    
    /**
     * Called if the user chose the cancel option or closed the dialog.
     * This method is called after the LoginUtility has hidden.
     * (Like doLogin, this method is called from the event thread).
     */
    public void doCancel();
    
    /**
     * Polled semi-continuously after doLogin until SUCCESS or FAILURE is returned.
     * (UNKNOWN should be returned if the login hasn't completed yet.
     */
    public int getLoginStatus();
    
    /**
     * Called if getLoginStatus returns false for an explanation to tell the user.
     */
    public String getMessage();

}
