package edu.gwu.blaise.version2.util;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

import edu.gwu.blaise.version2.transmission.*;
/**
* Presents a JDialog requesting their username and password.
 * It then contacts the blaise server to login.
 * @author Sean T. Hanlon
 * @author Michael R. Gaiman
 * @version 4 6/16/2004
 * @history 1 initial version
 *          2 3/20/2004 reworked everything.  Re did layout using FindFrame tech.
 *          3 6/12/2004 Switched to a GridBagLayout, added support for null parent.
 *          4 6/16/2004 Exposed the message to the user when the query failed.
 */
public class LoginUtility extends JDialog implements ActionListener, DocumentListener{
    
    /**
    * This login utility is for the blaise editor.
     */
    public static final byte CLIENT=1;
    public static final byte GRADER=2;	//Grader is not used
    
    /**
     * Returned by getOutput() when the cancel option was chosen
     */
    public static final String CANCEL_OPTION="cancelled";
    
    //for use with layout
    private static final int HEIGHT=130;
    private static final int WIDTH=320;
    private static final int NUM_COLS=20;
    private static final String LOGIN="login";
    private static final String CANCEL="cancel";
    
    private boolean pUseful=false;
    private boolean uUseful=false;
    
    private byte type;
    private JDialog thisDialog;
    private JFrame parent;		//parent frame
    private JTextField userField;       //field for username
    private JPasswordField passField;	//field for password
    private JButton loginButton;
    private JButton cancelButton;
    
    private Object output = null;
    private Thread loginThread=null;
    
    private static final boolean debug=true;
    
    /**
     * Constructor
     *
     */
    public LoginUtility(JFrame parent, byte type){
        super(parent, "Authentication Required", true);
        this.type = type;
        this.parent = parent;
        thisDialog = this;
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createGUI();
            }
        });
    }
    
    private void createGUI(){
        Container container=getContentPane();
        container.setLayout(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        c.gridx=0;
        c.gridy=0;
        //create the username row

        container.add(new JLabel("Username:",JLabel.RIGHT),c);
        userField=new JTextField(NUM_COLS);
        Dimension prefSize=userField.getPreferredSize();
        userField.setMinimumSize(prefSize);
        //userField.setMaximumSize(prefSize);
        userField.requestFocus();
        c.gridx=1;
        c.gridwidth=2;
        container.add(userField,c);
        userField.getDocument().addDocumentListener(this);
        
        c.gridy=1;
        c.gridx=0;
        c.gridwidth=1;
        //create password row
        container.add(new JLabel("Password:",JLabel.RIGHT),c);
        //passField is not set by columns but rather by prefSize.
        //this fixes a strangeness from windows.
        passField=new JPasswordField();
        passField.setMinimumSize(prefSize);
        passField.setPreferredSize(prefSize);
        //passField.setMaximumSize(prefSize);
        c.gridx=1;
        c.gridwidth=2;
        container.add(passField,c);
        passField.getDocument().addDocumentListener(this);

        //create button row
        c.gridy=2;
        c.gridwidth=1;
        c.gridx=1;
        cancelButton=new JButton(CANCEL);
        cancelButton.addActionListener(this);
        cancelButton.setFocusable(false);
        container.add(cancelButton,c);
        c.gridx=2;
        loginButton=new JButton(LOGIN);
        loginButton.addActionListener(this);
        loginButton.setEnabled(false);
        container.add(loginButton,c);
                
        setSize(WIDTH,HEIGHT);
        setResizable(false);
        
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.getX();
            pY=parent.getY();
            pW=parent.getWidth();
            pH=parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
    
        
        setLocation(myX,myY);
        getRootPane().setDefaultButton(loginButton);
    }
    
    /**
     * Returns different things depending upon the status of the login.
     * If the user cancels, CANCEL_OPTION is returned.
     * If there was a transmission failure then the String result of getMessage is returned.
     * If the login was successful then the LoginQuery is returned
     */
    public Object getOutput(){
        return output;
    }
    
    public void insertUpdate(DocumentEvent e){
        if(userField.getText().length()>0){
            uUseful=true;
        }

        if(passField.getPassword().length>0){
            pUseful=true;
        }
        
        if(uUseful && pUseful){
            loginButton.setEnabled(true);
        }
        else{
            loginButton.setEnabled(false);
        }
    }
    
    public void removeUpdate(DocumentEvent e){
        //check to see if it still has something.
        if(userField.getText().length()==0){
            uUseful=false;
        }

        if(passField.getPassword().length==0){
            pUseful=false;
        }
        
        if(uUseful && pUseful){
            loginButton.setEnabled(true);
        }
        else{
            loginButton.setEnabled(false);
        }
    }
    
    public void changedUpdate(DocumentEvent e){}
    
    /**
     * Shows the dialog.
     */
    public void confirmLogin(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                setVisible(true);
            }
        });
    }
    
    public Thread getLoginThread(){
        return loginThread;
    }
    
    public void actionPerformed(ActionEvent e){
        if(e.getActionCommand().equals(CANCEL)){
            output=CANCEL_OPTION;
            setVisible(false);
            return;
        }
        
        loginThread=new Thread(){
            public void run(){
                ProgressView progView=new ProgressView(parent,"Logging In...",0,true);
                //check server for admittance
                try{
                    thisDialog.setVisible(false);
                    String user = userField.getText();
                    String pass = new String(passField.getPassword());
                    
                    if(type == CLIENT){
                        
                        WriterLoginQuery temp = TransmissionHandler.getWriterLogin(user, pass, Constants.getUniversityID());
                        if(temp.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                            output = temp;
                        }
                        else{
                            output = temp.getMessage();
                            if(debug)System.out.println("Transmission failure.  Full message: "+temp.getFullText());
                        }
                        
                        thisDialog.setVisible(false);
                    }
                    else if(type == GRADER){
                        
                        GraderLoginQuery temp = TransmissionHandler.getGraderLogin(user,pass,Constants.getUniversityID());
                        if(temp.transmit(Constants.getGraderAddress(),Constants.getWriterAddress())){
                            output = temp;
                        }
                        else{
                            output = temp.getMessage();
                            if(debug)System.out.println("Transmission failure.  Full message: "+temp.getFullText());
                        }
                    }
                }finally{
                    progView.done();
                }
            }
        };
        loginThread.start();
    }
    
    public static void main(String[] argv){
        LoginUtility lu=new LoginUtility(null,CLIENT);
        lu.confirmLogin();
    }
}