package edu.gwu.blaise.version2.util;
import edu.gwu.blaise.version2.transmission.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
/**
 * A new and improved Login Utility.  This utility features several improvements
 * over plain old LoginUtility.  It does not return unless cancelled or successfully
 * logged in and it internally manages progress view.  (It also looks nicer).
 * @author Michael Gaiman
 * @version 1 6/30/2004
 */
public class NewLoginUtility extends JDialog implements ActionListener, DocumentListener{
    private LoginHandler handler;
    private Frame parent;
    private String title;
    private String message;
    private JTextField userField;
    private JPasswordField passField;
    private JLabel statusL;
    private JButton loginButton;
    private JButton cancelButton;
    private JProgressBar progBar;
    private Timer statusTimer;
    
    private static final int HEIGHT=190;
    private static final int WIDTH=320;
    private static final int NUM_COLS=16;
    
    private static final String LOGIN="Login";
    private static final String CANCEL="Cancel";
    
    private boolean pUseful=false;
    private boolean uUseful=false;
    
    public NewLoginUtility(Frame parent,String title,String message,LoginHandler handler){
        super(parent,title,true);
        this.parent=parent;
        this.title=title;
        this.message=message;
        this.handler=handler;
        
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private void createAndShowGUI(){
        Container container=getContentPane();
        container.setLayout(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        
        JLabel l=new JLabel("<html><p>"+message+"</p></html>");
        
        c.fill=c.BOTH;
        c.anchor=c.PAGE_START;
        c.weightx=.95;
        c.weighty=.6;
        c.gridwidth=c.REMAINDER;
        container.add(l,c);
        
        //create the username row
        c.fill=c.NONE;
        c.anchor=c.CENTER;
        c.gridwidth=c.RELATIVE;
        c.weightx=.4;
        c.weighty=0;
        container.add(new JLabel("Username:",JLabel.RIGHT),c);
        
        userField=new JTextField(NUM_COLS);
        Dimension prefSize=userField.getPreferredSize();
        userField.setMinimumSize(prefSize);
        userField.setMaximumSize(prefSize);
        userField.requestFocus();
        c.gridwidth=c.REMAINDER;
        c.weightx=.6;
        container.add(userField,c);
        userField.getDocument().addDocumentListener(this);
        
        
        //create password row
        c.fill=c.NONE;
        c.gridwidth=c.RELATIVE;
        c.weightx=.4;
        container.add(new JLabel("Password:",JLabel.RIGHT),c);
        
        //passField is not set by columns but rather by prefSize.
        //this fixes a strangeness from windows.
        passField=new JPasswordField();
        passField.setMinimumSize(prefSize);
        passField.setPreferredSize(prefSize);
        passField.setMaximumSize(prefSize);
        c.gridwidth=c.REMAINDER;
        c.weightx=.6;
        container.add(passField,c);
        passField.getDocument().addDocumentListener(this);
    
        statusL=new JLabel(" ");
        Font f=statusL.getFont();
        f=f.deriveFont(f.getSize2D()-2);//make the font small is
        statusL.setFont(f);
        //c.anchor=c.PAGE_START;
        c.fill=c.HORIZONTAL;
        c.weightx=1;
        c.weighty=.3;
        container.add(statusL,c);

        //create button row
        cancelButton=new JButton(CANCEL);
        cancelButton.addActionListener(this);
        cancelButton.setFocusable(false);
        c.anchor=c.LINE_END;
        c.fill=c.NONE;
        c.weighty=0;
        c.weightx=.4;
        c.gridwidth=c.RELATIVE;
        container.add(cancelButton,c);
        
        loginButton=new JButton(LOGIN);
        loginButton.addActionListener(this);
        loginButton.setEnabled(false);
        c.gridwidth=c.REMAINDER;
        container.add(loginButton,c);
        
        //create progress bar
        progBar=new JProgressBar();
        progBar.setStringPainted(false);
        c.anchor=c.PAGE_END;
        c.weighty=0;
        c.weightx=.6;
        c.fill=c.HORIZONTAL;
        container.add(progBar,c);
                
        setSize(WIDTH,HEIGHT);
        setResizable(false);
        
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.getX();
            pY=parent.getY();
            pW=parent.getWidth();
            pH=parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
    
        
        setLocation(myX,myY);
        addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
                //System.out.println("closing...");
                handler.doCancel();
            }
        });
        getRootPane().setDefaultButton(loginButton);
        setVisible(true);
    }
    
    public void insertUpdate(DocumentEvent e){
        if(userField.getText().length()>0){
            uUseful=true;
        }

        if(passField.getPassword().length>0){
            pUseful=true;
        }
        
        if(uUseful && pUseful){
            loginButton.setEnabled(true);
        }
        else{
            loginButton.setEnabled(false);
        }
    }
    
    public void removeUpdate(DocumentEvent e){
        //check to see if it still has something.
        if(userField.getText().length()==0){
            uUseful=false;
        }

        if(passField.getPassword().length==0){
            pUseful=false;
        }
        
        if(uUseful && pUseful){
            loginButton.setEnabled(true);
        }
        else{
            loginButton.setEnabled(false);
        }
    }
    
    public void changedUpdate(DocumentEvent e){}
    
    /**
     * Formats and displays the text in the status field.
     */
    private void setStatus(String text){
        statusL.setText("<html><p> "+text+"</p></html>");
    }
    
    public void actionPerformed(ActionEvent e){
        if(e.getActionCommand().equals(CANCEL)){
            setVisible(false);
            handler.doCancel();
            return;
        }
        //adjust the ui for working mode:
        userField.setEnabled(false);
        passField.setEnabled(false);
        loginButton.setEnabled(false);
        setStatus("Logging in...");
        progBar.setIndeterminate(true);
        handler.doLogin(userField.getText(),new String(passField.getPassword()));
        

        ActionListener taskPerformer = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                //check handler to see how it's doing
                int status=handler.getLoginStatus();
                if(status==LoginHandler.UNKNOWN){
                    return;
                }
                else if(status==LoginHandler.FAILURE){
                    progBar.setIndeterminate(false);
                    userField.setEnabled(true);
                    passField.setEnabled(true);
                    passField.setText("");
                    passField.requestFocus();
                    //loginButton.setEnabled(true);
                    
                    setStatus(handler.getMessage());
                    statusTimer.stop();
                }
                else if(status==LoginHandler.SUCCESS){
                    setStatus("Login successful");
                    statusTimer.stop();
                    dispose();
                }
            }
        };
        statusTimer=new Timer(100, taskPerformer);
        statusTimer.start();
    }
}
