package edu.gwu.blaise.version2.util;
import edu.gwu.blaise.version2.documents.Note;
import java.util.Vector;
/**
 * Classes that display Note objects will have to implement this
 * inteface.  It handles the interaction between notes and between the container.
 * @author Michael Gaiman
 * @version 1 10/24/2004
 */
public interface NoteControllerInterface {
    /**
     * Called by Note to display its note frame and icon in the layered pane.
     */
    public void addNoteToPane(Note.NoteFrame frame, Note.NoteIcon icon);
    
    /**
     * Called by Note to stop its note frame and icon from being displayed
     * in the layered pane.
     */
    public void removeNoteFromPane(Note.NoteFrame frame, Note.NoteIcon icon);
    
    /**
     * Removes all selected Notes (by calling removeNoteFromPane)
     */
    public void removeSelectedNotes();
    
    /**
     * Closes all open Notes
     */
    public void closeAll();
    
    /**
     * Opens all Notes
     */
    public void openAll();
    
    /**
     * Draws a selection rectangle around all note icons.
     */
    public void selectAllIcons();
    
    /**
     * Stops drawing a selection rectangle around all note icons.
     */
    public void deselectAllIcons();
    
    /**
     * Returns a Vector containing the Note objects of all selected Icons.
     */
    public Vector getSelectedIconNotes();
    
    /**
     * Selects the caller Note while deselecting all
     * other open notes.
     */
    public void selectedAndToFront(Note.NoteFrame caller);
    
    /**
     * Called to notify the controller that the user opened
     * caller Note.
     */
    public void noteOpened(Note.NoteFrame caller);
    
    /**
     * Called to notify the controller that the user closed
     * caller Note.
     */
    public void noteClosed(Note.NoteFrame caller);
    
    /**
     * Sets any selected Notes to have the supplied color as their background.
     */
    public void setSelectedNotesToColor(java.awt.Color c);
}
