package edu.gwu.blaise.version2.util;
import javax.swing.*;
import javax.swing.tree.*;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import edu.gwu.blaise.version2.documents.*;
/**
 * Displays a dialog containing information about the supplied
 * Blaise_Paper.
 * @author Michael Gaiman
 * @version 6/23/2004
 */
public class PaperInfoDialog extends JDialog{
    private Blaise_Paper paper;
    private Frame parent;
    /**
     * Instantiates and shows dialog with the supplied paper.
     */
    public PaperInfoDialog(Frame parent,Blaise_Paper bp){
        super(parent,"Paper Information",false);//non-modal
        paper=bp;
        this.parent=parent;
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private static final int WIDTH=300;
    private static final int HEIGHT=400;
    
    private void createAndShowGUI(){
        //build the tree
        JTree tree=BlaiseUtilities.getPaperTreeView(paper);
        
        //layout content
        Container pane=getContentPane();
        pane.setLayout(new GridBagLayout());
        GridBagConstraints c=new GridBagConstraints();
        c.weightx=1;
        c.gridx=0;
        c.gridy=0;
        c.gridwidth=4;
        c.weighty=.7;
        c.fill=GridBagConstraints.BOTH;
        pane.add(new JScrollPane(tree,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED),c);
        JButton button=new JButton("Close");
        button.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                setVisible(false);
                dispose();
            }
        });
        c=new GridBagConstraints();
        c.gridx=3;
        c.gridy=1;
        pane.add(button,c);
        
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(parent==null){
            //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=parent.getX();
            pY=parent.getY();
            pW=parent.getWidth();
            pH=parent.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
    
        setSize(WIDTH,HEIGHT);
        setLocation(myX,myY);
        
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setVisible(true);
    }
    
    public static void main(String[] argv){
        if(argv.length!=1){
            System.out.println("Usage: java PaperInfoDialog [blaise paper]");
            return;
        }
        try{
            Blaise_Paper p=null;
            BlaiseFile bf=new BlaiseFile(p,new java.io.File(argv[0]),new BackendFileHandler());
            bf.openPaper();
            new PaperInfoDialog(null,p);//no parent
        }catch(java.io.IOException e){
            System.out.println(e);
        }
    }
}
