package edu.gwu.blaise.version2.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

// This class brings up a small "progress bar" view

public class ProgressView extends JDialog {
    
    private JProgressBar progBar;             // JProgressBar is a Swing component that draws a progress bar
    private boolean isIndeterminate = false;  // Is the task time known?
    private int timeSeconds;                  // How long is the task, if known?
    private boolean threadStopped = false;    // The progress thread keeps checking this.
    private JFrame frame;
    
    private static final int WIDTH=300;
    private static final int HEIGHT=55;
    
  // We need the frame reference, the title for the dialog, time if known, whether known.
    
    public ProgressView (JFrame frame, String title, int timeSeconds, boolean isIndeterminate){
    // Must make it non-modal, else Swing can't continue:
        super (frame, title, false);
        // Store parameters:
        this.timeSeconds = timeSeconds;
        this.isIndeterminate = isIndeterminate;
        this.frame=frame;
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                createAndShowGUI();
            }
        });
    }
    
    private void createAndShowGUI(){
        // Hardcoded size for now.
        setSize (WIDTH, HEIGHT);
        // Offset a little into the frame:
        //figure out the location to open on the screen.
        int myX;
        int myY;
        int pX,pY,pW,pH;
        if(frame==null){
        //open it in the middle of the screen.
            Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
            pX=0;
            pY=0;
            pW=screen.width;
            pH=screen.height;
        }
        else{
            pX=frame.getX();
            pY=frame.getY();
            pW=frame.getWidth();
            pH=frame.getHeight();
        }
        
        myX=pX+(pW/2)-(WIDTH/2);
        myY=pY+(pH/2)-(HEIGHT/2);
        
        
        setLocation(myX,myY);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        
        // Create the progress bar itself. If time is known, we'll display "percent-done" string.
        progBar = new JProgressBar (0, 100);
        progBar.setIndeterminate (isIndeterminate);
        if (!isIndeterminate)
            progBar.setStringPainted (true);
        
        // Place the progress bar in the Dialog frame:
        getContentPane().add (progBar, BorderLayout.CENTER);
        setVisible (true);
        
        // Fire off a thread to handle the display of progress.
        showProgressThread ();
    }
    
  // Invoker calls this when done.
    public void done ()
    {
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                threadStopped = true;
                dispose();
            }
        });
    }
    
    public boolean isDone(){
        return threadStopped;
    }
    
    private void showProgressThread ()
    {
    // Make a thread and launch it.
        Thread thread = new Thread () {
            public void run ()
        {
                showProgress ();
        }
        };
        thread.start();
    }
    
    private void showProgress ()
    {
        if (isIndeterminate)
            return;
        threadStopped = false;
        int timeMillis = 1000*timeSeconds;        // Convert to millis.
        int intervalMillis = timeMillis / 10;     // We'll show progress every 10 units
        int counter = 0;
        
        
    // Sleep and move progress bar 10 times:
        while ( (counter < 100) && (! threadStopped) ) {
            try {
                Thread.sleep (intervalMillis);
            }
            catch (InterruptedException e) {
            }
            counter += 10;
            progBar.setValue (counter);
            System.out.println ("Progress: counter=" + counter);
        }
    }
} // end of class ProgressView
