package edu.gwu.blaise.version2.util;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import com.swabunga.spell.engine.SpellDictionaryHashMap;
import com.swabunga.spell.engine.Word;
import java.util.*;
import java.util.regex.*;
import java.io.*;
import javax.swing.plaf.TextUI;
import edu.gwu.blaise.version2.util.find.DocumentFind;
import edu.gwu.blaise.version2.util.contextmenu.*;
import edu.gwu.blaise.version2.util.Constants;

/**
 * Performs realtime spelling (AKA spelling as you type).
 * The internal spelling engine is http://jazzy.sourceforge.net/
 *
 * @author Michael Gaiman mgaiman@gwu.edu
 * @version 1.2 3/16/2004
 * @history: 1.0 3/13/2004 initial deligate version (previously a subclass of JTextPane)
 *           1.1 3/15/2004 import to blaise
 *           1.2 3/16/2004 uses contextmenu framework
 */
public class RealTimeSpelling implements CaretListener,ActionListener,ContextMenuListener{

    private JEditorPane editor;
    private SpellingHighlighter highlighter;
    
    //spelling related
    private static String dictFile = Constants.getDictionaryFile();
    private static String phonetFile = Constants.getPhonetFile();
    private SpellDictionaryHashMap spellCheck = null;
    private static final boolean debug=false;
    private int THRESHOLD=5;
    //private int numPopupItems=0;
    private JMenuItem add=null;
    private static final String ADD_COMMAND="_SpellingJTextPane.ADD";
    private static final String IGNORE_ALL_COMMAND="_SpellingJTextPane.IGNOREALL";
    private JMenuItem ignoreAll=null;
    private Hashtable ignoredWords=new Hashtable();
    private int oldDot=0;
    private boolean canRescan=true;//the rescan thread listens to this as a stopping case.
    
    /**
     * Constructs a new RealTimeSpelling instance.
     */
    public RealTimeSpelling(JEditorPane text,java.util.List ignoredWords){
        editor=text;
        //handle the dictionary and registration
        try{
            if(debug)System.out.println("Loading Spelling Engine (start:"+System.currentTimeMillis()+" with dictFile:"+dictFile+" phonetFile:"+phonetFile+")");
            spellCheck= new SpellDictionaryHashMap(new File(dictFile), new File(phonetFile));
            if(debug)System.out.println("Finished Loaded Spelling Engine (end:"+System.currentTimeMillis()+")");
            if(ignoredWords!=null){
                addIgnoredWords(ignoredWords);
            }
        }catch(IOException e){
            System.out.println(e);
        }
        
        //initialize the standard menu items.
        add=new JMenuItem("Learn Spelling");
        add.setActionCommand(ADD_COMMAND);
        add.addActionListener(this);
        ignoreAll=new JMenuItem("Ignore Spelling");
        ignoreAll.setActionCommand(IGNORE_ALL_COMMAND);
        ignoreAll.addActionListener(this);
    }
    
    public RealTimeSpelling(JEditorPane text){
        this(text,null);
    }
    
    
    public void startRealTimeSpelling(){
        editor.addCaretListener(this);
        ContextMenuManager.addContextMenuListener(this,editor,ContextMenuManager.FIRST);
        highlighter=new SpellingHighlighter();
        editor.setHighlighter(highlighter);
        if(editor.getDocument().getLength()>0){//non empty
            rescan();
        }
    }
    
    public void stopRealTimeSpelling(){
        canRescan=false;
        editor.removeCaretListener(this);
        ContextMenuManager.removeContextMenuListener(this,editor,ContextMenuManager.FIRST);
        highlighter.removeAllMisspellings();
        editor.setHighlighter(new DefaultHighlighter());
    }
    
    //CaretListener Method:
    public void caretUpdate(CaretEvent e){
		if(debug)System.out.println("caretUpdate()");

        if(replacingSelection){
            //the caret gets changed when user changes the word during the action event
            //processing a caretUpdate here would cause strange behavior.  Thus we return.
			editor.repaint();
            return;
        }
        int dot=e.getDot();
        try{
			if(debug)System.out.println("caretUpdate() --> try{...");

            int length, start,end;
            String word;
            boolean mark=false;
            RTSWord rtsword;
            
            if(Math.abs(oldDot-dot)>1){//first check to see if the dot has been clicked elsewhere.  If so, spell check old word.
                rtsword=getWordPosition(oldDot);
                mark=true;
            }
            else{//if not spell check the previous word.
                rtsword=shouldCheck(dot,oldDot);
                mark=(rtsword==null)?false:true;//shouldCheck returns null if you shouldn't check.
            }
            
            if(mark){
                start=rtsword.wordStart;
                end=rtsword.wordEnd;
                length=end-start;
                if(debug)System.out.println("getWordPosition/scan returned start:"+start+" end:"+end);
                if(length>0){
                    word=editor.getDocument().getText(start,length);
                    markSpellAttribute(start,length,checkSpelling(word));
                }
            }
            
            //make sure to turn off this word.
            rtsword=getWordPosition(dot);
            start=rtsword.wordStart;
            end=rtsword.wordEnd;
            length=end-start;
            if(debug)System.out.println("caret start:"+start+" end:"+end);
            if(length>0){
                markSpellAttribute(start,end-start,true);
            }
        }
        catch(BadLocationException ex){
            if(debug)System.out.println(ex);
			editor.repaint();
        }
        finally{
            oldDot=dot;
        }
    }
    
    /**
     * Returns a RTSWord object containing start and end location of the word this position is closest to.  
     * This method extensively uses the methods provided in swing.Utilities.
     * <p>As an example of how this method works take the text <pre>The quick  brown fox</pre> if the position is
     * 5 which is between q and u then the WordHolder will have wordStart=4 and wordEnd=9.  If on the other
     * hand position is 3 which is after the e in "The" then wordStart=0 and wordEnd=3.  If position is 10 which
     * is inbetween the two spaces between "quick" and "brown" then wordStart and wordEnd will both be set to 10.
     * <p>
     */
    private RTSWord getWordPosition(int pos) throws BadLocationException{
        int start,end;
        RTSWord word=new RTSWord();
        word.wordStart=word.wordEnd=pos;//reset to start.
        Document doc=editor.getDocument();
        int length=doc.getLength();
        
        
        if(debug)System.out.println("getWordPosition, length:"+length);
        
        //first check to see if pos is in the middle of whitespace.
        if(pos-1>=0 && pos+1<length){
            if(searchP.matcher(doc.getText(pos-1,2)).matches()){//the character before and after pos are whitespace
                return word;//wordStart/End have already been set.
            }
        }
        else{//one or both those were wrong.  Look into it more
            if(pos-1>=0 && pos+1>=length){
                //then we are at the end, so check if the preceeding area is whitespace
                if(debug)System.out.println("At end.");
                if(searchP.matcher(doc.getText(pos-1,1)).matches()){
                    return word;
                }
                else{//real stuff is at the end--find it here so standard cases below don't need to.
                    word.wordStart=Utilities.getWordStart(editor,firstNonApos(pos-1));
                    word.wordEnd=Utilities.getWordEnd(editor,word.wordStart);
                    return word;
                }
            }
            else if(pos+1<length && pos-1<0){
                //then we are the beginning of the doc, so check to see if the proceeding area is whitespace
                if(debug)System.out.println("at beginning");
                if(searchP.matcher(doc.getText(pos,1)).matches()){
                    return word;
                }
                else{//this is a real word.  Find its end
                    word.wordStart=0;
                    word.wordEnd=Utilities.getWordEnd(editor,0);
                    return word;
                }
            }
            else{
                if(debug)System.out.println("Blank document.");
                return word;//this doc is blank.  wordStart/End are valid for it.
            }
        }
        
        /* whitespace cases are handled.  Now let's look into standard word cases.
         * There are a couple.  First, the normal case: pos is in the middle of a word.
         * this is easily detected with Utilities.getWordStart/End.  Second, pos
         * is at the end of a word beginning of whitespace, here we need to use 
         * Utilites.getPreviousWord.  Lastly, pos is at the end of whitespace and
         * beginning of a word.  Utilities.getWordStart handles this nicely.
         */
        String s=editor.getDocument().getText(pos-1,2);
        if(AWSSearchP.matcher(s).matches()){//case 2
            start=Utilities.getPreviousWord(editor,firstNonApos(pos));
            end=Utilities.getWordEnd(editor,start);
        }
        /*else if(WSASearchP.matcher(s).matches()){//case 3
            start=Utilities.getWordStart(editor,pos);
            end=Utilities.getWordEnd(editor,start);
        }*/
        else{//case 1
            start=Utilities.getWordStart(editor,pos);
            end=Utilities.getWordEnd(editor,start);
        }

        word.wordStart=start; word.wordEnd=end;
        return word;
    }
    
    /**
     * Scans backwards in the document starting with position pos looking for the first non-Apostrophe.
     * @return position of first found non-Apostrophe or 0 if none are found.
     * @throws BadLocationException if initial position given is invalid.
     */
    private int firstNonApos(int pos) throws BadLocationException{
        String s;
        for(int i=pos;i>=0;i--){
            s=editor.getDocument().getText(pos,1);
            if(!s.equals("'")){
                return i;
            }
        }
        return 0;
    }

    /**
     * Determines whether the word given in position should be spell checked or not.
     * Returns an RTSWord object containing the area to be checked, or null if the position shouldn't
     * be checked.
     */
    private RTSWord shouldCheck(int dot,int oldDot) throws BadLocationException{
        int length=editor.getDocument().getLength();
		if(debug)System.out.println("RTSWord.shouldCheck();");

        editor.repaint();
		if(dot==length && oldDot>=length){
            if(debug)System.out.println("shouldCheck, dot==getLength && oldDot>=length");
            //we can't know if this is meant to be the end of the word or not, so be safe and do it anyway.
            //return getWordPosition(dot);
			//returning null here fixes the issue where misspelled text remains highlighted when at the end after
			//a backspace and the space afterwards gets highlighted, too.
			return null;
        }
    
        if(dot>oldDot){
            if(AWSSearchP.matcher(editor.getDocument().getText(oldDot-1,2)).matches()){
                return getWordPosition(oldDot);//found end of word--check for spelling
            }
        }
        else{
            if(debug)System.out.println("Going backwards");
            if(WSASearchP.matcher(editor.getDocument().getText(dot,2)).matches()){
                if(debug)System.out.println("Going backwards--word end.");
                return getWordPosition(oldDot);//found beginning of word--check for spelling
            }
        }
        return null;
    }
        
    /**
     * The regular expressions used internally to determine whitespace
     */
    private static final String searchRE="(\\p{Space}*|[\\p{Punct}&&[^']]*)";
    private static final Pattern searchP=Pattern.compile(searchRE);
    private static final String AWSSearchRE="(\\p{Alnum}|')(\\p{Space}*|[\\p{Punct}&&[^']]*)";
    private static final Pattern AWSSearchP=Pattern.compile(AWSSearchRE);
    private static final String WSASearchRE="(\\p{Space}*|[\\p{Punct}&&[^']]*)(\\p{Alnum}|')";
    private static final Pattern WSASearchP=Pattern.compile(WSASearchRE);
    
    
    public JMenuItem[] getContextMenuItems(ContextMenuEvent e){
        int x=e.getX(),y=e.getY();
        int offset=editor.viewToModel(new Point(e.getX(),e.getY()));
        java.util.List list=null;
        RTSWord rtsword;
        try{
            rtsword=getWordPosition(offset);
            int length=rtsword.wordEnd-rtsword.wordStart;
            if(length>0){
                String word=editor.getDocument().getText(rtsword.wordStart,length);
                if(checkSpelling(word)){//it is spelled correctly
                    //we have nothing to add the the Contextual Menu
                    return new JMenuItem[0];
                }
                else{//it is misspelled.
                    selectAndSpellOff(rtsword.wordStart,rtsword.wordEnd);//select it and turn off Spelling attribute.
                    list=spellCheck.getSuggestions(word,THRESHOLD);
                    return getPopup(list);
                }
                
            }
        }
        catch(BadLocationException ex){
            System.out.println(ex);
        }
        
        //if here, something out of the ordinary happened.  Just return nothing.
        return new JMenuItem[0];
    }
    
    
    /**
     * Returns a JPopupMenu filled with the appropriate items
     */
    private JMenuItem[] getPopup(java.util.List suggestions){
        ArrayList items;
        if(suggestions.size()>0){
            items=buildPopupWords(suggestions);
        }
        else{
            JMenuItem none=new JMenuItem("<html><i>No Spelling Suggestions</i></html>");
            none.setEnabled(false);
            items=new ArrayList();
            items.add(none);
        }
        //add generic popup menu items.
        items.add(ContextMenuManager.Separator);//add a separator first
        //instantiated in constructor
        items.add(add);
        items.add(ignoreAll);
        
        return (JMenuItem[])items.toArray(new JMenuItem[items.size()]);
    }
    
    /**
     * Assembles a JMenu with the given List of words and the standard
     * Spelling options.
     * @param words may be null if there are no words to display.
     * @param container an ArrayList of JMenuItems.
     */
    private ArrayList buildPopupWords(java.util.List suggestions){
        if(debug)System.out.println("Word Suggestions:"+suggestions);
        ArrayList popup=new ArrayList();
        JMenuItem item;
        Word w;
        boolean more=false;//add another popup menu with additional commands.
        if(suggestions!=null){
            int size=suggestions.size();
            if(size>9){
                size=9;
                more=true;
            }
            for(int i=0;i<size;i++){
                w=(Word)suggestions.remove(0);
                item=new JMenuItem(w.toString());
                item.addActionListener(this);
                popup.add(item);
            }
            if(more){//add another menu
                ArrayList popup2=buildPopupWords(suggestions);
                
                JMenu p2=new JMenu("More Suggestions");
                size=popup2.size();
                for(int i=0;i<size;++i){
                    p2.add((JMenuItem)popup2.get(i));
                }
                popup.add(p2);
            }
        }
        return popup;
    }
    
    /**
     * Selects the area and turns off any spelling attributes in that property.
     */
    private void selectAndSpellOff(int start, int end) throws BadLocationException{
        editor.select(start,end);
        markSpellAttribute(start,end-start,true);
    }
    
    
    private boolean replacingSelection=false;
    
    /**
     * The action selected by in the context-menu after right clicking a word.
     */
    public void actionPerformed(ActionEvent e){
        String command=e.getActionCommand();
        if(debug)System.out.println("ActionEvent ActionCommand:"+command);
        
        if(command.equals(ADD_COMMAND)){
            String selection=editor.getSelectedText();
            if(selection!=null){//just making sure...
                if(debug)System.out.println("Adding '"+selection+"' to spell checker");
                spellCheck.addWord(selection);
                rescanWord(selection,true);
                editor.repaint();
            }
            return;
        }
        else if(command.equals(IGNORE_ALL_COMMAND)){
            String selection=editor.getSelectedText();
            if(selection!=null){//just making sure...
                if(debug)System.out.println("Adding '"+selection+"' to ignored list");
                rescanWord(selection,true);
                ignoredWords.put(selection,Boolean.TRUE);//So we can keep ignored words 
                editor.repaint();
            }
            return;
        }
        else{//replace the word with the word selected.
            replacingSelection=true;//so caret related stuff can be called off during this.
            editor.replaceSelection(command);
            replacingSelection=false;
            if(debug)System.out.println("Selection replaced.");
            editor.repaint();
        }
    }
    
    /**
     * Causes all of the occurrences of word to be marked as per correct.
     * Internally, this uses Regex.
     */
    public void rescanWord(String word, boolean correct){
        //not correct--matches partial words...silly me.
        /*DocumentFind find=new DocumentFind(editor.getDocument());
        find.setIgnoreCase(false);
        find.setPattern(word);
        int result=0;
        int length=word.length();
        int offset=0;
        try{
            replacingSelection=true;
            result=find.findNext(offset);
            while(result>=0){
                offset=result+length;
                markSpellAttribute(result,offset,correct);
                result=find.findNext(offset);
            }
        }catch(BadLocationException ex){
            if(debug)System.out.println(ex);
            //replacingSelection=false;
        }
        finally{
            replacingSelection=false;
        }*/
        try{
            Document doc=editor.getDocument();
            Pattern pattern=Pattern.compile("\\b("+word+")\\b");//word boundry.
            Matcher matcher=pattern.matcher(doc.getText(0,doc.getLength()));
                        
            int start,length=word.length();
            replacingSelection=true;
            
            while(matcher.find()){
                start=matcher.start();
                if(debug)System.out.println(editor.getDocument().getText(start,length));
                markSpellAttribute(start,length,correct);
            }
        }
        catch(BadLocationException e){
            if(debug)System.out.println(e);
        }
        finally{
            replacingSelection=false;
        }
    }
    
    /**
     * Causes all of the words in the document to be spellchecked again.
     */
    public void rescan(){
        canRescan=true;//the stopping boolean.
        Thread t=new Thread(new Runnable(){
            public void run(){
                highlighter.removeAllMisspellings();
                int offset=0;
                Document doc=editor.getDocument();
                RTSWord word;
                String text;
                try{
                    word=getWordPosition(offset);//see if there's a word at 0.
                    while(word.wordEnd<doc.getLength() && canRescan){
                        if(debug)System.out.println("rescan() word found start:"+word.wordStart+" end:"+word.wordEnd);
                        if(word.wordEnd!=word.wordStart){
                            text=doc.getText(word.wordStart,(word.wordEnd-word.wordStart));
                            if((!checkSpelling(text))&&(!searchP.matcher(text).matches())){
                                //if it is incorrectly spelled, and not white space/punct.
                                highlighter.addMisspelling(word.wordStart, word.wordEnd);
                            }
                        }
                        offset=word.wordEnd;
                        offset=Utilities.getNextWord(editor, offset);
                        word=getWordPosition(offset);
                    }
                }catch(BadLocationException e){
                    if(debug)System.out.println("while in rescan():"+e);
                }
                if(debug)System.out.println("Rescan finished.");
            }
        });
        
        if(debug)System.out.println("Starting rescan thread");
        t.start();
    }
    
    /**
     * Adds to the list of words that should be ignored.
     * (List of Strings)
     */
    public void addIgnoredWords(java.util.List w){
        int size=w.size();
        for(int i=0;i<size;++i){
            if(!(w.get(i) instanceof java.lang.String)){
                throw new IllegalArgumentException("List may only contain Strings");
            }
            //add it to the hashtable.
            ignoredWords.put(w,Boolean.TRUE);
        }
        rescan();
    }
    
    /**
     * Returns the current list of ignored words (as Strings)
     */
    public java.util.List getIgnoredWords(){
        return (java.util.List)ignoredWords;
    }
    
    /**
     * Checks the String to see if it is either in the directory, or the internal
     * ignore all list.
     * @returns true if spelled correctly, false otherwise.
     */
    private boolean checkSpelling(String word){
        if(spellCheck.isCorrect(word)){
            return true;
        }
        else{
            //scan the ignore all list.
            if(ignoredWords.get(word)!=null){
                return true;
            }
            
            return false;//for the moment
        }
    }

    /**
     * Adjusts the highlighter to mark the area as either spelled correctly or not.
     */
    private void markSpellAttribute(int start, int length, boolean correct) throws BadLocationException{
        if(correct){
            highlighter.removeMisspelling(start);
        }
        else{
            //check to make sure it isn't white space.  This is really kind of a hack that masks other problems
            //but, it works...
			if(debug)System.out.println("marking "+start+" to "+(start+length)+" misspelled");
            if(searchP.matcher(editor.getText(start,length)).matches()){
                if(debug)System.out.println("Whitespace wanting to be marked as misspelled. (start:"+start+" length:"+length+")");
                return;//invalid
            }
            highlighter.addMisspelling(start, start+length);
        }
    }

    public static void main(String[] argv){
        JFrame frame=new JFrame();
        JTextPane p=new JTextPane();
        final RealTimeSpelling rts=new RealTimeSpelling(p);
        frame.getContentPane().add(new JScrollPane(p),BorderLayout.CENTER);
        JPanel panel=new JPanel();
        JCheckBox box=new JCheckBox("Check spelling as you type...");
        box.addItemListener(new ItemListener(){
            public void itemStateChanged(ItemEvent e){
                if(e.getStateChange()==e.SELECTED){
                    rts.startRealTimeSpelling();
                }
                else{
                    rts.stopRealTimeSpelling();
                }
            }
        });
        panel.add(box);
        frame.getContentPane().add(panel,BorderLayout.NORTH);
        frame.setSize(400,400);
        frame.setTitle("Spelling Tech Demo");
        frame.setVisible(true);
    }
    
    /**
     * Inner RTSWord Object is just a struct that holds the starting and ending locations of a word.
     */
    class RTSWord{
        /** The start of the word*/
        public int wordStart;
        /** The end of the word*/
        public int wordEnd;
    }
}

/**
 * Adds the ability to specify parts of the document this highligher belongs to
 * as misspelled.
 * Basically, spelling is piggy backed on top of highlighting.
 */
class SpellingHighlighter extends DefaultHighlighter{
    public static final SpellingHighlightPainter spellingPainter=new SpellingHighlightPainter();
    private ArrayList spellingInfo=new ArrayList();
    private JTextComponent component;
    
    private static final boolean debug=false;
    
    public void install(JTextComponent c){
        component=c;
        super.install(c);
        spellingInfo=new ArrayList();
    }
    
    public void addMisspelling(int start, int end) throws BadLocationException{
        //add it as a highlight
        Object tag=addHighlight(start,end,spellingPainter);
        
        //keep track of it as a spelling thing
        Document d=component.getDocument();
        Position s=d.createPosition(start);
        Position e=d.createPosition(end);
        spellingInfo.add(new SpellingHighlightInfo(s,e,tag,spellingPainter));
        
        //Damage the text to make it repaint.
        TextUI ui=component.getUI();
        //ui.damageRange(component,start,end);
        component.repaint();
    }
    
    
    /**
     * Removes the misspelling highlight at position.
     * If no misspelling highlight exists at pos, then false is returned.
     */
    public boolean removeMisspelling(int pos) throws BadLocationException{
        if(debug)System.out.println("remove spelling called with pos:"+pos);
        //find the tag for this misspelling.
        int size,i,start=0,end=0;
        size=spellingInfo.size();
        Object tag=null;
        SpellingHighlightInfo info;
        for(i=0;i<size;++i){
            info=(SpellingHighlightInfo)spellingInfo.get(i);
            start=info.getStartOffset();
            end=info.getEndOffset();
            
            if(debug)System.out.println("testing info with start:"+start+" end:"+end);
            
            if(pos>=start && pos<=end){
                if(debug)System.out.println("found");
                tag=info.getTag();
                //take it out of this arraylist
                spellingInfo.remove(i);
                break;
            }
        }
        //remove it from highlights
        if(tag==null){
            return false;
        }
        else{
            if(debug)System.out.println("Removing highlight at start:"+start+" end:"+end);
            removeHighlight(tag);
            //Damage the text to make it repaint.
            TextUI ui=component.getUI();
            //ui.damageRange(component,start,end);
            component.repaint();
            return true;
        }
    }
    
    public void removeAllMisspellings(){
        int i, size=spellingInfo.size();
        SpellingHighlightInfo info;
        for(i=0;i<size;++i){
            info=(SpellingHighlightInfo)spellingInfo.get(i);
            removeHighlight(info.getTag());
        }
        spellingInfo.clear();
    }
    
    /**
     * Removes all highlights except for spelling highlights.
     * To remove all highlights, call removeAllMisspellings() first.
     */
    public void removeAllHighlights(){
        //remove all highlights
        super.removeAllHighlights();
        //now add back the misspelling ones
        int size=spellingInfo.size(), i;
        SpellingHighlightInfo info;
        Object o;
        TextUI ui=component.getUI();
        try{
            for(i=0;i<size;++i){
                info=(SpellingHighlightInfo)spellingInfo.get(i);
                o=addHighlight(info.getStartOffset(),info.getEndOffset(),spellingPainter);
                info.setTag(o);//it probably changed...
                //damage to cause repaint
                //ui.damageRange(component,info.getStartOffset(),info.getEndOffset());
                component.repaint();
            }
        }catch(BadLocationException ex){
            System.out.println(ex);
        }
    }
}

/**
 * The class used as a reference to the misspelled highlight often referred to as a "tag"
 * in highlight documentation.
 */
class SpellingHighlightInfo implements Highlighter.Highlight{
    private Position start, end;
    private Object tag;
    private Highlighter.HighlightPainter painter;
    public SpellingHighlightInfo(Position start, Position end, Object tag,Highlighter.HighlightPainter painter){
        this.start=start;
        this.end=end;
        this.tag=tag;
        this.painter=painter;
    }
    
    public int getStartOffset(){
        return start.getOffset();
    }
    
    public int getEndOffset(){
        return end.getOffset();
    }
    
    public Object getTag(){
        return tag;
    }
    
    public void setTag(Object tag){
        this.tag=tag;
    }
    
    public Highlighter.HighlightPainter getPainter(){
        return painter;
    }
}

/**
 * Paints mispelling portions
 */
class SpellingHighlightPainter implements Highlighter.HighlightPainter{
    private static final boolean debug=false;

    public void paint(Graphics g, int s, int e, Shape bounds, JTextComponent c){
        if(debug)System.out.println("SpellingHighlightPainter s:"+s+" e:"+e);

		try{
            Rectangle start;
            Rectangle end;
            Color oldColor=g.getColor();
            g.setColor(Color.RED);
            int j,i;
            for(i=s;i<e;++i){
                start=c.modelToView(i);
                end=c.modelToView(i+1);
                for(j=start.x; j<end.x; ++j){
                    g.drawLine(j,start.y+start.height,j+1,end.y+end.height);
                    g.drawLine(j,start.y+1+start.height,j+1,end.y+1+end.height);//make it more blocky.
                    j+=2;
                }
            }
            //c.repaint();
            g.setColor(oldColor);
        }catch(BadLocationException ex){
            System.out.println(ex);
        }
    }
}