package edu.gwu.servlets;
/**
  * Lists all of the properties for {@link StaticContext} (and related).
  * These are all the valid properties to be used in the {@link StaticContext}
  * properties file.  See the various fields for more information on what they
  * configure, and what valid values are.
  *
  * StaticContext Properties can be changed without shutting down your servlet server.
  * Please note that changes may take as much as 1 minute to propagate through the system.
  *
  * @author Michael Gaiman
  * @version 1.0 6/2003
  */

public class SCProperties {

    /**
     * Specifies the directory that temperary files should be written to.
     * It should always end with a directory terminator.
     */
    public static final String writeDir			="writeDir";
    /**
     * Specifies whether debugging should be turned on.  
     * valid values are <code>true</code> and <code>false</code>
     * The default it <code>false</code>.
     */
    public static final String debugOn			="debugOn";
    /**
     * Specifies the file that should be used when debugging it on.
     * This file will be written in the directory you specify with {@link #writeDir}.
     */
    public static final String debugFileName		="debugFileName";

    /**
     * Specifies the file base fileName for the log files.  Subsequent
     * files will use this name with a number appended to it. For example
     * if <code>myLog</code> is the <code>logFileName</code>, then 
     * the log will be named <code>myLog1</code> followed by <code>myLog2</code>
     * when the first log is full.
     * The file will be placed in the directory you specify with {@link #writeDir}.
     */
    public static final String logFileName		="logFileName";
    /**
     * Specifies how many lines need to be accumulated in the log buffer before
     * the lines are written out.  <code>logFlushSize</code> must be >=0 and <=100000.
     */
    public static final String logFlushSize		="logFlushSize";
    /**
     * Specifies how many lines need to be written to the current log file before
     * a new file is started.  <code>logMaxSize</code> must be >=100.
     */
    public static final String logMaxSize		="logMaxSize";
    /**
     * Specifies how long the <code>ThreadedLogWriter</code> should sleep
     * before checked whether or not a write out need to happen.
     * <code>logMaxSize</code> must be >=1 and <=600 (10 minutes).
     */
    public static final String logSleepSeconds		="logSleepSeconds";
    
    /**
     * Specifies the driver to use in the when connecting to the database.
     * @see java.sql.Driver
     */
    public static final String jdbcDriver		="jdbcDriver";
    /**
     * Species the address to the database.  See your database documentation for
     * more details.
     * The URL should end in a backslash.
     */
    public static final String jdbcURL			="jdbcURL";
    /**
     * Which database to connect to.  See your database documentation for more
     * details.
     */
    public static final String jdbcDB			="jdbcDB";
    /**
     * The username used to connect to your database.
     */
    public static final String jdbcUser			="jdbcUser";
    /**
     * The password used to connect to your database.
     */
    public static final String jdbcPassword		="jdbcPassword";
    /**
     * The minimum number of connections to the database that the connection pooler
     * should keep active.  <code>jdbcMinConnections</code> must be >=0 and
     * does not have a hard coded maximum, but should not exceed the maximum of your
     * database.  (See database documentation for more information.)
     * @see #jdbcMaxConnections
     */
    public static final String jdbcMinConnections 	="jdbcMinConnections";
    /**
     * The maximum number of connections to the database that the connection pooler
     * should keep active.  <code>jdbcMaxConnections</code> must be >=1 and does
     * not have a hard coded maximum, but should not exceed the maximum connections
     * your database allows.  (See the database documentation for more information).
     * @see #jdbcMinConnections
     */
    public static final String jdbcMaxConnections	="jdbcMaxConnections";
    /**
     * The length of time to keep unused connections alive in seconds.
     * This value must be >=60 and <=3600. (1 minute, 1 hour).
     * There is a large trade off here, too small a value and you forfeit
     * the performance gains that connection pooling gives you.  Too large a value
     * and you could keep the database load artificially high.
     */
    public static final String jdbcConnectionTimeout	="jdbcConnectionTimeout";
    /**
     * How many times the database broker should attempt to get a connection.
     * This is used when all connections are in use.
     * <code>jdbcConnectionTrials</code> must be >=1 and <=500.
     */
    public static final String jdbcConnectionTrials	="jdbcConnectionTrials";
    
    /**
     * How long the {@link edu.gwu.servlets.ThreadedSecurityHandler} should sleep before
     * looking to see if passwords have changed.
     * This value must be >=1 and <=600 (10 minutes).
     */
    public static final String securitySleepTimeSeconds ="securitySleepTimeSeconds";
    /**
     * The properties file to where the usernames and passwords are stored.
     * <code>passwordFileName</code> should also specify the directory that the 
     * password file can be found.
     */
    public static final String passwordFileName		="passwordFileName";
    
    /**
     * How long the {@link edu.gwu.servlets.ThreadedEmailSender} should sleep before
     * looking to see if emails have been added to the EmailQueue
     * This value must be >=1 and <=600 (10 minutes)
     */
    public static final String emailSleepTimeSeconds = "emailSleepTimeSeconds";
}
