package edu.gwu.servlets;
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.gwu.blaise.version2.backend.packagers.*;
import edu.gwu.blaise.version2.backend.queries.*;
import edu.gwu.blaise.version2.backend.util.*;
import edu.gwu.blaise.version2.util.*;
import edu.gwu.blaise.version2.documents.*;
import edu.gwu.blaise.version2.transmission.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * File: ThreadedEmailSender.java
 *
 * Handles the periodic sending of emails.  Do not interact directly with this
 * class, instead use {@link StaticContext} or {@link BaseServlet}.
 */

public class ThreadedEmailSender implements Runnable, PropertyChangeListener{

  static final int SLEEP_TIME_SECONDS_DEFAULT = 60;

  boolean stopped = false;
  int sleepTimeSeconds = 60;              //sleep for one minute as the default
  Connection c = null;
  File tempDir=null;                      //needed for MailUtility
  boolean success;                        //whether or not the emailing succeeded
  PreparedStatement ps = null;            
  Blaise_PaperVectorGetter pp = null;     //used to get the Blaise_Paper object
  Blaise_Paper paper = null;              //the paper
  Vector thePaper = null;                 //what is returned by pp
  Vector pID = null;                      //holds the paper id in a vector
  String limitBy = "";                    //holds the type of sender
  GetQuery gq = null;
  MailUtility mu = null;
  Long limitID=null;                      //holds the id of the sender
  long paperID;                           //extracted from the vector
  String wait=null;                       //tells whether or not to send the email
  DeleteQuery dq=null;                    //to delete the emailQueue table entry
  Long eqID=null;
  Vector queueID=new Vector();

  public ThreadedEmailSender(int sleepTimeSeconds){
    try{
      tempDir = File.createTempFile("emailSender", "tmp");
    }catch(IOException e){
      StaticContext.writeLog("ThreadedEmailSender: Constructor error: " + e);
    }
    validateSleepTimeSeconds(Integer.toString(sleepTimeSeconds));
    if(StaticContext.debugOn()){
        StaticContext.writeDebug("ThreadedEmailSender: Registering Properties");
    }
    StaticContext.registerForProperty(SCProperties.emailSleepTimeSeconds,this);
    try{
      mu=new MailUtility(File.createTempFile("blaise","tmp"));
    }catch(IOException e){
      StaticContext.writeLog("ThreadedEmailSender: error in creating MailUtility");
    }
  }

  //needed to implement PropertyChange
  public void propertyChangeNotification(String property, String value){
    
    if(StaticContext.debugOn()){
        StaticContext.writeDebug("ThreadedEmailSender: Property Change Notification: Property "+property+" value="+value);
    }
    
    if(property.equals(SCProperties.emailSleepTimeSeconds)){
        validateSleepTimeSeconds(value);
        //change will be propagated once current sleep is done.
    }
  }

  void validateSleepTimeSeconds(String value){
    if(value==null){
      StaticContext.writeLog("emailSleepTimeSeconds is not a valid integer.  Using default.");
      sleepTimeSeconds=SLEEP_TIME_SECONDS_DEFAULT;
      return;
    }
    
    int parsedValue=0;
    try{
      parsedValue=Integer.parseInt(value);
    }
    catch(NumberFormatException e){
      StaticContext.writeLog("emailSleepTimeSeconds is not a valid integer.  Using default.");
      sleepTimeSeconds=SLEEP_TIME_SECONDS_DEFAULT;
      return;
    }
    
    if(parsedValue<1 || parsedValue>600){//less than 1 second, greater than 10 minutes.
      StaticContext.writeLog("ThreadedEmailSender: Invalid emailSleepTimeSeconds value.  Using default.");
      sleepTimeSeconds=SLEEP_TIME_SECONDS_DEFAULT;
      return;
    }
    
    sleepTimeSeconds=parsedValue;    
  }

  public void stop (){
    if(StaticContext.debugOn()){
        StaticContext.writeDebug("ThreadedEmailSender has been asked to stop.");
    }
    stopped = true;
  }

  private boolean doTypeOne(){
    limitBy = PackagingConstants.LIMIT_writer;
    //now that we have the limitID, get the paper
    //the method to get the paper takes in a Vector of paperIDs, so that has to be created first
    if(pID!=null){
      pID.clear();
    }else{
      pID=new Vector();
    }
    pID.add(new Long(paperID));
    //paper is returned in a vector, it should have only one entry, or else it did something wrong
    pp = new Blaise_PaperVectorGetter(pID, limitBy, limitID);
    try{
      thePaper = pp.getBlaise_PaperVector();
    }catch(SQLException e){
      StaticContext.writeLog("ThreadedEmailSender: error in SQL handling: " + e);      
      return false;
    }
    if(thePaper.size() != 1){
      StaticContext.writeLog("ThreadedEmailSender: Error in Type One handling: Blaise_Paper vector does not have size of one.");      
      return false;
    }
    else{
      paper = (Blaise_Paper)thePaper.firstElement();
    }
    //now actually do the emailing
    try{
      mu.sendReceipt(paper);    
    }catch(javax.mail.MessagingException e){
      StaticContext.writeLog("ThreadedEmailSender: Error in sending receipt: " + e);
      return false;
    }
    return true;
  }

  private boolean doTypeTwo(){
    //now that we have the limitID, get the paper
    //the method to get the paper takes in a Vector of paperIDs, so that has to be created first
    if(pID!=null){
      pID.clear();
    }else{
      pID=new Vector();
    }
    pID.add(new Long(paperID));
    //paper is returned in a vector, it should have only one entry, or else it did something wrong
    pp = new Blaise_PaperVectorGetter(pID, limitBy, limitID);
    try{
      thePaper = pp.getBlaise_PaperVector();
    }catch(SQLException e){
      StaticContext.writeLog("ThreadedEmailSender: error in SQL handling: " + e);      
      return false;
    }
    if(thePaper.size() != 1){
      StaticContext.writeLog("ThreadedEmailSender: Error in Type Two handling: Blaise_Paper vector does not have size of one.");      
      return false;
    }
    else{
      paper = (Blaise_Paper)thePaper.firstElement();
    }
    //now actually do the emailing
    try{
      mu.sendGraded(paper);
    }catch(javax.mail.MessagingException e){
      StaticContext.writeLog("ThreadedEmailSender: messaging error in sending the paper: " + e);
      return false;
    }
    return true;
  }
  
  public void run (){
    while (! stopped) {
    	//not exactly needed for sending email, but this needed to be in a repeating thread
		StaticContext.writeDebug("Max Memory on Heap: "+Runtime.getRuntime().maxMemory()+"\n Total Memory Available on Heap: "+Runtime.getRuntime().totalMemory());
		StaticContext.writeDebug("Free Memory on Heap: "+Runtime.getRuntime().freeMemory());
      try{
        gq=new GetQuery(null,PackagingConstants.TABLE_eq,PackagingConstants.LIMIT_admin,null);
        ResultSet rs = gq.execute();
        if(gq!=null){
          gq.free();
        }
        queueID.clear();
        while(rs.next()){
          paperID = rs.getLong("paperID");
          int type = rs.getInt("type");
          limitID = new Long(rs.getLong("limitID"));
          eqID=new Long(rs.getLong("queueID"));
          if(type == 1){//student has submitted paper but hasnt gotten receipt
            limitBy=PackagingConstants.LIMIT_writer;
            success = this.doTypeOne();
            if(success){
              queueID.add(eqID);
            }
          }
          else if(type == 2){//teacher has submitted grade, but graded paper hasn't been sent to student
            limitBy=PackagingConstants.LIMIT_grader;
            success = this.doTypeTwo();
            if(success){
              queueID.add(eqID);
            }
          }
          else{//invalid type
            StaticContext.writeLog("ThreadedEmailSender: Invalid email type given: " + type);
            success = false;
          }
        }
        if(queueID.size()>0){
          dq=new DeleteQuery(queueID,PackagingConstants.TABLE_eq,PackagingConstants.LIMIT_admin);
          int ex=dq.execute();
          if(ex!=queueID.size()){
            StaticContext.writeLog("ThreadedEmailSender: deleteQuery did not delete all entries");
          }
          if(dq!=null){
            dq.free();
          }
        }
      }catch(SQLException e){
        StaticContext.writeLog("ThreadedEmailSender: error in SQL handling: " + e);      
      }
      // Sleep
      try {
        Thread.sleep (sleepTimeSeconds * 1000);
      }
      catch (InterruptedException e) {}
    } //endwhile
  }
}
