package edu.gwu.blaise.version2.util.contextmenu;
import java.awt.*;
import java.awt.event.*;

/**
 * The Event passed to ContextMenuListeners.  It contains a reference to the originating
 * Component as well as the X and Y coordinates of the mouse click.
 * @see ContextMenuListener
 * @author Michael Gaiman mgaiman@gwu.edu
 * @version 1.1 3/16/2004
 * @history 1.0 3/16/2004 Initial version
 *          1.1 3/16/2004 Blaise packaging
 */
public class ContextMenuEvent extends ComponentEvent{
    /**
     * The ID for a context menu event.
     */
    public static final int CONTEXT_MENU_EVENT=RESERVED_ID_MAX+99;
    
    private int x,y,order,actualOrder;
    
    /**
     * Constructs a new ContextMenuEvent.
     * @param c The Component which received the ContextMenu request.
     * @param x The x coordinate of the request
     * @param y The y coordinate of the request
     * @param order The order that this listener requested for their items.
     * @param actualOrder The actual order they are being displayed (FIRST=0).
     */
    public ContextMenuEvent(Component c,int x, int y, int order, int actualOrder){
        this(c,x,y);
        this.order=order;
        this.actualOrder=actualOrder;
    }
    
    /**
     * ContextMenuManager may not know the order ahead of time.
     * this allows it to create only one instance of the event
     * and change the order when needed.
     */
    ContextMenuEvent(Component c,int x, int y){
        super(c,CONTEXT_MENU_EVENT);
        this.x=x;
        this.y=y;
    }

    public int getX(){
        return x;
    }
    
    public int getY(){
        return y;
    }
    
    /**
     * ContextMenuManager may change this.
     */
    void setOrder(int o){
        order=o;
    }
    
    /**
     *@return The order that this listener requested for their items.
     */
    public int getOrder(){
        return order;
    }
    
    void setActualOrder(int ao){
        actualOrder=ao;
    }
    
    /**
     * @return The actual order they are being displayed (FIRST=0).
     */
    public int getActualOrder(){
        return actualOrder;
    }
    
    /**
     * A debugging string.
     */
    public String paramString(){
        return "ContextMenuEvent [Component:"+getComponent()+" X:"+x+" Y:"+y+" parent:"+super.paramString()+"]";
    }

}